// SimpleRefDataExample.cpp : Defines the entry point for the console application.
//
#include <blpapi_session.h>
#include <blpapi_eventdispatcher.h>

#include <blpapi_event.h>
#include <blpapi_message.h>
#include <blpapi_element.h>
#include <blpapi_name.h>
#include <blpapi_request.h>
#include <blpapi_subscriptionlist.h>
#include <blpapi_defs.h>
#include <blpapi_exception.h>

#include <iomanip>
#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "mathlink.h"
extern void BBRetrieveIntraday(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime);

using namespace std;
using namespace BloombergLP;
using namespace blpapi;

namespace {
    const Name TICK_DATA("tickData");
    const Name COND_CODE("conditionCodes");
    const Name TICK_SIZE("size");
    const Name TIME("time");
    const Name TYPE("type");
    const Name VALUE("value");
    const Name RESPONSE_ERROR("responseError");
    const Name CATEGORY("category");
    const Name MESSAGE("message");
    const Name SESSION_TERMINATED("SessionTerminated");
	const Name SESSION_STARTUP_FAILURE("SessionStartupFailure");
};

class Intraday
{
	std::string         d_host;
    int                 d_port;

    void printUsage()
    {
		std::cout << "Usage:" << std::endl
			<< "	Retrieve reference data " << std::endl
			<< "		[-ip 		<ipAddress	= localhost>" << std::endl
			<< "		[-p 		<tcpPort	= 8194>" << std::endl;
		return;
    }

	void printErrorInfo(const char *leadingStr, const Element &errorInfo)
    {
        std::cout << leadingStr
            << errorInfo.getElementAsString(CATEGORY)
            << " ("
            << errorInfo.getElementAsString(MESSAGE)
            << ")" << std::endl;
    }

	string processEvent(Event &event)
    {
        string output = "Failed";
		MessageIterator msgIter(event);
        while (msgIter.next()) {
            Message msg = msgIter.message();
            if (msg.hasElement(RESPONSE_ERROR)) {
                printErrorInfo("REQUEST FAILED: ", 
                    msg.getElement(RESPONSE_ERROR));
                continue;
            }
            output = processMessage(msg);
        }
		return output;
    }

	string processMessage(Message &msg)
    {
		string output;
        Element data = msg.getElement(TICK_DATA).getElement(TICK_DATA);
        int numItems = data.numValues();
        std::cout << "TIME\t\t\t\tTYPE\tVALUE\t\tSIZE\tCC" << std::endl;
        std::cout << "----\t\t\t\t----\t-----\t\t----\t--" << std::endl;
        std::string cc;
        std::string type;
		output = "{";
        for (int i = 0; i < numItems; ++i) {
            Element item = data.getValueAsElement(i);
            Datetime time = item.getElementAsDatetime(TIME);
            std::string timeString = item.getElementAsString(TIME);
            type = item.getElementAsString(TYPE);
            double value = item.getElementAsFloat64(VALUE);
            int size = item.getElementAsInt32(TICK_SIZE);
            if (item.hasElement(COND_CODE)) {
                cc = item.getElementAsString(COND_CODE);
            }  else {
                cc.clear();
            }

            std::cout.setf(std::ios::fixed, std::ios::floatfield);
            std::cout << timeString <<  "\t"
                << type << "\t" 
                << std::setprecision(3)
                << std::showpoint << value << "\t\t"
                << size << "\t" << std::noshowpoint 
                << cc << std::endl;

			output += "{\"";
			output += timeString;
			output += "\", ";
			output += item.getElementAsString(VALUE);
			output += ", ";
			output += item.getElementAsString(TICK_SIZE);
			output += "}";
			if(i < numItems-1)
				output += ",";
        }
		output += "}";
		return output;
    }

    public:
	string Retrieve(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime)
    {
        string output = "Failed";
		d_host = "localhost";
        d_port = 8194;

        SessionOptions sessionOptions;
        sessionOptions.setServerHost(d_host.c_str());
        sessionOptions.setServerPort(d_port);

        std::cout << "Connecting to " <<  d_host << ":" 
                  << d_port << std::endl;
        Session session(sessionOptions);
        if (!session.start()) {
            std::cerr <<"Failed to start session." << std::endl;
            return "Failed to open";
        }
        if (!session.openService("//blp/refdata")) {
            std::cerr <<"Failed to open //blp/refdata" << std::endl;
            return "Failed to open";
        }

        Service refDataService = session.getService("//blp/refdata");
        Request request = refDataService.createRequest("IntradayTickRequest");

		// only one security/eventType per request
        request.set("security", Ticker);

        // Add fields to request
        Element eventTypes = request.getElement("eventTypes");
        eventTypes.appendValue(Field);
		
		// Times are in GMT
		request.set("startDateTime", StartTime);
		request.set("endDateTime", EndTime);

		request.set("includeConditionCodes", false);

        std::cout << "Sending Request: " << request << std::endl;
        session.sendRequest(request);

		bool done = false;
        while (!done) {
            Event event = session.nextEvent();
            if (event.eventType() == Event::PARTIAL_RESPONSE) {
                std::cout << "Processing Partial Response" << std::endl;
                output = processEvent(event);
            }
            else if (event.eventType() == Event::RESPONSE) {
                std::cout << "Processing Response" << std::endl;
                output = processEvent(event);
				done = true;
            } else {
                MessageIterator msgIter(event);
                while (msgIter.next()) {
                    Message msg = msgIter.message();
                    if (event.eventType() == Event::SESSION_STATUS) {
                        if (msg.messageType() == SESSION_TERMINATED ||
                            msg.messageType() == SESSION_STARTUP_FAILURE) {
                            done = true;
                        }
                    }
                }
            }
        }
		return output;
	}
};


void BBRetrieveIntraday(const char* Ticker,const char* Field,const char* StartTime, const char* EndTime)
{
	string temp;
	//Create SimpleHistoryExample Class Object
	Intraday Example;
	//Retrieve string created by the example object
	temp = Example.Retrieve(Ticker,Field,StartTime,EndTime);
	std::cout<<"Temp: "<<temp<<std::endl;
	//Put string to Mathematica
	MLPutString(stdlink, temp.c_str());
	return;
}



#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( (int)(argv_end - argv), argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif