(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   1174840,      23010]
NotebookOptionsPosition[   1172949,      22944]
NotebookOutlinePosition[   1173476,      22964]
CellTagsIndexPosition[   1173433,      22961]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Bloomberg Data to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\nPre-",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "-10"
}], "Title",
 CellChangeTimes->{{3.4348218733245*^9, 3.434821880183875*^9}, {
  3.44951690785*^9, 3.449516919975*^9}, {3.4542526066628895`*^9, 
  3.4542526102875414`*^9}, {3.454671279093625*^9, 3.4546712805155*^9}, {
  3.633136424865485*^9, 3.6331364394602127`*^9}}],

Cell["June 13, 2012", "Subsubtitle",
 CellChangeTimes->{{3.4348218841995*^9, 3.434821885262*^9}, {
  3.45466980885925*^9, 3.454669810093625*^9}, {3.45960533434375*^9, 
  3.459605336546875*^9}, {3.46385140536852*^9, 3.46385140624352*^9}, {
  3.465744082552105*^9, 3.465744082989582*^9}, {3.50306886475525*^9, 
  3.50306886694275*^9}, {3.548606507408375*^9, 3.548606510252125*^9}}],

Cell[TextData[{
 "This code will not work unless you have a Bloomberg Professional license, \
have installed the Bloomberg API, and are logged in to Bloomberg Professional \
when you run the functions below. It has been tested with versions of Windows \
from XP through Windows 8, and should work with all versions of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from about version 6 through at least version 10. However, if you are \
using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 10, please consider using the newer version of this notebook that takes \
advantage of built-in support for dynamical data, in addition to increasing \
flexibility, fixing a number of bugs and cleaning up the interface.\n\nTo \
install, create the folder \[OpenCurlyDoubleQuote]C:\\Program Files\\Wolfram \
Research\\Merrin Functions\\\[CloseCurlyDoubleQuote] and extract the attached \
zip file there. You may need to adjust some of the paths in this code \
according to the setup of your computer. If an earlier version of the folder \
exists in that location, replace it with the new (July 2009) version. This \
interface consists of six commands:\n\n",
 StyleBox["BBGetNF1b",
  FontWeight->"Bold"],
 ": loads historical data from Bloomberg and converts to \[OpenCurlyQuote]NF1\
\[CloseCurlyQuote] format, about which more below. Used for, for example, \
U.S. CPI monthly back to 1969, or the closing price of Apple Inc. daily since \
1978.\n",
 StyleBox["BBGetNF1",
  FontWeight->"Bold"],
 ": an old version of the above, lacking the ability to set name and date \
format on the fly.\n",
 StyleBox["BBGetCurrent",
  FontWeight->"Bold"],
 ": loads a current descriptive field (such as Walmart's state of \
incorporation or shares outstanding). The FLDS command in Bloomberg can be \
used to identify what data is available for each security. Uniquely among \
ther Merrin Capital functions, BBGetCurrent will operate on either a single \
security, or a list of them.\n",
 StyleBox["BBGetIntraday",
  FontWeight->"Bold"],
 ": loads intraday trade data (or ask data, or bid data). The\"field\" in its \
parameter list must be a string, either TRADE\" or \"BID\" or \"ASK\".\n",
 StyleBox["BBGetIntradayBars",
  FontWeight->"Bold"],
 ":  loads periodic intraday data (price snapshots every minute, for \
example).\n",
 StyleBox["bbg2Mdates",
  FontWeight->"Bold"],
 " will convert the Bloomberg data to a more usual date format, as is \
sometimes useful."
}], "Text",
 CellChangeTimes->{{3.449516680834375*^9, 3.449516759615625*^9}, {
   3.449517050928125*^9, 3.4495171807875*^9}, {3.45121680611825*^9, 
   3.451216810758875*^9}, {3.451226165587*^9, 3.451226216337*^9}, {
   3.4542526240830917`*^9, 3.45425270795004*^9}, 3.45466982904675*^9, {
   3.454669864468625*^9, 3.454670137843625*^9}, {3.454670220812375*^9, 
   3.45467024460925*^9}, {3.45960536375*^9, 3.459605372546875*^9}, {
   3.459605466140625*^9, 3.45960547009375*^9}, {3.463339646437875*^9, 
   3.463339662781625*^9}, 3.463851368790395*^9, {3.465744092957822*^9, 
   3.465744093426548*^9}, {3.4657446581476326`*^9, 3.4657446588819704`*^9}, 
   3.503068715239625*^9, {3.548673433158375*^9, 3.548673448814625*^9}, 
   3.54867356558025*^9, {3.570877419757226*^9, 3.570877429405176*^9}, {
   3.633136330013728*^9, 3.633136407197497*^9}, {3.633136443751577*^9, 
   3.633136490689228*^9}, {3.633136569928933*^9, 3.633136583073359*^9}, {
   3.633136862657213*^9, 3.633136871258483*^9}}],

Cell["\<\
BBGetNF1 produces output in what we call \"nf1 format,\" a data type that we \
find useful for financial data. It includes a header and a time series, in \
this format:

\t{{formatcode, name, timeunit, type}, {{date, data}, {date, data}, etc.}}

where formatcode, in this case \[Rule] \"nf1\", name \[Rule] \"IBM US \
Equity\", for example, timeunit \[Rule] \"monthly\" (or whatever), and type \
\[Rule] \"pctchanges\" (or \"delta,\" or \"value\").

We have created hundreds of functions that work with NF1 objects, providing \
different sorts of analysis, graphing, and so forth.\
\>", "Text",
 CellChangeTimes->{{3.4456145947571564`*^9, 3.4456146143821564`*^9}, {
  3.4456146618040314`*^9, 3.4456146816165314`*^9}, {3.4456148011009064`*^9, 
  3.4456148375852814`*^9}, {3.4456148694134064`*^9, 3.4456149245852814`*^9}, {
  3.44951718619375*^9, 3.44951720569375*^9}, {3.4542527350880594`*^9, 
  3.454252740415673*^9}, {3.454669949437375*^9, 3.454669949593625*^9}, {
  3.454670107843625*^9, 3.454670107999875*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "BBGetNF1b", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"adjust", "\[Rule]", "\"\<CALENDAR\>\""}], ",", 
     RowBox[{"type", "\[Rule]", "\"\<value\>\""}], ",", 
     RowBox[{"name", "\[Rule]", "\"\<df\>\""}], ",", 
     RowBox[{"dateformat", "\[Rule]", "\"\<bbg\>\""}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBGetNF1b", "[", 
    RowBox[{
    "ticker_String", ",", "field_String", ",", "start_", ",", "end_", ",", 
     "freq_String", ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "name1", ",", "path", ",", "output", ",", "link", ",", " ", "date"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"name1", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"OptionValue", "[", "name", "]"}], "\[Equal]", 
          "\"\<df\>\""}], ",", 
         RowBox[{"ticker", "<>", "\"\<: \>\"", "<>", "field"}], ",", 
         RowBox[{"OptionValue", "[", "name", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
      "path", " ", "=", " ", 
       "\"\<C:/Program Files/Wolfram Research/Merrin \
Functions/BBRetrieve/\>\""}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{"path", " ", "to", " ", "place", " ", "BBRetrieve"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"path", "<>", "\"\<BBRetrieve\>\""}], "]"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", " ", 
         "\"\<DAY\>\""}], ",", " ", 
        RowBox[{"date", " ", "=", " ", "\"\<DAILY\>\""}], ",", " ", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", " ", 
           "\"\<WEEK\>\""}], ",", " ", 
          RowBox[{"date", " ", "=", " ", "\"\<WEEKLY\>\""}], ",", " ", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
              " ", "\"\<MONTHLY\>\""}], "||", 
             RowBox[{
              RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
              " ", "\"\<MONTH\>\""}]}], ",", " ", 
            RowBox[{"date", " ", "=", " ", "\"\<MONTHLY\>\""}], ",", " ", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
               " ", "\"\<QUARTER\>\""}], ",", 
              RowBox[{"date", " ", "=", " ", "\"\<QUARTERLY\>\""}], ",", " ", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
                 " ", "\"\<YEAR\>\""}], ",", 
                RowBox[{"date", " ", "=", " ", "\"\<YEARLY\>\""}], ",", 
                RowBox[{"ToUpperCase", "[", "freq", "]"}]}], "]"}]}], "]"}]}],
            "]"}]}], "]"}]}], "]"}], ";", "  ", "\[IndentingNewLine]", " ", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\"\<nf1\>\"", ",", "name1", ",", 
           RowBox[{"ToLowerCase", "[", "freq", "]"}], ",", 
           RowBox[{"OptionValue", "[", "type", "]"}]}], "}"}], ",", 
         RowBox[{"ToExpression", "[", 
          RowBox[{"BBRetrieve", "[", 
           RowBox[{
           "ticker", ",", " ", "field", ",", "start", ",", "end", ",", "date",
             ",", 
            RowBox[{"ToUpperCase", "[", 
             RowBox[{"OptionValue", "[", "adjust", "]"}], "]"}]}], "]"}], 
          "]"}]}], "}"}]}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"build", " ", "nf1", " ", "format"}], ";", " ", 
        RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
         RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
         "list"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"close", " ", "link"}], " ", "*)"}], " ", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "dateformat", "]"}], "\[Equal]", 
         "\"\<bbg\>\""}], ",", "output", ",", 
        RowBox[{
         StyleBox["bbg2Mdates",
          FontWeight->"Bold"], 
         StyleBox["[",
          FontWeight->"Bold"], 
         RowBox[{"output", ",", 
          RowBox[{"format", "\[Rule]", 
           RowBox[{"OptionValue", "[", "dateformat", "]"}]}]}], "]"}]}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBGetNF1", "[", 
    RowBox[{
    "ticker_String", ",", "field_String", ",", "start_", ",", "end_", ",", 
     "freq_", ",", 
     RowBox[{"adjust_:", "\"\<CALENDAR\>\""}], ",", 
     RowBox[{"type_:", "\"\<value\>\""}]}], "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"path", ",", "output", ",", "link", ",", " ", "date"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Message", "[", 
       RowBox[{"BBGetNF1", "::", "\"\<deprecated\>\""}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
      "path", " ", "=", " ", 
       "\"\<C:/Program Files/Wolfram Research/Merrin \
Functions/BBRetrieve/\>\""}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{"path", " ", "to", " ", "place", " ", "BBRetrieve"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"path", "<>", "\"\<BBRetrieve\>\""}], "]"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", " ", 
         "\"\<DAY\>\""}], ",", " ", 
        RowBox[{"date", " ", "=", " ", "\"\<DAILY\>\""}], ",", " ", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", " ", 
           "\"\<WEEK\>\""}], ",", " ", 
          RowBox[{"date", " ", "=", " ", "\"\<WEEKLY\>\""}], ",", " ", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
              " ", "\"\<MONTHLY\>\""}], "||", 
             RowBox[{
              RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
              " ", "\"\<MONTH\>\""}]}], ",", " ", 
            RowBox[{"date", " ", "=", " ", "\"\<MONTHLY\>\""}], ",", " ", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
               " ", "\"\<QUARTER\>\""}], ",", 
              RowBox[{"date", " ", "=", " ", "\"\<QUARTERLY\>\""}], ",", " ", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", "\[Equal]", 
                 " ", "\"\<YEAR\>\""}], ",", 
                RowBox[{"date", " ", "=", " ", "\"\<YEARLY\>\""}], ",", 
                RowBox[{"ToUpperCase", "[", "freq", "]"}]}], "]"}]}], "]"}]}],
            "]"}]}], "]"}]}], "]"}], ";", "  ", "\[IndentingNewLine]", " ", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"\"\<nf1\>\"", ",", 
           RowBox[{"ticker", "<>", "\"\<: \>\"", "<>", "field"}], ",", 
           RowBox[{"ToLowerCase", "[", "freq", "]"}], ",", "type"}], "}"}], 
         ",", 
         RowBox[{"ToExpression", "[", 
          RowBox[{"BBRetrieve", "[", 
           RowBox[{
           "ticker", ",", " ", "field", ",", "start", ",", "end", ",", "date",
             ",", 
            RowBox[{"ToUpperCase", "[", "adjust", "]"}]}], "]"}], "]"}]}], 
        "}"}]}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"build", " ", "nf1", " ", "format"}], ";", " ", 
        RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
         RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
         "list"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"close", " ", "link"}], " ", "*)"}], " ", "output"}]}], 
    "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{"ticker_", ",", "field_"}], "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "path", ",", "output", ",", "link", ",", " ", "date", ",", "temp", ",", 
       " ", "x", ",", "mytickers"}], "}"}], ",", 
     RowBox[{
      RowBox[{
      "path", " ", "=", " ", 
       "\"\<C:/Program Files/Wolfram Research/Merrin \
Functions/BBRetrieve/\>\""}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{"path", " ", "to", " ", "place", " ", "BBRetrieveCurrent"}], 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"path", "<>", "\"\<BBRetrieveCurrent\>\""}], "]"}]}], ";", 
      " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], " ", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"StringQ", "[", "ticker", "]"}], ",", 
        RowBox[{"x", "=", "  ", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "ticker"}], "]"}]}], ",", 
        RowBox[{"x", " ", "=", " ", "ticker"}]}], "]"}], ";", 
      "\[IndentingNewLine]", " ", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"BBRetrieveCurrent", "[", 
        RowBox[{
         RowBox[{"StringJoin", "[", 
          RowBox[{"Riffle", "[", 
           RowBox[{"x", ",", "\"\< & \>\""}], "]"}], "]"}], ",", "field"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Partition", "[", 
         RowBox[{
          RowBox[{"StringSplit", "[", 
           RowBox[{"output", ",", 
            RowBox[{"{", 
             RowBox[{"\"\<{\>\"", ",", "\"\<}\>\"", ",", "\"\<,\>\""}], 
             "}"}]}], "]"}], ",", "2"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"mytickers", "=", 
       RowBox[{"output", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
         RowBox[{"output", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "output", "]"}], "==", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"NumberQ", "[", 
            RowBox[{
             RowBox[{"ToExpression", "[", "output", "]"}], "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}], "&&", 
           RowBox[{"Not", "[", 
            RowBox[{"StringMatchQ", "[", 
             RowBox[{
              RowBox[{"output", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", 
              RowBox[{
              "__", "~~", "\"\<-\>\"", "~~", "__", "~~", "\"\<-\>\"", "~~", 
               "__"}]}], "]"}], "]"}]}], ",", 
          RowBox[{"output", "=", 
           RowBox[{"ToExpression", "[", 
            RowBox[{"output", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
          RowBox[{"output", "=", 
           RowBox[{"output", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"output", " ", "=", " ", 
          RowBox[{"Map", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
            RowBox[{"output", "[", 
             RowBox[{"[", 
              RowBox[{"Flatten", "[", 
               RowBox[{"Map", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Position", "[", 
                   RowBox[{
                    RowBox[{"Map", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
                    "mytickers"}], "]"}], ",", "#"}], "]"}], "&"}], ",", 
                 RowBox[{"Map", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
                   "ticker"}], "]"}]}], "]"}], "]"}], "]"}], "]"}]}], "]"}]}],
          ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"NumberQ", "[", 
             RowBox[{
              RowBox[{"ToExpression", "[", "output", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "]"}], "&&", 
            RowBox[{"Not", "[", 
             RowBox[{"StringMatchQ", "[", 
              RowBox[{
               RowBox[{"output", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{
               "__", "~~", "\"\<-\>\"", "~~", "__", "~~", "\"\<-\>\"", "~~", 
                "__"}]}], "]"}], "]"}]}], ",", 
           RowBox[{"output", " ", "=", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"ToExpression", "[", "output", "]"}], "]"}]}], ",", 
           RowBox[{"output", "=", 
            RowBox[{"Flatten", "[", "output", "]"}]}]}], "]"}]}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", 
      RowBox[{"(*", " ", 
       RowBox[{"close", " ", "link"}], " ", "*)"}], " ", "output"}]}], 
    "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetCurrent", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<BBGetCurrent[ticker_,field_] retrieves specified data from Bloomberg \
for a given ticker, or for a {list of tickers}. \nPARAMETERS:\nticker: e.g., \
\\\"AAPL US Equity\\\" or {\\\"AAPL US Equity\\\", \\\"IBM US Equity\\\"}\n\
field:  Anything from the \\\"FLDS\\\" database on \
Bloomberg,\\\"General_Counsel\\\" for example\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBGetIntraday", "[", 
    RowBox[{"ticker_", ",", "field_", ",", "starttime_", ",", "endtime_"}], 
    "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "path", ",", "output", ",", " ", "link", ",", "st", ",", "et", ",", 
       "daylightsavingbeg", ",", "daylightsavingend", ",", "daylighttimezone",
        ",", "dateformat", ",", "x"}], "}"}], ",", " ", 
     RowBox[{
      RowBox[{
      "path", " ", "=", " ", 
       "\"\<C:/Program Files/Wolfram Research/Merrin \
Functions/BBRetrieve/\>\""}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{"path", " ", "to", " ", "place", " ", "BBRetrieveIntraday"}], 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"path", "<>", "\"\<BBRetrieveIntraday\>\""}], "]"}]}], ";", 
      " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"daylightsavingbeg", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", 
        RowBox[{"{", 
         RowBox[{"2009", ",", "3", ",", "8"}], "}"}], "]"}]}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
       "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
        "year"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"daylightsavingend", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", 
        RowBox[{"{", 
         RowBox[{"2009", ",", "11", ",", "1"}], "}"}], "]"}]}], ";", 
      RowBox[{"(*", 
       RowBox[{
       "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
        "year"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"daylighttimezone", " ", "=", " ", 
       RowBox[{"-", "4"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"dateformat", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
         "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
         "\"\<Time\>\""}], "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"st", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", "starttime", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"st", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"daylightsavingbeg", "\[LessEqual]", "st"}], "&&", 
          RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", "st"}]}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"st", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
               "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"st", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}], 
          "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"et", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", "endtime", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"et", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"daylightsavingbeg", "\[LessEqual]", "et"}], "&&", 
          RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", "et"}]}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"et", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
               "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"et", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}], 
          "]"}]}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
      RowBox[{"x", "=", 
       RowBox[{"BBRetrieveIntraday", "[", 
        RowBox[{"ticker", ",", "field", ",", "st", ",", "et"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"ToExpression", "[", "x", "]"}]}], ";", 
      RowBox[{"(*", 
       RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
        RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
        "list"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"output", "=", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"AbsoluteTime", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"StringDrop", "[", 
                RowBox[{
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
             "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"daylightsavingbeg", "\[LessEqual]", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "&&", " ", 
               RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}]}], ",", 
              RowBox[{"DateString", "[", 
               RowBox[{
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"daylighttimezone", ",", "\"\<Hour\>\""}], 
                    "}"}]}], "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                 "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
                  "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
                  "\"\<Time\>\""}], "}"}]}], "]"}], ",", 
              RowBox[{"DateString", "[", 
               RowBox[{
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"daylighttimezone", "-", "1"}], ",", 
                    "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                 "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
                  "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
                  "\"\<Time\>\""}], "}"}]}], "]"}]}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "3", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", 
      RowBox[{"(*", 
       RowBox[{"close", " ", "link"}], "*)"}], " ", "output"}]}], "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetIntraday", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<BBGetIntraday[ticker_,field_,starttime_,endtime_] retrieves intraday \
ticks from Bloomberg for a given ticker within the specified period. \n\
PARAMETERS:\nticker: e.g., \\\"AAPL Equity\\\"\nfield:  \\\"TRADE\\\" or \
\\\"BID\\\" or \\\"ASK\\\"\nstarttime: indicates start date and time e.g., \\\
\"2009-06-10T09:30:00\\\"\nendtime: indicates end date and time e.g., \
\\\"2009-06-10T016:00:00\\\"\n\nReturns a three-member list for every \
datapoint available: {time, price, size}. For a liquid security, the number \
of datapoints here is likely to be enormous, and you may want to consider \
using  BBGetIntradayBars instead.\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBGetIntradayBars", "[", 
    RowBox[{
    "ticker_", ",", "field_", ",", "starttime_", ",", "endtime_", ",", 
     "barsize_", ",", "fill_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "path", ",", "output", ",", " ", "link", ",", "st", ",", "et", ",", "x",
        ",", "daylightsavingbeg", ",", "daylightsavingend", ",", 
       "daylighttimezone", ",", "dateformat"}], "}"}], ",", " ", 
     RowBox[{
      RowBox[{
      "path", " ", "=", " ", 
       "\"\<C:/Program Files/Wolfram Research/Merrin \
Functions/BBRetrieve/\>\""}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{"path", " ", "to", " ", "place", " ", "BBRetrieveIntraday"}], 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"path", "<>", "\"\<BBRetrieveIntradayBars\>\""}], "]"}]}], 
      ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
      "\[IndentingNewLine]", " ", 
      RowBox[{"daylightsavingbeg", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", 
        RowBox[{"{", 
         RowBox[{"2009", ",", "3", ",", "8"}], "}"}], "]"}]}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
       "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
        "year"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"daylightsavingend", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", 
        RowBox[{"{", 
         RowBox[{"2009", ",", "11", ",", "1"}], "}"}], "]"}]}], ";", 
      RowBox[{"(*", 
       RowBox[{
       "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
        "year"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"daylighttimezone", " ", "=", " ", 
       RowBox[{"-", "4"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"dateformat", " ", "=", " ", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
         "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
         "\"\<Time\>\""}], "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"st", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", "starttime", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"st", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"daylightsavingbeg", "\[LessEqual]", "st"}], "&&", 
          RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", "st"}]}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"st", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
               "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"st", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}], 
          "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"et", " ", "=", " ", 
       RowBox[{"AbsoluteTime", "[", "endtime", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"et", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"daylightsavingbeg", "\[LessEqual]", "et"}], "&&", 
          RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", "et"}]}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"et", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
               "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
         RowBox[{"DateString", "[", 
          RowBox[{
           RowBox[{"DateList", "[", 
            RowBox[{"DatePlus", "[", 
             RowBox[{"et", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}], 
          "]"}]}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
      RowBox[{"x", "=", 
       RowBox[{"BBRetrieveIntradayBars", "[", 
        RowBox[{
        "ticker", ",", "field", ",", "st", ",", "et", ",", "barsize", ",", 
         "fill"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"ToExpression", "[", "x", "]"}]}], ";", 
      RowBox[{"(*", 
       RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
        RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
        "list"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"daylightsavingbeg", "\[LessEqual]", 
                RowBox[{"AbsoluteTime", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
                 "]"}]}], "&&", " ", 
               RowBox[{"daylightsavingend", "\[GreaterEqual]", " ", 
                RowBox[{"AbsoluteTime", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
                 "]"}]}]}], ",", 
              RowBox[{"DateString", "[", 
               RowBox[{
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", 
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"daylighttimezone", ",", "\"\<Hour\>\""}], 
                    "}"}]}], "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                 "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
                  "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
                  "\"\<Time\>\""}], "}"}]}], "]"}], ",", 
              RowBox[{"DateString", "[", 
               RowBox[{
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", 
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"daylighttimezone", "-", "1"}], ",", 
                    "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                 "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
                  "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
                  "\"\<Time\>\""}], "}"}]}], "]"}]}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "3", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "4", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "5", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "6", "]"}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "7", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", 
      RowBox[{"(*", 
       RowBox[{"close", " ", "link"}], "*)"}], " ", "output"}]}], "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetIntradayBars", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<BBGetIntradayBars[ticker_,field_,starttime_,endtime_,barsize_,fill_] \
retrieves intraday samples from Bloomberg for a given ticker within the \
specified period. \nPARAMETERS:\nticker: e.g., \\\"AAPL Equity\\\"\nfield:  \
\\\"TRADE\\\" or \\\"BID\\\" or \\\"ASK\\\"\nstarttime: indicates start date \
and time e.g., \\\"2009-06-10T09:30:00\\\"\nendtime: indicates end date and \
time e.g., \\\"2009-06-10T016:00:00\\\"\nbarsize: is the sample frequency, in \
minutes\nfill: If this is set to 1, the API will append previous day close \
information as first datapoint as the start of trading for the specified \
period. Otherwise, the first data point will be the actual first trading on \
that day.\n\nReturns a list of seven items for each bar \[LongDash]\n1: Time\n\
2: Open\n3: High\n4: Low\n5: Close\n6: Tick Counts per bar\n7: Shares per bar\
\n\nIf you want every trade, rather than periodic samples, use \
BBGetIntraday[].\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetNF1", "::", "deprecated"}], "=", 
    "\"\<Please use BBGetNF1b[] instead of BBGetNF1[].\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetNF1", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<BBGetNF1[ticker_,field_,start_,end_,freq_,adjust_:\\\"CALENDAR\\\",\
type_:\\\"value\\\"] retrieves data from Bloomberg for a given ticker and \
field within the specified period. \nPARAMETERS:\nticker: e.g., \\\"AAPL \
Equity\\\"\nfield: e.g., \\\"Px_Last\\\"\nstart: indicates start date e.g., \
\\\"20071031\\\"\nend: indicates end date e.g., \\\"20081031\\\"\nfreq: is \
the data frequency. Choices: \\\"day\\\", \\\"week\\\", \\\"month\\\", \
\\\"quarter\\\", \\\"year\\\"\nadjust: is the period adjustment (default is \
\\\"CALENDAR\\\"). Choices, \\\"ACTUAL\\\", \\\"CALENDAR\\\"\ntype: is NF1 \
datatype (default is \\\"value\\\"). Choices, \\\"delta\\\", \\\"pctchanges\\\
\", \\\"value\\\"\n\nThis has been deprecated in favor of BBGetNF1b[], which \
does the same thing but is a bit friendlier with inputs.\>\""}], " ", ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBGetNF1b", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<BBGetNF1[ticker_,field_,start_,end_,freq_,OptionsPattern[]] \
retrieves data from Bloomberg for a given ticker and field within the \
specified period. \nPARAMETERS:\nticker: e.g., \\\"AAPL Equity\\\"\nfield: \
e.g., \\\"Px_Last\\\"\nstart: indicates start date e.g., \\\"20071031\\\"\n\
end: indicates end date e.g., \\\"20081031\\\"\nfreq: is the data frequency. \
Choices: \\\"day\\\", \\\"week\\\", \\\"month\\\", \\\"quarter\\\", \
\\\"year\\\"\\n\n\nOptionally, you can pass \nadjust\[Rule] is the period \
adjustment (default is \\\"CALENDAR\\\"). Choices, \\\"ACTUAL\\\", \
\\\"CALENDAR\\\"\ntype\[Rule] is NF1 datatype (default is \\\"value\\\"). \
Choices, \\\"delta\\\", \\\"pctchanges\\\", \\\"value\\\"\n\
dateformat\[Rule]\\\"bbg\\\" by default, but you can change it to \
\\\"list\\\", \\\"absolutetime\\\" or \\\"conventional\\\"\n\
name\[Rule]\\\"whatever\\\" to set the name in the NF1 object.\>\""}], " ", 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"BBG2Cdates", "[", "nf1_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{
     RowBox[{"Message", "[", 
      RowBox[{"BBG2Cdates", "::", "deprecated"}], "]"}], ";", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"nf1", "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"DateString", "[", 
             RowBox[{
              RowBox[{"#", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
               "\"\<MonthShort\>\"", ",", "\"\</\>\"", ",", 
                "\"\<DayShort\>\"", ",", "\"\</\>\"", ",", "\"\<Year\>\""}], 
               "}"}]}], "]"}], ",", 
            RowBox[{"#", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], ",", 
         RowBox[{"nf1", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}]}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BBG2Cdates", "::", "usage"}], " ", "=", " ", 
   "\[IndentingNewLine]", 
   "\"\<BBG2Cdates[nf1_] converts the dates in the output of BBGetNF1 to \
conventional form (from 2008-01-01 to 1/1/2008). Most of the time this won't \
be needed. bbg2Mdates[] is more flexible and is preferred.\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"BBG2Cdates", "::", "deprecated"}], "=", 
    "\"\<Please use bbg2Mdates[] rather than the outdated BBG2Cdates[]\>\""}],
    ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "bbg2Mdates", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"format", "\[Rule]", "\"\<list\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bbg2Mdates", "[", 
   RowBox[{"nf1_List", ",", 
    RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
  RowBox[{"Switch", "[", 
   RowBox[{
    RowBox[{"OptionValue", "[", "format", "]"}], ",", "\[IndentingNewLine]", 
    "\"\<list\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"nf1", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Take", "[", 
            RowBox[{
             RowBox[{"DateList", "[", 
              RowBox[{"#", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", "3"}], "]"}], ",", 
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], ",", 
        RowBox[{"nf1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", "\"\<absolutetime\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"nf1", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"AbsoluteTime", "[", 
            RowBox[{"Take", "[", 
             RowBox[{
              RowBox[{"DateList", "[", 
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", "3"}], "]"}], 
            "]"}], ",", 
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], ",", 
        RowBox[{"nf1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "}"}], ",", 
    "\[IndentingNewLine]", "\"\<conventional\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"nf1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
      ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"DateString", "[", 
            RowBox[{
             RowBox[{
             "#1", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
             ",", 
             RowBox[{"{", 
              RowBox[{
              "\"\<MonthShort\>\"", ",", "\"\</\>\"", ",", "\"\<DayShort\>\"",
                ",", "\"\</\>\"", ",", "\"\<Year\>\""}], "}"}]}], "]"}], ",", 
           RowBox[{
           "#1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
          "}"}], "&"}], ")"}], "/@", 
       RowBox[{
       "nf1", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}]}], 
     "}"}]}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"bbg2Mdates", "::", "usage"}], "=", 
   "\"\<bbg2Mdates[nf1_] converts the dates in the output of BBGetNF1 to list \
form (from 2008-01-01 to {2008,1,1}). Can set format->\\\"absolutetime\\\" or \
\\\"conventional\\\" for other date types.\>\""}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.43377820609375*^9, 3.433778217125*^9}, {
   3.433778369359375*^9, 3.4337783988125*^9}, {3.433778468640625*^9, 
   3.433778489890625*^9}, {3.433778538796875*^9, 3.4337785634375*^9}, {
   3.433778790203125*^9, 3.433778845015625*^9}, {3.433778883484375*^9, 
   3.43377897465625*^9}, {3.43377905840625*^9, 3.4337794041875*^9}, {
   3.43377961803125*^9, 3.433779682890625*^9}, {3.433779729625*^9, 
   3.433779897828125*^9}, {3.4337799566875*^9, 3.4337799731875*^9}, {
   3.433780884796875*^9, 3.433780946109375*^9}, {3.433781254125*^9, 
   3.433781322390625*^9}, {3.433781357421875*^9, 3.43378158246875*^9}, 
   3.433781778125*^9, {3.433782220828125*^9, 3.43378224171875*^9}, {
   3.43378227840625*^9, 3.433782299484375*^9}, {3.433782354078125*^9, 
   3.43378242434375*^9}, {3.4337824698125*^9, 3.4337824986875*^9}, {
   3.433782655109375*^9, 3.433782658078125*^9}, {3.43378277053125*^9, 
   3.433782774078125*^9}, {3.433783084765625*^9, 3.433783159296875*^9}, {
   3.4337836699375*^9, 3.433783686203125*^9}, {3.43378376453125*^9, 
   3.43378379465625*^9}, {3.4337839035625*^9, 3.43378393*^9}, {
   3.4337840006875*^9, 3.433784010015625*^9}, {3.43378433059375*^9, 
   3.43378439140625*^9}, {3.43378443309375*^9, 3.433784477609375*^9}, {
   3.433784914015625*^9, 3.433784925765625*^9}, {3.449484662119875*^9, 
   3.4494846632605*^9}, {3.451214767875*^9, 3.451214772609375*^9}, {
   3.451214839390625*^9, 3.45121488534375*^9}, {3.451215319578125*^9, 
   3.451215326109375*^9}, {3.45121551015625*^9, 3.451215610640625*^9}, {
   3.45121574925*^9, 3.4512157503125*^9}, {3.451215789109375*^9, 
   3.4512158235625*^9}, {3.45121654265625*^9, 3.451216554578125*^9}, {
   3.45122493940625*^9, 3.451224945359375*^9}, {3.453720309296875*^9, 
   3.453720373859375*^9}, {3.453722020296875*^9, 3.4537221075*^9}, {
   3.453722138390625*^9, 3.4537222125*^9}, {3.453722867875*^9, 
   3.45372286840625*^9}, {3.45372289959375*^9, 3.4537229055625*^9}, {
   3.4537229406875*^9, 3.453722942203125*^9}, {3.453723266109375*^9, 
   3.4537233575*^9}, {3.453723671984375*^9, 3.453723679453125*^9}, {
   3.45372474390625*^9, 3.453724763796875*^9}, {3.45372495090625*^9, 
   3.4537249681875*^9}, {3.4537250716875*^9, 3.453725085953125*^9}, {
   3.453725140265625*^9, 3.45372516584375*^9}, {3.453725229140625*^9, 
   3.453725280875*^9}, {3.453726303921875*^9, 3.45372640371875*^9}, {
   3.45372648965625*^9, 3.453726493890625*^9}, {3.45372677446875*^9, 
   3.453726790484375*^9}, {3.454587560147625*^9, 3.4545875931945*^9}, {
   3.454587712491375*^9, 3.45458773335075*^9}, {3.45458779828825*^9, 
   3.454587813741375*^9}, {3.45468062529675*^9, 3.4546806765155*^9}, {
   3.45468071492175*^9, 3.45468099385925*^9}, {3.454681101093625*^9, 
   3.454681168578*^9}, {3.454681264281125*^9, 3.45468127704675*^9}, {
   3.4552881827285*^9, 3.455288186462875*^9}, {3.455288241087875*^9, 
   3.455288255712875*^9}, {3.455288573806625*^9, 3.455288596291*^9}, {
   3.45528868075975*^9, 3.45528868719725*^9}, {3.455289061306625*^9, 
   3.455289062962875*^9}, {3.45528922063475*^9, 3.455289289150375*^9}, {
   3.455289391400375*^9, 3.455289402494125*^9}, {3.45528944500975*^9, 
   3.455289495681625*^9}, {3.455289528025375*^9, 3.455289531916*^9}, {
   3.455289602056625*^9, 3.455289645166*^9}, {3.45528996400975*^9, 
   3.455290069931625*^9}, {3.455290252291*^9, 3.455290341025375*^9}, {
   3.455290674541*^9, 3.45529072182225*^9}, {3.45529085113475*^9, 
   3.455290897025375*^9}, {3.455291050244125*^9, 3.455291062416*^9}, {
   3.455291173275375*^9, 3.45529123819725*^9}, {3.455291320712875*^9, 
   3.455291354494125*^9}, {3.455291486525375*^9, 3.45529149082225*^9}, {
   3.4552915826035*^9, 3.455291590650375*^9}, {3.455291871525375*^9, 
   3.455291880041*^9}, {3.45529204319725*^9, 3.45529207863475*^9}, {
   3.4554578317285*^9, 3.45545787838475*^9}, {3.455458073619125*^9, 
   3.45545808157225*^9}, {3.4554581147285*^9, 3.455458120025375*^9}, {
   3.45545910250975*^9, 3.45545916763475*^9}, {3.455459276869125*^9, 
   3.45545931825975*^9}, {3.455459634994125*^9, 3.455459642056625*^9}, {
   3.455459690650375*^9, 3.45545973613475*^9}, {3.455459797306625*^9, 
   3.455459798462875*^9}, {3.455459860681625*^9, 3.45545986132225*^9}, {
   3.4554599044785*^9, 3.45545993744725*^9}, {3.455460045837875*^9, 
   3.45546004688475*^9}, {3.45546026813475*^9, 3.455460272806625*^9}, {
   3.45546059400975*^9, 3.45546066425975*^9}, {3.455460694400375*^9, 
   3.455460717244125*^9}, {3.455460798541*^9, 3.455460887900375*^9}, {
   3.455461010400375*^9, 3.455461048166*^9}, {3.455461084041*^9, 
   3.45546111444725*^9}, {3.45546122513475*^9, 3.455461234931625*^9}, {
   3.45546146963475*^9, 3.4554614739785*^9}, {3.455461523775375*^9, 
   3.45546153900975*^9}, {3.4554616149785*^9, 3.45546163294725*^9}, {
   3.45546179269725*^9, 3.455461796275375*^9}, {3.455464796712875*^9, 
   3.455464836041*^9}, {3.455464866494125*^9, 3.455464874494125*^9}, {
   3.455464932962875*^9, 3.4554649664785*^9}, {3.455465118931625*^9, 
   3.45546520775975*^9}, {3.455465384369125*^9, 3.45546538863475*^9}, {
   3.455465549837875*^9, 3.455465572119125*^9}, {3.45546563457225*^9, 
   3.45546563869725*^9}, {3.459185233875*^9, 3.45918524546875*^9}, {
   3.459185435109375*^9, 3.459185441234375*^9}, {3.45918557815625*^9, 
   3.459185583578125*^9}, {3.459187231875*^9, 3.459187239453125*^9}, {
   3.459187282234375*^9, 3.459187326484375*^9}, {3.459529027456584*^9, 
   3.459529046534709*^9}, {3.459532667394084*^9, 3.459532678050334*^9}, {
   3.459532741128459*^9, 3.459532768284709*^9}, {3.459533542565959*^9, 
   3.459533546144084*^9}, {3.459534496519084*^9, 3.459534536675334*^9}, {
   3.459534620300334*^9, 3.459534728894084*^9}, {3.459534915940959*^9, 
   3.459534982831584*^9}, 3.459535023722209*^9, {3.459535362222209*^9, 
   3.459535465019084*^9}, {3.459535502128459*^9, 3.459535548972209*^9}, {
   3.459535585190959*^9, 3.459535707987834*^9}, {3.459535856534709*^9, 
   3.459535864237834*^9}, {3.459536151175334*^9, 3.459536222284709*^9}, 
   3.459536318300334*^9, {3.459537205144084*^9, 3.459537233909709*^9}, {
   3.459537568847209*^9, 3.459537569409709*^9}, {3.459537695534709*^9, 
   3.459537839206584*^9}, {3.459537888769084*^9, 3.459537973987834*^9}, {
   3.459538071300334*^9, 3.459538072112834*^9}, {3.459538155347209*^9, 
   3.459538159519084*^9}, {3.459538236456584*^9, 3.459538237159709*^9}, {
   3.459538576409709*^9, 3.459538629019084*^9}, {3.459538678378459*^9, 
   3.459538679487834*^9}, {3.459538711581584*^9, 3.459538919940959*^9}, 
   3.459539029862834*^9, {3.462038129234375*^9, 3.46203813575*^9}, {
   3.46385132571227*^9, 3.46385134505602*^9}, 3.4657441558764753`*^9, {
   3.4657442125610733`*^9, 3.4657442129673023`*^9}, {3.5030681517865*^9, 
   3.503068190802125*^9}, {3.503068220864625*^9, 3.503068228599*^9}, {
   3.503068278927125*^9, 3.503068315270875*^9}, {3.503068670677125*^9, 
   3.5030687025365*^9}, {3.50306873606775*^9, 3.50306875813025*^9}, {
   3.548603887564625*^9, 3.548603940939625*^9}, {3.54860416989275*^9, 
   3.5486041943615*^9}, {3.548604243127125*^9, 3.54860425789275*^9}, {
   3.548604323049*^9, 3.548604446595875*^9}, {3.54860625558025*^9, 
   3.548606495939625*^9}, {3.5486067067365*^9, 3.548606706877125*^9}, {
   3.548606782002125*^9, 3.54860681533025*^9}, {3.548673457924*^9, 
   3.548673467877125*^9}, {3.54867358195525*^9, 3.548673739314625*^9}, {
   3.633136516942274*^9, 3.633136554136145*^9}, {3.6331366214795094`*^9, 
   3.633136621634354*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<IBM Equity\>\"", ",", "\"\<AAPL Equity\>\""}], "}"}], ",", 
     "\"\<PX_LAST\>\""}], "]"}], "[", 
   RowBox[{"[", "2", "]"}], "]"}], "*", "2"}]], "Input",
 CellChangeTimes->{{3.455290478791*^9, 3.455290479541*^9}, {
  3.45529072813475*^9, 3.45529074138475*^9}, {3.455290797431625*^9, 
  3.455290806087875*^9}, {3.455290906791*^9, 3.455290964650375*^9}, {
  3.455291070244125*^9, 3.45529108969725*^9}, {3.455292108587875*^9, 
  3.455292116525375*^9}, {3.455292361244125*^9, 3.455292364556625*^9}}],

Cell[BoxData["285.896`"], "Output",
 CellChangeTimes->{{3.455291377775375*^9, 3.45529140407225*^9}, 
   3.45529188863475*^9, 3.455292116916*^9, {3.455292357666*^9, 
   3.455292365041*^9}, {3.455465650181625*^9, 3.45546566438475*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BBGetCurrent", "[", 
  RowBox[{"\"\<IBM Equity\>\"", ",", "\"\<Open\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.4552916091035*^9, 3.455291637994125*^9}, {
  3.45529172294725*^9, 3.45529175088475*^9}, {3.45529209413475*^9, 
  3.455292127712875*^9}, {3.455376403166*^9, 3.455376404369125*^9}, {
  3.455376581494125*^9, 3.455376583650375*^9}}],

Cell[BoxData["105.`"], "Output",
 CellChangeTimes->{{3.455291626666*^9, 3.4552916386035*^9}, {
   3.455291725712875*^9, 3.45529175144725*^9}, 3.45529189100975*^9, {
   3.4552920889785*^9, 3.45529212838475*^9}, 3.455376405369125*^9, 
   3.455376584400375*^9, 3.455465405494125*^9, 3.455465661494125*^9, 
   3.45546583913475*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<AAPL US Equity\>\""}], "}"}],
      ",", "\"\<Name\>\""}], "]"}], ",", 
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<General_Counsel\>\""}], 
    "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.455291454837875*^9, 3.455291464025375*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"INTL BUSINESS MACHINES CORP\"\>", ",", "\<\"APPLE INC\"\>"}],
     "}"}], ",", "\<\"DAVID KAPPOS\"\>"}], "}"}]], "Output",
 CellChangeTimes->{{3.45529144569725*^9, 3.455291465025375*^9}, 
   3.455291502275375*^9, 3.45529159969725*^9, 3.455291718041*^9, 
   3.455292200181625*^9, 3.4552923547285*^9, 3.455465655931625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BBGetCurrent", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<UYG US Equity\>\"", ",", "\"\<SKF US Equity\>\"", ",", 
     "\"\<UWM US Equity\>\"", ",", "\"\<RWM US Equity\>\"", ",", 
     "\"\<SDD US Equity\>\"", ",", "\"\<SEF US Equity\>\"", ",", 
     "\"\<DAG US Equity\>\"", ",", "\"\<SAA US Equity\>\"", ",", 
     "\"\<SFK US Equity\>\"", ",", "\"\<SJH US Equity\>\"", ",", 
     "\"\<RRZ US Equity\>\"", ",", "\"\<UVT US Equity\>\"", ",", 
     "\"\<RRY US Equity\>\"", ",", "\"\<UKK US Equity\>\"", ",", 
     "\"\<RFL US Equity\>\"", ",", "\"\<RFN US Equity\>\"", ",", 
     "\"\<IBM US Equity\>\"", ",", "\"\<AAPL US Equity\>\"", ",", 
     "\"\<POT US Equity\>\"", ",", " ", "\"\<YHOO US Equity\>\"", ",", 
     "\"\<FST US Equity\>\"", ",", " ", "\"\<SPY US Equity\>\""}], "}"}], 
   ",", "\"\<Last_Price\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.45546567700975*^9, 3.455465825712875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3.94`", ",", "41.5614`", ",", "19.94`", ",", "55.7`", ",", "42.74`", ",", 
   "57.07`", ",", "9.52`", ",", "21.57`", ",", "54.64`", ",", "46.325`", ",", 
   "58.82`", ",", "14.85`", ",", "23.46`", ",", "22.49`", ",", "8.97`", ",", 
   "9.89`", ",", "105.12`", ",", "142.86`", ",", "91.57`", ",", "15.395`", 
   ",", "15.`", ",", "92.71`"}], "}"}]], "Output",
 CellChangeTimes->{{3.455465729837875*^9, 3.455465751291*^9}, {
  3.455465795556625*^9, 3.455465826900375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"bbg2Mdates", "[", 
   RowBox[{"BBGetNF1", "[", 
    RowBox[{
    "\"\<AAPL Equity\>\"", ",", "\"\<Px_Last\>\"", ",", "\"\<20070901\>\"", 
     ",", "\"\<20081020\>\"", ",", "\"\<monthly\>\""}], "]"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.433778445828125*^9, 3.433778459515625*^9}, {
   3.43377970003125*^9, 3.433779713328125*^9}, {3.433779992421875*^9, 
   3.43378003990625*^9}, {3.4337817308125*^9, 3.433781733046875*^9}, 
   3.433782531671875*^9, {3.433782590375*^9, 3.4337826209375*^9}, {
   3.43378382275*^9, 3.43378382365625*^9}, {3.433783862046875*^9, 
   3.433783880609375*^9}, {3.43378394296875*^9, 3.43378394953125*^9}, {
   3.43378495765625*^9, 3.43378499284375*^9}, {3.454681185437375*^9, 
   3.454681186218625*^9}, {3.503068827895875*^9, 3.503068830020875*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"nf1\"\>", ",", "\<\"AAPL Equity: Px_Last\"\>", 
     ",", "\<\"monthly\"\>", ",", "\<\"value\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2007", ",", "9", ",", "28"}], "}"}], ",", "153.47`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2007", ",", "10", ",", "31"}], "}"}], ",", "189.95`"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2007", ",", "11", ",", "30"}], "}"}], ",", "182.22`"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2007", ",", "12", ",", "31"}], "}"}], ",", "198.08`"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "1", ",", "31"}], "}"}], ",", "135.36`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "2", ",", "29"}], "}"}], ",", "125.02`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "3", ",", "31"}], "}"}], ",", "143.5`"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "4", ",", "30"}], "}"}], ",", "173.95`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "5", ",", "30"}], "}"}], ",", "188.75`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "6", ",", "30"}], "}"}], ",", "167.44`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "7", ",", "31"}], "}"}], ",", "158.95`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "8", ",", "29"}], "}"}], ",", "169.53`"}], "}"}],
      ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2008", ",", "9", ",", "30"}], "}"}], ",", "113.66`"}], 
      "}"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.454670428062375*^9, 3.4546704337655*^9}, 
   3.454670622187375*^9, 3.454681187062375*^9, 3.50306883119275*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"BBGetIntraday", "[", 
   RowBox[{
   "\"\<SPX Index\>\"", ",", "\"\<TRADE\>\"", ",", 
    "\"\<2009-02-03T09:30:00\>\"", ",", "\"\<2009-02-03T16:30:00\>\""}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.453722229078125*^9, 3.45372224440625*^9}, {
   3.45372252646875*^9, 3.453722563296875*^9}, {3.45372283796875*^9, 
   3.453722852375*^9}, {3.453723053*^9, 3.453723074375*^9}, {
   3.453723404671875*^9, 3.453723405109375*^9}, {3.453723643*^9, 
   3.453723643421875*^9}, {3.45372459578125*^9, 3.45372459971875*^9}, {
   3.453724728734375*^9, 3.45372473225*^9}, {3.45372511259375*^9, 
   3.453725113015625*^9}, {3.4592743951875*^9, 3.459274444046875*^9}, {
   3.459274482484375*^9, 3.45927448271875*^9}, {3.459274547890625*^9, 
   3.45927459928125*^9}, {3.459274696375*^9, 3.45927470203125*^9}, {
   3.4592748145625*^9, 3.459274836265625*^9}, {3.459526650144084*^9, 
   3.459526656644084*^9}, {3.459526911097209*^9, 3.459526921456584*^9}, {
   3.459528595940959*^9, 3.459528596394084*^9}, {3.459528738690959*^9, 
   3.459528742753459*^9}, 3.459528828269084*^9, {3.459528866784709*^9, 
   3.459528869409709*^9}, {3.459528908925334*^9, 3.459528909565959*^9}, {
   3.459528946659709*^9, 3.459528946925334*^9}, {3.459530212628459*^9, 
   3.459530217050334*^9}, {3.459536738925334*^9, 3.459536748456584*^9}, {
   3.459537472362834*^9, 3.459537509644084*^9}, {3.459537555878459*^9, 
   3.459537559894084*^9}, {3.459538035800334*^9, 3.459538035987834*^9}, {
   3.459538288581584*^9, 3.459538288737834*^9}, {3.459538354831584*^9, 
   3.459538363269084*^9}, {3.459538432909709*^9, 3.459538433222209*^9}, {
   3.459538939315959*^9, 3.459538939847209*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:30:02\"\>", ",", "825.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:30:17\"\>", ",", "827.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:30:32\"\>", ",", "827.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:30:47\"\>", ",", "829.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:31:02\"\>", ",", "829.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:31:17\"\>", ",", "829.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:31:32\"\>", ",", "830.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:31:47\"\>", ",", "829.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:32:02\"\>", ",", "830.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:32:17\"\>", ",", "830.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:32:32\"\>", ",", "830.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:32:47\"\>", ",", "830.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:33:02\"\>", ",", "830.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:33:17\"\>", ",", "829.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:33:32\"\>", ",", "829.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:33:47\"\>", ",", "829.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:34:02\"\>", ",", "828.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:34:17\"\>", ",", "828.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:34:32\"\>", ",", "828.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:34:47\"\>", ",", "828.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:35:02\"\>", ",", "828.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:35:17\"\>", ",", "828.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:35:32\"\>", ",", "829.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:35:47\"\>", ",", "829.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:36:02\"\>", ",", "829.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:36:17\"\>", ",", "829.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:36:32\"\>", ",", "828.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:36:47\"\>", ",", "828.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:37:02\"\>", ",", "827.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:37:17\"\>", ",", "827.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:37:32\"\>", ",", "827.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:37:47\"\>", ",", "827.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:38:02\"\>", ",", "827.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:38:17\"\>", ",", "826.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:38:32\"\>", ",", "826.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:38:47\"\>", ",", "826.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:39:02\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:39:17\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:39:32\"\>", ",", "826.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:39:47\"\>", ",", "826.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:40:02\"\>", ",", "826.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:40:17\"\>", ",", "827.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:40:32\"\>", ",", "827.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:40:47\"\>", ",", "826.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:41:02\"\>", ",", "826.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:41:17\"\>", ",", "825.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:41:32\"\>", ",", "824.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:41:47\"\>", ",", "824.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:42:02\"\>", ",", "825.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:42:17\"\>", ",", "825.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:42:32\"\>", ",", "826.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:42:47\"\>", ",", "826.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:43:02\"\>", ",", "826.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:43:17\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:43:32\"\>", ",", "826.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:43:47\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:44:02\"\>", ",", "826.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:44:17\"\>", ",", "825.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:44:32\"\>", ",", "825.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:44:47\"\>", ",", "825.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:45:02\"\>", ",", "825.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:45:17\"\>", ",", "824.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:45:32\"\>", ",", "824.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:45:47\"\>", ",", "824.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:46:02\"\>", ",", "823.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:46:17\"\>", ",", "823.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:46:32\"\>", ",", "823.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:46:47\"\>", ",", "823.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:47:02\"\>", ",", "823.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:47:17\"\>", ",", "823.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:47:32\"\>", ",", "823.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:47:47\"\>", ",", "823.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:48:02\"\>", ",", "823.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:48:17\"\>", ",", "822.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:48:32\"\>", ",", "822.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:48:47\"\>", ",", "822.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:49:02\"\>", ",", "823.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:49:17\"\>", ",", "822.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:49:32\"\>", ",", "823.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:49:47\"\>", ",", "823.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:50:02\"\>", ",", "823.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:50:17\"\>", ",", "824.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:50:32\"\>", ",", "824.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:50:47\"\>", ",", "823.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:51:02\"\>", ",", "823.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:51:17\"\>", ",", "822.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:51:32\"\>", ",", "821.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:51:47\"\>", ",", "821.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:52:02\"\>", ",", "822.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:52:17\"\>", ",", "822.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:52:32\"\>", ",", "822.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:52:47\"\>", ",", "822.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:53:02\"\>", ",", "822.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:53:17\"\>", ",", "822.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:53:32\"\>", ",", "823.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:53:47\"\>", ",", "822.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:54:02\"\>", ",", "823.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:54:17\"\>", ",", "822.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:54:32\"\>", ",", "822.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:54:47\"\>", ",", "822.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:55:02\"\>", ",", "822.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:55:17\"\>", ",", "822.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:55:32\"\>", ",", "822.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:55:47\"\>", ",", "823.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:56:02\"\>", ",", "822.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:56:17\"\>", ",", "822.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:56:32\"\>", ",", "822.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:56:47\"\>", ",", "823.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:57:02\"\>", ",", "823.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:57:17\"\>", ",", "823.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:57:32\"\>", ",", "823.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:57:47\"\>", ",", "824.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:58:02\"\>", ",", "823.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:58:17\"\>", ",", "824.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:58:32\"\>", ",", "824.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:58:47\"\>", ",", "824.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:59:02\"\>", ",", "824.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:59:17\"\>", ",", "824.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:59:32\"\>", ",", "823.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:59:47\"\>", ",", "823.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:00:02\"\>", ",", "824.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:00:17\"\>", ",", "825.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:00:32\"\>", ",", "826.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:00:47\"\>", ",", "825.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:01:02\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:01:17\"\>", ",", "826.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:01:32\"\>", ",", "826.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:01:47\"\>", ",", "826.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:02:02\"\>", ",", "825.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:02:17\"\>", ",", "825.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:02:32\"\>", ",", "824.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:02:47\"\>", ",", "825.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:03:02\"\>", ",", "825.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:03:17\"\>", ",", "825.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:03:32\"\>", ",", "825.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:03:47\"\>", ",", "824.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:04:02\"\>", ",", "824.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:04:17\"\>", ",", "824.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:04:32\"\>", ",", "823.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:04:47\"\>", ",", "823.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:05:02\"\>", ",", "824.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:05:17\"\>", ",", "824.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:05:32\"\>", ",", "825.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:05:47\"\>", ",", "825.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:06:02\"\>", ",", "825.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:06:17\"\>", ",", "826.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:06:32\"\>", ",", "826.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:06:47\"\>", ",", "826.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:07:02\"\>", ",", "826.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:07:17\"\>", ",", "826.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:07:32\"\>", ",", "827.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:07:47\"\>", ",", "827.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:08:02\"\>", ",", "828.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:08:17\"\>", ",", "828.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:08:32\"\>", ",", "828.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:08:47\"\>", ",", "827.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:09:02\"\>", ",", "827.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:09:17\"\>", ",", "827.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:09:32\"\>", ",", "827.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:09:47\"\>", ",", "827.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:10:02\"\>", ",", "827.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:10:17\"\>", ",", "827.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:10:32\"\>", ",", "827.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:10:47\"\>", ",", "827.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:11:02\"\>", ",", "827.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:11:17\"\>", ",", "827.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:11:32\"\>", ",", "827.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:11:47\"\>", ",", "827.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:12:02\"\>", ",", "827.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:12:17\"\>", ",", "827.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:12:32\"\>", ",", "827.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:12:47\"\>", ",", "827.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:13:02\"\>", ",", "826.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:13:17\"\>", ",", "826.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:13:32\"\>", ",", "825.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:13:47\"\>", ",", "825.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:14:02\"\>", ",", "825.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:14:17\"\>", ",", "826.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:14:32\"\>", ",", "826.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:14:47\"\>", ",", "826.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:15:02\"\>", ",", "826.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:15:17\"\>", ",", "826.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:15:32\"\>", ",", "826.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:15:47\"\>", ",", "826.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:16:02\"\>", ",", "826.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:16:17\"\>", ",", "826.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:16:32\"\>", ",", "827.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:16:47\"\>", ",", "827.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:17:02\"\>", ",", "826.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:17:17\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:17:32\"\>", ",", "826.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:17:47\"\>", ",", "826.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:18:02\"\>", ",", "826.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:18:17\"\>", ",", "826.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:18:32\"\>", ",", "826.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:18:47\"\>", ",", "826.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:19:02\"\>", ",", "825.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:19:17\"\>", ",", "824.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:19:32\"\>", ",", "823.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:19:47\"\>", ",", "823.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:20:02\"\>", ",", "823.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:20:17\"\>", ",", "823.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:20:32\"\>", ",", "823.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:20:47\"\>", ",", "823.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:21:02\"\>", ",", "823.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:21:17\"\>", ",", "823.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:21:32\"\>", ",", "823.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:21:47\"\>", ",", "823.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:22:02\"\>", ",", "823.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:22:17\"\>", ",", "822.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:22:32\"\>", ",", "822.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:22:47\"\>", ",", "823.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:23:02\"\>", ",", "823.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:23:17\"\>", ",", "823.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:23:32\"\>", ",", "823.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:23:47\"\>", ",", "823.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:24:02\"\>", ",", "823.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:24:17\"\>", ",", "823.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:24:32\"\>", ",", "823.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:24:47\"\>", ",", "823.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:25:02\"\>", ",", "823.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:25:17\"\>", ",", "822.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:25:32\"\>", ",", "822.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:25:47\"\>", ",", "822.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:26:02\"\>", ",", "823.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:26:17\"\>", ",", "823.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:26:32\"\>", ",", "823.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:26:47\"\>", ",", "823.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:27:02\"\>", ",", "823.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:27:17\"\>", ",", "823.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:27:32\"\>", ",", "824.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:27:47\"\>", ",", "824.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:28:02\"\>", ",", "824.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:28:17\"\>", ",", "824.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:28:32\"\>", ",", "825.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:28:47\"\>", ",", "825.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:29:02\"\>", ",", "825.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:29:17\"\>", ",", "825.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:29:32\"\>", ",", "825.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:29:47\"\>", ",", "825.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:30:02\"\>", ",", "825.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:30:17\"\>", ",", "824.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:30:32\"\>", ",", "825.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:30:47\"\>", ",", "825.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:31:02\"\>", ",", "825.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:31:17\"\>", ",", "825.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:31:32\"\>", ",", "825.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:31:47\"\>", ",", "825.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:32:02\"\>", ",", "825.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:32:17\"\>", ",", "825.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:32:32\"\>", ",", "825.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:32:47\"\>", ",", "825.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:33:02\"\>", ",", "825.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:33:17\"\>", ",", "826.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:33:32\"\>", ",", "825.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:33:47\"\>", ",", "824.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:34:02\"\>", ",", "824.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:34:17\"\>", ",", "824.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:34:32\"\>", ",", "824.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:34:47\"\>", ",", "824.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:35:02\"\>", ",", "825.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:35:17\"\>", ",", "827.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:35:32\"\>", ",", "826.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:35:47\"\>", ",", "827.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:36:02\"\>", ",", "826.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:36:17\"\>", ",", "826.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:36:32\"\>", ",", "826.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:36:47\"\>", ",", "826.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:37:02\"\>", ",", "827.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:37:17\"\>", ",", "827.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:37:32\"\>", ",", "827.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:37:47\"\>", ",", "827.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:38:02\"\>", ",", "827.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:38:17\"\>", ",", "826.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:38:32\"\>", ",", "826.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:38:47\"\>", ",", "826.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:39:02\"\>", ",", "826.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:39:17\"\>", ",", "826.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:39:32\"\>", ",", "827.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:39:47\"\>", ",", "827.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:40:02\"\>", ",", "826.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:40:17\"\>", ",", "826.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:40:32\"\>", ",", "826.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:40:47\"\>", ",", "826.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:41:02\"\>", ",", "827.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:41:17\"\>", ",", "827.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:41:32\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:41:47\"\>", ",", "826.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:42:02\"\>", ",", "826.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:42:17\"\>", ",", "826.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:42:32\"\>", ",", "826.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:42:47\"\>", ",", "827.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:43:02\"\>", ",", "826.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:43:17\"\>", ",", "826.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:43:32\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:43:47\"\>", ",", "827.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:44:02\"\>", ",", "827.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:44:17\"\>", ",", "827.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:44:32\"\>", ",", "827.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:44:47\"\>", ",", "827.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:45:02\"\>", ",", "828.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:45:17\"\>", ",", "829.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:45:32\"\>", ",", "829.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:45:47\"\>", ",", "829.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:46:02\"\>", ",", "829.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:46:17\"\>", ",", "829.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:46:32\"\>", ",", "829.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:46:47\"\>", ",", "829.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:47:02\"\>", ",", "829.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:47:17\"\>", ",", "829.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:47:32\"\>", ",", "829.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:47:47\"\>", ",", "829.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:48:02\"\>", ",", "830.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:48:17\"\>", ",", "830.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:48:32\"\>", ",", "830.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:48:47\"\>", ",", "830.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:49:02\"\>", ",", "830.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:49:17\"\>", ",", "830.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:49:32\"\>", ",", "829.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:49:47\"\>", ",", "829.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:50:02\"\>", ",", "829.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:50:17\"\>", ",", "829.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:50:32\"\>", ",", "829.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:50:47\"\>", ",", "830.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:51:02\"\>", ",", "830.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:51:17\"\>", ",", "829.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:51:32\"\>", ",", "829.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:51:47\"\>", ",", "829.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:52:02\"\>", ",", "829.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:52:17\"\>", ",", "829.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:52:32\"\>", ",", "829.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:52:47\"\>", ",", "829.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:53:02\"\>", ",", "829.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:53:17\"\>", ",", "829.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:53:32\"\>", ",", "828.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:53:47\"\>", ",", "828.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:54:02\"\>", ",", "828.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:54:17\"\>", ",", "828.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:54:32\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:54:47\"\>", ",", "828.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:55:02\"\>", ",", "828.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:55:17\"\>", ",", "828.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:55:32\"\>", ",", "827.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:55:47\"\>", ",", "828.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:56:02\"\>", ",", "828.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:56:17\"\>", ",", "828.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:56:32\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:56:47\"\>", ",", "828.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:57:02\"\>", ",", "828.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:57:17\"\>", ",", "828.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:57:32\"\>", ",", "827.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:57:47\"\>", ",", "827.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:58:02\"\>", ",", "827.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:58:17\"\>", ",", "827.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:58:32\"\>", ",", "827.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:58:47\"\>", ",", "827.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:59:02\"\>", ",", "828.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:59:17\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:59:32\"\>", ",", "827.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:59:47\"\>", ",", "827.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:00:02\"\>", ",", "828.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:00:17\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:00:32\"\>", ",", "828.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:00:47\"\>", ",", "828.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:01:02\"\>", ",", "828.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:01:17\"\>", ",", "828.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:01:32\"\>", ",", "828.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:01:47\"\>", ",", "828.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:02:02\"\>", ",", "828.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:02:17\"\>", ",", "828.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:02:32\"\>", ",", "828.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:02:47\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:03:02\"\>", ",", "828.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:03:17\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:03:32\"\>", ",", "828.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:03:47\"\>", ",", "828.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:04:02\"\>", ",", "829.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:04:17\"\>", ",", "829.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:04:32\"\>", ",", "829.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:04:47\"\>", ",", "830.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:05:02\"\>", ",", "829.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:05:17\"\>", ",", "829.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:05:32\"\>", ",", "829.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:05:47\"\>", ",", "830.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:06:02\"\>", ",", "830.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:06:17\"\>", ",", "830.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:06:32\"\>", ",", "830.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:06:47\"\>", ",", "830.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:07:02\"\>", ",", "830.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:07:17\"\>", ",", "831.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:07:32\"\>", ",", "831.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:07:47\"\>", ",", "831.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:08:02\"\>", ",", "831.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:08:17\"\>", ",", "832.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:08:32\"\>", ",", "832.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:08:47\"\>", ",", "832.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:09:02\"\>", ",", "831.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:09:17\"\>", ",", "831.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:09:32\"\>", ",", "831.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:09:47\"\>", ",", "831.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:10:02\"\>", ",", "831.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:10:17\"\>", ",", "831.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:10:32\"\>", ",", "831.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:10:47\"\>", ",", "832.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:11:02\"\>", ",", "832.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:11:17\"\>", ",", "831.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:11:32\"\>", ",", "831.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:11:47\"\>", ",", "831.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:12:02\"\>", ",", "831.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:12:17\"\>", ",", "831.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:12:32\"\>", ",", "832.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:12:47\"\>", ",", "831.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:13:02\"\>", ",", "832.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:13:17\"\>", ",", "831.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:13:32\"\>", ",", "832.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:13:47\"\>", ",", "832.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:14:02\"\>", ",", "831.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:14:17\"\>", ",", "831.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:14:32\"\>", ",", "831.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:14:47\"\>", ",", "831.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:15:02\"\>", ",", "832.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:15:17\"\>", ",", "832.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:15:32\"\>", ",", "832.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:15:47\"\>", ",", "832.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:16:02\"\>", ",", "832.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:16:17\"\>", ",", "832.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:16:32\"\>", ",", "831.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:16:47\"\>", ",", "831.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:17:02\"\>", ",", "830.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:17:17\"\>", ",", "830.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:17:32\"\>", ",", "830.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:17:47\"\>", ",", "830.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:18:02\"\>", ",", "830.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:18:17\"\>", ",", "830.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:18:32\"\>", ",", "830.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:18:47\"\>", ",", "831.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:19:02\"\>", ",", "830.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:19:17\"\>", ",", "830.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:19:32\"\>", ",", "830.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:19:47\"\>", ",", "830.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:20:02\"\>", ",", "830.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:20:17\"\>", ",", "831.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:20:32\"\>", ",", "830.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:20:47\"\>", ",", "830.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:21:02\"\>", ",", "830.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:21:17\"\>", ",", "830.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:21:32\"\>", ",", "830.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:21:47\"\>", ",", "830.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:22:02\"\>", ",", "830.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:22:17\"\>", ",", "830.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:22:32\"\>", ",", "830.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:22:47\"\>", ",", "830.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:23:02\"\>", ",", "831.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:23:17\"\>", ",", "832.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:23:32\"\>", ",", "832.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:23:47\"\>", ",", "832.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:24:02\"\>", ",", "832.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:24:17\"\>", ",", "832.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:24:32\"\>", ",", "832.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:24:47\"\>", ",", "831.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:25:02\"\>", ",", "831.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:25:17\"\>", ",", "831.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:25:32\"\>", ",", "831.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:25:47\"\>", ",", "831.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:26:02\"\>", ",", "831.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:26:17\"\>", ",", "831.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:26:32\"\>", ",", "831.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:26:47\"\>", ",", "831.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:27:02\"\>", ",", "831.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:27:17\"\>", ",", "831.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:27:32\"\>", ",", "831.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:27:47\"\>", ",", "831.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:28:02\"\>", ",", "831.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:28:17\"\>", ",", "831.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:28:32\"\>", ",", "830.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:28:47\"\>", ",", "831.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:29:02\"\>", ",", "831.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:29:17\"\>", ",", "831.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:29:32\"\>", ",", "830.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:29:47\"\>", ",", "830.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:30:02\"\>", ",", "830.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:30:17\"\>", ",", "830.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:30:32\"\>", ",", "830.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:30:47\"\>", ",", "830.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:31:02\"\>", ",", "830.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:31:17\"\>", ",", "831.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:31:32\"\>", ",", "831.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:31:47\"\>", ",", "830.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:32:02\"\>", ",", "830.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:32:17\"\>", ",", "830.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:32:32\"\>", ",", "830.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:32:47\"\>", ",", "830.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:33:02\"\>", ",", "830.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:33:17\"\>", ",", "830.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:33:32\"\>", ",", "830.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:33:47\"\>", ",", "830.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:34:02\"\>", ",", "830.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:34:17\"\>", ",", "830.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:34:32\"\>", ",", "830.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:34:47\"\>", ",", "830.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:35:02\"\>", ",", "830.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:35:17\"\>", ",", "830.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:35:32\"\>", ",", "830.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:35:47\"\>", ",", "830.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:36:02\"\>", ",", "830.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:36:17\"\>", ",", "830.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:36:32\"\>", ",", "829.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:36:47\"\>", ",", "829.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:37:02\"\>", ",", "829.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:37:17\"\>", ",", "829.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:37:32\"\>", ",", "829.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:37:47\"\>", ",", "829.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:38:02\"\>", ",", "829.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:38:17\"\>", ",", "829.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:38:32\"\>", ",", "828.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:38:47\"\>", ",", "828.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:39:02\"\>", ",", "828.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:39:17\"\>", ",", "828.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:39:32\"\>", ",", "828.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:39:47\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:40:02\"\>", ",", "828.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:40:17\"\>", ",", "828.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:40:32\"\>", ",", "828.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:40:47\"\>", ",", "828.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:41:17\"\>", ",", "828.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:41:32\"\>", ",", "827.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:41:47\"\>", ",", "828.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:42:02\"\>", ",", "828.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:42:17\"\>", ",", "827.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:42:32\"\>", ",", "828.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:42:47\"\>", ",", "828.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:43:02\"\>", ",", "828.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:43:17\"\>", ",", "828.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:43:32\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:43:47\"\>", ",", "828.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:44:02\"\>", ",", "829.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:44:17\"\>", ",", "828.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:44:32\"\>", ",", "829.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:44:47\"\>", ",", "828.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:45:02\"\>", ",", "829.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:45:17\"\>", ",", "829.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:45:32\"\>", ",", "828.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:45:47\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:46:02\"\>", ",", "828.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:46:17\"\>", ",", "828.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:46:32\"\>", ",", "828.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:46:47\"\>", ",", "829.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:47:02\"\>", ",", "828.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:47:17\"\>", ",", "828.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:47:32\"\>", ",", "828.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:47:47\"\>", ",", "828.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:48:02\"\>", ",", "828.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:48:17\"\>", ",", "828.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:48:32\"\>", ",", "828.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:48:47\"\>", ",", "828.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:49:02\"\>", ",", "828.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:49:17\"\>", ",", "828.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:49:32\"\>", ",", "829.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:49:47\"\>", ",", "829.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:50:02\"\>", ",", "829.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:50:17\"\>", ",", "829.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:50:32\"\>", ",", "829.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:50:47\"\>", ",", "829.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:51:02\"\>", ",", "829.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:51:17\"\>", ",", "829.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:51:32\"\>", ",", "829.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:51:47\"\>", ",", "829.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:52:02\"\>", ",", "829.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:52:17\"\>", ",", "829.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:52:32\"\>", ",", "829.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:52:47\"\>", ",", "829.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:53:02\"\>", ",", "829.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:53:17\"\>", ",", "829.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:53:32\"\>", ",", "829.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:53:47\"\>", ",", "829.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:54:02\"\>", ",", "829.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:54:17\"\>", ",", "829.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:54:32\"\>", ",", "829.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:54:47\"\>", ",", "829.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:55:02\"\>", ",", "829.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:55:17\"\>", ",", "829.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:55:32\"\>", ",", "829.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:55:47\"\>", ",", "829.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:56:02\"\>", ",", "829.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:56:17\"\>", ",", "828.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:56:32\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:56:47\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:57:02\"\>", ",", "827.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:57:17\"\>", ",", "827.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:57:32\"\>", ",", "827.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:57:47\"\>", ",", "827.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:58:02\"\>", ",", "827.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:58:17\"\>", ",", "827.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:58:32\"\>", ",", "827.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:58:47\"\>", ",", "827.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:59:02\"\>", ",", "827.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:59:17\"\>", ",", "827.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:59:32\"\>", ",", "827.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:59:47\"\>", ",", "827.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:00:02\"\>", ",", "827.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:00:17\"\>", ",", "828.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:00:32\"\>", ",", "828.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:00:47\"\>", ",", "828.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:01:02\"\>", ",", "828.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:01:17\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:01:32\"\>", ",", "828.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:01:47\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:02:02\"\>", ",", "828.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:02:17\"\>", ",", "828.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:02:32\"\>", ",", "827.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:02:47\"\>", ",", "827.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:03:02\"\>", ",", "827.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:03:17\"\>", ",", "827.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:03:32\"\>", ",", "827.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:03:47\"\>", ",", "828.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:04:02\"\>", ",", "828.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:04:17\"\>", ",", "828.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:04:32\"\>", ",", "828.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:04:47\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:05:02\"\>", ",", "828.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:05:17\"\>", ",", "828.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:05:32\"\>", ",", "828.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:05:47\"\>", ",", "828.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:06:02\"\>", ",", "828.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:06:17\"\>", ",", "828.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:06:32\"\>", ",", "828.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:06:47\"\>", ",", "828.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:07:02\"\>", ",", "827.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:07:17\"\>", ",", "827.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:07:32\"\>", ",", "827.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:07:47\"\>", ",", "828.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:08:02\"\>", ",", "828.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:08:17\"\>", ",", "828.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:08:32\"\>", ",", "828.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:08:47\"\>", ",", "827.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:09:02\"\>", ",", "827.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:09:17\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:09:32\"\>", ",", "827.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:09:47\"\>", ",", "827.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:10:02\"\>", ",", "827.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:10:17\"\>", ",", "826.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:10:32\"\>", ",", "826.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:10:47\"\>", ",", "826.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:11:02\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:11:17\"\>", ",", "826.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:11:32\"\>", ",", "826.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:11:47\"\>", ",", "827.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:12:02\"\>", ",", "826.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:12:17\"\>", ",", "826.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:12:32\"\>", ",", "826.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:12:47\"\>", ",", "826.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:13:02\"\>", ",", "826.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:13:17\"\>", ",", "827.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:13:32\"\>", ",", "827.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:13:47\"\>", ",", "827.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:14:02\"\>", ",", "827.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:14:17\"\>", ",", "827.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:14:32\"\>", ",", "827.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:14:47\"\>", ",", "827.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:15:02\"\>", ",", "827.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:15:17\"\>", ",", "827.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:15:32\"\>", ",", "827.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:15:47\"\>", ",", "828.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:16:02\"\>", ",", "828.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:16:17\"\>", ",", "828.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:16:32\"\>", ",", "828.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:16:47\"\>", ",", "828.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:17:02\"\>", ",", "828.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:17:17\"\>", ",", "828.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:17:32\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:17:47\"\>", ",", "828.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:18:02\"\>", ",", "828.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:18:17\"\>", ",", "827.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:18:32\"\>", ",", "826.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:18:47\"\>", ",", "826.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:19:02\"\>", ",", "826.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:19:17\"\>", ",", "825.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:19:32\"\>", ",", "825.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:19:47\"\>", ",", "825.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:20:02\"\>", ",", "826.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:20:17\"\>", ",", "826.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:20:32\"\>", ",", "826.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:20:47\"\>", ",", "826.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:21:02\"\>", ",", "826.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:21:17\"\>", ",", "826.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:21:32\"\>", ",", "826.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:21:47\"\>", ",", "826.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:22:02\"\>", ",", "826.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:22:17\"\>", ",", "826.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:22:32\"\>", ",", "826.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:22:47\"\>", ",", "826.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:23:02\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:23:17\"\>", ",", "826.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:23:32\"\>", ",", "826.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:23:47\"\>", ",", "826.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:24:02\"\>", ",", "826.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:24:17\"\>", ",", "826.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:24:32\"\>", ",", "826.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:24:47\"\>", ",", "825.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:25:02\"\>", ",", "826.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:25:17\"\>", ",", "826.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:25:32\"\>", ",", "826.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:25:47\"\>", ",", "826.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:26:02\"\>", ",", "826.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:26:17\"\>", ",", "826.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:26:32\"\>", ",", "826.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:26:47\"\>", ",", "826.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:27:02\"\>", ",", "826.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:27:17\"\>", ",", "826.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:27:32\"\>", ",", "826.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:27:47\"\>", ",", "826.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:28:02\"\>", ",", "826.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:28:17\"\>", ",", "827.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:28:32\"\>", ",", "826.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:28:47\"\>", ",", "826.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:29:02\"\>", ",", "826.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:29:17\"\>", ",", "826.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:29:32\"\>", ",", "826.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:29:47\"\>", ",", "826.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:30:02\"\>", ",", "827.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:30:17\"\>", ",", "827.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:30:32\"\>", ",", "827.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:30:47\"\>", ",", "827.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:31:02\"\>", ",", "827.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:31:17\"\>", ",", "827.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:31:32\"\>", ",", "827.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:31:47\"\>", ",", "827.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:32:02\"\>", ",", "827.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:32:17\"\>", ",", "827.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:32:32\"\>", ",", "827.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:32:47\"\>", ",", "827.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:33:02\"\>", ",", "827.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:33:17\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:33:32\"\>", ",", "828.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:33:47\"\>", ",", "828.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:34:02\"\>", ",", "828.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:34:17\"\>", ",", "828.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:34:32\"\>", ",", "828.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:34:47\"\>", ",", "828.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:35:02\"\>", ",", "828.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:35:17\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:35:32\"\>", ",", "828.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:35:47\"\>", ",", "828.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:36:02\"\>", ",", "828.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:36:17\"\>", ",", "828.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:36:32\"\>", ",", "828.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:36:47\"\>", ",", "827.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:37:02\"\>", ",", "827.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:37:17\"\>", ",", "828.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:37:32\"\>", ",", "828.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:37:47\"\>", ",", "828.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:38:02\"\>", ",", "828.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:38:17\"\>", ",", "828.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:38:32\"\>", ",", "827.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:38:47\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:39:02\"\>", ",", "827.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:39:17\"\>", ",", "827.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:39:32\"\>", ",", "827.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:39:47\"\>", ",", "827.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:40:02\"\>", ",", "828.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:40:17\"\>", ",", "828.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:40:32\"\>", ",", "828.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:40:47\"\>", ",", "827.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:41:02\"\>", ",", "827.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:41:17\"\>", ",", "828.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:41:32\"\>", ",", "828.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:41:47\"\>", ",", "828.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:42:02\"\>", ",", "828.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:42:17\"\>", ",", "828.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:42:32\"\>", ",", "828.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:42:47\"\>", ",", "828.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:43:02\"\>", ",", "828.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:43:17\"\>", ",", "828.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:43:32\"\>", ",", "828.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:43:47\"\>", ",", "829.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:44:02\"\>", ",", "829.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:44:17\"\>", ",", "829.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:44:32\"\>", ",", "829.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:44:47\"\>", ",", "829.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:45:02\"\>", ",", "829.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:45:17\"\>", ",", "829.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:45:32\"\>", ",", "829.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:45:47\"\>", ",", "829.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:46:02\"\>", ",", "829.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:46:17\"\>", ",", "829.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:46:32\"\>", ",", "829.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:46:47\"\>", ",", "829.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:47:02\"\>", ",", "829.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:47:17\"\>", ",", "829.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:47:32\"\>", ",", "829.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:47:47\"\>", ",", "829.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:48:02\"\>", ",", "829.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:48:17\"\>", ",", "828.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:48:32\"\>", ",", "828.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:48:47\"\>", ",", "828.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:49:17\"\>", ",", "828.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:49:32\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:49:47\"\>", ",", "828.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:50:02\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:50:17\"\>", ",", "828.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:50:32\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:50:47\"\>", ",", "828.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:51:02\"\>", ",", "828.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:51:17\"\>", ",", "828.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:51:32\"\>", ",", "828.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:51:47\"\>", ",", "828.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:52:02\"\>", ",", "829.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:52:17\"\>", ",", "829.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:52:32\"\>", ",", "829.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:52:47\"\>", ",", "828.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:53:02\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:53:17\"\>", ",", "828.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:53:32\"\>", ",", "828.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:53:47\"\>", ",", "828.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:54:02\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:54:17\"\>", ",", "828.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:54:32\"\>", ",", "828.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:54:47\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:55:02\"\>", ",", "828.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:55:17\"\>", ",", "828.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:55:32\"\>", ",", "828.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:55:47\"\>", ",", "828.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:56:02\"\>", ",", "828.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:56:17\"\>", ",", "828.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:56:32\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:56:47\"\>", ",", "828.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:57:02\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:57:17\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:57:32\"\>", ",", "828.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:57:47\"\>", ",", "828.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:58:02\"\>", ",", "827.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:58:17\"\>", ",", "827.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:58:32\"\>", ",", "827.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:58:47\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:59:02\"\>", ",", "826.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:59:17\"\>", ",", "826.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:59:32\"\>", ",", "826.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:59:47\"\>", ",", "826.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:00:02\"\>", ",", "825.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:00:17\"\>", ",", "825.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:00:32\"\>", ",", "825.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:00:47\"\>", ",", "825.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:01:02\"\>", ",", "825.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:01:17\"\>", ",", "825.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:01:32\"\>", ",", "825.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:01:47\"\>", ",", "825.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:02:02\"\>", ",", "826.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:02:17\"\>", ",", "826.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:02:32\"\>", ",", "826.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:02:47\"\>", ",", "826.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:03:02\"\>", ",", "826.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:03:17\"\>", ",", "826.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:03:32\"\>", ",", "826.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:03:47\"\>", ",", "826.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:04:02\"\>", ",", "826.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:04:17\"\>", ",", "826.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:04:32\"\>", ",", "825.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:04:47\"\>", ",", "825.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:05:02\"\>", ",", "825.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:05:17\"\>", ",", "825.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:05:32\"\>", ",", "825.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:05:47\"\>", ",", "825.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:06:02\"\>", ",", "825.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:06:17\"\>", ",", "825.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:06:32\"\>", ",", "825.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:06:47\"\>", ",", "826.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:07:02\"\>", ",", "826.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:07:17\"\>", ",", "826.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:07:32\"\>", ",", "826.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:07:47\"\>", ",", "826.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:08:02\"\>", ",", "826.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:08:17\"\>", ",", "826.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:08:32\"\>", ",", "826.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:08:47\"\>", ",", "826.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:09:02\"\>", ",", "826.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:09:17\"\>", ",", "826.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:09:32\"\>", ",", "826.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:09:47\"\>", ",", "826.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:10:02\"\>", ",", "826.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:10:17\"\>", ",", "826.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:10:32\"\>", ",", "826.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:10:47\"\>", ",", "826.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:11:02\"\>", ",", "826.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:11:17\"\>", ",", "826.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:11:32\"\>", ",", "826.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:11:47\"\>", ",", "826.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:12:02\"\>", ",", "826.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:12:17\"\>", ",", "826.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:12:32\"\>", ",", "826.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:12:47\"\>", ",", "826.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:13:02\"\>", ",", "826.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:13:17\"\>", ",", "826.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:13:32\"\>", ",", "826.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:13:47\"\>", ",", "826.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:14:02\"\>", ",", "826.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:14:17\"\>", ",", "826.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:14:32\"\>", ",", "827.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:14:47\"\>", ",", "827.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:15:02\"\>", ",", "827.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:15:17\"\>", ",", "827.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:15:32\"\>", ",", "827.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:15:47\"\>", ",", "828.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:16:02\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:16:17\"\>", ",", "828.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:16:32\"\>", ",", "828.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:16:47\"\>", ",", "828.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:17:02\"\>", ",", "828.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:17:17\"\>", ",", "828.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:17:32\"\>", ",", "828.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:17:47\"\>", ",", "828.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:18:02\"\>", ",", "828.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:18:17\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:18:32\"\>", ",", "828.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:18:47\"\>", ",", "828.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:19:02\"\>", ",", "828.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:19:17\"\>", ",", "828.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:19:32\"\>", ",", "828.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:19:47\"\>", ",", "828.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:20:02\"\>", ",", "828.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:20:17\"\>", ",", "828.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:20:32\"\>", ",", "828.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:20:47\"\>", ",", "827.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:21:02\"\>", ",", "827.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:21:17\"\>", ",", "827.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:21:32\"\>", ",", "827.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:21:47\"\>", ",", "827.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:22:02\"\>", ",", "827.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:22:17\"\>", ",", "827.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:22:32\"\>", ",", "827.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:22:47\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:23:02\"\>", ",", "827.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:23:17\"\>", ",", "827.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:23:32\"\>", ",", "827.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:23:47\"\>", ",", "827.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:24:17\"\>", ",", "827.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:24:32\"\>", ",", "827.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:24:47\"\>", ",", "827.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:25:02\"\>", ",", "827.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:25:17\"\>", ",", "827.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:25:32\"\>", ",", "827.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:25:47\"\>", ",", "827.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:26:02\"\>", ",", "827.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:26:17\"\>", ",", "827.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:26:32\"\>", ",", "827.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:26:47\"\>", ",", "827.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:27:02\"\>", ",", "827.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:27:17\"\>", ",", "827.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:27:32\"\>", ",", "826.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:27:47\"\>", ",", "826.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:28:02\"\>", ",", "826.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:28:17\"\>", ",", "826.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:28:32\"\>", ",", "826.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:28:47\"\>", ",", "826.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:29:02\"\>", ",", "826.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:29:17\"\>", ",", "826.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:29:32\"\>", ",", "826.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:29:47\"\>", ",", "826.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:30:02\"\>", ",", "826.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:30:17\"\>", ",", "826.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:30:32\"\>", ",", "826.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:30:47\"\>", ",", "826.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:31:02\"\>", ",", "826.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:31:17\"\>", ",", "826.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:31:32\"\>", ",", "826.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:31:47\"\>", ",", "826.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:32:02\"\>", ",", "826.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:32:17\"\>", ",", "826.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:32:32\"\>", ",", "827.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:32:47\"\>", ",", "827.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:33:02\"\>", ",", "826.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:33:17\"\>", ",", "826.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:33:32\"\>", ",", "827.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:33:47\"\>", ",", "827.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:34:02\"\>", ",", "827.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:34:17\"\>", ",", "827.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:34:32\"\>", ",", "828.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:34:47\"\>", ",", "828.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:35:02\"\>", ",", "827.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:35:17\"\>", ",", "827.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:35:32\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:35:47\"\>", ",", "828.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:36:02\"\>", ",", "828.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:36:17\"\>", ",", "828.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:36:32\"\>", ",", "828.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:36:47\"\>", ",", "828.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:37:02\"\>", ",", "828.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:37:17\"\>", ",", "829.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:37:32\"\>", ",", "829.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:37:47\"\>", ",", "830.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:38:02\"\>", ",", "830.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:38:17\"\>", ",", "829.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:38:32\"\>", ",", "829.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:38:47\"\>", ",", "829.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:39:02\"\>", ",", "829.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:39:17\"\>", ",", "829.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:39:32\"\>", ",", "829.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:39:47\"\>", ",", "829.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:40:02\"\>", ",", "829.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:40:17\"\>", ",", "829.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:40:32\"\>", ",", "830.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:40:47\"\>", ",", "830.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:41:02\"\>", ",", "830.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:41:17\"\>", ",", "830.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:41:32\"\>", ",", "830.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:41:47\"\>", ",", "830.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:42:02\"\>", ",", "829.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:42:17\"\>", ",", "830.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:42:32\"\>", ",", "829.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:42:47\"\>", ",", "829.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:43:02\"\>", ",", "829.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:43:17\"\>", ",", "829.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:43:32\"\>", ",", "829.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:43:47\"\>", ",", "829.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:44:02\"\>", ",", "829.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:44:17\"\>", ",", "829.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:44:32\"\>", ",", "829.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:44:47\"\>", ",", "829.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:45:02\"\>", ",", "829.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:45:17\"\>", ",", "829.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:45:32\"\>", ",", "830.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:45:47\"\>", ",", "830.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:46:02\"\>", ",", "829.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:46:17\"\>", ",", "829.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:46:32\"\>", ",", "829.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:46:47\"\>", ",", "829.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:47:02\"\>", ",", "829.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:47:17\"\>", ",", "829.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:47:32\"\>", ",", "829.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:47:47\"\>", ",", "829.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:48:02\"\>", ",", "828.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:48:17\"\>", ",", "828.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:48:32\"\>", ",", "827.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:48:47\"\>", ",", "828.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:49:02\"\>", ",", "828.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:49:17\"\>", ",", "828.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:49:32\"\>", ",", "828.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:49:47\"\>", ",", "828.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:50:02\"\>", ",", "828.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:50:17\"\>", ",", "827.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:50:32\"\>", ",", "827.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:50:47\"\>", ",", "827.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:51:02\"\>", ",", "826.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:51:17\"\>", ",", "827.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:51:32\"\>", ",", "826.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:51:47\"\>", ",", "826.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:52:02\"\>", ",", "827.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:52:17\"\>", ",", "827.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:52:32\"\>", ",", "827.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:52:47\"\>", ",", "827.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:53:02\"\>", ",", "827.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:53:17\"\>", ",", "827.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:53:32\"\>", ",", "827.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:53:47\"\>", ",", "826.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:54:02\"\>", ",", "827.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:54:17\"\>", ",", "827.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:54:32\"\>", ",", "827.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:54:47\"\>", ",", "827.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:55:02\"\>", ",", "827.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:55:17\"\>", ",", "827.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:55:32\"\>", ",", "827.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:55:47\"\>", ",", "827.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:56:02\"\>", ",", "827.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:56:17\"\>", ",", "827.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:56:32\"\>", ",", "828.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:56:47\"\>", ",", "828.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:57:02\"\>", ",", "828.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:57:17\"\>", ",", "829.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:57:32\"\>", ",", "829.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:57:47\"\>", ",", "829.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:58:02\"\>", ",", "828.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:58:17\"\>", ",", "828.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:58:32\"\>", ",", "828.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:58:47\"\>", ",", "828.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:59:02\"\>", ",", "828.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:59:17\"\>", ",", "828.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:59:32\"\>", ",", "828.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:59:47\"\>", ",", "828.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:00:02\"\>", ",", "828.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:00:17\"\>", ",", "829.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:00:32\"\>", ",", "829.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:00:47\"\>", ",", "829.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:01:02\"\>", ",", "829.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:01:17\"\>", ",", "829.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:01:32\"\>", ",", "829.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:01:47\"\>", ",", "829.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:02:02\"\>", ",", "829.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:02:17\"\>", ",", "829.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:02:32\"\>", ",", "828.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:02:47\"\>", ",", "828.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:03:02\"\>", ",", "829.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:03:17\"\>", ",", "829.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:03:32\"\>", ",", "828.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:03:47\"\>", ",", "828.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:04:02\"\>", ",", "828.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:04:17\"\>", ",", "828.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:04:32\"\>", ",", "828.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:04:47\"\>", ",", "828.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:05:02\"\>", ",", "827.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:05:17\"\>", ",", "828.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:05:32\"\>", ",", "828.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:05:47\"\>", ",", "828.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:06:02\"\>", ",", "828.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:06:17\"\>", ",", "828.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:06:32\"\>", ",", "828.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:06:47\"\>", ",", "829.61`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:07:02\"\>", ",", "830.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:07:17\"\>", ",", "830.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:07:32\"\>", ",", "830.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:07:47\"\>", ",", "830.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:08:02\"\>", ",", "830.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:08:17\"\>", ",", "830.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:08:32\"\>", ",", "830.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:08:47\"\>", ",", "830.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:09:02\"\>", ",", "830.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:09:17\"\>", ",", "831.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:09:32\"\>", ",", "832.03`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:09:47\"\>", ",", "831.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:10:02\"\>", ",", "832.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:10:17\"\>", ",", "832.85`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:10:32\"\>", ",", "832.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:10:47\"\>", ",", "832.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:11:02\"\>", ",", "832.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:11:17\"\>", ",", "833.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:11:32\"\>", ",", "834.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:11:47\"\>", ",", "834.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:12:02\"\>", ",", "834.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:12:17\"\>", ",", "834.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:12:32\"\>", ",", "834.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:12:47\"\>", ",", "833.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:13:02\"\>", ",", "834.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:13:17\"\>", ",", "833.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:13:32\"\>", ",", "833.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:13:47\"\>", ",", "833.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:14:02\"\>", ",", "833.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:14:17\"\>", ",", "834.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:14:32\"\>", ",", "834.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:14:47\"\>", ",", "834.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:15:02\"\>", ",", "834.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:15:17\"\>", ",", "834.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:15:32\"\>", ",", "834.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:15:47\"\>", ",", "834.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:16:02\"\>", ",", "834.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:16:17\"\>", ",", "833.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:16:32\"\>", ",", "833.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:16:47\"\>", ",", "834.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:17:02\"\>", ",", "834.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:17:17\"\>", ",", "834.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:17:32\"\>", ",", "835.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:17:47\"\>", ",", "835.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:18:02\"\>", ",", "835.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:18:17\"\>", ",", "835.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:18:32\"\>", ",", "835.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:18:47\"\>", ",", "835.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:19:02\"\>", ",", "835.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:19:17\"\>", ",", "835.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:19:32\"\>", ",", "835.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:19:47\"\>", ",", "835.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:20:02\"\>", ",", "834.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:20:17\"\>", ",", "834.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:20:32\"\>", ",", "834.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:20:47\"\>", ",", "834.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:21:02\"\>", ",", "834.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:21:17\"\>", ",", "834.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:21:32\"\>", ",", "834.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:21:47\"\>", ",", "834.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:22:02\"\>", ",", "834.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:22:17\"\>", ",", "834.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:22:32\"\>", ",", "834.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:22:47\"\>", ",", "834.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:23:02\"\>", ",", "834.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:23:17\"\>", ",", "834.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:23:32\"\>", ",", "834.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:23:47\"\>", ",", "835.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:24:02\"\>", ",", "835.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:24:17\"\>", ",", "834.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:24:32\"\>", ",", "834.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:24:47\"\>", ",", "834.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:25:02\"\>", ",", "835.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:25:17\"\>", ",", "834.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:25:32\"\>", ",", "834.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:25:47\"\>", ",", "834.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:26:02\"\>", ",", "835.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:26:17\"\>", ",", "835.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:26:32\"\>", ",", "835.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:26:47\"\>", ",", "835.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:27:02\"\>", ",", "835.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:27:17\"\>", ",", "835.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:27:32\"\>", ",", "835.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:27:47\"\>", ",", "834.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:28:02\"\>", ",", "834.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:28:17\"\>", ",", "833.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:28:32\"\>", ",", "834.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:28:47\"\>", ",", "834.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:29:02\"\>", ",", "834.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:29:17\"\>", ",", "834.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:29:32\"\>", ",", "833.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:29:47\"\>", ",", "833.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:30:02\"\>", ",", "833.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:30:17\"\>", ",", "833.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:30:32\"\>", ",", "834.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:30:47\"\>", ",", "833.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:31:02\"\>", ",", "834.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:31:17\"\>", ",", "834.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:31:32\"\>", ",", "834.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:31:47\"\>", ",", "834.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:32:02\"\>", ",", "834.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:32:17\"\>", ",", "834.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:32:32\"\>", ",", "834.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:32:47\"\>", ",", "834.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:33:02\"\>", ",", "834.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:33:17\"\>", ",", "834.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:33:32\"\>", ",", "834.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:33:47\"\>", ",", "834.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:34:02\"\>", ",", "833.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:34:17\"\>", ",", "834.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:34:32\"\>", ",", "834.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:34:47\"\>", ",", "834.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:35:02\"\>", ",", "835.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:35:17\"\>", ",", "835.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:35:32\"\>", ",", "835.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:35:47\"\>", ",", "836.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:36:02\"\>", ",", "836.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:36:17\"\>", ",", "836.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:36:32\"\>", ",", "836.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:36:47\"\>", ",", "836.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:37:02\"\>", ",", "835.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:37:17\"\>", ",", "835.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:37:32\"\>", ",", "835.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:37:47\"\>", ",", "835.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:38:02\"\>", ",", "835.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:38:17\"\>", ",", "836.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:38:32\"\>", ",", "836.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:38:47\"\>", ",", "836.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:39:02\"\>", ",", "836.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:39:17\"\>", ",", "836.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:39:32\"\>", ",", "836.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:39:47\"\>", ",", "836.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:40:02\"\>", ",", "835.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:40:17\"\>", ",", "836.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:40:32\"\>", ",", "836.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:40:47\"\>", ",", "836.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:41:02\"\>", ",", "836.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:41:17\"\>", ",", "836.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:41:32\"\>", ",", "836.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:41:47\"\>", ",", "836.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:42:02\"\>", ",", "837.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:42:17\"\>", ",", "837.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:42:32\"\>", ",", "836.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:42:47\"\>", ",", "836.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:43:02\"\>", ",", "836.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:43:17\"\>", ",", "836.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:43:32\"\>", ",", "836.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:43:47\"\>", ",", "836.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:44:02\"\>", ",", "836.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:44:17\"\>", ",", "836.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:44:32\"\>", ",", "836.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:44:47\"\>", ",", "836.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:45:02\"\>", ",", "836.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:45:17\"\>", ",", "836.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:45:32\"\>", ",", "836.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:45:47\"\>", ",", "836.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:46:02\"\>", ",", "836.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:46:17\"\>", ",", "836.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:46:32\"\>", ",", "836.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:46:47\"\>", ",", "836.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:47:02\"\>", ",", "836.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:47:17\"\>", ",", "836.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:47:32\"\>", ",", "836.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:47:47\"\>", ",", "836.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:48:02\"\>", ",", "837.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:48:17\"\>", ",", "836.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:48:32\"\>", ",", "836.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:48:47\"\>", ",", "836.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:49:02\"\>", ",", "836.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:49:17\"\>", ",", "836.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:49:32\"\>", ",", "836.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:49:47\"\>", ",", "836.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:50:02\"\>", ",", "836.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:50:17\"\>", ",", "836.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:50:32\"\>", ",", "836.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:50:47\"\>", ",", "836.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:51:02\"\>", ",", "836.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:51:17\"\>", ",", "836.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:51:32\"\>", ",", "836.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:51:47\"\>", ",", "836.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:52:02\"\>", ",", "836.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:52:17\"\>", ",", "836.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:52:32\"\>", ",", "836.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:52:47\"\>", ",", "836.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:53:02\"\>", ",", "836.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:53:17\"\>", ",", "836.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:53:32\"\>", ",", "836.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:53:47\"\>", ",", "835.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:54:02\"\>", ",", "836.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:54:17\"\>", ",", "835.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:54:32\"\>", ",", "835.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:54:47\"\>", ",", "835.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:55:02\"\>", ",", "834.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:55:17\"\>", ",", "834.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:55:32\"\>", ",", "834.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:55:47\"\>", ",", "834.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:56:02\"\>", ",", "834.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:56:17\"\>", ",", "834.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:56:32\"\>", ",", "834.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:56:47\"\>", ",", "834.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:57:02\"\>", ",", "834.52`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:57:17\"\>", ",", "834.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:57:32\"\>", ",", "834.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:57:47\"\>", ",", "834.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:58:02\"\>", ",", "834.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:58:17\"\>", ",", "835.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:58:32\"\>", ",", "835.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:58:47\"\>", ",", "835.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:59:02\"\>", ",", "835.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:59:17\"\>", ",", "835.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:59:32\"\>", ",", "835.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:59:47\"\>", ",", "835.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:00:02\"\>", ",", "835.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:00:17\"\>", ",", "834.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:00:32\"\>", ",", "835.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:00:47\"\>", ",", "834.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:01:02\"\>", ",", "836.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:01:17\"\>", ",", "836.05`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:01:32\"\>", ",", "836.06`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:01:47\"\>", ",", "836.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:02:02\"\>", ",", "836.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:02:17\"\>", ",", "835.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:02:32\"\>", ",", "835.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:02:47\"\>", ",", "835.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:03:02\"\>", ",", "835.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:03:17\"\>", ",", "835.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:03:32\"\>", ",", "835.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:03:47\"\>", ",", "834.89`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:04:02\"\>", ",", "835.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:04:17\"\>", ",", "835.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:04:32\"\>", ",", "834.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:04:47\"\>", ",", "834.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:05:02\"\>", ",", "834.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:05:17\"\>", ",", "834.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:05:32\"\>", ",", "834.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:05:47\"\>", ",", "834.64`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:06:02\"\>", ",", "834.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:06:17\"\>", ",", "835.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:06:32\"\>", ",", "835.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:06:47\"\>", ",", "835.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:07:02\"\>", ",", "835.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:07:17\"\>", ",", "835.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:07:32\"\>", ",", "835.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:07:47\"\>", ",", "835.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:08:02\"\>", ",", "835.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:08:17\"\>", ",", "835.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:08:32\"\>", ",", "835.99`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:08:47\"\>", ",", "836.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:09:02\"\>", ",", "837.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:09:17\"\>", ",", "837.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:09:32\"\>", ",", "838.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:09:47\"\>", ",", "838.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:10:02\"\>", ",", "838.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:10:17\"\>", ",", "839.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:10:32\"\>", ",", "840.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:10:47\"\>", ",", "839.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:11:02\"\>", ",", "839.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:11:17\"\>", ",", "839.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:11:32\"\>", ",", "839.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:11:47\"\>", ",", "839.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:12:02\"\>", ",", "839.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:12:17\"\>", ",", "840.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:12:32\"\>", ",", "840.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:12:47\"\>", ",", "840.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:13:02\"\>", ",", "839.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:13:17\"\>", ",", "839.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:13:32\"\>", ",", "840.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:13:47\"\>", ",", "840.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:14:02\"\>", ",", "839.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:14:17\"\>", ",", "839.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:14:32\"\>", ",", "839.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:14:47\"\>", ",", "839.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:15:02\"\>", ",", "839.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:15:17\"\>", ",", "839.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:15:32\"\>", ",", "839.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:15:47\"\>", ",", "839.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:16:02\"\>", ",", "839.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:16:17\"\>", ",", "839.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:16:32\"\>", ",", "839.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:16:47\"\>", ",", "839.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:17:02\"\>", ",", "839.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:17:17\"\>", ",", "839.93`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:17:32\"\>", ",", "841.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:17:47\"\>", ",", "841.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:18:02\"\>", ",", "841.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:18:17\"\>", ",", "841.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:18:32\"\>", ",", "840.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:18:47\"\>", ",", "840.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:19:02\"\>", ",", "840.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:19:17\"\>", ",", "840.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:19:32\"\>", ",", "840.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:19:47\"\>", ",", "840.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:20:02\"\>", ",", "840.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:20:17\"\>", ",", "840.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:20:32\"\>", ",", "840.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:20:47\"\>", ",", "840.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:21:02\"\>", ",", "840.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:21:17\"\>", ",", "840.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:21:32\"\>", ",", "839.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:21:47\"\>", ",", "839.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:22:02\"\>", ",", "839.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:22:17\"\>", ",", "839.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:22:32\"\>", ",", "839.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:22:47\"\>", ",", "839.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:23:02\"\>", ",", "839.72`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:23:17\"\>", ",", "839.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:23:32\"\>", ",", "839.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:23:47\"\>", ",", "840.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:24:02\"\>", ",", "842.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:24:17\"\>", ",", "842.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:24:32\"\>", ",", "842.11`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:24:47\"\>", ",", "842.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:25:02\"\>", ",", "842.23`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:25:17\"\>", ",", "842.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:25:32\"\>", ",", "841.81`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:25:47\"\>", ",", "841.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:26:02\"\>", ",", "841.67`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:26:17\"\>", ",", "841.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:26:32\"\>", ",", "841.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:26:47\"\>", ",", "841.9`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:27:02\"\>", ",", "841.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:27:17\"\>", ",", "841.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:27:32\"\>", ",", "841.57`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:27:47\"\>", ",", "841.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:28:02\"\>", ",", "841.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:28:17\"\>", ",", "841.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:28:32\"\>", ",", "841.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:28:47\"\>", ",", "841.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:29:02\"\>", ",", "841.58`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:29:17\"\>", ",", "841.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:29:32\"\>", ",", "840.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:29:47\"\>", ",", "841.26`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:30:02\"\>", ",", "841.19`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:30:17\"\>", ",", "841.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:30:32\"\>", ",", "841.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:30:47\"\>", ",", "841.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:31:02\"\>", ",", "841.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:31:17\"\>", ",", "841.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:31:32\"\>", ",", "841.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:31:47\"\>", ",", "841.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:32:02\"\>", ",", "841.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:32:17\"\>", ",", "841.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:32:32\"\>", ",", "841.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:32:47\"\>", ",", "841.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:33:02\"\>", ",", "841.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:33:17\"\>", ",", "841.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:33:32\"\>", ",", "841.6`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:33:47\"\>", ",", "841.1`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:34:02\"\>", ",", "841.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:34:17\"\>", ",", "840.55`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:34:32\"\>", ",", "839.78`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:34:47\"\>", ",", "839.62`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:35:02\"\>", ",", "839.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:35:17\"\>", ",", "839.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:35:32\"\>", ",", "839.28`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:35:47\"\>", ",", "839.09`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:36:02\"\>", ",", "838.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:36:17\"\>", ",", "838.96`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:36:32\"\>", ",", "839.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:36:47\"\>", ",", "839.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:37:02\"\>", ",", "839.02`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:37:17\"\>", ",", "839.14`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:37:32\"\>", ",", "839.38`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:37:47\"\>", ",", "839.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:38:02\"\>", ",", "839.27`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:38:17\"\>", ",", "839.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:38:32\"\>", ",", "839.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:38:47\"\>", ",", "840.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:39:02\"\>", ",", "840.`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:39:17\"\>", ",", "839.7`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:39:32\"\>", ",", "839.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:39:47\"\>", ",", "839.71`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:40:02\"\>", ",", "840.29`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:40:17\"\>", ",", "840.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:40:32\"\>", ",", "839.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:40:47\"\>", ",", "839.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:41:02\"\>", ",", "839.35`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:41:17\"\>", ",", "839.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:41:32\"\>", ",", "839.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:41:47\"\>", ",", "839.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:42:02\"\>", ",", "839.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:42:17\"\>", ",", "838.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:42:32\"\>", ",", "837.88`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:42:47\"\>", ",", "837.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:43:02\"\>", ",", "838.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:43:17\"\>", ",", "838.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:43:32\"\>", ",", "838.56`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:43:47\"\>", ",", "838.42`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:44:02\"\>", ",", "838.24`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:44:17\"\>", ",", "838.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:44:32\"\>", ",", "838.07`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:44:47\"\>", ",", "837.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:45:02\"\>", ",", "837.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:45:17\"\>", ",", "837.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:45:32\"\>", ",", "837.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:45:47\"\>", ",", "838.31`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:46:02\"\>", ",", "838.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:46:17\"\>", ",", "838.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:46:32\"\>", ",", "839.12`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:46:47\"\>", ",", "838.92`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:47:02\"\>", ",", "838.84`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:47:17\"\>", ",", "838.3`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:47:32\"\>", ",", "838.21`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:47:47\"\>", ",", "838.15`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:48:02\"\>", ",", "837.8`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:48:17\"\>", ",", "838.36`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:48:32\"\>", ",", "838.37`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:48:47\"\>", ",", "838.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:49:02\"\>", ",", "838.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:49:17\"\>", ",", "838.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:49:32\"\>", ",", "838.39`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:49:47\"\>", ",", "838.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:50:02\"\>", ",", "838.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:50:17\"\>", ",", "838.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:50:32\"\>", ",", "838.63`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:50:47\"\>", ",", "838.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:51:02\"\>", ",", "839.41`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:51:17\"\>", ",", "839.17`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:51:32\"\>", ",", "839.59`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:51:47\"\>", ",", "839.73`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:52:02\"\>", ",", "840.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:52:17\"\>", ",", "840.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:52:32\"\>", ",", "839.95`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:52:47\"\>", ",", "839.69`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:53:02\"\>", ",", "839.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:53:17\"\>", ",", "839.53`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:53:32\"\>", ",", "839.54`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:53:47\"\>", ",", "839.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:54:02\"\>", ",", "840.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:54:17\"\>", ",", "839.79`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:54:32\"\>", ",", "839.91`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:54:47\"\>", ",", "839.75`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:55:02\"\>", ",", "839.65`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:55:17\"\>", ",", "840.01`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:55:32\"\>", ",", "839.87`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:55:47\"\>", ",", "839.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:56:02\"\>", ",", "840.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:56:17\"\>", ",", "840.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:56:32\"\>", ",", "840.08`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:56:47\"\>", ",", "839.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:57:02\"\>", ",", "839.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:57:17\"\>", ",", "839.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:57:32\"\>", ",", "838.97`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:57:47\"\>", ",", "839.18`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:58:02\"\>", ",", "839.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:58:17\"\>", ",", "838.98`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:58:32\"\>", ",", "838.76`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:58:47\"\>", ",", "838.82`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:59:02\"\>", ",", "838.66`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:59:17\"\>", ",", "838.77`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:59:32\"\>", ",", "838.68`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:59:47\"\>", ",", "837.74`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:00:02\"\>", ",", "837.83`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:00:17\"\>", ",", "837.86`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:00:32\"\>", ",", "837.94`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:00:47\"\>", ",", "838.04`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:01:02\"\>", ",", "838.13`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:01:17\"\>", ",", "838.16`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:01:32\"\>", ",", "838.2`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:01:47\"\>", ",", "838.22`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:02:02\"\>", ",", "838.25`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:02:17\"\>", ",", "838.32`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:02:32\"\>", ",", "838.33`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:02:47\"\>", ",", "838.34`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:03:02\"\>", ",", "838.4`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:03:17\"\>", ",", "838.43`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:03:32\"\>", ",", "838.44`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:03:47\"\>", ",", "838.45`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:04:02\"\>", ",", "838.46`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:04:17\"\>", ",", "838.47`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:04:32\"\>", ",", "838.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:04:47\"\>", ",", "838.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:05:02\"\>", ",", "838.48`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:05:17\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:05:32\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:05:47\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:06:02\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:06:17\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:06:32\"\>", ",", "838.49`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:06:47\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:07:02\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:07:17\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:07:32\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:07:47\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:08:02\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:08:17\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:08:32\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:08:47\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:09:02\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:09:17\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:09:32\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:09:47\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:10:02\"\>", ",", "838.5`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:10:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:10:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:10:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:11:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:11:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:11:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:11:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:12:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:12:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:12:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:12:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:13:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:13:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:13:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:13:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:14:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:14:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:14:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:14:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:15:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:15:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:15:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:15:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:16:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:16:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:16:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:16:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:17:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:17:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:17:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:17:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:18:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:18:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:18:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:19:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:19:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:19:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:19:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:20:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:20:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:20:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:20:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:21:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:21:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:21:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:21:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:22:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:22:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:22:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:22:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:23:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:23:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:23:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:23:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:24:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:24:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:24:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:24:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:25:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:25:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:25:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:25:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:26:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:26:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:26:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:26:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:27:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:27:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:27:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:27:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:28:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:28:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:28:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:28:47\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:29:02\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:29:17\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:29:32\"\>", ",", "838.51`", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:29:47\"\>", ",", "838.51`", ",", "0"}], 
    "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.459538949628459*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<Name\>\""}], "]"}], ",", 
   RowBox[{"BBGetCurrent", "[", 
    RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<General_Counsel\>\""}], 
    "]"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"INTL BUSINESS MACHINES CORP\"\>", 
   ",", "\<\"DAVID KAPPOS\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.455292189291*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"BBGetIntradayBars", "[", 
   RowBox[{
   "\"\<INDU Index\>\"", ",", "\"\<TRADE\>\"", ",", 
    "\"\<2009-02-02T09:30:00\>\"", ",", "\"\<2009-08-17T16:30:00\>\"", ",", 
    "10", ",", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.454587848335125*^9, 3.454587859507*^9}, {
   3.45458790747575*^9, 3.454587927960125*^9}, {3.45458807360075*^9, 
   3.45458809510075*^9}, {3.459275881515625*^9, 3.45927592615625*^9}, {
   3.459276168515625*^9, 3.4592761741875*^9}, {3.459276238203125*^9, 
   3.459276300421875*^9}, {3.459276330921875*^9, 3.459276410234375*^9}, {
   3.45927644059375*^9, 3.459276440921875*^9}, {3.4592768755625*^9, 
   3.459276880734375*^9}, {3.459276914078125*^9, 3.45927691478125*^9}, {
   3.459276982796875*^9, 3.459277007109375*^9}, {3.459529880003459*^9, 
   3.459530002284709*^9}, {3.459530269034709*^9, 3.459530284925334*^9}, {
   3.459531241440959*^9, 3.459531277034709*^9}, {3.459531384425334*^9, 
   3.459531387597209*^9}, {3.459532305581584*^9, 3.459532307831584*^9}, {
   3.459534557331584*^9, 3.459534561800334*^9}, {3.459535735472209*^9, 
   3.459535738894084*^9}, {3.459536102940959*^9, 3.459536103206584*^9}, 
   3.459536596815959*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:30:00\"\>", ",", "7936.99`", ",", "7987.4`", 
     ",", "7936.99`", ",", "7946.07`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:40:00\"\>", ",", "7946.38`", ",", "7954.99`", 
     ",", "7912.45`", ",", "7926.31`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T09:50:00\"\>", ",", "7927.35`", ",", "7932.13`", 
     ",", "7905.6`", ",", "7919.46`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:00:00\"\>", ",", "7930.06`", ",", "7967.01`", 
     ",", "7919.06`", ",", "7956.34`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:10:00\"\>", ",", "7956.9`", ",", "7963.03`", 
     ",", "7926.23`", ",", "7929.5`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:20:00\"\>", ",", "7930.06`", ",", "7944.63`", 
     ",", "7918.03`", ",", "7937.38`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:30:00\"\>", ",", "7936.43`", ",", "7962.63`", 
     ",", "7931.81`", ",", "7947.1`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:40:00\"\>", ",", "7948.85`", ",", "7980.63`", 
     ",", "7948.85`", ",", "7974.26`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T10:50:00\"\>", ",", "7974.18`", ",", "7982.15`", 
     ",", "7955.7`", ",", "7963.11`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:00:00\"\>", ",", "7963.35`", ",", "8006.36`", 
     ",", "7963.35`", ",", "7997.2`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:10:00\"\>", ",", "7998.63`", ",", "8004.85`", 
     ",", "7983.74`", ",", "7989.55`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:20:00\"\>", ",", "7990.03`", ",", "8001.9`", 
     ",", "7981.83`", ",", "7985.17`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:30:00\"\>", ",", "7986.45`", ",", "7990.75`", 
     ",", "7968.13`", ",", "7969.88`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:40:00\"\>", ",", "7969.56`", ",", "7978.88`", 
     ",", "7961.76`", ",", "7977.84`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T11:50:00\"\>", ",", "7977.92`", ",", "7980.15`", 
     ",", "7957.38`", ",", "7965.82`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:00:00\"\>", ",", "7965.58`", ",", "7970.68`", 
     ",", "7958.73`", ",", "7962.63`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:10:00\"\>", ",", "7962.95`", ",", "7972.75`", 
     ",", "7943.91`", ",", "7949.97`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:20:00\"\>", ",", "7949.81`", ",", "7960.96`", 
     ",", "7943.2`", ",", "7957.14`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:30:00\"\>", ",", "7957.77`", ",", "7971.95`", 
     ",", "7957.77`", ",", "7969.88`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:40:00\"\>", ",", "7969.96`", ",", "7984.38`", 
     ",", "7967.33`", ",", "7975.06`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T12:50:00\"\>", ",", "7975.14`", ",", "7982.78`", 
     ",", "7954.67`", ",", "7954.67`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:00:00\"\>", ",", "7953.71`", ",", "7963.67`", 
     ",", "7949.25`", ",", "7958.01`", ",", "303", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:10:00\"\>", ",", "7958.33`", ",", "7981.19`", 
     ",", "7956.18`", ",", "7978.24`", ",", "305", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:20:00\"\>", ",", "7978.16`", ",", "7978.16`", 
     ",", "7959.92`", ",", "7964.94`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:30:00\"\>", ",", "7964.7`", ",", "7994.09`", 
     ",", "7961.36`", ",", "7990.83`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:40:00\"\>", ",", "7991.38`", ",", "7997.28`", 
     ",", "7978.08`", ",", "7983.02`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T13:50:00\"\>", ",", "7982.78`", ",", "7992.82`", 
     ",", "7969.4`", ",", "7987.4`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:00:00\"\>", ",", "7988.04`", ",", "8024.12`", 
     ",", "7982.38`", ",", "8023.96`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:10:00\"\>", ",", "8024.68`", ",", "8057.81`", 
     ",", "8024.68`", ",", "8049.29`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:20:00\"\>", ",", "8049.45`", ",", "8055.98`", 
     ",", "8030.17`", ",", "8030.17`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:30:00\"\>", ",", "8031.45`", ",", "8060.52`", 
     ",", "8031.05`", ",", "8052.55`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:40:00\"\>", ",", "8052.79`", ",", "8066.41`", 
     ",", "8052.16`", ",", "8052.63`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T14:50:00\"\>", ",", "8053.83`", ",", "8058.21`", 
     ",", "8035.43`", ",", "8049.29`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:00:00\"\>", ",", "8049.21`", ",", "8080.43`", 
     ",", "8037.82`", ",", "8080.35`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:10:00\"\>", ",", "8082.74`", ",", "8101.94`", 
     ",", "8082.74`", ",", "8093.41`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:20:00\"\>", ",", "8093.73`", ",", "8111.02`", 
     ",", "8079.71`", ",", "8096.44`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:30:00\"\>", ",", "8096.52`", ",", "8106.87`", 
     ",", "8077.01`", ",", "8092.06`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:40:00\"\>", ",", "8092.22`", ",", "8099.23`", 
     ",", "8066.65`", ",", "8074.62`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T15:50:00\"\>", ",", "8075.73`", ",", "8097.48`", 
     ",", "8070.87`", ",", "8070.87`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-03T16:00:00\"\>", ",", "8070.79`", ",", "8078.36`", 
     ",", "8070.79`", ",", "8078.36`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T09:30:00\"\>", ",", "8070.32`", ",", "8074.94`", 
     ",", "8046.74`", ",", "8068.88`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T09:40:00\"\>", ",", "8069.6`", ",", "8089.03`", 
     ",", "8066.65`", ",", "8079.48`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T09:50:00\"\>", ",", "8079.4`", ",", "8108.07`", 
     ",", "8074.14`", ",", "8097.63`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:00:00\"\>", ",", "8106.64`", ",", "8136.42`", 
     ",", "8106.64`", ",", "8131.01`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:10:00\"\>", ",", "8130.77`", ",", "8161.99`", 
     ",", "8129.26`", ",", "8140.33`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:20:00\"\>", ",", "8140.09`", ",", "8151.16`", 
     ",", "8135.07`", ",", "8142.8`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:30:00\"\>", ",", "8137.7`", ",", "8160.64`", 
     ",", "8136.03`", ",", "8141.04`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:40:00\"\>", ",", "8141.28`", ",", "8145.34`", 
     ",", "8122.56`", ",", "8124.16`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T10:50:00\"\>", ",", "8124.`", ",", "8143.91`", 
     ",", "8120.02`", ",", "8135.79`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:00:00\"\>", ",", "8135.15`", ",", "8137.06`", 
     ",", "8123.44`", ",", "8132.28`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:10:00\"\>", ",", "8133.64`", ",", "8135.63`", 
     ",", "8103.29`", ",", "8116.75`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:20:00\"\>", ",", "8116.59`", ",", "8126.71`", 
     ",", "8103.93`", ",", "8106.32`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:30:00\"\>", ",", "8106.24`", ",", "8117.47`", 
     ",", "8102.41`", ",", "8104.56`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:40:00\"\>", ",", "8104.4`", ",", "8111.57`", 
     ",", "8087.76`", ",", "8092.54`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T11:50:00\"\>", ",", "8092.3`", ",", "8101.14`", 
     ",", "8092.14`", ",", "8093.33`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:00:00\"\>", ",", "8093.89`", ",", "8108.07`", 
     ",", "8091.18`", ",", "8103.53`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:10:00\"\>", ",", "8103.37`", ",", "8106.79`", 
     ",", "8091.82`", ",", "8096.44`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:20:00\"\>", ",", "8096.04`", ",", "8096.12`", 
     ",", "8076.37`", ",", "8078.12`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:30:00\"\>", ",", "8077.88`", ",", "8090.23`", 
     ",", "8076.85`", ",", "8084.73`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:40:00\"\>", ",", "8084.49`", ",", "8098.67`", 
     ",", "8083.46`", ",", "8090.31`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T12:50:00\"\>", ",", "8091.74`", ",", "8093.17`", 
     ",", "8063.63`", ",", "8068.17`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:00:00\"\>", ",", "8068.56`", ",", "8068.56`", 
     ",", "8047.14`", ",", "8058.53`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:10:00\"\>", ",", "8058.45`", ",", "8058.45`", 
     ",", "8039.17`", ",", "8040.85`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:20:00\"\>", ",", "8041.64`", ",", "8064.1`", 
     ",", "8041.48`", ",", "8062.67`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:30:00\"\>", ",", "8063.07`", ",", "8067.05`", 
     ",", "8023.32`", ",", "8023.32`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:40:00\"\>", ",", "8022.77`", ",", "8023.32`", 
     ",", "7978.72`", ",", "7979.36`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T13:50:00\"\>", ",", "7978.72`", ",", "7986.05`", 
     ",", "7949.97`", ",", "7976.17`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:00:00\"\>", ",", "7976.09`", ",", "7980.47`", 
     ",", "7957.69`", ",", "7957.69`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:10:00\"\>", ",", "7957.3`", ",", "7979.6`", 
     ",", "7956.02`", ",", "7961.92`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:20:00\"\>", ",", "7962.15`", ",", "7971.23`", 
     ",", "7929.34`", ",", "7950.76`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:30:00\"\>", ",", "7951.`", ",", "7970.84`", 
     ",", "7940.89`", ",", "7970.84`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:40:00\"\>", ",", "7971.31`", ",", "7978.`", 
     ",", "7952.6`", ",", "7965.34`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T14:50:00\"\>", ",", "7966.22`", ",", "7998.23`", 
     ",", "7940.09`", ",", "7986.21`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:00:00\"\>", ",", "7986.69`", ",", "7993.77`", 
     ",", "7958.33`", ",", "7985.73`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:10:00\"\>", ",", "7985.81`", ",", "7989.15`", 
     ",", "7951.88`", ",", "7972.19`", ",", "301", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:20:00\"\>", ",", "7971.87`", ",", "7971.87`", 
     ",", "7941.6`", ",", "7959.84`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:30:00\"\>", ",", "7959.53`", ",", "7971.79`", 
     ",", "7947.82`", ",", "7961.52`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:40:00\"\>", ",", "7961.76`", ",", "7973.38`", 
     ",", "7945.75`", ",", "7948.85`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T15:50:00\"\>", ",", "7948.69`", ",", "7969.96`", 
     ",", "7946.54`", ",", "7954.91`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-04T16:00:00\"\>", ",", "7954.83`", ",", "7956.74`", 
     ",", "7954.75`", ",", "7956.66`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T09:30:00\"\>", ",", "7954.83`", ",", "7954.83`", 
     ",", "7897.48`", ",", "7910.94`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T09:40:00\"\>", ",", "7911.34`", ",", "7923.84`", 
     ",", "7884.42`", ",", "7891.19`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T09:50:00\"\>", ",", "7889.12`", ",", "7891.9`", 
     ",", "7849.05`", ",", "7853.12`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:00:00\"\>", ",", "7848.5`", ",", "7861.72`", 
     ",", "7845.39`", ",", "7848.18`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:10:00\"\>", ",", "7848.58`", ",", "7875.98`", 
     ",", "7848.1`", ",", "7867.13`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:20:00\"\>", ",", "7867.21`", ",", "7872.39`", 
     ",", "7851.36`", ",", "7856.7`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:30:00\"\>", ",", "7856.38`", ",", "7892.22`", 
     ",", "7846.82`", ",", "7887.84`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:40:00\"\>", ",", "7888.32`", ",", "7928.22`", 
     ",", "7885.93`", ",", "7916.28`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T10:50:00\"\>", ",", "7917.47`", ",", "7987.88`", 
     ",", "7912.61`", ",", "7966.77`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:00:00\"\>", ",", "7968.13`", ",", "8002.69`", 
     ",", "7962.87`", ",", "7990.91`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:10:00\"\>", ",", "7989.39`", ",", "7997.28`", 
     ",", "7963.03`", ",", "7968.76`", ",", "301", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:20:00\"\>", ",", "7968.61`", ",", "7990.43`", 
     ",", "7963.83`", ",", "7978.96`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:30:00\"\>", ",", "7977.37`", ",", "8011.93`", 
     ",", "7976.49`", ",", "7995.29`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:40:00\"\>", ",", "7995.21`", ",", "7999.51`", 
     ",", "7985.25`", ",", "7992.42`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T11:50:00\"\>", ",", "7992.34`", ",", "8023.88`", 
     ",", "7992.34`", ",", "8022.21`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:00:00\"\>", ",", "8023.48`", ",", "8082.66`", 
     ",", "8023.48`", ",", "8074.86`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:10:00\"\>", ",", "8075.49`", ",", "8094.69`", 
     ",", "8057.81`", ",", "8058.05`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:20:00\"\>", ",", "8060.04`", ",", "8065.38`", 
     ",", "8039.17`", ",", "8041.32`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:30:00\"\>", ",", "8040.69`", ",", "8049.37`", 
     ",", "8033.52`", ",", "8041.96`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:40:00\"\>", ",", "8040.21`", ",", "8065.22`", 
     ",", "8040.21`", ",", "8051.76`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T12:50:00\"\>", ",", "8051.92`", ",", "8066.17`", 
     ",", "8028.34`", ",", "8062.03`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:00:00\"\>", ",", "8062.19`", ",", "8080.59`", 
     ",", "8055.02`", ",", "8080.59`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:10:00\"\>", ",", "8081.63`", ",", "8086.01`", 
     ",", "8040.77`", ",", "8045.55`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:20:00\"\>", ",", "8046.66`", ",", "8048.73`", 
     ",", "8017.43`", ",", "8021.81`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:30:00\"\>", ",", "8022.61`", ",", "8044.11`", 
     ",", "8020.62`", ",", "8032.08`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:40:00\"\>", ",", "8032.01`", ",", "8062.35`", 
     ",", "8022.45`", ",", "8059.8`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T13:50:00\"\>", ",", "8059.48`", ",", "8059.48`", 
     ",", "8031.45`", ",", "8035.27`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:00:00\"\>", ",", "8034.63`", ",", "8080.03`", 
     ",", "8034.63`", ",", "8074.94`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:10:00\"\>", ",", "8074.7`", ",", "8074.7`", 
     ",", "8049.45`", ",", "8062.75`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:20:00\"\>", ",", "8061.79`", ",", "8100.26`", 
     ",", "8060.44`", ",", "8100.02`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:30:00\"\>", ",", "8100.26`", ",", "8107.27`", 
     ",", "8079.24`", ",", "8095.25`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:40:00\"\>", ",", "8095.64`", ",", "8096.84`", 
     ",", "8067.93`", ",", "8068.8`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T14:50:00\"\>", ",", "8068.24`", ",", "8068.64`", 
     ",", "8030.09`", ",", "8037.58`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:00:00\"\>", ",", "8037.82`", ",", "8053.43`", 
     ",", "8005.24`", ",", "8023.8`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:10:00\"\>", ",", "8023.4`", ",", "8049.21`", 
     ",", "8023.4`", ",", "8031.85`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:20:00\"\>", ",", "8031.61`", ",", "8074.22`", 
     ",", "8031.61`", ",", "8071.67`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:30:00\"\>", ",", "8072.63`", ",", "8077.96`", 
     ",", "8048.01`", ",", "8052.24`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:40:00\"\>", ",", "8052.87`", ",", "8083.3`", 
     ",", "8052.87`", ",", "8059.17`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T15:50:00\"\>", ",", "8061.16`", ",", "8073.9`", 
     ",", "8047.93`", ",", "8058.93`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-05T16:00:00\"\>", ",", "8059.24`", ",", "8063.47`", 
     ",", "8059.24`", ",", "8063.07`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T09:30:00\"\>", ",", "8056.38`", ",", "8171.31`", 
     ",", "8056.38`", ",", "8161.99`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T09:40:00\"\>", ",", "8161.43`", ",", "8196.88`", 
     ",", "8158.09`", ",", "8182.94`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T09:50:00\"\>", ",", "8182.7`", ",", "8218.78`", 
     ",", "8180.71`", ",", "8211.29`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:00:00\"\>", ",", "8213.04`", ",", "8220.29`", 
     ",", "8192.42`", ",", "8197.27`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:10:00\"\>", ",", "8198.95`", ",", "8215.04`", 
     ",", "8193.05`", ",", "8195.68`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:20:00\"\>", ",", "8195.28`", ",", "8207.63`", 
     ",", "8180.15`", ",", "8203.41`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:30:00\"\>", ",", "8203.57`", ",", "8211.37`", 
     ",", "8196.88`", ",", "8208.11`", ",", "314", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:40:00\"\>", ",", "8206.19`", ",", "8217.5`", 
     ",", "8186.52`", ",", "8214.08`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T10:50:00\"\>", ",", "8214.16`", ",", "8231.68`", 
     ",", "8208.43`", ",", "8214.96`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:00:00\"\>", ",", "8215.2`", ",", "8250.72`", 
     ",", "8215.2`", ",", "8242.75`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:10:00\"\>", ",", "8243.15`", ",", "8260.28`", 
     ",", "8242.91`", ",", "8255.42`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:20:00\"\>", ",", "8254.86`", ",", "8254.86`", 
     ",", "8231.76`", ",", "8240.28`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:30:00\"\>", ",", "8240.52`", ",", "8244.43`", 
     ",", "8233.12`", ",", "8237.02`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:40:00\"\>", ",", "8237.1`", ",", "8258.13`", 
     ",", "8237.1`", ",", "8248.17`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T11:50:00\"\>", ",", "8247.45`", ",", "8260.91`", 
     ",", "8241.48`", ",", "8250.72`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:00:00\"\>", ",", "8250.64`", ",", "8251.36`", 
     ",", "8233.04`", ",", "8240.2`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:10:00\"\>", ",", "8240.44`", ",", "8249.13`", 
     ",", "8215.27`", ",", "8215.83`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:20:00\"\>", ",", "8215.75`", ",", "8238.93`", 
     ",", "8215.67`", ",", "8220.21`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:30:00\"\>", ",", "8220.29`", ",", "8251.99`", 
     ",", "8217.58`", ",", "8245.62`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:40:00\"\>", ",", "8245.78`", ",", "8253.82`", 
     ",", "8241.08`", ",", "8244.27`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T12:50:00\"\>", ",", "8244.19`", ",", "8249.76`", 
     ",", "8234.15`", ",", "8243.87`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:00:00\"\>", ",", "8243.23`", ",", "8268.48`", 
     ",", "8233.35`", ",", "8266.89`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:10:00\"\>", ",", "8266.73`", ",", "8288.47`", 
     ",", "8260.75`", ",", "8288.47`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:20:00\"\>", ",", "8287.83`", ",", "8294.29`", 
     ",", "8277.4`", ",", "8286.72`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:30:00\"\>", ",", "8286.56`", ",", "8299.38`", 
     ",", "8286.56`", ",", "8299.38`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:40:00\"\>", ",", "8298.75`", ",", "8299.06`", 
     ",", "8284.17`", ",", "8287.59`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T13:50:00\"\>", ",", "8287.52`", ",", "8293.33`", 
     ",", "8281.78`", ",", "8291.18`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:00:00\"\>", ",", "8291.02`", ",", "8291.26`", 
     ",", "8269.12`", ",", "8277.4`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:10:00\"\>", ",", "8276.92`", ",", "8290.94`", 
     ",", "8260.28`", ",", "8266.65`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:20:00\"\>", ",", "8266.25`", ",", "8289.75`", 
     ",", "8263.46`", ",", "8286.72`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:30:00\"\>", ",", "8286.8`", ",", "8312.29`", 
     ",", "8286.56`", ",", "8287.44`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:40:00\"\>", ",", "8288.47`", ",", "8296.2`", 
     ",", "8273.02`", ",", "8273.58`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T14:50:00\"\>", ",", "8272.3`", ",", "8278.91`", 
     ",", "8260.12`", ",", "8275.57`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:00:00\"\>", ",", "8275.25`", ",", "8275.25`", 
     ",", "8239.57`", ",", "8243.47`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:10:00\"\>", ",", "8243.31`", ",", "8262.27`", 
     ",", "8230.81`", ",", "8256.77`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:20:00\"\>", ",", "8256.13`", ",", "8269.99`", 
     ",", "8241.8`", ",", "8263.06`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:30:00\"\>", ",", "8263.38`", ",", "8282.98`", 
     ",", "8261.87`", ",", "8281.38`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:40:00\"\>", ",", "8283.69`", ",", "8311.57`", 
     ",", "8281.3`", ",", "8294.05`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T15:50:00\"\>", ",", "8294.92`", ",", "8299.54`", 
     ",", "8261.55`", ",", "8277.56`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-06T16:00:00\"\>", ",", "8275.09`", ",", "8280.59`", 
     ",", "8275.09`", ",", "8280.59`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T09:30:00\"\>", ",", "8281.38`", ",", "8281.38`", 
     ",", "8233.35`", ",", "8245.22`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T09:40:00\"\>", ",", "8244.82`", ",", "8260.04`", 
     ",", "8233.67`", ",", "8257.41`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T09:50:00\"\>", ",", "8257.57`", ",", "8257.57`", 
     ",", "8235.82`", ",", "8242.83`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:00:00\"\>", ",", "8242.59`", ",", "8242.59`", 
     ",", "8196.8`", ",", "8204.44`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:10:00\"\>", ",", "8204.76`", ",", "8225.15`", 
     ",", "8202.05`", ",", "8219.97`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:20:00\"\>", ",", "8220.53`", ",", "8236.86`", 
     ",", "8219.42`", ",", "8226.27`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:30:00\"\>", ",", "8226.82`", ",", "8253.9`", 
     ",", "8219.18`", ",", "8253.74`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:40:00\"\>", ",", "8254.46`", ",", "8298.43`", 
     ",", "8253.9`", ",", "8295.88`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T10:50:00\"\>", ",", "8295.8`", ",", "8314.75`", 
     ",", "8284.25`", ",", "8312.13`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:00:00\"\>", ",", "8311.89`", ",", "8312.21`", 
     ",", "8283.77`", ",", "8295.4`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:10:00\"\>", ",", "8295.56`", ",", "8296.67`", 
     ",", "8276.68`", ",", "8280.03`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:20:00\"\>", ",", "8279.55`", ",", "8293.33`", 
     ",", "8275.89`", ",", "8281.06`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:30:00\"\>", ",", "8280.9`", ",", "8283.21`", 
     ",", "8268.24`", ",", "8272.3`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:40:00\"\>", ",", "8272.62`", ",", "8272.7`", 
     ",", "8254.22`", ",", "8259.72`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T11:50:00\"\>", ",", "8260.99`", ",", "8272.3`", 
     ",", "8260.36`", ",", "8271.35`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:00:00\"\>", ",", "8271.59`", ",", "8277.24`", 
     ",", "8258.84`", ",", "8264.5`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:10:00\"\>", ",", "8263.3`", ",", "8282.1`", 
     ",", "8260.75`", ",", "8277.48`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:20:00\"\>", ",", "8278.52`", ",", "8289.43`", 
     ",", "8271.98`", ",", "8287.2`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:30:00\"\>", ",", "8289.11`", ",", "8290.94`", 
     ",", "8278.91`", ",", "8281.62`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:40:00\"\>", ",", "8281.7`", ",", "8288.87`", 
     ",", "8271.82`", ",", "8283.93`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T12:50:00\"\>", ",", "8283.69`", ",", "8301.14`", 
     ",", "8283.69`", ",", "8296.12`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:00:00\"\>", ",", "8296.04`", ",", "8305.99`", 
     ",", "8293.25`", ",", "8300.98`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:10:00\"\>", ",", "8300.26`", ",", "8305.99`", 
     ",", "8285.05`", ",", "8285.44`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:20:00\"\>", ",", "8285.29`", ",", "8285.68`", 
     ",", "8277.8`", ",", "8277.8`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:30:00\"\>", ",", "8278.04`", ",", "8278.52`", 
     ",", "8236.54`", ",", "8250.24`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:40:00\"\>", ",", "8249.84`", ",", "8256.05`", 
     ",", "8245.06`", ",", "8255.9`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T13:50:00\"\>", ",", "8255.74`", ",", "8263.46`", 
     ",", "8244.9`", ",", "8263.46`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:00:00\"\>", ",", "8263.78`", ",", "8269.2`", 
     ",", "8261.39`", ",", "8264.34`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:10:00\"\>", ",", "8263.7`", ",", "8264.02`", 
     ",", "8233.67`", ",", "8239.73`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:20:00\"\>", ",", "8239.97`", ",", "8253.03`", 
     ",", "8219.42`", ",", "8227.54`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:30:00\"\>", ",", "8228.1`", ",", "8237.42`", 
     ",", "8224.27`", ",", "8235.35`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:40:00\"\>", ",", "8235.43`", ",", "8240.28`", 
     ",", "8215.75`", ",", "8230.09`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T14:50:00\"\>", ",", "8230.25`", ",", "8246.18`", 
     ",", "8229.69`", ",", "8239.25`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:00:00\"\>", ",", "8239.09`", ",", "8245.38`", 
     ",", "8231.36`", ",", "8235.19`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:10:00\"\>", ",", "8235.27`", ",", "8256.13`", 
     ",", "8223.72`", ",", "8251.12`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:20:00\"\>", ",", "8251.2`", ",", "8290.3`", 
     ",", "8250.72`", ",", "8287.04`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:30:00\"\>", ",", "8287.83`", ",", "8297.63`", 
     ",", "8267.92`", ",", "8267.92`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:40:00\"\>", ",", "8268.48`", ",", "8278.28`", 
     ",", "8257.49`", ",", "8271.59`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T15:50:00\"\>", ",", "8271.27`", ",", "8273.18`", 
     ",", "8252.31`", ",", "8268.32`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-09T16:00:00\"\>", ",", "8267.92`", ",", "8271.03`", 
     ",", "8267.6`", ",", "8270.87`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T09:30:00\"\>", ",", "8269.36`", ",", "8269.36`", 
     ",", "8200.06`", ",", "8221.73`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T09:40:00\"\>", ",", "8222.84`", ",", "8251.2`", 
     ",", "8209.78`", ",", "8240.12`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T09:50:00\"\>", ",", "8240.92`", ",", "8257.41`", 
     ",", "8215.12`", ",", "8221.17`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:00:00\"\>", ",", "8221.01`", ",", "8236.14`", 
     ",", "8208.03`", ",", "8232.16`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:10:00\"\>", ",", "8233.2`", ",", "8235.03`", 
     ",", "8187.48`", ",", "8187.48`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:20:00\"\>", ",", "8188.83`", ",", "8190.03`", 
     ",", "8154.5`", ",", "8160.8`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:30:00\"\>", ",", "8161.35`", ",", "8180.79`", 
     ",", "8158.72`", ",", "8168.92`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:40:00\"\>", ",", "8169.32`", ",", "8185.96`", 
     ",", "8161.11`", ",", "8184.29`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T10:50:00\"\>", ",", "8183.1`", ",", "8188.11`", 
     ",", "8173.14`", ",", "8180.87`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:00:00\"\>", ",", "8181.03`", ",", "8228.18`", 
     ",", "8082.26`", ",", "8094.29`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:10:00\"\>", ",", "8092.3`", ",", "8101.14`", 
     ",", "8040.13`", ",", "8077.8`", ",", "298", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:20:00\"\>", ",", "8078.44`", ",", "8105.28`", 
     ",", "8064.18`", ",", "8079.08`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:30:00\"\>", ",", "8077.48`", ",", "8077.48`", 
     ",", "7959.61`", ",", "7966.85`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:40:00\"\>", ",", "7964.22`", ",", "7994.49`", 
     ",", "7953.47`", ",", "7986.69`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T11:50:00\"\>", ",", "7986.77`", ",", "7999.59`", 
     ",", "7963.35`", ",", "7987.88`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:00:00\"\>", ",", "7989.95`", ",", "8003.41`", 
     ",", "7974.66`", ",", "7976.17`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:10:00\"\>", ",", "7978.48`", ",", "8007.95`", 
     ",", "7978.48`", ",", "7998.31`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:20:00\"\>", ",", "7998.15`", ",", "8004.85`", 
     ",", "7974.26`", ",", "7982.38`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:30:00\"\>", ",", "7983.02`", ",", "7997.76`", 
     ",", "7970.76`", ",", "7992.82`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:40:00\"\>", ",", "7993.61`", ",", "8004.37`", 
     ",", "7985.41`", ",", "8003.89`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T12:50:00\"\>", ",", "8004.21`", ",", "8014.96`", 
     ",", "8000.46`", ",", "8000.7`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:00:00\"\>", ",", "8000.54`", ",", "8001.18`", 
     ",", "7958.73`", ",", "7964.94`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:10:00\"\>", ",", "7965.42`", ",", "7972.99`", 
     ",", "7934.83`", ",", "7934.83`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:20:00\"\>", ",", "7934.68`", ",", "7934.68`", 
     ",", "7903.93`", ",", "7922.41`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:30:00\"\>", ",", "7920.74`", ",", "7948.69`", 
     ",", "7914.37`", ",", "7945.03`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:40:00\"\>", ",", "7944.39`", ",", "7949.57`", 
     ",", "7926.79`", ",", "7935.79`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T13:50:00\"\>", ",", "7936.99`", ",", "7943.83`", 
     ",", "7924.8`", ",", "7925.68`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:00:00\"\>", ",", "7925.04`", ",", "7947.9`", 
     ",", "7921.45`", ",", "7931.97`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:10:00\"\>", ",", "7932.84`", ",", "7937.14`", 
     ",", "7914.37`", ",", "7922.33`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:20:00\"\>", ",", "7921.77`", ",", "7923.29`", 
     ",", "7899.87`", ",", "7900.27`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:30:00\"\>", ",", "7899.87`", ",", "7901.38`", 
     ",", "7884.34`", ",", "7901.38`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:40:00\"\>", ",", "7900.98`", ",", "7920.1`", 
     ",", "7900.98`", ",", "7909.35`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T14:50:00\"\>", ",", "7909.43`", ",", "7910.06`", 
     ",", "7879.08`", ",", "7879.64`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:00:00\"\>", ",", "7883.14`", ",", "7919.78`", 
     ",", "7883.14`", ",", "7914.44`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:10:00\"\>", ",", "7914.68`", ",", "7917.15`", 
     ",", "7870.4`", ",", "7875.66`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:20:00\"\>", ",", "7877.73`", ",", "7894.06`", 
     ",", "7861.96`", ",", "7882.11`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:30:00\"\>", ",", "7880.67`", ",", "7901.38`", 
     ",", "7863.47`", ",", "7865.46`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:40:00\"\>", ",", "7864.67`", ",", "7912.69`", 
     ",", "7849.05`", ",", "7904.73`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T15:50:00\"\>", ",", "7903.85`", ",", "7931.65`", 
     ",", "7885.05`", ",", "7885.37`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-10T16:00:00\"\>", ",", "7884.26`", ",", "7888.88`", 
     ",", "7884.02`", ",", "7888.88`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T09:30:00\"\>", ",", "7887.05`", ",", "7968.69`", 
     ",", "7887.05`", ",", "7959.84`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T09:40:00\"\>", ",", "7963.91`", ",", "7972.83`", 
     ",", "7937.06`", ",", "7969.56`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T09:50:00\"\>", ",", "7970.2`", ",", "7974.42`", 
     ",", "7952.99`", ",", "7954.51`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:00:00\"\>", ",", "7952.44`", ",", "7965.74`", 
     ",", "7925.99`", ",", "7927.27`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:10:00\"\>", ",", "7926.79`", ",", "7937.94`", 
     ",", "7912.77`", ",", "7912.77`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:20:00\"\>", ",", "7911.9`", ",", "7932.76`", 
     ",", "7901.46`", ",", "7901.46`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:30:00\"\>", ",", "7898.83`", ",", "7907.83`", 
     ",", "7882.67`", ",", "7901.46`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:40:00\"\>", ",", "7902.34`", ",", "7938.18`", 
     ",", "7901.3`", ",", "7936.51`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T10:50:00\"\>", ",", "7937.38`", ",", "7950.05`", 
     ",", "7937.06`", ",", "7943.44`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:00:00\"\>", ",", "7943.12`", ",", "7948.85`", 
     ",", "7921.93`", ",", "7925.04`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:10:00\"\>", ",", "7924.72`", ",", "7967.09`", 
     ",", "7922.09`", ",", "7960.32`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:20:00\"\>", ",", "7960.8`", ",", "7971.23`", 
     ",", "7951.72`", ",", "7968.29`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:30:00\"\>", ",", "7967.25`", ",", "7983.26`", 
     ",", "7956.02`", ",", "7957.22`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:40:00\"\>", ",", "7956.5`", ",", "7963.43`", 
     ",", "7945.03`", ",", "7946.46`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T11:50:00\"\>", ",", "7946.78`", ",", "7955.94`", 
     ",", "7940.97`", ",", "7947.26`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:00:00\"\>", ",", "7947.18`", ",", "7956.1`", 
     ",", "7945.43`", ",", "7954.99`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:10:00\"\>", ",", "7955.38`", ",", "7956.42`", 
     ",", "7921.69`", ",", "7928.62`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:20:00\"\>", ",", "7928.3`", ",", "7939.06`", 
     ",", "7925.52`", ",", "7937.86`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:30:00\"\>", ",", "7936.75`", ",", "7948.14`", 
     ",", "7922.97`", ",", "7948.14`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:40:00\"\>", ",", "7948.77`", ",", "7960.4`", 
     ",", "7947.34`", ",", "7950.29`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T12:50:00\"\>", ",", "7950.45`", ",", "7956.66`", 
     ",", "7918.75`", ",", "7923.45`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:00:00\"\>", ",", "7923.84`", ",", "7942.24`", 
     ",", "7908.31`", ",", "7910.54`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:10:00\"\>", ",", "7910.3`", ",", "7925.75`", 
     ",", "7908.79`", ",", "7918.43`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:20:00\"\>", ",", "7918.51`", ",", "7928.14`", 
     ",", "7908.47`", ",", "7919.14`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:30:00\"\>", ",", "7919.38`", ",", "7919.38`", 
     ",", "7900.35`", ",", "7905.76`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:40:00\"\>", ",", "7905.6`", ",", "7906.16`", 
     ",", "7866.18`", ",", "7870.24`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T13:50:00\"\>", ",", "7869.84`", ",", "7883.38`", 
     ",", "7864.82`", ",", "7875.34`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:00:00\"\>", ",", "7875.5`", ",", "7894.13`", 
     ",", "7875.5`", ",", "7893.42`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:10:00\"\>", ",", "7893.26`", ",", "7893.5`", 
     ",", "7858.13`", ",", "7862.2`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:20:00\"\>", ",", "7861.8`", ",", "7898.67`", 
     ",", "7852.32`", ",", "7892.14`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:30:00\"\>", ",", "7891.98`", ",", "7908.87`", 
     ",", "7885.45`", ",", "7905.92`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:40:00\"\>", ",", "7906.`", ",", "7912.37`", 
     ",", "7897.72`", ",", "7898.83`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T14:50:00\"\>", ",", "7898.36`", ",", "7966.85`", 
     ",", "7898.04`", ",", "7958.25`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:00:00\"\>", ",", "7959.84`", ",", "7963.51`", 
     ",", "7938.82`", ",", "7942.64`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:10:00\"\>", ",", "7942.24`", ",", "7976.89`", 
     ",", "7940.73`", ",", "7955.62`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:20:00\"\>", ",", "7955.7`", ",", "7955.86`", 
     ",", "7933.64`", ",", "7942.8`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:30:00\"\>", ",", "7941.68`", ",", "7956.66`", 
     ",", "7913.25`", ",", "7916.75`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:40:00\"\>", ",", "7916.68`", ",", "7924.96`", 
     ",", "7902.1`", ",", "7915.48`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T15:50:00\"\>", ",", "7915.08`", ",", "7942.08`", 
     ",", "7910.7`", ",", "7941.29`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-11T16:00:00\"\>", ",", "7941.13`", ",", "7941.29`", 
     ",", "7939.53`", ",", "7939.53`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T09:30:00\"\>", ",", "7931.97`", ",", "7931.97`", 
     ",", "7776.97`", ",", "7780.88`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T09:40:00\"\>", ",", "7779.6`", ",", "7795.77`", 
     ",", "7774.98`", ",", "7793.46`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T09:50:00\"\>", ",", "7793.14`", ",", "7794.18`", 
     ",", "7740.49`", ",", "7749.89`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:00:00\"\>", ",", "7749.1`", ",", "7765.26`", 
     ",", "7723.61`", ",", "7765.26`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:10:00\"\>", ",", "7764.47`", ",", "7770.12`", 
     ",", "7738.34`", ",", "7744.64`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:20:00\"\>", ",", "7744.87`", ",", "7772.91`", 
     ",", "7735.24`", ",", "7768.77`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:30:00\"\>", ",", "7765.03`", ",", "7771.08`", 
     ",", "7751.49`", ",", "7760.09`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:40:00\"\>", ",", "7760.17`", ",", "7797.04`", 
     ",", "7748.46`", ",", "7794.02`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T10:50:00\"\>", ",", "7794.65`", ",", "7800.15`", 
     ",", "7779.68`", ",", "7789.64`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:00:00\"\>", ",", "7789.96`", ",", "7801.74`", 
     ",", "7767.57`", ",", "7796.81`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:10:00\"\>", ",", "7797.04`", ",", "7798.48`", 
     ",", "7758.65`", ",", "7761.92`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:20:00\"\>", ",", "7760.88`", ",", "7792.58`", 
     ",", "7752.2`", ",", "7785.18`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:30:00\"\>", ",", "7784.3`", ",", "7801.35`", 
     ",", "7783.9`", ",", "7795.29`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:40:00\"\>", ",", "7795.61`", ",", "7835.04`", 
     ",", "7795.61`", ",", "7829.86`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T11:50:00\"\>", ",", "7829.22`", ",", "7836.63`", 
     ",", "7804.45`", ",", "7804.45`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:00:00\"\>", ",", "7803.18`", ",", "7830.74`", 
     ",", "7801.66`", ",", "7819.19`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:10:00\"\>", ",", "7819.43`", ",", "7827.63`", 
     ",", "7810.19`", ",", "7826.75`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:20:00\"\>", ",", "7828.11`", ",", "7859.49`", 
     ",", "7824.52`", ",", "7852.16`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:30:00\"\>", ",", "7852.32`", ",", "7856.3`", 
     ",", "7840.69`", ",", "7843.72`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:40:00\"\>", ",", "7843.88`", ",", "7846.51`", 
     ",", "7825.24`", ",", "7829.86`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T12:50:00\"\>", ",", "7829.94`", ",", "7837.66`", 
     ",", "7822.93`", ",", "7823.57`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:00:00\"\>", ",", "7824.36`", ",", "7825.08`", 
     ",", "7799.59`", ",", "7816.08`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:10:00\"\>", ",", "7816.72`", ",", "7833.92`", 
     ",", "7791.71`", ",", "7793.46`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:20:00\"\>", ",", "7792.66`", ",", "7809.95`", 
     ",", "7784.38`", ",", "7796.49`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:30:00\"\>", ",", "7796.57`", ",", "7806.92`", 
     ",", "7766.62`", ",", "7766.7`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:40:00\"\>", ",", "7767.34`", ",", "7776.34`", 
     ",", "7762.08`", ",", "7775.14`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T13:50:00\"\>", ",", "7775.38`", ",", "7802.38`", 
     ",", "7775.38`", ",", "7797.28`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:00:00\"\>", ",", "7796.81`", ",", "7796.81`", 
     ",", "7765.03`", ",", "7784.14`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:10:00\"\>", ",", "7784.3`", ",", "7812.73`", 
     ",", "7781.19`", ",", "7793.06`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:20:00\"\>", ",", "7793.22`", ",", "7797.6`", 
     ",", "7767.81`", ",", "7770.2`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:30:00\"\>", ",", "7770.44`", ",", "7779.76`", 
     ",", "7738.34`", ",", "7747.03`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:40:00\"\>", ",", "7745.43`", ",", "7748.46`", 
     ",", "7715.48`", ",", "7729.18`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T14:50:00\"\>", ",", "7729.58`", ",", "7731.18`", 
     ",", "7710.87`", ",", "7719.87`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:00:00\"\>", ",", "7720.34`", ",", "7725.12`", 
     ",", "7694.38`", ",", "7725.12`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:10:00\"\>", ",", "7726.72`", ",", "7765.03`", 
     ",", "7726.72`", ",", "7761.92`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:20:00\"\>", ",", "7762.95`", ",", "7836.07`", 
     ",", "7745.19`", ",", "7836.07`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:30:00\"\>", ",", "7835.99`", ",", "7854.23`", 
     ",", "7808.19`", ",", "7830.26`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:40:00\"\>", ",", "7830.18`", ",", "7911.42`", 
     ",", "7830.18`", ",", "7878.52`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T15:50:00\"\>", ",", "7877.97`", ",", "7938.02`", 
     ",", "7871.75`", ",", "7928.78`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-12T16:00:00\"\>", ",", "7930.14`", ",", "7932.84`", 
     ",", "7929.5`", ",", "7932.76`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T09:30:00\"\>", ",", "7933.`", ",", "7945.19`", 
     ",", "7892.46`", ",", "7893.02`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T09:40:00\"\>", ",", "7893.18`", ",", "7900.9`", 
     ",", "7878.76`", ",", "7898.04`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T09:50:00\"\>", ",", "7898.83`", ",", "7913.49`", 
     ",", "7873.27`", ",", "7911.9`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:00:00\"\>", ",", "7911.58`", ",", "7927.99`", 
     ",", "7902.9`", ",", "7916.68`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:10:00\"\>", ",", "7915.16`", ",", "7928.14`", 
     ",", "7891.43`", ",", "7898.91`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:20:00\"\>", ",", "7900.67`", ",", "7938.98`", 
     ",", "7900.19`", ",", "7938.58`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:30:00\"\>", ",", "7936.91`", ",", "7968.92`", 
     ",", "7936.91`", ",", "7965.18`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:40:00\"\>", ",", "7965.02`", ",", "7970.44`", 
     ",", "7943.91`", ",", "7946.54`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T10:50:00\"\>", ",", "7945.99`", ",", "7962.23`", 
     ",", "7941.6`", ",", "7951.64`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:00:00\"\>", ",", "7951.56`", ",", "7952.36`", 
     ",", "7926.71`", ",", "7933.08`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:10:00\"\>", ",", "7932.92`", ",", "7939.3`", 
     ",", "7906.72`", ",", "7909.98`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:20:00\"\>", ",", "7910.06`", ",", "7910.06`", 
     ",", "7886.97`", ",", "7894.85`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:30:00\"\>", ",", "7894.77`", ",", "7909.19`", 
     ",", "7889.28`", ",", "7902.34`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:40:00\"\>", ",", "7902.42`", ",", "7907.12`", 
     ",", "7864.35`", ",", "7870.96`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T11:50:00\"\>", ",", "7871.12`", ",", "7879.88`", 
     ",", "7861.24`", ",", "7861.96`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:00:00\"\>", ",", "7861.24`", ",", "7881.79`", 
     ",", "7860.84`", ",", "7871.12`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:10:00\"\>", ",", "7870.96`", ",", "7881.63`", 
     ",", "7854.47`", ",", "7862.59`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:20:00\"\>", ",", "7859.09`", ",", "7880.44`", 
     ",", "7854.39`", ",", "7878.44`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:30:00\"\>", ",", "7879.96`", ",", "7885.53`", 
     ",", "7867.69`", ",", "7867.69`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:40:00\"\>", ",", "7867.77`", ",", "7873.27`", 
     ",", "7853.75`", ",", "7864.82`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T12:50:00\"\>", ",", "7865.46`", ",", "7865.94`", 
     ",", "7840.53`", ",", "7850.09`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:00:00\"\>", ",", "7848.82`", ",", "7865.94`", 
     ",", "7842.76`", ",", "7862.12`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:10:00\"\>", ",", "7862.04`", ",", "7876.61`", 
     ",", "7848.89`", ",", "7871.44`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:20:00\"\>", ",", "7871.28`", ",", "7880.99`", 
     ",", "7867.85`", ",", "7878.05`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:30:00\"\>", ",", "7876.77`", ",", "7886.17`", 
     ",", "7869.68`", ",", "7878.92`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:40:00\"\>", ",", "7879.`", ",", "7926.39`", 
     ",", "7875.5`", ",", "7925.68`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T13:50:00\"\>", ",", "7924.96`", ",", "7941.84`", 
     ",", "7920.82`", ",", "7926.87`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:00:00\"\>", ",", "7927.19`", ",", "7943.99`", 
     ",", "7902.58`", ",", "7909.83`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:10:00\"\>", ",", "7910.38`", ",", "7924.72`", 
     ",", "7896.6`", ",", "7919.22`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:20:00\"\>", ",", "7919.38`", ",", "7925.6`", 
     ",", "7906.48`", ",", "7912.53`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:30:00\"\>", ",", "7910.38`", ",", "7910.38`", 
     ",", "7891.51`", ",", "7904.49`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:40:00\"\>", ",", "7904.25`", ",", "7918.67`", 
     ",", "7893.34`", ",", "7902.58`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T14:50:00\"\>", ",", "7902.66`", ",", "7921.69`", 
     ",", "7896.52`", ",", "7907.04`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:00:00\"\>", ",", "7907.52`", ",", "7912.61`", 
     ",", "7869.76`", ",", "7875.98`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:10:00\"\>", ",", "7877.33`", ",", "7877.57`", 
     ",", "7852.48`", ",", "7861.08`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:20:00\"\>", ",", "7859.97`", ",", "7890.63`", 
     ",", "7858.69`", ",", "7885.93`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:30:00\"\>", ",", "7886.89`", ",", "7911.34`", 
     ",", "7881.47`", ",", "7892.78`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:40:00\"\>", ",", "7892.06`", ",", "7913.09`", 
     ",", "7882.67`", ",", "7906.32`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T15:50:00\"\>", ",", "7906.16`", ",", "7908.39`", 
     ",", "7843.24`", ",", "7843.24`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-13T16:00:00\"\>", ",", "7843.32`", ",", "7850.41`", 
     ",", "7843.32`", ",", "7850.41`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T09:30:00\"\>", ",", "7845.47`", ",", "7845.47`", 
     ",", "7632.33`", ",", "7632.33`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T09:40:00\"\>", ",", "7633.53`", ",", "7651.13`", 
     ",", "7593.46`", ",", "7598.4`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T09:50:00\"\>", ",", "7592.35`", ",", "7600.23`", 
     ",", "7554.91`", ",", "7589.4`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:00:00\"\>", ",", "7587.57`", ",", "7608.2`", 
     ",", "7585.18`", ",", "7601.67`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:10:00\"\>", ",", "7601.59`", ",", "7609.31`", 
     ",", "7573.07`", ",", "7575.22`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:20:00\"\>", ",", "7575.54`", ",", "7587.89`", 
     ",", "7567.9`", ",", "7585.34`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:30:00\"\>", ",", "7586.06`", ",", "7607.48`", 
     ",", "7580.48`", ",", "7588.84`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:40:00\"\>", ",", "7589.16`", ",", "7617.76`", 
     ",", "7583.75`", ",", "7608.92`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T10:50:00\"\>", ",", "7608.76`", ",", "7635.92`", 
     ",", "7608.68`", ",", "7618.79`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:00:00\"\>", ",", "7619.27`", ",", "7619.27`", 
     ",", "7594.82`", ",", "7598.8`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:10:00\"\>", ",", "7598.72`", ",", "7616.96`", 
     ",", "7598.48`", ",", "7608.2`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:20:00\"\>", ",", "7608.28`", ",", "7620.78`", 
     ",", "7598.24`", ",", "7600.15`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:30:00\"\>", ",", "7599.92`", ",", "7600.07`", 
     ",", "7585.1`", ",", "7590.52`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:40:00\"\>", ",", "7590.36`", ",", "7600.23`", 
     ",", "7585.66`", ",", "7590.12`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T11:50:00\"\>", ",", "7590.36`", ",", "7590.36`", 
     ",", "7568.38`", ",", "7580.08`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:00:00\"\>", ",", "7580.56`", ",", "7599.92`", 
     ",", "7579.69`", ",", "7583.75`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:10:00\"\>", ",", "7583.99`", ",", "7591.15`", 
     ",", "7573.95`", ",", "7580.16`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:20:00\"\>", ",", "7580.4`", ",", "7582.79`", 
     ",", "7561.92`", ",", "7578.81`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:30:00\"\>", ",", "7580.56`", ",", "7590.36`", 
     ",", "7579.37`", ",", "7590.36`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:40:00\"\>", ",", "7590.76`", ",", "7597.37`", 
     ",", "7582.07`", ",", "7583.67`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T12:50:00\"\>", ",", "7583.43`", ",", "7594.02`", 
     ",", "7577.45`", ",", "7593.62`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:00:00\"\>", ",", "7595.14`", ",", "7595.93`", 
     ",", "7569.09`", ",", "7571.96`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:10:00\"\>", ",", "7572.2`", ",", "7582.95`", 
     ",", "7562.4`", ",", "7569.09`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:20:00\"\>", ",", "7569.17`", ",", "7571.88`", 
     ",", "7556.75`", ",", "7571.24`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:30:00\"\>", ",", "7571.08`", ",", "7588.37`", 
     ",", "7560.41`", ",", "7579.05`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:40:00\"\>", ",", "7580.32`", ",", "7618.55`", 
     ",", "7577.45`", ",", "7607.88`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T13:50:00\"\>", ",", "7607.96`", ",", "7610.91`", 
     ",", "7580.56`", ",", "7587.73`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:00:00\"\>", ",", "7587.81`", ",", "7614.81`", 
     ",", "7577.06`", ",", "7606.37`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:10:00\"\>", ",", "7606.13`", ",", "7621.02`", 
     ",", "7604.85`", ",", "7607.96`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:20:00\"\>", ",", "7607.72`", ",", "7618.08`", 
     ",", "7588.05`", ",", "7591.23`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:30:00\"\>", ",", "7591.63`", ",", "7639.18`", 
     ",", "7589.08`", ",", "7632.81`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:40:00\"\>", ",", "7632.57`", ",", "7633.45`", 
     ",", "7606.29`", ",", "7606.29`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T14:50:00\"\>", ",", "7607.8`", ",", "7619.19`", 
     ",", "7587.17`", ",", "7601.35`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:00:00\"\>", ",", "7598.88`", ",", "7619.03`", 
     ",", "7578.73`", ",", "7610.83`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:10:00\"\>", ",", "7610.91`", ",", "7622.62`", 
     ",", "7590.2`", ",", "7590.44`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:20:00\"\>", ",", "7588.53`", ",", "7605.17`", 
     ",", "7573.95`", ",", "7586.38`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:30:00\"\>", ",", "7586.22`", ",", "7673.83`", 
     ",", "7586.06`", ",", "7657.5`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:40:00\"\>", ",", "7656.86`", ",", "7658.14`", 
     ",", "7590.04`", ",", "7592.43`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T15:50:00\"\>", ",", "7592.19`", ",", "7598.88`", 
     ",", "7551.33`", ",", "7555.23`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-17T16:00:00\"\>", ",", "7553.8`", ",", "7554.52`", 
     ",", "7552.29`", ",", "7552.6`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T09:30:00\"\>", ",", "7546.35`", ",", "7577.93`", 
     ",", "7546.35`", ",", "7558.34`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T09:40:00\"\>", ",", "7557.78`", ",", "7591.07`", 
     ",", "7551.97`", ",", "7569.89`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T09:50:00\"\>", ",", "7569.41`", ",", "7577.38`", 
     ",", "7533.65`", ",", "7534.68`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:00:00\"\>", ",", "7536.83`", ",", "7546.79`", 
     ",", "7499.24`", ",", "7501.31`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:10:00\"\>", ",", "7500.12`", ",", "7504.26`", 
     ",", "7480.05`", ",", "7490.8`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:20:00\"\>", ",", "7491.36`", ",", "7528.07`", 
     ",", "7491.36`", ",", "7520.03`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:30:00\"\>", ",", "7520.35`", ",", "7552.45`", 
     ",", "7520.35`", ",", "7530.54`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:40:00\"\>", ",", "7530.3`", ",", "7561.45`", 
     ",", "7530.3`", ",", "7539.7`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T10:50:00\"\>", ",", "7539.54`", ",", "7544.`", 
     ",", "7517.88`", ",", "7529.19`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:00:00\"\>", ",", "7529.35`", ",", "7530.86`", 
     ",", "7512.06`", ",", "7528.95`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:10:00\"\>", ",", "7529.03`", ",", "7564.15`", 
     ",", "7528.07`", ",", "7556.27`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:20:00\"\>", ",", "7556.67`", ",", "7593.46`", 
     ",", "7556.59`", ",", "7582.87`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:30:00\"\>", ",", "7581.2`", ",", "7586.85`", 
     ",", "7569.33`", ",", "7586.06`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:40:00\"\>", ",", "7586.14`", ",", "7601.75`", 
     ",", "7576.82`", ",", "7578.57`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T11:50:00\"\>", ",", "7578.25`", ",", "7592.35`", 
     ",", "7570.13`", ",", "7580.96`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:00:00\"\>", ",", "7579.76`", ",", "7609.63`", 
     ",", "7572.76`", ",", "7608.52`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:10:00\"\>", ",", "7608.76`", ",", "7617.76`", 
     ",", "7594.58`", ",", "7595.22`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:20:00\"\>", ",", "7595.38`", ",", "7606.61`", 
     ",", "7576.02`", ",", "7582.47`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:30:00\"\>", ",", "7583.03`", ",", "7592.59`", 
     ",", "7552.92`", ",", "7553.16`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:40:00\"\>", ",", "7553.96`", ",", "7573.63`", 
     ",", "7553.16`", ",", "7553.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T12:50:00\"\>", ",", "7553.32`", ",", "7579.05`", 
     ",", "7545.99`", ",", "7576.02`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:00:00\"\>", ",", "7575.22`", ",", "7595.38`", 
     ",", "7554.12`", ",", "7560.97`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:10:00\"\>", ",", "7561.29`", ",", "7570.76`", 
     ",", "7538.99`", ",", "7538.99`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:20:00\"\>", ",", "7539.14`", ",", "7553.16`", 
     ",", "7533.49`", ",", "7550.85`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:30:00\"\>", ",", "7550.93`", ",", "7552.21`", 
     ",", "7523.53`", ",", "7523.93`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:40:00\"\>", ",", "7523.85`", ",", "7534.68`", 
     ",", "7517.48`", ",", "7526.8`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T13:50:00\"\>", ",", "7525.05`", ",", "7539.3`", 
     ",", "7517.4`", ",", "7530.86`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:00:00\"\>", ",", "7531.34`", ",", "7558.34`", 
     ",", "7516.44`", ",", "7557.86`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:10:00\"\>", ",", "7558.42`", ",", "7558.5`", 
     ",", "7506.41`", ",", "7523.21`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:20:00\"\>", ",", "7523.53`", ",", "7547.75`", 
     ",", "7523.37`", ",", "7544.4`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:30:00\"\>", ",", "7544.72`", ",", "7559.14`", 
     ",", "7523.45`", ",", "7528.07`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:40:00\"\>", ",", "7528.31`", ",", "7535.48`", 
     ",", "7514.85`", ",", "7523.14`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T14:50:00\"\>", ",", "7523.21`", ",", "7569.65`", 
     ",", "7510.15`", ",", "7569.65`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:00:00\"\>", ",", "7570.29`", ",", "7580.72`", 
     ",", "7540.66`", ",", "7541.37`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:10:00\"\>", ",", "7540.5`", ",", "7577.3`", 
     ",", "7540.5`", ",", "7563.36`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:20:00\"\>", ",", "7563.52`", ",", "7573.55`", 
     ",", "7530.62`", ",", "7537.15`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:30:00\"\>", ",", "7537.47`", ",", "7552.05`", 
     ",", "7531.42`", ",", "7542.57`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:40:00\"\>", ",", "7542.49`", ",", "7567.66`", 
     ",", "7532.22`", ",", "7552.53`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T15:50:00\"\>", ",", "7552.6`", ",", "7570.37`", 
     ",", "7549.9`", ",", "7551.25`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-18T16:00:00\"\>", ",", "7551.33`", ",", "7555.63`", 
     ",", "7551.33`", ",", "7555.63`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T09:30:00\"\>", ",", "7555.23`", ",", "7609.71`", 
     ",", "7551.81`", ",", "7592.03`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T09:40:00\"\>", ",", "7591.15`", ",", "7614.57`", 
     ",", "7588.13`", ",", "7594.26`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T09:50:00\"\>", ",", "7594.02`", ",", "7609.79`", 
     ",", "7587.25`", ",", "7604.38`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:00:00\"\>", ",", "7595.54`", ",", "7607.`", 
     ",", "7586.93`", ",", "7599.76`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:10:00\"\>", ",", "7598.4`", ",", "7598.4`", 
     ",", "7561.92`", ",", "7562.72`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:20:00\"\>", ",", "7562.56`", ",", "7571.72`", 
     ",", "7546.47`", ",", "7560.57`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:30:00\"\>", ",", "7559.85`", ",", "7560.81`", 
     ",", "7536.68`", ",", "7546.71`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:40:00\"\>", ",", "7546.39`", ",", "7560.09`", 
     ",", "7542.65`", ",", "7554.2`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T10:50:00\"\>", ",", "7554.44`", ",", "7556.91`", 
     ",", "7535.16`", ",", "7536.2`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:00:00\"\>", ",", "7538.43`", ",", "7578.89`", 
     ",", "7531.42`", ",", "7577.38`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:10:00\"\>", ",", "7578.33`", ",", "7581.68`", 
     ",", "7573.55`", ",", "7575.15`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:20:00\"\>", ",", "7574.83`", ",", "7575.86`", 
     ",", "7551.65`", ",", "7558.42`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:30:00\"\>", ",", "7558.98`", ",", "7559.14`", 
     ",", "7539.78`", ",", "7544.72`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:40:00\"\>", ",", "7544.8`", ",", "7549.02`", 
     ",", "7516.05`", ",", "7516.21`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T11:50:00\"\>", ",", "7516.37`", ",", "7529.59`", 
     ",", "7494.7`", ",", "7500.36`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:00:00\"\>", ",", "7499.96`", ",", "7502.51`", 
     ",", "7480.13`", ",", "7499.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:10:00\"\>", ",", "7500.44`", ",", "7502.59`", 
     ",", "7479.73`", ",", "7499.72`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:20:00\"\>", ",", "7499.64`", ",", "7510.15`", 
     ",", "7492.47`", ",", "7500.12`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:30:00\"\>", ",", "7499.8`", ",", "7512.46`", 
     ",", "7493.59`", ",", "7511.98`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:40:00\"\>", ",", "7513.34`", ",", "7515.65`", 
     ",", "7504.42`", ",", "7508.96`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T12:50:00\"\>", ",", "7508.72`", ",", "7520.59`", 
     ",", "7507.44`", ",", "7508.88`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:00:00\"\>", ",", "7509.75`", ",", "7526.48`", 
     ",", "7496.13`", ",", "7526.48`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:10:00\"\>", ",", "7526.96`", ",", "7544.08`", 
     ",", "7521.94`", ",", "7540.9`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:20:00\"\>", ",", "7540.74`", ",", "7553.4`", 
     ",", "7534.29`", ",", "7550.22`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:30:00\"\>", ",", "7549.98`", ",", "7549.98`", 
     ",", "7528.55`", ",", "7536.99`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:40:00\"\>", ",", "7536.75`", ",", "7536.75`", 
     ",", "7519.87`", ",", "7525.84`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T13:50:00\"\>", ",", "7525.76`", ",", "7529.51`", 
     ",", "7504.9`", ",", "7513.26`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:00:00\"\>", ",", "7513.34`", ",", "7530.38`", 
     ",", "7510.87`", ",", "7526.08`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:10:00\"\>", ",", "7528.39`", ",", "7530.22`", 
     ",", "7488.33`", ",", "7489.05`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:20:00\"\>", ",", "7488.89`", ",", "7511.59`", 
     ",", "7488.25`", ",", "7505.53`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:30:00\"\>", ",", "7505.93`", ",", "7515.49`", 
     ",", "7493.9`", ",", "7515.49`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:40:00\"\>", ",", "7514.61`", ",", "7514.61`", 
     ",", "7483.55`", ",", "7483.55`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T14:50:00\"\>", ",", "7484.19`", ",", "7517.24`", 
     ",", "7480.6`", ",", "7507.76`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:00:00\"\>", ",", "7507.13`", ",", "7521.46`", 
     ",", "7493.27`", ",", "7495.82`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:10:00\"\>", ",", "7494.86`", ",", "7494.86`", 
     ",", "7463.16`", ",", "7474.63`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:20:00\"\>", ",", "7475.43`", ",", "7504.58`", 
     ",", "7469.85`", ",", "7499.56`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:30:00\"\>", ",", "7499.32`", ",", "7500.44`", 
     ",", "7452.41`", ",", "7452.89`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:40:00\"\>", ",", "7454.08`", ",", "7517.4`", 
     ",", "7447.71`", ",", "7502.35`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T15:50:00\"\>", ",", "7501.39`", ",", "7509.36`", 
     ",", "7462.36`", ",", "7466.66`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-19T16:00:00\"\>", ",", "7466.35`", ",", "7466.82`", 
     ",", "7465.55`", ",", "7465.95`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T09:30:00\"\>", ",", "7461.49`", ",", "7461.49`", 
     ",", "7327.84`", ",", "7340.58`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T09:40:00\"\>", ",", "7340.9`", ",", "7367.42`", 
     ",", "7333.18`", ",", "7360.41`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T09:50:00\"\>", ",", "7360.65`", ",", "7385.66`", 
     ",", "7355.24`", ",", "7376.34`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:00:00\"\>", ",", "7377.54`", ",", "7422.7`", 
     ",", "7365.19`", ",", "7407.01`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:10:00\"\>", ",", "7407.57`", ",", "7415.85`", 
     ",", "7378.81`", ",", "7381.36`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:20:00\"\>", ",", "7380.96`", ",", "7400.24`", 
     ",", "7367.5`", ",", "7397.53`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:30:00\"\>", ",", "7397.29`", ",", "7400.8`", 
     ",", "7379.05`", ",", "7398.65`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:40:00\"\>", ",", "7398.41`", ",", "7414.81`", 
     ",", "7383.75`", ",", "7393.07`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T10:50:00\"\>", ",", "7392.99`", ",", "7392.99`", 
     ",", "7354.04`", ",", "7359.7`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:00:00\"\>", ",", "7358.74`", ",", "7369.34`", 
     ",", "7338.19`", ",", "7341.3`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:10:00\"\>", ",", "7341.38`", ",", "7342.33`", 
     ",", "7312.31`", ",", "7328.56`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:20:00\"\>", ",", "7328.71`", ",", "7338.11`", 
     ",", "7320.67`", ",", "7326.56`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:30:00\"\>", ",", "7326.96`", ",", "7345.2`", 
     ",", "7311.91`", ",", "7345.2`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:40:00\"\>", ",", "7344.41`", ",", "7346.95`", 
     ",", "7324.81`", ",", "7337.87`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T11:50:00\"\>", ",", "7338.35`", ",", "7339.39`", 
     ",", "7323.86`", ",", "7327.92`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:00:00\"\>", ",", "7327.44`", ",", "7353.41`", 
     ",", "7326.72`", ",", "7353.41`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:10:00\"\>", ",", "7353.64`", ",", "7370.45`", 
     ",", "7346.64`", ",", "7357.95`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:20:00\"\>", ",", "7355.32`", ",", "7358.5`", 
     ",", "7340.82`", ",", "7347.91`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:30:00\"\>", ",", "7347.51`", ",", "7348.79`", 
     ",", "7334.53`", ",", "7343.45`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:40:00\"\>", ",", "7345.04`", ",", "7346.72`", 
     ",", "7295.82`", ",", "7295.9`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T12:50:00\"\>", ",", "7296.14`", ",", "7303.23`", 
     ",", "7281.8`", ",", "7294.63`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:00:00\"\>", ",", "7294.55`", ",", "7295.74`", 
     ",", "7275.51`", ",", "7278.38`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:10:00\"\>", ",", "7278.22`", ",", "7282.2`", 
     ",", "7249.54`", ",", "7258.94`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:20:00\"\>", ",", "7259.18`", ",", "7287.3`", 
     ",", "7258.7`", ",", "7276.55`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:30:00\"\>", ",", "7276.31`", ",", "7289.45`", 
     ",", "7269.62`", ",", "7284.35`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:40:00\"\>", ",", "7284.27`", ",", "7303.31`", 
     ",", "7281.48`", ",", "7295.5`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T13:50:00\"\>", ",", "7295.18`", ",", "7320.59`", 
     ",", "7293.99`", ",", "7311.19`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:00:00\"\>", ",", "7310.16`", ",", "7315.17`", 
     ",", "7286.82`", ",", "7309.04`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:10:00\"\>", ",", "7309.2`", ",", "7386.14`", 
     ",", "7297.89`", ",", "7359.38`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:20:00\"\>", ",", "7357.23`", ",", "7437.51`", 
     ",", "7348.47`", ",", "7375.23`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:30:00\"\>", ",", "7375.39`", ",", "7423.34`", 
     ",", "7366.31`", ",", "7412.03`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:40:00\"\>", ",", "7413.62`", ",", "7442.37`", 
     ",", "7396.5`", ",", "7441.81`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T14:50:00\"\>", ",", "7441.58`", ",", "7468.97`", 
     ",", "7407.09`", ",", "7415.53`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:00:00\"\>", ",", "7415.05`", ",", "7415.29`", 
     ",", "7373.8`", ",", "7397.05`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:10:00\"\>", ",", "7396.26`", ",", "7408.68`", 
     ",", "7368.86`", ",", "7396.18`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:20:00\"\>", ",", "7396.02`", ",", "7405.73`", 
     ",", "7344.41`", ",", "7350.22`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:30:00\"\>", ",", "7348.39`", ",", "7379.53`", 
     ",", "7328.64`", ",", "7364.24`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:40:00\"\>", ",", "7362.09`", ",", "7370.69`", 
     ",", "7330.15`", ",", "7353.33`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T15:50:00\"\>", ",", "7353.72`", ",", "7389.73`", 
     ",", "7345.92`", ",", "7369.1`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-20T16:00:00\"\>", ",", "7369.49`", ",", "7369.49`", 
     ",", "7365.67`", ",", "7365.67`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T09:30:00\"\>", ",", "7365.99`", ",", "7440.54`", 
     ",", "7365.99`", ",", "7437.83`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T09:40:00\"\>", ",", "7437.51`", ",", "7437.51`", 
     ",", "7389.88`", ",", "7395.38`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T09:50:00\"\>", ",", "7396.57`", ",", "7413.94`", 
     ",", "7381.84`", ",", "7387.81`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:00:00\"\>", ",", "7387.26`", ",", "7396.26`", 
     ",", "7365.35`", ",", "7376.5`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:10:00\"\>", ",", "7375.55`", ",", "7376.74`", 
     ",", "7345.12`", ",", "7349.26`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:20:00\"\>", ",", "7351.97`", ",", "7364.32`", 
     ",", "7346.87`", ",", "7353.8`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:30:00\"\>", ",", "7353.88`", ",", "7353.88`", 
     ",", "7314.14`", ",", "7317.25`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:40:00\"\>", ",", "7317.09`", ",", "7337.16`", 
     ",", "7306.25`", ",", "7332.3`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T10:50:00\"\>", ",", "7332.46`", ",", "7337.16`", 
     ",", "7289.29`", ",", "7289.29`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:00:00\"\>", ",", "7291.04`", ",", "7308.01`", 
     ",", "7289.85`", ",", "7300.6`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:10:00\"\>", ",", "7301.32`", ",", "7318.76`", 
     ",", "7291.04`", ",", "7291.04`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:20:00\"\>", ",", "7291.2`", ",", "7314.22`", 
     ",", "7291.2`", ",", "7307.85`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:30:00\"\>", ",", "7305.22`", ",", "7305.22`", 
     ",", "7275.83`", ",", "7291.84`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:40:00\"\>", ",", "7291.68`", ",", "7291.76`", 
     ",", "7269.7`", ",", "7279.73`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T11:50:00\"\>", ",", "7280.69`", ",", "7289.29`", 
     ",", "7265.63`", ",", "7275.75`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:00:00\"\>", ",", "7275.83`", ",", "7275.83`", 
     ",", "7257.35`", ",", "7257.35`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:10:00\"\>", ",", "7257.59`", ",", "7276.86`", 
     ",", "7256.47`", ",", "7262.61`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:20:00\"\>", ",", "7262.05`", ",", "7272.88`", 
     ",", "7256.55`", ",", "7258.55`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:30:00\"\>", ",", "7258.15`", ",", "7258.23`", 
     ",", "7237.2`", ",", "7245.08`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:40:00\"\>", ",", "7244.85`", ",", "7245.88`", 
     ",", "7219.36`", ",", "7220.15`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T12:50:00\"\>", ",", "7219.92`", ",", "7226.92`", 
     ",", "7191.56`", ",", "7191.56`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:00:00\"\>", ",", "7191.`", ",", "7222.86`", 
     ",", "7188.77`", ",", "7221.67`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:10:00\"\>", ",", "7222.78`", ",", "7234.33`", 
     ",", "7221.27`", ",", "7223.74`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:20:00\"\>", ",", "7223.66`", ",", "7233.38`", 
     ",", "7218.08`", ",", "7221.43`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:30:00\"\>", ",", "7221.03`", ",", "7229.71`", 
     ",", "7204.54`", ",", "7221.03`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:40:00\"\>", ",", "7221.27`", ",", "7237.6`", 
     ",", "7219.68`", ",", "7222.23`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T13:50:00\"\>", ",", "7222.31`", ",", "7226.61`", 
     ",", "7205.26`", ",", "7213.23`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:00:00\"\>", ",", "7212.99`", ",", "7221.27`", 
     ",", "7196.42`", ",", "7204.3`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:10:00\"\>", ",", "7203.59`", ",", "7248.43`", 
     ",", "7199.92`", ",", "7237.12`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:20:00\"\>", ",", "7237.52`", ",", "7258.31`", 
     ",", "7236.08`", ",", "7256.95`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:30:00\"\>", ",", "7255.68`", ",", "7260.38`", 
     ",", "7224.69`", ",", "7237.04`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:40:00\"\>", ",", "7237.12`", ",", "7240.62`", 
     ",", "7192.2`", ",", "7196.58`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T14:50:00\"\>", ",", "7196.82`", ",", "7201.04`", 
     ",", "7152.14`", ",", "7154.21`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:00:00\"\>", ",", "7150.3`", ",", "7188.85`", 
     ",", "7142.82`", ",", "7174.04`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:10:00\"\>", ",", "7169.5`", ",", "7189.41`", 
     ",", "7159.3`", ",", "7169.34`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:20:00\"\>", ",", "7170.3`", ",", "7173.08`", 
     ",", "7119.`", ",", "7142.98`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:30:00\"\>", ",", "7140.19`", ",", "7174.28`", 
     ",", "7136.6`", ",", "7141.22`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:40:00\"\>", ",", "7142.02`", ",", "7147.6`", 
     ",", "7117.65`", ",", "7121.07`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T15:50:00\"\>", ",", "7120.6`", ",", "7139.79`", 
     ",", "7106.26`", ",", "7114.22`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-23T16:00:00\"\>", ",", "7114.46`", ",", "7115.5`", 
     ",", "7113.98`", ",", "7114.78`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T09:30:00\"\>", ",", "7115.34`", ",", "7199.69`", 
     ",", "7115.34`", ",", "7194.03`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T09:40:00\"\>", ",", "7193.07`", ",", "7193.07`", 
     ",", "7169.1`", ",", "7181.53`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T09:50:00\"\>", ",", "7182.32`", ",", "7197.3`", 
     ",", "7164.4`", ",", "7164.72`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:00:00\"\>", ",", "7156.76`", ",", "7157.23`", 
     ",", "7121.23`", ",", "7144.09`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:10:00\"\>", ",", "7144.01`", ",", "7152.37`", 
     ",", "7114.78`", ",", "7128.08`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:20:00\"\>", ",", "7128.24`", ",", "7173.96`", 
     ",", "7118.13`", ",", "7170.14`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:30:00\"\>", ",", "7168.94`", ",", "7209.16`", 
     ",", "7166.15`", ",", "7193.47`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:40:00\"\>", ",", "7192.99`", ",", "7240.78`", 
     ",", "7185.43`", ",", "7237.2`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T10:50:00\"\>", ",", "7238.55`", ",", "7238.71`", 
     ",", "7196.66`", ",", "7205.74`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:00:00\"\>", ",", "7205.66`", ",", "7219.68`", 
     ",", "7196.1`", ",", "7196.1`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:10:00\"\>", ",", "7195.62`", ",", "7216.73`", 
     ",", "7190.37`", ",", "7200.4`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:20:00\"\>", ",", "7200.56`", ",", "7211.95`", 
     ",", "7186.54`", ",", "7197.46`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:30:00\"\>", ",", "7198.01`", ",", "7201.44`", 
     ",", "7176.03`", ",", "7186.7`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:40:00\"\>", ",", "7187.26`", ",", "7193.55`", 
     ",", "7156.12`", ",", "7156.12`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T11:50:00\"\>", ",", "7155.24`", ",", "7176.43`", 
     ",", "7154.53`", ",", "7164.88`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:00:00\"\>", ",", "7164.64`", ",", "7188.85`", 
     ",", "7162.97`", ",", "7169.02`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:10:00\"\>", ",", "7168.7`", ",", "7168.7`", 
     ",", "7147.83`", ",", "7163.76`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:20:00\"\>", ",", "7163.6`", ",", "7171.57`", 
     ",", "7159.38`", ",", "7169.58`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:30:00\"\>", ",", "7168.94`", ",", "7201.6`", 
     ",", "7168.94`", ",", "7190.05`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:40:00\"\>", ",", "7189.97`", ",", "7206.46`", 
     ",", "7187.34`", ",", "7203.51`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T12:50:00\"\>", ",", "7203.83`", ",", "7213.23`", 
     ",", "7199.29`", ",", "7204.7`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:00:00\"\>", ",", "7204.38`", ",", "7222.07`", 
     ",", "7195.15`", ",", "7213.38`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:10:00\"\>", ",", "7212.99`", ",", "7214.42`", 
     ",", "7180.09`", ",", "7185.91`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:20:00\"\>", ",", "7186.07`", ",", "7258.47`", 
     ",", "7186.07`", ",", "7258.23`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:30:00\"\>", ",", "7259.1`", ",", "7277.74`", 
     ",", "7255.52`", ",", "7271.93`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:40:00\"\>", ",", "7273.04`", ",", "7280.21`", 
     ",", "7259.42`", ",", "7264.36`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T13:50:00\"\>", ",", "7264.44`", ",", "7293.19`", 
     ",", "7264.12`", ",", "7287.3`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:00:00\"\>", ",", "7286.9`", ",", "7293.59`", 
     ",", "7276.78`", ",", "7278.7`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:10:00\"\>", ",", "7277.5`", ",", "7311.27`", 
     ",", "7264.84`", ",", "7310.63`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:20:00\"\>", ",", "7310.95`", ",", "7314.86`", 
     ",", "7299.01`", ",", "7304.98`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:30:00\"\>", ",", "7304.9`", ",", "7309.92`", 
     ",", "7288.89`", ",", "7298.61`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:40:00\"\>", ",", "7299.01`", ",", "7305.46`", 
     ",", "7279.33`", ",", "7295.26`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T14:50:00\"\>", ",", "7294.23`", ",", "7317.25`", 
     ",", "7283.63`", ",", "7309.6`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:00:00\"\>", ",", "7310.`", ",", "7342.1`", ",",
      "7310.`", ",", "7341.86`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:10:00\"\>", ",", "7341.7`", ",", "7367.34`", 
     ",", "7337.56`", ",", "7367.34`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:20:00\"\>", ",", "7367.5`", ",", "7378.97`", 
     ",", "7354.36`", ",", "7356.51`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:30:00\"\>", ",", "7356.59`", ",", "7364.56`", 
     ",", "7333.25`", ",", "7351.57`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:40:00\"\>", ",", "7351.33`", ",", "7367.74`", 
     ",", "7337.95`", ",", "7357.39`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T15:50:00\"\>", ",", "7355.48`", ",", "7364.72`", 
     ",", "7345.36`", ",", "7347.27`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-24T16:00:00\"\>", ",", "7346.`", ",", "7350.94`", 
     ",", "7345.84`", ",", "7350.94`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T09:30:00\"\>", ",", "7349.58`", ",", "7350.14`", 
     ",", "7293.99`", ",", "7302.19`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T09:40:00\"\>", ",", "7301.16`", ",", "7308.8`", 
     ",", "7255.12`", ",", "7271.77`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T09:50:00\"\>", ",", "7271.93`", ",", "7274.47`", 
     ",", "7254.01`", ",", "7258.47`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:00:00\"\>", ",", "7251.3`", ",", "7251.3`", 
     ",", "7188.93`", ",", "7200.4`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:10:00\"\>", ",", "7200.56`", ",", "7211.`", 
     ",", "7176.11`", ",", "7197.69`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:20:00\"\>", ",", "7197.93`", ",", "7214.02`", 
     ",", "7191.08`", ",", "7213.7`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:30:00\"\>", ",", "7214.02`", ",", "7216.97`", 
     ",", "7188.14`", ",", "7191.4`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:40:00\"\>", ",", "7191.64`", ",", "7197.85`", 
     ",", "7156.99`", ",", "7166.15`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T10:50:00\"\>", ",", "7167.19`", ",", "7180.49`", 
     ",", "7165.91`", ",", "7174.28`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:00:00\"\>", ",", "7174.04`", ",", "7208.29`", 
     ",", "7173.56`", ",", "7202.39`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:10:00\"\>", ",", "7202.07`", ",", "7202.07`", 
     ",", "7190.05`", ",", "7197.61`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:20:00\"\>", ",", "7197.46`", ",", "7227.96`", 
     ",", "7196.58`", ",", "7223.58`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:30:00\"\>", ",", "7223.74`", ",", "7233.3`", 
     ",", "7216.81`", ",", "7223.02`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:40:00\"\>", ",", "7222.62`", ",", "7273.84`", 
     ",", "7222.38`", ",", "7257.35`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T11:50:00\"\>", ",", "7257.11`", ",", "7260.38`", 
     ",", "7210.04`", ",", "7214.34`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:00:00\"\>", ",", "7214.26`", ",", "7228.76`", 
     ",", "7212.83`", ",", "7220.31`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:10:00\"\>", ",", "7220.87`", ",", "7222.54`", 
     ",", "7201.52`", ",", "7202.47`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:20:00\"\>", ",", "7202.55`", ",", "7208.92`", 
     ",", "7189.17`", ",", "7194.11`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:30:00\"\>", ",", "7193.79`", ",", "7207.65`", 
     ",", "7193.15`", ",", "7207.09`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:40:00\"\>", ",", "7207.33`", ",", "7209.08`", 
     ",", "7174.2`", ",", "7187.42`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T12:50:00\"\>", ",", "7187.34`", ",", "7210.36`", 
     ",", "7183.44`", ",", "7204.62`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:00:00\"\>", ",", "7204.54`", ",", "7235.53`", 
     ",", "7198.73`", ",", "7226.37`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:10:00\"\>", ",", "7227.`", ",", "7228.84`", 
     ",", "7208.92`", ",", "7223.26`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:20:00\"\>", ",", "7223.18`", ",", "7223.66`", 
     ",", "7199.53`", ",", "7211.79`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:30:00\"\>", ",", "7211.55`", ",", "7255.36`", 
     ",", "7211.15`", ",", "7251.22`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:40:00\"\>", ",", "7251.54`", ",", "7295.58`", 
     ",", "7251.54`", ",", "7295.1`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T13:50:00\"\>", ",", "7295.26`", ",", "7331.98`", 
     ",", "7283.55`", ",", "7321.31`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:00:00\"\>", ",", "7320.99`", ",", "7328.79`", 
     ",", "7242.77`", ",", "7252.01`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:10:00\"\>", ",", "7254.32`", ",", "7272.48`", 
     ",", "7226.13`", ",", "7230.59`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:20:00\"\>", ",", "7230.11`", ",", "7278.06`", 
     ",", "7218.72`", ",", "7274.55`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:30:00\"\>", ",", "7280.13`", ",", "7285.86`", 
     ",", "7255.68`", ",", "7265.63`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:40:00\"\>", ",", "7265.71`", ",", "7319.63`", 
     ",", "7264.12`", ",", "7298.45`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T14:50:00\"\>", ",", "7297.81`", ",", "7330.71`", 
     ",", "7296.62`", ",", "7306.73`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:00:00\"\>", ",", "7306.57`", ",", "7362.33`", 
     ",", "7299.32`", ",", "7348.71`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:10:00\"\>", ",", "7349.34`", ",", "7396.65`", 
     ",", "7317.48`", ",", "7386.54`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:20:00\"\>", ",", "7384.39`", ",", "7404.46`", 
     ",", "7374.19`", ",", "7384.95`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:30:00\"\>", ",", "7385.11`", ",", "7395.06`", 
     ",", "7359.06`", ",", "7359.14`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:40:00\"\>", ",", "7360.1`", ",", "7367.5`", 
     ",", "7320.51`", ",", "7325.05`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T15:50:00\"\>", ",", "7321.94`", ",", "7330.07`", 
     ",", "7262.61`", ",", "7270.97`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-25T16:00:00\"\>", ",", "7270.89`", ",", "7271.21`", 
     ",", "7270.09`", ",", "7270.89`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T09:30:00\"\>", ",", "7269.06`", ",", "7356.43`", 
     ",", "7269.06`", ",", "7322.82`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T09:40:00\"\>", ",", "7321.71`", ",", "7337.16`", 
     ",", "7302.59`", ",", "7308.8`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T09:50:00\"\>", ",", "7307.93`", ",", "7343.53`", 
     ",", "7307.05`", ",", "7343.53`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:00:00\"\>", ",", "7344.01`", ",", "7378.1`", 
     ",", "7334.61`", ",", "7353.49`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:10:00\"\>", ",", "7353.72`", ",", "7354.28`", 
     ",", "7320.51`", ",", "7345.12`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:20:00\"\>", ",", "7345.68`", ",", "7378.34`", 
     ",", "7344.88`", ",", "7372.92`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:30:00\"\>", ",", "7373.95`", ",", "7391.88`", 
     ",", "7348.63`", ",", "7384.87`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:40:00\"\>", ",", "7384.95`", ",", "7402.23`", 
     ",", "7383.27`", ",", "7388.85`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T10:50:00\"\>", ",", "7388.69`", ",", "7393.47`", 
     ",", "7372.6`", ",", "7390.92`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:00:00\"\>", ",", "7391.48`", ",", "7394.5`", 
     ",", "7347.67`", ",", "7350.46`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:10:00\"\>", ",", "7350.54`", ",", "7362.72`", 
     ",", "7344.33`", ",", "7360.89`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:20:00\"\>", ",", "7360.81`", ",", "7369.81`", 
     ",", "7343.69`", ",", "7344.01`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:30:00\"\>", ",", "7344.17`", ",", "7353.17`", 
     ",", "7337.72`", ",", "7346.16`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:40:00\"\>", ",", "7346.`", ",", "7348.15`", 
     ",", "7310.32`", ",", "7318.92`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T11:50:00\"\>", ",", "7318.68`", ",", "7334.45`", 
     ",", "7318.68`", ",", "7332.62`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:00:00\"\>", ",", "7332.54`", ",", "7343.13`", 
     ",", "7330.15`", ",", "7341.7`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:10:00\"\>", ",", "7341.22`", ",", "7341.62`", 
     ",", "7314.62`", ",", "7319.71`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:20:00\"\>", ",", "7319.63`", ",", "7319.79`", 
     ",", "7282.44`", ",", "7290.56`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:30:00\"\>", ",", "7291.52`", ",", "7310.32`", 
     ",", "7284.83`", ",", "7299.96`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:40:00\"\>", ",", "7300.04`", ",", "7315.81`", 
     ",", "7289.37`", ",", "7313.58`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T12:50:00\"\>", ",", "7313.74`", ",", "7324.81`", 
     ",", "7292.24`", ",", "7297.65`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:00:00\"\>", ",", "7299.25`", ",", "7304.02`", 
     ",", "7279.41`", ",", "7279.41`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:10:00\"\>", ",", "7278.46`", ",", "7278.46`", 
     ",", "7252.89`", ",", "7261.41`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:20:00\"\>", ",", "7261.09`", ",", "7293.91`", 
     ",", "7261.01`", ",", "7286.58`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:30:00\"\>", ",", "7286.66`", ",", "7298.85`", 
     ",", "7276.15`", ",", "7277.66`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:40:00\"\>", ",", "7277.58`", ",", "7314.86`", 
     ",", "7277.1`", ",", "7306.73`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T13:50:00\"\>", ",", "7306.81`", ",", "7315.73`", 
     ",", "7292.55`", ",", "7304.18`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:00:00\"\>", ",", "7301.32`", ",", "7304.42`", 
     ",", "7267.31`", ",", "7277.26`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:10:00\"\>", ",", "7278.06`", ",", "7295.98`", 
     ",", "7277.74`", ",", "7293.19`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:20:00\"\>", ",", "7293.11`", ",", "7309.44`", 
     ",", "7293.03`", ",", "7301.`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:30:00\"\>", ",", "7301.55`", ",", "7301.55`", 
     ",", "7269.14`", ",", "7283.55`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:40:00\"\>", ",", "7282.92`", ",", "7282.92`", 
     ",", "7261.49`", ",", "7271.53`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T14:50:00\"\>", ",", "7271.21`", ",", "7273.2`", 
     ",", "7214.5`", ",", "7226.61`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:00:00\"\>", ",", "7222.7`", ",", "7225.65`", 
     ",", "7197.46`", ",", "7218.72`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:10:00\"\>", ",", "7217.21`", ",", "7220.15`", 
     ",", "7177.86`", ",", "7182.56`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:20:00\"\>", ",", "7183.44`", ",", "7206.85`", 
     ",", "7173.88`", ",", "7194.35`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:30:00\"\>", ",", "7193.79`", ",", "7226.13`", 
     ",", "7183.52`", ",", "7216.65`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:40:00\"\>", ",", "7215.22`", ",", "7238.95`", 
     ",", "7214.58`", ",", "7236.4`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T15:50:00\"\>", ",", "7237.04`", ",", "7237.04`", 
     ",", "7178.58`", ",", "7178.58`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-26T16:00:00\"\>", ",", "7178.66`", ",", "7182.16`", 
     ",", "7178.5`", ",", "7182.08`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T09:30:00\"\>", ",", "7180.97`", ",", "7180.97`", 
     ",", "7039.59`", ",", "7041.82`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T09:40:00\"\>", ",", "7041.5`", ",", "7077.98`", 
     ",", "7034.02`", ",", "7060.3`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T09:50:00\"\>", ",", "7061.1`", ",", "7108.25`", 
     ",", "7050.58`", ",", "7090.49`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:00:00\"\>", ",", "7090.01`", ",", "7118.13`", 
     ",", "7082.84`", ",", "7082.84`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:10:00\"\>", ",", "7083.08`", ",", "7098.85`", 
     ",", "7071.37`", ",", "7098.85`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:20:00\"\>", ",", "7099.01`", ",", "7147.75`", 
     ",", "7099.01`", ",", "7121.39`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:30:00\"\>", ",", "7119.4`", ",", "7133.9`", 
     ",", "7098.85`", ",", "7113.75`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:40:00\"\>", ",", "7113.43`", ",", "7147.44`", 
     ",", "7103.07`", ",", "7133.58`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T10:50:00\"\>", ",", "7132.54`", ",", "7148.63`", 
     ",", "7111.52`", ",", "7123.78`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:00:00\"\>", ",", "7125.13`", ",", "7130.07`", 
     ",", "7099.97`", ",", "7112.47`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:10:00\"\>", ",", "7112.15`", ",", "7116.61`", 
     ",", "7086.9`", ",", "7094.95`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:20:00\"\>", ",", "7094.15`", ",", "7120.04`", 
     ",", "7090.81`", ",", "7119.24`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:30:00\"\>", ",", "7122.59`", ",", "7129.67`", 
     ",", "7109.52`", ",", "7124.42`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:40:00\"\>", ",", "7124.66`", ",", "7177.07`", 
     ",", "7124.66`", ",", "7163.37`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T11:50:00\"\>", ",", "7159.54`", ",", "7171.81`", 
     ",", "7139.23`", ",", "7169.18`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:00:00\"\>", ",", "7168.62`", ",", "7187.98`", 
     ",", "7165.52`", ",", "7174.76`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:10:00\"\>", ",", "7174.44`", ",", "7181.92`", 
     ",", "7148.71`", ",", "7151.1`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:20:00\"\>", ",", "7151.58`", ",", "7168.86`", 
     ",", "7148.79`", ",", "7160.9`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:30:00\"\>", ",", "7161.22`", ",", "7169.02`", 
     ",", "7138.12`", ",", "7138.44`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:40:00\"\>", ",", "7138.36`", ",", "7145.13`", 
     ",", "7130.23`", ",", "7141.78`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T12:50:00\"\>", ",", "7142.26`", ",", "7156.68`", 
     ",", "7135.81`", ",", "7156.28`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:00:00\"\>", ",", "7159.06`", ",", "7179.14`", 
     ",", "7158.75`", ",", "7171.49`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:10:00\"\>", ",", "7171.73`", ",", "7177.15`", 
     ",", "7134.29`", ",", "7150.54`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:20:00\"\>", ",", "7151.26`", ",", "7161.85`", 
     ",", "7144.17`", ",", "7153.25`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:30:00\"\>", ",", "7154.05`", ",", "7182.08`", 
     ",", "7102.04`", ",", "7115.5`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:40:00\"\>", ",", "7116.37`", ",", "7152.45`", 
     ",", "7108.17`", ",", "7150.46`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T13:50:00\"\>", ",", "7151.82`", ",", "7174.52`", 
     ",", "7143.06`", ",", "7171.89`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:00:00\"\>", ",", "7171.17`", ",", "7195.3`", 
     ",", "7160.74`", ",", "7169.26`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:10:00\"\>", ",", "7167.43`", ",", "7178.34`", 
     ",", "7152.37`", ",", "7163.68`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:20:00\"\>", ",", "7163.6`", ",", "7169.9`", 
     ",", "7126.09`", ",", "7129.36`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:30:00\"\>", ",", "7131.03`", ",", "7138.68`", 
     ",", "7116.77`", ",", "7118.84`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:40:00\"\>", ",", "7119.48`", ",", "7163.13`", 
     ",", "7116.29`", ",", "7162.49`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T14:50:00\"\>", ",", "7161.45`", ",", "7175.71`", 
     ",", "7147.91`", ",", "7160.5`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:00:00\"\>", ",", "7160.34`", ",", "7160.34`", 
     ",", "7118.52`", ",", "7119.08`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:10:00\"\>", ",", "7119.48`", ",", "7119.48`", 
     ",", "7072.65`", ",", "7080.37`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:20:00\"\>", ",", "7081.01`", ",", "7118.76`", 
     ",", "7072.81`", ",", "7088.5`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:30:00\"\>", ",", "7086.11`", ",", "7125.45`", 
     ",", "7082.92`", ",", "7094.95`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:40:00\"\>", ",", "7094.07`", ",", "7147.83`", 
     ",", "7090.89`", ",", "7133.66`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T15:50:00\"\>", ",", "7135.81`", ",", "7137.72`", 
     ",", "7063.57`", ",", "7066.28`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-02-27T16:00:00\"\>", ",", "7068.9`", ",", "7068.9`", 
     ",", "7062.61`", ",", "7062.93`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T09:30:00\"\>", ",", "7056.48`", ",", "7056.48`", 
     ",", "6918.45`", ",", "6937.88`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T09:40:00\"\>", ",", "6936.93`", ",", "6963.05`", 
     ",", "6936.93`", ",", "6958.11`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T09:50:00\"\>", ",", "6958.59`", ",", "6976.83`", 
     ",", "6935.97`", ",", "6937.33`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:00:00\"\>", ",", "6940.83`", ",", "6960.98`", 
     ",", "6915.74`", ",", "6919.96`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:10:00\"\>", ",", "6920.2`", ",", "6923.71`", 
     ",", "6895.83`", ",", "6907.54`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:20:00\"\>", ",", "6907.3`", ",", "6929.2`", 
     ",", "6898.62`", ",", "6912.32`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:30:00\"\>", ",", "6911.28`", ",", "6926.17`", 
     ",", "6904.35`", ",", "6909.45`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:40:00\"\>", ",", "6909.21`", ",", "6909.29`", 
     ",", "6888.9`", ",", "6897.34`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T10:50:00\"\>", ",", "6896.63`", ",", "6897.42`", 
     ",", "6879.58`", ",", "6890.17`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:00:00\"\>", ",", "6889.38`", ",", "6903.48`", 
     ",", "6872.25`", ",", "6877.83`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:10:00\"\>", ",", "6878.78`", ",", "6887.39`", 
     ",", "6872.01`", ",", "6886.67`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:20:00\"\>", ",", "6888.02`", ",", "6902.84`", 
     ",", "6861.1`", ",", "6861.1`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:30:00\"\>", ",", "6861.26`", ",", "6866.52`", 
     ",", "6842.78`", ",", "6854.57`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:40:00\"\>", ",", "6852.58`", ",", "6860.47`", 
     ",", "6844.14`", ",", "6844.93`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T11:50:00\"\>", ",", "6846.61`", ",", "6851.07`", 
     ",", "6820.72`", ",", "6834.9`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:00:00\"\>", ",", "6834.66`", ",", "6848.12`", 
     ",", "6821.68`", ",", "6840.71`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:10:00\"\>", ",", "6839.92`", ",", "6840.71`", 
     ",", "6816.02`", ",", "6818.01`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:20:00\"\>", ",", "6821.84`", ",", "6848.84`", 
     ",", "6821.52`", ",", "6839.2`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:30:00\"\>", ",", "6839.36`", ",", "6849.79`", 
     ",", "6835.22`", ",", "6837.13`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:40:00\"\>", ",", "6837.92`", ",", "6837.92`", 
     ",", "6812.92`", ",", "6824.54`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T12:50:00\"\>", ",", "6824.86`", ",", "6841.67`", 
     ",", "6807.66`", ",", "6811.72`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:00:00\"\>", ",", "6810.69`", ",", "6841.03`", 
     ",", "6810.53`", ",", "6839.84`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:10:00\"\>", ",", "6838.88`", ",", "6839.2`", 
     ",", "6813.15`", ",", "6816.98`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:20:00\"\>", ",", "6817.93`", ",", "6828.92`", 
     ",", "6807.26`", ",", "6822.08`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:30:00\"\>", ",", "6823.03`", ",", "6824.54`", 
     ",", "6804.63`", ",", "6811.24`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:40:00\"\>", ",", "6810.92`", ",", "6835.3`", 
     ",", "6806.23`", ",", "6835.3`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T13:50:00\"\>", ",", "6834.98`", ",", "6859.11`", 
     ",", "6829.48`", ",", "6859.11`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:00:00\"\>", ",", "6857.12`", ",", "6857.12`", 
     ",", "6829.48`", ",", "6835.14`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:10:00\"\>", ",", "6838.64`", ",", "6861.82`", 
     ",", "6837.53`", ",", "6841.59`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:20:00\"\>", ",", "6840.95`", ",", "6841.59`", 
     ",", "6815.78`", ",", "6825.5`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:30:00\"\>", ",", "6825.26`", ",", "6882.85`", 
     ",", "6814.35`", ",", "6858.55`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:40:00\"\>", ",", "6856.8`", ",", "6880.06`", 
     ",", "6840.95`", ",", "6840.95`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T14:50:00\"\>", ",", "6840.63`", ",", "6855.29`", 
     ",", "6816.1`", ",", "6823.19`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:00:00\"\>", ",", "6822.47`", ",", "6824.7`", 
     ",", "6792.45`", ",", "6797.46`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:10:00\"\>", ",", "6798.18`", ",", "6819.21`", 
     ",", "6779.54`", ",", "6780.82`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:20:00\"\>", ",", "6782.81`", ",", "6810.29`", 
     ",", "6778.67`", ",", "6805.03`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:30:00\"\>", ",", "6806.07`", ",", "6829.96`", 
     ",", "6778.99`", ",", "6783.84`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:40:00\"\>", ",", "6786.55`", ",", "6795.87`", 
     ",", "6755.81`", ",", "6762.34`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T15:50:00\"\>", ",", "6761.06`", ",", "6783.37`", 
     ",", "6755.33`", ",", "6763.61`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-02T16:00:00\"\>", ",", "6762.26`", ",", "6763.29`", 
     ",", "6761.78`", ",", "6763.29`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T09:30:00\"\>", ",", "6764.81`", ",", "6848.28`", 
     ",", "6764.81`", ",", "6845.89`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T09:40:00\"\>", ",", "6842.54`", ",", "6845.65`", 
     ",", "6798.1`", ",", "6810.21`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T09:50:00\"\>", ",", "6810.29`", ",", "6831.47`", 
     ",", "6805.35`", ",", "6819.05`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:00:00\"\>", ",", "6810.84`", ",", "6830.6`", 
     ",", "6789.26`", ",", "6816.02`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:10:00\"\>", ",", "6815.38`", ",", "6815.38`", 
     ",", "6776.91`", ",", "6793.16`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:20:00\"\>", ",", "6792.53`", ",", "6817.46`", 
     ",", "6787.35`", ",", "6811.4`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:30:00\"\>", ",", "6808.53`", ",", "6826.06`", 
     ",", "6794.52`", ",", "6822.08`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:40:00\"\>", ",", "6820.72`", ",", "6833.39`", 
     ",", "6810.29`", ",", "6810.29`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T10:50:00\"\>", ",", "6809.97`", ",", "6817.69`", 
     ",", "6776.99`", ",", "6784.56`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:00:00\"\>", ",", "6783.68`", ",", "6793.32`", 
     ",", "6758.04`", ",", "6775.08`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:10:00\"\>", ",", "6774.92`", ",", "6774.92`", 
     ",", "6714.15`", ",", "6724.67`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:20:00\"\>", ",", "6724.19`", ",", "6753.1`", 
     ",", "6723.23`", ",", "6747.13`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:30:00\"\>", ",", "6747.05`", ",", "6750.95`", 
     ",", "6718.53`", ",", "6718.93`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:40:00\"\>", ",", "6720.13`", ",", "6734.38`", 
     ",", "6714.71`", ",", "6726.66`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T11:50:00\"\>", ",", "6726.9`", ",", "6731.6`", 
     ",", "6705.71`", ",", "6715.9`", ",", "313", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:00:00\"\>", ",", "6715.67`", ",", "6751.83`", 
     ",", "6715.35`", ",", "6745.21`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:10:00\"\>", ",", "6746.17`", ",", "6772.53`", 
     ",", "6738.37`", ",", "6768.55`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:20:00\"\>", ",", "6768.23`", ",", "6797.94`", 
     ",", "6763.69`", ",", "6790.69`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:30:00\"\>", ",", "6790.61`", ",", "6790.61`", 
     ",", "6765.45`", ",", "6766.08`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:40:00\"\>", ",", "6766.16`", ",", "6782.57`", 
     ",", "6760.35`", ",", "6771.02`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T12:50:00\"\>", ",", "6771.66`", ",", "6771.74`", 
     ",", "6740.12`", ",", "6744.02`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:00:00\"\>", ",", "6744.1`", ",", "6748.24`", 
     ",", "6724.75`", ",", "6735.34`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:10:00\"\>", ",", "6735.58`", ",", "6761.86`", 
     ",", "6728.41`", ",", "6761.86`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:20:00\"\>", ",", "6762.9`", ",", "6773.73`", 
     ",", "6735.58`", ",", "6735.58`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:30:00\"\>", ",", "6737.25`", ",", "6742.91`", 
     ",", "6711.76`", ",", "6719.97`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:40:00\"\>", ",", "6719.81`", ",", "6739.4`", 
     ",", "6716.86`", ",", "6717.42`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T13:50:00\"\>", ",", "6717.5`", ",", "6776.04`", 
     ",", "6717.5`", ",", "6764.41`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:00:00\"\>", ",", "6762.98`", ",", "6794.36`", 
     ",", "6762.18`", ",", "6778.67`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:10:00\"\>", ",", "6778.91`", ",", "6809.01`", 
     ",", "6772.69`", ",", "6785.28`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:20:00\"\>", ",", "6784.8`", ",", "6816.9`", 
     ",", "6778.99`", ",", "6799.53`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:30:00\"\>", ",", "6799.3`", ",", "6845.17`", 
     ",", "6792.53`", ",", "6840.39`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:40:00\"\>", ",", "6840.47`", ",", "6849.23`", 
     ",", "6805.83`", ",", "6823.03`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T14:50:00\"\>", ",", "6823.19`", ",", "6855.29`", 
     ",", "6801.05`", ",", "6840.55`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:00:00\"\>", ",", "6839.36`", ",", "6839.36`", 
     ",", "6791.65`", ",", "6794.92`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:10:00\"\>", ",", "6795.07`", ",", "6809.73`", 
     ",", "6773.65`", ",", "6785.36`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:20:00\"\>", ",", "6786.07`", ",", "6795.07`", 
     ",", "6740.67`", ",", "6744.5`", ",", "323", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:30:00\"\>", ",", "6745.14`", ",", "6757.88`", 
     ",", "6719.09`", ",", "6719.09`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:40:00\"\>", ",", "6718.21`", ",", "6735.42`", 
     ",", "6714.39`", ",", "6726.26`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T15:50:00\"\>", ",", "6728.01`", ",", "6768.71`", 
     ",", "6723.79`", ",", "6724.03`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-03T16:00:00\"\>", ",", "6723.39`", ",", "6726.02`", 
     ",", "6723.31`", ",", "6726.02`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T09:30:00\"\>", ",", "6726.5`", ",", "6846.93`", 
     ",", "6726.5`", ",", "6835.46`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T09:40:00\"\>", ",", "6835.54`", ",", "6847.72`", 
     ",", "6823.27`", ",", "6830.6`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T09:50:00\"\>", ",", "6831.15`", ",", "6831.15`", 
     ",", "6798.42`", ",", "6803.04`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:00:00\"\>", ",", "6806.38`", ",", "6833.7`", 
     ",", "6803.44`", ",", "6803.44`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:10:00\"\>", ",", "6803.2`", ",", "6803.84`", 
     ",", "6768.07`", ",", "6772.45`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:20:00\"\>", ",", "6772.85`", ",", "6792.21`", 
     ",", "6755.97`", ",", "6792.21`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:30:00\"\>", ",", "6792.76`", ",", "6802.`", 
     ",", "6778.03`", ",", "6797.15`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:40:00\"\>", ",", "6796.35`", ",", "6834.9`", 
     ",", "6794.2`", ",", "6825.66`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T10:50:00\"\>", ",", "6825.58`", ",", "6843.66`", 
     ",", "6820.16`", ",", "6827.73`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:00:00\"\>", ",", "6827.89`", ",", "6874.09`", 
     ",", "6827.89`", ",", "6858.47`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:10:00\"\>", ",", "6859.11`", ",", "6873.93`", 
     ",", "6851.39`", ",", "6854.65`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:20:00\"\>", ",", "6853.7`", ",", "6883.24`", 
     ",", "6842.62`", ",", "6881.97`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:30:00\"\>", ",", "6883.09`", ",", "6890.25`", 
     ",", "6874.72`", ",", "6877.43`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:40:00\"\>", ",", "6877.03`", ",", "6879.1`", 
     ",", "6856.08`", ",", "6872.73`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T11:50:00\"\>", ",", "6872.65`", ",", "6900.21`", 
     ",", "6871.85`", ",", "6889.3`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:00:00\"\>", ",", "6890.33`", ",", "6892.64`", 
     ",", "6853.77`", ",", "6856.96`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:10:00\"\>", ",", "6857.04`", ",", "6866.52`", 
     ",", "6845.25`", ",", "6845.25`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:20:00\"\>", ",", "6844.77`", ",", "6865.24`", 
     ",", "6844.77`", ",", "6860.86`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:30:00\"\>", ",", "6860.78`", ",", "6877.43`", 
     ",", "6858.31`", ",", "6876.87`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:40:00\"\>", ",", "6876.95`", ",", "6876.95`", 
     ",", "6834.26`", ",", "6835.85`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T12:50:00\"\>", ",", "6835.38`", ",", "6845.49`", 
     ",", "6826.06`", ",", "6839.04`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:00:00\"\>", ",", "6838.8`", ",", "6840.`", ",",
      "6814.11`", ",", "6831.55`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:10:00\"\>", ",", "6832.19`", ",", "6848.76`", 
     ",", "6826.61`", ",", "6847.96`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:20:00\"\>", ",", "6847.72`", ",", "6866.12`", 
     ",", "6837.21`", ",", "6851.23`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:30:00\"\>", ",", "6852.9`", ",", "6876.87`", 
     ",", "6845.89`", ",", "6847.16`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:40:00\"\>", ",", "6846.93`", ",", "6874.32`", 
     ",", "6844.46`", ",", "6872.17`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T13:50:00\"\>", ",", "6872.09`", ",", "6874.48`", 
     ",", "6851.7`", ",", "6851.7`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:00:00\"\>", ",", "6851.86`", ",", "6895.19`", 
     ",", "6841.59`", ",", "6895.19`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:10:00\"\>", ",", "6894.71`", ",", "6916.22`", 
     ",", "6884.04`", ",", "6910.32`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:20:00\"\>", ",", "6910.48`", ",", "6910.48`", 
     ",", "6884.68`", ",", "6889.46`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:30:00\"\>", ",", "6889.7`", ",", "6920.12`", 
     ",", "6878.15`", ",", "6912.95`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:40:00\"\>", ",", "6912.79`", ",", "6943.22`", 
     ",", "6903.71`", ",", "6934.62`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T14:50:00\"\>", ",", "6934.86`", ",", "6952.7`", 
     ",", "6929.28`", ",", "6944.97`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:00:00\"\>", ",", "6944.49`", ",", "6953.89`", 
     ",", "6930.79`", ",", "6941.39`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:10:00\"\>", ",", "6941.31`", ",", "6973.96`", 
     ",", "6940.51`", ",", "6968.15`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:20:00\"\>", ",", "6967.83`", ",", "6976.99`", 
     ",", "6945.21`", ",", "6976.99`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:30:00\"\>", ",", "6976.91`", ",", "6978.74`", 
     ",", "6933.58`", ",", "6945.05`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:40:00\"\>", ",", "6945.21`", ",", "6945.77`", 
     ",", "6912.55`", ",", "6915.34`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T15:50:00\"\>", ",", "6914.94`", ",", "6919.64`", 
     ",", "6869.55`", ",", "6876.39`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-04T16:00:00\"\>", ",", "6876.32`", ",", "6876.55`", 
     ",", "6875.2`", ",", "6875.84`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T09:30:00\"\>", ",", "6874.01`", ",", "6874.01`", 
     ",", "6733.03`", ",", "6772.53`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T09:40:00\"\>", ",", "6771.82`", ",", "6791.17`", 
     ",", "6762.5`", ",", "6763.14`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T09:50:00\"\>", ",", "6764.41`", ",", "6783.53`", 
     ",", "6758.52`", ",", "6766.88`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:00:00\"\>", ",", "6778.35`", ",", "6797.46`", 
     ",", "6763.37`", ",", "6769.83`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:10:00\"\>", ",", "6768.87`", ",", "6768.87`", 
     ",", "6738.13`", ",", "6745.61`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:20:00\"\>", ",", "6740.67`", ",", "6745.61`", 
     ",", "6718.21`", ",", "6741.15`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:30:00\"\>", ",", "6738.84`", ",", "6755.73`", 
     ",", "6734.46`", ",", "6746.41`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:40:00\"\>", ",", "6744.66`", ",", "6766.08`", 
     ",", "6721.24`", ",", "6729.21`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T10:50:00\"\>", ",", "6729.36`", ",", "6738.76`", 
     ",", "6708.98`", ",", "6711.84`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:00:00\"\>", ",", "6711.21`", ",", "6725.86`", 
     ",", "6705.07`", ",", "6714.55`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:10:00\"\>", ",", "6714.23`", ",", "6717.9`", 
     ",", "6667.56`", ",", "6670.74`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:20:00\"\>", ",", "6669.15`", ",", "6678.07`", 
     ",", "6657.6`", ",", "6663.34`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:30:00\"\>", ",", "6663.26`", ",", "6677.67`", 
     ",", "6654.58`", ",", "6677.28`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:40:00\"\>", ",", "6677.83`", ",", "6685.24`", 
     ",", "6663.5`", ",", "6677.43`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T11:50:00\"\>", ",", "6677.2`", ",", "6678.23`", 
     ",", "6646.53`", ",", "6660.07`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:00:00\"\>", ",", "6659.91`", ",", "6668.83`", 
     ",", "6641.27`", ",", "6643.42`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:10:00\"\>", ",", "6642.95`", ",", "6656.97`", 
     ",", "6630.44`", ",", "6656.73`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:20:00\"\>", ",", "6656.65`", ",", "6679.82`", 
     ",", "6655.37`", ",", "6679.82`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:30:00\"\>", ",", "6680.46`", ",", "6683.17`", 
     ",", "6665.33`", ",", "6679.43`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:40:00\"\>", ",", "6679.51`", ",", "6686.43`", 
     ",", "6649.4`", ",", "6652.03`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T12:50:00\"\>", ",", "6652.19`", ",", "6656.33`", 
     ",", "6636.1`", ",", "6645.81`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:00:00\"\>", ",", "6646.45`", ",", "6677.99`", 
     ",", "6642.39`", ",", "6677.99`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:10:00\"\>", ",", "6678.23`", ",", "6701.97`", 
     ",", "6677.51`", ",", "6698.54`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:20:00\"\>", ",", "6698.3`", ",", "6698.46`", 
     ",", "6660.63`", ",", "6674.65`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:30:00\"\>", ",", "6673.21`", ",", "6673.37`", 
     ",", "6644.54`", ",", "6650.83`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:40:00\"\>", ",", "6651.07`", ",", "6668.99`", 
     ",", "6624.23`", ",", "6627.18`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T13:50:00\"\>", ",", "6628.05`", ",", "6654.1`", 
     ",", "6621.76`", ",", "6647.65`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:00:00\"\>", ",", "6646.85`", ",", "6646.85`", 
     ",", "6620.73`", ",", "6627.89`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:10:00\"\>", ",", "6628.05`", ",", "6644.22`", 
     ",", "6613.64`", ",", "6639.6`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:20:00\"\>", ",", "6639.52`", ",", "6639.52`", 
     ",", "6602.96`", ",", "6614.75`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:30:00\"\>", ",", "6614.19`", ",", "6643.42`", 
     ",", "6606.87`", ",", "6634.35`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:40:00\"\>", ",", "6634.27`", ",", "6642.07`", 
     ",", "6617.54`", ",", "6627.42`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T14:50:00\"\>", ",", "6627.5`", ",", "6630.68`", 
     ",", "6601.77`", ",", "6611.09`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:00:00\"\>", ",", "6611.17`", ",", "6645.89`", 
     ",", "6608.38`", ",", "6632.04`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:10:00\"\>", ",", "6632.35`", ",", "6662.06`", 
     ",", "6630.44`", ",", "6630.44`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:20:00\"\>", ",", "6631.48`", ",", "6648.84`", 
     ",", "6606.55`", ",", "6613.48`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:30:00\"\>", ",", "6613.4`", ",", "6619.69`", 
     ",", "6553.5`", ",", "6560.43`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:40:00\"\>", ",", "6560.35`", ",", "6610.85`", 
     ",", "6546.25`", ",", "6610.37`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T15:50:00\"\>", ",", "6609.49`", ",", "6609.49`", 
     ",", "6585.92`", ",", "6594.76`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-05T16:00:00\"\>", ",", "6594.68`", ",", "6595.8`", 
     ",", "6594.28`", ",", "6594.44`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T09:30:00\"\>", ",", "6595.16`", ",", "6672.26`", 
     ",", "6595.16`", ",", "6664.05`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T09:40:00\"\>", ",", "6663.34`", ",", "6754.93`", 
     ",", "6660.47`", ",", "6729.68`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T09:50:00\"\>", ",", "6727.06`", ",", "6728.73`", 
     ",", "6672.26`", ",", "6672.26`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:00:00\"\>", ",", "6674.17`", ",", "6688.35`", 
     ",", "6647.33`", ",", "6660.31`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:10:00\"\>", ",", "6659.67`", ",", "6672.18`", 
     ",", "6613.24`", ",", "6621.28`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:20:00\"\>", ",", "6621.92`", ",", "6639.36`", 
     ",", "6611.25`", ",", "6638.01`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:30:00\"\>", ",", "6638.41`", ",", "6647.01`", 
     ",", "6595.16`", ",", "6598.18`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:40:00\"\>", ",", "6597.87`", ",", "6622.96`", 
     ",", "6581.54`", ",", "6587.43`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T10:50:00\"\>", ",", "6587.03`", ",", "6587.03`", 
     ",", "6546.81`", ",", "6556.37`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:00:00\"\>", ",", "6556.93`", ",", "6568.24`", 
     ",", "6543.15`", ",", "6561.55`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:10:00\"\>", ",", "6561.39`", ",", "6582.73`", 
     ",", "6559.48`", ",", "6573.49`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:20:00\"\>", ",", "6574.37`", ",", "6579.07`", 
     ",", "6550.87`", ",", "6565.21`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:30:00\"\>", ",", "6565.45`", ",", "6596.83`", 
     ",", "6564.26`", ",", "6573.89`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:40:00\"\>", ",", "6573.73`", ",", "6575.96`", 
     ",", "6548.33`", ",", "6561.79`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T11:50:00\"\>", ",", "6561.55`", ",", "6590.3`", 
     ",", "6561.55`", ",", "6575.72`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:00:00\"\>", ",", "6575.8`", ",", "6609.49`", 
     ",", "6574.29`", ",", "6602.57`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:10:00\"\>", ",", "6602.65`", ",", "6612.12`", 
     ",", "6587.59`", ",", "6589.34`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:20:00\"\>", ",", "6589.11`", ",", "6598.42`", 
     ",", "6566.09`", ",", "6569.43`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:30:00\"\>", ",", "6567.68`", ",", "6569.59`", 
     ",", "6550.79`", ",", "6560.19`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:40:00\"\>", ",", "6560.35`", ",", "6560.35`", 
     ",", "6517.26`", ",", "6519.97`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T12:50:00\"\>", ",", "6520.77`", ",", "6551.27`", 
     ",", "6520.77`", ",", "6551.03`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:00:00\"\>", ",", "6550.87`", ",", "6550.87`", 
     ",", "6532.48`", ",", "6537.97`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:10:00\"\>", ",", "6537.89`", ",", "6545.94`", 
     ",", "6515.35`", ",", "6522.2`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:20:00\"\>", ",", "6521.72`", ",", "6531.12`", 
     ",", "6512.72`", ",", "6524.27`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:30:00\"\>", ",", "6524.51`", ",", "6528.89`", 
     ",", "6504.76`", ",", "6513.6`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:40:00\"\>", ",", "6514.24`", ",", "6521.48`", 
     ",", "6500.93`", ",", "6515.67`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T13:50:00\"\>", ",", "6516.47`", ",", "6525.86`", 
     ",", "6508.1`", ",", "6523.87`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:00:00\"\>", ",", "6523.95`", ",", "6551.67`", 
     ",", "6522.04`", ",", "6548.56`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:10:00\"\>", ",", "6549.12`", ",", "6549.44`", 
     ",", "6504.12`", ",", "6508.66`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:20:00\"\>", ",", "6509.38`", ",", "6536.86`", 
     ",", "6503.64`", ",", "6536.38`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:30:00\"\>", ",", "6538.21`", ",", "6540.92`", 
     ",", "6510.17`", ",", "6528.73`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:40:00\"\>", ",", "6526.42`", ",", "6526.42`", 
     ",", "6488.83`", ",", "6495.52`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T14:50:00\"\>", ",", "6496.`", ",", "6502.05`", 
     ",", "6479.51`", ",", "6482.62`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:00:00\"\>", ",", "6482.7`", ",", "6496.`", ",",
      "6474.89`", ",", "6487.`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:10:00\"\>", ",", "6488.27`", ",", "6512.32`", 
     ",", "6485.72`", ",", "6506.99`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:20:00\"\>", ",", "6505.4`", ",", "6505.4`", 
     ",", "6470.11`", ",", "6480.55`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:30:00\"\>", ",", "6480.31`", ",", "6537.17`", 
     ",", "6480.31`", ",", "6519.33`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:40:00\"\>", ",", "6519.25`", ",", "6609.73`", 
     ",", "6518.7`", ",", "6566.96`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T15:50:00\"\>", ",", "6565.05`", ",", "6647.49`", 
     ",", "6563.3`", ",", "6633.47`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-06T16:00:00\"\>", ",", "6633.23`", ",", "6633.23`", 
     ",", "6626.94`", ",", "6626.94`", ",", "27", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T09:30:00\"\>", ",", "6625.66`", ",", "6625.66`", 
     ",", "6556.77`", ",", "6600.97`", ",", "301", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T09:40:00\"\>", ",", "6601.53`", ",", "6642.07`", 
     ",", "6591.02`", ",", "6639.44`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T09:50:00\"\>", ",", "6638.49`", ",", "6652.98`", 
     ",", "6608.94`", ",", "6609.81`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:00:00\"\>", ",", "6609.18`", ",", "6676.64`", 
     ",", "6604.4`", ",", "6664.05`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:10:00\"\>", ",", "6662.38`", ",", "6709.21`", 
     ",", "6659.35`", ",", "6696.71`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:20:00\"\>", ",", "6698.94`", ",", "6699.5`", 
     ",", "6666.92`", ",", "6666.92`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:30:00\"\>", ",", "6666.52`", ",", "6680.86`", 
     ",", "6635.38`", ",", "6643.19`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:40:00\"\>", ",", "6643.98`", ",", "6673.05`", 
     ",", "6638.81`", ",", "6649.64`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T10:50:00\"\>", ",", "6648.6`", ",", "6659.59`", 
     ",", "6613.24`", ",", "6619.53`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:00:00\"\>", ",", "6619.77`", ",", "6643.27`", 
     ",", "6599.3`", ",", "6603.44`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:10:00\"\>", ",", "6604.`", ",", "6625.5`", ",",
      "6602.17`", ",", "6603.04`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:20:00\"\>", ",", "6604.72`", ",", "6626.14`", 
     ",", "6588.31`", ",", "6619.45`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:30:00\"\>", ",", "6619.53`", ",", "6640.08`", 
     ",", "6614.35`", ",", "6629.65`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:40:00\"\>", ",", "6629.96`", ",", "6639.04`", 
     ",", "6613.32`", ",", "6630.2`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T11:50:00\"\>", ",", "6630.28`", ",", "6634.03`", 
     ",", "6597.71`", ",", "6614.11`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:00:00\"\>", ",", "6614.51`", ",", "6625.42`", 
     ",", "6605.11`", ",", "6617.94`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:10:00\"\>", ",", "6618.5`", ",", "6651.71`", 
     ",", "6618.26`", ",", "6646.77`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:20:00\"\>", ",", "6646.29`", ",", "6660.15`", 
     ",", "6633.87`", ",", "6636.02`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:30:00\"\>", ",", "6636.34`", ",", "6636.89`", 
     ",", "6599.3`", ",", "6599.3`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:40:00\"\>", ",", "6597.55`", ",", "6605.03`", 
     ",", "6592.29`", ",", "6603.52`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T12:50:00\"\>", ",", "6603.44`", ",", "6608.14`", 
     ",", "6598.82`", ",", "6602.72`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:00:00\"\>", ",", "6601.61`", ",", "6601.61`", 
     ",", "6554.86`", ",", "6559.95`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:10:00\"\>", ",", "6558.6`", ",", "6568.64`", 
     ",", "6552.71`", ",", "6565.37`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:20:00\"\>", ",", "6565.93`", ",", "6568.32`", 
     ",", "6545.46`", ",", "6564.49`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:30:00\"\>", ",", "6564.02`", ",", "6578.51`", 
     ",", "6555.49`", ",", "6561.15`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:40:00\"\>", ",", "6561.31`", ",", "6582.89`", 
     ",", "6559.24`", ",", "6582.89`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T13:50:00\"\>", ",", "6583.21`", ",", "6591.73`", 
     ",", "6568.72`", ",", "6573.49`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:00:00\"\>", ",", "6577.32`", ",", "6619.37`", 
     ",", "6567.12`", ",", "6613.72`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:10:00\"\>", ",", "6613.56`", ",", "6616.74`", 
     ",", "6592.61`", ",", "6603.28`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:20:00\"\>", ",", "6603.76`", ",", "6605.03`", 
     ",", "6568.4`", ",", "6584.64`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:30:00\"\>", ",", "6584.72`", ",", "6584.88`", 
     ",", "6549.68`", ",", "6566.41`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:40:00\"\>", ",", "6565.85`", ",", "6594.36`", 
     ",", "6557.09`", ",", "6587.67`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T14:50:00\"\>", ",", "6587.59`", ",", "6605.83`", 
     ",", "6572.62`", ",", "6577.48`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:00:00\"\>", ",", "6577.72`", ",", "6597.39`", 
     ",", "6554.62`", ",", "6569.75`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:10:00\"\>", ",", "6570.79`", ",", "6570.79`", 
     ",", "6517.02`", ",", "6528.57`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:20:00\"\>", ",", "6528.17`", ",", "6555.25`", 
     ",", "6524.11`", ",", "6546.17`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:30:00\"\>", ",", "6545.62`", ",", "6567.2`", 
     ",", "6528.17`", ",", "6530.33`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:40:00\"\>", ",", "6527.7`", ",", "6544.66`", 
     ",", "6523.24`", ",", "6525.71`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T15:50:00\"\>", ",", "6525.39`", ",", "6556.69`", 
     ",", "6521.8`", ",", "6547.05`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-09T16:00:00\"\>", ",", "6547.13`", ",", "6547.13`", 
     ",", "6546.25`", ",", "6547.05`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T09:30:00\"\>", ",", "6547.01`", ",", "6697.74`", 
     ",", "6547.01`", ",", "6690.42`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T09:40:00\"\>", ",", "6689.7`", ",", "6740.75`", 
     ",", "6685.8`", ",", "6736.53`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T09:50:00\"\>", ",", "6737.65`", ",", "6769.27`", 
     ",", "6730.48`", ",", "6747.92`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:00:00\"\>", ",", "6747.45`", ",", "6785.12`", 
     ",", "6747.45`", ",", "6785.12`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:10:00\"\>", ",", "6785.91`", ",", "6821.84`", 
     ",", "6783.13`", ",", "6812.04`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:20:00\"\>", ",", "6813.47`", ",", "6813.47`", 
     ",", "6791.89`", ",", "6794.68`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:30:00\"\>", ",", "6796.43`", ",", "6804.31`", 
     ",", "6778.43`", ",", "6790.53`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:40:00\"\>", ",", "6790.06`", ",", "6820.64`", 
     ",", "6787.35`", ",", "6815.62`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T10:50:00\"\>", ",", "6812.2`", ",", "6822.47`", 
     ",", "6794.76`", ",", "6798.58`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:00:00\"\>", ",", "6798.5`", ",", "6803.28`", 
     ",", "6788.07`", ",", "6789.18`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:10:00\"\>", ",", "6788.86`", ",", "6806.46`", 
     ",", "6783.13`", ",", "6801.45`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:20:00\"\>", ",", "6801.05`", ",", "6823.67`", 
     ",", "6796.83`", ",", "6820.24`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:30:00\"\>", ",", "6820.32`", ",", "6823.43`", 
     ",", "6804.95`", ",", "6805.83`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:40:00\"\>", ",", "6806.78`", ",", "6809.33`", 
     ",", "6798.18`", ",", "6801.29`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T11:50:00\"\>", ",", "6801.84`", ",", "6856.`", 
     ",", "6801.69`", ",", "6856.`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:00:00\"\>", ",", "6856.96`", ",", "6880.46`", 
     ",", "6852.66`", ",", "6860.15`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:10:00\"\>", ",", "6860.39`", ",", "6865.16`", 
     ",", "6851.07`", ",", "6854.73`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:20:00\"\>", ",", "6854.01`", ",", "6864.53`", 
     ",", "6845.33`", ",", "6864.21`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:30:00\"\>", ",", "6863.89`", ",", "6868.59`", 
     ",", "6851.31`", ",", "6851.31`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:40:00\"\>", ",", "6851.7`", ",", "6856.56`", 
     ",", "6847.16`", ",", "6847.24`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T12:50:00\"\>", ",", "6847.08`", ",", "6852.98`", 
     ",", "6835.3`", ",", "6836.09`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:00:00\"\>", ",", "6835.38`", ",", "6840.95`", 
     ",", "6828.77`", ",", "6834.98`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:10:00\"\>", ",", "6834.74`", ",", "6848.92`", 
     ",", "6831.47`", ",", "6838.64`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:20:00\"\>", ",", "6838.72`", ",", "6844.22`", 
     ",", "6824.46`", ",", "6834.98`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:30:00\"\>", ",", "6833.23`", ",", "6860.86`", 
     ",", "6831.39`", ",", "6846.85`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:40:00\"\>", ",", "6846.05`", ",", "6877.35`", 
     ",", "6846.05`", ",", "6866.76`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T13:50:00\"\>", ",", "6867.24`", ",", "6869.15`", 
     ",", "6831.87`", ",", "6838.32`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:00:00\"\>", ",", "6838.96`", ",", "6852.98`", 
     ",", "6833.07`", ",", "6837.29`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:10:00\"\>", ",", "6837.45`", ",", "6846.45`", 
     ",", "6817.54`", ",", "6832.19`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:20:00\"\>", ",", "6832.03`", ",", "6852.9`", 
     ",", "6822.23`", ",", "6841.83`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:30:00\"\>", ",", "6841.99`", ",", "6865.56`", 
     ",", "6837.13`", ",", "6842.07`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:40:00\"\>", ",", "6843.02`", ",", "6851.54`", 
     ",", "6809.25`", ",", "6810.77`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T14:50:00\"\>", ",", "6810.84`", ",", "6859.19`", 
     ",", "6810.84`", ",", "6858.95`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:00:00\"\>", ",", "6858.63`", ",", "6874.88`", 
     ",", "6850.35`", ",", "6866.44`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:10:00\"\>", ",", "6866.84`", ",", "6871.62`", 
     ",", "6844.46`", ",", "6857.68`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:20:00\"\>", ",", "6856.96`", ",", "6896.07`", 
     ",", "6852.26`", ",", "6886.03`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:30:00\"\>", ",", "6887.15`", ",", "6890.57`", 
     ",", "6866.68`", ",", "6884.92`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:40:00\"\>", ",", "6884.68`", ",", "6896.39`", 
     ",", "6863.97`", ",", "6875.76`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T15:50:00\"\>", ",", "6876.16`", ",", "6913.91`", 
     ",", "6876.16`", ",", "6901.48`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-10T16:00:00\"\>", ",", "6902.76`", ",", "6926.49`", 
     ",", "6902.68`", ",", "6926.49`", ",", "26", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T09:30:00\"\>", ",", "6923.13`", ",", "6948.79`", 
     ",", "6903.79`", ",", "6948.64`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T09:40:00\"\>", ",", "6949.99`", ",", "6998.89`", 
     ",", "6936.21`", ",", "6989.26`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T09:50:00\"\>", ",", "6989.65`", ",", "7014.82`", 
     ",", "6981.69`", ",", "6993.64`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:00:00\"\>", ",", "6993.72`", ",", "7007.81`", 
     ",", "6973.72`", ",", "6974.68`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:10:00\"\>", ",", "6973.33`", ",", "6978.26`", 
     ",", "6962.18`", ",", "6967.67`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:20:00\"\>", ",", "6967.03`", ",", "6989.89`", 
     ",", "6962.41`", ",", "6988.38`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:30:00\"\>", ",", "6989.49`", ",", "6994.35`", 
     ",", "6956.44`", ",", "6964.09`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:40:00\"\>", ",", "6963.13`", ",", "6969.58`", 
     ",", "6947.84`", ",", "6954.61`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T10:50:00\"\>", ",", "6954.53`", ",", "6973.25`", 
     ",", "6954.53`", ",", "6969.42`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:00:00\"\>", ",", "6968.31`", ",", "6986.39`", 
     ",", "6968.31`", ",", "6983.28`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:10:00\"\>", ",", "6983.92`", ",", "6987.5`", 
     ",", "6964.56`", ",", "6967.03`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:20:00\"\>", ",", "6966.56`", ",", "6966.56`", 
     ",", "6940.67`", ",", "6951.26`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:30:00\"\>", ",", "6951.98`", ",", "6953.81`", 
     ",", "6921.87`", ",", "6945.45`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:40:00\"\>", ",", "6943.94`", ",", "6944.1`", 
     ",", "6926.81`", ",", "6935.41`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T11:50:00\"\>", ",", "6935.49`", ",", "6951.42`", 
     ",", "6933.66`", ",", "6948.95`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:00:00\"\>", ",", "6949.19`", ",", "6959.15`", 
     ",", "6949.19`", ",", "6953.97`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:10:00\"\>", ",", "6953.89`", ",", "6961.86`", 
     ",", "6942.02`", ",", "6942.26`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:20:00\"\>", ",", "6942.02`", ",", "6942.34`", 
     ",", "6926.41`", ",", "6935.89`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:30:00\"\>", ",", "6936.05`", ",", "6944.65`", 
     ",", "6909.85`", ",", "6909.85`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:40:00\"\>", ",", "6908.33`", ",", "6919.09`", 
     ",", "6902.76`", ",", "6914.23`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T12:50:00\"\>", ",", "6914.47`", ",", "6919.09`", 
     ",", "6891.53`", ",", "6891.53`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:00:00\"\>", ",", "6891.69`", ",", "6910.32`", 
     ",", "6891.69`", ",", "6896.15`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:10:00\"\>", ",", "6896.23`", ",", "6927.21`", 
     ",", "6895.11`", ",", "6925.54`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:20:00\"\>", ",", "6925.86`", ",", "6927.13`", 
     ",", "6892.8`", ",", "6898.94`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:30:00\"\>", ",", "6903.95`", ",", "6941.39`", 
     ",", "6897.58`", ",", "6941.39`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:40:00\"\>", ",", "6946.33`", ",", "6946.33`", 
     ",", "6926.25`", ",", "6926.49`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T13:50:00\"\>", ",", "6926.73`", ",", "6932.31`", 
     ",", "6911.68`", ",", "6932.31`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:00:00\"\>", ",", "6932.63`", ",", "6946.09`", 
     ",", "6897.34`", ",", "6897.34`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:10:00\"\>", ",", "6896.55`", ",", "6905.31`", 
     ",", "6868.59`", ",", "6873.29`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:20:00\"\>", ",", "6872.41`", ",", "6882.29`", 
     ",", "6867.63`", ",", "6881.41`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:30:00\"\>", ",", "6881.97`", ",", "6898.94`", 
     ",", "6881.81`", ",", "6894.63`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:40:00\"\>", ",", "6894.95`", ",", "6937.33`", 
     ",", "6887.94`", ",", "6923.47`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T14:50:00\"\>", ",", "6924.98`", ",", "6929.52`", 
     ",", "6909.53`", ",", "6909.85`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:00:00\"\>", ",", "6910.4`", ",", "6949.03`", 
     ",", "6910.4`", ",", "6949.03`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:10:00\"\>", ",", "6949.19`", ",", "6963.53`", 
     ",", "6944.1`", ",", "6955.49`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:20:00\"\>", ",", "6957.72`", ",", "6974.68`", 
     ",", "6953.33`", ",", "6956.84`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:30:00\"\>", ",", "6956.92`", ",", "6983.68`", 
     ",", "6948.`", ",", "6981.29`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:40:00\"\>", ",", "6981.53`", ",", "6989.26`", 
     ",", "6945.85`", ",", "6954.53`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T15:50:00\"\>", ",", "6954.93`", ",", "6962.81`", 
     ",", "6926.41`", ",", "6929.52`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-11T16:00:00\"\>", ",", "6928.96`", ",", "6930.4`", 
     ",", "6928.64`", ",", "6930.4`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T09:30:00\"\>", ",", "6932.39`", ",", "6935.65`", 
     ",", "6901.32`", ",", "6901.32`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T09:40:00\"\>", ",", "6901.48`", ",", "6905.39`", 
     ",", "6872.25`", ",", "6887.55`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T09:50:00\"\>", ",", "6887.7`", ",", "6913.03`", 
     ",", "6881.09`", ",", "6913.03`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:00:00\"\>", ",", "6915.82`", ",", "6955.25`", 
     ",", "6915.82`", ",", "6955.25`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:10:00\"\>", ",", "6955.72`", ",", "6980.41`", 
     ",", "6950.07`", ",", "6975.24`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:20:00\"\>", ",", "6975.48`", ",", "7006.14`", 
     ",", "6975.48`", ",", "6988.78`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:30:00\"\>", ",", "6991.09`", ",", "7003.83`", 
     ",", "6978.5`", ",", "6985.59`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:40:00\"\>", ",", "6985.75`", ",", "7047.24`", 
     ",", "6985.19`", ",", "7035.85`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T10:50:00\"\>", ",", "7035.77`", ",", "7037.6`", 
     ",", "7022.63`", ",", "7025.89`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:00:00\"\>", ",", "7023.74`", ",", "7033.78`", 
     ",", "7009.8`", ",", "7018.41`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:10:00\"\>", ",", "7018.01`", ",", "7032.27`", 
     ",", "7012.59`", ",", "7024.54`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:20:00\"\>", ",", "7024.62`", ",", "7025.02`", 
     ",", "6995.23`", ",", "7000.57`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:30:00\"\>", ",", "7000.49`", ",", "7008.69`", 
     ",", "6993.16`", ",", "7007.97`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:40:00\"\>", ",", "7007.89`", ",", "7042.94`", 
     ",", "7007.65`", ",", "7029.`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T11:50:00\"\>", ",", "7028.44`", ",", "7040.23`", 
     ",", "7017.61`", ",", "7025.97`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:00:00\"\>", ",", "7025.73`", ",", "7053.69`", 
     ",", "7025.5`", ",", "7051.22`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:10:00\"\>", ",", "7051.38`", ",", "7061.66`", 
     ",", "7038.32`", ",", "7046.36`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:20:00\"\>", ",", "7046.52`", ",", "7055.12`", 
     ",", "7028.6`", ",", "7037.52`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:30:00\"\>", ",", "7038.16`", ",", "7058.71`", 
     ",", "7032.35`", ",", "7052.97`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:40:00\"\>", ",", "7053.05`", ",", "7057.04`", 
     ",", "7045.65`", ",", "7056.88`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T12:50:00\"\>", ",", "7057.04`", ",", "7086.82`", 
     ",", "7049.71`", ",", "7082.05`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:00:00\"\>", ",", "7082.13`", ",", "7087.94`", 
     ",", "7064.92`", ",", "7084.28`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:10:00\"\>", ",", "7084.36`", ",", "7093.36`", 
     ",", "7071.85`", ",", "7074.88`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:20:00\"\>", ",", "7074.32`", ",", "7091.68`", 
     ",", "7059.66`", ",", "7063.25`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:30:00\"\>", ",", "7065.`", ",", "7078.54`", 
     ",", "7061.26`", ",", "7077.9`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:40:00\"\>", ",", "7078.22`", ",", "7088.26`", 
     ",", "7077.51`", ",", "7084.67`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T13:50:00\"\>", ",", "7085.87`", ",", "7097.74`", 
     ",", "7076.15`", ",", "7083.08`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:00:00\"\>", ",", "7083.4`", ",", "7094.07`", 
     ",", "7069.46`", ",", "7075.91`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:10:00\"\>", ",", "7076.47`", ",", "7122.59`", 
     ",", "7076.47`", ",", "7120.6`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:20:00\"\>", ",", "7119.56`", ",", "7129.12`", 
     ",", "7107.21`", ",", "7117.73`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:30:00\"\>", ",", "7116.85`", ",", "7129.2`", 
     ",", "7108.65`", ",", "7120.44`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:40:00\"\>", ",", "7121.31`", ",", "7148.31`", 
     ",", "7121.15`", ",", "7148.23`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T14:50:00\"\>", ",", "7149.35`", ",", "7154.29`", 
     ",", "7129.12`", ",", "7129.12`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:00:00\"\>", ",", "7129.28`", ",", "7131.75`", 
     ",", "7117.81`", ",", "7130.07`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:10:00\"\>", ",", "7129.75`", ",", "7148.31`", 
     ",", "7124.34`", ",", "7139.87`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:20:00\"\>", ",", "7140.51`", ",", "7142.02`", 
     ",", "7118.21`", ",", "7133.9`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:30:00\"\>", ",", "7133.58`", ",", "7156.68`", 
     ",", "7127.92`", ",", "7153.09`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:40:00\"\>", ",", "7153.01`", ",", "7184.55`", 
     ",", "7151.02`", ",", "7177.62`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T15:50:00\"\>", ",", "7179.14`", ",", "7183.36`", 
     ",", "7150.3`", ",", "7167.51`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-12T16:00:00\"\>", ",", "7166.79`", ",", "7170.61`", 
     ",", "7166.79`", ",", "7170.06`", ",", "26", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T09:30:00\"\>", ",", "7167.35`", ",", "7211.63`", 
     ",", "7167.35`", ",", "7202.39`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T09:40:00\"\>", ",", "7202.31`", ",", "7212.59`", 
     ",", "7161.45`", ",", "7184.87`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T09:50:00\"\>", ",", "7184.23`", ",", "7215.38`", 
     ",", "7168.86`", ",", "7215.38`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:00:00\"\>", ",", "7215.85`", ",", "7242.38`", 
     ",", "7194.35`", ",", "7239.27`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:10:00\"\>", ",", "7239.19`", ",", "7240.78`", 
     ",", "7217.53`", ",", "7223.5`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:20:00\"\>", ",", "7221.11`", ",", "7230.27`", 
     ",", "7179.69`", ",", "7181.21`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:30:00\"\>", ",", "7180.73`", ",", "7188.06`", 
     ",", "7163.53`", ",", "7186.3`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:40:00\"\>", ",", "7186.15`", ",", "7233.77`", 
     ",", "7186.15`", ",", "7227.72`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T10:50:00\"\>", ",", "7228.68`", ",", "7228.68`", 
     ",", "7178.42`", ",", "7184.47`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:00:00\"\>", ",", "7182.32`", ",", "7204.07`", 
     ",", "7160.66`", ",", "7162.49`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:10:00\"\>", ",", "7162.25`", ",", "7175.39`", 
     ",", "7152.45`", ",", "7156.91`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:20:00\"\>", ",", "7156.04`", ",", "7171.73`", 
     ",", "7147.75`", ",", "7171.09`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:30:00\"\>", ",", "7171.25`", ",", "7171.25`", 
     ",", "7140.35`", ",", "7152.22`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:40:00\"\>", ",", "7151.98`", ",", "7159.22`", 
     ",", "7122.83`", ",", "7129.36`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T11:50:00\"\>", ",", "7129.12`", ",", "7143.14`", 
     ",", "7122.9`", ",", "7140.75`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:00:00\"\>", ",", "7140.03`", ",", "7152.77`", 
     ",", "7133.42`", ",", "7137.8`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:10:00\"\>", ",", "7137.72`", ",", "7137.72`", 
     ",", "7109.13`", ",", "7111.28`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:20:00\"\>", ",", "7110.64`", ",", "7132.3`", 
     ",", "7106.26`", ",", "7121.87`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:30:00\"\>", ",", "7121.55`", ",", "7136.6`", 
     ",", "7121.55`", ",", "7126.01`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:40:00\"\>", ",", "7125.85`", ",", "7164.`", 
     ",", "7125.37`", ",", "7149.83`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T12:50:00\"\>", ",", "7149.51`", ",", "7162.09`", 
     ",", "7134.53`", ",", "7161.93`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:00:00\"\>", ",", "7162.09`", ",", "7166.55`", 
     ",", "7154.53`", ",", "7163.13`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:10:00\"\>", ",", "7163.21`", ",", "7192.44`", 
     ",", "7162.97`", ",", "7186.22`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:20:00\"\>", ",", "7186.62`", ",", "7199.13`", 
     ",", "7180.89`", ",", "7194.75`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:30:00\"\>", ",", "7195.07`", ",", "7196.74`", 
     ",", "7166.71`", ",", "7168.78`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:40:00\"\>", ",", "7169.5`", ",", "7181.92`", 
     ",", "7169.5`", ",", "7176.67`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T13:50:00\"\>", ",", "7177.15`", ",", "7191.8`", 
     ",", "7176.51`", ",", "7187.58`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:00:00\"\>", ",", "7188.61`", ",", "7213.31`", 
     ",", "7185.91`", ",", "7208.53`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:10:00\"\>", ",", "7208.13`", ",", "7208.13`", 
     ",", "7179.53`", ",", "7183.99`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:20:00\"\>", ",", "7181.21`", ",", "7187.1`", 
     ",", "7172.13`", ",", "7182.4`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:30:00\"\>", ",", "7181.29`", ",", "7213.62`", 
     ",", "7179.06`", ",", "7200.96`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:40:00\"\>", ",", "7200.88`", ",", "7227.16`", 
     ",", "7196.26`", ",", "7214.26`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T14:50:00\"\>", ",", "7213.94`", ",", "7215.61`", 
     ",", "7192.12`", ",", "7196.5`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:00:00\"\>", ",", "7195.62`", ",", "7209.`", 
     ",", "7193.31`", ",", "7206.69`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:10:00\"\>", ",", "7206.46`", ",", "7234.33`", 
     ",", "7202.07`", ",", "7230.83`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:20:00\"\>", ",", "7230.75`", ",", "7239.99`", 
     ",", "7223.34`", ",", "7229.71`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:30:00\"\>", ",", "7230.91`", ",", "7233.62`", 
     ",", "7201.68`", ",", "7213.07`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:40:00\"\>", ",", "7212.83`", ",", "7230.51`", 
     ",", "7198.73`", ",", "7199.84`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T15:50:00\"\>", ",", "7199.13`", ",", "7223.74`", 
     ",", "7198.49`", ",", "7219.2`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-13T16:00:00\"\>", ",", "7219.04`", ",", "7223.98`", 
     ",", "7219.04`", ",", "7223.98`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T09:30:00\"\>", ",", "7225.33`", ",", "7311.27`", 
     ",", "7225.33`", ",", "7307.05`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T09:40:00\"\>", ",", "7306.81`", ",", "7308.96`", 
     ",", "7285.39`", ",", "7290.4`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T09:50:00\"\>", ",", "7289.53`", ",", "7301.4`", 
     ",", "7275.11`", ",", "7290.88`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:00:00\"\>", ",", "7290.72`", ",", "7297.65`", 
     ",", "7268.34`", ",", "7278.3`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:10:00\"\>", ",", "7278.14`", ",", "7278.14`", 
     ",", "7255.28`", ",", "7264.44`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:20:00\"\>", ",", "7265.24`", ",", "7278.46`", 
     ",", "7256.79`", ",", "7260.06`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:30:00\"\>", ",", "7261.97`", ",", "7313.82`", 
     ",", "7261.89`", ",", "7305.54`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:40:00\"\>", ",", "7306.57`", ",", "7311.27`", 
     ",", "7296.06`", ",", "7306.41`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T10:50:00\"\>", ",", "7306.73`", ",", "7308.17`", 
     ",", "7284.67`", ",", "7285.31`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:00:00\"\>", ",", "7285.23`", ",", "7309.84`", 
     ",", "7284.03`", ",", "7302.51`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:10:00\"\>", ",", "7302.43`", ",", "7321.55`", 
     ",", "7299.96`", ",", "7321.39`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:20:00\"\>", ",", "7321.07`", ",", "7344.96`", 
     ",", "7317.17`", ",", "7334.05`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:30:00\"\>", ",", "7334.13`", ",", "7344.33`", 
     ",", "7334.05`", ",", "7336.76`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:40:00\"\>", ",", "7336.68`", ",", "7336.76`", 
     ",", "7324.1`", ",", "7332.62`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T11:50:00\"\>", ",", "7332.46`", ",", "7335.56`", 
     ",", "7318.2`", ",", "7324.1`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:00:00\"\>", ",", "7323.86`", ",", "7323.86`", 
     ",", "7308.4`", ",", "7316.69`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:10:00\"\>", ",", "7316.93`", ",", "7329.59`", 
     ",", "7316.93`", ",", "7325.93`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:20:00\"\>", ",", "7324.65`", ",", "7332.3`", 
     ",", "7319.79`", ",", "7332.06`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:30:00\"\>", ",", "7333.33`", ",", "7359.22`", 
     ",", "7333.33`", ",", "7351.97`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:40:00\"\>", ",", "7351.89`", ",", "7361.69`", 
     ",", "7350.62`", ",", "7361.69`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T12:50:00\"\>", ",", "7362.09`", ",", "7382.56`", 
     ",", "7356.83`", ",", "7379.53`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:00:00\"\>", ",", "7378.1`", ",", "7381.28`", 
     ",", "7361.21`", ",", "7369.26`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:10:00\"\>", ",", "7369.34`", ",", "7379.13`", 
     ",", "7363.28`", ",", "7379.13`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:20:00\"\>", ",", "7378.97`", ",", "7380.57`", 
     ",", "7359.14`", ",", "7370.45`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:30:00\"\>", ",", "7370.61`", ",", "7386.3`", 
     ",", "7361.37`", ",", "7382.95`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:40:00\"\>", ",", "7382.72`", ",", "7392.83`", 
     ",", "7378.42`", ",", "7382.72`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T13:50:00\"\>", ",", "7382.4`", ",", "7382.56`", 
     ",", "7351.73`", ",", "7354.76`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:00:00\"\>", ",", "7354.04`", ",", "7357.31`", 
     ",", "7335.01`", ",", "7341.78`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:10:00\"\>", ",", "7342.02`", ",", "7355.56`", 
     ",", "7338.19`", ",", "7354.36`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:20:00\"\>", ",", "7351.97`", ",", "7352.45`", 
     ",", "7324.17`", ",", "7328.4`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:30:00\"\>", ",", "7328.64`", ",", "7336.44`", 
     ",", "7315.89`", ",", "7326.48`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:40:00\"\>", ",", "7326.17`", ",", "7353.41`", 
     ",", "7326.17`", ",", "7348.23`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T14:50:00\"\>", ",", "7351.18`", ",", "7351.97`", 
     ",", "7298.85`", ",", "7305.7`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:00:00\"\>", ",", "7306.81`", ",", "7308.56`", 
     ",", "7280.85`", ",", "7298.29`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:10:00\"\>", ",", "7295.18`", ",", "7298.77`", 
     ",", "7267.07`", ",", "7273.68`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:20:00\"\>", ",", "7271.05`", ",", "7272.32`", 
     ",", "7244.37`", ",", "7266.83`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:30:00\"\>", ",", "7267.78`", ",", "7268.66`", 
     ",", "7227.48`", ",", "7232.58`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:40:00\"\>", ",", "7233.85`", ",", "7238.39`", 
     ",", "7209.8`", ",", "7222.62`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T15:50:00\"\>", ",", "7223.1`", ",", "7233.46`", 
     ",", "7207.09`", ",", "7215.69`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-16T16:00:00\"\>", ",", "7215.3`", ",", "7218.24`", 
     ",", "7215.3`", ",", "7216.97`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T09:30:00\"\>", ",", "7218.16`", ",", "7237.12`", 
     ",", "7197.61`", ",", "7197.61`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T09:40:00\"\>", ",", "7197.53`", ",", "7197.69`", 
     ",", "7172.37`", ",", "7187.42`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T09:50:00\"\>", ",", "7185.11`", ",", "7219.44`", 
     ",", "7183.76`", ",", "7199.92`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:00:00\"\>", ",", "7199.84`", ",", "7240.54`", 
     ",", "7199.84`", ",", "7216.09`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:10:00\"\>", ",", "7216.33`", ",", "7224.77`", 
     ",", "7194.51`", ",", "7204.86`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:20:00\"\>", ",", "7204.46`", ",", "7223.02`", 
     ",", "7189.25`", ",", "7191.32`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:30:00\"\>", ",", "7190.92`", ",", "7210.76`", 
     ",", "7183.28`", ",", "7205.1`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:40:00\"\>", ",", "7204.46`", ",", "7252.57`", 
     ",", "7195.23`", ",", "7245.4`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T10:50:00\"\>", ",", "7245.48`", ",", "7253.69`", 
     ",", "7239.27`", ",", "7242.3`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:00:00\"\>", ",", "7241.18`", ",", "7241.18`", 
     ",", "7224.3`", ",", "7236.72`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:10:00\"\>", ",", "7236.48`", ",", "7240.39`", 
     ",", "7216.41`", ",", "7219.52`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:20:00\"\>", ",", "7218.`", ",", "7234.17`", 
     ",", "7209.24`", ",", "7215.77`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:30:00\"\>", ",", "7214.58`", ",", "7219.52`", 
     ",", "7203.83`", ",", "7214.18`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:40:00\"\>", ",", "7214.1`", ",", "7235.37`", 
     ",", "7210.2`", ",", "7231.86`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T11:50:00\"\>", ",", "7232.18`", ",", "7243.57`", 
     ",", "7232.18`", ",", "7241.9`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:00:00\"\>", ",", "7242.14`", ",", "7252.65`", 
     ",", "7240.23`", ",", "7240.86`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:10:00\"\>", ",", "7240.54`", ",", "7246.68`", 
     ",", "7228.12`", ",", "7244.61`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:20:00\"\>", ",", "7244.77`", ",", "7270.89`", 
     ",", "7244.69`", ",", "7263.88`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:30:00\"\>", ",", "7263.96`", ",", "7293.51`", 
     ",", "7254.8`", ",", "7290.64`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:40:00\"\>", ",", "7290.56`", ",", "7310.24`", 
     ",", "7287.54`", ",", "7293.11`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T12:50:00\"\>", ",", "7292.87`", ",", "7299.17`", 
     ",", "7289.29`", ",", "7294.55`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:00:00\"\>", ",", "7294.47`", ",", "7301.48`", 
     ",", "7283.47`", ",", "7284.43`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:10:00\"\>", ",", "7284.75`", ",", "7309.6`", 
     ",", "7283.4`", ",", "7299.8`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:20:00\"\>", ",", "7300.04`", ",", "7316.29`", 
     ",", "7294.23`", ",", "7301.95`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:30:00\"\>", ",", "7302.03`", ",", "7329.59`", 
     ",", "7301.95`", ",", "7318.76`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:40:00\"\>", ",", "7319.`", ",", "7323.46`", 
     ",", "7306.17`", ",", "7307.29`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T13:50:00\"\>", ",", "7307.05`", ",", "7307.13`", 
     ",", "7283.63`", ",", "7287.62`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:00:00\"\>", ",", "7288.01`", ",", "7294.31`", 
     ",", "7277.66`", ",", "7286.26`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:10:00\"\>", ",", "7286.42`", ",", "7300.44`", 
     ",", "7280.21`", ",", "7293.91`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:20:00\"\>", ",", "7293.83`", ",", "7301.32`", 
     ",", "7269.78`", ",", "7269.86`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:30:00\"\>", ",", "7269.78`", ",", "7289.45`", 
     ",", "7269.22`", ",", "7287.06`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:40:00\"\>", ",", "7286.98`", ",", "7316.53`", 
     ",", "7286.98`", ",", "7316.45`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T14:50:00\"\>", ",", "7316.29`", ",", "7328.95`", 
     ",", "7304.98`", ",", "7308.09`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:00:00\"\>", ",", "7308.32`", ",", "7349.82`", 
     ",", "7301.55`", ",", "7347.75`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:10:00\"\>", ",", "7350.3`", ",", "7351.57`", 
     ",", "7333.02`", ",", "7338.43`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:20:00\"\>", ",", "7337.`", ",", "7358.1`", ",",
      "7329.59`", ",", "7331.82`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:30:00\"\>", ",", "7331.26`", ",", "7338.51`", 
     ",", "7314.94`", ",", "7336.12`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:40:00\"\>", ",", "7337.16`", ",", "7367.11`", 
     ",", "7337.16`", ",", "7354.44`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T15:50:00\"\>", ",", "7347.83`", ",", "7396.34`", 
     ",", "7346.64`", ",", "7396.34`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-17T16:00:00\"\>", ",", "7396.26`", ",", "7396.73`", 
     ",", "7395.7`", ",", "7395.7`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T09:30:00\"\>", ",", "7395.62`", ",", "7395.62`", 
     ",", "7321.07`", ",", "7334.05`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T09:40:00\"\>", ",", "7333.89`", ",", "7347.51`", 
     ",", "7316.13`", ",", "7320.75`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T09:50:00\"\>", ",", "7320.51`", ",", "7326.64`", 
     ",", "7299.17`", ",", "7312.07`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:00:00\"\>", ",", "7311.99`", ",", "7320.19`", 
     ",", "7290.72`", ",", "7291.44`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:10:00\"\>", ",", "7291.28`", ",", "7302.11`", 
     ",", "7282.12`", ",", "7289.21`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:20:00\"\>", ",", "7286.66`", ",", "7288.89`", 
     ",", "7274.55`", ",", "7283.16`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:30:00\"\>", ",", "7279.57`", ",", "7287.3`", 
     ",", "7257.27`", ",", "7269.14`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:40:00\"\>", ",", "7268.98`", ",", "7293.27`", 
     ",", "7262.13`", ",", "7293.19`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T10:50:00\"\>", ",", "7291.44`", ",", "7314.14`", 
     ",", "7282.36`", ",", "7314.06`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:00:00\"\>", ",", "7314.46`", ",", "7324.17`", 
     ",", "7306.57`", ",", "7309.52`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:10:00\"\>", ",", "7308.32`", ",", "7315.49`", 
     ",", "7297.73`", ",", "7297.73`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:20:00\"\>", ",", "7297.89`", ",", "7306.49`", 
     ",", "7286.82`", ",", "7296.62`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:30:00\"\>", ",", "7296.7`", ",", "7309.76`", 
     ",", "7280.85`", ",", "7283.`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:40:00\"\>", ",", "7282.92`", ",", "7306.89`", 
     ",", "7282.6`", ",", "7292.08`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T11:50:00\"\>", ",", "7291.84`", ",", "7317.8`", 
     ",", "7290.64`", ",", "7317.72`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:00:00\"\>", ",", "7318.04`", ",", "7338.03`", 
     ",", "7318.04`", ",", "7337.24`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:10:00\"\>", ",", "7337.32`", ",", "7351.1`", 
     ",", "7327.52`", ",", "7348.15`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:20:00\"\>", ",", "7348.47`", ",", "7350.3`", 
     ",", "7329.03`", ",", "7334.53`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:30:00\"\>", ",", "7334.77`", ",", "7336.36`", 
     ",", "7313.18`", ",", "7314.62`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:40:00\"\>", ",", "7314.94`", ",", "7331.98`", 
     ",", "7313.58`", ",", "7323.94`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T12:50:00\"\>", ",", "7323.86`", ",", "7355.64`", 
     ",", "7311.99`", ",", "7352.21`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:00:00\"\>", ",", "7352.45`", ",", "7352.61`", 
     ",", "7341.54`", ",", "7344.49`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:10:00\"\>", ",", "7345.04`", ",", "7345.28`", 
     ",", "7328.79`", ",", "7330.07`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:20:00\"\>", ",", "7330.63`", ",", "7349.18`", 
     ",", "7330.63`", ",", "7344.8`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:30:00\"\>", ",", "7344.56`", ",", "7350.46`", 
     ",", "7340.66`", ",", "7344.01`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:40:00\"\>", ",", "7344.25`", ",", "7372.76`", 
     ",", "7343.53`", ",", "7357.47`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T13:50:00\"\>", ",", "7357.71`", ",", "7369.26`", 
     ",", "7346.`", ",", "7347.35`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:00:00\"\>", ",", "7347.19`", ",", "7347.91`", 
     ",", "7332.14`", ",", "7340.1`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:10:00\"\>", ",", "7339.63`", ",", "7438.63`", 
     ",", "7337.72`", ",", "7407.09`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:20:00\"\>", ",", "7406.29`", ",", "7525.37`", 
     ",", "7403.98`", ",", "7471.92`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:30:00\"\>", ",", "7474.79`", ",", "7499.96`", 
     ",", "7444.12`", ",", "7473.75`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:40:00\"\>", ",", "7475.9`", ",", "7546.39`", 
     ",", "7464.12`", ",", "7546.39`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T14:50:00\"\>", ",", "7543.45`", ",", "7571.08`", 
     ",", "7515.89`", ",", "7550.53`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:00:00\"\>", ",", "7550.45`", ",", "7550.45`", 
     ",", "7499.88`", ",", "7525.84`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:10:00\"\>", ",", "7525.05`", ",", "7536.83`", 
     ",", "7472.24`", ",", "7476.54`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:20:00\"\>", ",", "7476.86`", ",", "7476.86`", 
     ",", "7407.96`", ",", "7457.59`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:30:00\"\>", ",", "7461.73`", ",", "7468.89`", 
     ",", "7425.57`", ",", "7441.42`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:40:00\"\>", ",", "7440.86`", ",", "7478.05`", 
     ",", "7440.7`", ",", "7468.89`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T15:50:00\"\>", ",", "7469.61`", ",", "7496.77`", 
     ",", "7469.37`", ",", "7487.85`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-18T16:00:00\"\>", ",", "7487.69`", ",", "7488.81`", 
     ",", "7486.58`", ",", "7486.58`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T09:30:00\"\>", ",", "7489.68`", ",", "7548.46`", 
     ",", "7489.68`", ",", "7514.45`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T09:40:00\"\>", ",", "7514.21`", ",", "7524.01`", 
     ",", "7493.75`", ",", "7522.42`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T09:50:00\"\>", ",", "7521.54`", ",", "7521.54`", 
     ",", "7456.31`", ",", "7456.31`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:00:00\"\>", ",", "7460.21`", ",", "7466.19`", 
     ",", "7441.58`", ",", "7441.89`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:10:00\"\>", ",", "7440.46`", ",", "7486.34`", 
     ",", "7436.8`", ",", "7476.46`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:20:00\"\>", ",", "7477.97`", ",", "7491.75`", 
     ",", "7461.89`", ",", "7462.6`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:30:00\"\>", ",", "7462.05`", ",", "7482.59`", 
     ",", "7453.92`", ",", "7473.67`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:40:00\"\>", ",", "7474.23`", ",", "7496.61`", 
     ",", "7474.23`", ",", "7477.66`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T10:50:00\"\>", ",", "7481.24`", ",", "7481.24`", 
     ",", "7449.78`", ",", "7462.92`", ",", "325", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:00:00\"\>", ",", "7463.64`", ",", "7485.7`", 
     ",", "7446.28`", ",", "7446.28`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:10:00\"\>", ",", "7445.64`", ",", "7446.59`", 
     ",", "7429.23`", ",", "7438.23`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:20:00\"\>", ",", "7438.47`", ",", "7453.2`", 
     ",", "7409.88`", ",", "7409.88`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:30:00\"\>", ",", "7409.48`", ",", "7426.04`", 
     ",", "7407.57`", ",", "7416.81`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:40:00\"\>", ",", "7416.57`", ",", "7416.57`", 
     ",", "7385.74`", ",", "7391.8`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T11:50:00\"\>", ",", "7391.4`", ",", "7406.93`", 
     ",", "7369.65`", ",", "7406.29`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:00:00\"\>", ",", "7406.21`", ",", "7411.23`", 
     ",", "7396.65`", ",", "7402.87`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:10:00\"\>", ",", "7402.63`", ",", "7410.91`", 
     ",", "7383.59`", ",", "7387.1`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:20:00\"\>", ",", "7387.02`", ",", "7408.76`", 
     ",", "7378.26`", ",", "7407.09`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:30:00\"\>", ",", "7407.57`", ",", "7421.58`", 
     ",", "7397.53`", ",", "7406.69`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:40:00\"\>", ",", "7405.42`", ",", "7426.12`", 
     ",", "7399.44`", ",", "7419.75`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T12:50:00\"\>", ",", "7420.79`", ",", "7420.79`", 
     ",", "7393.15`", ",", "7395.06`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:00:00\"\>", ",", "7395.38`", ",", "7402.95`", 
     ",", "7384.95`", ",", "7388.37`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:10:00\"\>", ",", "7388.69`", ",", "7412.19`", 
     ",", "7387.34`", ",", "7411.63`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:20:00\"\>", ",", "7411.71`", ",", "7433.69`", 
     ",", "7410.99`", ",", "7421.74`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:30:00\"\>", ",", "7423.1`", ",", "7428.83`", 
     ",", "7398.41`", ",", "7416.33`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:40:00\"\>", ",", "7416.09`", ",", "7446.28`", 
     ",", "7413.62`", ",", "7442.93`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T13:50:00\"\>", ",", "7443.09`", ",", "7449.62`", 
     ",", "7433.29`", ",", "7436.4`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:00:00\"\>", ",", "7437.35`", ",", "7462.44`", 
     ",", "7437.35`", ",", "7446.43`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:10:00\"\>", ",", "7446.59`", ",", "7453.52`", 
     ",", "7430.35`", ",", "7436.4`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:20:00\"\>", ",", "7436.48`", ",", "7436.8`", 
     ",", "7408.92`", ",", "7414.42`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:30:00\"\>", ",", "7415.13`", ",", "7420.95`", 
     ",", "7396.57`", ",", "7406.85`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:40:00\"\>", ",", "7407.09`", ",", "7415.29`", 
     ",", "7387.89`", ",", "7395.78`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T14:50:00\"\>", ",", "7393.63`", ",", "7400.64`", 
     ",", "7376.58`", ",", "7392.51`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:00:00\"\>", ",", "7392.75`", ",", "7421.66`", 
     ",", "7391.16`", ",", "7409.32`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:10:00\"\>", ",", "7409.16`", ",", "7424.13`", 
     ",", "7396.18`", ",", "7415.77`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:20:00\"\>", ",", "7415.61`", ",", "7417.28`", 
     ",", "7391.08`", ",", "7406.29`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:30:00\"\>", ",", "7406.77`", ",", "7435.44`", 
     ",", "7403.42`", ",", "7405.02`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:40:00\"\>", ",", "7405.26`", ",", "7428.91`", 
     ",", "7390.76`", ",", "7418.`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T15:50:00\"\>", ",", "7417.44`", ",", "7421.11`", 
     ",", "7376.5`", ",", "7396.81`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-19T16:00:00\"\>", ",", "7396.73`", ",", "7400.8`", 
     ",", "7395.46`", ",", "7400.8`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T09:30:00\"\>", ",", "7402.23`", ",", "7442.37`", 
     ",", "7401.67`", ",", "7430.03`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T09:40:00\"\>", ",", "7428.43`", ",", "7452.89`", 
     ",", "7415.61`", ",", "7440.86`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T09:50:00\"\>", ",", "7441.66`", ",", "7460.45`", 
     ",", "7407.88`", ",", "7421.42`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:00:00\"\>", ",", "7421.58`", ",", "7427.88`", 
     ",", "7393.15`", ",", "7411.87`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:10:00\"\>", ",", "7412.42`", ",", "7422.94`", 
     ",", "7402.15`", ",", "7402.95`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:20:00\"\>", ",", "7401.19`", ",", "7443.25`", 
     ",", "7398.01`", ",", "7443.25`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:30:00\"\>", ",", "7443.73`", ",", "7443.73`", 
     ",", "7421.74`", ",", "7427.16`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:40:00\"\>", ",", "7427.32`", ",", "7436.8`", 
     ",", "7420.31`", ",", "7429.63`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T10:50:00\"\>", ",", "7429.55`", ",", "7447.63`", 
     ",", "7420.15`", ",", "7441.02`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:00:00\"\>", ",", "7441.1`", ",", "7441.1`", 
     ",", "7413.3`", ",", "7417.84`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:10:00\"\>", ",", "7417.92`", ",", "7417.92`", 
     ",", "7392.11`", ",", "7401.59`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:20:00\"\>", ",", "7401.75`", ",", "7404.06`", 
     ",", "7389.8`", ",", "7396.97`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:30:00\"\>", ",", "7396.73`", ",", "7417.68`", 
     ",", "7393.31`", ",", "7411.95`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:40:00\"\>", ",", "7412.98`", ",", "7423.18`", 
     ",", "7403.82`", ",", "7407.25`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T11:50:00\"\>", ",", "7406.21`", ",", "7417.6`", 
     ",", "7403.5`", ",", "7411.87`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:00:00\"\>", ",", "7412.11`", ",", "7415.45`", 
     ",", "7399.76`", ",", "7405.18`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:10:00\"\>", ",", "7404.7`", ",", "7433.13`", 
     ",", "7404.46`", ",", "7433.13`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:20:00\"\>", ",", "7433.45`", ",", "7435.28`", 
     ",", "7416.09`", ",", "7419.51`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:30:00\"\>", ",", "7419.83`", ",", "7434.49`", 
     ",", "7414.89`", ",", "7415.85`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:40:00\"\>", ",", "7416.17`", ",", "7420.07`", 
     ",", "7396.02`", ",", "7396.1`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T12:50:00\"\>", ",", "7396.34`", ",", "7396.89`", 
     ",", "7359.22`", ",", "7372.52`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:00:00\"\>", ",", "7371.33`", ",", "7375.23`", 
     ",", "7367.18`", ",", "7367.18`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:10:00\"\>", ",", "7367.5`", ",", "7375.87`", 
     ",", "7360.73`", ",", "7362.01`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:20:00\"\>", ",", "7360.18`", ",", "7360.18`", 
     ",", "7331.98`", ",", "7336.76`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:30:00\"\>", ",", "7335.96`", ",", "7348.07`", 
     ",", "7319.48`", ",", "7321.79`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:40:00\"\>", ",", "7322.42`", ",", "7325.45`", 
     ",", "7304.26`", ",", "7307.53`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T13:50:00\"\>", ",", "7308.09`", ",", "7319.32`", 
     ",", "7290.96`", ",", "7292.79`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:00:00\"\>", ",", "7292.95`", ",", "7294.63`", 
     ",", "7269.22`", ",", "7275.35`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:10:00\"\>", ",", "7274.87`", ",", "7289.85`", 
     ",", "7272.09`", ",", "7285.23`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:20:00\"\>", ",", "7285.55`", ",", "7292.24`", 
     ",", "7272.01`", ",", "7291.84`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:30:00\"\>", ",", "7290.32`", ",", "7294.31`", 
     ",", "7277.34`", ",", "7288.81`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:40:00\"\>", ",", "7289.37`", ",", "7289.61`", 
     ",", "7258.15`", ",", "7273.44`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T14:50:00\"\>", ",", "7273.6`", ",", "7295.42`", 
     ",", "7262.53`", ",", "7291.04`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:00:00\"\>", ",", "7290.88`", ",", "7308.56`", 
     ",", "7290.4`", ",", "7302.83`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:10:00\"\>", ",", "7302.91`", ",", "7315.57`", 
     ",", "7280.93`", ",", "7287.22`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:20:00\"\>", ",", "7287.54`", ",", "7325.37`", 
     ",", "7287.38`", ",", "7307.05`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:30:00\"\>", ",", "7306.81`", ",", "7337.64`", 
     ",", "7303.94`", ",", "7309.84`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:40:00\"\>", ",", "7311.51`", ",", "7314.62`", 
     ",", "7281.64`", ",", "7281.64`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T15:50:00\"\>", ",", "7283.47`", ",", "7295.9`", 
     ",", "7269.7`", ",", "7271.45`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-20T16:00:00\"\>", ",", "7271.37`", ",", "7278.38`", 
     ",", "7271.21`", ",", "7278.38`", ",", "26", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T09:30:00\"\>", ",", "7279.25`", ",", "7465.55`", 
     ",", "7279.25`", ",", "7458.06`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T09:40:00\"\>", ",", "7461.41`", ",", "7464.04`", 
     ",", "7425.96`", ",", "7431.54`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T09:50:00\"\>", ",", "7430.66`", ",", "7455.67`", 
     ",", "7415.69`", ",", "7453.52`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:00:00\"\>", ",", "7458.54`", ",", "7499.8`", 
     ",", "7458.54`", ",", "7485.38`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:10:00\"\>", ",", "7485.7`", ",", "7543.92`", 
     ",", "7477.1`", ",", "7533.57`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:20:00\"\>", ",", "7537.71`", ",", "7548.3`", 
     ",", "7531.82`", ",", "7534.92`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:30:00\"\>", ",", "7536.12`", ",", "7565.75`", 
     ",", "7533.97`", ",", "7540.98`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:40:00\"\>", ",", "7540.82`", ",", "7546.95`", 
     ",", "7528.63`", ",", "7536.91`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T10:50:00\"\>", ",", "7537.87`", ",", "7579.45`", 
     ",", "7537.87`", ",", "7574.51`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:00:00\"\>", ",", "7576.18`", ",", "7583.27`", 
     ",", "7566.62`", ",", "7569.81`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:10:00\"\>", ",", "7569.65`", ",", "7569.65`", 
     ",", "7558.1`", ",", "7559.69`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:20:00\"\>", ",", "7559.37`", ",", "7565.91`", 
     ",", "7545.52`", ",", "7550.37`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:30:00\"\>", ",", "7551.01`", ",", "7561.37`", 
     ",", "7550.37`", ",", "7552.05`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:40:00\"\>", ",", "7550.37`", ",", "7568.45`", 
     ",", "7549.5`", ",", "7565.27`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T11:50:00\"\>", ",", "7565.43`", ",", "7604.3`", 
     ",", "7564.71`", ",", "7599.52`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:00:00\"\>", ",", "7599.6`", ",", "7599.6`", 
     ",", "7585.34`", ",", "7597.69`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:10:00\"\>", ",", "7597.92`", ",", "7603.02`", 
     ",", "7593.54`", ",", "7593.94`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:20:00\"\>", ",", "7593.7`", ",", "7600.79`", 
     ",", "7583.59`", ",", "7600.79`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:30:00\"\>", ",", "7600.71`", ",", "7606.21`", 
     ",", "7588.92`", ",", "7592.67`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:40:00\"\>", ",", "7592.83`", ",", "7598.`", 
     ",", "7584.07`", ",", "7588.37`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T12:50:00\"\>", ",", "7588.45`", ",", "7601.43`", 
     ",", "7580.88`", ",", "7580.88`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:00:00\"\>", ",", "7580.64`", ",", "7585.42`", 
     ",", "7570.37`", ",", "7580.32`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:10:00\"\>", ",", "7580.`", ",", "7615.05`", 
     ",", "7580.`", ",", "7597.53`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:20:00\"\>", ",", "7597.61`", ",", "7598.72`", 
     ",", "7556.03`", ",", "7558.34`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:30:00\"\>", ",", "7558.9`", ",", "7563.52`", 
     ",", "7543.29`", ",", "7554.44`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:40:00\"\>", ",", "7551.01`", ",", "7562.72`", 
     ",", "7548.38`", ",", "7562.16`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T13:50:00\"\>", ",", "7561.92`", ",", "7566.94`", 
     ",", "7547.59`", ",", "7556.51`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:00:00\"\>", ",", "7556.27`", ",", "7571.64`", 
     ",", "7556.19`", ",", "7567.9`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:10:00\"\>", ",", "7567.74`", ",", "7567.74`", 
     ",", "7542.41`", ",", "7553.96`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:20:00\"\>", ",", "7553.48`", ",", "7583.59`", 
     ",", "7553.48`", ",", "7581.99`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:30:00\"\>", ",", "7581.92`", ",", "7601.91`", 
     ",", "7581.2`", ",", "7591.31`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:40:00\"\>", ",", "7592.91`", ",", "7623.33`", 
     ",", "7586.85`", ",", "7623.33`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T14:50:00\"\>", ",", "7621.66`", ",", "7660.69`", 
     ",", "7619.03`", ",", "7655.99`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:00:00\"\>", ",", "7656.23`", ",", "7662.44`", 
     ",", "7639.5`", ",", "7646.27`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:10:00\"\>", ",", "7646.03`", ",", "7685.7`", 
     ",", "7644.68`", ",", "7677.25`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:20:00\"\>", ",", "7676.7`", ",", "7683.94`", 
     ",", "7664.35`", ",", "7683.71`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:30:00\"\>", ",", "7686.57`", ",", "7706.72`", 
     ",", "7675.66`", ",", "7705.93`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:40:00\"\>", ",", "7706.17`", ",", "7738.1`", 
     ",", "7704.41`", ",", "7738.1`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T15:50:00\"\>", ",", "7736.67`", ",", "7779.84`", 
     ",", "7736.11`", ",", "7779.84`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-23T16:00:00\"\>", ",", "7779.76`", ",", "7779.92`", 
     ",", "7775.3`", ",", "7775.86`", ",", "25", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T09:30:00\"\>", ",", "7773.47`", ",", "7773.47`", 
     ",", "7674.15`", ",", "7699.63`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T09:40:00\"\>", ",", "7700.59`", ",", "7704.73`", 
     ",", "7672.79`", ",", "7684.9`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T09:50:00\"\>", ",", "7684.34`", ",", "7722.73`", 
     ",", "7681.63`", ",", "7722.73`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:00:00\"\>", ",", "7723.85`", ",", "7751.72`", 
     ",", "7723.85`", ",", "7732.93`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:10:00\"\>", ",", "7733.01`", ",", "7746.39`", 
     ",", "7705.85`", ",", "7705.85`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:20:00\"\>", ",", "7705.29`", ",", "7712.06`", 
     ",", "7678.45`", ",", "7680.12`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:30:00\"\>", ",", "7679.88`", ",", "7685.38`", 
     ",", "7667.93`", ",", "7684.66`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:40:00\"\>", ",", "7684.9`", ",", "7691.91`", 
     ",", "7664.43`", ",", "7675.66`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T10:50:00\"\>", ",", "7675.58`", ",", "7697.17`", 
     ",", "7675.58`", ",", "7690.55`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:00:00\"\>", ",", "7690.87`", ",", "7701.15`", 
     ",", "7679.32`", ",", "7699.`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:10:00\"\>", ",", "7698.92`", ",", "7713.25`", 
     ",", "7695.25`", ",", "7697.01`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:20:00\"\>", ",", "7696.45`", ",", "7729.74`", 
     ",", "7695.25`", ",", "7723.77`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:30:00\"\>", ",", "7723.61`", ",", "7735.95`", 
     ",", "7718.99`", ",", "7735.95`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:40:00\"\>", ",", "7735.87`", ",", "7736.91`", 
     ",", "7724.01`", ",", "7732.21`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T11:50:00\"\>", ",", "7732.05`", ",", "7736.19`", 
     ",", "7706.09`", ",", "7706.48`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:00:00\"\>", ",", "7705.45`", ",", "7708.4`", 
     ",", "7694.7`", ",", "7694.7`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:10:00\"\>", ",", "7693.18`", ",", "7723.69`", 
     ",", "7689.84`", ",", "7722.73`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:20:00\"\>", ",", "7722.81`", ",", "7727.75`", 
     ",", "7714.37`", ",", "7727.75`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:30:00\"\>", ",", "7727.43`", ",", "7737.71`", 
     ",", "7721.78`", ",", "7722.1`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:40:00\"\>", ",", "7721.7`", ",", "7745.03`", 
     ",", "7717.79`", ",", "7745.03`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T12:50:00\"\>", ",", "7745.11`", ",", "7768.85`", 
     ",", "7742.72`", ",", "7764.23`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:00:00\"\>", ",", "7763.99`", ",", "7767.97`", 
     ",", "7750.13`", ",", "7767.57`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:10:00\"\>", ",", "7767.49`", ",", "7782.07`", 
     ",", "7748.62`", ",", "7775.3`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:20:00\"\>", ",", "7775.62`", ",", "7796.25`", 
     ",", "7775.54`", ",", "7789.`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:30:00\"\>", ",", "7788.52`", ",", "7789.24`", 
     ",", "7753.48`", ",", "7757.06`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:40:00\"\>", ",", "7757.14`", ",", "7774.03`", 
     ",", "7755.31`", ",", "7756.5`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T13:50:00\"\>", ",", "7756.66`", ",", "7760.33`", 
     ",", "7742.88`", ",", "7755.15`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:00:00\"\>", ",", "7755.39`", ",", "7779.84`", 
     ",", "7746.79`", ",", "7778.49`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:10:00\"\>", ",", "7778.8`", ",", "7787.49`", 
     ",", "7766.22`", ",", "7779.2`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:20:00\"\>", ",", "7778.65`", ",", "7792.58`", 
     ",", "7751.88`", ",", "7754.43`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:30:00\"\>", ",", "7754.83`", ",", "7767.1`", 
     ",", "7743.52`", ",", "7755.95`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:40:00\"\>", ",", "7756.26`", ",", "7770.04`", 
     ",", "7750.93`", ",", "7753.87`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T14:50:00\"\>", ",", "7753.56`", ",", "7762.32`", 
     ",", "7725.28`", ",", "7738.74`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:00:00\"\>", ",", "7739.06`", ",", "7745.27`", 
     ",", "7704.49`", ",", "7712.7`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:10:00\"\>", ",", "7713.02`", ",", "7732.45`", 
     ",", "7708.87`", ",", "7719.71`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:20:00\"\>", ",", "7719.31`", ",", "7731.65`", 
     ",", "7698.52`", ",", "7714.61`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:30:00\"\>", ",", "7714.69`", ",", "7736.27`", 
     ",", "7706.01`", ",", "7728.95`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:40:00\"\>", ",", "7726.95`", ",", "7731.89`", 
     ",", "7703.54`", ",", "7703.54`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T15:50:00\"\>", ",", "7702.58`", ",", "7707.52`", 
     ",", "7647.47`", ",", "7659.25`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T16:00:00\"\>", ",", "7659.49`", ",", "7660.53`", 
     ",", "7659.49`", ",", "7660.29`", ",", "33", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-24T16:10:00\"\>", ",", "7660.37`", ",", "7660.61`", 
     ",", "7660.29`", ",", "7660.61`", ",", "6", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T09:30:00\"\>", ",", "7659.81`", ",", "7747.34`", 
     ",", "7659.81`", ",", "7740.02`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T09:40:00\"\>", ",", "7739.06`", ",", "7760.72`", 
     ",", "7736.11`", ",", "7747.18`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T09:50:00\"\>", ",", "7746.63`", ",", "7788.6`", 
     ",", "7746.63`", ",", "7778.09`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:00:00\"\>", ",", "7778.65`", ",", "7844.75`", 
     ",", "7778.65`", ",", "7842.76`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:10:00\"\>", ",", "7842.44`", ",", "7842.52`", 
     ",", "7812.34`", ",", "7819.35`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:20:00\"\>", ",", "7820.38`", ",", "7822.61`", 
     ",", "7802.22`", ",", "7808.35`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:30:00\"\>", ",", "7807.72`", ",", "7863.23`", 
     ",", "7807.72`", ",", "7859.57`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:40:00\"\>", ",", "7858.05`", ",", "7858.53`", 
     ",", "7830.74`", ",", "7835.67`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T10:50:00\"\>", ",", "7834.96`", ",", "7841.17`", 
     ",", "7814.96`", ",", "7815.2`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:00:00\"\>", ",", "7815.68`", ",", "7848.42`", 
     ",", "7815.36`", ",", "7841.17`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:10:00\"\>", ",", "7840.93`", ",", "7845.71`", 
     ",", "7804.85`", ",", "7811.06`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:20:00\"\>", ",", "7811.46`", ",", "7821.5`", 
     ",", "7800.79`", ",", "7818.63`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:30:00\"\>", ",", "7818.79`", ",", "7827.31`", 
     ",", "7804.37`", ",", "7816.88`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:40:00\"\>", ",", "7816.24`", ",", "7839.18`", 
     ",", "7806.12`", ",", "7834.16`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T11:50:00\"\>", ",", "7833.44`", ",", "7838.06`", 
     ",", "7817.19`", ",", "7817.35`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:00:00\"\>", ",", "7817.59`", ",", "7849.61`", 
     ",", "7811.86`", ",", "7840.05`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:10:00\"\>", ",", "7839.89`", ",", "7839.89`", 
     ",", "7808.51`", ",", "7808.67`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:20:00\"\>", ",", "7808.27`", ",", "7812.89`", 
     ",", "7788.84`", ",", "7800.15`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:30:00\"\>", ",", "7800.63`", ",", "7809.79`", 
     ",", "7787.49`", ",", "7798.56`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:40:00\"\>", ",", "7798.24`", ",", "7809.63`", 
     ",", "7790.59`", ",", "7791.71`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T12:50:00\"\>", ",", "7791.23`", ",", "7792.58`", 
     ",", "7777.29`", ",", "7792.03`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:00:00\"\>", ",", "7791.87`", ",", "7805.49`", 
     ",", "7781.03`", ",", "7781.11`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:10:00\"\>", ",", "7781.19`", ",", "7786.13`", 
     ",", "7747.74`", ",", "7749.18`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:20:00\"\>", ",", "7748.86`", ",", "7757.06`", 
     ",", "7717.79`", ",", "7726.64`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:30:00\"\>", ",", "7726.48`", ",", "7732.69`", 
     ",", "7709.51`", ",", "7709.59`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:40:00\"\>", ",", "7708.24`", ",", "7713.89`", 
     ",", "7685.06`", ",", "7692.86`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T13:50:00\"\>", ",", "7690.63`", ",", "7715.17`", 
     ",", "7688.4`", ",", "7688.64`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:00:00\"\>", ",", "7688.96`", ",", "7689.68`", 
     ",", "7609.47`", ",", "7619.03`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:10:00\"\>", ",", "7619.27`", ",", "7642.37`", 
     ",", "7618.39`", ",", "7634.48`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:20:00\"\>", ",", "7634.56`", ",", "7648.74`", 
     ",", "7620.07`", ",", "7620.07`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:30:00\"\>", ",", "7620.86`", ",", "7627.`", 
     ",", "7597.21`", ",", "7597.21`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:40:00\"\>", ",", "7596.97`", ",", "7606.05`", 
     ",", "7561.45`", ",", "7590.04`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T14:50:00\"\>", ",", "7587.89`", ",", "7602.86`", 
     ",", "7570.53`", ",", "7571.88`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:00:00\"\>", ",", "7571.4`", ",", "7589.88`", 
     ",", "7550.06`", ",", "7577.93`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:10:00\"\>", ",", "7577.85`", ",", "7611.46`", 
     ",", "7576.1`", ",", "7607.24`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:20:00\"\>", ",", "7607.96`", ",", "7637.83`", 
     ",", "7603.9`", ",", "7633.29`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:30:00\"\>", ",", "7634.48`", ",", "7651.21`", 
     ",", "7627.47`", ",", "7640.14`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:40:00\"\>", ",", "7639.9`", ",", "7675.66`", 
     ",", "7617.84`", ",", "7659.09`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T15:50:00\"\>", ",", "7658.38`", ",", "7765.82`", 
     ",", "7649.46`", ",", "7746.39`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-25T16:00:00\"\>", ",", "7746.55`", ",", "7750.53`", 
     ",", "7746.47`", ",", "7749.81`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T09:30:00\"\>", ",", "7752.28`", ",", "7834.56`", 
     ",", "7752.28`", ",", "7821.1`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T09:40:00\"\>", ",", "7822.77`", ",", "7854.39`", 
     ",", "7815.04`", ",", "7815.04`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T09:50:00\"\>", ",", "7815.36`", ",", "7820.94`", 
     ",", "7788.84`", ",", "7792.11`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:00:00\"\>", ",", "7791.95`", ",", "7816.96`", 
     ",", "7790.83`", ",", "7800.07`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:10:00\"\>", ",", "7799.43`", ",", "7799.43`", 
     ",", "7758.18`", ",", "7779.52`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:20:00\"\>", ",", "7780.`", ",", "7793.22`", 
     ",", "7761.2`", ",", "7762.88`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:30:00\"\>", ",", "7761.52`", ",", "7807.24`", 
     ",", "7755.55`", ",", "7797.52`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:40:00\"\>", ",", "7797.36`", ",", "7819.19`", 
     ",", "7791.79`", ",", "7792.11`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T10:50:00\"\>", ",", "7790.04`", ",", "7800.39`", 
     ",", "7778.17`", ",", "7799.99`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:00:00\"\>", ",", "7798.32`", ",", "7827.55`", 
     ",", "7792.66`", ",", "7815.68`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:10:00\"\>", ",", "7814.89`", ",", "7822.93`", 
     ",", "7804.61`", ",", "7807.96`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:20:00\"\>", ",", "7808.35`", ",", "7822.69`", 
     ",", "7791.71`", ",", "7792.58`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:30:00\"\>", ",", "7792.34`", ",", "7834.96`", 
     ",", "7791.15`", ",", "7824.12`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:40:00\"\>", ",", "7823.96`", ",", "7833.76`", 
     ",", "7815.52`", ",", "7819.43`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T11:50:00\"\>", ",", "7820.22`", ",", "7825.08`", 
     ",", "7789.32`", ",", "7811.7`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:00:00\"\>", ",", "7812.34`", ",", "7844.2`", 
     ",", "7808.75`", ",", "7841.41`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:10:00\"\>", ",", "7841.01`", ",", "7841.89`", 
     ",", "7818.15`", ",", "7840.77`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:20:00\"\>", ",", "7843.64`", ",", "7859.49`", 
     ",", "7843.24`", ",", "7859.49`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:30:00\"\>", ",", "7859.81`", ",", "7861.16`", 
     ",", "7832.81`", ",", "7848.89`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:40:00\"\>", ",", "7849.77`", ",", "7852.48`", 
     ",", "7815.44`", ",", "7826.67`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T12:50:00\"\>", ",", "7826.99`", ",", "7829.06`", 
     ",", "7810.11`", ",", "7822.53`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:00:00\"\>", ",", "7821.5`", ",", "7904.41`", 
     ",", "7821.5`", ",", "7900.9`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:10:00\"\>", ",", "7902.82`", ",", "7904.09`", 
     ",", "7883.94`", ",", "7892.7`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:20:00\"\>", ",", "7890.79`", ",", "7922.97`", 
     ",", "7890.79`", ",", "7903.37`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:30:00\"\>", ",", "7903.77`", ",", "7912.21`", 
     ",", "7886.49`", ",", "7890.31`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:40:00\"\>", ",", "7890.55`", ",", "7892.7`", 
     ",", "7878.84`", ",", "7881.87`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T13:50:00\"\>", ",", "7881.95`", ",", "7902.74`", 
     ",", "7876.13`", ",", "7883.94`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:00:00\"\>", ",", "7883.3`", ",", "7920.34`", 
     ",", "7883.3`", ",", "7910.46`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:10:00\"\>", ",", "7910.22`", ",", "7914.92`", 
     ",", "7886.65`", ",", "7891.35`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:20:00\"\>", ",", "7890.71`", ",", "7911.5`", 
     ",", "7869.84`", ",", "7870.56`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:30:00\"\>", ",", "7870.8`", ",", "7881.07`", 
     ",", "7854.55`", ",", "7868.65`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:40:00\"\>", ",", "7868.81`", ",", "7887.36`", 
     ",", "7858.37`", ",", "7873.35`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T14:50:00\"\>", ",", "7871.51`", ",", "7871.51`", 
     ",", "7821.1`", ",", "7821.1`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:00:00\"\>", ",", "7821.18`", ",", "7874.06`", 
     ",", "7820.46`", ",", "7862.83`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:10:00\"\>", ",", "7861.64`", ",", "7872.15`", 
     ",", "7835.51`", ",", "7847.62`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:20:00\"\>", ",", "7847.54`", ",", "7896.68`", 
     ",", "7844.67`", ",", "7892.06`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:30:00\"\>", ",", "7891.82`", ",", "7907.52`", 
     ",", "7880.83`", ",", "7900.11`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:40:00\"\>", ",", "7900.03`", ",", "7931.17`", 
     ",", "7900.03`", ",", "7909.51`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T15:50:00\"\>", ",", "7910.7`", ",", "7926.07`", 
     ",", "7881.55`", ",", "7921.21`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-26T16:00:00\"\>", ",", "7921.61`", ",", "7924.64`", 
     ",", "7921.29`", ",", "7924.56`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T09:30:00\"\>", ",", "7922.57`", ",", "7922.57`", 
     ",", "7791.55`", ",", "7805.09`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T09:40:00\"\>", ",", "7805.25`", ",", "7818.79`", 
     ",", "7793.3`", ",", "7808.75`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T09:50:00\"\>", ",", "7808.04`", ",", "7827.55`", 
     ",", "7799.04`", ",", "7808.83`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:00:00\"\>", ",", "7808.91`", ",", "7817.35`", 
     ",", "7771.8`", ",", "7771.8`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:10:00\"\>", ",", "7771.95`", ",", "7792.58`", 
     ",", "7771.64`", ",", "7775.22`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:20:00\"\>", ",", "7774.66`", ",", "7800.23`", 
     ",", "7774.5`", ",", "7798.48`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:30:00\"\>", ",", "7799.19`", ",", "7814.73`", 
     ",", "7780.96`", ",", "7808.83`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:40:00\"\>", ",", "7809.47`", ",", "7821.26`", 
     ",", "7798.16`", ",", "7798.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T10:50:00\"\>", ",", "7797.36`", ",", "7822.29`", 
     ",", "7797.36`", ",", "7803.58`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:00:00\"\>", ",", "7803.5`", ",", "7821.18`", 
     ",", "7792.27`", ",", "7809.95`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:10:00\"\>", ",", "7809.71`", ",", "7826.83`", 
     ",", "7801.27`", ",", "7810.98`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:20:00\"\>", ",", "7809.55`", ",", "7830.97`", 
     ",", "7807.16`", ",", "7810.19`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:30:00\"\>", ",", "7810.11`", ",", "7818.07`", 
     ",", "7800.07`", ",", "7808.35`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:40:00\"\>", ",", "7808.12`", ",", "7839.02`", 
     ",", "7805.25`", ",", "7829.86`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T11:50:00\"\>", ",", "7829.78`", ",", "7837.35`", 
     ",", "7819.11`", ",", "7823.25`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:00:00\"\>", ",", "7823.09`", ",", "7841.97`", 
     ",", "7819.19`", ",", "7841.41`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:10:00\"\>", ",", "7839.74`", ",", "7839.74`", 
     ",", "7808.35`", ",", "7815.04`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:20:00\"\>", ",", "7814.41`", ",", "7814.49`", 
     ",", "7799.43`", ",", "7800.47`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:30:00\"\>", ",", "7799.75`", ",", "7809.47`", 
     ",", "7795.53`", ",", "7796.88`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:40:00\"\>", ",", "7796.96`", ",", "7824.12`", 
     ",", "7796.96`", ",", "7811.06`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T12:50:00\"\>", ",", "7811.14`", ",", "7815.04`", 
     ",", "7784.3`", ",", "7789.96`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:00:00\"\>", ",", "7790.04`", ",", "7794.18`", 
     ",", "7777.69`", ",", "7781.67`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:10:00\"\>", ",", "7781.83`", ",", "7789.96`", 
     ",", "7768.85`", ",", "7781.83`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:20:00\"\>", ",", "7782.23`", ",", "7791.55`", 
     ",", "7779.76`", ",", "7789.4`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:30:00\"\>", ",", "7789.96`", ",", "7814.96`", 
     ",", "7780.72`", ",", "7785.18`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:40:00\"\>", ",", "7784.46`", ",", "7788.44`", 
     ",", "7747.42`", ",", "7764.79`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T13:50:00\"\>", ",", "7764.87`", ",", "7764.87`", 
     ",", "7739.7`", ",", "7752.68`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:00:00\"\>", ",", "7753.16`", ",", "7764.95`", 
     ",", "7736.27`", ",", "7760.72`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:10:00\"\>", ",", "7760.41`", ",", "7775.14`", 
     ",", "7754.51`", ",", "7774.82`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:20:00\"\>", ",", "7774.74`", ",", "7795.85`", 
     ",", "7769.88`", ",", "7781.67`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:30:00\"\>", ",", "7781.51`", ",", "7785.5`", 
     ",", "7764.07`", ",", "7773.63`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:40:00\"\>", ",", "7773.87`", ",", "7791.31`", 
     ",", "7755.55`", ",", "7767.97`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T14:50:00\"\>", ",", "7767.49`", ",", "7798.16`", 
     ",", "7757.22`", ",", "7797.52`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:00:00\"\>", ",", "7798.08`", ",", "7829.14`", 
     ",", "7798.08`", ",", "7808.27`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:10:00\"\>", ",", "7807.16`", ",", "7821.18`", 
     ",", "7794.18`", ",", "7813.05`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:20:00\"\>", ",", "7812.89`", ",", "7816.08`", 
     ",", "7785.18`", ",", "7791.63`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:30:00\"\>", ",", "7791.31`", ",", "7808.04`", 
     ",", "7766.22`", ",", "7769.09`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:40:00\"\>", ",", "7768.93`", ",", "7804.77`", 
     ",", "7760.8`", ",", "7797.52`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T15:50:00\"\>", ",", "7798.`", ",", "7810.5`", ",",
      "7747.98`", ",", "7776.73`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-27T16:00:00\"\>", ",", "7776.02`", ",", "7776.18`", 
     ",", "7775.3`", ",", "7776.18`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T09:30:00\"\>", ",", "7773.31`", ",", "7773.31`", 
     ",", "7571.8`", ",", "7576.74`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T09:40:00\"\>", ",", "7577.14`", ",", "7577.77`", 
     ",", "7543.37`", ",", "7545.6`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T09:50:00\"\>", ",", "7544.88`", ",", "7555.15`", 
     ",", "7533.49`", ",", "7553.8`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:00:00\"\>", ",", "7550.3`", ",", "7556.91`", 
     ",", "7534.21`", ",", "7556.67`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:10:00\"\>", ",", "7556.43`", ",", "7562.4`", 
     ",", "7536.75`", ",", "7540.26`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:20:00\"\>", ",", "7540.9`", ",", "7550.61`", 
     ",", "7529.27`", ",", "7544.8`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:30:00\"\>", ",", "7543.37`", ",", "7543.37`", 
     ",", "7501.87`", ",", "7510.87`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:40:00\"\>", ",", "7510.71`", ",", "7513.74`", 
     ",", "7497.09`", ",", "7506.89`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T10:50:00\"\>", ",", "7506.97`", ",", "7517.08`", 
     ",", "7505.21`", ",", "7510.23`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:00:00\"\>", ",", "7510.15`", ",", "7511.35`", 
     ",", "7487.13`", ",", "7494.46`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:10:00\"\>", ",", "7494.38`", ",", "7525.05`", 
     ",", "7494.38`", ",", "7517.16`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:20:00\"\>", ",", "7516.92`", ",", "7529.67`", 
     ",", "7492.79`", ",", "7492.95`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:30:00\"\>", ",", "7492.07`", ",", "7494.86`", 
     ",", "7475.67`", ",", "7492.15`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:40:00\"\>", ",", "7492.07`", ",", "7499.24`", 
     ",", "7483.15`", ",", "7499.24`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T11:50:00\"\>", ",", "7499.32`", ",", "7503.94`", 
     ",", "7483.95`", ",", "7503.62`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:00:00\"\>", ",", "7503.46`", ",", "7506.81`", 
     ",", "7497.09`", ",", "7500.75`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:10:00\"\>", ",", "7500.91`", ",", "7520.75`", 
     ",", "7492.39`", ",", "7520.75`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:20:00\"\>", ",", "7520.83`", ",", "7530.62`", 
     ",", "7512.06`", ",", "7523.77`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:30:00\"\>", ",", "7521.78`", ",", "7525.13`", 
     ",", "7509.91`", ",", "7519.23`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:40:00\"\>", ",", "7519.15`", ",", "7519.95`", 
     ",", "7507.44`", ",", "7515.01`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T12:50:00\"\>", ",", "7515.33`", ",", "7518.6`", 
     ",", "7495.98`", ",", "7497.89`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:00:00\"\>", ",", "7497.97`", ",", "7505.69`", 
     ",", "7491.75`", ",", "7496.21`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:10:00\"\>", ",", "7496.13`", ",", "7507.05`", 
     ",", "7491.04`", ",", "7497.49`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:20:00\"\>", ",", "7495.5`", ",", "7522.74`", 
     ",", "7493.03`", ",", "7514.45`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:30:00\"\>", ",", "7514.29`", ",", "7520.59`", 
     ",", "7485.46`", ",", "7493.82`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:40:00\"\>", ",", "7493.75`", ",", "7499.48`", 
     ",", "7465.87`", ",", "7475.03`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T13:50:00\"\>", ",", "7474.39`", ",", "7476.94`", 
     ",", "7466.43`", ",", "7470.01`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:00:00\"\>", ",", "7463.32`", ",", "7481.4`", 
     ",", "7462.92`", ",", "7474.95`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:10:00\"\>", ",", "7475.11`", ",", "7502.43`", 
     ",", "7474.71`", ",", "7494.38`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:20:00\"\>", ",", "7494.3`", ",", "7498.13`", 
     ",", "7473.99`", ",", "7480.13`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:30:00\"\>", ",", "7481.56`", ",", "7488.57`", 
     ",", "7467.78`", ",", "7467.78`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:40:00\"\>", ",", "7470.09`", ",", "7470.09`", 
     ",", "7443.97`", ",", "7458.62`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T14:50:00\"\>", ",", "7458.54`", ",", "7466.19`", 
     ",", "7437.75`", ",", "7438.71`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:00:00\"\>", ",", "7439.11`", ",", "7470.81`", 
     ",", "7438.95`", ",", "7469.93`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:10:00\"\>", ",", "7468.18`", ",", "7493.43`", 
     ",", "7466.51`", ",", "7472.4`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:20:00\"\>", ",", "7471.84`", ",", "7504.02`", 
     ",", "7471.76`", ",", "7481.08`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:30:00\"\>", ",", "7481.`", ",", "7510.79`", 
     ",", "7480.44`", ",", "7484.35`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:40:00\"\>", ",", "7484.27`", ",", "7499.72`", 
     ",", "7470.33`", ",", "7485.54`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T15:50:00\"\>", ",", "7486.34`", ",", "7535.`", 
     ",", "7485.14`", ",", "7527.36`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-30T16:00:00\"\>", ",", "7527.44`", ",", "7527.44`", 
     ",", "7522.02`", ",", "7522.02`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T09:30:00\"\>", ",", "7523.77`", ",", "7612.18`", 
     ",", "7523.77`", ",", "7590.84`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T09:40:00\"\>", ",", "7591.31`", ",", "7591.31`", 
     ",", "7562.8`", ",", "7572.76`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T09:50:00\"\>", ",", "7572.68`", ",", "7583.51`", 
     ",", "7556.59`", ",", "7576.26`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:00:00\"\>", ",", "7571.56`", ",", "7607.88`", 
     ",", "7560.09`", ",", "7601.91`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:10:00\"\>", ",", "7602.78`", ",", "7604.85`", 
     ",", "7573.23`", ",", "7590.92`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:20:00\"\>", ",", "7591.39`", ",", "7609.95`", 
     ",", "7581.36`", ",", "7609.95`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:30:00\"\>", ",", "7612.34`", ",", "7631.54`", 
     ",", "7612.34`", ",", "7616.72`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:40:00\"\>", ",", "7616.88`", ",", "7642.05`", 
     ",", "7612.58`", ",", "7629.86`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T10:50:00\"\>", ",", "7628.99`", ",", "7636.39`", 
     ",", "7604.22`", ",", "7611.7`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:00:00\"\>", ",", "7613.14`", ",", "7643.01`", 
     ",", "7613.14`", ",", "7643.01`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:10:00\"\>", ",", "7643.24`", ",", "7643.24`", 
     ",", "7628.91`", ",", "7637.11`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:20:00\"\>", ",", "7637.03`", ",", "7643.88`", 
     ",", "7628.11`", ",", "7635.44`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:30:00\"\>", ",", "7635.92`", ",", "7635.92`", 
     ",", "7618.87`", ",", "7622.85`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:40:00\"\>", ",", "7621.66`", ",", "7637.19`", 
     ",", "7611.86`", ",", "7636.55`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T11:50:00\"\>", ",", "7636.63`", ",", "7640.77`", 
     ",", "7623.73`", ",", "7630.18`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:00:00\"\>", ",", "7630.1`", ",", "7630.82`", 
     ",", "7606.21`", ",", "7612.34`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:10:00\"\>", ",", "7612.42`", ",", "7625.24`", 
     ",", "7610.11`", ",", "7620.54`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:20:00\"\>", ",", "7620.46`", ",", "7622.85`", 
     ",", "7596.57`", ",", "7603.82`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:30:00\"\>", ",", "7602.86`", ",", "7610.11`", 
     ",", "7595.22`", ",", "7609.31`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:40:00\"\>", ",", "7609.08`", ",", "7620.31`", 
     ",", "7609.08`", ",", "7617.92`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T12:50:00\"\>", ",", "7617.84`", ",", "7636.79`", 
     ",", "7616.72`", ",", "7635.68`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:00:00\"\>", ",", "7635.52`", ",", "7657.02`", 
     ",", "7634.8`", ",", "7656.7`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:10:00\"\>", ",", "7657.1`", ",", "7665.7`", 
     ",", "7655.83`", ",", "7658.46`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:20:00\"\>", ",", "7658.38`", ",", "7668.89`", 
     ",", "7655.43`", ",", "7659.41`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:30:00\"\>", ",", "7659.65`", ",", "7671.76`", 
     ",", "7657.1`", ",", "7665.86`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:40:00\"\>", ",", "7665.63`", ",", "7669.21`", 
     ",", "7657.9`", ",", "7665.07`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T13:50:00\"\>", ",", "7666.1`", ",", "7674.7`", 
     ",", "7648.42`", ",", "7671.84`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:00:00\"\>", ",", "7672.32`", ",", "7679.24`", 
     ",", "7665.55`", ",", "7677.41`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:10:00\"\>", ",", "7677.25`", ",", "7692.23`", 
     ",", "7674.07`", ",", "7690.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:20:00\"\>", ",", "7688.96`", ",", "7688.96`", 
     ",", "7669.29`", ",", "7679.24`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:30:00\"\>", ",", "7680.84`", ",", "7714.69`", 
     ",", "7680.84`", ",", "7714.69`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:40:00\"\>", ",", "7714.21`", ",", "7720.74`", 
     ",", "7706.8`", ",", "7712.06`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T14:50:00\"\>", ",", "7711.98`", ",", "7725.36`", 
     ",", "7701.39`", ",", "7701.71`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:00:00\"\>", ",", "7704.1`", ",", "7708.24`", 
     ",", "7684.26`", ",", "7693.34`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:10:00\"\>", ",", "7692.71`", ",", "7699.79`", 
     ",", "7680.12`", ",", "7698.2`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:20:00\"\>", ",", "7698.04`", ",", "7715.33`", 
     ",", "7688.01`", ",", "7715.33`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:30:00\"\>", ",", "7714.29`", ",", "7714.29`", 
     ",", "7649.62`", ",", "7678.29`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:40:00\"\>", ",", "7669.85`", ",", "7677.65`", 
     ",", "7601.03`", ",", "7601.03`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T15:50:00\"\>", ",", "7602.15`", ",", "7653.52`", 
     ",", "7594.26`", ",", "7598.4`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-03-31T16:00:00\"\>", ",", "7598.8`", ",", "7608.92`", 
     ",", "7598.8`", ",", "7608.92`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T09:30:00\"\>", ",", "7606.13`", ",", "7606.13`", 
     ",", "7484.11`", ",", "7514.69`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T09:40:00\"\>", ",", "7514.29`", ",", "7527.99`", 
     ",", "7503.7`", ",", "7503.78`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T09:50:00\"\>", ",", "7503.7`", ",", "7528.55`", 
     ",", "7502.27`", ",", "7527.83`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:00:00\"\>", ",", "7528.95`", ",", "7576.66`", 
     ",", "7528.95`", ",", "7570.76`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:10:00\"\>", ",", "7571.88`", ",", "7611.7`", 
     ",", "7571.88`", ",", "7611.23`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:20:00\"\>", ",", "7611.07`", ",", "7635.04`", 
     ",", "7607.64`", ",", "7619.11`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:30:00\"\>", ",", "7616.64`", ",", "7654.95`", 
     ",", "7607.56`", ",", "7654.79`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:40:00\"\>", ",", "7654.08`", ",", "7665.31`", 
     ",", "7644.84`", ",", "7645.55`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T10:50:00\"\>", ",", "7645.87`", ",", "7669.93`", 
     ",", "7644.6`", ",", "7651.53`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:00:00\"\>", ",", "7653.6`", ",", "7661.8`", 
     ",", "7643.96`", ",", "7647.15`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:10:00\"\>", ",", "7647.23`", ",", "7659.41`", 
     ",", "7633.93`", ",", "7647.39`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:20:00\"\>", ",", "7646.67`", ",", "7665.63`", 
     ",", "7640.77`", ",", "7663.32`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:30:00\"\>", ",", "7664.11`", ",", "7683.31`", 
     ",", "7661.8`", ",", "7676.62`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:40:00\"\>", ",", "7677.01`", ",", "7696.93`", 
     ",", "7676.7`", ",", "7683.15`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T11:50:00\"\>", ",", "7683.23`", ",", "7694.94`", 
     ",", "7674.55`", ",", "7692.07`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:00:00\"\>", ",", "7692.47`", ",", "7719.39`", 
     ",", "7692.47`", ",", "7714.85`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:10:00\"\>", ",", "7715.01`", ",", "7729.5`", 
     ",", "7715.01`", ",", "7715.88`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:20:00\"\>", ",", "7715.25`", ",", "7719.31`", 
     ",", "7708.79`", ",", "7712.54`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:30:00\"\>", ",", "7712.86`", ",", "7719.39`", 
     ",", "7694.06`", ",", "7700.51`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:40:00\"\>", ",", "7700.67`", ",", "7709.11`", 
     ",", "7693.34`", ",", "7706.17`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T12:50:00\"\>", ",", "7705.85`", ",", "7707.6`", 
     ",", "7679.09`", ",", "7683.07`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:00:00\"\>", ",", "7682.99`", ",", "7699.4`", 
     ",", "7673.35`", ",", "7675.82`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:10:00\"\>", ",", "7675.66`", ",", "7699.56`", 
     ",", "7669.69`", ",", "7692.86`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:20:00\"\>", ",", "7692.55`", ",", "7705.53`", 
     ",", "7691.11`", ",", "7697.72`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:30:00\"\>", ",", "7697.48`", ",", "7724.09`", 
     ",", "7683.47`", ",", "7722.49`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:40:00\"\>", ",", "7720.1`", ",", "7729.26`", 
     ",", "7711.98`", ",", "7714.53`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T13:50:00\"\>", ",", "7714.77`", ",", "7739.3`", 
     ",", "7710.55`", ",", "7737.07`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:00:00\"\>", ",", "7737.15`", ",", "7767.1`", 
     ",", "7735.08`", ",", "7765.18`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:10:00\"\>", ",", "7765.42`", ",", "7773.39`", 
     ",", "7758.34`", ",", "7761.12`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:20:00\"\>", ",", "7760.8`", ",", "7760.8`", 
     ",", "7742.49`", ",", "7747.74`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:30:00\"\>", ",", "7748.7`", ",", "7769.25`", 
     ",", "7748.62`", ",", "7760.33`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:40:00\"\>", ",", "7761.12`", ",", "7761.92`", 
     ",", "7736.83`", ",", "7750.93`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T14:50:00\"\>", ",", "7750.77`", ",", "7768.45`", 
     ",", "7748.62`", ",", "7755.79`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:00:00\"\>", ",", "7757.22`", ",", "7757.78`", 
     ",", "7711.98`", ",", "7716.04`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:10:00\"\>", ",", "7713.57`", ",", "7720.1`", 
     ",", "7684.34`", ",", "7714.13`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:20:00\"\>", ",", "7714.45`", ",", "7727.51`", 
     ",", "7703.94`", ",", "7722.49`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:30:00\"\>", ",", "7723.93`", ",", "7770.6`", 
     ",", "7703.06`", ",", "7766.06`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:40:00\"\>", ",", "7766.14`", ",", "7782.31`", 
     ",", "7765.11`", ",", "7773.39`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T15:50:00\"\>", ",", "7771.`", ",", "7789.`", ",", 
     "7745.27`", ",", "7749.18`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-01T16:00:00\"\>", ",", "7748.94`", ",", "7761.6`", 
     ",", "7748.94`", ",", "7761.6`", ",", "25", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T09:30:00\"\>", ",", "7763.99`", ",", "7933.72`", 
     ",", "7763.99`", ",", "7914.84`", ",", "298", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T09:40:00\"\>", ",", "7914.44`", ",", "7943.76`", 
     ",", "7898.44`", ",", "7939.45`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T09:50:00\"\>", ",", "7939.14`", ",", "7955.38`", 
     ",", "7919.3`", ",", "7955.3`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:00:00\"\>", ",", "7954.91`", ",", "7977.05`", 
     ",", "7954.91`", ",", "7968.92`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:10:00\"\>", ",", "7968.76`", ",", "7991.54`", 
     ",", "7955.62`", ",", "7978.4`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:20:00\"\>", ",", "7978.88`", ",", "7980.47`", 
     ",", "7935.71`", ",", "7940.01`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:30:00\"\>", ",", "7939.85`", ",", "7958.41`", 
     ",", "7934.28`", ",", "7953.23`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:40:00\"\>", ",", "7951.56`", ",", "7990.35`", 
     ",", "7938.58`", ",", "7988.68`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T10:50:00\"\>", ",", "7989.07`", ",", "7992.66`", 
     ",", "7972.83`", ",", "7987.64`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:00:00\"\>", ",", "7987.96`", ",", "8002.46`", 
     ",", "7984.22`", ",", "7994.97`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:10:00\"\>", ",", "7994.41`", ",", "8043.95`", 
     ",", "7993.61`", ",", "8042.12`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:20:00\"\>", ",", "8041.64`", ",", "8044.35`", 
     ",", "8030.57`", ",", "8036.86`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:30:00\"\>", ",", "8036.55`", ",", "8037.26`", 
     ",", "8018.07`", ",", "8025.55`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:40:00\"\>", ",", "8025.39`", ",", "8033.44`", 
     ",", "8013.21`", ",", "8015.76`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T11:50:00\"\>", ",", "8015.68`", ",", "8028.1`", 
     ",", "8007.87`", ",", "8024.52`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:00:00\"\>", ",", "8024.28`", ",", "8028.9`", 
     ",", "8011.3`", ",", "8028.18`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:10:00\"\>", ",", "8028.02`", ",", "8048.81`", 
     ",", "8025.71`", ",", "8045.78`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:20:00\"\>", ",", "8044.99`", ",", "8073.02`", 
     ",", "8044.67`", ",", "8060.92`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:30:00\"\>", ",", "8061.4`", ",", "8075.65`", 
     ",", "8061.4`", ",", "8066.09`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:40:00\"\>", ",", "8066.17`", ",", "8072.15`", 
     ",", "8061.24`", ",", "8065.7`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T12:50:00\"\>", ",", "8065.78`", ",", "8073.9`", 
     ",", "8046.02`", ",", "8046.02`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:00:00\"\>", ",", "8046.26`", ",", "8050.01`", 
     ",", "8028.82`", ",", "8037.34`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:10:00\"\>", ",", "8037.58`", ",", "8041.96`", 
     ",", "8031.61`", ",", "8037.1`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:20:00\"\>", ",", "8036.94`", ",", "8037.5`", 
     ",", "8014.72`", ",", "8024.84`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:30:00\"\>", ",", "8024.76`", ",", "8036.07`", 
     ",", "8003.49`", ",", "8006.2`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:40:00\"\>", ",", "8004.92`", ",", "8018.78`", 
     ",", "7992.98`", ",", "8007.71`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T13:50:00\"\>", ",", "8007.95`", ",", "8021.41`", 
     ",", "7992.26`", ",", "8005.64`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:00:00\"\>", ",", "8006.28`", ",", "8029.38`", 
     ",", "8004.21`", ",", "8010.82`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:10:00\"\>", ",", "8011.22`", ",", "8014.96`", 
     ",", "7998.08`", ",", "8005.4`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:20:00\"\>", ",", "8006.04`", ",", "8041.64`", 
     ",", "8006.04`", ",", "8037.5`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:30:00\"\>", ",", "8038.62`", ",", "8056.46`", 
     ",", "8038.62`", ",", "8054.94`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:40:00\"\>", ",", "8055.02`", ",", "8073.74`", 
     ",", "8052.63`", ",", "8066.01`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T14:50:00\"\>", ",", "8065.78`", ",", "8066.97`", 
     ",", "8029.46`", ",", "8036.39`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:00:00\"\>", ",", "8036.23`", ",", "8054.78`", 
     ",", "8005.`", ",", "8020.46`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:10:00\"\>", ",", "8021.65`", ",", "8043.79`", 
     ",", "8010.18`", ",", "8012.65`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:20:00\"\>", ",", "8011.93`", ",", "8043.95`", 
     ",", "8002.22`", ",", "8029.22`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:30:00\"\>", ",", "8030.49`", ",", "8043.95`", 
     ",", "8006.68`", ",", "8009.23`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:40:00\"\>", ",", "8007.23`", ",", "8031.77`", 
     ",", "7971.71`", ",", "7994.97`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T15:50:00\"\>", ",", "7993.06`", ",", "7993.61`", 
     ",", "7940.17`", ",", "7972.67`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-02T16:00:00\"\>", ",", "7972.91`", ",", "7978.08`", 
     ",", "7972.91`", ",", "7978.08`", ",", "27", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T09:30:00\"\>", ",", "7980.87`", ",", "7998.08`", 
     ",", "7956.9`", ",", "7971.87`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T09:40:00\"\>", ",", "7971.63`", ",", "7994.01`", 
     ",", "7944.23`", ",", "7952.36`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T09:50:00\"\>", ",", "7950.05`", ",", "7969.64`", 
     ",", "7940.33`", ",", "7963.59`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:00:00\"\>", ",", "7961.28`", ",", "7963.03`", 
     ",", "7914.44`", ",", "7934.36`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:10:00\"\>", ",", "7934.44`", ",", "7945.51`", 
     ",", "7925.6`", ",", "7926.87`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:20:00\"\>", ",", "7926.55`", ",", "7942.96`", 
     ",", "7920.02`", ",", "7935.95`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:30:00\"\>", ",", "7936.11`", ",", "7940.81`", 
     ",", "7910.46`", ",", "7920.42`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:40:00\"\>", ",", "7921.06`", ",", "7925.36`", 
     ",", "7897.24`", ",", "7906.48`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T10:50:00\"\>", ",", "7907.12`", ",", "7927.27`", 
     ",", "7906.4`", ",", "7919.94`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:00:00\"\>", ",", "7920.02`", ",", "7932.52`", 
     ",", "7906.24`", ",", "7921.77`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:10:00\"\>", ",", "7921.53`", ",", "7922.41`", 
     ",", "7911.34`", ",", "7921.37`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:20:00\"\>", ",", "7921.53`", ",", "7925.6`", 
     ",", "7906.`", ",", "7925.28`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:30:00\"\>", ",", "7925.52`", ",", "7946.3`", 
     ",", "7923.76`", ",", "7939.14`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:40:00\"\>", ",", "7939.3`", ",", "7953.15`", 
     ",", "7938.82`", ",", "7949.81`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T11:50:00\"\>", ",", "7950.05`", ",", "7961.52`", 
     ",", "7948.06`", ",", "7954.43`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:00:00\"\>", ",", "7953.71`", ",", "7968.45`", 
     ",", "7951.48`", ",", "7961.28`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:10:00\"\>", ",", "7960.96`", ",", "7981.03`", 
     ",", "7960.`", ",", "7971.47`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:20:00\"\>", ",", "7971.71`", ",", "7978.4`", 
     ",", "7963.91`", ",", "7978.16`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:30:00\"\>", ",", "7978.24`", ",", "7986.29`", 
     ",", "7966.38`", ",", "7971.79`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:40:00\"\>", ",", "7971.39`", ",", "7979.04`", 
     ",", "7955.46`", ",", "7957.06`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T12:50:00\"\>", ",", "7957.77`", ",", "7961.52`", 
     ",", "7950.37`", ",", "7955.94`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:00:00\"\>", ",", "7956.18`", ",", "7967.01`", 
     ",", "7942.8`", ",", "7944.71`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:10:00\"\>", ",", "7944.63`", ",", "7950.76`", 
     ",", "7935.55`", ",", "7948.53`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:20:00\"\>", ",", "7948.22`", ",", "7954.51`", 
     ",", "7928.94`", ",", "7954.43`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:30:00\"\>", ",", "7955.78`", ",", "7959.45`", 
     ",", "7942.72`", ",", "7942.72`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:40:00\"\>", ",", "7941.68`", ",", "7948.45`", 
     ",", "7928.3`", ",", "7946.7`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T13:50:00\"\>", ",", "7946.54`", ",", "7971.07`", 
     ",", "7943.2`", ",", "7970.92`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:00:00\"\>", ",", "7971.39`", ",", "7977.61`", 
     ",", "7957.61`", ",", "7958.41`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:10:00\"\>", ",", "7958.81`", ",", "7981.91`", 
     ",", "7957.38`", ",", "7979.76`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:20:00\"\>", ",", "7979.84`", ",", "8009.78`", 
     ",", "7974.66`", ",", "8009.39`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:30:00\"\>", ",", "8009.62`", ",", "8016.87`", 
     ",", "8003.73`", ",", "8008.27`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:40:00\"\>", ",", "8008.67`", ",", "8012.25`", 
     ",", "7980.95`", ",", "7982.07`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T14:50:00\"\>", ",", "7982.46`", ",", "7999.43`", 
     ",", "7979.04`", ",", "7995.45`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:00:00\"\>", ",", "7994.73`", ",", "7998.63`", 
     ",", "7961.84`", ",", "7967.49`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:10:00\"\>", ",", "7968.21`", ",", "7975.46`", 
     ",", "7950.76`", ",", "7966.93`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:20:00\"\>", ",", "7966.69`", ",", "7978.4`", 
     ",", "7951.`", ",", "7956.9`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:30:00\"\>", ",", "7955.38`", ",", "7970.76`", 
     ",", "7939.77`", ",", "7945.83`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:40:00\"\>", ",", "7944.39`", ",", "7990.51`", 
     ",", "7944.39`", ",", "7974.34`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T15:50:00\"\>", ",", "7975.93`", ",", "8018.86`", 
     ",", "7975.85`", ",", "8017.11`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-03T16:00:00\"\>", ",", "8017.27`", ",", "8018.15`", 
     ",", "8017.03`", ",", "8017.59`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T09:30:00\"\>", ",", "8016.`", ",", "8016.`", ",", 
     "7933.32`", ",", "7947.42`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T09:40:00\"\>", ",", "7947.74`", ",", "7950.61`", 
     ",", "7918.03`", ",", "7919.86`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T09:50:00\"\>", ",", "7920.26`", ",", "7943.52`", 
     ",", "7916.2`", ",", "7942.4`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:00:00\"\>", ",", "7940.89`", ",", "7970.92`", 
     ",", "7938.26`", ",", "7968.37`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:10:00\"\>", ",", "7968.53`", ",", "7969.`", 
     ",", "7935.55`", ",", "7943.12`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:20:00\"\>", ",", "7941.92`", ",", "7949.25`", 
     ",", "7910.62`", ",", "7912.69`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:30:00\"\>", ",", "7912.93`", ",", "7913.25`", 
     ",", "7904.65`", ",", "7909.9`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:40:00\"\>", ",", "7910.14`", ",", "7928.54`", 
     ",", "7906.96`", ",", "7924.08`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T10:50:00\"\>", ",", "7922.89`", ",", "7928.14`", 
     ",", "7913.73`", ",", "7917.71`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:00:00\"\>", ",", "7917.63`", ",", "7918.11`", 
     ",", "7889.52`", ",", "7897.88`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:10:00\"\>", ",", "7897.56`", ",", "7902.58`", 
     ",", "7888.96`", ",", "7901.14`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:20:00\"\>", ",", "7901.38`", ",", "7911.18`", 
     ",", "7897.88`", ",", "7907.2`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:30:00\"\>", ",", "7906.56`", ",", "7915.96`", 
     ",", "7899.79`", ",", "7904.41`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:40:00\"\>", ",", "7904.81`", ",", "7904.81`", 
     ",", "7888.72`", ",", "7890.87`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T11:50:00\"\>", ",", "7890.63`", ",", "7891.75`", 
     ",", "7875.74`", ",", "7882.03`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:00:00\"\>", ",", "7882.27`", ",", "7890.31`", 
     ",", "7879.48`", ",", "7888.48`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:10:00\"\>", ",", "7888.24`", ",", "7889.04`", 
     ",", "7869.2`", ",", "7869.44`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:20:00\"\>", ",", "7869.28`", ",", "7870.72`", 
     ",", "7862.36`", ",", "7870.24`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:30:00\"\>", ",", "7870.32`", ",", "7871.2`", 
     ",", "7863.23`", ",", "7866.82`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:40:00\"\>", ",", "7866.9`", ",", "7878.76`", 
     ",", "7866.74`", ",", "7874.3`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T12:50:00\"\>", ",", "7874.46`", ",", "7894.21`", 
     ",", "7871.99`", ",", "7894.21`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:00:00\"\>", ",", "7894.53`", ",", "7903.77`", 
     ",", "7888.56`", ",", "7888.88`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:10:00\"\>", ",", "7889.99`", ",", "7905.29`", 
     ",", "7886.49`", ",", "7889.12`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:20:00\"\>", ",", "7888.64`", ",", "7902.5`", 
     ",", "7884.9`", ",", "7885.77`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:30:00\"\>", ",", "7885.69`", ",", "7885.69`", 
     ",", "7869.52`", ",", "7877.17`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:40:00\"\>", ",", "7877.09`", ",", "7917.39`", 
     ",", "7875.98`", ",", "7915.32`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T13:50:00\"\>", ",", "7914.68`", ",", "7916.99`", 
     ",", "7906.32`", ",", "7911.34`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:00:00\"\>", ",", "7910.94`", ",", "7920.66`", 
     ",", "7906.88`", ",", "7918.91`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:10:00\"\>", ",", "7919.3`", ",", "7927.43`", 
     ",", "7898.2`", ",", "7908.55`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:20:00\"\>", ",", "7908.31`", ",", "7929.1`", 
     ",", "7902.98`", ",", "7920.66`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:30:00\"\>", ",", "7920.74`", ",", "7926.87`", 
     ",", "7893.26`", ",", "7897.32`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:40:00\"\>", ",", "7897.24`", ",", "7913.25`", 
     ",", "7889.52`", ",", "7907.67`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T14:50:00\"\>", ",", "7906.56`", ",", "7939.69`", 
     ",", "7905.84`", ",", "7924.08`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:00:00\"\>", ",", "7924.88`", ",", "7933.4`", 
     ",", "7911.9`", ",", "7920.1`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:10:00\"\>", ",", "7920.02`", ",", "7949.33`", 
     ",", "7906.72`", ",", "7947.58`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:20:00\"\>", ",", "7948.37`", ",", "7951.64`", 
     ",", "7919.14`", ",", "7926.31`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:30:00\"\>", ",", "7924.24`", ",", "7952.84`", 
     ",", "7919.22`", ",", "7919.22`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:40:00\"\>", ",", "7920.9`", ",", "7949.97`", 
     ",", "7920.9`", ",", "7939.85`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T15:50:00\"\>", ",", "7941.13`", ",", "7994.09`", 
     ",", "7941.13`", ",", "7972.35`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-06T16:00:00\"\>", ",", "7972.43`", ",", "7975.85`", 
     ",", "7972.43`", ",", "7975.85`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T09:30:00\"\>", ",", "7968.92`", ",", "7968.92`", 
     ",", "7820.46`", ",", "7831.85`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T09:40:00\"\>", ",", "7830.97`", ",", "7832.41`", 
     ",", "7810.5`", ",", "7824.04`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T09:50:00\"\>", ",", "7822.53`", ",", "7825.96`", 
     ",", "7808.12`", ",", "7817.43`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:00:00\"\>", ",", "7816.`", ",", "7823.57`", 
     ",", "7796.81`", ",", "7805.96`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:10:00\"\>", ",", "7806.76`", ",", "7814.73`", 
     ",", "7795.69`", ",", "7800.95`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:20:00\"\>", ",", "7799.83`", ",", "7830.5`", 
     ",", "7799.83`", ",", "7830.26`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:30:00\"\>", ",", "7830.58`", ",", "7849.29`", 
     ",", "7828.74`", ",", "7848.02`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:40:00\"\>", ",", "7848.18`", ",", "7864.59`", 
     ",", "7838.94`", ",", "7861.32`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T10:50:00\"\>", ",", "7860.68`", ",", "7861.`", 
     ",", "7847.14`", ",", "7848.26`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:00:00\"\>", ",", "7848.1`", ",", "7858.13`", 
     ",", "7834.`", ",", "7836.95`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:10:00\"\>", ",", "7836.87`", ",", "7842.2`", 
     ",", "7829.14`", ",", "7831.61`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:20:00\"\>", ",", "7831.37`", ",", "7843.08`", 
     ",", "7821.81`", ",", "7828.35`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:30:00\"\>", ",", "7826.91`", ",", "7846.43`", 
     ",", "7817.83`", ",", "7819.19`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:40:00\"\>", ",", "7819.03`", ",", "7819.5`", 
     ",", "7808.04`", ",", "7810.98`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T11:50:00\"\>", ",", "7811.3`", ",", "7818.07`", 
     ",", "7805.49`", ",", "7807.96`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:00:00\"\>", ",", "7808.83`", ",", "7820.38`", 
     ",", "7808.04`", ",", "7812.66`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:10:00\"\>", ",", "7813.13`", ",", "7836.63`", 
     ",", "7812.18`", ",", "7823.81`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:20:00\"\>", ",", "7824.04`", ",", "7828.27`", 
     ",", "7804.21`", ",", "7806.84`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:30:00\"\>", ",", "7806.92`", ",", "7817.04`", 
     ",", "7798.64`", ",", "7798.88`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:40:00\"\>", ",", "7800.07`", ",", "7818.79`", 
     ",", "7799.91`", ",", "7818.63`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T12:50:00\"\>", ",", "7818.71`", ",", "7824.76`", 
     ",", "7817.51`", ",", "7822.53`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:00:00\"\>", ",", "7822.45`", ",", "7825.32`", 
     ",", "7800.63`", ",", "7802.54`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:10:00\"\>", ",", "7802.46`", ",", "7803.1`", 
     ",", "7788.12`", ",", "7789.64`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:20:00\"\>", ",", "7789.24`", ",", "7791.07`", 
     ",", "7782.07`", ",", "7786.37`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:30:00\"\>", ",", "7785.42`", ",", "7785.42`", 
     ",", "7767.42`", ",", "7769.65`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:40:00\"\>", ",", "7769.41`", ",", "7781.51`", 
     ",", "7769.41`", ",", "7774.03`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T13:50:00\"\>", ",", "7772.67`", ",", "7788.28`", 
     ",", "7769.01`", ",", "7780.8`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:00:00\"\>", ",", "7781.03`", ",", "7781.03`", 
     ",", "7762.16`", ",", "7779.68`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:10:00\"\>", ",", "7779.44`", ",", "7794.34`", 
     ",", "7775.62`", ",", "7785.34`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:20:00\"\>", ",", "7785.5`", ",", "7789.4`", 
     ",", "7774.26`", ",", "7784.3`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:30:00\"\>", ",", "7785.1`", ",", "7785.1`", 
     ",", "7767.34`", ",", "7767.81`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:40:00\"\>", ",", "7767.65`", ",", "7779.28`", 
     ",", "7765.74`", ",", "7773.55`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T14:50:00\"\>", ",", "7773.95`", ",", "7796.41`", 
     ",", "7773.95`", ",", "7795.61`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:00:00\"\>", ",", "7795.29`", ",", "7797.04`", 
     ",", "7777.45`", ",", "7795.69`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:10:00\"\>", ",", "7795.93`", ",", "7825.56`", 
     ",", "7795.93`", ",", "7818.07`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:20:00\"\>", ",", "7817.19`", ",", "7820.54`", 
     ",", "7795.77`", ",", "7797.68`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:30:00\"\>", ",", "7798.`", ",", "7822.13`", 
     ",", "7796.96`", ",", "7814.96`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:40:00\"\>", ",", "7811.7`", ",", "7851.12`", 
     ",", "7807.4`", ",", "7826.59`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T15:50:00\"\>", ",", "7827.31`", ",", "7835.51`", 
     ",", "7783.19`", ",", "7784.54`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-07T16:00:00\"\>", ",", "7785.18`", ",", "7789.56`", 
     ",", "7785.18`", ",", "7789.56`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T09:30:00\"\>", ",", "7788.76`", ",", "7837.51`", 
     ",", "7788.76`", ",", "7823.57`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T09:40:00\"\>", ",", "7823.73`", ",", "7857.26`", 
     ",", "7813.05`", ",", "7846.74`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T09:50:00\"\>", ",", "7846.27`", ",", "7846.66`", 
     ",", "7828.66`", ",", "7833.92`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:00:00\"\>", ",", "7834.96`", ",", "7836.39`", 
     ",", "7816.88`", ",", "7819.19`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:10:00\"\>", ",", "7819.11`", ",", "7831.21`", 
     ",", "7808.91`", ",", "7809.71`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:20:00\"\>", ",", "7809.63`", ",", "7813.45`", 
     ",", "7780.72`", ",", "7787.49`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:30:00\"\>", ",", "7790.51`", ",", "7796.17`", 
     ",", "7772.83`", ",", "7780.24`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:40:00\"\>", ",", "7780.4`", ",", "7799.27`", 
     ",", "7779.84`", ",", "7781.11`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T10:50:00\"\>", ",", "7781.35`", ",", "7798.56`", 
     ",", "7780.16`", ",", "7798.56`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:00:00\"\>", ",", "7798.72`", ",", "7820.7`", 
     ",", "7798.48`", ",", "7815.2`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:10:00\"\>", ",", "7814.57`", ",", "7827.39`", 
     ",", "7808.43`", ",", "7816.08`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:20:00\"\>", ",", "7816.16`", ",", "7834.16`", 
     ",", "7816.16`", ",", "7817.35`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:30:00\"\>", ",", "7816.8`", ",", "7843.64`", 
     ",", "7811.3`", ",", "7839.02`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:40:00\"\>", ",", "7839.58`", ",", "7847.78`", 
     ",", "7829.54`", ",", "7840.85`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T11:50:00\"\>", ",", "7841.65`", ",", "7848.5`", 
     ",", "7831.13`", ",", "7831.69`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:00:00\"\>", ",", "7833.68`", ",", "7834.32`", 
     ",", "7823.81`", ",", "7825.48`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:10:00\"\>", ",", "7825.72`", ",", "7832.97`", 
     ",", "7813.69`", ",", "7824.44`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:20:00\"\>", ",", "7824.28`", ",", "7846.11`", 
     ",", "7822.85`", ",", "7836.71`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:30:00\"\>", ",", "7836.79`", ",", "7839.18`", 
     ",", "7831.05`", ",", "7834.56`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:40:00\"\>", ",", "7834.32`", ",", "7867.45`", 
     ",", "7834.32`", ",", "7864.03`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T12:50:00\"\>", ",", "7864.27`", ",", "7887.36`", 
     ",", "7861.4`", ",", "7875.5`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:00:00\"\>", ",", "7875.02`", ",", "7876.05`", 
     ",", "7868.41`", ",", "7868.41`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:10:00\"\>", ",", "7868.65`", ",", "7871.83`", 
     ",", "7854.63`", ",", "7854.63`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:20:00\"\>", ",", "7856.3`", ",", "7868.09`", 
     ",", "7854.95`", ",", "7862.28`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:30:00\"\>", ",", "7862.51`", ",", "7870.72`", 
     ",", "7850.01`", ",", "7851.76`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:40:00\"\>", ",", "7852.`", ",", "7861.32`", 
     ",", "7849.93`", ",", "7856.38`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T13:50:00\"\>", ",", "7856.3`", ",", "7858.29`", 
     ",", "7844.75`", ",", "7851.68`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:00:00\"\>", ",", "7851.92`", ",", "7853.59`", 
     ",", "7814.33`", ",", "7818.63`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:10:00\"\>", ",", "7818.47`", ",", "7828.82`", 
     ",", "7804.29`", ",", "7810.27`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:20:00\"\>", ",", "7810.74`", ",", "7815.92`", 
     ",", "7797.52`", ",", "7812.18`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:30:00\"\>", ",", "7811.86`", ",", "7816.32`", 
     ",", "7802.86`", ",", "7809.07`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:40:00\"\>", ",", "7808.91`", ",", "7823.49`", 
     ",", "7797.36`", ",", "7812.42`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T14:50:00\"\>", ",", "7812.1`", ",", "7812.1`", 
     ",", "7778.01`", ",", "7791.39`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:00:00\"\>", ",", "7791.23`", ",", "7791.23`", 
     ",", "7756.66`", ",", "7760.49`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:10:00\"\>", ",", "7759.77`", ",", "7777.61`", 
     ",", "7751.01`", ",", "7772.27`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:20:00\"\>", ",", "7772.75`", ",", "7793.62`", 
     ",", "7770.84`", ",", "7775.78`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:30:00\"\>", ",", "7776.73`", ",", "7813.77`", 
     ",", "7776.73`", ",", "7813.77`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:40:00\"\>", ",", "7813.61`", ",", "7830.66`", 
     ",", "7806.6`", ",", "7819.27`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T15:50:00\"\>", ",", "7820.22`", ",", "7837.51`", 
     ",", "7804.77`", ",", "7834.72`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-08T16:00:00\"\>", ",", "7834.96`", ",", "7837.58`", 
     ",", "7834.96`", ",", "7837.11`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T09:30:00\"\>", ",", "7839.97`", ",", "7997.68`", 
     ",", "7839.97`", ",", "7977.45`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T09:40:00\"\>", ",", "7977.92`", ",", "8002.62`", 
     ",", "7970.28`", ",", "7998.31`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T09:50:00\"\>", ",", "7999.75`", ",", "8037.66`", 
     ",", "7999.75`", ",", "8032.88`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:00:00\"\>", ",", "8033.2`", ",", "8044.03`", 
     ",", "8028.5`", ",", "8028.66`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:10:00\"\>", ",", "8028.18`", ",", "8035.59`", 
     ",", "8023.72`", ",", "8024.52`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:20:00\"\>", ",", "8024.44`", ",", "8042.28`", 
     ",", "8024.44`", ",", "8035.03`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:30:00\"\>", ",", "8035.67`", ",", "8047.14`", 
     ",", "8026.35`", ",", "8044.19`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:40:00\"\>", ",", "8043.55`", ",", "8043.79`", 
     ",", "8007.47`", ",", "8015.84`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T10:50:00\"\>", ",", "8015.36`", ",", "8023.96`", 
     ",", "8013.93`", ",", "8016.95`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:00:00\"\>", ",", "8017.27`", ",", "8020.7`", 
     ",", "8007.08`", ",", "8015.6`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:10:00\"\>", ",", "8016.95`", ",", "8027.78`", 
     ",", "8011.22`", ",", "8027.78`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:20:00\"\>", ",", "8027.15`", ",", "8030.33`", 
     ",", "7999.91`", ",", "8000.23`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:30:00\"\>", ",", "7999.59`", ",", "8006.28`", 
     ",", "7991.62`", ",", "7996.16`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:40:00\"\>", ",", "7996.32`", ",", "7998.47`", 
     ",", "7981.35`", ",", "7997.28`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T11:50:00\"\>", ",", "7996.56`", ",", "8008.19`", 
     ",", "7992.26`", ",", "8008.19`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:00:00\"\>", ",", "8008.35`", ",", "8011.38`", 
     ",", "7997.52`", ",", "8009.78`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:10:00\"\>", ",", "8010.02`", ",", "8021.49`", 
     ",", "7998.71`", ",", "8005.56`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:20:00\"\>", ",", "8005.72`", ",", "8020.14`", 
     ",", "8005.56`", ",", "8020.06`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:30:00\"\>", ",", "8019.9`", ",", "8025.95`", 
     ",", "8015.92`", ",", "8020.62`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:40:00\"\>", ",", "8020.3`", ",", "8027.23`", 
     ",", "8019.26`", ",", "8022.77`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T12:50:00\"\>", ",", "8021.81`", ",", "8024.52`", 
     ",", "8015.04`", ",", "8016.08`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:00:00\"\>", ",", "8016.`", ",", "8037.42`", 
     ",", "8012.81`", ",", "8036.78`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:10:00\"\>", ",", "8036.39`", ",", "8039.49`", 
     ",", "8031.69`", ",", "8036.23`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:20:00\"\>", ",", "8036.07`", ",", "8044.83`", 
     ",", "8025.79`", ",", "8029.06`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:30:00\"\>", ",", "8029.3`", ",", "8032.24`", 
     ",", "8021.49`", ",", "8025.31`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:40:00\"\>", ",", "8025.55`", ",", "8037.74`", 
     ",", "8020.93`", ",", "8036.62`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T13:50:00\"\>", ",", "8036.7`", ",", "8042.6`", 
     ",", "8032.56`", ",", "8041.72`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:00:00\"\>", ",", "8041.96`", ",", "8044.11`", 
     ",", "8031.85`", ",", "8043.95`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:10:00\"\>", ",", "8043.32`", ",", "8047.54`", 
     ",", "8010.02`", ",", "8019.26`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:20:00\"\>", ",", "8019.74`", ",", "8036.15`", 
     ",", "8018.78`", ",", "8029.06`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:30:00\"\>", ",", "8029.7`", ",", "8063.63`", 
     ",", "8029.22`", ",", "8062.27`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:40:00\"\>", ",", "8063.31`", ",", "8079.95`", 
     ",", "8055.82`", ",", "8072.86`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T14:50:00\"\>", ",", "8072.94`", ",", "8079.48`", 
     ",", "8059.88`", ",", "8064.58`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:00:00\"\>", ",", "8064.5`", ",", "8070.63`", 
     ",", "8059.72`", ",", "8069.44`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:10:00\"\>", ",", "8069.76`", ",", "8073.02`", 
     ",", "8050.72`", ",", "8070.71`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:20:00\"\>", ",", "8072.23`", ",", "8073.9`", 
     ",", "8053.67`", ",", "8061.79`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:30:00\"\>", ",", "8062.19`", ",", "8072.63`", 
     ",", "8050.09`", ",", "8067.29`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:40:00\"\>", ",", "8065.3`", ",", "8076.45`", 
     ",", "8046.42`", ",", "8065.22`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T15:50:00\"\>", ",", "8065.54`", ",", "8086.56`", 
     ",", "8054.15`", ",", "8086.56`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-09T16:00:00\"\>", ",", "8086.72`", ",", "8086.72`", 
     ",", "8083.22`", ",", "8083.38`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T09:30:00\"\>", ",", "8082.02`", ",", "8082.02`", 
     ",", "7994.33`", ",", "8002.22`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T09:40:00\"\>", ",", "8002.85`", ",", "8007.55`", 
     ",", "7975.69`", ",", "7989.39`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T09:50:00\"\>", ",", "7987.96`", ",", "7994.49`", 
     ",", "7969.`", ",", "7972.83`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:00:00\"\>", ",", "7972.91`", ",", "7997.92`", 
     ",", "7970.12`", ",", "7997.84`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:10:00\"\>", ",", "7996.88`", ",", "8022.37`", 
     ",", "7996.56`", ",", "8011.69`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:20:00\"\>", ",", "8012.09`", ",", "8024.12`", 
     ",", "8007.87`", ",", "8008.11`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:30:00\"\>", ",", "8008.43`", ",", "8009.54`", 
     ",", "7989.79`", ",", "7989.79`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:40:00\"\>", ",", "7988.92`", ",", "7993.77`", 
     ",", "7980.71`", ",", "7990.19`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T10:50:00\"\>", ",", "7991.7`", ",", "8002.38`", 
     ",", "7981.11`", ",", "7985.25`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:00:00\"\>", ",", "7985.41`", ",", "7985.41`", 
     ",", "7964.07`", ",", "7969.4`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:10:00\"\>", ",", "7969.`", ",", "7982.38`", 
     ",", "7968.92`", ",", "7978.56`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:20:00\"\>", ",", "7977.13`", ",", "8002.93`", 
     ",", "7975.06`", ",", "7996.96`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:30:00\"\>", ",", "7996.8`", ",", "8000.54`", 
     ",", "7986.53`", ",", "7987.88`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:40:00\"\>", ",", "7988.12`", ",", "7995.53`", 
     ",", "7976.01`", ",", "7987.8`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T11:50:00\"\>", ",", "7984.93`", ",", "7986.13`", 
     ",", "7968.05`", ",", "7982.15`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:00:00\"\>", ",", "7982.23`", ",", "8000.54`", 
     ",", "7980.63`", ",", "7994.65`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:10:00\"\>", ",", "7994.57`", ",", "8005.8`", 
     ",", "7993.3`", ",", "8003.65`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:20:00\"\>", ",", "8003.41`", ",", "8003.49`", 
     ",", "7991.94`", ",", "7998.23`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:30:00\"\>", ",", "7998.08`", ",", "8012.41`", 
     ",", "7997.44`", ",", "8006.36`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:40:00\"\>", ",", "8003.57`", ",", "8009.78`", 
     ",", "8000.07`", ",", "8004.45`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T12:50:00\"\>", ",", "8004.53`", ",", "8019.1`", 
     ",", "8003.65`", ",", "8016.63`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:00:00\"\>", ",", "8015.68`", ",", "8015.92`", 
     ",", "7997.44`", ",", "8000.86`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:10:00\"\>", ",", "8000.7`", ",", "8010.58`", 
     ",", "7997.44`", ",", "8010.58`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:20:00\"\>", ",", "8011.54`", ",", "8037.26`", 
     ",", "8011.54`", ",", "8032.96`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:30:00\"\>", ",", "8032.64`", ",", "8047.93`", 
     ",", "8032.64`", ",", "8036.23`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:40:00\"\>", ",", "8036.39`", ",", "8048.17`", 
     ",", "8033.6`", ",", "8041.24`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T13:50:00\"\>", ",", "8041.4`", ",", "8054.55`", 
     ",", "8041.16`", ",", "8053.67`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:00:00\"\>", ",", "8053.75`", ",", "8058.45`", 
     ",", "8039.73`", ",", "8041.4`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:10:00\"\>", ",", "8042.04`", ",", "8042.52`", 
     ",", "8026.43`", ",", "8036.07`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:20:00\"\>", ",", "8036.55`", ",", "8043.87`", 
     ",", "8032.96`", ",", "8037.34`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:30:00\"\>", ",", "8037.42`", ",", "8052.39`", 
     ",", "8035.91`", ",", "8040.61`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:40:00\"\>", ",", "8039.89`", ",", "8052.87`", 
     ",", "8026.83`", ",", "8028.5`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T14:50:00\"\>", ",", "8028.74`", ",", "8043.16`", 
     ",", "8024.44`", ",", "8025.71`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:00:00\"\>", ",", "8024.68`", ",", "8061.95`", 
     ",", "8013.13`", ",", "8050.56`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:10:00\"\>", ",", "8050.01`", ",", "8080.67`", 
     ",", "8050.01`", ",", "8076.05`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:20:00\"\>", ",", "8076.77`", ",", "8088.08`", 
     ",", "8071.27`", ",", "8081.94`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:30:00\"\>", ",", "8082.26`", ",", "8099.79`", 
     ",", "8076.29`", ",", "8094.85`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:40:00\"\>", ",", "8095.4`", ",", "8113.41`", 
     ",", "8084.81`", ",", "8086.96`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T15:50:00\"\>", ",", "8088.48`", ",", "8093.89`", 
     ",", "8047.78`", ",", "8047.86`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-13T16:00:00\"\>", ",", "8047.93`", ",", "8057.81`", 
     ",", "8047.93`", ",", "8057.81`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T09:30:00\"\>", ",", "8057.41`", ",", "8057.41`", 
     ",", "7949.73`", ",", "7964.07`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T09:40:00\"\>", ",", "7965.58`", ",", "7965.58`", 
     ",", "7924.8`", ",", "7941.21`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T09:50:00\"\>", ",", "7942.16`", ",", "7963.59`", 
     ",", "7940.25`", ",", "7960.`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:00:00\"\>", ",", "7958.17`", ",", "7972.35`", 
     ",", "7951.88`", ",", "7965.02`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:10:00\"\>", ",", "7963.75`", ",", "7981.59`", 
     ",", "7962.39`", ",", "7976.41`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:20:00\"\>", ",", "7976.33`", ",", "7995.69`", 
     ",", "7973.3`", ",", "7995.69`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:30:00\"\>", ",", "7995.84`", ",", "8019.66`", 
     ",", "7995.05`", ",", "8012.09`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:40:00\"\>", ",", "8011.93`", ",", "8028.66`", 
     ",", "8008.75`", ",", "8020.3`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T10:50:00\"\>", ",", "8020.93`", ",", "8020.93`", 
     ",", "8004.21`", ",", "8008.11`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:00:00\"\>", ",", "8007.79`", ",", "8014.24`", 
     ",", "7992.98`", ",", "7998.31`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:10:00\"\>", ",", "7997.6`", ",", "7999.35`", 
     ",", "7986.37`", ",", "7986.37`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:20:00\"\>", ",", "7986.05`", ",", "7987.96`", 
     ",", "7975.38`", ",", "7975.38`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:30:00\"\>", ",", "7975.06`", ",", "7995.21`", 
     ",", "7975.06`", ",", "7993.61`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:40:00\"\>", ",", "7994.81`", ",", "8003.33`", 
     ",", "7988.76`", ",", "7996.`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T11:50:00\"\>", ",", "7995.92`", ",", "7997.6`", 
     ",", "7979.76`", ",", "7983.74`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:00:00\"\>", ",", "7983.18`", ",", "7983.18`", 
     ",", "7974.02`", ",", "7974.02`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:10:00\"\>", ",", "7973.86`", ",", "7974.9`", 
     ",", "7954.67`", ",", "7965.5`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:20:00\"\>", ",", "7965.02`", ",", "7975.61`", 
     ",", "7957.06`", ",", "7970.6`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:30:00\"\>", ",", "7971.39`", ",", "7977.76`", 
     ",", "7969.16`", ",", "7970.04`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:40:00\"\>", ",", "7970.12`", ",", "7970.12`", 
     ",", "7911.82`", ",", "7917.55`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T12:50:00\"\>", ",", "7917.63`", ",", "7921.69`", 
     ",", "7905.21`", ",", "7916.6`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:00:00\"\>", ",", "7916.44`", ",", "7920.5`", 
     ",", "7902.66`", ",", "7909.59`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:10:00\"\>", ",", "7909.43`", ",", "7929.42`", 
     ",", "7903.69`", ",", "7923.68`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:20:00\"\>", ",", "7924.24`", ",", "7930.29`", 
     ",", "7915.24`", ",", "7930.29`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:30:00\"\>", ",", "7927.91`", ",", "7933.48`", 
     ",", "7918.43`", ",", "7918.43`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:40:00\"\>", ",", "7918.27`", ",", "7927.83`", 
     ",", "7910.46`", ",", "7915.8`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T13:50:00\"\>", ",", "7915.88`", ",", "7930.37`", 
     ",", "7911.9`", ",", "7929.26`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:00:00\"\>", ",", "7929.18`", ",", "7929.74`", 
     ",", "7906.24`", ",", "7911.9`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:10:00\"\>", ",", "7911.82`", ",", "7936.43`", 
     ",", "7903.69`", ",", "7935.79`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:20:00\"\>", ",", "7935.95`", ",", "7951.8`", 
     ",", "7927.83`", ",", "7950.92`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:30:00\"\>", ",", "7951.16`", ",", "7951.8`", 
     ",", "7936.59`", ",", "7948.69`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:40:00\"\>", ",", "7950.45`", ",", "7952.12`", 
     ",", "7931.57`", ",", "7938.34`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T14:50:00\"\>", ",", "7941.68`", ",", "7951.24`", 
     ",", "7921.85`", ",", "7951.24`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:00:00\"\>", ",", "7956.66`", ",", "7967.89`", 
     ",", "7944.87`", ",", "7946.3`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:10:00\"\>", ",", "7946.22`", ",", "7969.32`", 
     ",", "7940.09`", ",", "7968.61`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:20:00\"\>", ",", "7968.45`", ",", "7969.56`", 
     ",", "7934.52`", ",", "7943.04`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:30:00\"\>", ",", "7942.64`", ",", "7947.5`", 
     ",", "7925.99`", ",", "7926.87`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:40:00\"\>", ",", "7926.31`", ",", "7959.53`", 
     ",", "7920.9`", ",", "7940.65`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T15:50:00\"\>", ",", "7941.84`", ",", "7950.76`", 
     ",", "7910.94`", ",", "7919.54`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-14T16:00:00\"\>", ",", "7919.46`", ",", "7920.98`", 
     ",", "7919.46`", ",", "7920.18`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T09:30:00\"\>", ",", "7914.92`", ",", "7914.92`", 
     ",", "7871.12`", ",", "7885.93`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T09:40:00\"\>", ",", "7885.05`", ",", "7913.97`", 
     ",", "7874.06`", ",", "7904.81`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T09:50:00\"\>", ",", "7905.68`", ",", "7937.94`", 
     ",", "7904.57`", ",", "7936.59`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:00:00\"\>", ",", "7936.75`", ",", "7955.46`", 
     ",", "7932.29`", ",", "7955.46`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:10:00\"\>", ",", "7956.82`", ",", "7962.55`", 
     ",", "7939.77`", ",", "7943.36`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:20:00\"\>", ",", "7942.56`", ",", "7957.69`", 
     ",", "7942.56`", ",", "7949.01`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:30:00\"\>", ",", "7947.02`", ",", "7950.13`", 
     ",", "7929.66`", ",", "7940.73`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:40:00\"\>", ",", "7939.22`", ",", "7953.63`", 
     ",", "7934.99`", ",", "7939.53`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T10:50:00\"\>", ",", "7939.3`", ",", "7939.3`", 
     ",", "7920.18`", ",", "7922.09`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:00:00\"\>", ",", "7921.77`", ",", "7929.26`", 
     ",", "7911.74`", ",", "7917.31`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:10:00\"\>", ",", "7916.99`", ",", "7920.34`", 
     ",", "7900.75`", ",", "7920.18`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:20:00\"\>", ",", "7920.5`", ",", "7936.43`", 
     ",", "7920.5`", ",", "7928.22`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:30:00\"\>", ",", "7927.99`", ",", "7950.29`", 
     ",", "7927.83`", ",", "7944.07`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:40:00\"\>", ",", "7944.79`", ",", "7949.01`", 
     ",", "7933.88`", ",", "7942.56`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T11:50:00\"\>", ",", "7943.12`", ",", "7963.19`", 
     ",", "7939.93`", ",", "7957.38`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:00:00\"\>", ",", "7958.65`", ",", "7971.39`", 
     ",", "7956.82`", ",", "7967.57`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:10:00\"\>", ",", "7967.73`", ",", "7970.04`", 
     ",", "7955.46`", ",", "7961.6`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:20:00\"\>", ",", "7962.07`", ",", "7979.84`", 
     ",", "7958.25`", ",", "7966.77`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:30:00\"\>", ",", "7966.85`", ",", "7974.18`", 
     ",", "7964.54`", ",", "7972.19`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:40:00\"\>", ",", "7971.71`", ",", "7986.84`", 
     ",", "7966.06`", ",", "7981.51`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T12:50:00\"\>", ",", "7982.62`", ",", "7998.31`", 
     ",", "7982.62`", ",", "7997.12`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:00:00\"\>", ",", "8002.62`", ",", "8009.31`", 
     ",", "7976.17`", ",", "7984.38`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:10:00\"\>", ",", "7984.54`", ",", "7992.34`", 
     ",", "7979.04`", ",", "7979.04`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:20:00\"\>", ",", "7978.72`", ",", "7987.`", 
     ",", "7969.64`", ",", "7976.01`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:30:00\"\>", ",", "7975.77`", ",", "7979.04`", 
     ",", "7961.84`", ",", "7961.84`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:40:00\"\>", ",", "7961.6`", ",", "7963.19`", 
     ",", "7941.37`", ",", "7941.6`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T13:50:00\"\>", ",", "7941.53`", ",", "7953.07`", 
     ",", "7938.26`", ",", "7953.07`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:00:00\"\>", ",", "7953.39`", ",", "7979.2`", 
     ",", "7944.63`", ",", "7976.89`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:10:00\"\>", ",", "7977.13`", ",", "7978.96`", 
     ",", "7934.68`", ",", "7935.15`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:20:00\"\>", ",", "7934.76`", ",", "7938.58`", 
     ",", "7921.85`", ",", "7926.63`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:30:00\"\>", ",", "7925.28`", ",", "7940.09`", 
     ",", "7918.67`", ",", "7921.93`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:40:00\"\>", ",", "7922.01`", ",", "7935.15`", 
     ",", "7912.29`", ",", "7926.79`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T14:50:00\"\>", ",", "7927.19`", ",", "7932.84`", 
     ",", "7912.14`", ",", "7922.01`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:00:00\"\>", ",", "7922.17`", ",", "7946.86`", 
     ",", "7920.66`", ",", "7930.77`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:10:00\"\>", ",", "7931.17`", ",", "7975.06`", 
     ",", "7928.7`", ",", "7967.25`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:20:00\"\>", ",", "7969.56`", ",", "7974.42`", 
     ",", "7958.57`", ",", "7965.18`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:30:00\"\>", ",", "7965.5`", ",", "7978.4`", 
     ",", "7955.78`", ",", "7977.45`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:40:00\"\>", ",", "7977.69`", ",", "7999.35`", 
     ",", "7977.69`", ",", "7996.96`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T15:50:00\"\>", ",", "7996.8`", ",", "8041.24`", 
     ",", "7990.75`", ",", "8028.18`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-15T16:00:00\"\>", ",", "8028.1`", ",", "8029.93`", 
     ",", "8027.86`", ",", "8029.62`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T09:30:00\"\>", ",", "8029.14`", ",", "8073.26`", 
     ",", "8029.14`", ",", "8069.12`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T09:40:00\"\>", ",", "8067.21`", ",", "8067.21`", 
     ",", "8018.23`", ",", "8027.86`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T09:50:00\"\>", ",", "8027.47`", ",", "8048.97`", 
     ",", "8014.32`", ",", "8041.88`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:00:00\"\>", ",", "8043.71`", ",", "8051.76`", 
     ",", "7999.35`", ",", "8004.69`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:10:00\"\>", ",", "8005.88`", ",", "8023.24`", 
     ",", "7999.75`", ",", "8015.6`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:20:00\"\>", ",", "8016.95`", ",", "8030.33`", 
     ",", "8009.78`", ",", "8023.48`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:30:00\"\>", ",", "8023.8`", ",", "8036.47`", 
     ",", "8010.18`", ",", "8013.61`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:40:00\"\>", ",", "8013.77`", ",", "8013.77`", 
     ",", "7970.68`", ",", "7980.`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T10:50:00\"\>", ",", "7980.55`", ",", "7982.46`", 
     ",", "7962.79`", ",", "7978.`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:00:00\"\>", ",", "7977.92`", ",", "7994.49`", 
     ",", "7973.23`", ",", "7992.58`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:10:00\"\>", ",", "7992.74`", ",", "7996.88`", 
     ",", "7988.52`", ",", "7992.82`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:20:00\"\>", ",", "7992.74`", ",", "8005.64`", 
     ",", "7989.55`", ",", "8005.64`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:30:00\"\>", ",", "8006.04`", ",", "8019.58`", 
     ",", "7999.91`", ",", "8017.99`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:40:00\"\>", ",", "8018.07`", ",", "8030.09`", 
     ",", "8012.89`", ",", "8025.24`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T11:50:00\"\>", ",", "8025.31`", ",", "8033.12`", 
     ",", "8020.77`", ",", "8024.36`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:00:00\"\>", ",", "8024.28`", ",", "8024.28`", 
     ",", "8015.44`", ",", "8022.37`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:10:00\"\>", ",", "8022.77`", ",", "8037.9`", 
     ",", "8022.21`", ",", "8031.69`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:20:00\"\>", ",", "8031.93`", ",", "8033.44`", 
     ",", "8014.`", ",", "8021.25`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:30:00\"\>", ",", "8021.33`", ",", "8024.6`", 
     ",", "8016.79`", ",", "8017.99`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:40:00\"\>", ",", "8018.15`", ",", "8040.53`", 
     ",", "8014.8`", ",", "8040.13`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T12:50:00\"\>", ",", "8040.21`", ",", "8057.25`", 
     ",", "8038.78`", ",", "8045.86`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:00:00\"\>", ",", "8045.7`", ",", "8058.29`", 
     ",", "8045.7`", ",", "8050.24`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:10:00\"\>", ",", "8050.09`", ",", "8050.09`", 
     ",", "8031.85`", ",", "8038.7`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:20:00\"\>", ",", "8038.54`", ",", "8050.24`", 
     ",", "8038.54`", ",", "8046.58`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:30:00\"\>", ",", "8045.94`", ",", "8048.49`", 
     ",", "8019.1`", ",", "8019.74`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:40:00\"\>", ",", "8019.66`", ",", "8022.45`", 
     ",", "8011.77`", ",", "8011.93`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T13:50:00\"\>", ",", "8011.69`", ",", "8036.31`", 
     ",", "8008.91`", ",", "8031.93`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:00:00\"\>", ",", "8032.4`", ",", "8046.1`", 
     ",", "8032.32`", ",", "8042.12`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:10:00\"\>", ",", "8042.44`", ",", "8058.61`", 
     ",", "8042.04`", ",", "8051.44`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:20:00\"\>", ",", "8051.68`", ",", "8067.53`", 
     ",", "8051.12`", ",", "8066.89`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:30:00\"\>", ",", "8066.97`", ",", "8095.4`", 
     ",", "8063.47`", ",", "8091.5`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:40:00\"\>", ",", "8092.14`", ",", "8107.91`", 
     ",", "8091.34`", ",", "8101.22`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T14:50:00\"\>", ",", "8101.54`", ",", "8132.28`", 
     ",", "8100.42`", ",", "8122.96`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:00:00\"\>", ",", "8124.`", ",", "8143.19`", 
     ",", "8113.56`", ",", "8141.68`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:10:00\"\>", ",", "8141.6`", ",", "8151.72`", 
     ",", "8133.95`", ",", "8151.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:20:00\"\>", ",", "8151.8`", ",", "8161.91`", 
     ",", "8139.37`", ",", "8146.06`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:30:00\"\>", ",", "8146.22`", ",", "8162.31`", 
     ",", "8136.82`", ",", "8162.31`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:40:00\"\>", ",", "8162.23`", ",", "8167.8`", 
     ",", "8113.41`", ",", "8114.92`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T15:50:00\"\>", ",", "8115.64`", ",", "8145.34`", 
     ",", "8115.64`", ",", "8123.28`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-16T16:00:00\"\>", ",", "8123.36`", ",", "8125.43`", 
     ",", "8123.36`", ",", "8125.43`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T09:30:00\"\>", ",", "8124.`", ",", "8159.44`", 
     ",", "8124.`", ",", "8132.28`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T09:40:00\"\>", ",", "8130.37`", ",", "8143.83`", 
     ",", "8119.06`", ",", "8143.83`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T09:50:00\"\>", ",", "8143.67`", ",", "8159.44`", 
     ",", "8117.87`", ",", "8133.32`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:00:00\"\>", ",", "8133.64`", ",", "8152.51`", 
     ",", "8125.83`", ",", "8149.72`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:10:00\"\>", ",", "8149.57`", ",", "8149.88`", 
     ",", "8118.82`", ",", "8131.72`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:20:00\"\>", ",", "8131.8`", ",", "8135.79`", 
     ",", "8119.38`", ",", "8121.21`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:30:00\"\>", ",", "8120.25`", ",", "8120.65`", 
     ",", "8087.36`", ",", "8093.81`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:40:00\"\>", ",", "8094.13`", ",", "8117.07`", 
     ",", "8091.1`", ",", "8117.07`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T10:50:00\"\>", ",", "8116.99`", ",", "8117.39`", 
     ",", "8105.28`", ",", "8106.79`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:00:00\"\>", ",", "8106.87`", ",", "8114.12`", 
     ",", "8097.24`", ",", "8113.88`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:10:00\"\>", ",", "8114.12`", ",", "8119.62`", 
     ",", "8096.76`", ",", "8098.59`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:20:00\"\>", ",", "8097.71`", ",", "8117.87`", 
     ",", "8091.58`", ",", "8116.43`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:30:00\"\>", ",", "8115.95`", ",", "8136.74`", 
     ",", "8108.94`", ",", "8130.61`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:40:00\"\>", ",", "8130.37`", ",", "8141.92`", 
     ",", "8128.54`", ",", "8131.56`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T11:50:00\"\>", ",", "8131.64`", ",", "8144.39`", 
     ",", "8128.06`", ",", "8132.04`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:00:00\"\>", ",", "8132.44`", ",", "8142.48`", 
     ",", "8131.25`", ",", "8142.4`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:10:00\"\>", ",", "8142.8`", ",", "8146.38`", 
     ",", "8122.41`", ",", "8125.59`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:20:00\"\>", ",", "8126.07`", ",", "8139.21`", 
     ",", "8126.07`", ",", "8139.21`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:30:00\"\>", ",", "8139.29`", ",", "8139.29`", 
     ",", "8110.22`", ",", "8112.77`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:40:00\"\>", ",", "8113.01`", ",", "8141.76`", 
     ",", "8111.57`", ",", "8137.62`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T12:50:00\"\>", ",", "8137.46`", ",", "8141.76`", 
     ",", "8131.25`", ",", "8140.01`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:00:00\"\>", ",", "8140.33`", ",", "8156.89`", 
     ",", "8135.95`", ",", "8154.66`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:10:00\"\>", ",", "8154.9`", ",", "8154.9`", 
     ",", "8144.79`", ",", "8152.67`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:20:00\"\>", ",", "8152.51`", ",", "8163.66`", 
     ",", "8152.51`", ",", "8155.06`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:30:00\"\>", ",", "8155.14`", ",", "8156.26`", 
     ",", "8145.26`", ",", "8154.66`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:40:00\"\>", ",", "8155.78`", ",", "8167.72`", 
     ",", "8155.78`", ",", "8163.03`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T13:50:00\"\>", ",", "8163.11`", ",", "8174.42`", 
     ",", "8160.`", ",", "8170.83`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:00:00\"\>", ",", "8171.39`", ",", "8173.94`", 
     ",", "8153.63`", ",", "8155.94`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:10:00\"\>", ",", "8156.26`", ",", "8159.12`", 
     ",", "8149.41`", ",", "8159.12`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:20:00\"\>", ",", "8159.04`", ",", "8164.46`", 
     ",", "8154.98`", ",", "8164.22`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:30:00\"\>", ",", "8165.49`", ",", "8186.36`", 
     ",", "8165.49`", ",", "8185.65`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:40:00\"\>", ",", "8186.12`", ",", "8190.27`", 
     ",", "8162.87`", ",", "8162.87`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T14:50:00\"\>", ",", "8162.23`", ",", "8162.23`", 
     ",", "8139.85`", ",", "8143.67`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:00:00\"\>", ",", "8143.51`", ",", "8158.41`", 
     ",", "8141.68`", ",", "8154.26`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:10:00\"\>", ",", "8155.14`", ",", "8179.99`", 
     ",", "8155.14`", ",", "8158.09`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:20:00\"\>", ",", "8157.21`", ",", "8178.24`", 
     ",", "8152.59`", ",", "8169.`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:30:00\"\>", ",", "8169.56`", ",", "8181.98`", 
     ",", "8151.08`", ",", "8177.6`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:40:00\"\>", ",", "8177.84`", ",", "8190.58`", 
     ",", "8161.59`", ",", "8172.19`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T15:50:00\"\>", ",", "8172.03`", ",", "8176.73`", 
     ",", "8122.25`", ",", "8136.66`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-17T16:00:00\"\>", ",", "8136.74`", ",", "8136.9`", 
     ",", "8131.33`", ",", "8131.33`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T09:30:00\"\>", ",", "8128.94`", ",", "8128.94`", 
     ",", "7999.11`", ",", "8004.61`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T09:40:00\"\>", ",", "8004.92`", ",", "8008.83`", 
     ",", "7953.31`", ",", "7960.8`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T09:50:00\"\>", ",", "7961.92`", ",", "7961.92`", 
     ",", "7946.7`", ",", "7954.03`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:00:00\"\>", ",", "7953.07`", ",", "7953.47`", 
     ",", "7937.3`", ",", "7944.31`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:10:00\"\>", ",", "7944.87`", ",", "7945.67`", 
     ",", "7919.78`", ",", "7920.18`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:20:00\"\>", ",", "7916.28`", ",", "7933.56`", 
     ",", "7914.92`", ",", "7924.08`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:30:00\"\>", ",", "7923.52`", ",", "7926.55`", 
     ",", "7908.55`", ",", "7919.86`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:40:00\"\>", ",", "7920.1`", ",", "7922.57`", 
     ",", "7893.34`", ",", "7904.73`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T10:50:00\"\>", ",", "7904.65`", ",", "7910.22`", 
     ",", "7897.72`", ",", "7904.25`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:00:00\"\>", ",", "7903.85`", ",", "7907.83`", 
     ",", "7890.95`", ",", "7905.92`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:10:00\"\>", ",", "7905.68`", ",", "7911.74`", 
     ",", "7897.24`", ",", "7911.74`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:20:00\"\>", ",", "7911.82`", ",", "7923.84`", 
     ",", "7911.26`", ",", "7917.07`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:30:00\"\>", ",", "7917.55`", ",", "7921.85`", 
     ",", "7908.95`", ",", "7910.22`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:40:00\"\>", ",", "7910.38`", ",", "7914.05`", 
     ",", "7895.97`", ",", "7905.29`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T11:50:00\"\>", ",", "7904.97`", ",", "7919.3`", 
     ",", "7898.52`", ",", "7902.02`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:00:00\"\>", ",", "7902.1`", ",", "7905.52`", 
     ",", "7891.11`", ",", "7899.55`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:10:00\"\>", ",", "7899.47`", ",", "7912.37`", 
     ",", "7893.02`", ",", "7902.1`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:20:00\"\>", ",", "7901.7`", ",", "7911.5`", 
     ",", "7901.22`", ",", "7907.2`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:30:00\"\>", ",", "7906.48`", ",", "7909.98`", 
     ",", "7897.56`", ",", "7898.6`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:40:00\"\>", ",", "7898.83`", ",", "7909.03`", 
     ",", "7898.67`", ",", "7908.55`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T12:50:00\"\>", ",", "7908.47`", ",", "7920.1`", 
     ",", "7902.58`", ",", "7916.6`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:00:00\"\>", ",", "7916.2`", ",", "7916.2`", 
     ",", "7882.51`", ",", "7882.51`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:10:00\"\>", ",", "7883.06`", ",", "7884.58`", 
     ",", "7875.5`", ",", "7877.73`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:20:00\"\>", ",", "7877.65`", ",", "7883.54`", 
     ",", "7875.5`", ",", "7877.41`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:30:00\"\>", ",", "7877.33`", ",", "7888.48`", 
     ",", "7873.98`", ",", "7888.08`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:40:00\"\>", ",", "7888.48`", ",", "7893.58`", 
     ",", "7884.42`", ",", "7885.53`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T13:50:00\"\>", ",", "7885.85`", ",", "7898.2`", 
     ",", "7880.59`", ",", "7892.94`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:00:00\"\>", ",", "7890.63`", ",", "7890.63`", 
     ",", "7877.57`", ",", "7879.32`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:10:00\"\>", ",", "7879.64`", ",", "7879.88`", 
     ",", "7868.81`", ",", "7876.37`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:20:00\"\>", ",", "7876.21`", ",", "7879.`", 
     ",", "7863.39`", ",", "7874.3`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:30:00\"\>", ",", "7874.78`", ",", "7876.05`", 
     ",", "7856.38`", ",", "7861.8`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:40:00\"\>", ",", "7861.4`", ",", "7880.51`", 
     ",", "7860.76`", ",", "7872.95`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T14:50:00\"\>", ",", "7873.51`", ",", "7876.29`", 
     ",", "7844.04`", ",", "7848.02`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:00:00\"\>", ",", "7847.86`", ",", "7863.95`", 
     ",", "7846.03`", ",", "7862.36`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:10:00\"\>", ",", "7862.43`", ",", "7866.5`", 
     ",", "7850.65`", ",", "7852.24`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:20:00\"\>", ",", "7852.48`", ",", "7857.5`", 
     ",", "7840.61`", ",", "7849.21`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:30:00\"\>", ",", "7849.69`", ",", "7879.88`", 
     ",", "7849.69`", ",", "7850.33`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:40:00\"\>", ",", "7850.09`", ",", "7879.`", 
     ",", "7845.55`", ",", "7869.68`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T15:50:00\"\>", ",", "7870.56`", ",", "7871.51`", 
     ",", "7842.68`", ",", "7844.28`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-20T16:00:00\"\>", ",", "7843.4`", ",", "7844.35`", 
     ",", "7841.65`", ",", "7841.73`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T09:30:00\"\>", ",", "7841.81`", ",", "7841.81`", 
     ",", "7792.66`", ",", "7824.44`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T09:40:00\"\>", ",", "7825.88`", ",", "7845.79`", 
     ",", "7824.28`", ",", "7825.56`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T09:50:00\"\>", ",", "7826.35`", ",", "7849.53`", 
     ",", "7826.35`", ",", "7836.79`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:00:00\"\>", ",", "7836.55`", ",", "7841.41`", 
     ",", "7811.86`", ",", "7832.17`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:10:00\"\>", ",", "7832.49`", ",", "7837.19`", 
     ",", "7810.03`", ",", "7811.14`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:20:00\"\>", ",", "7811.22`", ",", "7814.25`", 
     ",", "7798.48`", ",", "7811.62`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:30:00\"\>", ",", "7811.54`", ",", "7815.36`", 
     ",", "7795.37`", ",", "7809.39`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:40:00\"\>", ",", "7808.75`", ",", "7827.47`", 
     ",", "7800.31`", ",", "7824.28`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T10:50:00\"\>", ",", "7823.89`", ",", "7841.97`", 
     ",", "7819.66`", ",", "7841.81`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:00:00\"\>", ",", "7842.36`", ",", "7861.48`", 
     ",", "7842.36`", ",", "7850.09`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:10:00\"\>", ",", "7849.53`", ",", "7884.58`", 
     ",", "7849.53`", ",", "7884.58`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:20:00\"\>", ",", "7885.77`", ",", "7900.27`", 
     ",", "7884.26`", ",", "7889.59`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:30:00\"\>", ",", "7889.67`", ",", "7896.76`", 
     ",", "7884.82`", ",", "7886.89`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:40:00\"\>", ",", "7887.6`", ",", "7923.45`", 
     ",", "7884.98`", ",", "7921.61`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T11:50:00\"\>", ",", "7921.14`", ",", "7923.13`", 
     ",", "7904.89`", ",", "7914.37`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:00:00\"\>", ",", "7915.48`", ",", "7922.17`", 
     ",", "7909.19`", ",", "7910.46`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:10:00\"\>", ",", "7910.22`", ",", "7912.45`", 
     ",", "7900.9`", ",", "7911.42`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:20:00\"\>", ",", "7911.82`", ",", "7918.75`", 
     ",", "7903.29`", ",", "7906.88`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:30:00\"\>", ",", "7907.28`", ",", "7927.03`", 
     ",", "7907.04`", ",", "7914.21`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:40:00\"\>", ",", "7914.84`", ",", "7930.77`", 
     ",", "7913.81`", ",", "7925.83`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T12:50:00\"\>", ",", "7925.68`", ",", "7932.21`", 
     ",", "7907.2`", ",", "7911.26`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:00:00\"\>", ",", "7911.18`", ",", "7913.09`", 
     ",", "7903.21`", ",", "7908.31`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:10:00\"\>", ",", "7908.23`", ",", "7922.65`", 
     ",", "7897.08`", ",", "7898.44`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:20:00\"\>", ",", "7898.28`", ",", "7910.06`", 
     ",", "7896.92`", ",", "7904.41`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:30:00\"\>", ",", "7905.21`", ",", "7912.06`", 
     ",", "7894.21`", ",", "7905.76`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:40:00\"\>", ",", "7905.05`", ",", "7924.64`", 
     ",", "7904.81`", ",", "7913.81`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T13:50:00\"\>", ",", "7913.41`", ",", "7918.43`", 
     ",", "7904.17`", ",", "7912.85`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:00:00\"\>", ",", "7913.65`", ",", "7923.52`", 
     ",", "7898.28`", ",", "7908.15`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:10:00\"\>", ",", "7908.63`", ",", "7912.53`", 
     ",", "7888.4`", ",", "7892.14`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:20:00\"\>", ",", "7892.3`", ",", "7916.99`", 
     ",", "7891.98`", ",", "7916.68`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:30:00\"\>", ",", "7922.01`", ",", "7937.3`", 
     ",", "7917.07`", ",", "7931.73`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:40:00\"\>", ",", "7934.91`", ",", "7944.47`", 
     ",", "7925.75`", ",", "7930.85`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T14:50:00\"\>", ",", "7931.01`", ",", "7936.83`", 
     ",", "7916.83`", ",", "7918.51`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:00:00\"\>", ",", "7919.46`", ",", "7952.76`", 
     ",", "7913.41`", ",", "7952.76`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:10:00\"\>", ",", "7952.68`", ",", "7978.56`", 
     ",", "7952.68`", ",", "7976.89`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:20:00\"\>", ",", "7976.81`", ",", "7979.04`", 
     ",", "7953.63`", ",", "7955.14`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:30:00\"\>", ",", "7954.99`", ",", "7960.88`", 
     ",", "7941.68`", ",", "7960.88`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:40:00\"\>", ",", "7958.97`", ",", "7963.91`", 
     ",", "7927.19`", ",", "7928.86`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T15:50:00\"\>", ",", "7929.9`", ",", "7966.77`", 
     ",", "7929.9`", ",", "7966.06`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-21T16:00:00\"\>", ",", "7965.98`", ",", "7969.56`", 
     ",", "7965.98`", ",", "7969.56`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T09:30:00\"\>", ",", "7964.78`", ",", "7964.78`", 
     ",", "7894.93`", ",", "7930.45`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T09:40:00\"\>", ",", "7931.01`", ",", "7934.91`", 
     ",", "7899.71`", ",", "7905.21`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T09:50:00\"\>", ",", "7903.53`", ",", "7934.28`", 
     ",", "7895.81`", ",", "7929.66`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:00:00\"\>", ",", "7929.9`", ",", "7976.57`", 
     ",", "7929.9`", ",", "7976.57`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:10:00\"\>", ",", "7976.33`", ",", "8010.74`", 
     ",", "7963.59`", ",", "8002.62`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:20:00\"\>", ",", "8006.12`", ",", "8039.73`", 
     ",", "8004.85`", ",", "8038.62`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:30:00\"\>", ",", "8035.75`", ",", "8044.27`", 
     ",", "8016.47`", ",", "8026.83`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:40:00\"\>", ",", "8024.52`", ",", "8033.68`", 
     ",", "8006.84`", ",", "8016.39`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T10:50:00\"\>", ",", "8016.31`", ",", "8034.79`", 
     ",", "8002.93`", ",", "8006.6`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:00:00\"\>", ",", "8006.44`", ",", "8014.24`", 
     ",", "7997.92`", ",", "8009.46`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:10:00\"\>", ",", "8009.39`", ",", "8018.31`", 
     ",", "7998.31`", ",", "8014.`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:20:00\"\>", ",", "8014.4`", ",", "8021.65`", 
     ",", "8009.78`", ",", "8021.65`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:30:00\"\>", ",", "8022.29`", ",", "8036.15`", 
     ",", "8019.66`", ",", "8032.32`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:40:00\"\>", ",", "8032.72`", ",", "8042.68`", 
     ",", "8026.59`", ",", "8032.72`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T11:50:00\"\>", ",", "8031.93`", ",", "8041.48`", 
     ",", "8025.71`", ",", "8039.01`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:00:00\"\>", ",", "8039.17`", ",", "8040.29`", 
     ",", "8019.02`", ",", "8029.22`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:10:00\"\>", ",", "8027.86`", ",", "8030.81`", 
     ",", "8012.17`", ",", "8012.17`", ",", "207", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:20:00\"\>", ",", "8012.81`", ",", "8013.21`", 
     ",", "7982.94`", ",", "7983.18`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:30:00\"\>", ",", "7981.27`", ",", "7985.09`", 
     ",", "7970.92`", ",", "7979.12`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:40:00\"\>", ",", "7979.04`", ",", "7979.92`", 
     ",", "7943.28`", ",", "7948.77`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T12:50:00\"\>", ",", "7949.25`", ",", "7956.02`", 
     ",", "7936.43`", ",", "7949.73`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:00:00\"\>", ",", "7949.49`", ",", "7959.29`", 
     ",", "7942.32`", ",", "7953.23`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:10:00\"\>", ",", "7951.88`", ",", "7966.53`", 
     ",", "7933.72`", ",", "7957.93`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:20:00\"\>", ",", "7958.25`", ",", "7976.65`", 
     ",", "7958.25`", ",", "7972.19`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:30:00\"\>", ",", "7972.43`", ",", "7996.64`", 
     ",", "7964.7`", ",", "7990.51`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:40:00\"\>", ",", "7989.47`", ",", "7993.38`", 
     ",", "7977.21`", ",", "7984.06`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T13:50:00\"\>", ",", "7983.66`", ",", "7987.96`", 
     ",", "7955.62`", ",", "7956.66`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:00:00\"\>", ",", "7956.1`", ",", "7974.74`", 
     ",", "7953.23`", ",", "7968.84`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:10:00\"\>", ",", "7968.76`", ",", "7998.95`", 
     ",", "7966.69`", ",", "7997.84`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:20:00\"\>", ",", "7998.39`", ",", "8009.62`", 
     ",", "7996.72`", ",", "8004.85`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:30:00\"\>", ",", "8006.52`", ",", "8037.34`", 
     ",", "8006.52`", ",", "8032.56`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:40:00\"\>", ",", "8032.72`", ",", "8035.75`", 
     ",", "8025.55`", ",", "8027.15`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T14:50:00\"\>", ",", "8027.86`", ",", "8035.91`", 
     ",", "8004.29`", ",", "8007.47`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:00:00\"\>", ",", "8007.95`", ",", "8008.35`", 
     ",", "7995.13`", ",", "8006.52`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:10:00\"\>", ",", "8007.55`", ",", "8039.89`", 
     ",", "7990.99`", ",", "7996.24`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:20:00\"\>", ",", "7996.`", ",", "8007.79`", 
     ",", "7984.93`", ",", "7997.12`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:30:00\"\>", ",", "7994.25`", ",", "8006.12`", 
     ",", "7948.06`", ",", "7953.63`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:40:00\"\>", ",", "7951.96`", ",", "7959.84`", 
     ",", "7902.18`", ",", "7916.04`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T15:50:00\"\>", ",", "7916.44`", ",", "7928.46`", 
     ",", "7868.25`", ",", "7885.37`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-22T16:00:00\"\>", ",", "7884.1`", ",", "7886.57`", 
     ",", "7883.3`", ",", "7886.57`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T09:30:00\"\>", ",", "7886.97`", ",", "7908.23`", 
     ",", "7868.73`", ",", "7897.72`", ",", "299", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T09:40:00\"\>", ",", "7894.06`", ",", "7911.5`", 
     ",", "7887.44`", ",", "7899.47`", ",", "298", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T09:50:00\"\>", ",", "7900.19`", ",", "7905.6`", 
     ",", "7857.74`", ",", "7860.36`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:00:00\"\>", ",", "7852.48`", ",", "7852.88`", 
     ",", "7816.72`", ",", "7832.33`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:10:00\"\>", ",", "7834.08`", ",", "7857.34`", 
     ",", "7834.08`", ",", "7852.16`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:20:00\"\>", ",", "7848.82`", ",", "7884.9`", 
     ",", "7841.09`", ",", "7882.51`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:30:00\"\>", ",", "7883.06`", ",", "7899.47`", 
     ",", "7867.77`", ",", "7891.19`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:40:00\"\>", ",", "7892.78`", ",", "7897.32`", 
     ",", "7849.61`", ",", "7856.62`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T10:50:00\"\>", ",", "7856.54`", ",", "7861.88`", 
     ",", "7839.74`", ",", "7839.74`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:00:00\"\>", ",", "7839.34`", ",", "7853.04`", 
     ",", "7829.54`", ",", "7841.49`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:10:00\"\>", ",", "7839.89`", ",", "7846.19`", 
     ",", "7807.72`", ",", "7807.96`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:20:00\"\>", ",", "7808.27`", ",", "7833.76`", 
     ",", "7807.64`", ",", "7824.2`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:30:00\"\>", ",", "7824.12`", ",", "7834.`", 
     ",", "7814.25`", ",", "7826.67`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:40:00\"\>", ",", "7826.43`", ",", "7831.05`", 
     ",", "7818.55`", ",", "7823.01`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T11:50:00\"\>", ",", "7823.41`", ",", "7830.02`", 
     ",", "7804.69`", ",", "7812.89`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:00:00\"\>", ",", "7812.5`", ",", "7822.21`", 
     ",", "7809.39`", ",", "7817.59`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:10:00\"\>", ",", "7817.83`", ",", "7846.51`", 
     ",", "7817.83`", ",", "7846.03`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:20:00\"\>", ",", "7845.79`", ",", "7850.49`", 
     ",", "7837.03`", ",", "7843.`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:30:00\"\>", ",", "7842.76`", ",", "7848.34`", 
     ",", "7828.27`", ",", "7848.34`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:40:00\"\>", ",", "7848.42`", ",", "7867.21`", 
     ",", "7847.46`", ",", "7858.85`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T12:50:00\"\>", ",", "7859.73`", ",", "7873.59`", 
     ",", "7859.49`", ",", "7866.58`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:00:00\"\>", ",", "7866.82`", ",", "7891.27`", 
     ",", "7866.82`", ",", "7886.41`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:10:00\"\>", ",", "7886.89`", ",", "7889.44`", 
     ",", "7871.67`", ",", "7886.41`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:20:00\"\>", ",", "7886.33`", ",", "7911.74`", 
     ",", "7880.83`", ",", "7904.97`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:30:00\"\>", ",", "7905.37`", ",", "7917.47`", 
     ",", "7897.4`", ",", "7913.33`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:40:00\"\>", ",", "7913.57`", ",", "7915.`", 
     ",", "7895.01`", ",", "7904.33`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T13:50:00\"\>", ",", "7904.17`", ",", "7911.5`", 
     ",", "7891.9`", ",", "7907.91`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:00:00\"\>", ",", "7907.99`", ",", "7907.99`", 
     ",", "7884.9`", ",", "7887.84`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:10:00\"\>", ",", "7888.16`", ",", "7904.57`", 
     ",", "7878.21`", ",", "7898.83`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:20:00\"\>", ",", "7898.99`", ",", "7906.96`", 
     ",", "7889.04`", ",", "7902.98`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:30:00\"\>", ",", "7904.01`", ",", "7928.62`", 
     ",", "7902.18`", ",", "7927.51`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:40:00\"\>", ",", "7927.83`", ",", "7946.54`", 
     ",", "7926.63`", ",", "7942.48`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T14:50:00\"\>", ",", "7942.24`", ",", "7946.54`", 
     ",", "7914.84`", ",", "7917.55`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:00:00\"\>", ",", "7917.63`", ",", "7923.52`", 
     ",", "7897.`", ",", "7902.66`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:10:00\"\>", ",", "7901.7`", ",", "7901.7`", 
     ",", "7865.7`", ",", "7876.13`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:20:00\"\>", ",", "7875.34`", ",", "7891.82`", 
     ",", "7863.63`", ",", "7889.04`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:30:00\"\>", ",", "7888.72`", ",", "7900.9`", 
     ",", "7870.8`", ",", "7885.37`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:40:00\"\>", ",", "7886.41`", ",", "7937.22`", 
     ",", "7886.41`", ",", "7933.8`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T15:50:00\"\>", ",", "7933.48`", ",", "7979.44`", 
     ",", "7925.2`", ",", "7959.53`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-23T16:00:00\"\>", ",", "7959.21`", ",", "7959.21`", 
     ",", "7957.06`", ",", "7957.06`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T09:30:00\"\>", ",", "7957.45`", ",", "8022.05`", 
     ",", "7957.45`", ",", "8003.01`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T09:40:00\"\>", ",", "8001.98`", ",", "8023.48`", 
     ",", "7996.8`", ",", "8012.89`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T09:50:00\"\>", ",", "8012.09`", ",", "8051.68`", 
     ",", "8007.95`", ",", "8051.68`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:00:00\"\>", ",", "8059.56`", ",", "8067.45`", 
     ",", "8036.7`", ",", "8036.7`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:10:00\"\>", ",", "8038.14`", ",", "8044.59`", 
     ",", "8023.56`", ",", "8034.71`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:20:00\"\>", ",", "8037.26`", ",", "8037.9`", 
     ",", "8007.95`", ",", "8030.25`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:30:00\"\>", ",", "8030.73`", ",", "8046.18`", 
     ",", "8025.`", ",", "8035.83`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:40:00\"\>", ",", "8036.15`", ",", "8055.9`", 
     ",", "8024.84`", ",", "8050.64`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T10:50:00\"\>", ",", "8051.28`", ",", "8063.47`", 
     ",", "8047.7`", ",", "8055.18`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:00:00\"\>", ",", "8055.42`", ",", "8064.34`", 
     ",", "8042.76`", ",", "8055.18`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:10:00\"\>", ",", "8055.42`", ",", "8067.53`", 
     ",", "8053.27`", ",", "8058.85`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:20:00\"\>", ",", "8059.4`", ",", "8063.07`", 
     ",", "8045.31`", ",", "8059.88`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:30:00\"\>", ",", "8061.55`", ",", "8081.78`", 
     ",", "8057.97`", ",", "8079.`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:40:00\"\>", ",", "8079.16`", ",", "8096.6`", 
     ",", "8071.03`", ",", "8087.68`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T11:50:00\"\>", ",", "8087.28`", ",", "8095.48`", 
     ",", "8082.5`", ",", "8089.27`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:00:00\"\>", ",", "8089.11`", ",", "8092.54`", 
     ",", "8078.68`", ",", "8081.71`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:10:00\"\>", ",", "8080.99`", ",", "8089.03`", 
     ",", "8078.28`", ",", "8081.78`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:20:00\"\>", ",", "8081.94`", ",", "8086.01`", 
     ",", "8078.2`", ",", "8082.66`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:30:00\"\>", ",", "8082.1`", ",", "8096.68`", 
     ",", "8080.75`", ",", "8096.68`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:40:00\"\>", ",", "8096.92`", ",", "8097.24`", 
     ",", "8082.1`", ",", "8082.66`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T12:50:00\"\>", ",", "8082.82`", ",", "8097.87`", 
     ",", "8076.37`", ",", "8094.21`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:00:00\"\>", ",", "8094.45`", ",", "8095.17`", 
     ",", "8086.25`", ",", "8088.56`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:10:00\"\>", ",", "8089.35`", ",", "8096.52`", 
     ",", "8085.21`", ",", "8095.17`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:20:00\"\>", ",", "8095.25`", ",", "8106.56`", 
     ",", "8092.3`", ",", "8095.4`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:30:00\"\>", ",", "8095.48`", ",", "8095.48`", 
     ",", "8068.24`", ",", "8072.23`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:40:00\"\>", ",", "8071.99`", ",", "8080.43`", 
     ",", "8070.71`", ",", "8077.32`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T13:50:00\"\>", ",", "8077.17`", ",", "8077.96`", 
     ",", "8072.71`", ",", "8074.62`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:00:00\"\>", ",", "8075.49`", ",", "8097.56`", 
     ",", "8047.93`", ",", "8048.73`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:10:00\"\>", ",", "8049.13`", ",", "8057.17`", 
     ",", "8011.54`", ",", "8044.59`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:20:00\"\>", ",", "8044.35`", ",", "8066.57`", 
     ",", "8037.98`", ",", "8066.57`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:30:00\"\>", ",", "8068.24`", ",", "8089.19`", 
     ",", "8063.7`", ",", "8087.44`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:40:00\"\>", ",", "8086.8`", ",", "8116.43`", 
     ",", "8079.4`", ",", "8094.13`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T14:50:00\"\>", ",", "8094.45`", ",", "8112.21`", 
     ",", "8093.02`", ",", "8112.21`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:00:00\"\>", ",", "8112.85`", ",", "8127.5`", 
     ",", "8102.02`", ",", "8117.55`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:10:00\"\>", ",", "8117.79`", ",", "8121.13`", 
     ",", "8097.71`", ",", "8097.71`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:20:00\"\>", ",", "8096.6`", ",", "8118.02`", 
     ",", "8095.96`", ",", "8101.38`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:30:00\"\>", ",", "8102.02`", ",", "8120.89`", 
     ",", "8097.63`", ",", "8111.41`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:40:00\"\>", ",", "8111.49`", ",", "8121.13`", 
     ",", "8100.5`", ",", "8111.18`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T15:50:00\"\>", ",", "8112.05`", ",", "8114.12`", 
     ",", "8026.83`", ",", "8073.74`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-24T16:00:00\"\>", ",", "8073.82`", ",", "8076.53`", 
     ",", "8073.82`", ",", "8076.29`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T09:30:00\"\>", ",", "8073.82`", ",", "8073.82`", 
     ",", "7989.87`", ",", "8007.55`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T09:40:00\"\>", ",", "8008.43`", ",", "8032.8`", 
     ",", "7987.4`", ",", "8029.7`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T09:50:00\"\>", ",", "8030.01`", ",", "8044.67`", 
     ",", "8019.66`", ",", "8036.23`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:00:00\"\>", ",", "8037.82`", ",", "8051.44`", 
     ",", "8016.63`", ",", "8022.93`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:10:00\"\>", ",", "8023.32`", ",", "8058.05`", 
     ",", "8000.62`", ",", "8055.26`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:20:00\"\>", ",", "8055.9`", ",", "8073.18`", 
     ",", "8047.14`", ",", "8065.06`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:30:00\"\>", ",", "8063.86`", ",", "8085.29`", 
     ",", "8051.12`", ",", "8082.66`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:40:00\"\>", ",", "8082.74`", ",", "8100.98`", 
     ",", "8081.86`", ",", "8098.67`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T10:50:00\"\>", ",", "8098.03`", ",", "8106.32`", 
     ",", "8088.48`", ",", "8105.52`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:00:00\"\>", ",", "8105.84`", ",", "8118.58`", 
     ",", "8105.52`", ",", "8113.33`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:10:00\"\>", ",", "8113.25`", ",", "8113.25`", 
     ",", "8089.51`", ",", "8091.74`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:20:00\"\>", ",", "8091.58`", ",", "8100.34`", 
     ",", "8089.03`", ",", "8098.83`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:30:00\"\>", ",", "8100.5`", ",", "8107.19`", 
     ",", "8090.94`", ",", "8102.33`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:40:00\"\>", ",", "8102.49`", ",", "8115.87`", 
     ",", "8095.09`", ",", "8115.32`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T11:50:00\"\>", ",", "8116.51`", ",", "8116.51`", 
     ",", "8104.17`", ",", "8110.46`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:00:00\"\>", ",", "8110.78`", ",", "8122.49`", 
     ",", "8108.71`", ",", "8110.78`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:10:00\"\>", ",", "8110.46`", ",", "8110.54`", 
     ",", "8092.46`", ",", "8092.46`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:20:00\"\>", ",", "8092.62`", ",", "8097.87`", 
     ",", "8080.27`", ",", "8087.36`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:30:00\"\>", ",", "8087.84`", ",", "8088.24`", 
     ",", "8071.11`", ",", "8078.84`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:40:00\"\>", ",", "8078.6`", ",", "8078.6`", 
     ",", "8060.92`", ",", "8064.26`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T12:50:00\"\>", ",", "8064.5`", ",", "8069.44`", 
     ",", "8053.51`", ",", "8067.93`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:00:00\"\>", ",", "8066.65`", ",", "8069.2`", 
     ",", "8052.79`", ",", "8057.57`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:10:00\"\>", ",", "8057.25`", ",", "8065.22`", 
     ",", "8041.8`", ",", "8041.8`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:20:00\"\>", ",", "8041.56`", ",", "8041.56`", 
     ",", "8024.6`", ",", "8024.68`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:30:00\"\>", ",", "8024.44`", ",", "8027.86`", 
     ",", "8012.97`", ",", "8022.61`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:40:00\"\>", ",", "8022.53`", ",", "8032.16`", 
     ",", "8005.8`", ",", "8005.88`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T13:50:00\"\>", ",", "8006.04`", ",", "8009.7`", 
     ",", "7989.47`", ",", "8005.4`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:00:00\"\>", ",", "8004.69`", ",", "8007.95`", 
     ",", "7990.99`", ",", "7997.44`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:10:00\"\>", ",", "7997.12`", ",", "8019.5`", 
     ",", "7996.24`", ",", "8005.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:20:00\"\>", ",", "8005.48`", ",", "8019.9`", 
     ",", "8000.94`", ",", "8019.1`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:30:00\"\>", ",", "8019.18`", ",", "8039.65`", 
     ",", "8019.18`", ",", "8034.08`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:40:00\"\>", ",", "8034.31`", ",", "8037.26`", 
     ",", "8020.7`", ",", "8027.47`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T14:50:00\"\>", ",", "8028.42`", ",", "8037.02`", 
     ",", "8019.34`", ",", "8031.85`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:00:00\"\>", ",", "8031.69`", ",", "8033.2`", 
     ",", "8000.07`", ",", "8003.57`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:10:00\"\>", ",", "8003.73`", ",", "8045.94`", 
     ",", "8001.02`", ",", "8045.94`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:20:00\"\>", ",", "8046.26`", ",", "8051.2`", 
     ",", "8013.05`", ",", "8018.39`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:30:00\"\>", ",", "8017.11`", ",", "8026.99`", 
     ",", "8009.23`", ",", "8017.35`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:40:00\"\>", ",", "8016.87`", ",", "8064.1`", 
     ",", "8011.93`", ",", "8048.81`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T15:50:00\"\>", ",", "8045.62`", ",", "8048.25`", 
     ",", "8015.92`", ",", "8021.89`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-27T16:00:00\"\>", ",", "8022.93`", ",", "8025.55`", 
     ",", "8022.69`", ",", "8025.`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T09:30:00\"\>", ",", "8023.56`", ",", "8023.56`", 
     ",", "7939.45`", ",", "7970.52`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T09:40:00\"\>", ",", "7970.28`", ",", "7982.3`", 
     ",", "7965.66`", ",", "7967.17`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T09:50:00\"\>", ",", "7967.09`", ",", "7993.61`", 
     ",", "7967.09`", ",", "7992.02`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:00:00\"\>", ",", "8004.61`", ",", "8035.03`", 
     ",", "8002.3`", ",", "8026.03`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:10:00\"\>", ",", "8025.47`", ",", "8037.1`", 
     ",", "8015.44`", ",", "8035.43`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:20:00\"\>", ",", "8035.91`", ",", "8047.06`", 
     ",", "8033.2`", ",", "8045.07`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:30:00\"\>", ",", "8045.62`", ",", "8048.81`", 
     ",", "8031.13`", ",", "8048.73`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:40:00\"\>", ",", "8048.57`", ",", "8072.31`", 
     ",", "8048.57`", ",", "8057.73`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T10:50:00\"\>", ",", "8057.65`", ",", "8063.07`", 
     ",", "8042.28`", ",", "8045.86`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:00:00\"\>", ",", "8046.1`", ",", "8053.83`", 
     ",", "8035.91`", ",", "8038.46`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:10:00\"\>", ",", "8036.78`", ",", "8040.77`", 
     ",", "8023.48`", ",", "8026.51`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:20:00\"\>", ",", "8026.59`", ",", "8033.36`", 
     ",", "8021.49`", ",", "8021.65`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:30:00\"\>", ",", "8022.69`", ",", "8034.79`", 
     ",", "8016.47`", ",", "8032.08`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:40:00\"\>", ",", "8032.24`", ",", "8034.47`", 
     ",", "8008.03`", ",", "8008.11`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T11:50:00\"\>", ",", "8007.79`", ",", "8020.38`", 
     ",", "8003.49`", ",", "8018.23`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:00:00\"\>", ",", "8018.07`", ",", "8035.35`", 
     ",", "8018.07`", ",", "8030.49`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:10:00\"\>", ",", "8030.89`", ",", "8038.06`", 
     ",", "8030.25`", ",", "8037.9`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:20:00\"\>", ",", "8038.85`", ",", "8053.83`", 
     ",", "8038.85`", ",", "8047.78`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:30:00\"\>", ",", "8048.01`", ",", "8054.07`", 
     ",", "8040.37`", ",", "8040.77`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:40:00\"\>", ",", "8040.05`", ",", "8049.29`", 
     ",", "8039.33`", ",", "8047.78`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T12:50:00\"\>", ",", "8048.41`", ",", "8049.93`", 
     ",", "8040.77`", ",", "8047.46`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:00:00\"\>", ",", "8051.12`", ",", "8059.17`", 
     ",", "8028.02`", ",", "8032.08`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:10:00\"\>", ",", "8031.53`", ",", "8034.63`", 
     ",", "8025.71`", ",", "8030.09`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:20:00\"\>", ",", "8029.85`", ",", "8034.87`", 
     ",", "8017.91`", ",", "8030.65`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:30:00\"\>", ",", "8029.7`", ",", "8055.98`", 
     ",", "8027.62`", ",", "8050.88`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:40:00\"\>", ",", "8050.56`", ",", "8061.87`", 
     ",", "8042.28`", ",", "8047.54`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T13:50:00\"\>", ",", "8047.46`", ",", "8048.33`", 
     ",", "8031.29`", ",", "8035.99`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:00:00\"\>", ",", "8038.54`", ",", "8057.97`", 
     ",", "8038.54`", ",", "8049.13`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:10:00\"\>", ",", "8049.45`", ",", "8057.49`", 
     ",", "8038.38`", ",", "8047.06`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:20:00\"\>", ",", "8045.94`", ",", "8079.48`", 
     ",", "8045.23`", ",", "8073.82`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:30:00\"\>", ",", "8076.13`", ",", "8091.58`", 
     ",", "8076.13`", ",", "8088.24`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:40:00\"\>", ",", "8088.79`", ",", "8088.87`", 
     ",", "8065.7`", ",", "8068.88`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T14:50:00\"\>", ",", "8068.96`", ",", "8078.12`", 
     ",", "8064.02`", ",", "8064.02`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:00:00\"\>", ",", "8063.86`", ",", "8076.69`", 
     ",", "8057.33`", ",", "8069.52`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:10:00\"\>", ",", "8069.68`", ",", "8081.07`", 
     ",", "8040.21`", ",", "8040.77`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:20:00\"\>", ",", "8040.29`", ",", "8049.21`", 
     ",", "8032.01`", ",", "8032.01`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:30:00\"\>", ",", "8032.64`", ",", "8039.17`", 
     ",", "8018.78`", ",", "8032.72`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:40:00\"\>", ",", "8031.69`", ",", "8038.85`", 
     ",", "8010.58`", ",", "8032.48`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T15:50:00\"\>", ",", "8032.72`", ",", "8034.95`", 
     ",", "8010.34`", ",", "8012.73`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-28T16:00:00\"\>", ",", "8012.65`", ",", "8016.95`", 
     ",", "8012.65`", ",", "8016.95`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T09:30:00\"\>", ",", "8018.31`", ",", "8105.52`", 
     ",", "8018.31`", ",", "8105.52`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T09:40:00\"\>", ",", "8108.31`", ",", "8119.14`", 
     ",", "8097.95`", ",", "8112.53`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T09:50:00\"\>", ",", "8113.96`", ",", "8132.04`", 
     ",", "8112.85`", ",", "8129.65`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:00:00\"\>", ",", "8130.37`", ",", "8136.9`", 
     ",", "8117.23`", ",", "8121.93`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:10:00\"\>", ",", "8121.69`", ",", "8140.25`", 
     ",", "8113.96`", ",", "8139.45`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:20:00\"\>", ",", "8139.29`", ",", "8157.53`", 
     ",", "8136.9`", ",", "8156.97`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:30:00\"\>", ",", "8156.18`", ",", "8163.66`", 
     ",", "8151.56`", ",", "8162.63`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:40:00\"\>", ",", "8163.03`", ",", "8186.84`", 
     ",", "8162.95`", ",", "8175.29`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T10:50:00\"\>", ",", "8173.78`", ",", "8178.72`", 
     ",", "8167.49`", ",", "8175.53`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:00:00\"\>", ",", "8175.37`", ",", "8177.84`", 
     ",", "8162.15`", ",", "8166.13`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:10:00\"\>", ",", "8165.97`", ",", "8175.05`", 
     ",", "8164.22`", ",", "8171.39`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:20:00\"\>", ",", "8171.63`", ",", "8188.91`", 
     ",", "8171.63`", ",", "8188.91`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:30:00\"\>", ",", "8189.47`", ",", "8190.74`", 
     ",", "8175.69`", ",", "8177.44`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:40:00\"\>", ",", "8177.6`", ",", "8181.82`", 
     ",", "8167.72`", ",", "8177.2`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T11:50:00\"\>", ",", "8177.12`", ",", "8180.55`", 
     ",", "8163.03`", ",", "8163.66`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:00:00\"\>", ",", "8163.9`", ",", "8172.5`", 
     ",", "8163.9`", ",", "8164.3`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:10:00\"\>", ",", "8163.66`", ",", "8173.06`", 
     ",", "8159.76`", ",", "8166.29`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:20:00\"\>", ",", "8166.37`", ",", "8167.17`", 
     ",", "8156.57`", ",", "8165.1`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:30:00\"\>", ",", "8165.26`", ",", "8170.91`", 
     ",", "8159.12`", ",", "8159.12`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:40:00\"\>", ",", "8160.32`", ",", "8171.63`", 
     ",", "8158.33`", ",", "8163.5`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T12:50:00\"\>", ",", "8163.58`", ",", "8167.25`", 
     ",", "8161.35`", ",", "8166.45`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:00:00\"\>", ",", "8166.61`", ",", "8178.08`", 
     ",", "8166.21`", ",", "8172.03`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:10:00\"\>", ",", "8172.34`", ",", "8180.79`", 
     ",", "8169.`", ",", "8178.64`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:20:00\"\>", ",", "8178.8`", ",", "8214.4`", 
     ",", "8178.4`", ",", "8210.5`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:30:00\"\>", ",", "8210.42`", ",", "8211.61`", 
     ",", "8201.65`", ",", "8202.13`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:40:00\"\>", ",", "8202.21`", ",", "8206.19`", 
     ",", "8201.73`", ",", "8203.73`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T13:50:00\"\>", ",", "8203.49`", ",", "8204.68`", 
     ",", "8196.48`", ",", "8196.48`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:00:00\"\>", ",", "8196.32`", ",", "8204.36`", 
     ",", "8191.94`", ",", "8201.34`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:10:00\"\>", ",", "8201.26`", ",", "8224.59`", 
     ",", "8176.88`", ",", "8192.58`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:20:00\"\>", ",", "8187.96`", ",", "8217.11`", 
     ",", "8181.34`", ",", "8210.58`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:30:00\"\>", ",", "8211.13`", ",", "8257.49`", 
     ",", "8206.27`", ",", "8242.67`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:40:00\"\>", ",", "8243.87`", ",", "8249.6`", 
     ",", "8206.27`", ",", "8210.97`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T14:50:00\"\>", ",", "8211.05`", ",", "8217.66`", 
     ",", "8198.07`", ",", "8217.03`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:00:00\"\>", ",", "8217.11`", ",", "8226.74`", 
     ",", "8201.89`", ",", "8208.98`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:10:00\"\>", ",", "8211.37`", ",", "8218.94`", 
     ",", "8184.29`", ",", "8187.48`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:20:00\"\>", ",", "8187.64`", ",", "8194.01`", 
     ",", "8159.92`", ",", "8167.41`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:30:00\"\>", ",", "8167.72`", ",", "8185.65`", 
     ",", "8148.29`", ",", "8184.21`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:40:00\"\>", ",", "8183.97`", ",", "8191.94`", 
     ",", "8132.84`", ",", "8144.47`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T15:50:00\"\>", ",", "8145.5`", ",", "8182.22`", 
     ",", "8139.21`", ",", "8181.66`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-29T16:00:00\"\>", ",", "8181.42`", ",", "8185.81`", 
     ",", "8181.27`", ",", "8185.73`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T09:30:00\"\>", ",", "8188.51`", ",", "8264.74`", 
     ",", "8188.51`", ",", "8264.74`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T09:40:00\"\>", ",", "8266.57`", ",", "8296.67`", 
     ",", "8253.82`", ",", "8257.01`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T09:50:00\"\>", ",", "8255.97`", ",", "8279.95`", 
     ",", "8252.79`", ",", "8267.84`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:00:00\"\>", ",", "8267.21`", ",", "8301.45`", 
     ",", "8267.21`", ",", "8294.37`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:10:00\"\>", ",", "8294.05`", ",", "8301.77`", 
     ",", "8284.73`", ",", "8286.`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:20:00\"\>", ",", "8286.72`", ",", "8293.73`", 
     ",", "8271.82`", ",", "8293.57`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:30:00\"\>", ",", "8295.16`", ",", "8298.59`", 
     ",", "8284.81`", ",", "8288.23`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:40:00\"\>", ",", "8288.07`", ",", "8289.03`", 
     ",", "8261.23`", ",", "8269.75`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T10:50:00\"\>", ",", "8270.95`", ",", "8290.54`", 
     ",", "8265.93`", ",", "8287.28`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:00:00\"\>", ",", "8287.36`", ",", "8295.`", 
     ",", "8283.45`", ",", "8288.15`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:10:00\"\>", ",", "8288.79`", ",", "8307.51`", 
     ",", "8288.79`", ",", "8297.23`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:20:00\"\>", ",", "8296.99`", ",", "8298.03`", 
     ",", "8282.02`", ",", "8290.54`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:30:00\"\>", ",", "8290.22`", ",", "8294.76`", 
     ",", "8270.63`", ",", "8270.63`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:40:00\"\>", ",", "8270.87`", ",", "8281.7`", 
     ",", "8270.87`", ",", "8280.27`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T11:50:00\"\>", ",", "8280.11`", ",", "8281.06`", 
     ",", "8251.59`", ",", "8252.47`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:00:00\"\>", ",", "8252.15`", ",", "8260.59`", 
     ",", "8239.09`", ",", "8247.05`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:10:00\"\>", ",", "8246.97`", ",", "8256.13`", 
     ",", "8242.2`", ",", "8252.23`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:20:00\"\>", ",", "8252.15`", ",", "8260.99`", 
     ",", "8248.97`", ",", "8254.46`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:30:00\"\>", ",", "8254.62`", ",", "8255.1`", 
     ",", "8212.49`", ",", "8219.97`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:40:00\"\>", ",", "8219.74`", ",", "8219.74`", 
     ",", "8183.18`", ",", "8183.65`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T12:50:00\"\>", ",", "8184.05`", ",", "8194.81`", 
     ",", "8179.43`", ",", "8193.53`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:00:00\"\>", ",", "8194.17`", ",", "8203.41`", 
     ",", "8181.82`", ",", "8195.6`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:10:00\"\>", ",", "8195.68`", ",", "8195.68`", 
     ",", "8170.83`", ",", "8170.83`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:20:00\"\>", ",", "8170.91`", ",", "8170.91`", 
     ",", "8149.8`", ",", "8162.95`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:30:00\"\>", ",", "8161.75`", ",", "8181.27`", 
     ",", "8156.89`", ",", "8177.68`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:40:00\"\>", ",", "8177.52`", ",", "8192.97`", 
     ",", "8176.33`", ",", "8188.43`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T13:50:00\"\>", ",", "8188.91`", ",", "8205.08`", 
     ",", "8181.27`", ",", "8198.63`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:00:00\"\>", ",", "8198.95`", ",", "8206.12`", 
     ",", "8195.68`", ",", "8199.03`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:10:00\"\>", ",", "8198.87`", ",", "8216.07`", 
     ",", "8182.94`", ",", "8216.07`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:20:00\"\>", ",", "8215.67`", ",", "8224.51`", 
     ",", "8208.11`", ",", "8217.82`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:30:00\"\>", ",", "8216.79`", ",", "8219.58`", 
     ",", "8201.81`", ",", "8214.32`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:40:00\"\>", ",", "8214.4`", ",", "8214.8`", 
     ",", "8185.73`", ",", "8195.76`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T14:50:00\"\>", ",", "8195.28`", ",", "8217.03`", 
     ",", "8188.99`", ",", "8211.45`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:00:00\"\>", ",", "8211.77`", ",", "8215.35`", 
     ",", "8156.34`", ",", "8163.82`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:10:00\"\>", ",", "8162.15`", ",", "8181.03`", 
     ",", "8137.06`", ",", "8151.16`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:20:00\"\>", ",", "8149.64`", ",", "8167.57`", 
     ",", "8145.58`", ",", "8167.33`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:30:00\"\>", ",", "8166.77`", ",", "8200.54`", 
     ",", "8163.98`", ",", "8192.89`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:40:00\"\>", ",", "8192.65`", ",", "8216.55`", 
     ",", "8179.43`", ",", "8203.09`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T15:50:00\"\>", ",", "8202.93`", ",", "8208.98`", 
     ",", "8156.65`", ",", "8168.36`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-04-30T16:00:00\"\>", ",", "8169.08`", ",", "8169.08`", 
     ",", "8166.93`", ",", "8168.12`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T09:30:00\"\>", ",", "8167.41`", ",", "8183.5`", 
     ",", "8137.54`", ",", "8137.54`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T09:40:00\"\>", ",", "8134.99`", ",", "8144.15`", 
     ",", "8127.1`", ",", "8128.3`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T09:50:00\"\>", ",", "8128.54`", ",", "8160.`", 
     ",", "8128.14`", ",", "8156.34`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:00:00\"\>", ",", "8160.16`", ",", "8169.8`", 
     ",", "8140.8`", ",", "8141.2`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:10:00\"\>", ",", "8141.12`", ",", "8143.43`", 
     ",", "8106.32`", ",", "8106.64`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:20:00\"\>", ",", "8105.28`", ",", "8113.72`", 
     ",", "8099.39`", ",", "8113.09`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:30:00\"\>", ",", "8112.37`", ",", "8126.47`", 
     ",", "8107.03`", ",", "8125.35`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:40:00\"\>", ",", "8125.03`", ",", "8136.03`", 
     ",", "8120.18`", ",", "8134.83`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T10:50:00\"\>", ",", "8134.67`", ",", "8148.85`", 
     ",", "8131.64`", ",", "8143.27`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:00:00\"\>", ",", "8143.43`", ",", "8161.99`", 
     ",", "8142.32`", ",", "8156.49`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:10:00\"\>", ",", "8156.65`", ",", "8184.85`", 
     ",", "8156.02`", ",", "8179.03`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:20:00\"\>", ",", "8179.27`", ",", "8194.57`", 
     ",", "8179.19`", ",", "8186.44`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:30:00\"\>", ",", "8185.33`", ",", "8185.33`", 
     ",", "8172.74`", ",", "8176.73`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:40:00\"\>", ",", "8176.8`", ",", "8184.21`", 
     ",", "8171.71`", ",", "8174.02`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T11:50:00\"\>", ",", "8173.7`", ",", "8175.37`", 
     ",", "8155.78`", ",", "8159.28`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:00:00\"\>", ",", "8158.96`", ",", "8159.68`", 
     ",", "8149.17`", ",", "8154.03`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:10:00\"\>", ",", "8153.39`", ",", "8172.58`", 
     ",", "8146.62`", ",", "8169.32`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:20:00\"\>", ",", "8169.24`", ",", "8170.91`", 
     ",", "8154.18`", ",", "8154.58`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:30:00\"\>", ",", "8153.07`", ",", "8176.17`", 
     ",", "8147.02`", ",", "8172.58`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:40:00\"\>", ",", "8174.18`", ",", "8175.77`", 
     ",", "8158.8`", ",", "8162.55`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T12:50:00\"\>", ",", "8162.87`", ",", "8175.29`", 
     ",", "8152.03`", ",", "8167.72`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:00:00\"\>", ",", "8168.04`", ",", "8175.69`", 
     ",", "8161.43`", ",", "8173.14`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:10:00\"\>", ",", "8173.06`", ",", "8189.63`", 
     ",", "8169.48`", ",", "8187.64`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:20:00\"\>", ",", "8187.72`", ",", "8195.6`", 
     ",", "8184.45`", ",", "8195.36`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:30:00\"\>", ",", "8195.04`", ",", "8210.1`", 
     ",", "8191.46`", ",", "8207.71`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:40:00\"\>", ",", "8207.87`", ",", "8219.74`", 
     ",", "8206.04`", ",", "8207.07`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T13:50:00\"\>", ",", "8208.03`", ",", "8208.74`", 
     ",", "8196.8`", ",", "8196.8`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:00:00\"\>", ",", "8196.88`", ",", "8211.21`", 
     ",", "8193.21`", ",", "8207.87`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:10:00\"\>", ",", "8207.23`", ",", "8207.39`", 
     ",", "8185.73`", ",", "8185.73`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:20:00\"\>", ",", "8187.32`", ",", "8201.26`", 
     ",", "8187.`", ",", "8191.7`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:30:00\"\>", ",", "8190.9`", ",", "8197.35`", 
     ",", "8184.69`", ",", "8186.76`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:40:00\"\>", ",", "8185.57`", ",", "8188.99`", 
     ",", "8177.84`", ",", "8184.45`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T14:50:00\"\>", ",", "8184.37`", ",", "8184.37`", 
     ",", "8174.73`", ",", "8177.12`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:00:00\"\>", ",", "8177.28`", ",", "8197.67`", 
     ",", "8177.2`", ",", "8178.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:10:00\"\>", ",", "8177.52`", ",", "8177.84`", 
     ",", "8141.28`", ",", "8144.71`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:20:00\"\>", ",", "8144.55`", ",", "8153.39`", 
     ",", "8142.32`", ",", "8147.26`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:30:00\"\>", ",", "8147.34`", ",", "8158.01`", 
     ",", "8134.35`", ",", "8156.81`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:40:00\"\>", ",", "8157.53`", ",", "8178.`", 
     ",", "8157.53`", ",", "8165.34`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T15:50:00\"\>", ",", "8166.85`", ",", "8209.54`", 
     ",", "8162.55`", ",", "8209.54`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-01T16:00:00\"\>", ",", "8209.46`", ",", "8212.41`", 
     ",", "8209.46`", ",", "8212.41`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T09:30:00\"\>", ",", "8213.6`", ",", "8282.74`", 
     ",", "8213.6`", ",", "8280.82`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T09:40:00\"\>", ",", "8281.22`", ",", "8311.09`", 
     ",", "8281.22`", ",", "8306.07`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T09:50:00\"\>", ",", "8306.23`", ",", "8334.27`", 
     ",", "8295.08`", ",", "8334.27`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:00:00\"\>", ",", "8336.98`", ",", "8375.37`", 
     ",", "8335.14`", ",", "8359.52`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:10:00\"\>", ",", "8359.84`", ",", "8384.37`", 
     ",", "8358.88`", ",", "8382.77`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:20:00\"\>", ",", "8384.61`", ",", "8394.64`", 
     ",", "8380.38`", ",", "8383.57`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:30:00\"\>", ",", "8383.73`", ",", "8387.87`", 
     ",", "8378.47`", ",", "8387.31`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:40:00\"\>", ",", "8387.23`", ",", "8388.75`", 
     ",", "8375.77`", ",", "8388.75`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T10:50:00\"\>", ",", "8388.83`", ",", "8392.97`", 
     ",", "8370.59`", ",", "8381.9`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:00:00\"\>", ",", "8382.3`", ",", "8412.88`", 
     ",", "8382.3`", ",", "8406.11`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:10:00\"\>", ",", "8406.19`", ",", "8415.91`", 
     ",", "8402.61`", ",", "8404.12`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:20:00\"\>", ",", "8403.8`", ",", "8409.14`", 
     ",", "8389.86`", ",", "8399.66`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:30:00\"\>", ",", "8400.62`", ",", "8403.48`", 
     ",", "8390.18`", ",", "8392.17`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:40:00\"\>", ",", "8390.1`", ",", "8391.93`", 
     ",", "8375.45`", ",", "8385.4`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T11:50:00\"\>", ",", "8385.8`", ",", "8389.54`", 
     ",", "8369.79`", ",", "8376.88`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:00:00\"\>", ",", "8377.04`", ",", "8398.15`", 
     ",", "8371.07`", ",", "8372.82`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:10:00\"\>", ",", "8373.46`", ",", "8375.29`", 
     ",", "8355.14`", ",", "8360.39`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:20:00\"\>", ",", "8360.63`", ",", "8366.13`", 
     ",", "8348.37`", ",", "8353.54`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:30:00\"\>", ",", "8353.22`", ",", "8364.38`", 
     ",", "8350.91`", ",", "8360.79`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:40:00\"\>", ",", "8359.68`", ",", "8379.43`", 
     ",", "8357.69`", ",", "8375.77`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T12:50:00\"\>", ",", "8375.69`", ",", "8375.69`", 
     ",", "8362.15`", ",", "8368.44`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:00:00\"\>", ",", "8368.52`", ",", "8381.58`", 
     ",", "8364.93`", ",", "8378.63`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:10:00\"\>", ",", "8378.71`", ",", "8380.94`", 
     ",", "8363.9`", ",", "8365.97`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:20:00\"\>", ",", "8365.89`", ",", "8387.79`", 
     ",", "8365.41`", ",", "8381.66`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:30:00\"\>", ",", "8381.98`", ",", "8398.62`", 
     ",", "8381.98`", ",", "8396.87`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:40:00\"\>", ",", "8397.51`", ",", "8404.76`", 
     ",", "8394.16`", ",", "8399.9`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T13:50:00\"\>", ",", "8400.22`", ",", "8405.71`", 
     ",", "8387.39`", ",", "8396.79`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:00:00\"\>", ",", "8397.03`", ",", "8401.81`", 
     ",", "8375.92`", ",", "8376.32`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:10:00\"\>", ",", "8376.64`", ",", "8398.78`", 
     ",", "8373.93`", ",", "8396.47`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:20:00\"\>", ",", "8396.55`", ",", "8411.85`", 
     ",", "8390.9`", ",", "8410.09`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:30:00\"\>", ",", "8410.17`", ",", "8413.68`", 
     ",", "8388.67`", ",", "8394.48`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:40:00\"\>", ",", "8396.23`", ",", "8405.16`", 
     ",", "8388.35`", ",", "8396.95`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T14:50:00\"\>", ",", "8397.19`", ",", "8397.83`", 
     ",", "8377.2`", ",", "8387.23`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:00:00\"\>", ",", "8386.84`", ",", "8401.57`", 
     ",", "8386.04`", ",", "8390.18`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:10:00\"\>", ",", "8391.62`", ",", "8400.54`", 
     ",", "8383.01`", ",", "8400.54`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:20:00\"\>", ",", "8401.17`", ",", "8403.24`", 
     ",", "8381.58`", ",", "8385.48`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:30:00\"\>", ",", "8385.56`", ",", "8396.31`", 
     ",", "8371.07`", ",", "8377.6`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:40:00\"\>", ",", "8377.36`", ",", "8394.72`", 
     ",", "8374.33`", ",", "8383.65`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T15:50:00\"\>", ",", "8384.13`", ",", "8433.67`", 
     ",", "8384.13`", ",", "8433.67`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-04T16:00:00\"\>", ",", "8433.27`", ",", "8433.35`", 
     ",", "8426.58`", ",", "8426.74`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T09:30:00\"\>", ",", "8425.62`", ",", "8445.22`", 
     ",", "8402.77`", ",", "8438.85`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T09:40:00\"\>", ",", "8439.96`", ",", "8443.86`", 
     ",", "8412.32`", ",", "8426.5`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T09:50:00\"\>", ",", "8427.06`", ",", "8440.92`", 
     ",", "8417.9`", ",", "8440.6`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:00:00\"\>", ",", "8449.28`", ",", "8457.88`", 
     ",", "8419.33`", ",", "8426.42`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:10:00\"\>", ",", "8427.3`", ",", "8446.01`", 
     ",", "8426.1`", ",", "8443.07`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:20:00\"\>", ",", "8442.91`", ",", "8443.31`", 
     ",", "8409.14`", ",", "8415.59`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:30:00\"\>", ",", "8412.32`", ",", "8412.32`", 
     ",", "8394.32`", ",", "8396.63`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:40:00\"\>", ",", "8397.99`", ",", "8401.81`", 
     ",", "8388.35`", ",", "8389.62`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T10:50:00\"\>", ",", "8389.54`", ",", "8397.75`", 
     ",", "8377.76`", ",", "8377.76`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:00:00\"\>", ",", "8377.52`", ",", "8394.64`", 
     ",", "8377.12`", ",", "8391.22`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:10:00\"\>", ",", "8390.9`", ",", "8400.62`", 
     ",", "8388.83`", ",", "8389.94`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:20:00\"\>", ",", "8389.7`", ",", "8402.77`", 
     ",", "8384.13`", ",", "8397.43`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:30:00\"\>", ",", "8397.83`", ",", "8412.88`", 
     ",", "8397.43`", ",", "8407.78`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:40:00\"\>", ",", "8408.18`", ",", "8418.06`", 
     ",", "8408.1`", ",", "8414.`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T11:50:00\"\>", ",", "8414.31`", ",", "8419.25`", 
     ",", "8408.1`", ",", "8411.69`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:00:00\"\>", ",", "8411.85`", ",", "8413.84`", 
     ",", "8404.36`", ",", "8412.4`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:10:00\"\>", ",", "8412.64`", ",", "8412.64`", 
     ",", "8404.52`", ",", "8411.05`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:20:00\"\>", ",", "8410.49`", ",", "8414.47`", 
     ",", "8401.33`", ",", "8407.31`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:30:00\"\>", ",", "8407.7`", ",", "8408.82`", 
     ",", "8391.22`", ",", "8394.4`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:40:00\"\>", ",", "8394.`", ",", "8395.28`", 
     ",", "8385.88`", ",", "8394.88`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T12:50:00\"\>", ",", "8394.8`", ",", "8399.66`", 
     ",", "8387.71`", ",", "8392.81`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:00:00\"\>", ",", "8392.73`", ",", "8408.98`", 
     ",", "8388.35`", ",", "8401.01`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:10:00\"\>", ",", "8400.62`", ",", "8405.31`", 
     ",", "8384.13`", ",", "8392.25`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:20:00\"\>", ",", "8392.65`", ",", "8394.56`", 
     ",", "8377.12`", ",", "8380.07`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:30:00\"\>", ",", "8380.23`", ",", "8382.69`", 
     ",", "8372.18`", ",", "8382.22`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:40:00\"\>", ",", "8382.3`", ",", "8384.84`", 
     ",", "8372.82`", ",", "8382.77`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T13:50:00\"\>", ",", "8383.65`", ",", "8395.12`", 
     ",", "8383.65`", ",", "8389.54`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:00:00\"\>", ",", "8389.78`", ",", "8392.73`", 
     ",", "8381.66`", ",", "8392.73`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:10:00\"\>", ",", "8392.97`", ",", "8406.11`", 
     ",", "8392.97`", ",", "8397.27`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:20:00\"\>", ",", "8397.35`", ",", "8398.62`", 
     ",", "8366.53`", ",", "8366.53`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:30:00\"\>", ",", "8365.09`", ",", "8385.24`", 
     ",", "8363.02`", ",", "8380.31`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:40:00\"\>", ",", "8380.94`", ",", "8393.85`", 
     ",", "8374.81`", ",", "8389.7`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T14:50:00\"\>", ",", "8390.26`", ",", "8397.67`", 
     ",", "8384.05`", ",", "8384.05`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:00:00\"\>", ",", "8383.57`", ",", "8405.95`", 
     ",", "8381.74`", ",", "8397.51`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:10:00\"\>", ",", "8397.75`", ",", "8398.62`", 
     ",", "8379.91`", ",", "8395.28`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:20:00\"\>", ",", "8395.76`", ",", "8399.58`", 
     ",", "8379.75`", ",", "8392.41`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:30:00\"\>", ",", "8392.01`", ",", "8416.94`", 
     ",", "8392.01`", ",", "8402.37`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:40:00\"\>", ",", "8402.13`", ",", "8420.77`", 
     ",", "8402.13`", ",", "8415.43`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T15:50:00\"\>", ",", "8415.91`", ",", "8420.05`", 
     ",", "8386.2`", ",", "8411.29`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-05T16:00:00\"\>", ",", "8412.`", ",", "8412.56`", 
     ",", "8410.49`", ",", "8410.65`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T09:30:00\"\>", ",", "8403.48`", ",", "8504.48`", 
     ",", "8403.08`", ",", "8476.2`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T09:40:00\"\>", ",", "8473.97`", ",", "8475.32`", 
     ",", "8459.4`", ",", "8469.75`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T09:50:00\"\>", ",", "8468.63`", ",", "8492.85`", 
     ",", "8462.5`", ",", "8484.01`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:00:00\"\>", ",", "8485.36`", ",", "8489.02`", 
     ",", "8443.47`", ",", "8456.85`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:10:00\"\>", ",", "8456.69`", ",", "8461.86`", 
     ",", "8442.43`", ",", "8444.18`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:20:00\"\>", ",", "8447.29`", ",", "8450.08`", 
     ",", "8413.92`", ",", "8415.59`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:30:00\"\>", ",", "8415.27`", ",", "8432.32`", 
     ",", "8412.32`", ",", "8429.53`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:40:00\"\>", ",", "8429.93`", ",", "8436.06`", 
     ",", "8414.39`", ",", "8423.47`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T10:50:00\"\>", ",", "8423.24`", ",", "8447.45`", 
     ",", "8422.2`", ",", "8445.14`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:00:00\"\>", ",", "8444.9`", ",", "8448.88`", 
     ",", "8427.3`", ",", "8435.74`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:10:00\"\>", ",", "8435.58`", ",", "8442.83`", 
     ",", "8416.7`", ",", "8416.7`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:20:00\"\>", ",", "8416.62`", ",", "8423.71`", 
     ",", "8411.21`", ",", "8420.45`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:30:00\"\>", ",", "8420.77`", ",", "8442.99`", 
     ",", "8415.67`", ",", "8438.69`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:40:00\"\>", ",", "8438.77`", ",", "8438.77`", 
     ",", "8430.64`", ",", "8430.72`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T11:50:00\"\>", ",", "8430.64`", ",", "8450.`", 
     ",", "8423.71`", ",", "8450.`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:00:00\"\>", ",", "8449.68`", ",", "8455.57`", 
     ",", "8446.97`", ",", "8450.79`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:10:00\"\>", ",", "8450.95`", ",", "8466.01`", 
     ",", "8450.95`", ",", "8459.79`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:20:00\"\>", ",", "8460.99`", ",", "8475.17`", 
     ",", "8455.65`", ",", "8475.01`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:30:00\"\>", ",", "8475.64`", ",", "8475.64`", 
     ",", "8459.`", ",", "8459.47`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:40:00\"\>", ",", "8459.87`", ",", "8469.91`", 
     ",", "8459.79`", ",", "8466.17`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T12:50:00\"\>", ",", "8466.25`", ",", "8471.98`", 
     ",", "8448.88`", ",", "8451.75`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:00:00\"\>", ",", "8450.95`", ",", "8456.77`", 
     ",", "8436.93`", ",", "8440.04`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:10:00\"\>", ",", "8439.09`", ",", "8446.25`", 
     ",", "8433.91`", ",", "8444.66`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:20:00\"\>", ",", "8445.14`", ",", "8460.51`", 
     ",", "8435.02`", ",", "8435.02`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:30:00\"\>", ",", "8434.47`", ",", "8458.2`", 
     ",", "8428.57`", ",", "8449.36`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:40:00\"\>", ",", "8449.92`", ",", "8463.78`", 
     ",", "8440.92`", ",", "8456.61`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T13:50:00\"\>", ",", "8457.01`", ",", "8492.21`", 
     ",", "8452.23`", ",", "8492.21`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:00:00\"\>", ",", "8495.4`", ",", "8498.58`", 
     ",", "8485.6`", ",", "8493.33`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:10:00\"\>", ",", "8493.17`", ",", "8495.48`", 
     ",", "8484.01`", ",", "8489.34`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:20:00\"\>", ",", "8491.81`", ",", "8508.06`", 
     ",", "8491.81`", ",", "8506.15`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:30:00\"\>", ",", "8506.95`", ",", "8518.33`", 
     ",", "8505.11`", ",", "8512.68`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:40:00\"\>", ",", "8514.67`", ",", "8516.5`", 
     ",", "8492.53`", ",", "8513.24`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T14:50:00\"\>", ",", "8513.08`", ",", "8517.54`", 
     ",", "8495.71`", ",", "8497.31`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:00:00\"\>", ",", "8497.94`", ",", "8508.14`", 
     ",", "8485.04`", ",", "8502.17`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:10:00\"\>", ",", "8499.3`", ",", "8501.77`", 
     ",", "8476.44`", ",", "8476.92`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:20:00\"\>", ",", "8476.76`", ",", "8486.63`", 
     ",", "8443.47`", ",", "8467.44`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:30:00\"\>", ",", "8466.88`", ",", "8479.47`", 
     ",", "8452.63`", ",", "8459.79`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:40:00\"\>", ",", "8460.11`", ",", "8495.`", 
     ",", "8460.11`", ",", "8495.`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T15:50:00\"\>", ",", "8495.71`", ",", "8515.07`", 
     ",", "8469.43`", ",", "8515.07`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-06T16:00:00\"\>", ",", "8514.91`", ",", "8514.91`", 
     ",", "8512.2`", ",", "8512.28`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T09:30:00\"\>", ",", "8513.56`", ",", "8576.4`", 
     ",", "8511.33`", ",", "8569.71`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T09:40:00\"\>", ",", "8567.56`", ",", "8571.06`", 
     ",", "8518.02`", ",", "8518.02`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T09:50:00\"\>", ",", "8518.49`", ",", "8535.38`", 
     ",", "8513.87`", ",", "8518.1`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:00:00\"\>", ",", "8517.78`", ",", "8528.13`", 
     ",", "8479.39`", ",", "8494.04`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:10:00\"\>", ",", "8495.`", ",", "8495.79`", 
     ",", "8465.61`", ",", "8490.78`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:20:00\"\>", ",", "8490.22`", ",", "8491.57`", 
     ",", "8429.69`", ",", "8440.6`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:30:00\"\>", ",", "8437.89`", ",", "8449.2`", 
     ",", "8421.88`", ",", "8449.2`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:40:00\"\>", ",", "8449.92`", ",", "8462.18`", 
     ",", "8438.45`", ",", "8462.18`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T10:50:00\"\>", ",", "8462.34`", ",", "8471.82`", 
     ",", "8456.61`", ",", "8457.56`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:00:00\"\>", ",", "8457.64`", ",", "8466.72`", 
     ",", "8431.84`", ",", "8432.`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:10:00\"\>", ",", "8432.08`", ",", "8436.06`", 
     ",", "8407.86`", ",", "8414.55`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:20:00\"\>", ",", "8414.24`", ",", "8428.65`", 
     ",", "8410.89`", ",", "8428.65`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:30:00\"\>", ",", "8428.81`", ",", "8444.18`", 
     ",", "8422.28`", ",", "8434.7`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:40:00\"\>", ",", "8434.78`", ",", "8441.79`", 
     ",", "8430.01`", ",", "8432.32`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T11:50:00\"\>", ",", "8433.11`", ",", "8458.36`", 
     ",", "8432.63`", ",", "8453.9`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:00:00\"\>", ",", "8453.74`", ",", "8453.74`", 
     ",", "8426.42`", ",", "8426.74`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:10:00\"\>", ",", "8426.98`", ",", "8435.5`", 
     ",", "8424.59`", ",", "8429.37`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:20:00\"\>", ",", "8429.69`", ",", "8444.26`", 
     ",", "8428.81`", ",", "8436.85`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:30:00\"\>", ",", "8436.93`", ",", "8445.7`", 
     ",", "8431.28`", ",", "8443.07`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:40:00\"\>", ",", "8443.31`", ",", "8466.56`", 
     ",", "8443.15`", ",", "8461.71`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T12:50:00\"\>", ",", "8462.18`", ",", "8462.74`", 
     ",", "8449.84`", ",", "8454.14`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:00:00\"\>", ",", "8454.38`", ",", "8456.93`", 
     ",", "8423.63`", ",", "8426.58`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:10:00\"\>", ",", "8426.66`", ",", "8430.8`", 
     ",", "8407.86`", ",", "8412.48`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:20:00\"\>", ",", "8412.64`", ",", "8422.04`", 
     ",", "8406.51`", ",", "8407.15`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:30:00\"\>", ",", "8407.54`", ",", "8411.61`", 
     ",", "8394.64`", ",", "8397.83`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:40:00\"\>", ",", "8397.35`", ",", "8397.35`", 
     ",", "8385.24`", ",", "8391.69`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T13:50:00\"\>", ",", "8391.62`", ",", "8404.6`", 
     ",", "8382.93`", ",", "8402.61`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:00:00\"\>", ",", "8402.93`", ",", "8417.66`", 
     ",", "8395.28`", ",", "8411.69`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:10:00\"\>", ",", "8411.77`", ",", "8412.08`", 
     ",", "8384.77`", ",", "8393.85`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:20:00\"\>", ",", "8393.69`", ",", "8415.11`", 
     ",", "8393.53`", ",", "8413.12`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:30:00\"\>", ",", "8412.96`", ",", "8413.68`", 
     ",", "8380.94`", ",", "8383.17`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:40:00\"\>", ",", "8382.3`", ",", "8391.14`", 
     ",", "8371.07`", ",", "8391.14`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T14:50:00\"\>", ",", "8393.53`", ",", "8402.13`", 
     ",", "8377.84`", ",", "8388.83`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:00:00\"\>", ",", "8390.02`", ",", "8407.31`", 
     ",", "8374.57`", ",", "8380.31`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:10:00\"\>", ",", "8380.38`", ",", "8387.08`", 
     ",", "8370.43`", ",", "8385.16`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:20:00\"\>", ",", "8384.77`", ",", "8408.66`", 
     ",", "8381.82`", ",", "8398.62`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:30:00\"\>", ",", "8398.23`", ",", "8398.23`", 
     ",", "8358.08`", ",", "8380.46`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:40:00\"\>", ",", "8380.31`", ",", "8407.54`", 
     ",", "8374.65`", ",", "8405.`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T15:50:00\"\>", ",", "8407.23`", ",", "8430.16`", 
     ",", "8392.57`", ",", "8406.83`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-07T16:00:00\"\>", ",", "8407.07`", ",", "8410.57`", 
     ",", "8407.07`", ",", "8409.85`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T09:30:00\"\>", ",", "8410.73`", ",", "8519.77`", 
     ",", "8410.73`", ",", "8518.26`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T09:40:00\"\>", ",", "8520.88`", ",", "8550.91`", 
     ",", "8508.46`", ",", "8540.56`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T09:50:00\"\>", ",", "8540.`", ",", "8551.63`", 
     ",", "8526.86`", ",", "8527.89`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:00:00\"\>", ",", "8528.45`", ",", "8533.71`", 
     ",", "8502.96`", ",", "8510.53`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:10:00\"\>", ",", "8511.01`", ",", "8511.17`", 
     ",", "8467.2`", ",", "8480.18`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:20:00\"\>", ",", "8480.74`", ",", "8482.02`", 
     ",", "8453.18`", ",", "8467.2`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:30:00\"\>", ",", "8468.32`", ",", "8490.62`", 
     ",", "8454.54`", ",", "8481.3`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:40:00\"\>", ",", "8481.7`", ",", "8492.93`", 
     ",", "8472.54`", ",", "8480.66`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T10:50:00\"\>", ",", "8480.02`", ",", "8486.4`", 
     ",", "8463.46`", ",", "8473.97`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:00:00\"\>", ",", "8473.89`", ",", "8504.71`", 
     ",", "8473.09`", ",", "8503.44`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:10:00\"\>", ",", "8503.52`", ",", "8513.95`", 
     ",", "8493.01`", ",", "8507.66`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:20:00\"\>", ",", "8507.5`", ",", "8507.5`", 
     ",", "8481.38`", ",", "8486.87`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:30:00\"\>", ",", "8487.43`", ",", "8495.87`", 
     ",", "8484.64`", ",", "8485.76`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:40:00\"\>", ",", "8485.44`", ",", "8509.33`", 
     ",", "8484.88`", ",", "8502.48`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T11:50:00\"\>", ",", "8502.72`", ",", "8524.71`", 
     ",", "8502.72`", ",", "8524.23`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:00:00\"\>", ",", "8525.26`", ",", "8536.02`", 
     ",", "8517.22`", ",", "8530.36`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:10:00\"\>", ",", "8530.12`", ",", "8548.68`", 
     ",", "8528.21`", ",", "8547.33`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:20:00\"\>", ",", "8547.41`", ",", "8548.76`", 
     ",", "8525.5`", ",", "8525.5`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:30:00\"\>", ",", "8525.58`", ",", "8531.`", 
     ",", "8522.32`", ",", "8528.85`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:40:00\"\>", ",", "8529.17`", ",", "8539.28`", 
     ",", "8514.67`", ",", "8516.02`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T12:50:00\"\>", ",", "8516.18`", ",", "8548.6`", 
     ",", "8514.51`", ",", "8546.37`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:00:00\"\>", ",", "8545.49`", ",", "8546.05`", 
     ",", "8530.84`", ",", "8537.93`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:10:00\"\>", ",", "8538.01`", ",", "8542.39`", 
     ",", "8530.2`", ",", "8535.46`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:20:00\"\>", ",", "8535.54`", ",", "8547.17`", 
     ",", "8529.09`", ",", "8529.49`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:30:00\"\>", ",", "8529.57`", ",", "8529.96`", 
     ",", "8512.68`", ",", "8524.95`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:40:00\"\>", ",", "8525.18`", ",", "8539.28`", 
     ",", "8515.31`", ",", "8534.5`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T13:50:00\"\>", ",", "8535.3`", ",", "8535.94`", 
     ",", "8514.11`", ",", "8516.66`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:00:00\"\>", ",", "8517.78`", ",", "8542.71`", 
     ",", "8517.62`", ",", "8536.73`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:10:00\"\>", ",", "8536.65`", ",", "8563.81`", 
     ",", "8536.57`", ",", "8559.67`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:20:00\"\>", ",", "8559.59`", ",", "8564.45`", 
     ",", "8552.74`", ",", "8552.74`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:30:00\"\>", ",", "8553.7`", ",", "8566.52`", 
     ",", "8553.7`", ",", "8563.34`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:40:00\"\>", ",", "8563.57`", ",", "8583.33`", 
     ",", "8551.55`", ",", "8582.29`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T14:50:00\"\>", ",", "8582.69`", ",", "8587.23`", 
     ",", "8572.5`", ",", "8572.97`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:00:00\"\>", ",", "8573.77`", ",", "8580.46`", 
     ",", "8549.8`", ",", "8559.67`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:10:00\"\>", ",", "8558.08`", ",", "8559.19`", 
     ",", "8535.14`", ",", "8541.35`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:20:00\"\>", ",", "8542.79`", ",", "8566.04`", 
     ",", "8542.79`", ",", "8549.88`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:30:00\"\>", ",", "8550.59`", ",", "8557.36`", 
     ",", "8535.62`", ",", "8536.02`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:40:00\"\>", ",", "8535.94`", ",", "8561.42`", 
     ",", "8535.94`", ",", "8545.26`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T15:50:00\"\>", ",", "8543.74`", ",", "8574.65`", 
     ",", "8521.6`", ",", "8572.97`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-08T16:00:00\"\>", ",", "8575.68`", ",", "8575.68`", 
     ",", "8574.09`", ",", "8574.65`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T09:30:00\"\>", ",", "8569.23`", ",", "8569.23`", 
     ",", "8456.85`", ",", "8463.38`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T09:40:00\"\>", ",", "8467.68`", ",", "8471.9`", 
     ",", "8423.16`", ",", "8424.83`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T09:50:00\"\>", ",", "8425.39`", ",", "8452.7`", 
     ",", "8425.39`", ",", "8448.96`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:00:00\"\>", ",", "8449.12`", ",", "8458.6`", 
     ",", "8435.58`", ",", "8455.41`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:10:00\"\>", ",", "8455.89`", ",", "8467.12`", 
     ",", "8451.03`", ",", "8464.41`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:20:00\"\>", ",", "8464.33`", ",", "8473.97`", 
     ",", "8448.16`", ",", "8448.16`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:30:00\"\>", ",", "8448.72`", ",", "8474.53`", 
     ",", "8436.46`", ",", "8466.25`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:40:00\"\>", ",", "8466.8`", ",", "8483.29`", 
     ",", "8466.09`", ",", "8475.25`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T10:50:00\"\>", ",", "8474.93`", ",", "8477.79`", 
     ",", "8463.3`", ",", "8467.28`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:00:00\"\>", ",", "8467.12`", ",", "8475.17`", 
     ",", "8455.41`", ",", "8462.66`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:10:00\"\>", ",", "8462.82`", ",", "8486.24`", 
     ",", "8461.23`", ",", "8479.86`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:20:00\"\>", ",", "8479.31`", ",", "8479.94`", 
     ",", "8460.99`", ",", "8464.09`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:30:00\"\>", ",", "8464.49`", ",", "8470.86`", 
     ",", "8451.83`", ",", "8456.53`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:40:00\"\>", ",", "8456.69`", ",", "8457.56`", 
     ",", "8448.32`", ",", "8455.57`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T11:50:00\"\>", ",", "8455.25`", ",", "8458.44`", 
     ",", "8444.26`", ",", "8444.82`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:00:00\"\>", ",", "8443.31`", ",", "8451.51`", 
     ",", "8436.46`", ",", "8448.16`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:10:00\"\>", ",", "8446.81`", ",", "8460.43`", 
     ",", "8444.74`", ",", "8459.4`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:20:00\"\>", ",", "8459.71`", ",", "8467.92`", 
     ",", "8457.24`", ",", "8462.74`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:30:00\"\>", ",", "8462.5`", ",", "8467.44`", 
     ",", "8459.08`", ",", "8465.77`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:40:00\"\>", ",", "8465.61`", ",", "8467.6`", 
     ",", "8457.96`", ",", "8457.96`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T12:50:00\"\>", ",", "8458.6`", ",", "8458.6`", 
     ",", "8446.09`", ",", "8446.41`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:00:00\"\>", ",", "8446.49`", ",", "8448.24`", 
     ",", "8432.63`", ",", "8443.94`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:10:00\"\>", ",", "8444.18`", ",", "8462.74`", 
     ",", "8443.86`", ",", "8462.5`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:20:00\"\>", ",", "8462.42`", ",", "8464.65`", 
     ",", "8454.62`", ",", "8461.39`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:30:00\"\>", ",", "8460.83`", ",", "8478.03`", 
     ",", "8454.22`", ",", "8476.04`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:40:00\"\>", ",", "8476.52`", ",", "8478.83`", 
     ",", "8471.5`", ",", "8477.24`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T13:50:00\"\>", ",", "8477.`", ",", "8480.34`", 
     ",", "8466.48`", ",", "8470.07`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:00:00\"\>", ",", "8469.99`", ",", "8469.99`", 
     ",", "8450.63`", ",", "8457.56`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:10:00\"\>", ",", "8457.4`", ",", "8459.08`", 
     ",", "8448.48`", ",", "8453.42`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:20:00\"\>", ",", "8454.14`", ",", "8459.95`", 
     ",", "8446.49`", ",", "8459.55`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:30:00\"\>", ",", "8459.24`", ",", "8469.83`", 
     ",", "8449.84`", ",", "8456.21`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:40:00\"\>", ",", "8456.61`", ",", "8468.87`", 
     ",", "8450.95`", ",", "8468.87`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T14:50:00\"\>", ",", "8469.11`", ",", "8473.49`", 
     ",", "8458.12`", ",", "8459.71`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:00:00\"\>", ",", "8459.`", ",", "8462.98`", 
     ",", "8442.91`", ",", "8453.18`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:10:00\"\>", ",", "8452.31`", ",", "8465.77`", 
     ",", "8450.87`", ",", "8454.06`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:20:00\"\>", ",", "8454.3`", ",", "8455.65`", 
     ",", "8429.77`", ",", "8440.36`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:30:00\"\>", ",", "8440.52`", ",", "8440.52`", 
     ",", "8421.72`", ",", "8422.84`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:40:00\"\>", ",", "8422.76`", ",", "8445.38`", 
     ",", "8421.48`", ",", "8439.72`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T15:50:00\"\>", ",", "8440.04`", ",", "8448.24`", 
     ",", "8410.65`", ",", "8421.32`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-11T16:00:00\"\>", ",", "8420.53`", ",", "8421.64`", 
     ",", "8418.77`", ",", "8418.77`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T09:30:00\"\>", ",", "8419.17`", ",", "8480.9`", 
     ",", "8419.17`", ",", "8441.`", ",", "301", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T09:40:00\"\>", ",", "8438.93`", ",", "8442.59`", 
     ",", "8413.12`", ",", "8421.64`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T09:50:00\"\>", ",", "8420.37`", ",", "8438.29`", 
     ",", "8412.24`", ",", "8430.88`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:00:00\"\>", ",", "8431.44`", ",", "8431.44`", 
     ",", "8414.63`", ",", "8419.65`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:10:00\"\>", ",", "8418.3`", ",", "8418.3`", 
     ",", "8391.93`", ",", "8399.42`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:20:00\"\>", ",", "8399.66`", ",", "8416.31`", 
     ",", "8398.94`", ",", "8411.21`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:30:00\"\>", ",", "8410.01`", ",", "8410.01`", 
     ",", "8386.36`", ",", "8393.13`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:40:00\"\>", ",", "8392.25`", ",", "8412.8`", 
     ",", "8386.04`", ",", "8412.8`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T10:50:00\"\>", ",", "8413.2`", ",", "8421.72`", 
     ",", "8401.09`", ",", "8421.72`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:00:00\"\>", ",", "8423.`", ",", "8430.24`", 
     ",", "8418.38`", ",", "8418.85`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:10:00\"\>", ",", "8418.46`", ",", "8428.49`", 
     ",", "8410.57`", ",", "8416.54`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:20:00\"\>", ",", "8417.34`", ",", "8417.42`", 
     ",", "8393.77`", ",", "8404.36`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:30:00\"\>", ",", "8404.28`", ",", "8415.51`", 
     ",", "8397.35`", ",", "8410.73`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:40:00\"\>", ",", "8409.14`", ",", "8412.48`", 
     ",", "8393.05`", ",", "8393.05`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T11:50:00\"\>", ",", "8392.41`", ",", "8398.15`", 
     ",", "8389.31`", ",", "8397.11`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:00:00\"\>", ",", "8397.03`", ",", "8408.5`", 
     ",", "8386.44`", ",", "8387.95`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:10:00\"\>", ",", "8387.08`", ",", "8391.3`", 
     ",", "8370.67`", ",", "8372.5`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:20:00\"\>", ",", "8372.42`", ",", "8381.02`", 
     ",", "8369.79`", ",", "8379.19`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:30:00\"\>", ",", "8379.75`", ",", "8387.95`", 
     ",", "8375.61`", ",", "8381.58`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:40:00\"\>", ",", "8382.06`", ",", "8388.19`", 
     ",", "8375.61`", ",", "8386.28`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T12:50:00\"\>", ",", "8385.72`", ",", "8385.72`", 
     ",", "8366.13`", ",", "8367.24`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:00:00\"\>", ",", "8366.69`", ",", "8379.51`", 
     ",", "8365.73`", ",", "8378.71`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:10:00\"\>", ",", "8378.79`", ",", "8388.83`", 
     ",", "8376.96`", ",", "8386.6`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:20:00\"\>", ",", "8386.76`", ",", "8394.96`", 
     ",", "8385.16`", ",", "8388.51`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:30:00\"\>", ",", "8388.67`", ",", "8404.92`", 
     ",", "8384.77`", ",", "8401.89`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:40:00\"\>", ",", "8401.33`", ",", "8402.45`", 
     ",", "8391.62`", ",", "8397.59`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T13:50:00\"\>", ",", "8398.78`", ",", "8401.17`", 
     ",", "8384.77`", ",", "8387.23`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:00:00\"\>", ",", "8387.15`", ",", "8407.54`", 
     ",", "8382.69`", ",", "8402.29`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:10:00\"\>", ",", "8402.13`", ",", "8410.01`", 
     ",", "8400.54`", ",", "8408.82`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:20:00\"\>", ",", "8408.9`", ",", "8416.31`", 
     ",", "8400.46`", ",", "8403.64`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:30:00\"\>", ",", "8403.88`", ",", "8428.17`", 
     ",", "8398.07`", ",", "8427.93`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:40:00\"\>", ",", "8427.85`", ",", "8440.6`", 
     ",", "8426.82`", ",", "8437.33`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T14:50:00\"\>", ",", "8436.85`", ",", "8470.55`", 
     ",", "8436.85`", ",", "8463.62`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:00:00\"\>", ",", "8465.29`", ",", "8489.18`", 
     ",", "8465.29`", ",", "8478.83`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:10:00\"\>", ",", "8478.51`", ",", "8481.3`", 
     ",", "8465.45`", ",", "8475.48`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:20:00\"\>", ",", "8475.4`", ",", "8516.18`", 
     ",", "8469.99`", ",", "8498.5`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:30:00\"\>", ",", "8498.74`", ",", "8503.52`", 
     ",", "8480.74`", ",", "8484.72`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:40:00\"\>", ",", "8485.04`", ",", "8489.26`", 
     ",", "8468.95`", ",", "8474.77`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T15:50:00\"\>", ",", "8472.7`", ",", "8478.51`", 
     ",", "8451.35`", ",", "8468.32`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-12T16:00:00\"\>", ",", "8468.4`", ",", "8469.43`", 
     ",", "8468.32`", ",", "8469.11`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T09:30:00\"\>", ",", "8461.88`", ",", "8461.88`", 
     ",", "8306.39`", ",", "8309.82`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T09:40:00\"\>", ",", "8309.18`", ",", "8331.64`", 
     ",", "8303.68`", ",", "8328.53`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T09:50:00\"\>", ",", "8329.01`", ",", "8339.13`", 
     ",", "8321.45`", ",", "8334.67`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:00:00\"\>", ",", "8332.12`", ",", "8332.12`", 
     ",", "8310.21`", ",", "8321.6`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:10:00\"\>", ",", "8321.84`", ",", "8354.58`", 
     ",", "8321.76`", ",", "8343.51`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:20:00\"\>", ",", "8343.83`", ",", "8357.37`", 
     ",", "8334.03`", ",", "8357.37`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:30:00\"\>", ",", "8360.95`", ",", "8366.21`", 
     ",", "8345.9`", ",", "8347.57`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:40:00\"\>", ",", "8347.49`", ",", "8347.49`", 
     ",", "8318.82`", ",", "8328.85`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T10:50:00\"\>", ",", "8328.69`", ",", "8334.83`", 
     ",", "8313.08`", ",", "8313.08`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:00:00\"\>", ",", "8312.92`", ",", "8323.28`", 
     ",", "8305.2`", ",", "8307.98`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:10:00\"\>", ",", "8308.22`", ",", "8320.81`", 
     ",", "8307.51`", ",", "8320.81`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:20:00\"\>", ",", "8321.84`", ",", "8330.05`", 
     ",", "8321.84`", ",", "8326.94`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:30:00\"\>", ",", "8325.03`", ",", "8325.67`", 
     ",", "8296.12`", ",", "8296.83`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:40:00\"\>", ",", "8296.6`", ",", "8302.49`", 
     ",", "8290.14`", ",", "8301.93`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T11:50:00\"\>", ",", "8301.61`", ",", "8309.5`", 
     ",", "8301.53`", ",", "8302.89`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:00:00\"\>", ",", "8301.29`", ",", "8316.35`", 
     ",", "8300.34`", ",", "8311.33`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:10:00\"\>", ",", "8311.09`", ",", "8333.15`", 
     ",", "8311.09`", ",", "8325.03`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:20:00\"\>", ",", "8325.35`", ",", "8329.25`", 
     ",", "8320.57`", ",", "8324.07`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:30:00\"\>", ",", "8324.47`", ",", "8325.03`", 
     ",", "8309.98`", ",", "8319.77`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:40:00\"\>", ",", "8319.37`", ",", "8322.8`", 
     ",", "8308.22`", ",", "8310.69`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T12:50:00\"\>", ",", "8310.14`", ",", "8312.13`", 
     ",", "8289.43`", ",", "8289.43`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:00:00\"\>", ",", "8289.35`", ",", "8295.08`", 
     ",", "8271.59`", ",", "8272.94`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:10:00\"\>", ",", "8273.02`", ",", "8284.65`", 
     ",", "8271.11`", ",", "8284.65`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:20:00\"\>", ",", "8283.77`", ",", "8288.23`", 
     ",", "8272.46`", ",", "8272.86`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:30:00\"\>", ",", "8273.5`", ",", "8287.99`", 
     ",", "8262.74`", ",", "8285.52`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:40:00\"\>", ",", "8285.6`", ",", "8288.79`", 
     ",", "8277.64`", ",", "8285.84`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T13:50:00\"\>", ",", "8285.92`", ",", "8309.9`", 
     ",", "8284.89`", ",", "8309.9`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:00:00\"\>", ",", "8309.82`", ",", "8313.16`", 
     ",", "8288.23`", ",", "8293.57`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:10:00\"\>", ",", "8293.65`", ",", "8301.53`", 
     ",", "8273.98`", ",", "8273.98`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:20:00\"\>", ",", "8273.58`", ",", "8294.6`", 
     ",", "8271.35`", ",", "8288.95`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:30:00\"\>", ",", "8288.55`", ",", "8295.96`", 
     ",", "8281.06`", ",", "8295.48`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:40:00\"\>", ",", "8294.44`", ",", "8299.94`", 
     ",", "8289.51`", ",", "8293.33`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T14:50:00\"\>", ",", "8293.73`", ",", "8316.91`", 
     ",", "8293.57`", ",", "8315.79`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:00:00\"\>", ",", "8315.71`", ",", "8322.4`", 
     ",", "8289.98`", ",", "8298.35`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:10:00\"\>", ",", "8296.75`", ",", "8299.14`", 
     ",", "8280.19`", ",", "8296.91`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:20:00\"\>", ",", "8297.23`", ",", "8310.69`", 
     ",", "8282.58`", ",", "8288.79`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:30:00\"\>", ",", "8289.59`", ",", "8310.45`", 
     ",", "8287.99`", ",", "8294.05`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:40:00\"\>", ",", "8294.68`", ",", "8314.2`", 
     ",", "8276.13`", ",", "8303.92`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T15:50:00\"\>", ",", "8303.05`", ",", "8310.21`", 
     ",", "8280.82`", ",", "8285.6`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-13T16:00:00\"\>", ",", "8284.65`", ",", "8285.36`", 
     ",", "8283.37`", ",", "8284.89`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T09:30:00\"\>", ",", "8285.92`", ",", "8339.53`", 
     ",", "8281.38`", ",", "8305.12`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T09:40:00\"\>", ",", "8303.05`", ",", "8329.81`", 
     ",", "8291.66`", ",", "8291.66`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T09:50:00\"\>", ",", "8290.78`", ",", "8299.86`", 
     ",", "8273.98`", ",", "8294.92`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:00:00\"\>", ",", "8295.48`", ",", "8329.49`", 
     ",", "8295.48`", ",", "8315.47`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:10:00\"\>", ",", "8316.99`", ",", "8332.36`", 
     ",", "8313.56`", ",", "8319.77`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:20:00\"\>", ",", "8319.53`", ",", "8330.45`", 
     ",", "8307.67`", ",", "8309.02`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:30:00\"\>", ",", "8310.21`", ",", "8334.67`", 
     ",", "8307.51`", ",", "8307.67`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:40:00\"\>", ",", "8308.06`", ",", "8312.6`", 
     ",", "8291.9`", ",", "8295.16`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T10:50:00\"\>", ",", "8294.92`", ",", "8314.68`", 
     ",", "8294.76`", ",", "8312.76`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:00:00\"\>", ",", "8312.68`", ",", "8324.79`", 
     ",", "8309.82`", ",", "8320.57`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:10:00\"\>", ",", "8320.73`", ",", "8336.82`", 
     ",", "8320.73`", ",", "8331.48`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:20:00\"\>", ",", "8331.32`", ",", "8339.76`", 
     ",", "8327.74`", ",", "8330.37`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:30:00\"\>", ",", "8330.6`", ",", "8335.14`", 
     ",", "8318.34`", ",", "8325.91`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:40:00\"\>", ",", "8326.22`", ",", "8331.32`", 
     ",", "8316.43`", ",", "8327.5`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T11:50:00\"\>", ",", "8326.94`", ",", "8332.68`", 
     ",", "8320.41`", ",", "8328.61`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:00:00\"\>", ",", "8330.21`", ",", "8343.27`", 
     ",", "8328.93`", ",", "8343.27`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:10:00\"\>", ",", "8343.59`", ",", "8356.89`", 
     ",", "8343.59`", ",", "8348.37`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:20:00\"\>", ",", "8348.13`", ",", "8353.62`", 
     ",", "8345.42`", ",", "8347.65`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:30:00\"\>", ",", "8347.33`", ",", "8363.18`", 
     ",", "8344.3`", ",", "8362.62`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:40:00\"\>", ",", "8362.07`", ",", "8362.07`", 
     ",", "8343.59`", ",", "8350.76`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T12:50:00\"\>", ",", "8350.52`", ",", "8357.92`", 
     ",", "8341.28`", ",", "8341.68`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:00:00\"\>", ",", "8341.6`", ",", "8347.73`", 
     ",", "8337.85`", ",", "8345.42`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:10:00\"\>", ",", "8345.5`", ",", "8347.65`", 
     ",", "8325.35`", ",", "8327.66`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:20:00\"\>", ",", "8327.42`", ",", "8330.29`", 
     ",", "8319.61`", ",", "8325.67`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:30:00\"\>", ",", "8324.71`", ",", "8337.22`", 
     ",", "8323.52`", ",", "8335.78`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:40:00\"\>", ",", "8335.54`", ",", "8338.17`", 
     ",", "8330.84`", ",", "8333.95`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T13:50:00\"\>", ",", "8333.55`", ",", "8342.23`", 
     ",", "8327.82`", ",", "8327.82`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:00:00\"\>", ",", "8327.34`", ",", "8344.38`", 
     ",", "8327.34`", ",", "8341.04`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:10:00\"\>", ",", "8340.96`", ",", "8351.47`", 
     ",", "8339.13`", ",", "8351.47`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:20:00\"\>", ",", "8351.63`", ",", "8360.87`", 
     ",", "8351.39`", ",", "8356.73`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:30:00\"\>", ",", "8356.81`", ",", "8372.02`", 
     ",", "8352.75`", ",", "8371.86`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:40:00\"\>", ",", "8371.7`", ",", "8376.64`", 
     ",", "8360.79`", ",", "8364.22`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T14:50:00\"\>", ",", "8364.3`", ",", "8367.`", ",",
      "8356.97`", ",", "8365.01`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:00:00\"\>", ",", "8365.09`", ",", "8370.67`", 
     ",", "8353.94`", ",", "8356.01`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:10:00\"\>", ",", "8356.17`", ",", "8367.`", 
     ",", "8348.76`", ",", "8363.66`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:20:00\"\>", ",", "8363.42`", ",", "8368.44`", 
     ",", "8354.9`", ",", "8361.03`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:30:00\"\>", ",", "8360.63`", ",", "8360.63`", 
     ",", "8321.37`", ",", "8325.19`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:40:00\"\>", ",", "8325.03`", ",", "8352.67`", 
     ",", "8325.03`", ",", "8346.45`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T15:50:00\"\>", ",", "8347.73`", ",", "8355.3`", 
     ",", "8322.24`", ",", "8324.39`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-14T16:00:00\"\>", ",", "8324.55`", ",", "8331.32`", 
     ",", "8324.55`", ",", "8331.32`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T09:30:00\"\>", ",", "8326.22`", ",", "8364.3`", 
     ",", "8319.53`", ",", "8346.14`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T09:40:00\"\>", ",", "8346.3`", ",", "8355.53`", 
     ",", "8326.06`", ",", "8354.66`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T09:50:00\"\>", ",", "8354.98`", ",", "8364.77`", 
     ",", "8342.07`", ",", "8342.71`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:00:00\"\>", ",", "8344.3`", ",", "8345.18`", 
     ",", "8323.76`", ",", "8338.89`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:10:00\"\>", ",", "8339.05`", ",", "8340.16`", 
     ",", "8318.1`", ",", "8337.3`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:20:00\"\>", ",", "8336.66`", ",", "8355.85`", 
     ",", "8330.21`", ",", "8352.19`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:30:00\"\>", ",", "8351.95`", ",", "8386.44`", 
     ",", "8346.45`", ",", "8386.44`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:40:00\"\>", ",", "8387.79`", ",", "8394.16`", 
     ",", "8378.87`", ",", "8381.9`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T10:50:00\"\>", ",", "8381.98`", ",", "8385.24`", 
     ",", "8371.78`", ",", "8375.37`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:00:00\"\>", ",", "8375.21`", ",", "8382.69`", 
     ",", "8367.8`", ",", "8368.28`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:10:00\"\>", ",", "8368.2`", ",", "8378.07`", 
     ",", "8367.4`", ",", "8374.17`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:20:00\"\>", ",", "8374.33`", ",", "8374.33`", 
     ",", "8355.93`", ",", "8363.74`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:30:00\"\>", ",", "8364.38`", ",", "8366.05`", 
     ",", "8345.18`", ",", "8345.34`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:40:00\"\>", ",", "8345.02`", ",", "8345.02`", 
     ",", "8325.59`", ",", "8327.42`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T11:50:00\"\>", ",", "8327.34`", ",", "8327.98`", 
     ",", "8295.4`", ",", "8295.96`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:00:00\"\>", ",", "8294.29`", ",", "8300.5`", 
     ",", "8288.15`", ",", "8294.76`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:10:00\"\>", ",", "8294.84`", ",", "8301.06`", 
     ",", "8285.52`", ",", "8286.48`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:20:00\"\>", ",", "8286.64`", ",", "8302.17`", 
     ",", "8286.`", ",", "8296.36`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:30:00\"\>", ",", "8296.12`", ",", "8299.62`", 
     ",", "8288.87`", ",", "8298.27`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:40:00\"\>", ",", "8298.51`", ",", "8298.98`", 
     ",", "8281.78`", ",", "8284.57`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T12:50:00\"\>", ",", "8285.92`", ",", "8292.45`", 
     ",", "8274.69`", ",", "8281.22`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:00:00\"\>", ",", "8280.9`", ",", "8289.19`", 
     ",", "8277.16`", ",", "8288.15`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:10:00\"\>", ",", "8288.23`", ",", "8295.88`", 
     ",", "8278.99`", ",", "8293.01`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:20:00\"\>", ",", "8293.33`", ",", "8293.41`", 
     ",", "8275.97`", ",", "8277.96`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:30:00\"\>", ",", "8277.88`", ",", "8286.4`", 
     ",", "8271.82`", ",", "8275.73`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:40:00\"\>", ",", "8275.41`", ",", "8283.53`", 
     ",", "8268.48`", ",", "8283.53`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T13:50:00\"\>", ",", "8283.45`", ",", "8283.93`", 
     ",", "8275.65`", ",", "8279.39`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:00:00\"\>", ",", "8278.12`", ",", "8281.3`", 
     ",", "8265.77`", ",", "8279.71`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:10:00\"\>", ",", "8279.79`", ",", "8287.52`", 
     ",", "8271.9`", ",", "8279.87`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:20:00\"\>", ",", "8280.27`", ",", "8288.63`", 
     ",", "8269.99`", ",", "8275.01`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:30:00\"\>", ",", "8274.69`", ",", "8276.92`", 
     ",", "8258.52`", ",", "8262.27`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:40:00\"\>", ",", "8262.19`", ",", "8263.38`", 
     ",", "8238.05`", ",", "8241.96`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T14:50:00\"\>", ",", "8241.72`", ",", "8250.16`", 
     ",", "8237.5`", ",", "8245.94`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:00:00\"\>", ",", "8245.86`", ",", "8246.89`", 
     ",", "8230.25`", ",", "8238.61`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:10:00\"\>", ",", "8238.85`", ",", "8255.18`", 
     ",", "8237.02`", ",", "8243.31`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:20:00\"\>", ",", "8242.83`", ",", "8273.82`", 
     ",", "8242.51`", ",", "8271.27`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:30:00\"\>", ",", "8272.94`", ",", "8277.16`", 
     ",", "8252.79`", ",", "8255.1`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:40:00\"\>", ",", "8254.94`", ",", "8276.44`", 
     ",", "8247.77`", ",", "8266.25`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T15:50:00\"\>", ",", "8267.68`", ",", "8274.85`", 
     ",", "8257.65`", ",", "8273.5`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-15T16:00:00\"\>", ",", "8273.74`", ",", "8273.74`", 
     ",", "8268.64`", ",", "8268.64`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T09:30:00\"\>", ",", "8270.15`", ",", "8383.97`", 
     ",", "8270.15`", ",", "8379.27`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T09:40:00\"\>", ",", "8380.23`", ",", "8388.99`", 
     ",", "8355.69`", ",", "8384.45`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T09:50:00\"\>", ",", "8384.05`", ",", "8395.52`", 
     ",", "8375.53`", ",", "8377.6`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:00:00\"\>", ",", "8378.31`", ",", "8386.84`", 
     ",", "8366.05`", ",", "8375.29`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:10:00\"\>", ",", "8374.33`", ",", "8385.08`", 
     ",", "8361.75`", ",", "8383.57`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:20:00\"\>", ",", "8384.37`", ",", "8407.78`", 
     ",", "8384.37`", ",", "8399.42`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:30:00\"\>", ",", "8401.49`", ",", "8414.24`", 
     ",", "8397.19`", ",", "8399.9`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:40:00\"\>", ",", "8399.66`", ",", "8425.07`", 
     ",", "8392.73`", ",", "8424.43`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T10:50:00\"\>", ",", "8424.91`", ",", "8426.26`", 
     ",", "8419.33`", ",", "8422.52`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:00:00\"\>", ",", "8422.28`", ",", "8428.09`", 
     ",", "8415.27`", ",", "8416.39`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:10:00\"\>", ",", "8415.75`", ",", "8423.08`", 
     ",", "8412.88`", ",", "8416.7`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:20:00\"\>", ",", "8416.39`", ",", "8424.99`", 
     ",", "8414.71`", ",", "8424.99`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:30:00\"\>", ",", "8427.3`", ",", "8432.95`", 
     ",", "8424.35`", ",", "8427.78`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:40:00\"\>", ",", "8427.38`", ",", "8432.24`", 
     ",", "8421.48`", ",", "8421.72`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T11:50:00\"\>", ",", "8421.24`", ",", "8434.7`", 
     ",", "8416.54`", ",", "8416.7`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:00:00\"\>", ",", "8418.46`", ",", "8420.61`", 
     ",", "8414.47`", ",", "8417.5`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:10:00\"\>", ",", "8417.34`", ",", "8419.73`", 
     ",", "8413.92`", ",", "8416.23`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:20:00\"\>", ",", "8416.31`", ",", "8434.23`", 
     ",", "8414.08`", ",", "8433.51`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:30:00\"\>", ",", "8433.59`", ",", "8434.39`", 
     ",", "8428.09`", ",", "8431.68`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:40:00\"\>", ",", "8431.52`", ",", "8457.01`", 
     ",", "8431.52`", ",", "8455.57`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T12:50:00\"\>", ",", "8455.89`", ",", "8458.04`", 
     ",", "8450.47`", ",", "8456.29`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:00:00\"\>", ",", "8456.37`", ",", "8459.4`", 
     ",", "8441.32`", ",", "8446.33`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:10:00\"\>", ",", "8446.17`", ",", "8452.63`", 
     ",", "8440.92`", ",", "8446.25`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:20:00\"\>", ",", "8446.41`", ",", "8446.49`", 
     ",", "8435.02`", ",", "8443.55`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:30:00\"\>", ",", "8443.78`", ",", "8447.69`", 
     ",", "8440.6`", ",", "8443.55`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:40:00\"\>", ",", "8443.31`", ",", "8444.1`", 
     ",", "8432.55`", ",", "8437.57`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T13:50:00\"\>", ",", "8437.41`", ",", "8446.65`", 
     ",", "8437.41`", ",", "8439.96`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:00:00\"\>", ",", "8440.28`", ",", "8449.76`", 
     ",", "8440.28`", ",", "8447.45`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:10:00\"\>", ",", "8447.53`", ",", "8454.3`", 
     ",", "8442.59`", ",", "8446.09`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:20:00\"\>", ",", "8445.78`", ",", "8447.45`", 
     ",", "8429.21`", ",", "8436.3`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:30:00\"\>", ",", "8436.46`", ",", "8455.89`", 
     ",", "8436.46`", ",", "8454.3`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:40:00\"\>", ",", "8454.14`", ",", "8478.35`", 
     ",", "8448.09`", ",", "8473.97`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T14:50:00\"\>", ",", "8473.65`", ",", "8479.07`", 
     ",", "8472.46`", ",", "8475.32`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:00:00\"\>", ",", "8474.53`", ",", "8474.53`", 
     ",", "8461.78`", ",", "8472.14`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:10:00\"\>", ",", "8472.3`", ",", "8473.02`", 
     ",", "8457.72`", ",", "8463.38`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:20:00\"\>", ",", "8462.98`", ",", "8468.16`", 
     ",", "8455.57`", ",", "8456.53`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:30:00\"\>", ",", "8456.61`", ",", "8474.53`", 
     ",", "8449.68`", ",", "8474.53`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:40:00\"\>", ",", "8473.73`", ",", "8487.51`", 
     ",", "8459.16`", ",", "8486.16`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T15:50:00\"\>", ",", "8486.79`", ",", "8511.88`", 
     ",", "8485.28`", ",", "8511.17`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-18T16:00:00\"\>", ",", "8510.21`", ",", "8510.37`", 
     ",", "8504.08`", ",", "8504.08`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T09:30:00\"\>", ",", "8502.56`", ",", "8512.36`", 
     ",", "8478.27`", ",", "8507.34`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T09:40:00\"\>", ",", "8502.72`", ",", "8504.56`", 
     ",", "8478.51`", ",", "8489.02`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T09:50:00\"\>", ",", "8488.15`", ",", "8488.23`", 
     ",", "8465.05`", ",", "8467.92`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:00:00\"\>", ",", "8466.72`", ",", "8485.52`", 
     ",", "8463.62`", ",", "8481.86`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:10:00\"\>", ",", "8482.09`", ",", "8499.14`", 
     ",", "8478.91`", ",", "8496.59`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:20:00\"\>", ",", "8495.95`", ",", "8521.28`", 
     ",", "8483.45`", ",", "8521.04`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:30:00\"\>", ",", "8521.2`", ",", "8537.61`", 
     ",", "8521.2`", ",", "8534.98`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:40:00\"\>", ",", "8534.74`", ",", "8537.53`", 
     ",", "8518.1`", ",", "8520.8`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T10:50:00\"\>", ",", "8520.72`", ",", "8525.5`", 
     ",", "8509.89`", ",", "8513.72`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:00:00\"\>", ",", "8513.56`", ",", "8525.18`", 
     ",", "8500.89`", ",", "8505.27`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:10:00\"\>", ",", "8504.87`", ",", "8506.55`", 
     ",", "8494.04`", ",", "8501.69`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:20:00\"\>", ",", "8501.13`", ",", "8510.45`", 
     ",", "8490.3`", ",", "8510.13`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:30:00\"\>", ",", "8510.69`", ",", "8524.23`", 
     ",", "8510.69`", ",", "8522.32`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:40:00\"\>", ",", "8522.24`", ",", "8531.95`", 
     ",", "8516.98`", ",", "8531.95`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T11:50:00\"\>", ",", "8531.8`", ",", "8532.35`", 
     ",", "8517.06`", ",", "8524.23`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:00:00\"\>", ",", "8524.55`", ",", "8526.14`", 
     ",", "8517.78`", ",", "8523.67`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:10:00\"\>", ",", "8523.99`", ",", "8528.77`", 
     ",", "8512.84`", ",", "8526.62`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:20:00\"\>", ",", "8526.46`", ",", "8534.1`", 
     ",", "8520.17`", ",", "8530.92`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:30:00\"\>", ",", "8530.84`", ",", "8531.24`", 
     ",", "8514.59`", ",", "8516.1`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:40:00\"\>", ",", "8516.42`", ",", "8520.41`", 
     ",", "8513.56`", ",", "8514.91`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T12:50:00\"\>", ",", "8514.67`", ",", "8515.07`", 
     ",", "8503.92`", ",", "8503.92`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:00:00\"\>", ",", "8503.68`", ",", "8520.49`", 
     ",", "8503.68`", ",", "8519.13`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:10:00\"\>", ",", "8520.01`", ",", "8526.78`", 
     ",", "8513.08`", ",", "8526.38`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:20:00\"\>", ",", "8526.3`", ",", "8535.06`", 
     ",", "8525.66`", ",", "8530.68`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:30:00\"\>", ",", "8531.16`", ",", "8534.42`", 
     ",", "8517.78`", ",", "8525.82`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:40:00\"\>", ",", "8526.94`", ",", "8529.88`", 
     ",", "8516.5`", ",", "8520.96`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T13:50:00\"\>", ",", "8520.8`", ",", "8523.59`", 
     ",", "8506.87`", ",", "8513.79`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:00:00\"\>", ",", "8513.87`", ",", "8515.63`", 
     ",", "8495.56`", ",", "8506.55`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:10:00\"\>", ",", "8506.95`", ",", "8512.84`", 
     ",", "8500.41`", ",", "8512.84`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:20:00\"\>", ",", "8512.92`", ",", "8521.76`", 
     ",", "8512.28`", ",", "8520.01`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:30:00\"\>", ",", "8520.09`", ",", "8527.57`", 
     ",", "8517.78`", ",", "8523.19`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:40:00\"\>", ",", "8523.27`", ",", "8534.98`", 
     ",", "8521.68`", ",", "8530.04`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T14:50:00\"\>", ",", "8531.56`", ",", "8532.67`", 
     ",", "8522.56`", ",", "8523.19`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:00:00\"\>", ",", "8524.55`", ",", "8533.47`", 
     ",", "8523.51`", ",", "8527.18`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:10:00\"\>", ",", "8527.33`", ",", "8528.29`", 
     ",", "8515.95`", ",", "8525.98`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:20:00\"\>", ",", "8525.66`", ",", "8533.47`", 
     ",", "8522.56`", ",", "8529.64`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:30:00\"\>", ",", "8529.72`", ",", "8529.72`", 
     ",", "8503.36`", ",", "8515.23`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:40:00\"\>", ",", "8514.91`", ",", "8520.09`", 
     ",", "8491.41`", ",", "8491.41`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T15:50:00\"\>", ",", "8489.66`", ",", "8502.41`", 
     ",", "8470.15`", ",", "8476.28`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-19T16:00:00\"\>", ",", "8476.36`", ",", "8476.68`", 
     ",", "8474.69`", ",", "8474.85`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T09:30:00\"\>", ",", "8471.9`", ",", "8555.93`", 
     ",", "8471.9`", ",", "8527.89`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T09:40:00\"\>", ",", "8527.49`", ",", "8563.18`", 
     ",", "8524.79`", ",", "8559.11`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T09:50:00\"\>", ",", "8560.23`", ",", "8576.64`", 
     ",", "8555.69`", ",", "8564.69`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:00:00\"\>", ",", "8566.76`", ",", "8578.39`", 
     ",", "8566.76`", ",", "8572.57`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:10:00\"\>", ",", "8572.18`", ",", "8578.55`", 
     ",", "8561.74`", ",", "8577.51`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:20:00\"\>", ",", "8576.96`", ",", "8591.77`", 
     ",", "8571.3`", ",", "8571.62`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:30:00\"\>", ",", "8572.73`", ",", "8577.04`", 
     ",", "8566.6`", ",", "8568.51`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:40:00\"\>", ",", "8568.67`", ",", "8574.81`", 
     ",", "8557.84`", ",", "8562.46`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T10:50:00\"\>", ",", "8563.57`", ",", "8566.36`", 
     ",", "8549.`", ",", "8552.74`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:00:00\"\>", ",", "8552.66`", ",", "8560.71`", 
     ",", "8543.9`", ",", "8543.9`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:10:00\"\>", ",", "8543.11`", ",", "8545.18`", 
     ",", "8531.56`", ",", "8537.85`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:20:00\"\>", ",", "8537.53`", ",", "8540.32`", 
     ",", "8523.43`", ",", "8529.49`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:30:00\"\>", ",", "8530.2`", ",", "8538.01`", 
     ",", "8528.53`", ",", "8534.18`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:40:00\"\>", ",", "8533.39`", ",", "8541.27`", 
     ",", "8523.91`", ",", "8524.23`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T11:50:00\"\>", ",", "8524.39`", ",", "8530.52`", 
     ",", "8488.39`", ",", "8488.63`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:00:00\"\>", ",", "8487.83`", ",", "8502.48`", 
     ",", "8486.63`", ",", "8487.75`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:10:00\"\>", ",", "8487.35`", ",", "8498.74`", 
     ",", "8483.61`", ",", "8495.64`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:20:00\"\>", ",", "8496.03`", ",", "8497.55`", 
     ",", "8485.92`", ",", "8494.28`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:30:00\"\>", ",", "8494.04`", ",", "8496.27`", 
     ",", "8473.57`", ",", "8480.74`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:40:00\"\>", ",", "8480.58`", ",", "8492.61`", 
     ",", "8478.67`", ",", "8486.56`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T12:50:00\"\>", ",", "8486.16`", ",", "8490.78`", 
     ",", "8473.25`", ",", "8473.33`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:00:00\"\>", ",", "8473.17`", ",", "8486.08`", 
     ",", "8468.63`", ",", "8482.57`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:10:00\"\>", ",", "8482.41`", ",", "8491.49`", 
     ",", "8481.62`", ",", "8482.65`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:20:00\"\>", ",", "8482.73`", ",", "8497.94`", 
     ",", "8473.97`", ",", "8496.11`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:30:00\"\>", ",", "8495.16`", ",", "8506.39`", 
     ",", "8489.74`", ",", "8502.64`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:40:00\"\>", ",", "8502.72`", ",", "8517.54`", 
     ",", "8502.72`", ",", "8513.`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T13:50:00\"\>", ",", "8513.16`", ",", "8515.07`", 
     ",", "8508.78`", ",", "8512.28`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:00:00\"\>", ",", "8512.2`", ",", "8512.2`", 
     ",", "8476.92`", ",", "8497.07`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:10:00\"\>", ",", "8496.83`", ",", "8518.49`", 
     ",", "8485.04`", ",", "8518.49`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:20:00\"\>", ",", "8518.26`", ",", "8529.33`", 
     ",", "8512.6`", ",", "8513.79`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:30:00\"\>", ",", "8513.24`", ",", "8514.19`", 
     ",", "8486.32`", ",", "8499.94`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:40:00\"\>", ",", "8499.3`", ",", "8520.25`", 
     ",", "8498.66`", ",", "8515.71`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T14:50:00\"\>", ",", "8515.55`", ",", "8525.18`", 
     ",", "8505.27`", ",", "8511.09`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:00:00\"\>", ",", "8510.29`", ",", "8510.29`", 
     ",", "8478.03`", ",", "8478.03`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:10:00\"\>", ",", "8477.56`", ",", "8478.35`", 
     ",", "8452.7`", ",", "8462.02`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:20:00\"\>", ",", "8460.83`", ",", "8460.83`", 
     ",", "8435.5`", ",", "8443.7`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:30:00\"\>", ",", "8444.5`", ",", "8450.55`", 
     ",", "8424.99`", ",", "8440.36`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:40:00\"\>", ",", "8440.12`", ",", "8442.51`", 
     ",", "8405.63`", ",", "8419.65`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T15:50:00\"\>", ",", "8420.37`", ",", "8428.17`", 
     ",", "8408.66`", ",", "8417.82`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-20T16:00:00\"\>", ",", "8418.14`", ",", "8422.28`", 
     ",", "8418.06`", ",", "8422.04`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T09:30:00\"\>", ",", "8416.07`", ",", "8416.07`", 
     ",", "8289.51`", ",", "8296.99`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T09:40:00\"\>", ",", "8295.72`", ",", "8317.14`", 
     ",", "8293.57`", ",", "8314.52`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T09:50:00\"\>", ",", "8316.35`", ",", "8325.19`", 
     ",", "8289.11`", ",", "8289.11`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:00:00\"\>", ",", "8290.86`", ",", "8321.68`", 
     ",", "8288.47`", ",", "8320.17`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:10:00\"\>", ",", "8320.09`", ",", "8320.09`", 
     ",", "8280.82`", ",", "8282.18`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:20:00\"\>", ",", "8279.23`", ",", "8284.81`", 
     ",", "8268.88`", ",", "8271.43`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:30:00\"\>", ",", "8268.24`", ",", "8282.02`", 
     ",", "8266.81`", ",", "8280.51`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:40:00\"\>", ",", "8280.43`", ",", "8289.35`", 
     ",", "8278.12`", ",", "8281.38`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T10:50:00\"\>", ",", "8281.7`", ",", "8281.7`", 
     ",", "8254.62`", ",", "8256.05`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:00:00\"\>", ",", "8255.97`", ",", "8261.39`", 
     ",", "8251.91`", ",", "8257.25`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:10:00\"\>", ",", "8257.89`", ",", "8273.5`", 
     ",", "8257.33`", ",", "8271.75`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:20:00\"\>", ",", "8271.67`", ",", "8276.44`", 
     ",", "8268.96`", ",", "8273.42`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:30:00\"\>", ",", "8272.62`", ",", "8284.17`", 
     ",", "8267.28`", ",", "8281.62`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:40:00\"\>", ",", "8281.7`", ",", "8284.89`", 
     ",", "8277.72`", ",", "8279.79`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T11:50:00\"\>", ",", "8280.03`", ",", "8298.75`", 
     ",", "8279.79`", ",", "8298.75`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:00:00\"\>", ",", "8299.14`", ",", "8304.08`", 
     ",", "8294.37`", ",", "8297.39`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:10:00\"\>", ",", "8297.31`", ",", "8299.38`", 
     ",", "8284.41`", ",", "8285.92`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:20:00\"\>", ",", "8284.25`", ",", "8288.71`", 
     ",", "8274.69`", ",", "8280.03`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:30:00\"\>", ",", "8280.27`", ",", "8294.05`", 
     ",", "8277.16`", ",", "8290.38`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:40:00\"\>", ",", "8291.02`", ",", "8291.02`", 
     ",", "8281.14`", ",", "8290.3`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T12:50:00\"\>", ",", "8290.38`", ",", "8298.03`", 
     ",", "8280.27`", ",", "8296.83`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:00:00\"\>", ",", "8295.64`", ",", "8307.91`", 
     ",", "8283.85`", ",", "8285.21`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:10:00\"\>", ",", "8285.05`", ",", "8288.95`", 
     ",", "8273.66`", ",", "8274.21`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:20:00\"\>", ",", "8273.98`", ",", "8275.33`", 
     ",", "8261.07`", ",", "8266.97`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:30:00\"\>", ",", "8267.52`", ",", "8268.88`", 
     ",", "8255.42`", ",", "8261.87`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:40:00\"\>", ",", "8261.63`", ",", "8280.27`", 
     ",", "8261.47`", ",", "8278.44`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T13:50:00\"\>", ",", "8278.36`", ",", "8279.23`", 
     ",", "8266.97`", ",", "8272.54`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:00:00\"\>", ",", "8272.7`", ",", "8284.97`", 
     ",", "8242.99`", ",", "8245.54`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:10:00\"\>", ",", "8245.46`", ",", "8247.77`", 
     ",", "8228.81`", ",", "8236.94`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:20:00\"\>", ",", "8237.42`", ",", "8244.11`", 
     ",", "8228.89`", ",", "8240.92`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:30:00\"\>", ",", "8240.84`", ",", "8254.94`", 
     ",", "8233.35`", ",", "8243.55`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:40:00\"\>", ",", "8243.47`", ",", "8245.14`", 
     ",", "8223.72`", ",", "8226.35`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T14:50:00\"\>", ",", "8226.43`", ",", "8238.45`", 
     ",", "8221.09`", ",", "8237.42`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:00:00\"\>", ",", "8237.74`", ",", "8251.59`", 
     ",", "8234.47`", ",", "8235.9`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:10:00\"\>", ",", "8235.43`", ",", "8255.5`", 
     ",", "8233.28`", ",", "8250.48`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:20:00\"\>", ",", "8250.08`", ",", "8250.96`", 
     ",", "8233.75`", ",", "8241.8`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:30:00\"\>", ",", "8243.87`", ",", "8271.19`", 
     ",", "8243.87`", ",", "8258.36`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:40:00\"\>", ",", "8257.97`", ",", "8282.1`", 
     ",", "8248.01`", ",", "8271.19`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T15:50:00\"\>", ",", "8271.75`", ",", "8297.07`", 
     ",", "8261.39`", ",", "8292.93`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-21T16:00:00\"\>", ",", "8292.53`", ",", "8293.01`", 
     ",", "8291.74`", ",", "8292.13`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T09:30:00\"\>", ",", "8292.29`", ",", "8328.77`", 
     ",", "8292.29`", ",", "8302.17`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T09:40:00\"\>", ",", "8301.45`", ",", "8307.43`", 
     ",", "8267.68`", ",", "8276.84`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T09:50:00\"\>", ",", "8275.73`", ",", "8282.9`", 
     ",", "8257.17`", ",", "8279.71`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:00:00\"\>", ",", "8279.63`", ",", "8300.9`", 
     ",", "8271.19`", ",", "8290.14`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:10:00\"\>", ",", "8290.06`", ",", "8321.37`", 
     ",", "8286.48`", ",", "8318.58`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:20:00\"\>", ",", "8318.34`", ",", "8329.33`", 
     ",", "8316.11`", ",", "8320.25`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:30:00\"\>", ",", "8319.61`", ",", "8319.61`", 
     ",", "8303.6`", ",", "8312.13`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:40:00\"\>", ",", "8312.05`", ",", "8349.16`", 
     ",", "8310.53`", ",", "8345.58`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T10:50:00\"\>", ",", "8346.06`", ",", "8355.85`", 
     ",", "8342.07`", ",", "8347.09`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:00:00\"\>", ",", "8346.93`", ",", "8355.06`", 
     ",", "8337.06`", ",", "8353.54`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:10:00\"\>", ",", "8353.15`", ",", "8363.9`", 
     ",", "8352.35`", ",", "8361.75`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:20:00\"\>", ",", "8361.67`", ",", "8363.82`", 
     ",", "8349.16`", ",", "8352.75`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:30:00\"\>", ",", "8352.91`", ",", "8357.69`", 
     ",", "8340.72`", ",", "8340.96`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:40:00\"\>", ",", "8340.88`", ",", "8347.33`", 
     ",", "8340.8`", ",", "8345.34`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T11:50:00\"\>", ",", "8345.26`", ",", "8347.09`", 
     ",", "8330.37`", ",", "8336.1`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:00:00\"\>", ",", "8336.26`", ",", "8346.06`", 
     ",", "8336.1`", ",", "8345.98`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:10:00\"\>", ",", "8346.06`", ",", "8352.19`", 
     ",", "8335.86`", ",", "8340.88`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:20:00\"\>", ",", "8341.52`", ",", "8343.19`", 
     ",", "8328.85`", ",", "8328.85`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:30:00\"\>", ",", "8328.45`", ",", "8334.43`", 
     ",", "8324.39`", ",", "8334.43`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:40:00\"\>", ",", "8334.59`", ",", "8339.68`", 
     ",", "8330.84`", ",", "8331.08`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T12:50:00\"\>", ",", "8330.76`", ",", "8351.55`", 
     ",", "8330.76`", ",", "8348.37`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:00:00\"\>", ",", "8348.92`", ",", "8359.6`", 
     ",", "8348.37`", ",", "8349.72`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:10:00\"\>", ",", "8349.88`", ",", "8350.36`", 
     ",", "8342.39`", ",", "8349.24`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:20:00\"\>", ",", "8348.92`", ",", "8358.4`", 
     ",", "8346.3`", ",", "8346.3`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:30:00\"\>", ",", "8346.38`", ",", "8347.33`", 
     ",", "8320.57`", ",", "8325.67`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:40:00\"\>", ",", "8325.59`", ",", "8341.36`", 
     ",", "8323.12`", ",", "8335.46`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T13:50:00\"\>", ",", "8336.34`", ",", "8340.4`", 
     ",", "8327.74`", ",", "8340.16`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:00:00\"\>", ",", "8340.08`", ",", "8349.08`", 
     ",", "8339.68`", ",", "8342.07`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:10:00\"\>", ",", "8341.91`", ",", "8345.66`", 
     ",", "8334.67`", ",", "8341.76`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:20:00\"\>", ",", "8341.6`", ",", "8348.61`", 
     ",", "8339.53`", ",", "8344.38`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:30:00\"\>", ",", "8342.71`", ",", "8347.73`", 
     ",", "8338.33`", ",", "8345.74`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:40:00\"\>", ",", "8345.5`", ",", "8355.61`", 
     ",", "8345.5`", ",", "8346.22`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T14:50:00\"\>", ",", "8346.45`", ",", "8356.49`", 
     ",", "8340.64`", ",", "8354.74`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:00:00\"\>", ",", "8354.02`", ",", "8354.1`", 
     ",", "8344.14`", ",", "8349.64`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:10:00\"\>", ",", "8349.56`", ",", "8363.9`", 
     ",", "8335.46`", ",", "8335.62`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:20:00\"\>", ",", "8335.46`", ",", "8335.94`", 
     ",", "8314.6`", ",", "8320.17`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:30:00\"\>", ",", "8319.61`", ",", "8337.22`", 
     ",", "8315.39`", ",", "8329.97`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:40:00\"\>", ",", "8328.85`", ",", "8332.2`", 
     ",", "8295.48`", ",", "8302.41`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T15:50:00\"\>", ",", "8302.49`", ",", "8305.28`", 
     ",", "8273.66`", ",", "8280.27`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-22T16:00:00\"\>", ",", "8280.11`", ",", "8280.19`", 
     ",", "8277.24`", ",", "8277.32`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T09:30:00\"\>", ",", "8275.49`", ",", "8275.49`", 
     ",", "8227.46`", ",", "8251.67`", ",", "575", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T09:40:00\"\>", ",", "8247.93`", ",", "8292.37`", 
     ",", "8247.05`", ",", "8284.33`", ",", "567", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T09:50:00\"\>", ",", "8284.89`", ",", "8299.54`", 
     ",", "8279.39`", ",", "8298.35`", ",", "552", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:00:00\"\>", ",", "8304.72`", ",", "8402.85`", 
     ",", "8304.72`", ",", "8388.35`", ",", "581", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:10:00\"\>", ",", "8390.66`", ",", "8433.11`", 
     ",", "8386.52`", ",", "8421.24`", ",", "565", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:20:00\"\>", ",", "8420.77`", ",", "8433.51`", 
     ",", "8407.78`", ",", "8414.47`", ",", "569", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:30:00\"\>", ",", "8414.87`", ",", "8446.65`", 
     ",", "8414.87`", ",", "8429.61`", ",", "568", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:40:00\"\>", ",", "8429.93`", ",", "8440.36`", 
     ",", "8428.01`", ",", "8433.27`", ",", "549", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T10:50:00\"\>", ",", "8433.35`", ",", "8450.47`", 
     ",", "8431.2`", ",", "8435.98`", ",", "517", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:00:00\"\>", ",", "8435.98`", ",", "8437.49`", 
     ",", "8427.54`", ",", "8428.41`", ",", "529", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:10:00\"\>", ",", "8428.09`", ",", "8437.81`", 
     ",", "8426.42`", ",", "8437.81`", ",", "509", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:20:00\"\>", ",", "8437.65`", ",", "8447.77`", 
     ",", "8435.9`", ",", "8443.15`", ",", "502", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:30:00\"\>", ",", "8443.23`", ",", "8445.06`", 
     ",", "8437.17`", ",", "8438.93`", ",", "501", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:40:00\"\>", ",", "8439.01`", ",", "8456.37`", 
     ",", "8439.01`", ",", "8448.32`", ",", "499", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T11:50:00\"\>", ",", "8447.69`", ",", "8454.3`", 
     ",", "8446.25`", ",", "8447.69`", ",", "495", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:00:00\"\>", ",", "8447.77`", ",", "8448.01`", 
     ",", "8432.63`", ",", "8433.11`", ",", "521", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:10:00\"\>", ",", "8433.35`", ",", "8445.62`", 
     ",", "8433.27`", ",", "8443.23`", ",", "491", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:20:00\"\>", ",", "8443.39`", ",", "8458.68`", 
     ",", "8442.59`", ",", "8455.09`", ",", "484", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:30:00\"\>", ",", "8454.62`", ",", "8461.15`", 
     ",", "8449.44`", ",", "8453.82`", ",", "483", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:40:00\"\>", ",", "8453.66`", ",", "8470.15`", 
     ",", "8449.76`", ",", "8470.15`", ",", "411", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T12:50:00\"\>", ",", "8468.87`", ",", "8488.15`", 
     ",", "8466.88`", ",", "8481.62`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:00:00\"\>", ",", "8482.25`", ",", "8490.22`", 
     ",", "8480.26`", ",", "8483.45`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:10:00\"\>", ",", "8483.53`", ",", "8484.8`", 
     ",", "8475.32`", ",", "8480.9`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:20:00\"\>", ",", "8480.66`", ",", "8481.06`", 
     ",", "8469.59`", ",", "8479.63`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:30:00\"\>", ",", "8479.71`", ",", "8480.18`", 
     ",", "8465.69`", ",", "8472.38`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:40:00\"\>", ",", "8472.22`", ",", "8487.51`", 
     ",", "8469.35`", ",", "8482.89`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T13:50:00\"\>", ",", "8482.73`", ",", "8484.17`", 
     ",", "8476.6`", ",", "8482.73`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:00:00\"\>", ",", "8483.61`", ",", "8491.89`", 
     ",", "8480.02`", ",", "8487.19`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:10:00\"\>", ",", "8487.03`", ",", "8493.17`", 
     ",", "8476.84`", ",", "8485.52`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:20:00\"\>", ",", "8485.2`", ",", "8485.2`", 
     ",", "8469.91`", ",", "8476.84`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:30:00\"\>", ",", "8477.48`", ",", "8489.5`", 
     ",", "8476.76`", ",", "8482.73`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:40:00\"\>", ",", "8484.48`", ",", "8495.71`", 
     ",", "8476.6`", ",", "8476.6`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T14:50:00\"\>", ",", "8476.04`", ",", "8477.`", 
     ",", "8461.71`", ",", "8461.71`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:00:00\"\>", ",", "8461.63`", ",", "8470.23`", 
     ",", "8457.32`", ",", "8469.19`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:10:00\"\>", ",", "8469.11`", ",", "8479.39`", 
     ",", "8444.34`", ",", "8454.06`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:20:00\"\>", ",", "8453.98`", ",", "8463.38`", 
     ",", "8450.08`", ",", "8453.66`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:30:00\"\>", ",", "8452.86`", ",", "8471.02`", 
     ",", "8447.61`", ",", "8466.25`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:40:00\"\>", ",", "8464.97`", ",", "8469.51`", 
     ",", "8425.07`", ",", "8436.54`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T15:50:00\"\>", ",", "8436.93`", ",", "8476.84`", 
     ",", "8431.04`", ",", "8473.25`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-26T16:00:00\"\>", ",", "8473.02`", ",", "8474.53`", 
     ",", "8473.02`", ",", "8473.49`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T09:30:00\"\>", ",", "8473.65`", ",", "8476.84`", 
     ",", "8448.8`", ",", "8458.36`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T09:40:00\"\>", ",", "8458.6`", ",", "8463.3`", 
     ",", "8433.91`", ",", "8462.5`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T09:50:00\"\>", ",", "8463.14`", ",", "8472.46`", 
     ",", "8459.71`", ",", "8466.8`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:00:00\"\>", ",", "8469.67`", ",", "8496.27`", 
     ",", "8420.37`", ",", "8429.77`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:10:00\"\>", ",", "8428.57`", ",", "8447.37`", 
     ",", "8428.57`", ",", "8446.09`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:20:00\"\>", ",", "8446.73`", ",", "8469.03`", 
     ",", "8445.78`", ",", "8459.24`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:30:00\"\>", ",", "8461.86`", ",", "8466.4`", 
     ",", "8446.33`", ",", "8463.7`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:40:00\"\>", ",", "8463.78`", ",", "8465.05`", 
     ",", "8427.78`", ",", "8443.63`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T10:50:00\"\>", ",", "8444.1`", ",", "8457.8`", 
     ",", "8441.55`", ",", "8456.93`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:00:00\"\>", ",", "8456.61`", ",", "8456.93`", 
     ",", "8427.14`", ",", "8432.39`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:10:00\"\>", ",", "8432.55`", ",", "8452.07`", 
     ",", "8431.68`", ",", "8448.72`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:20:00\"\>", ",", "8448.96`", ",", "8456.77`", 
     ",", "8438.61`", ",", "8449.36`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:30:00\"\>", ",", "8449.2`", ",", "8468.71`", 
     ",", "8442.27`", ",", "8466.25`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:40:00\"\>", ",", "8465.69`", ",", "8466.25`", 
     ",", "8455.01`", ",", "8458.44`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T11:50:00\"\>", ",", "8459.24`", ",", "8464.81`", 
     ",", "8446.17`", ",", "8456.77`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:00:00\"\>", ",", "8457.09`", ",", "8468.79`", 
     ",", "8456.61`", ",", "8467.44`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:10:00\"\>", ",", "8467.68`", ",", "8477.4`", 
     ",", "8463.14`", ",", "8469.51`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:20:00\"\>", ",", "8469.99`", ",", "8470.15`", 
     ",", "8458.28`", ",", "8459.08`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:30:00\"\>", ",", "8459.87`", ",", "8460.19`", 
     ",", "8445.78`", ",", "8448.72`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:40:00\"\>", ",", "8448.64`", ",", "8456.69`", 
     ",", "8443.47`", ",", "8453.42`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T12:50:00\"\>", ",", "8453.58`", ",", "8455.25`", 
     ",", "8434.55`", ",", "8439.56`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:00:00\"\>", ",", "8439.88`", ",", "8459.08`", 
     ",", "8438.93`", ",", "8452.23`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:10:00\"\>", ",", "8454.7`", ",", "8457.88`", 
     ",", "8445.46`", ",", "8453.5`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:20:00\"\>", ",", "8453.74`", ",", "8457.01`", 
     ",", "8447.85`", ",", "8449.6`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:30:00\"\>", ",", "8448.88`", ",", "8449.28`", 
     ",", "8413.84`", ",", "8413.84`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:40:00\"\>", ",", "8412.72`", ",", "8420.85`", 
     ",", "8396.87`", ",", "8396.87`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T13:50:00\"\>", ",", "8396.23`", ",", "8396.23`", 
     ",", "8365.25`", ",", "8381.82`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:00:00\"\>", ",", "8380.07`", ",", "8382.06`", 
     ",", "8354.82`", ",", "8358.`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:10:00\"\>", ",", "8357.21`", ",", "8372.9`", 
     ",", "8357.21`", ",", "8369.47`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:20:00\"\>", ",", "8369.63`", ",", "8369.63`", 
     ",", "8339.92`", ",", "8360.79`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:30:00\"\>", ",", "8359.84`", ",", "8373.38`", 
     ",", "8349.96`", ",", "8372.66`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:40:00\"\>", ",", "8372.98`", ",", "8385.56`", 
     ",", "8367.32`", ",", "8382.14`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T14:50:00\"\>", ",", "8381.82`", ",", "8382.3`", 
     ",", "8362.3`", ",", "8362.7`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:00:00\"\>", ",", "8362.62`", ",", "8384.77`", 
     ",", "8360.39`", ",", "8374.09`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:10:00\"\>", ",", "8374.01`", ",", "8374.01`", 
     ",", "8358.08`", ",", "8358.08`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:20:00\"\>", ",", "8356.81`", ",", "8357.45`", 
     ",", "8301.29`", ",", "8307.43`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:30:00\"\>", ",", "8307.75`", ",", "8317.94`", 
     ",", "8289.35`", ",", "8298.11`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:40:00\"\>", ",", "8297.71`", ",", "8330.53`", 
     ",", "8297.71`", ",", "8326.86`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T15:50:00\"\>", ",", "8328.22`", ",", "8331.32`", 
     ",", "8296.83`", ",", "8300.26`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-27T16:00:00\"\>", ",", "8300.9`", ",", "8302.17`", 
     ",", "8300.02`", ",", "8300.02`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T09:30:00\"\>", ",", "8300.5`", ",", "8387.47`", 
     ",", "8300.5`", ",", "8367.72`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T09:40:00\"\>", ",", "8369.15`", ",", "8376.64`", 
     ",", "8354.82`", ",", "8361.27`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T09:50:00\"\>", ",", "8363.74`", ",", "8368.04`", 
     ",", "8341.99`", ",", "8348.29`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:00:00\"\>", ",", "8342.87`", ",", "8342.87`", 
     ",", "8273.42`", ",", "8281.86`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:10:00\"\>", ",", "8282.02`", ",", "8282.02`", 
     ",", "8252.07`", ",", "8268.32`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:20:00\"\>", ",", "8268.`", ",", "8274.77`", 
     ",", "8246.66`", ",", "8266.65`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:30:00\"\>", ",", "8266.73`", ",", "8281.06`", 
     ",", "8258.44`", ",", "8271.51`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:40:00\"\>", ",", "8271.82`", ",", "8292.21`", 
     ",", "8266.33`", ",", "8289.83`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T10:50:00\"\>", ",", "8289.98`", ",", "8307.35`", 
     ",", "8280.35`", ",", "8305.04`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:00:00\"\>", ",", "8308.3`", ",", "8335.07`", 
     ",", "8306.55`", ",", "8330.53`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:10:00\"\>", ",", "8330.6`", ",", "8338.17`", 
     ",", "8322.48`", ",", "8323.76`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:20:00\"\>", ",", "8323.91`", ",", "8340.08`", 
     ",", "8321.84`", ",", "8327.18`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:30:00\"\>", ",", "8327.9`", ",", "8336.1`", 
     ",", "8324.23`", ",", "8324.87`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:40:00\"\>", ",", "8324.95`", ",", "8351.47`", 
     ",", "8317.86`", ",", "8351.23`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T11:50:00\"\>", ",", "8351.31`", ",", "8357.45`", 
     ",", "8342.07`", ",", "8349.72`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:00:00\"\>", ",", "8352.43`", ",", "8356.89`", 
     ",", "8317.22`", ",", "8317.22`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:10:00\"\>", ",", "8317.06`", ",", "8318.34`", 
     ",", "8297.31`", ",", "8298.67`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:20:00\"\>", ",", "8297.79`", ",", "8303.52`", 
     ",", "8293.73`", ",", "8294.37`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:30:00\"\>", ",", "8294.6`", ",", "8300.9`", 
     ",", "8291.5`", ",", "8298.51`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:40:00\"\>", ",", "8298.43`", ",", "8302.81`", 
     ",", "8280.03`", ",", "8283.77`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T12:50:00\"\>", ",", "8283.45`", ",", "8285.05`", 
     ",", "8270.23`", ",", "8277.64`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:00:00\"\>", ",", "8278.67`", ",", "8319.14`", 
     ",", "8271.03`", ",", "8308.14`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:10:00\"\>", ",", "8307.75`", ",", "8380.23`", 
     ",", "8307.27`", ",", "8374.73`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:20:00\"\>", ",", "8377.2`", ",", "8384.53`", 
     ",", "8370.75`", ",", "8370.91`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:30:00\"\>", ",", "8372.02`", ",", "8388.27`", 
     ",", "8364.3`", ",", "8379.27`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:40:00\"\>", ",", "8378.95`", ",", "8386.6`", 
     ",", "8372.18`", ",", "8374.49`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T13:50:00\"\>", ",", "8373.77`", ",", "8380.7`", 
     ",", "8350.04`", ",", "8367.96`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:00:00\"\>", ",", "8367.88`", ",", "8384.05`", 
     ",", "8364.46`", ",", "8381.1`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:10:00\"\>", ",", "8380.94`", ",", "8386.6`", 
     ",", "8365.65`", ",", "8380.15`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:20:00\"\>", ",", "8379.99`", ",", "8422.12`", 
     ",", "8379.99`", ",", "8414.31`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:30:00\"\>", ",", "8415.51`", ",", "8433.91`", 
     ",", "8415.19`", ",", "8423.31`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:40:00\"\>", ",", "8424.91`", ",", "8425.62`", 
     ",", "8397.59`", ",", "8414.08`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T14:50:00\"\>", ",", "8417.9`", ",", "8423.39`", 
     ",", "8404.52`", ",", "8419.09`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:00:00\"\>", ",", "8419.41`", ",", "8428.57`", 
     ",", "8409.14`", ",", "8409.46`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:10:00\"\>", ",", "8409.14`", ",", "8409.3`", 
     ",", "8386.92`", ",", "8397.19`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:20:00\"\>", ",", "8396.95`", ",", "8398.78`", 
     ",", "8373.38`", ",", "8395.52`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:30:00\"\>", ",", "8391.93`", ",", "8408.1`", 
     ",", "8383.89`", ",", "8403.4`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:40:00\"\>", ",", "8404.04`", ",", "8420.45`", 
     ",", "8395.92`", ",", "8408.98`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T15:50:00\"\>", ",", "8408.34`", ",", "8417.1`", 
     ",", "8378.47`", ",", "8401.57`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-28T16:00:00\"\>", ",", "8401.41`", ",", "8403.8`", 
     ",", "8400.85`", ",", "8403.8`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T09:30:00\"\>", ",", "8404.04`", ",", "8436.85`", 
     ",", "8404.04`", ",", "8432.79`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T09:40:00\"\>", ",", "8433.03`", ",", "8462.58`", 
     ",", "8372.34`", ",", "8382.22`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T09:50:00\"\>", ",", "8382.77`", ",", "8411.45`", 
     ",", "8377.68`", ",", "8409.14`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:00:00\"\>", ",", "8409.06`", ",", "8415.99`", 
     ",", "8389.7`", ",", "8408.98`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:10:00\"\>", ",", "8408.34`", ",", "8428.33`", 
     ",", "8394.64`", ",", "8421.48`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:20:00\"\>", ",", "8421.4`", ",", "8439.56`", 
     ",", "8421.16`", ",", "8435.5`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:30:00\"\>", ",", "8435.82`", ",", "8437.81`", 
     ",", "8419.49`", ",", "8431.92`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:40:00\"\>", ",", "8432.32`", ",", "8438.37`", 
     ",", "8404.68`", ",", "8413.84`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T10:50:00\"\>", ",", "8414.`", ",", "8429.61`", 
     ",", "8409.77`", ",", "8419.57`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:00:00\"\>", ",", "8419.49`", ",", "8423.24`", 
     ",", "8403.72`", ",", "8405.71`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:10:00\"\>", ",", "8406.03`", ",", "8406.35`", 
     ",", "8393.77`", ",", "8394.56`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:20:00\"\>", ",", "8392.81`", ",", "8419.41`", 
     ",", "8387.55`", ",", "8387.55`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:30:00\"\>", ",", "8384.21`", ",", "8389.07`", 
     ",", "8374.73`", ",", "8374.81`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:40:00\"\>", ",", "8374.89`", ",", "8393.21`", 
     ",", "8372.5`", ",", "8390.18`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T11:50:00\"\>", ",", "8390.82`", ",", "8392.97`", 
     ",", "8368.92`", ",", "8374.25`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:00:00\"\>", ",", "8374.33`", ",", "8390.58`", 
     ",", "8374.01`", ",", "8387.63`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:10:00\"\>", ",", "8387.31`", ",", "8402.13`", 
     ",", "8386.52`", ",", "8400.77`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:20:00\"\>", ",", "8400.69`", ",", "8406.59`", 
     ",", "8392.57`", ",", "8401.73`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:30:00\"\>", ",", "8401.09`", ",", "8401.09`", 
     ",", "8384.92`", ",", "8386.6`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:40:00\"\>", ",", "8380.38`", ",", "8396.87`", 
     ",", "8379.43`", ",", "8383.25`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T12:50:00\"\>", ",", "8383.41`", ",", "8393.37`", 
     ",", "8383.01`", ",", "8390.18`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:00:00\"\>", ",", "8390.42`", ",", "8406.11`", 
     ",", "8390.42`", ",", "8395.76`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:10:00\"\>", ",", "8396.`", ",", "8400.06`", 
     ",", "8391.77`", ",", "8396.47`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:20:00\"\>", ",", "8396.23`", ",", "8407.07`", 
     ",", "8394.96`", ",", "8406.59`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:30:00\"\>", ",", "8406.67`", ",", "8423.47`", 
     ",", "8406.03`", ",", "8418.46`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:40:00\"\>", ",", "8418.54`", ",", "8423.`", 
     ",", "8415.91`", ",", "8415.91`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T13:50:00\"\>", ",", "8415.59`", ",", "8417.42`", 
     ",", "8408.1`", ",", "8413.92`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:00:00\"\>", ",", "8414.16`", ",", "8419.09`", 
     ",", "8405.71`", ",", "8417.9`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:10:00\"\>", ",", "8418.06`", ",", "8427.14`", 
     ",", "8417.66`", ",", "8424.51`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:20:00\"\>", ",", "8424.67`", ",", "8425.23`", 
     ",", "8411.53`", ",", "8413.36`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:30:00\"\>", ",", "8413.6`", ",", "8416.23`", 
     ",", "8403.88`", ",", "8407.23`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:40:00\"\>", ",", "8407.07`", ",", "8411.53`", 
     ",", "8396.71`", ",", "8398.23`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T14:50:00\"\>", ",", "8398.15`", ",", "8399.98`", 
     ",", "8381.18`", ",", "8399.26`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:00:00\"\>", ",", "8399.02`", ",", "8411.93`", 
     ",", "8390.74`", ",", "8407.54`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:10:00\"\>", ",", "8407.7`", ",", "8417.42`", 
     ",", "8403.08`", ",", "8417.42`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:20:00\"\>", ",", "8417.74`", ",", "8423.63`", 
     ",", "8400.54`", ",", "8420.53`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:30:00\"\>", ",", "8419.49`", ",", "8421.88`", 
     ",", "8403.`", ",", "8420.37`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:40:00\"\>", ",", "8419.73`", ",", "8444.42`", 
     ",", "8417.02`", ",", "8442.59`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T15:50:00\"\>", ",", "8442.19`", ",", "8521.92`", 
     ",", "8439.48`", ",", "8521.92`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-05-29T16:00:00\"\>", ",", "8519.37`", ",", "8519.37`", 
     ",", "8500.33`", ",", "8500.33`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T09:30:00\"\>", ",", "8501.53`", ",", "8611.28`", 
     ",", "8501.53`", ",", "8599.34`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T09:40:00\"\>", ",", "8599.1`", ",", "8638.13`", 
     ",", "8599.1`", ",", "8633.59`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T09:50:00\"\>", ",", "8637.17`", ",", "8657.32`", 
     ",", "8633.35`", ",", "8644.82`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:00:00\"\>", ",", "8644.5`", ",", "8686.15`", 
     ",", "8644.5`", ",", "8674.92`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:10:00\"\>", ",", "8675.24`", ",", "8694.75`", 
     ",", "8673.17`", ",", "8694.44`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:20:00\"\>", ",", "8693.8`", ",", "8701.6`", 
     ",", "8684.8`", ",", "8684.8`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:30:00\"\>", ",", "8684.64`", ",", "8701.37`", 
     ",", "8679.62`", ",", "8699.61`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:40:00\"\>", ",", "8699.45`", ",", "8699.53`", 
     ",", "8686.47`", ",", "8690.06`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T10:50:00\"\>", ",", "8690.45`", ",", "8709.89`", 
     ",", "8690.45`", ",", "8705.51`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:00:00\"\>", ",", "8706.22`", ",", "8716.02`", 
     ",", "8703.2`", ",", "8703.52`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:10:00\"\>", ",", "8703.04`", ",", "8707.5`", 
     ",", "8698.1`", ",", "8700.09`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:20:00\"\>", ",", "8700.33`", ",", "8708.77`", 
     ",", "8700.33`", ",", "8703.44`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:30:00\"\>", ",", "8703.6`", ",", "8706.86`", 
     ",", "8699.37`", ",", "8704.79`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:40:00\"\>", ",", "8704.71`", ",", "8712.12`", 
     ",", "8700.81`", ",", "8707.98`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T11:50:00\"\>", ",", "8708.14`", ",", "8723.19`", 
     ",", "8701.6`", ",", "8718.41`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:00:00\"\>", ",", "8716.98`", ",", "8719.13`", 
     ",", "8708.77`", ",", "8715.62`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:10:00\"\>", ",", "8715.78`", ",", "8718.57`", 
     ",", "8711.56`", ",", "8712.28`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:20:00\"\>", ",", "8712.44`", ",", "8721.12`", 
     ",", "8711.8`", ",", "8719.45`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:30:00\"\>", ",", "8719.76`", ",", "8723.43`", 
     ",", "8717.14`", ",", "8717.61`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:40:00\"\>", ",", "8717.53`", ",", "8723.67`", 
     ",", "8714.51`", ",", "8720.24`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T12:50:00\"\>", ",", "8720.4`", ",", "8723.83`", 
     ",", "8703.91`", ",", "8705.75`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:00:00\"\>", ",", "8705.43`", ",", "8710.45`", 
     ",", "8703.52`", ",", "8709.25`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:10:00\"\>", ",", "8709.81`", ",", "8711.24`", 
     ",", "8704.07`", ",", "8710.21`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:20:00\"\>", ",", "8710.45`", ",", "8715.86`", 
     ",", "8708.69`", ",", "8714.35`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:30:00\"\>", ",", "8714.27`", ",", "8720.16`", 
     ",", "8713.47`", ",", "8720.16`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:40:00\"\>", ",", "8720.4`", ",", "8733.94`", 
     ",", "8719.53`", ",", "8728.05`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T13:50:00\"\>", ",", "8728.21`", ",", "8746.13`", 
     ",", "8723.51`", ",", "8742.07`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:00:00\"\>", ",", "8741.99`", ",", "8750.75`", 
     ",", "8734.5`", ",", "8734.9`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:10:00\"\>", ",", "8734.66`", ",", "8740.07`", 
     ",", "8725.58`", ",", "8727.09`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:20:00\"\>", ",", "8727.25`", ",", "8741.99`", 
     ",", "8726.85`", ",", "8730.6`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:30:00\"\>", ",", "8730.76`", ",", "8735.69`", 
     ",", "8721.52`", ",", "8732.11`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:40:00\"\>", ",", "8732.03`", ",", "8744.85`", 
     ",", "8724.07`", ",", "8730.04`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T14:50:00\"\>", ",", "8729.64`", ",", "8736.09`", 
     ",", "8720.88`", ",", "8733.22`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:00:00\"\>", ",", "8733.3`", ",", "8736.09`", 
     ",", "8709.97`", ",", "8716.5`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:10:00\"\>", ",", "8716.74`", ",", "8721.28`", 
     ",", "8704.55`", ",", "8709.81`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:20:00\"\>", ",", "8709.41`", ",", "8743.1`", 
     ",", "8707.82`", ",", "8742.15`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:30:00\"\>", ",", "8741.91`", ",", "8753.85`", 
     ",", "8736.25`", ",", "8750.75`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:40:00\"\>", ",", "8750.51`", ",", "8759.75`", 
     ",", "8726.06`", ",", "8736.73`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T15:50:00\"\>", ",", "8736.81`", ",", "8755.05`", 
     ",", "8706.22`", ",", "8729.72`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-01T16:00:00\"\>", ",", "8732.27`", ",", "8732.27`", 
     ",", "8721.36`", ",", "8721.44`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T09:30:00\"\>", ",", "8721.6`", ",", "8733.13`", 
     ",", "8694.12`", ",", "8732.25`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T09:40:00\"\>", ",", "8731.3`", ",", "8736.71`", 
     ",", "8718.08`", ",", "8726.12`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T09:50:00\"\>", ",", "8726.04`", ",", "8746.99`", 
     ",", "8723.65`", ",", "8746.99`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:00:00\"\>", ",", "8756.31`", ",", "8785.7`", 
     ",", "8756.31`", ",", "8775.58`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:10:00\"\>", ",", "8775.5`", ",", "8778.29`", 
     ",", "8742.13`", ",", "8744.36`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:20:00\"\>", ",", "8744.68`", ",", "8767.38`", 
     ",", "8739.42`", ",", "8762.12`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:30:00\"\>", ",", "8763.`", ",", "8768.65`", 
     ",", "8750.09`", ",", "8758.46`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:40:00\"\>", ",", "8759.01`", ",", "8765.31`", 
     ",", "8747.47`", ",", "8754.32`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T10:50:00\"\>", ",", "8754.24`", ",", "8756.31`", 
     ",", "8735.12`", ",", "8742.69`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:00:00\"\>", ",", "8744.92`", ",", "8753.04`", 
     ",", "8742.45`", ",", "8744.68`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:10:00\"\>", ",", "8744.92`", ",", "8753.68`", 
     ",", "8736.31`", ",", "8750.65`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:20:00\"\>", ",", "8750.81`", ",", "8759.57`", 
     ",", "8747.55`", ",", "8750.81`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:30:00\"\>", ",", "8748.9`", ",", "8751.13`", 
     ",", "8717.12`", ",", "8717.52`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:40:00\"\>", ",", "8717.28`", ",", "8727.95`", 
     ",", "8717.12`", ",", "8719.83`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T11:50:00\"\>", ",", "8719.91`", ",", "8733.05`", 
     ",", "8719.35`", ",", "8726.68`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:00:00\"\>", ",", "8728.03`", ",", "8736.31`", 
     ",", "8725.24`", ",", "8727.95`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:10:00\"\>", ",", "8728.35`", ",", "8731.62`", 
     ",", "8725.64`", ",", "8730.18`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:20:00\"\>", ",", "8730.02`", ",", "8730.58`", 
     ",", "8707.56`", ",", "8711.07`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:30:00\"\>", ",", "8710.99`", ",", "8720.46`", 
     ",", "8707.24`", ",", "8719.67`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:40:00\"\>", ",", "8719.75`", ",", "8723.41`", 
     ",", "8719.75`", ",", "8721.18`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T12:50:00\"\>", ",", "8721.26`", ",", "8733.93`", 
     ",", "8717.92`", ",", "8728.51`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:00:00\"\>", ",", "8728.27`", ",", "8728.27`", 
     ",", "8722.3`", ",", "8724.13`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:10:00\"\>", ",", "8724.05`", ",", "8725.32`", 
     ",", "8714.57`", ",", "8716.4`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:20:00\"\>", ",", "8716.48`", ",", "8736.63`", 
     ",", "8714.89`", ",", "8735.04`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:30:00\"\>", ",", "8734.8`", ",", "8745.39`", 
     ",", "8734.72`", ",", "8738.47`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:40:00\"\>", ",", "8738.62`", ",", "8752.32`", 
     ",", "8738.15`", ",", "8746.83`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T13:50:00\"\>", ",", "8747.23`", ",", "8749.62`", 
     ",", "8740.46`", ",", "8745.87`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:00:00\"\>", ",", "8745.79`", ",", "8746.19`", 
     ",", "8731.7`", ",", "8738.31`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:10:00\"\>", ",", "8738.15`", ",", "8755.03`", 
     ",", "8733.69`", ",", "8750.81`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:20:00\"\>", ",", "8751.05`", ",", "8762.44`", 
     ",", "8750.73`", ",", "8757.18`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:30:00\"\>", ",", "8757.58`", ",", "8758.3`", 
     ",", "8748.9`", ",", "8756.94`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:40:00\"\>", ",", "8756.86`", ",", "8771.6`", 
     ",", "8754.71`", ",", "8763.47`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T14:50:00\"\>", ",", "8763.63`", ",", "8768.73`", 
     ",", "8751.21`", ",", "8751.93`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:00:00\"\>", ",", "8753.04`", ",", "8775.1`", 
     ",", "8753.04`", ",", "8771.36`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:10:00\"\>", ",", "8772.47`", ",", "8774.71`", 
     ",", "8758.46`", ",", "8761.72`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:20:00\"\>", ",", "8759.57`", ",", "8770.32`", 
     ",", "8749.14`", ",", "8755.59`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:30:00\"\>", ",", "8755.03`", ",", "8766.98`", 
     ",", "8742.37`", ",", "8764.75`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:40:00\"\>", ",", "8765.94`", ",", "8769.77`", 
     ",", "8757.34`", ",", "8768.81`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T15:50:00\"\>", ",", "8769.29`", ",", "8770.01`", 
     ",", "8734.16`", ",", "8742.29`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-02T16:00:00\"\>", ",", "8742.53`", ",", "8742.77`", 
     ",", "8740.55`", ",", "8740.87`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T09:30:00\"\>", ",", "8740.07`", ",", "8740.07`", 
     ",", "8676.91`", ",", "8693.8`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T09:40:00\"\>", ",", "8694.2`", ",", "8696.75`", 
     ",", "8668.95`", ",", "8678.67`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T09:50:00\"\>", ",", "8676.91`", ",", "8690.14`", 
     ",", "8648.24`", ",", "8660.83`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:00:00\"\>", ",", "8660.75`", ",", "8660.75`", 
     ",", "8640.83`", ",", "8654.61`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:10:00\"\>", ",", "8654.14`", ",", "8663.45`", 
     ",", "8638.05`", ",", "8639.`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:20:00\"\>", ",", "8637.73`", ",", "8649.44`", 
     ",", "8629.52`", ",", "8647.36`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:30:00\"\>", ",", "8645.69`", ",", "8649.44`", 
     ",", "8640.99`", ",", "8647.13`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:40:00\"\>", ",", "8646.81`", ",", "8660.83`", 
     ",", "8639.56`", ",", "8653.98`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T10:50:00\"\>", ",", "8653.9`", ",", "8668.23`", 
     ",", "8653.9`", ",", "8666.88`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:00:00\"\>", ",", "8667.04`", ",", "8668.23`", 
     ",", "8654.14`", ",", "8655.73`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:10:00\"\>", ",", "8654.85`", ",", "8656.76`", 
     ",", "8644.74`", ",", "8652.38`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:20:00\"\>", ",", "8652.62`", ",", "8675.4`", 
     ",", "8652.62`", ",", "8666.88`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:30:00\"\>", ",", "8665.6`", ",", "8668.63`", 
     ",", "8659.87`", ",", "8662.5`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:40:00\"\>", ",", "8663.06`", ",", "8675.48`", 
     ",", "8657.88`", ",", "8660.59`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T11:50:00\"\>", ",", "8660.35`", ",", "8662.34`", 
     ",", "8647.36`", ",", "8651.98`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:00:00\"\>", ",", "8651.59`", ",", "8655.57`", 
     ",", "8646.73`", ",", "8652.94`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:10:00\"\>", ",", "8652.7`", ",", "8658.04`", 
     ",", "8644.82`", ",", "8656.44`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:20:00\"\>", ",", "8656.52`", ",", "8660.51`", 
     ",", "8638.68`", ",", "8638.68`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:30:00\"\>", ",", "8638.6`", ",", "8649.28`", 
     ",", "8634.22`", ",", "8649.04`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:40:00\"\>", ",", "8650.07`", ",", "8653.1`", 
     ",", "8643.94`", ",", "8645.53`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T12:50:00\"\>", ",", "8645.69`", ",", "8646.97`", 
     ",", "8629.84`", ",", "8634.14`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:00:00\"\>", ",", "8633.59`", ",", "8634.06`", 
     ",", "8628.09`", ",", "8628.81`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:10:00\"\>", ",", "8628.57`", ",", "8629.28`", 
     ",", "8624.35`", ",", "8627.45`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:20:00\"\>", ",", "8627.61`", ",", "8628.09`", 
     ",", "8611.92`", ",", "8618.37`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:30:00\"\>", ",", "8618.61`", ",", "8623.71`", 
     ",", "8613.67`", ",", "8620.13`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:40:00\"\>", ",", "8620.68`", ",", "8620.76`", 
     ",", "8603.48`", ",", "8604.75`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T13:50:00\"\>", ",", "8604.59`", ",", "8617.02`", 
     ",", "8603.4`", ",", "8615.98`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:00:00\"\>", ",", "8615.74`", ",", "8622.36`", 
     ",", "8611.92`", ",", "8613.83`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:10:00\"\>", ",", "8613.75`", ",", "8626.1`", 
     ",", "8611.36`", ",", "8620.84`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:20:00\"\>", ",", "8620.92`", ",", "8622.99`", 
     ",", "8609.53`", ",", "8613.36`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:30:00\"\>", ",", "8613.2`", ",", "8613.2`", 
     ",", "8603.08`", ",", "8607.22`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:40:00\"\>", ",", "8606.98`", ",", "8612.72`", 
     ",", "8602.2`", ",", "8611.44`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T14:50:00\"\>", ",", "8611.28`", ",", "8633.51`", 
     ",", "8609.05`", ",", "8625.38`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:00:00\"\>", ",", "8625.3`", ",", "8634.78`", 
     ",", "8621.88`", ",", "8632.63`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:10:00\"\>", ",", "8633.11`", ",", "8636.05`", 
     ",", "8622.12`", ",", "8628.41`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:20:00\"\>", ",", "8628.57`", ",", "8630.56`", 
     ",", "8611.13`", ",", "8611.36`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:30:00\"\>", ",", "8611.52`", ",", "8619.25`", 
     ",", "8598.54`", ",", "8607.94`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:40:00\"\>", ",", "8607.7`", ",", "8646.41`", 
     ",", "8606.9`", ",", "8643.3`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T15:50:00\"\>", ",", "8644.34`", ",", "8681.69`", 
     ",", "8639.88`", ",", "8681.45`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T16:00:00\"\>", ",", "8682.65`", ",", "8682.73`", 
     ",", "8675.88`", ",", "8675.88`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-03T16:10:00\"\>", ",", "8675.24`", ",", "8675.24`", 
     ",", "8675.24`", ",", "8675.24`", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T09:30:00\"\>", ",", "8665.72`", ",", "8703.94`", 
     ",", "8665.72`", ",", "8691.12`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T09:40:00\"\>", ",", "8691.76`", ",", "8702.11`", 
     ",", "8662.92`", ",", "8664.2`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T09:50:00\"\>", ",", "8663.32`", ",", "8671.05`", 
     ",", "8651.61`", ",", "8671.05`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:00:00\"\>", ",", "8671.37`", ",", "8671.92`", 
     ",", "8634.73`", ",", "8640.3`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:10:00\"\>", ",", "8640.54`", ",", "8648.91`", 
     ",", "8638.79`", ",", "8643.81`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:20:00\"\>", ",", "8643.97`", ",", "8686.34`", 
     ",", "8643.97`", ",", "8681.72`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:30:00\"\>", ",", "8681.8`", ",", "8690.96`", 
     ",", "8672.96`", ",", "8683.71`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:40:00\"\>", ",", "8683.95`", ",", "8700.12`", 
     ",", "8676.62`", ",", "8696.06`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T10:50:00\"\>", ",", "8697.09`", ",", "8722.58`", 
     ",", "8696.46`", ",", "8715.49`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:00:00\"\>", ",", "8715.57`", ",", "8720.99`", 
     ",", "8705.93`", ",", "8707.21`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:10:00\"\>", ",", "8707.13`", ",", "8718.76`", 
     ",", "8704.98`", ",", "8716.37`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:20:00\"\>", ",", "8716.92`", ",", "8717.96`", 
     ",", "8701.15`", ",", "8703.86`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:30:00\"\>", ",", "8703.62`", ",", "8705.06`", 
     ",", "8696.3`", ",", "8701.55`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:40:00\"\>", ",", "8701.31`", ",", "8703.15`", 
     ",", "8698.53`", ",", "8700.28`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T11:50:00\"\>", ",", "8700.36`", ",", "8704.34`", 
     ",", "8688.25`", ",", "8688.25`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:00:00\"\>", ",", "8689.21`", ",", "8689.21`", 
     ",", "8676.7`", ",", "8683.15`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:10:00\"\>", ",", "8683.23`", ",", "8685.07`", 
     ",", "8674.39`", ",", "8684.03`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:20:00\"\>", ",", "8684.19`", ",", "8702.27`", 
     ",", "8683.87`", ",", "8702.27`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:30:00\"\>", ",", "8702.59`", ",", "8705.85`", 
     ",", "8700.76`", ",", "8704.58`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:40:00\"\>", ",", "8704.18`", ",", "8715.17`", 
     ",", "8702.11`", ",", "8702.11`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T12:50:00\"\>", ",", "8702.43`", ",", "8709.12`", 
     ",", "8700.6`", ",", "8708.88`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:00:00\"\>", ",", "8708.56`", ",", "8713.34`", 
     ",", "8704.26`", ",", "8710.31`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:10:00\"\>", ",", "8710.71`", ",", "8724.09`", 
     ",", "8710.71`", ",", "8723.77`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:20:00\"\>", ",", "8722.82`", ",", "8730.07`", 
     ",", "8722.82`", ",", "8723.77`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:30:00\"\>", ",", "8723.62`", ",", "8744.48`", 
     ",", "8722.98`", ",", "8744.48`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:40:00\"\>", ",", "8744.`", ",", "8746.63`", 
     ",", "8734.13`", ",", "8742.41`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T13:50:00\"\>", ",", "8742.17`", ",", "8743.21`", 
     ",", "8727.52`", ",", "8733.81`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:00:00\"\>", ",", "8733.97`", ",", "8748.07`", 
     ",", "8730.07`", ",", "8745.68`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:10:00\"\>", ",", "8746.08`", ",", "8749.02`", 
     ",", "8737.47`", ",", "8742.33`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:20:00\"\>", ",", "8742.17`", ",", "8747.27`", 
     ",", "8738.59`", ",", "8743.29`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:30:00\"\>", ",", "8744.`", ",", "8746.95`", 
     ",", "8729.03`", ",", "8732.22`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:40:00\"\>", ",", "8732.46`", ",", "8745.44`", 
     ",", "8729.99`", ",", "8740.58`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T14:50:00\"\>", ",", "8739.86`", ",", "8743.93`", 
     ",", "8727.6`", ",", "8729.19`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:00:00\"\>", ",", "8729.27`", ",", "8732.69`", 
     ",", "8719.31`", ",", "8722.1`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:10:00\"\>", ",", "8721.07`", ",", "8732.69`", 
     ",", "8720.11`", ",", "8732.46`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:20:00\"\>", ",", "8732.54`", ",", "8739.15`", 
     ",", "8729.43`", ",", "8737.08`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:30:00\"\>", ",", "8736.04`", ",", "8744.72`", 
     ",", "8723.93`", ",", "8741.22`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:40:00\"\>", ",", "8741.54`", ",", "8746.16`", 
     ",", "8727.2`", ",", "8727.44`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T15:50:00\"\>", ",", "8726.4`", ",", "8751.17`", 
     ",", "8724.41`", ",", "8745.44`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-04T16:00:00\"\>", ",", "8742.73`", ",", "8750.24`", 
     ",", "8742.73`", ",", "8750.24`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T09:30:00\"\>", ",", "8751.67`", ",", "8839.`", 
     ",", "8751.67`", ",", "8830.95`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T09:40:00\"\>", ",", "8830.39`", ",", "8838.36`", 
     ",", "8759.35`", ",", "8770.74`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T09:50:00\"\>", ",", "8770.5`", ",", "8793.2`", 
     ",", "8754.81`", ",", "8774.48`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:00:00\"\>", ",", "8769.15`", ",", "8780.77`", 
     ",", "8725.82`", ",", "8725.82`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:10:00\"\>", ",", "8727.09`", ",", "8735.69`", 
     ",", "8700.41`", ",", "8735.69`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:20:00\"\>", ",", "8735.53`", ",", "8759.19`", 
     ",", "8734.34`", ",", "8749.63`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:30:00\"\>", ",", "8749.07`", ",", "8780.77`", 
     ",", "8739.52`", ",", "8775.52`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:40:00\"\>", ",", "8774.56`", ",", "8804.19`", 
     ",", "8774.`", ",", "8794.55`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T10:50:00\"\>", ",", "8794.47`", ",", "8799.41`", 
     ",", "8781.81`", ",", "8799.41`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:00:00\"\>", ",", "8799.09`", ",", "8823.62`", 
     ",", "8798.14`", ",", "8815.66`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:10:00\"\>", ",", "8815.82`", ",", "8816.3`", 
     ",", "8800.69`", ",", "8804.35`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:20:00\"\>", ",", "8805.07`", ",", "8817.89`", 
     ",", "8795.27`", ",", "8797.34`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:30:00\"\>", ",", "8797.42`", ",", "8797.42`", 
     ",", "8777.27`", ",", "8779.82`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:40:00\"\>", ",", "8779.74`", ",", "8792.32`", 
     ",", "8770.5`", ",", "8786.67`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T11:50:00\"\>", ",", "8786.03`", ",", "8799.17`", 
     ",", "8772.73`", ",", "8798.54`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:00:00\"\>", ",", "8798.38`", ",", "8804.43`", 
     ",", "8791.92`", ",", "8792.16`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:10:00\"\>", ",", "8792.08`", ",", "8797.02`", 
     ",", "8780.3`", ",", "8794.63`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:20:00\"\>", ",", "8794.23`", ",", "8814.54`", 
     ",", "8788.82`", ",", "8810.88`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:30:00\"\>", ",", "8811.12`", ",", "8812.08`", 
     ",", "8800.05`", ",", "8806.42`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:40:00\"\>", ",", "8807.46`", ",", "8811.44`", 
     ",", "8800.21`", ",", "8810.96`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T12:50:00\"\>", ",", "8810.8`", ",", "8816.3`", 
     ",", "8805.47`", ",", "8813.83`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:00:00\"\>", ",", "8814.07`", ",", "8814.54`", 
     ",", "8809.21`", ",", "8809.29`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:10:00\"\>", ",", "8809.53`", ",", "8810.32`", 
     ",", "8789.69`", ",", "8799.41`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:20:00\"\>", ",", "8799.17`", ",", "8802.04`", 
     ",", "8788.5`", ",", "8794.87`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:30:00\"\>", ",", "8794.79`", ",", "8800.29`", 
     ",", "8788.98`", ",", "8792.32`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:40:00\"\>", ",", "8791.77`", ",", "8807.77`", 
     ",", "8790.81`", ",", "8803.16`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T13:50:00\"\>", ",", "8803.23`", ",", "8804.11`", 
     ",", "8783.96`", ",", "8789.62`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:00:00\"\>", ",", "8789.69`", ",", "8792.56`", 
     ",", "8764.61`", ",", "8765.88`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:10:00\"\>", ",", "8765.72`", ",", "8771.46`", 
     ",", "8747.32`", ",", "8750.59`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:20:00\"\>", ",", "8749.71`", ",", "8757.2`", 
     ",", "8735.38`", ",", "8751.86`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:30:00\"\>", ",", "8751.3`", ",", "8751.3`", 
     ",", "8732.75`", ",", "8745.33`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:40:00\"\>", ",", "8745.01`", ",", "8766.36`", 
     ",", "8743.34`", ",", "8766.36`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T14:50:00\"\>", ",", "8766.52`", ",", "8773.69`", 
     ",", "8766.52`", ",", "8769.94`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:00:00\"\>", ",", "8769.46`", ",", "8782.77`", 
     ",", "8756.`", ",", "8770.98`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:10:00\"\>", ",", "8770.66`", ",", "8778.62`", 
     ",", "8764.69`", ",", "8778.38`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:20:00\"\>", ",", "8778.07`", ",", "8782.61`", 
     ",", "8763.41`", ",", "8763.41`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:30:00\"\>", ",", "8763.65`", ",", "8794.55`", 
     ",", "8761.1`", ",", "8794.23`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:40:00\"\>", ",", "8794.31`", ",", "8805.15`", 
     ",", "8753.22`", ",", "8754.65`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T15:50:00\"\>", ",", "8754.57`", ",", "8772.17`", 
     ",", "8752.58`", ",", "8768.19`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-05T16:00:00\"\>", ",", "8768.43`", ",", "8768.91`", 
     ",", "8762.77`", ",", "8763.13`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T09:30:00\"\>", ",", "8759.28`", ",", "8759.28`", 
     ",", "8684.91`", ",", "8687.41`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T09:40:00\"\>", ",", "8686.12`", ",", "8690.05`", 
     ",", "8663.22`", ",", "8663.68`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T09:50:00\"\>", ",", "8664.43`", ",", "8669.8`", 
     ",", "8651.21`", ",", "8656.57`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:00:00\"\>", ",", "8654.68`", ",", "8673.2`", 
     ",", "8646.14`", ",", "8669.34`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:10:00\"\>", ",", "8667.53`", ",", "8673.5`", 
     ",", "8638.06`", ",", "8641.83`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:20:00\"\>", ",", "8641.53`", ",", "8652.49`", 
     ",", "8637.75`", ",", "8650.22`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:30:00\"\>", ",", "8649.24`", ",", "8674.86`", 
     ",", "8647.73`", ",", "8674.33`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:40:00\"\>", ",", "8674.94`", ",", "8691.41`", 
     ",", "8674.86`", ",", "8680.53`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T10:50:00\"\>", ",", "8680.98`", ",", "8688.31`", 
     ",", "8676.6`", ",", "8678.34`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:00:00\"\>", ",", "8677.96`", ",", "8679.92`", 
     ",", "8666.4`", ",", "8668.06`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:10:00\"\>", ",", "8668.29`", ",", "8685.14`", 
     ",", "8665.57`", ",", "8680.45`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:20:00\"\>", ",", "8680.`", ",", "8684.76`", 
     ",", "8662.09`", ",", "8665.41`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:30:00\"\>", ",", "8665.64`", ",", "8669.04`", 
     ",", "8656.35`", ",", "8669.04`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:40:00\"\>", ",", "8668.89`", ",", "8669.8`", 
     ",", "8654.91`", ",", "8658.16`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T11:50:00\"\>", ",", "8658.01`", ",", "8659.52`", 
     ",", "8649.77`", ",", "8659.52`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:00:00\"\>", ",", "8659.67`", ",", "8662.69`", 
     ",", "8645.76`", ",", "8649.77`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:10:00\"\>", ",", "8649.85`", ",", "8650.22`", 
     ",", "8634.81`", ",", "8639.04`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:20:00\"\>", ",", "8638.66`", ",", "8647.35`", 
     ",", "8637.53`", ",", "8643.65`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:30:00\"\>", ",", "8643.04`", ",", "8651.21`", 
     ",", "8633.14`", ",", "8649.54`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:40:00\"\>", ",", "8650.07`", ",", "8657.03`", 
     ",", "8646.6`", ",", "8649.69`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T12:50:00\"\>", ",", "8649.24`", ",", "8653.55`", 
     ",", "8646.22`", ",", "8651.36`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:00:00\"\>", ",", "8651.43`", ",", "8666.62`", 
     ",", "8651.43`", ",", "8664.51`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:10:00\"\>", ",", "8664.36`", ",", "8666.62`", 
     ",", "8653.47`", ",", "8663.45`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:20:00\"\>", ",", "8663.15`", ",", "8665.34`", 
     ",", "8653.47`", ",", "8656.65`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:30:00\"\>", ",", "8656.8`", ",", "8660.43`", 
     ",", "8654.15`", ",", "8657.86`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:40:00\"\>", ",", "8657.63`", ",", "8678.64`", 
     ",", "8651.66`", ",", "8673.5`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T13:50:00\"\>", ",", "8673.65`", ",", "8673.65`", 
     ",", "8663.37`", ",", "8672.37`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:00:00\"\>", ",", "8672.14`", ",", "8675.62`", 
     ",", "8666.55`", ",", "8667.61`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:10:00\"\>", ",", "8668.44`", ",", "8672.59`", 
     ",", "8665.64`", ",", "8669.19`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:20:00\"\>", ",", "8669.04`", ",", "8673.73`", 
     ",", "8661.94`", ",", "8673.73`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:30:00\"\>", ",", "8674.48`", ",", "8685.37`", 
     ",", "8667.38`", ",", "8679.62`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:40:00\"\>", ",", "8679.92`", ",", "8680.`", 
     ",", "8663.9`", ",", "8676.9`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T14:50:00\"\>", ",", "8677.13`", ",", "8683.25`", 
     ",", "8666.93`", ",", "8680.3`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:00:00\"\>", ",", "8680.53`", ",", "8696.63`", 
     ",", "8680.3`", ",", "8693.91`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:10:00\"\>", ",", "8694.13`", ",", "8724.14`", 
     ",", "8687.18`", ",", "8718.32`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:20:00\"\>", ",", "8718.47`", ",", "8774.7`", 
     ",", "8718.47`", ",", "8761.17`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:30:00\"\>", ",", "8764.04`", ",", "8823.97`", 
     ",", "8764.04`", ",", "8794.5`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:40:00\"\>", ",", "8795.63`", ",", "8811.27`", 
     ",", "8787.77`", ",", "8794.5`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T15:50:00\"\>", ",", "8794.72`", ",", "8800.54`", 
     ",", "8757.54`", ",", "8763.21`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-08T16:00:00\"\>", ",", "8763.06`", ",", "8764.57`", 
     ",", "8763.06`", ",", "8764.49`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T09:30:00\"\>", ",", "8764.83`", ",", "8791.62`", 
     ",", "8764.42`", ",", "8786.71`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T09:40:00\"\>", ",", "8785.28`", ",", "8802.28`", 
     ",", "8775.15`", ",", "8792.61`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T09:50:00\"\>", ",", "8792.68`", ",", "8792.98`", 
     ",", "8757.84`", ",", "8761.55`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:00:00\"\>", ",", "8762.75`", ",", "8774.32`", 
     ",", "8750.13`", ",", "8767.89`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:10:00\"\>", ",", "8768.12`", ",", "8780.67`", 
     ",", "8761.39`", ",", "8766.84`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:20:00\"\>", ",", "8766.38`", ",", "8766.46`", 
     ",", "8746.88`", ",", "8754.44`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:30:00\"\>", ",", "8755.27`", ",", "8763.96`", 
     ",", "8753.16`", ",", "8757.69`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:40:00\"\>", ",", "8757.62`", ",", "8775.15`", 
     ",", "8746.51`", ",", "8746.51`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T10:50:00\"\>", ",", "8746.73`", ",", "8746.73`", 
     ",", "8735.02`", ",", "8739.93`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:00:00\"\>", ",", "8739.7`", ",", "8739.7`", 
     ",", "8725.27`", ",", "8737.36`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:10:00\"\>", ",", "8737.66`", ",", "8740.38`", 
     ",", "8733.43`", ",", "8733.88`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:20:00\"\>", ",", "8734.11`", ",", "8744.62`", 
     ",", "8726.71`", ",", "8736.08`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:30:00\"\>", ",", "8735.7`", ",", "8739.1`", 
     ",", "8730.71`", ",", "8737.13`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:40:00\"\>", ",", "8737.29`", ",", "8740.46`", 
     ",", "8731.24`", ",", "8732.37`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T11:50:00\"\>", ",", "8731.62`", ",", "8736.68`", 
     ",", "8729.35`", ",", "8734.34`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:00:00\"\>", ",", "8734.41`", ",", "8746.43`", 
     ",", "8732.45`", ",", "8744.39`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:10:00\"\>", ",", "8744.09`", ",", "8748.77`", 
     ",", "8742.88`", ",", "8746.2`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:20:00\"\>", ",", "8746.28`", ",", "8763.13`", 
     ",", "8743.86`", ",", "8763.13`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:30:00\"\>", ",", "8763.21`", ",", "8765.55`", 
     ",", "8760.03`", ",", "8764.19`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:40:00\"\>", ",", "8763.74`", ",", "8765.32`", 
     ",", "8749.38`", ",", "8762.15`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T12:50:00\"\>", ",", "8762.45`", ",", "8762.83`", 
     ",", "8754.21`", ",", "8758.52`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:00:00\"\>", ",", "8758.14`", ",", "8777.34`", 
     ",", "8756.86`", ",", "8763.81`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:10:00\"\>", ",", "8763.89`", ",", "8764.19`", 
     ",", "8745.22`", ",", "8749.76`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:20:00\"\>", ",", "8749.83`", ",", "8767.74`", 
     ",", "8748.85`", ",", "8765.85`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:30:00\"\>", ",", "8765.55`", ",", "8767.36`", 
     ",", "8752.63`", ",", "8767.36`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:40:00\"\>", ",", "8767.29`", ",", "8772.81`", 
     ",", "8759.13`", ",", "8759.13`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T13:50:00\"\>", ",", "8758.9`", ",", "8782.33`", 
     ",", "8758.22`", ",", "8781.19`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:00:00\"\>", ",", "8781.27`", ",", "8789.81`", 
     ",", "8774.09`", ",", "8782.86`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:10:00\"\>", ",", "8782.78`", ",", "8796.31`", 
     ",", "8777.94`", ",", "8793.36`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:20:00\"\>", ",", "8794.65`", ",", "8796.01`", 
     ",", "8781.04`", ",", "8793.06`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:30:00\"\>", ",", "8794.34`", ",", "8803.04`", 
     ",", "8788.6`", ",", "8789.89`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:40:00\"\>", ",", "8789.73`", ",", "8801.68`", 
     ",", "8773.86`", ",", "8779.`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T14:50:00\"\>", ",", "8779.23`", ",", "8786.33`", 
     ",", "8770.99`", ",", "8783.31`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:00:00\"\>", ",", "8786.64`", ",", "8792.76`", 
     ",", "8774.7`", ",", "8780.82`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:10:00\"\>", ",", "8780.36`", ",", "8781.12`", 
     ",", "8763.81`", ",", "8776.21`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:20:00\"\>", ",", "8776.13`", ",", "8776.13`", 
     ",", "8760.41`", ",", "8769.86`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:30:00\"\>", ",", "8769.18`", ",", "8789.66`", 
     ",", "8761.77`", ",", "8782.03`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:40:00\"\>", ",", "8782.4`", ",", "8788.45`", 
     ",", "8761.39`", ",", "8770.39`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T15:50:00\"\>", ",", "8769.86`", ",", "8778.93`", 
     ",", "8757.92`", ",", "8760.26`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-09T16:00:00\"\>", ",", "8760.03`", ",", "8763.06`", 
     ",", "8760.03`", ",", "8763.06`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T09:30:00\"\>", ",", "8763.66`", ",", "8834.55`", 
     ",", "8763.66`", ",", "8824.27`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T09:40:00\"\>", ",", "8824.35`", ",", "8824.35`", 
     ",", "8797.37`", ",", "8798.35`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T09:50:00\"\>", ",", "8798.43`", ",", "8816.79`", 
     ",", "8795.63`", ",", "8816.79`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:00:00\"\>", ",", "8816.56`", ",", "8819.44`", 
     ",", "8777.34`", ",", "8784.07`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:10:00\"\>", ",", "8783.76`", ",", "8785.73`", 
     ",", "8760.71`", ",", "8767.14`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:20:00\"\>", ",", "8766.91`", ",", "8782.71`", 
     ",", "8764.34`", ",", "8778.93`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:30:00\"\>", ",", "8782.48`", ",", "8784.52`", 
     ",", "8770.92`", ",", "8774.02`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:40:00\"\>", ",", "8773.94`", ",", "8774.32`", 
     ",", "8743.63`", ",", "8746.58`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T10:50:00\"\>", ",", "8746.66`", ",", "8755.88`", 
     ",", "8741.44`", ",", "8753.69`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:00:00\"\>", ",", "8753.38`", ",", "8753.84`", 
     ",", "8735.62`", ",", "8738.95`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:10:00\"\>", ",", "8734.87`", ",", "8745.37`", 
     ",", "8724.36`", ",", "8732.3`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:20:00\"\>", ",", "8732.15`", ",", "8750.21`", 
     ",", "8728.59`", ",", "8748.09`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:30:00\"\>", ",", "8748.02`", ",", "8752.33`", 
     ",", "8746.96`", ",", "8751.95`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:40:00\"\>", ",", "8752.1`", ",", "8753.31`", 
     ",", "8743.63`", ",", "8744.09`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T11:50:00\"\>", ",", "8745.07`", ",", "8749.53`", 
     ",", "8741.22`", ",", "8744.09`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:00:00\"\>", ",", "8743.94`", ",", "8753.76`", 
     ",", "8740.08`", ",", "8744.39`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:10:00\"\>", ",", "8744.62`", ",", "8761.55`", 
     ",", "8744.62`", ",", "8759.5`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:20:00\"\>", ",", "8759.43`", ",", "8761.7`", 
     ",", "8738.42`", ",", "8743.03`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:30:00\"\>", ",", "8743.18`", ",", "8743.48`", 
     ",", "8730.79`", ",", "8742.27`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:40:00\"\>", ",", "8742.5`", ",", "8742.5`", 
     ",", "8727.99`", ",", "8728.22`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T12:50:00\"\>", ",", "8727.99`", ",", "8740.23`", 
     ",", "8727.16`", ",", "8736.76`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:00:00\"\>", ",", "8736.38`", ",", "8736.38`", 
     ",", "8689.75`", ",", "8694.96`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:10:00\"\>", ",", "8695.11`", ",", "8700.03`", 
     ",", "8692.32`", ",", "8695.11`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:20:00\"\>", ",", "8694.21`", ",", "8698.21`", 
     ",", "8685.67`", ",", "8690.73`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:30:00\"\>", ",", "8690.66`", ",", "8691.03`", 
     ",", "8667.3`", ",", "8667.3`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:40:00\"\>", ",", "8667.08`", ",", "8672.14`", 
     ",", "8643.57`", ",", "8646.67`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T13:50:00\"\>", ",", "8646.6`", ",", "8665.19`", 
     ",", "8646.52`", ",", "8662.84`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:00:00\"\>", ",", "8662.01`", ",", "8662.84`", 
     ",", "8639.95`", ",", "8655.97`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:10:00\"\>", ",", "8656.12`", ",", "8675.77`", 
     ",", "8656.12`", ",", "8675.77`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:20:00\"\>", ",", "8677.13`", ",", "8680.45`", 
     ",", "8664.43`", ",", "8668.66`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:30:00\"\>", ",", "8668.74`", ",", "8688.77`", 
     ",", "8658.08`", ",", "8680.45`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:40:00\"\>", ",", "8680.53`", ",", "8693.91`", 
     ",", "8674.86`", ",", "8692.55`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T14:50:00\"\>", ",", "8693.83`", ",", "8707.21`", 
     ",", "8685.37`", ",", "8696.78`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:00:00\"\>", ",", "8696.63`", ",", "8698.44`", 
     ",", "8673.8`", ",", "8674.71`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:10:00\"\>", ",", "8674.63`", ",", "8678.72`", 
     ",", "8662.16`", ",", "8670.18`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:20:00\"\>", ",", "8670.02`", ",", "8681.06`", 
     ",", "8663.6`", ",", "8670.25`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:30:00\"\>", ",", "8670.7`", ",", "8708.19`", 
     ",", "8669.95`", ",", "8705.47`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:40:00\"\>", ",", "8705.24`", ",", "8721.72`", 
     ",", "8691.03`", ",", "8716.5`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T15:50:00\"\>", ",", "8716.35`", ",", "8743.26`", 
     ",", "8706.38`", ",", "8738.72`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-10T16:00:00\"\>", ",", "8738.87`", ",", "8739.33`", 
     ",", "8738.27`", ",", "8739.02`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T09:30:00\"\>", ",", "8736.23`", ",", "8798.2`", 
     ",", "8736.23`", ",", "8781.27`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T09:40:00\"\>", ",", "8781.5`", ",", "8823.37`", 
     ",", "8778.1`", ",", "8819.89`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T09:50:00\"\>", ",", "8819.96`", ",", "8829.11`", 
     ",", "8811.8`", ",", "8815.58`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:00:00\"\>", ",", "8816.26`", ",", "8821.55`", 
     ",", "8805.83`", ",", "8812.71`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:10:00\"\>", ",", "8812.33`", ",", "8843.54`", 
     ",", "8812.33`", ",", "8830.92`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:20:00\"\>", ",", "8832.28`", ",", "8833.72`", 
     ",", "8813.92`", ",", "8817.24`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:30:00\"\>", ",", "8818.38`", ",", "8821.25`", 
     ",", "8795.33`", ",", "8808.1`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:40:00\"\>", ",", "8807.87`", ",", "8815.35`", 
     ",", "8802.73`", ",", "8814.37`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T10:50:00\"\>", ",", "8815.13`", ",", "8815.88`", 
     ",", "8794.8`", ",", "8794.8`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:00:00\"\>", ",", "8794.65`", ",", "8794.72`", 
     ",", "8784.52`", ",", "8788.83`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:10:00\"\>", ",", "8788.45`", ",", "8807.34`", 
     ",", "8784.6`", ",", "8807.34`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:20:00\"\>", ",", "8807.19`", ",", "8823.37`", 
     ",", "8805.`", ",", "8823.37`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:30:00\"\>", ",", "8823.14`", ",", "8823.21`", 
     ",", "8802.36`", ",", "8806.81`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:40:00\"\>", ",", "8806.89`", ",", "8813.09`", 
     ",", "8795.1`", ",", "8802.51`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T11:50:00\"\>", ",", "8802.88`", ",", "8820.72`", 
     ",", "8802.58`", ",", "8808.63`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:00:00\"\>", ",", "8808.55`", ",", "8808.55`", 
     ",", "8792.76`", ",", "8796.08`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:10:00\"\>", ",", "8795.93`", ",", "8799.48`", 
     ",", "8785.35`", ",", "8795.86`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:20:00\"\>", ",", "8795.71`", ",", "8806.89`", 
     ",", "8795.71`", ",", "8798.5`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:30:00\"\>", ",", "8798.43`", ",", "8811.12`", 
     ",", "8798.05`", ",", "8806.21`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:40:00\"\>", ",", "8806.06`", ",", "8821.55`", 
     ",", "8803.41`", ",", "8813.09`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T12:50:00\"\>", ",", "8812.86`", ",", "8838.33`", 
     ",", "8811.05`", ",", "8834.25`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:00:00\"\>", ",", "8834.32`", ",", "8853.14`", 
     ",", "8826.24`", ",", "8827.67`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:10:00\"\>", ",", "8827.82`", ",", "8841.58`", 
     ",", "8827.67`", ",", "8841.35`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:20:00\"\>", ",", "8841.2`", ",", "8869.92`", 
     ",", "8840.52`", ",", "8869.92`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:30:00\"\>", ",", "8869.54`", ",", "8875.66`", 
     ",", "8858.43`", ",", "8858.66`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:40:00\"\>", ",", "8857.83`", ",", "8860.09`", 
     ",", "8846.42`", ",", "8850.27`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T13:50:00\"\>", ",", "8849.74`", ",", "8851.03`", 
     ",", "8839.92`", ",", "8843.01`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:00:00\"\>", ",", "8842.86`", ",", "8843.7`", 
     ",", "8825.41`", ",", "8826.46`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:10:00\"\>", ",", "8826.39`", ",", "8844.53`", 
     ",", "8824.65`", ",", "8843.39`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:20:00\"\>", ",", "8843.47`", ",", "8847.7`", 
     ",", "8834.17`", ",", "8846.79`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:30:00\"\>", ",", "8846.94`", ",", "8853.44`", 
     ",", "8816.11`", ",", "8822.38`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:40:00\"\>", ",", "8822.16`", ",", "8832.28`", 
     ",", "8817.47`", ",", "8824.2`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T14:50:00\"\>", ",", "8824.12`", ",", "8845.96`", 
     ",", "8817.17`", ",", "8842.11`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:00:00\"\>", ",", "8842.33`", ",", "8851.03`", 
     ",", "8827.9`", ",", "8844.`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:10:00\"\>", ",", "8844.53`", ",", "8851.86`", 
     ",", "8835.38`", ",", "8838.71`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:20:00\"\>", ",", "8839.01`", ",", "8842.49`", 
     ",", "8822.31`", ",", "8823.29`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:30:00\"\>", ",", "8820.04`", ",", "8820.04`", 
     ",", "8785.88`", ",", "8807.8`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:40:00\"\>", ",", "8807.95`", ",", "8816.64`", 
     ",", "8794.65`", ",", "8802.05`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T15:50:00\"\>", ",", "8802.2`", ",", "8811.35`", 
     ",", "8758.45`", ",", "8768.57`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-11T16:00:00\"\>", ",", "8768.2`", ",", "8770.92`", 
     ",", "8768.2`", ",", "8770.92`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T09:30:00\"\>", ",", "8770.01`", ",", "8770.01`", 
     ",", "8727.69`", ",", "8747.41`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T09:40:00\"\>", ",", "8749.83`", ",", "8754.59`", 
     ",", "8737.13`", ",", "8740.54`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T09:50:00\"\>", ",", "8742.05`", ",", "8753.08`", 
     ",", "8724.97`", ",", "8733.96`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:00:00\"\>", ",", "8731.09`", ",", "8732.52`", 
     ",", "8717.11`", ",", "8730.11`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:10:00\"\>", ",", "8730.03`", ",", "8761.17`", 
     ",", "8724.89`", ",", "8754.59`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:20:00\"\>", ",", "8754.37`", ",", "8767.14`", 
     ",", "8744.39`", ",", "8746.05`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:30:00\"\>", ",", "8745.98`", ",", "8768.2`", 
     ",", "8740.61`", ",", "8762.68`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:40:00\"\>", ",", "8762.6`", ",", "8777.94`", 
     ",", "8762.53`", ",", "8765.4`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T10:50:00\"\>", ",", "8765.25`", ",", "8782.1`", 
     ",", "8765.25`", ",", "8766.23`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:00:00\"\>", ",", "8766.31`", ",", "8772.28`", 
     ",", "8760.94`", ",", "8763.28`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:10:00\"\>", ",", "8763.36`", ",", "8766.91`", 
     ",", "8754.44`", ",", "8755.88`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:20:00\"\>", ",", "8756.03`", ",", "8761.55`", 
     ",", "8756.03`", ",", "8760.79`", ",", "200", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:30:00\"\>", ",", "8760.87`", ",", "8766.08`", 
     ",", "8758.45`", ",", "8763.28`", ",", "176", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:40:00\"\>", ",", "8763.36`", ",", "8764.72`", 
     ",", "8746.81`", ",", "8755.27`", ",", "188", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T11:50:00\"\>", ",", "8755.2`", ",", "8756.63`", 
     ",", "8751.72`", ",", "8752.02`", ",", "163", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:00:00\"\>", ",", "8752.1`", ",", "8758.22`", 
     ",", "8752.1`", ",", "8757.77`", ",", "105", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:10:00\"\>", ",", "8757.84`", ",", "8762.38`", 
     ",", "8757.77`", ",", "8762.38`", ",", "106", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:20:00\"\>", ",", "8762.45`", ",", "8762.45`", 
     ",", "8756.33`", ",", "8758.98`", ",", "102", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:30:00\"\>", ",", "8759.13`", ",", "8759.81`", 
     ",", "8746.81`", ",", "8754.29`", ",", "164", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:40:00\"\>", ",", "8754.14`", ",", "8765.63`", 
     ",", "8753.76`", ",", "8764.79`", ",", "139", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T12:50:00\"\>", ",", "8764.72`", ",", "8764.79`", 
     ",", "8748.7`", ",", "8764.11`", ",", "168", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:00:00\"\>", ",", "8764.27`", ",", "8771.37`", 
     ",", "8752.25`", ",", "8755.12`", ",", "208", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:10:00\"\>", ",", "8755.57`", ",", "8765.78`", 
     ",", "8753.31`", ",", "8760.26`", ",", "189", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:20:00\"\>", ",", "8760.49`", ",", "8766.61`", 
     ",", "8757.92`", ",", "8758.22`", ",", "201", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:30:00\"\>", ",", "8758.37`", ",", "8763.36`", 
     ",", "8756.1`", ",", "8763.36`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:40:00\"\>", ",", "8763.51`", ",", "8768.65`", 
     ",", "8758.75`", ",", "8767.29`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T13:50:00\"\>", ",", "8767.36`", ",", "8775.38`", 
     ",", "8765.17`", ",", "8770.54`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:00:00\"\>", ",", "8770.46`", ",", "8770.99`", 
     ",", "8758.22`", ",", "8762.45`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:10:00\"\>", ",", "8761.47`", ",", "8766.76`", 
     ",", "8760.41`", ",", "8766.23`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:20:00\"\>", ",", "8766.31`", ",", "8772.35`", 
     ",", "8752.78`", ",", "8757.54`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:30:00\"\>", ",", "8757.77`", ",", "8770.84`", 
     ",", "8757.77`", ",", "8766.99`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:40:00\"\>", ",", "8767.06`", ",", "8805.3`", 
     ",", "8767.06`", ",", "8793.59`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T14:50:00\"\>", ",", "8793.51`", ",", "8799.48`", 
     ",", "8786.71`", ",", "8789.21`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:00:00\"\>", ",", "8791.02`", ",", "8800.69`", 
     ",", "8790.94`", ",", "8791.62`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:10:00\"\>", ",", "8791.7`", ",", "8800.39`", 
     ",", "8791.1`", ",", "8798.12`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:20:00\"\>", ",", "8798.05`", ",", "8798.58`", 
     ",", "8784.75`", ",", "8791.32`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:30:00\"\>", ",", "8791.93`", ",", "8796.99`", 
     ",", "8763.96`", ",", "8765.25`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:40:00\"\>", ",", "8766.16`", ",", "8774.17`", 
     ",", "8757.69`", ",", "8762.15`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T15:50:00\"\>", ",", "8762.68`", ",", "8801.07`", 
     ",", "8762.53`", ",", "8801.07`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-12T16:00:00\"\>", ",", "8801.22`", ",", "8801.6`", 
     ",", "8799.26`", ",", "8799.26`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T09:30:00\"\>", ",", "8798.5`", ",", "8798.5`", 
     ",", "8670.02`", ",", "8687.41`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T09:40:00\"\>", ",", "8684.16`", ",", "8689.98`", 
     ",", "8639.27`", ",", "8645.69`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T09:50:00\"\>", ",", "8645.61`", ",", "8673.73`", 
     ",", "8642.14`", ",", "8669.27`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:00:00\"\>", ",", "8668.81`", ",", "8668.81`", 
     ",", "8651.51`", ",", "8659.07`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:10:00\"\>", ",", "8658.99`", ",", "8663.6`", 
     ",", "8638.74`", ",", "8645.69`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:20:00\"\>", ",", "8642.74`", ",", "8652.94`", 
     ",", "8639.79`", ",", "8647.43`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:30:00\"\>", ",", "8647.5`", ",", "8647.5`", 
     ",", "8586.59`", ",", "8589.99`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:40:00\"\>", ",", "8590.22`", ",", "8605.48`", 
     ",", "8586.59`", ",", "8602.54`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T10:50:00\"\>", ",", "8602.01`", ",", "8614.02`", 
     ",", "8598.53`", ",", "8607.6`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:00:00\"\>", ",", "8607.67`", ",", "8611.23`", 
     ",", "8590.22`", ",", "8592.11`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:10:00\"\>", ",", "8591.28`", ",", "8598.98`", 
     ",", "8587.72`", ",", "8591.5`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:20:00\"\>", ",", "8591.28`", ",", "8592.11`", 
     ",", "8581.`", ",", "8591.12`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:30:00\"\>", ",", "8591.2`", ",", "8602.54`", 
     ",", "8591.2`", ",", "8598.83`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:40:00\"\>", ",", "8601.1`", ",", "8614.1`", 
     ",", "8600.34`", ",", "8608.73`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T11:50:00\"\>", ",", "8609.19`", ",", "8609.94`", 
     ",", "8596.34`", ",", "8602.08`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:00:00\"\>", ",", "8601.93`", ",", "8607.07`", 
     ",", "8594.52`", ",", "8599.51`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:10:00\"\>", ",", "8599.66`", ",", "8600.19`", 
     ",", "8591.8`", ",", "8596.34`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:20:00\"\>", ",", "8596.41`", ",", "8604.65`", 
     ",", "8588.55`", ",", "8593.62`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:30:00\"\>", ",", "8593.77`", ",", "8613.72`", 
     ",", "8593.47`", ",", "8611.83`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:40:00\"\>", ",", "8611.53`", ",", "8615.76`", 
     ",", "8600.8`", ",", "8605.18`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T12:50:00\"\>", ",", "8605.33`", ",", "8615.38`", 
     ",", "8601.02`", ",", "8613.87`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:00:00\"\>", ",", "8605.94`", ",", "8610.4`", 
     ",", "8589.69`", ",", "8596.26`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:10:00\"\>", ",", "8596.04`", ",", "8613.19`", 
     ",", "8595.81`", ",", "8598.08`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:20:00\"\>", ",", "8598.`", ",", "8600.34`", 
     ",", "8589.54`", ",", "8591.2`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:30:00\"\>", ",", "8591.43`", ",", "8591.43`", 
     ",", "8579.79`", ",", "8587.57`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:40:00\"\>", ",", "8587.8`", ",", "8601.93`", 
     ",", "8582.28`", ",", "8599.89`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T13:50:00\"\>", ",", "8599.82`", ",", "8603.67`", 
     ",", "8592.41`", ",", "8592.71`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:00:00\"\>", ",", "8592.86`", ",", "8607.98`", 
     ",", "8588.25`", ",", "8602.01`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:10:00\"\>", ",", "8602.16`", ",", "8604.05`", 
     ",", "8590.59`", ",", "8599.13`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:20:00\"\>", ",", "8599.06`", ",", "8611.45`", 
     ",", "8597.09`", ",", "8606.54`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:30:00\"\>", ",", "8605.41`", ",", "8605.41`", 
     ",", "8590.9`", ",", "8597.85`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:40:00\"\>", ",", "8593.84`", ",", "8593.84`", 
     ",", "8579.49`", ",", "8586.82`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T14:50:00\"\>", ",", "8586.74`", ",", "8594.45`", 
     ",", "8577.9`", ",", "8594.45`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:00:00\"\>", ",", "8595.2`", ",", "8598.68`", 
     ",", "8581.07`", ",", "8598.68`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:10:00\"\>", ",", "8601.18`", ",", "8615.61`", 
     ",", "8598.08`", ",", "8608.66`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:20:00\"\>", ",", "8610.09`", ",", "8613.65`", 
     ",", "8598.61`", ",", "8603.44`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:30:00\"\>", ",", "8605.94`", ",", "8625.36`", 
     ",", "8599.59`", ",", "8615.53`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:40:00\"\>", ",", "8614.4`", ",", "8626.42`", 
     ",", "8599.59`", ",", "8622.64`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T15:50:00\"\>", ",", "8623.02`", ",", "8624.45`", 
     ",", "8609.56`", ",", "8615.08`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-15T16:00:00\"\>", ",", "8615.46`", ",", "8615.84`", 
     ",", "8612.13`", ",", "8612.13`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T09:30:00\"\>", ",", "8612.44`", ",", "8643.5`", 
     ",", "8608.81`", ",", "8614.55`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T09:40:00\"\>", ",", "8614.48`", ",", "8640.1`", 
     ",", "8611.76`", ",", "8636.24`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T09:50:00\"\>", ",", "8635.86`", ",", "8641.61`", 
     ",", "8611.15`", ",", "8616.82`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:00:00\"\>", ",", "8616.89`", ",", "8640.4`", 
     ",", "8611.53`", ",", "8633.75`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:10:00\"\>", ",", "8633.6`", ",", "8639.79`", 
     ",", "8606.77`", ",", "8610.02`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:20:00\"\>", ",", "8610.62`", ",", "8623.85`", 
     ",", "8606.92`", ",", "8618.48`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:30:00\"\>", ",", "8618.1`", ",", "8628.84`", 
     ",", "8612.13`", ",", "8628.61`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:40:00\"\>", ",", "8628.76`", ",", "8632.09`", 
     ",", "8620.22`", ",", "8620.82`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T10:50:00\"\>", ",", "8620.67`", ",", "8624.3`", 
     ",", "8617.35`", ",", "8621.58`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:00:00\"\>", ",", "8621.43`", ",", "8636.47`", 
     ",", "8621.43`", ",", "8629.21`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:10:00\"\>", ",", "8629.52`", ",", "8636.7`", 
     ",", "8628.46`", ",", "8631.03`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:20:00\"\>", ",", "8631.1`", ",", "8631.56`", 
     ",", "8610.77`", ",", "8616.97`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:30:00\"\>", ",", "8617.12`", ",", "8617.12`", 
     ",", "8606.09`", ",", "8614.7`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:40:00\"\>", ",", "8614.48`", ",", "8615.38`", 
     ",", "8591.8`", ",", "8592.11`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T11:50:00\"\>", ",", "8591.88`", ",", "8595.96`", 
     ",", "8584.93`", ",", "8585.68`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:00:00\"\>", ",", "8585.83`", ",", "8594.`", 
     ",", "8581.`", ",", "8588.86`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:10:00\"\>", ",", "8588.71`", ",", "8595.66`", 
     ",", "8583.26`", ",", "8585.68`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:20:00\"\>", ",", "8585.76`", ",", "8592.71`", 
     ",", "8580.47`", ",", "8589.61`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:30:00\"\>", ",", "8589.99`", ",", "8590.14`", 
     ",", "8565.35`", ",", "8566.49`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:40:00\"\>", ",", "8566.41`", ",", "8566.41`", 
     ",", "8541.47`", ",", "8545.02`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T12:50:00\"\>", ",", "8545.17`", ",", "8550.84`", 
     ",", "8539.96`", ",", "8547.59`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:00:00\"\>", ",", "8547.67`", ",", "8547.67`", 
     ",", "8527.34`", ",", "8543.06`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:10:00\"\>", ",", "8543.36`", ",", "8544.8`", 
     ",", "8535.73`", ",", "8536.33`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:20:00\"\>", ",", "8536.41`", ",", "8538.45`", 
     ",", "8521.52`", ",", "8537.99`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:30:00\"\>", ",", "8538.07`", ",", "8539.36`", 
     ",", "8532.48`", ",", "8538.52`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:40:00\"\>", ",", "8538.07`", ",", "8546.99`", 
     ",", "8536.11`", ",", "8545.02`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T13:50:00\"\>", ",", "8544.04`", ",", "8544.42`", 
     ",", "8529.3`", ",", "8536.11`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:00:00\"\>", ",", "8535.73`", ",", "8535.73`", 
     ",", "8510.33`", ",", "8520.76`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:10:00\"\>", ",", "8519.93`", ",", "8537.24`", 
     ",", "8518.42`", ",", "8535.73`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:20:00\"\>", ",", "8535.95`", ",", "8535.95`", 
     ",", "8516.68`", ",", "8521.29`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:30:00\"\>", ",", "8520.69`", ",", "8537.39`", 
     ",", "8507.39`", ",", "8536.63`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:40:00\"\>", ",", "8536.41`", ",", "8536.71`", 
     ",", "8525.6`", ",", "8529.91`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T14:50:00\"\>", ",", "8530.14`", ",", "8540.79`", 
     ",", "8528.4`", ",", "8537.24`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:00:00\"\>", ",", "8537.16`", ",", "8541.62`", 
     ",", "8522.2`", ",", "8527.41`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:10:00\"\>", ",", "8527.26`", ",", "8547.9`", 
     ",", "8517.97`", ",", "8544.57`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:20:00\"\>", ",", "8544.19`", ",", "8544.72`", 
     ",", "8520.08`", ",", "8531.65`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:30:00\"\>", ",", "8531.5`", ",", "8548.42`", 
     ",", "8531.5`", ",", "8546.76`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:40:00\"\>", ",", "8545.55`", ",", "8558.4`", 
     ",", "8537.01`", ",", "8537.69`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T15:50:00\"\>", ",", "8538.98`", ",", "8539.28`", 
     ",", "8499.45`", ",", "8503.15`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-16T16:00:00\"\>", ",", "8503.23`", ",", "8505.12`", 
     ",", "8503.23`", ",", "8504.67`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T09:30:00\"\>", ",", "8504.44`", ",", "8528.93`", 
     ",", "8497.03`", ",", "8519.71`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T09:40:00\"\>", ",", "8518.95`", ",", "8521.07`", 
     ",", "8491.89`", ",", "8491.89`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T09:50:00\"\>", ",", "8493.33`", ",", "8505.8`", 
     ",", "8488.27`", ",", "8490.53`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:00:00\"\>", ",", "8489.17`", ",", "8518.12`", 
     ",", "8484.03`", ",", "8514.49`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:10:00\"\>", ",", "8514.42`", ",", "8521.29`", 
     ",", "8507.84`", ",", "8512.3`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:20:00\"\>", ",", "8512.`", ",", "8513.81`", 
     ",", "8485.17`", ",", "8493.33`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:30:00\"\>", ",", "8493.56`", ",", "8493.63`", 
     ",", "8471.19`", ",", "8474.21`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:40:00\"\>", ",", "8474.06`", ",", "8484.94`", 
     ",", "8461.36`", ",", "8470.13`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T10:50:00\"\>", ",", "8469.68`", ",", "8482.07`", 
     ",", "8464.31`", ",", "8477.38`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:00:00\"\>", ",", "8477.16`", ",", "8497.94`", 
     ",", "8476.78`", ",", "8489.48`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:10:00\"\>", ",", "8489.55`", ",", "8506.78`", 
     ",", "8481.69`", ",", "8506.4`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:20:00\"\>", ",", "8506.1`", ",", "8516.38`", 
     ",", "8505.12`", ",", "8508.6`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:30:00\"\>", ",", "8508.45`", ",", "8513.66`", 
     ",", "8500.28`", ",", "8501.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:40:00\"\>", ",", "8502.25`", ",", "8514.11`", 
     ",", "8498.92`", ",", "8509.65`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T11:50:00\"\>", ",", "8509.81`", ",", "8515.17`", 
     ",", "8491.52`", ",", "8513.58`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:00:00\"\>", ",", "8513.51`", ",", "8528.7`", 
     ",", "8513.51`", ",", "8528.02`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:10:00\"\>", ",", "8527.79`", ",", "8532.1`", 
     ",", "8518.72`", ",", "8527.26`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:20:00\"\>", ",", "8527.34`", ",", "8546.61`", 
     ",", "8527.34`", ",", "8535.12`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:30:00\"\>", ",", "8535.27`", ",", "8535.5`", 
     ",", "8524.16`", ",", "8530.66`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:40:00\"\>", ",", "8530.51`", ",", "8534.06`", 
     ",", "8519.55`", ",", "8522.35`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T12:50:00\"\>", ",", "8522.28`", ",", "8522.28`", 
     ",", "8509.58`", ",", "8518.72`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:00:00\"\>", ",", "8518.57`", ",", "8524.84`", 
     ",", "8512.53`", ",", "8521.97`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:10:00\"\>", ",", "8522.2`", ",", "8529.98`", 
     ",", "8516.61`", ",", "8516.91`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:20:00\"\>", ",", "8516.98`", ",", "8537.24`", 
     ",", "8516.76`", ",", "8537.09`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:30:00\"\>", ",", "8537.16`", ",", "8547.97`", 
     ",", "8533.91`", ",", "8547.82`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:40:00\"\>", ",", "8547.67`", ",", "8562.93`", 
     ",", "8547.67`", ",", "8555.`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T13:50:00\"\>", ",", "8554.24`", ",", "8555.83`", 
     ",", "8543.44`", ",", "8545.63`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:00:00\"\>", ",", "8545.55`", ",", "8559.46`", 
     ",", "8539.36`", ",", "8542.76`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:10:00\"\>", ",", "8542.91`", ",", "8553.03`", 
     ",", "8542.91`", ",", "8543.97`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:20:00\"\>", ",", "8544.27`", ",", "8557.95`", 
     ",", "8542.15`", ",", "8557.49`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:30:00\"\>", ",", "8558.48`", ",", "8559.08`", 
     ",", "8535.65`", ",", "8535.65`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:40:00\"\>", ",", "8535.35`", ",", "8535.35`", 
     ",", "8520.61`", ",", "8527.64`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T14:50:00\"\>", ",", "8526.96`", ",", "8533.01`", 
     ",", "8517.67`", ",", "8529.3`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:00:00\"\>", ",", "8529.23`", ",", "8540.72`", 
     ",", "8517.59`", ",", "8525.98`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:10:00\"\>", ",", "8526.58`", ",", "8535.35`", 
     ",", "8522.12`", ",", "8529.45`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:20:00\"\>", ",", "8529.61`", ",", "8537.47`", 
     ",", "8505.88`", ",", "8507.46`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:30:00\"\>", ",", "8507.24`", ",", "8507.24`", 
     ",", "8492.8`", ",", "8502.25`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:40:00\"\>", ",", "8500.66`", ",", "8528.47`", 
     ",", "8492.8`", ",", "8528.09`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T15:50:00\"\>", ",", "8528.47`", ",", "8528.55`", 
     ",", "8492.95`", ",", "8500.13`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-17T16:00:00\"\>", ",", "8499.75`", ",", "8499.91`", 
     ",", "8497.18`", ",", "8497.18`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T09:30:00\"\>", ",", "8496.73`", ",", "8506.78`", 
     ",", "8479.27`", ",", "8481.16`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T09:40:00\"\>", ",", "8479.12`", ",", "8506.18`", 
     ",", "8475.72`", ",", "8502.32`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T09:50:00\"\>", ",", "8500.06`", ",", "8548.8`", 
     ",", "8494.39`", ",", "8544.72`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:00:00\"\>", ",", "8544.87`", ",", "8560.14`", 
     ",", "8543.74`", ",", "8550.92`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:10:00\"\>", ",", "8551.14`", ",", "8580.17`", 
     ",", "8551.14`", ",", "8572.23`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:20:00\"\>", ",", "8574.27`", ",", "8578.13`", 
     ",", "8564.45`", ",", "8568.07`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:30:00\"\>", ",", "8567.47`", ",", "8576.84`", 
     ",", "8566.71`", ",", "8568.38`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:40:00\"\>", ",", "8568.68`", ",", "8569.28`", 
     ",", "8551.52`", ",", "8552.13`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T10:50:00\"\>", ",", "8551.98`", ",", "8557.8`", 
     ",", "8545.85`", ",", "8553.56`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:00:00\"\>", ",", "8555.3`", ",", "8567.92`", 
     ",", "8555.3`", ",", "8563.84`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:10:00\"\>", ",", "8563.54`", ",", "8572.08`", 
     ",", "8559.84`", ",", "8571.02`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:20:00\"\>", ",", "8571.1`", ",", "8574.65`", 
     ",", "8564.14`", ",", "8564.82`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:30:00\"\>", ",", "8564.6`", ",", "8565.96`", 
     ",", "8550.62`", ",", "8557.8`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:40:00\"\>", ",", "8557.87`", ",", "8558.48`", 
     ",", "8546.31`", ",", "8549.71`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T11:50:00\"\>", ",", "8550.01`", ",", "8566.18`", 
     ",", "8549.94`", ",", "8565.58`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:00:00\"\>", ",", "8565.66`", ",", "8567.92`", 
     ",", "8562.56`", ",", "8565.5`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:10:00\"\>", ",", "8565.43`", ",", "8568.9`", 
     ",", "8561.8`", ",", "8562.18`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:20:00\"\>", ",", "8561.65`", ",", "8565.05`", 
     ",", "8553.56`", ",", "8563.92`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:30:00\"\>", ",", "8564.29`", ",", "8569.66`", 
     ",", "8559.91`", ",", "8560.59`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:40:00\"\>", ",", "8560.67`", ",", "8572.08`", 
     ",", "8559.31`", ",", "8571.63`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T12:50:00\"\>", ",", "8571.78`", ",", "8575.78`", 
     ",", "8566.03`", ",", "8574.27`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:00:00\"\>", ",", "8574.42`", ",", "8590.44`", 
     ",", "8570.87`", ",", "8589.08`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:10:00\"\>", ",", "8589.31`", ",", "8589.39`", 
     ",", "8574.2`", ",", "8578.96`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:20:00\"\>", ",", "8579.11`", ",", "8586.97`", 
     ",", "8577.9`", ",", "8586.97`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:30:00\"\>", ",", "8586.82`", ",", "8588.1`", 
     ",", "8568.75`", ",", "8568.75`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:40:00\"\>", ",", "8568.6`", ",", "8580.47`", 
     ",", "8567.02`", ",", "8578.43`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T13:50:00\"\>", ",", "8578.35`", ",", "8581.22`", 
     ",", "8566.18`", ",", "8566.41`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:00:00\"\>", ",", "8567.24`", ",", "8569.36`", 
     ",", "8554.17`", ",", "8557.72`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:10:00\"\>", ",", "8557.64`", ",", "8565.96`", 
     ",", "8546.69`", ",", "8563.99`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:20:00\"\>", ",", "8564.9`", ",", "8566.34`", 
     ",", "8548.12`", ",", "8558.93`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:30:00\"\>", ",", "8561.65`", ",", "8563.39`", 
     ",", "8549.86`", ",", "8554.39`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:40:00\"\>", ",", "8554.24`", ",", "8570.72`", 
     ",", "8553.94`", ",", "8564.6`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T14:50:00\"\>", ",", "8564.52`", ",", "8570.57`", 
     ",", "8557.95`", ",", "8568.6`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:00:00\"\>", ",", "8569.36`", ",", "8574.04`", 
     ",", "8562.41`", ",", "8573.06`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:10:00\"\>", ",", "8572.91`", ",", "8579.33`", 
     ",", "8565.35`", ",", "8573.82`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:20:00\"\>", ",", "8573.89`", ",", "8574.2`", 
     ",", "8561.05`", ",", "8563.39`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:30:00\"\>", ",", "8563.46`", ",", "8572.23`", 
     ",", "8553.71`", ",", "8554.39`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:40:00\"\>", ",", "8553.11`", ",", "8565.43`", 
     ",", "8539.05`", ",", "8547.97`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T15:50:00\"\>", ",", "8547.74`", ",", "8556.44`", 
     ",", "8530.06`", ",", "8549.71`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-18T16:00:00\"\>", ",", "8549.86`", ",", "8555.6`", 
     ",", "8549.86`", ",", "8555.6`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T09:30:00\"\>", ",", "8556.96`", ",", "8615.69`", 
     ",", "8556.96`", ",", "8603.22`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T09:40:00\"\>", ",", "8603.44`", ",", "8615.99`", 
     ",", "8597.55`", ",", "8608.43`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T09:50:00\"\>", ",", "8607.37`", ",", "8611.98`", 
     ",", "8602.31`", ",", "8607.07`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:00:00\"\>", ",", "8607.3`", ",", "8607.75`", 
     ",", "8593.16`", ",", "8594.3`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:10:00\"\>", ",", "8594.37`", ",", "8596.87`", 
     ",", "8576.61`", ",", "8586.29`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:20:00\"\>", ",", "8586.36`", ",", "8590.82`", 
     ",", "8578.2`", ",", "8589.08`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:30:00\"\>", ",", "8588.71`", ",", "8600.12`", 
     ",", "8587.72`", ",", "8591.5`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:40:00\"\>", ",", "8591.05`", ",", "8601.86`", 
     ",", "8585.76`", ",", "8598.91`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T10:50:00\"\>", ",", "8598.53`", ",", "8612.66`", 
     ",", "8594.6`", ",", "8605.41`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:00:00\"\>", ",", "8605.33`", ",", "8613.04`", 
     ",", "8605.33`", ",", "8609.34`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:10:00\"\>", ",", "8609.19`", ",", "8609.87`", 
     ",", "8598.53`", ",", "8599.29`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:20:00\"\>", ",", "8599.13`", ",", "8603.06`", 
     ",", "8592.64`", ",", "8593.92`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:30:00\"\>", ",", "8593.77`", ",", "8594.37`", 
     ",", "8587.87`", ",", "8590.59`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:40:00\"\>", ",", "8590.52`", ",", "8596.87`", 
     ",", "8577.67`", ",", "8577.75`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T11:50:00\"\>", ",", "8577.67`", ",", "8581.3`", 
     ",", "8576.31`", ",", "8581.15`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:00:00\"\>", ",", "8580.92`", ",", "8583.04`", 
     ",", "8572.68`", ",", "8572.76`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:10:00\"\>", ",", "8572.83`", ",", "8572.83`", 
     ",", "8557.19`", ",", "8565.13`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:20:00\"\>", ",", "8564.9`", ",", "8565.05`", 
     ",", "8541.77`", ",", "8544.57`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:30:00\"\>", ",", "8545.48`", ",", "8545.48`", 
     ",", "8531.65`", ",", "8534.9`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:40:00\"\>", ",", "8535.05`", ",", "8543.44`", 
     ",", "8534.75`", ",", "8537.99`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T12:50:00\"\>", ",", "8538.07`", ",", "8541.92`", 
     ",", "8527.79`", ",", "8527.94`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:00:00\"\>", ",", "8528.02`", ",", "8535.8`", 
     ",", "8527.72`", ",", "8535.8`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:10:00\"\>", ",", "8535.73`", ",", "8544.72`", 
     ",", "8532.93`", ",", "8539.88`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:20:00\"\>", ",", "8539.73`", ",", "8540.87`", 
     ",", "8529.3`", ",", "8537.01`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:30:00\"\>", ",", "8537.16`", ",", "8553.49`", 
     ",", "8537.16`", ",", "8547.9`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:40:00\"\>", ",", "8548.05`", ",", "8552.13`", 
     ",", "8533.16`", ",", "8535.95`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T13:50:00\"\>", ",", "8536.18`", ",", "8544.65`", 
     ",", "8530.44`", ",", "8532.4`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:00:00\"\>", ",", "8532.1`", ",", "8535.43`", 
     ",", "8524.01`", ",", "8524.69`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:10:00\"\>", ",", "8524.62`", ",", "8525.07`", 
     ",", "8502.4`", ",", "8504.14`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:20:00\"\>", ",", "8504.06`", ",", "8508.07`", 
     ",", "8496.81`", ",", "8505.95`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:30:00\"\>", ",", "8506.18`", ",", "8519.4`", 
     ",", "8504.52`", ",", "8509.73`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:40:00\"\>", ",", "8509.88`", ",", "8515.93`", 
     ",", "8509.43`", ",", "8514.11`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T14:50:00\"\>", ",", "8513.89`", ",", "8526.13`", 
     ",", "8512.75`", ",", "8521.67`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:00:00\"\>", ",", "8521.97`", ",", "8532.48`", 
     ",", "8511.24`", ",", "8516.61`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:10:00\"\>", ",", "8516.46`", ",", "8539.58`", 
     ",", "8516.3`", ",", "8539.28`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:20:00\"\>", ",", "8539.81`", ",", "8558.78`", 
     ",", "8534.14`", ",", "8558.78`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:30:00\"\>", ",", "8562.86`", ",", "8565.28`", 
     ",", "8553.87`", ",", "8553.94`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:40:00\"\>", ",", "8553.71`", ",", "8559.08`", 
     ",", "8537.16`", ",", "8552.88`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T15:50:00\"\>", ",", "8552.58`", ",", "8552.58`", 
     ",", "8528.32`", ",", "8535.5`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-19T16:00:00\"\>", ",", "8535.65`", ",", "8539.73`", 
     ",", "8535.35`", ",", "8539.73`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T09:30:00\"\>", ",", "8538.67`", ",", "8538.67`", 
     ",", "8455.77`", ",", "8461.66`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T09:40:00\"\>", ",", "8462.5`", ",", "8462.5`", 
     ",", "8432.42`", ",", "8439.45`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T09:50:00\"\>", ",", "8439.52`", ",", "8444.06`", 
     ",", "8418.36`", ",", "8423.88`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:00:00\"\>", ",", "8423.35`", ",", "8436.12`", 
     ",", "8411.63`", ",", "8433.4`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:10:00\"\>", ",", "8433.17`", ",", "8438.76`", 
     ",", "8409.97`", ",", "8410.35`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:20:00\"\>", ",", "8409.37`", ",", "8431.28`", 
     ",", "8409.37`", ",", "8421.99`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:30:00\"\>", ",", "8421.16`", ",", "8421.16`", 
     ",", "8399.47`", ",", "8407.1`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:40:00\"\>", ",", "8407.17`", ",", "8408.16`", 
     ",", "8392.29`", ",", "8404.83`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T10:50:00\"\>", ",", "8404.91`", ",", "8409.06`", 
     ",", "8391.3`", ",", "8399.47`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:00:00\"\>", ",", "8399.31`", ",", "8399.84`", 
     ",", "8381.4`", ",", "8390.93`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:10:00\"\>", ",", "8391.`", ",", "8401.13`", 
     ",", "8386.69`", ",", "8400.52`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:20:00\"\>", ",", "8400.37`", ",", "8405.06`", 
     ",", "8390.02`", ",", "8390.02`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:30:00\"\>", ",", "8389.19`", ",", "8390.25`", 
     ",", "8364.25`", ",", "8365.46`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:40:00\"\>", ",", "8365.68`", ",", "8376.04`", 
     ",", "8362.89`", ",", "8363.04`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T11:50:00\"\>", ",", "8362.74`", ",", "8371.05`", 
     ",", "8359.03`", ",", "8362.89`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:00:00\"\>", ",", "8362.66`", ",", "8375.21`", 
     ",", "8359.56`", ",", "8367.12`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:10:00\"\>", ",", "8368.25`", ",", "8377.4`", 
     ",", "8368.25`", ",", "8374.`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:20:00\"\>", ",", "8374.07`", ",", "8376.79`", 
     ",", "8365.61`", ",", "8372.49`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:30:00\"\>", ",", "8373.85`", ",", "8382.76`", 
     ",", "8368.48`", ",", "8370.75`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:40:00\"\>", ",", "8370.82`", ",", "8376.72`", 
     ",", "8363.11`", ",", "8364.85`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T12:50:00\"\>", ",", "8365.08`", ",", "8375.81`", 
     ",", "8364.7`", ",", "8373.85`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:00:00\"\>", ",", "8373.7`", ",", "8374.68`", 
     ",", "8367.5`", ",", "8371.58`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:10:00\"\>", ",", "8371.73`", ",", "8371.96`", 
     ",", "8348.53`", ",", "8352.99`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:20:00\"\>", ",", "8352.91`", ",", "8355.33`", 
     ",", "8340.97`", ",", "8353.97`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:30:00\"\>", ",", "8354.05`", ",", "8369.01`", 
     ",", "8353.82`", ",", "8368.03`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:40:00\"\>", ",", "8368.1`", ",", "8373.24`", 
     ",", "8366.97`", ",", "8373.24`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T13:50:00\"\>", ",", "8373.39`", ",", "8374.98`", 
     ",", "8365.23`", ",", "8372.41`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:00:00\"\>", ",", "8369.99`", ",", "8372.11`", 
     ",", "8363.34`", ",", "8367.12`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:10:00\"\>", ",", "8367.27`", ",", "8379.97`", 
     ",", "8362.51`", ",", "8379.21`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:20:00\"\>", ",", "8379.36`", ",", "8380.95`", 
     ",", "8367.35`", ",", "8367.35`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:30:00\"\>", ",", "8367.2`", ",", "8384.12`", 
     ",", "8358.43`", ",", "8382.46`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:40:00\"\>", ",", "8382.31`", ",", "8382.31`", 
     ",", "8368.56`", ",", "8375.13`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T14:50:00\"\>", ",", "8375.06`", ",", "8378.08`", 
     ",", "8367.65`", ",", "8377.02`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:00:00\"\>", ",", "8377.1`", ",", "8392.51`", 
     ",", "8376.64`", ",", "8381.18`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:10:00\"\>", ",", "8380.87`", ",", "8382.76`", 
     ",", "8366.74`", ",", "8378.53`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:20:00\"\>", ",", "8378.61`", ",", "8381.71`", 
     ",", "8369.77`", ",", "8376.94`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:30:00\"\>", ",", "8377.47`", ",", "8395.76`", 
     ",", "8369.99`", ",", "8395.01`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:40:00\"\>", ",", "8394.33`", ",", "8394.33`", 
     ",", "8361.45`", ",", "8370.22`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T15:50:00\"\>", ",", "8370.82`", ",", "8371.2`", 
     ",", "8334.85`", ",", "8337.72`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-22T16:00:00\"\>", ",", "8337.42`", ",", "8339.54`", 
     ",", "8337.42`", ",", "8339.01`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T09:30:00\"\>", ",", "8340.44`", ",", "8358.05`", 
     ",", "8324.65`", ",", "8349.66`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T09:40:00\"\>", ",", "8349.96`", ",", "8361.07`", 
     ",", "8327.9`", ",", "8346.94`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T09:50:00\"\>", ",", "8349.13`", ",", "8364.78`", 
     ",", "8349.13`", ",", "8362.43`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:00:00\"\>", ",", "8353.06`", ",", "8370.75`", 
     ",", "8338.4`", ",", "8346.26`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:10:00\"\>", ",", "8345.81`", ",", "8345.81`", 
     ",", "8315.65`", ",", "8330.62`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:20:00\"\>", ",", "8330.01`", ",", "8338.1`", 
     ",", "8309.61`", ",", "8309.61`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:30:00\"\>", ",", "8313.99`", ",", "8328.43`", 
     ",", "8311.95`", ",", "8322.98`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:40:00\"\>", ",", "8322.68`", ",", "8322.68`", 
     ",", "8297.97`", ",", "8302.81`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T10:50:00\"\>", ",", "8302.73`", ",", "8310.36`", 
     ",", "8291.77`", ",", "8306.13`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:00:00\"\>", ",", "8306.21`", ",", "8306.21`", 
     ",", "8288.3`", ",", "8288.82`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:10:00\"\>", ",", "8288.9`", ",", "8305.`", ",",
      "8286.56`", ",", "8304.92`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:20:00\"\>", ",", "8304.85`", ",", "8321.09`", 
     ",", "8301.37`", ",", "8316.18`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:30:00\"\>", ",", "8316.33`", ",", "8329.94`", 
     ",", "8314.14`", ",", "8326.23`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:40:00\"\>", ",", "8326.01`", ",", "8326.31`", 
     ",", "8317.32`", ",", "8326.08`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T11:50:00\"\>", ",", "8326.39`", ",", "8336.81`", 
     ",", "8324.8`", ",", "8335.61`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:00:00\"\>", ",", "8335.91`", ",", "8337.65`", 
     ",", "8327.97`", ",", "8332.58`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:10:00\"\>", ",", "8332.36`", ",", "8332.51`", 
     ",", "8315.58`", ",", "8319.21`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:20:00\"\>", ",", "8319.13`", ",", "8326.54`", 
     ",", "8313.31`", ",", "8315.58`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:30:00\"\>", ",", "8315.35`", ",", "8343.09`", 
     ",", "8315.35`", ",", "8336.13`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:40:00\"\>", ",", "8336.21`", ",", "8339.54`", 
     ",", "8327.52`", ",", "8334.77`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T12:50:00\"\>", ",", "8334.85`", ",", "8339.08`", 
     ",", "8323.82`", ",", "8324.42`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:00:00\"\>", ",", "8324.34`", ",", "8342.1`", 
     ",", "8320.79`", ",", "8339.38`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:10:00\"\>", ",", "8339.46`", ",", "8349.44`", 
     ",", "8332.51`", ",", "8341.05`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:20:00\"\>", ",", "8341.35`", ",", "8341.58`", 
     ",", "8324.65`", ",", "8327.29`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:30:00\"\>", ",", "8328.88`", ",", "8335.45`", 
     ",", "8328.88`", ",", "8334.47`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:40:00\"\>", ",", "8334.55`", ",", "8335.`", 
     ",", "8321.4`", ",", "8324.72`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T13:50:00\"\>", ",", "8324.42`", ",", "8330.54`", 
     ",", "8311.65`", ",", "8312.63`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:00:00\"\>", ",", "8312.4`", ",", "8328.5`", 
     ",", "8308.1`", ",", "8326.39`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:10:00\"\>", ",", "8326.61`", ",", "8347.85`", 
     ",", "8326.61`", ",", "8344.3`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:20:00\"\>", ",", "8343.16`", ",", "8346.34`", 
     ",", "8332.58`", ",", "8333.72`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:30:00\"\>", ",", "8333.56`", ",", "8335.`", 
     ",", "8318.45`", ",", "8319.05`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:40:00\"\>", ",", "8319.28`", ",", "8325.33`", 
     ",", "8311.72`", ",", "8320.04`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T14:50:00\"\>", ",", "8320.26`", ",", "8336.36`", 
     ",", "8318.75`", ",", "8322.68`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:00:00\"\>", ",", "8322.91`", ",", "8323.59`", 
     ",", "8308.25`", ",", "8309.23`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:10:00\"\>", ",", "8309.15`", ",", "8326.46`", 
     ",", "8306.66`", ",", "8320.64`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:20:00\"\>", ",", "8320.49`", ",", "8334.93`", 
     ",", "8312.25`", ",", "8332.66`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:30:00\"\>", ",", "8332.36`", ",", "8332.36`", 
     ",", "8315.8`", ",", "8325.55`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:40:00\"\>", ",", "8325.86`", ",", "8347.47`", 
     ",", "8320.87`", ",", "8347.32`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T15:50:00\"\>", ",", "8347.77`", ",", "8348.15`", 
     ",", "8316.26`", ",", "8317.54`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-23T16:00:00\"\>", ",", "8317.69`", ",", "8322.91`", 
     ",", "8317.69`", ",", "8322.91`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T09:30:00\"\>", ",", "8323.51`", ",", "8381.93`", 
     ",", "8323.51`", ",", "8374.15`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T09:40:00\"\>", ",", "8373.7`", ",", "8381.25`", 
     ",", "8369.39`", ",", "8377.7`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T09:50:00\"\>", ",", "8378.08`", ",", "8390.47`", 
     ",", "8375.13`", ",", "8387.9`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:00:00\"\>", ",", "8384.28`", ",", "8394.4`", 
     ",", "8373.32`", ",", "8389.57`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:10:00\"\>", ",", "8389.94`", ",", "8414.51`", 
     ",", "8389.94`", ",", "8413.07`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:20:00\"\>", ",", "8414.73`", ",", "8428.34`", 
     ",", "8414.28`", ",", "8414.28`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:30:00\"\>", ",", "8417.6`", ",", "8419.12`", 
     ",", "8405.89`", ",", "8412.16`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:40:00\"\>", ",", "8412.01`", ",", "8422.06`", 
     ",", "8403.77`", ",", "8422.06`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T10:50:00\"\>", ",", "8421.99`", ",", "8425.69`", 
     ",", "8415.11`", ",", "8416.92`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:00:00\"\>", ",", "8416.85`", ",", "8427.05`", 
     ",", "8408.54`", ",", "8408.61`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:10:00\"\>", ",", "8408.46`", ",", "8411.48`", 
     ",", "8407.85`", ",", "8409.22`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:20:00\"\>", ",", "8410.42`", ",", "8423.65`", 
     ",", "8407.17`", ",", "8416.47`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:30:00\"\>", ",", "8419.34`", ",", "8420.1`", 
     ",", "8404.`", ",", "8404.15`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:40:00\"\>", ",", "8400.22`", ",", "8404.38`", 
     ",", "8395.01`", ",", "8395.39`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T11:50:00\"\>", ",", "8395.31`", ",", "8396.29`", 
     ",", "8391.15`", ",", "8395.46`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:00:00\"\>", ",", "8396.07`", ",", "8399.09`", 
     ",", "8390.25`", ",", "8394.48`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:10:00\"\>", ",", "8394.55`", ",", "8394.55`", 
     ",", "8382.99`", ",", "8383.75`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:20:00\"\>", ",", "8383.52`", ",", "8387.15`", 
     ",", "8377.62`", ",", "8382.01`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:30:00\"\>", ",", "8381.93`", ",", "8382.16`", 
     ",", "8377.02`", ",", "8377.02`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:40:00\"\>", ",", "8376.87`", ",", "8377.25`", 
     ",", "8370.29`", ",", "8374.15`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T12:50:00\"\>", ",", "8374.68`", ",", "8378.83`", 
     ",", "8373.01`", ",", "8374.07`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:00:00\"\>", ",", "8374.22`", ",", "8390.32`", 
     ",", "8373.39`", ",", "8389.64`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:10:00\"\>", ",", "8389.72`", ",", "8392.14`", 
     ",", "8387.15`", ",", "8387.6`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:20:00\"\>", ",", "8387.83`", ",", "8388.96`", 
     ",", "8381.78`", ",", "8384.2`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:30:00\"\>", ",", "8383.82`", ",", "8383.82`", 
     ",", "8372.11`", ",", "8379.51`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:40:00\"\>", ",", "8379.67`", ",", "8382.46`", 
     ",", "8364.17`", ",", "8365.68`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T13:50:00\"\>", ",", "8365.99`", ",", "8372.94`", 
     ",", "8363.79`", ",", "8364.47`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:00:00\"\>", ",", "8364.4`", ",", "8373.39`", 
     ",", "8364.02`", ",", "8373.17`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:10:00\"\>", ",", "8373.09`", ",", "8382.08`", 
     ",", "8348.83`", ",", "8362.21`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:20:00\"\>", ",", "8362.74`", ",", "8375.96`", 
     ",", "8326.01`", ",", "8333.26`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:30:00\"\>", ",", "8331.45`", ",", "8338.1`", 
     ",", "8289.81`", ",", "8292.23`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:40:00\"\>", ",", "8291.09`", ",", "8323.82`", 
     ",", "8287.84`", ",", "8302.73`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T14:50:00\"\>", ",", "8302.65`", ",", "8326.23`", 
     ",", "8302.35`", ",", "8306.28`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:00:00\"\>", ",", "8307.19`", ",", "8316.41`", 
     ",", "8270.91`", ",", "8277.11`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:10:00\"\>", ",", "8276.58`", ",", "8279.3`", 
     ",", "8260.18`", ",", "8268.12`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:20:00\"\>", ",", "8266.91`", ",", "8295.4`", 
     ",", "8263.51`", ",", "8284.21`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:30:00\"\>", ",", "8284.82`", ",", "8296.08`", 
     ",", "8281.04`", ",", "8285.27`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:40:00\"\>", ",", "8285.5`", ",", "8307.72`", 
     ",", "8262.45`", ",", "8302.13`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T15:50:00\"\>", ",", "8301.67`", ",", "8307.79`", 
     ",", "8284.59`", ",", "8301.52`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-24T16:00:00\"\>", ",", "8301.45`", ",", "8301.6`", 
     ",", "8299.86`", ",", "8299.86`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T09:30:00\"\>", ",", "8299.25`", ",", "8299.25`", 
     ",", "8259.58`", ",", "8280.13`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T09:40:00\"\>", ",", "8280.66`", ",", "8305.9`", 
     ",", "8278.85`", ",", "8292.15`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T09:50:00\"\>", ",", "8292.`", ",", "8326.01`", 
     ",", "8288.37`", ",", "8321.93`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:00:00\"\>", ",", "8323.51`", ",", "8350.57`", 
     ",", "8323.21`", ",", "8349.44`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:10:00\"\>", ",", "8349.81`", ",", "8368.48`", 
     ",", "8341.88`", ",", "8363.87`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:20:00\"\>", ",", "8364.63`", ",", "8365.46`", 
     ",", "8346.87`", ",", "8361.15`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:30:00\"\>", ",", "8359.11`", ",", "8359.26`", 
     ",", "8344.45`", ",", "8347.62`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:40:00\"\>", ",", "8347.7`", ",", "8351.55`", 
     ",", "8331.9`", ",", "8334.09`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T10:50:00\"\>", ",", "8334.55`", ",", "8343.39`", 
     ",", "8332.96`", ",", "8340.29`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:00:00\"\>", ",", "8340.37`", ",", "8356.01`", 
     ",", "8335.61`", ",", "8352.53`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:10:00\"\>", ",", "8352.69`", ",", "8353.59`", 
     ",", "8341.8`", ",", "8349.21`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:20:00\"\>", ",", "8349.28`", ",", "8380.12`", 
     ",", "8349.13`", ",", "8380.12`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:30:00\"\>", ",", "8379.82`", ",", "8407.85`", 
     ",", "8379.82`", ",", "8402.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:40:00\"\>", ",", "8402.87`", ",", "8423.95`", 
     ",", "8402.87`", ",", "8422.21`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T11:50:00\"\>", ",", "8422.06`", ",", "8447.76`", 
     ",", "8415.03`", ",", "8445.94`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:00:00\"\>", ",", "8446.47`", ",", "8456.`", 
     ",", "8443.6`", ",", "8446.62`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:10:00\"\>", ",", "8447.3`", ",", "8452.22`", 
     ",", "8445.04`", ",", "8449.27`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:20:00\"\>", ",", "8449.35`", ",", "8458.94`", 
     ",", "8448.29`", ",", "8454.03`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:30:00\"\>", ",", "8455.54`", ",", "8461.44`", 
     ",", "8436.8`", ",", "8439.22`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:40:00\"\>", ",", "8440.05`", ",", "8442.77`", 
     ",", "8434.76`", ",", "8437.48`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T12:50:00\"\>", ",", "8437.4`", ",", "8447.08`", 
     ",", "8435.97`", ",", "8436.57`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:00:00\"\>", ",", "8436.27`", ",", "8479.42`", 
     ",", "8432.87`", ",", "8477.23`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:10:00\"\>", ",", "8477.08`", ",", "8487.89`", 
     ",", "8467.48`", ",", "8485.55`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:20:00\"\>", ",", "8485.77`", ",", "8490.31`", 
     ",", "8477.84`", ",", "8480.71`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:30:00\"\>", ",", "8482.14`", ",", "8485.55`", 
     ",", "8470.28`", ",", "8473.3`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:40:00\"\>", ",", "8473.15`", ",", "8475.04`", 
     ",", "8463.25`", ",", "8463.4`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T13:50:00\"\>", ",", "8463.33`", ",", "8468.92`", 
     ",", "8458.26`", ",", "8466.65`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:00:00\"\>", ",", "8467.86`", ",", "8470.66`", 
     ",", "8440.73`", ",", "8440.73`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:10:00\"\>", ",", "8440.88`", ",", "8447.3`", 
     ",", "8430.23`", ",", "8437.03`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:20:00\"\>", ",", "8437.78`", ",", "8448.89`", 
     ",", "8435.06`", ",", "8448.89`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:30:00\"\>", ",", "8448.97`", ",", "8453.43`", 
     ",", "8436.57`", ",", "8436.88`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:40:00\"\>", ",", "8439.22`", ",", "8454.86`", 
     ",", "8434.15`", ",", "8451.76`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T14:50:00\"\>", ",", "8451.23`", ",", "8456.15`", 
     ",", "8437.33`", ",", "8446.62`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:00:00\"\>", ",", "8446.55`", ",", "8465.37`", 
     ",", "8440.2`", ",", "8463.1`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:10:00\"\>", ",", "8463.18`", ",", "8469.83`", 
     ",", "8448.51`", ",", "8454.03`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:20:00\"\>", ",", "8453.96`", ",", "8462.65`", 
     ",", "8447.99`", ",", "8458.57`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:30:00\"\>", ",", "8458.41`", ",", "8459.47`", 
     ",", "8417.3`", ",", "8423.73`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:40:00\"\>", ",", "8425.69`", ",", "8459.09`", 
     ",", "8422.21`", ",", "8454.56`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T15:50:00\"\>", ",", "8454.41`", ",", "8478.67`", 
     ",", "8446.02`", ",", "8473.38`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-25T16:00:00\"\>", ",", "8473.76`", ",", "8473.83`", 
     ",", "8472.24`", ",", "8472.4`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T09:30:00\"\>", ",", "8468.54`", ",", "8468.54`", 
     ",", "8421.38`", ",", "8445.79`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T09:40:00\"\>", ",", "8447.15`", ",", "8465.67`", 
     ",", "8446.32`", ",", "8462.27`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T09:50:00\"\>", ",", "8461.97`", ",", "8465.07`", 
     ",", "8428.34`", ",", "8431.89`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:00:00\"\>", ",", "8431.21`", ",", "8439.67`", 
     ",", "8421.84`", ",", "8424.48`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:10:00\"\>", ",", "8424.78`", ",", "8434.84`", 
     ",", "8414.96`", ",", "8433.55`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:20:00\"\>", ",", "8433.25`", ",", "8453.2`", 
     ",", "8431.74`", ",", "8442.77`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:30:00\"\>", ",", "8443.3`", ",", "8448.44`", 
     ",", "8423.5`", ",", "8424.18`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:40:00\"\>", ",", "8424.03`", ",", "8428.64`", 
     ",", "8408.61`", ",", "8418.89`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T10:50:00\"\>", ",", "8419.04`", ",", "8425.54`", 
     ",", "8411.26`", ",", "8421.38`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:00:00\"\>", ",", "8421.61`", ",", "8421.61`", 
     ",", "8401.36`", ",", "8414.43`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:10:00\"\>", ",", "8414.28`", ",", "8421.08`", 
     ",", "8411.56`", ",", "8415.49`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:20:00\"\>", ",", "8415.56`", ",", "8419.72`", 
     ",", "8407.33`", ",", "8414.28`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:30:00\"\>", ",", "8415.41`", ",", "8420.85`", 
     ",", "8408.46`", ",", "8409.9`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:40:00\"\>", ",", "8409.82`", ",", "8413.22`", 
     ",", "8401.58`", ",", "8402.94`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T11:50:00\"\>", ",", "8402.87`", ",", "8422.21`", 
     ",", "8402.34`", ",", "8419.72`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:00:00\"\>", ",", "8419.8`", ",", "8433.55`", 
     ",", "8418.74`", ",", "8427.35`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:10:00\"\>", ",", "8427.96`", ",", "8428.03`", 
     ",", "8415.49`", ",", "8415.49`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:20:00\"\>", ",", "8415.41`", ",", "8421.`", 
     ",", "8412.99`", ",", "8415.79`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:30:00\"\>", ",", "8415.64`", ",", "8423.57`", 
     ",", "8408.31`", ",", "8421.61`", ",", "210", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:40:00\"\>", ",", "8421.99`", ",", "8437.33`", 
     ",", "8419.64`", ",", "8437.25`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T12:50:00\"\>", ",", "8436.88`", ",", "8439.97`", 
     ",", "8431.74`", ",", "8431.74`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:00:00\"\>", ",", "8431.81`", ",", "8433.17`", 
     ",", "8423.12`", ",", "8425.54`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:10:00\"\>", ",", "8425.46`", ",", "8426.22`", 
     ",", "8419.8`", ",", "8424.18`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:20:00\"\>", ",", "8423.88`", ",", "8426.6`", 
     ",", "8418.81`", ",", "8422.97`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:30:00\"\>", ",", "8423.05`", ",", "8423.05`", 
     ",", "8410.35`", ",", "8420.25`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:40:00\"\>", ",", "8420.1`", ",", "8427.43`", 
     ",", "8415.49`", ",", "8421.69`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T13:50:00\"\>", ",", "8421.76`", ",", "8430.75`", 
     ",", "8420.32`", ",", "8421.91`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:00:00\"\>", ",", "8421.46`", ",", "8421.69`", 
     ",", "8410.35`", ",", "8415.26`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:10:00\"\>", ",", "8415.19`", ",", "8421.16`", 
     ",", "8413.98`", ",", "8419.87`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:20:00\"\>", ",", "8419.72`", ",", "8421.`", 
     ",", "8417.08`", ",", "8417.08`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:30:00\"\>", ",", "8417.15`", ",", "8427.58`", 
     ",", "8413.45`", ",", "8421.16`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:40:00\"\>", ",", "8421.53`", ",", "8430.53`", 
     ",", "8418.59`", ",", "8424.03`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T14:50:00\"\>", ",", "8424.25`", ",", "8438.99`", 
     ",", "8423.88`", ",", "8438.99`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:00:00\"\>", ",", "8438.92`", ",", "8451.69`", 
     ",", "8438.92`", ",", "8444.89`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:10:00\"\>", ",", "8444.74`", ",", "8448.67`", 
     ",", "8434.84`", ",", "8437.48`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:20:00\"\>", ",", "8437.63`", ",", "8441.71`", 
     ",", "8431.28`", ",", "8439.22`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:30:00\"\>", ",", "8439.29`", ",", "8444.66`", 
     ",", "8433.1`", ",", "8435.14`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:40:00\"\>", ",", "8435.59`", ",", "8465.59`", 
     ",", "8435.59`", ",", "8456.3`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T15:50:00\"\>", ",", "8456.6`", ",", "8460.61`", 
     ",", "8429.39`", ",", "8430.45`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-26T16:00:00\"\>", ",", "8430.98`", ",", "8438.39`", 
     ",", "8430.23`", ",", "8438.39`", ",", "24", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T09:30:00\"\>", ",", "8440.13`", ",", "8467.11`", 
     ",", "8439.97`", ",", "8447.08`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T09:40:00\"\>", ",", "8446.17`", ",", "8462.87`", 
     ",", "8441.94`", ",", "8445.79`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T09:50:00\"\>", ",", "8445.87`", ",", "8446.25`", 
     ",", "8429.09`", ",", "8438.24`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:00:00\"\>", ",", "8437.78`", ",", "8453.73`", 
     ",", "8433.17`", ",", "8452.67`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:10:00\"\>", ",", "8452.6`", ",", "8479.8`", 
     ",", "8449.65`", ",", "8477.99`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:20:00\"\>", ",", "8477.91`", ",", "8503.76`", 
     ",", "8471.72`", ",", "8503.76`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:30:00\"\>", ",", "8503.46`", ",", "8521.52`", 
     ",", "8503.46`", ",", "8518.27`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:40:00\"\>", ",", "8518.19`", ",", "8530.97`", 
     ",", "8512.07`", ",", "8526.28`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T10:50:00\"\>", ",", "8526.21`", ",", "8527.64`", 
     ",", "8516.76`", ",", "8521.52`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:00:00\"\>", ",", "8520.61`", ",", "8523.56`", 
     ",", "8513.96`", ",", "8514.57`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:10:00\"\>", ",", "8514.72`", ",", "8524.77`", 
     ",", "8510.71`", ",", "8511.69`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:20:00\"\>", ",", "8511.47`", ",", "8522.65`", 
     ",", "8511.09`", ",", "8519.1`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:30:00\"\>", ",", "8520.01`", ",", "8520.84`", 
     ",", "8509.65`", ",", "8509.96`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:40:00\"\>", ",", "8509.88`", ",", "8523.11`", 
     ",", "8508.29`", ",", "8519.63`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T11:50:00\"\>", ",", "8519.55`", ",", "8528.17`", 
     ",", "8519.03`", ",", "8528.17`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:00:00\"\>", ",", "8528.02`", ",", "8529.`", 
     ",", "8520.08`", ",", "8526.13`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:10:00\"\>", ",", "8526.28`", ",", "8529.23`", 
     ",", "8524.84`", ",", "8528.25`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:20:00\"\>", ",", "8528.09`", ",", "8530.14`", 
     ",", "8521.9`", ",", "8526.58`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:30:00\"\>", ",", "8526.36`", ",", "8533.16`", 
     ",", "8522.73`", ",", "8525.3`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:40:00\"\>", ",", "8525.22`", ",", "8526.05`", 
     ",", "8522.28`", ",", "8522.88`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T12:50:00\"\>", ",", "8523.03`", ",", "8523.86`", 
     ",", "8515.7`", ",", "8515.7`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:00:00\"\>", ",", "8515.78`", ",", "8522.43`", 
     ",", "8514.72`", ",", "8520.08`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:10:00\"\>", ",", "8520.39`", ",", "8526.21`", 
     ",", "8520.01`", ",", "8521.37`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:20:00\"\>", ",", "8521.29`", ",", "8526.43`", 
     ",", "8519.4`", ",", "8520.16`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:30:00\"\>", ",", "8520.23`", ",", "8525.98`", 
     ",", "8518.65`", ",", "8525.07`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:40:00\"\>", ",", "8525.15`", ",", "8525.52`", 
     ",", "8518.19`", ",", "8519.71`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T13:50:00\"\>", ",", "8519.63`", ",", "8520.01`", 
     ",", "8510.71`", ",", "8516.61`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:00:00\"\>", ",", "8516.76`", ",", "8516.76`", 
     ",", "8499.45`", ",", "8503.53`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:10:00\"\>", ",", "8503.46`", ",", "8511.09`", 
     ",", "8503.46`", ",", "8509.96`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:20:00\"\>", ",", "8510.03`", ",", "8510.26`", 
     ",", "8505.35`", ",", "8506.93`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:30:00\"\>", ",", "8507.16`", ",", "8516.`", 
     ",", "8507.16`", ",", "8507.84`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:40:00\"\>", ",", "8506.56`", ",", "8513.66`", 
     ",", "8504.44`", ",", "8510.11`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T14:50:00\"\>", ",", "8510.03`", ",", "8519.48`", 
     ",", "8506.4`", ",", "8517.36`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:00:00\"\>", ",", "8517.29`", ",", "8522.88`", 
     ",", "8514.42`", ",", "8516.46`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:10:00\"\>", ",", "8516.83`", ",", "8531.04`", 
     ",", "8515.78`", ",", "8527.64`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:20:00\"\>", ",", "8527.87`", ",", "8532.93`", 
     ",", "8523.71`", ",", "8528.7`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:30:00\"\>", ",", "8527.87`", ",", "8528.4`", 
     ",", "8506.1`", ",", "8506.33`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:40:00\"\>", ",", "8507.31`", ",", "8521.6`", 
     ",", "8507.31`", ",", "8510.33`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T15:50:00\"\>", ",", "8510.11`", ",", "8531.19`", 
     ",", "8502.47`", ",", "8529.76`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-29T16:00:00\"\>", ",", "8529.91`", ",", "8530.14`", 
     ",", "8528.85`", ",", "8529.38`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T09:30:00\"\>", ",", "8528.93`", ",", "8549.63`", 
     ",", "8524.16`", ",", "8545.78`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T09:40:00\"\>", ",", "8545.63`", ",", "8560.06`", 
     ",", "8541.02`", ",", "8546.53`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T09:50:00\"\>", ",", "8544.87`", ",", "8552.28`", 
     ",", "8533.61`", ",", "8550.54`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:00:00\"\>", ",", "8544.42`", ",", "8544.42`", 
     ",", "8500.28`", ",", "8516.08`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:10:00\"\>", ",", "8516.76`", ",", "8517.67`", 
     ",", "8482.22`", ",", "8482.22`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:20:00\"\>", ",", "8481.69`", ",", "8484.11`", 
     ",", "8455.24`", ",", "8462.34`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:30:00\"\>", ",", "8458.79`", ",", "8459.32`", 
     ",", "8422.29`", ",", "8427.35`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:40:00\"\>", ",", "8427.81`", ",", "8436.95`", 
     ",", "8419.72`", ",", "8436.95`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T10:50:00\"\>", ",", "8435.89`", ",", "8439.07`", 
     ",", "8430.91`", ",", "8435.14`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:00:00\"\>", ",", "8435.52`", ",", "8440.43`", 
     ",", "8417.53`", ",", "8420.7`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:10:00\"\>", ",", "8420.4`", ",", "8427.43`", 
     ",", "8413.07`", ",", "8427.35`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:20:00\"\>", ",", "8427.28`", ",", "8429.24`", 
     ",", "8420.78`", ",", "8422.14`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:30:00\"\>", ",", "8419.19`", ",", "8419.19`", 
     ",", "8394.25`", ",", "8400.52`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:40:00\"\>", ",", "8400.83`", ",", "8412.92`", 
     ",", "8400.6`", ",", "8412.92`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T11:50:00\"\>", ",", "8409.67`", ",", "8413.22`", 
     ",", "8404.08`", ",", "8406.12`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:00:00\"\>", ",", "8406.57`", ",", "8410.12`", 
     ",", "8404.61`", ",", "8409.22`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:10:00\"\>", ",", "8409.29`", ",", "8416.32`", 
     ",", "8408.38`", ",", "8409.06`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:20:00\"\>", ",", "8409.22`", ",", "8414.96`", 
     ",", "8397.8`", ",", "8414.96`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:30:00\"\>", ",", "8414.51`", ",", "8428.56`", 
     ",", "8413.9`", ",", "8419.8`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:40:00\"\>", ",", "8419.64`", ",", "8425.92`", 
     ",", "8418.89`", ",", "8419.95`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T12:50:00\"\>", ",", "8420.17`", ",", "8424.86`", 
     ",", "8419.04`", ",", "8423.65`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:00:00\"\>", ",", "8423.8`", ",", "8424.71`", 
     ",", "8415.03`", ",", "8424.03`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:10:00\"\>", ",", "8423.88`", ",", "8424.25`", 
     ",", "8412.54`", ",", "8412.62`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:20:00\"\>", ",", "8412.69`", ",", "8420.1`", 
     ",", "8410.73`", ",", "8413.37`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:30:00\"\>", ",", "8412.16`", ",", "8415.19`", 
     ",", "8409.59`", ",", "8415.19`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:40:00\"\>", ",", "8414.96`", ",", "8423.12`", 
     ",", "8414.2`", ",", "8417.53`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T13:50:00\"\>", ",", "8417.6`", ",", "8419.12`", 
     ",", "8406.34`", ",", "8409.22`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:00:00\"\>", ",", "8409.29`", ",", "8413.3`", 
     ",", "8403.4`", ",", "8410.2`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:10:00\"\>", ",", "8411.48`", ",", "8421.53`", 
     ",", "8411.48`", ",", "8418.21`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:20:00\"\>", ",", "8418.74`", ",", "8428.79`", 
     ",", "8416.17`", ",", "8425.16`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:30:00\"\>", ",", "8426.07`", ",", "8434.53`", 
     ",", "8416.77`", ",", "8433.4`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:40:00\"\>", ",", "8433.55`", ",", "8437.03`", 
     ",", "8427.58`", ",", "8427.88`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T14:50:00\"\>", ",", "8427.73`", ",", "8437.48`", 
     ",", "8427.58`", ",", "8427.58`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:00:00\"\>", ",", "8427.88`", ",", "8431.74`", 
     ",", "8416.39`", ",", "8419.95`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:10:00\"\>", ",", "8420.02`", ",", "8431.89`", 
     ",", "8416.85`", ",", "8417.6`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:20:00\"\>", ",", "8417.76`", ",", "8437.78`", 
     ",", "8416.39`", ",", "8430.75`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:30:00\"\>", ",", "8430.68`", ",", "8448.89`", 
     ",", "8430.68`", ",", "8445.34`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:40:00\"\>", ",", "8444.06`", ",", "8456.9`", 
     ",", "8435.06`", ",", "8454.26`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T15:50:00\"\>", ",", "8453.5`", ",", "8453.5`", 
     ",", "8430.07`", ",", "8449.42`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-06-30T16:00:00\"\>", ",", "8448.51`", ",", "8448.97`", 
     ",", "8447.`", ",", "8447.`", ",", "23", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T09:30:00\"\>", ",", "8447.53`", ",", "8523.41`", 
     ",", "8447.53`", ",", "8516.15`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T09:40:00\"\>", ",", "8519.25`", ",", "8524.92`", 
     ",", "8511.01`", ",", "8524.92`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T09:50:00\"\>", ",", "8526.89`", ",", "8542.76`", 
     ",", "8525.6`", ",", "8542.38`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:00:00\"\>", ",", "8542.23`", ",", "8542.6`", 
     ",", "8498.7`", ",", "8511.09`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:10:00\"\>", ",", "8510.94`", ",", "8546.38`", 
     ",", "8505.2`", ",", "8541.77`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:20:00\"\>", ",", "8542.76`", ",", "8574.95`", 
     ",", "8542.76`", ",", "8574.27`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:30:00\"\>", ",", "8574.95`", ",", "8580.39`", 
     ",", "8569.89`", ",", "8570.64`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:40:00\"\>", ",", "8570.72`", ",", "8574.95`", 
     ",", "8566.79`", ",", "8567.09`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T10:50:00\"\>", ",", "8566.94`", ",", "8569.96`", 
     ",", "8561.73`", ",", "8565.88`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:00:00\"\>", ",", "8566.11`", ",", "8570.79`", 
     ",", "8558.17`", ",", "8560.74`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:10:00\"\>", ",", "8559.53`", ",", "8566.64`", 
     ",", "8556.21`", ",", "8556.36`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:20:00\"\>", ",", "8556.59`", ",", "8557.87`", 
     ",", "8543.13`", ",", "8543.13`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:30:00\"\>", ",", "8542.45`", ",", "8554.85`", 
     ",", "8542.45`", ",", "8552.28`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:40:00\"\>", ",", "8552.05`", ",", "8552.2`", 
     ",", "8543.66`", ",", "8545.02`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T11:50:00\"\>", ",", "8545.1`", ",", "8555.`", ",",
      "8544.57`", ",", "8552.2`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:00:00\"\>", ",", "8552.28`", ",", "8560.21`", 
     ",", "8544.34`", ",", "8556.51`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:10:00\"\>", ",", "8556.44`", ",", "8558.7`", 
     ",", "8538.9`", ",", "8548.42`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:20:00\"\>", ",", "8548.58`", ",", "8558.4`", 
     ",", "8545.02`", ",", "8556.06`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:30:00\"\>", ",", "8555.98`", ",", "8556.89`", 
     ",", "8550.99`", ",", "8551.07`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:40:00\"\>", ",", "8550.99`", ",", "8552.13`", 
     ",", "8548.2`", ",", "8549.41`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T12:50:00\"\>", ",", "8548.58`", ",", "8548.58`", 
     ",", "8536.33`", ",", "8536.48`", ",", "212", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:00:00\"\>", ",", "8536.63`", ",", "8544.57`", 
     ",", "8536.56`", ",", "8544.57`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:10:00\"\>", ",", "8544.65`", ",", "8547.52`", 
     ",", "8542.53`", ",", "8543.66`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:20:00\"\>", ",", "8543.44`", ",", "8552.35`", 
     ",", "8541.09`", ",", "8552.2`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:30:00\"\>", ",", "8551.67`", ",", "8552.73`", 
     ",", "8546.23`", ",", "8548.73`", ",", "216", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:40:00\"\>", ",", "8548.58`", ",", "8549.03`", 
     ",", "8539.43`", ",", "8539.73`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T13:50:00\"\>", ",", "8539.66`", ",", "8546.38`", 
     ",", "8539.05`", ",", "8546.31`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:00:00\"\>", ",", "8546.53`", ",", "8546.84`", 
     ",", "8523.86`", ",", "8524.77`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:10:00\"\>", ",", "8524.62`", ",", "8526.66`", 
     ",", "8520.54`", ",", "8525.75`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:20:00\"\>", ",", "8525.83`", ",", "8529.15`", 
     ",", "8522.65`", ",", "8528.32`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:30:00\"\>", ",", "8527.94`", ",", "8529.08`", 
     ",", "8516.`", ",", "8526.66`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:40:00\"\>", ",", "8526.89`", ",", "8531.95`", 
     ",", "8525.07`", ",", "8528.4`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T14:50:00\"\>", ",", "8528.32`", ",", "8529.53`", 
     ",", "8517.51`", ",", "8523.94`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:00:00\"\>", ",", "8523.86`", ",", "8529.53`", 
     ",", "8521.82`", ",", "8523.26`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:10:00\"\>", ",", "8524.09`", ",", "8526.36`", 
     ",", "8517.21`", ",", "8522.5`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:20:00\"\>", ",", "8522.65`", ",", "8532.25`", 
     ",", "8512.98`", ",", "8514.72`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:30:00\"\>", ",", "8514.57`", ",", "8522.2`", 
     ",", "8510.03`", ",", "8521.52`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:40:00\"\>", ",", "8523.03`", ",", "8528.77`", 
     ",", "8501.19`", ",", "8501.42`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T15:50:00\"\>", ",", "8498.62`", ",", "8515.1`", 
     ",", "8492.5`", ",", "8501.42`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-01T16:00:00\"\>", ",", "8501.64`", ",", "8504.14`", 
     ",", "8501.64`", ",", "8504.06`", ",", "16", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T09:30:00\"\>", ",", "8503.`", ",", "8503.`", ",", 
     "8353.89`", ",", "8355.93`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T09:40:00\"\>", ",", "8352.08`", ",", "8359.11`", 
     ",", "8344.45`", ",", "8359.11`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T09:50:00\"\>", ",", "8359.03`", ",", "8361.3`", 
     ",", "8333.72`", ",", "8344.75`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:00:00\"\>", ",", "8351.1`", ",", "8356.46`", 
     ",", "8323.66`", ",", "8329.03`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:10:00\"\>", ",", "8329.41`", ",", "8333.94`", 
     ",", "8323.21`", ",", "8326.01`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:20:00\"\>", ",", "8326.91`", ",", "8329.56`", 
     ",", "8314.44`", ",", "8326.99`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:30:00\"\>", ",", "8327.07`", ",", "8334.09`", 
     ",", "8317.85`", ",", "8329.18`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:40:00\"\>", ",", "8329.11`", ",", "8338.63`", 
     ",", "8327.97`", ",", "8335.15`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T10:50:00\"\>", ",", "8335.3`", ",", "8345.66`", 
     ",", "8331.45`", ",", "8339.23`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:00:00\"\>", ",", "8339.16`", ",", "8346.79`", 
     ",", "8334.93`", ",", "8345.2`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:10:00\"\>", ",", "8345.13`", ",", "8347.7`", 
     ",", "8338.86`", ",", "8342.18`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:20:00\"\>", ",", "8341.95`", ",", "8342.33`", 
     ",", "8332.2`", ",", "8335.83`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:30:00\"\>", ",", "8335.76`", ",", "8335.76`", 
     ",", "8324.65`", ",", "8332.66`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:40:00\"\>", ",", "8332.73`", ",", "8332.81`", 
     ",", "8317.77`", ",", "8326.46`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T11:50:00\"\>", ",", "8326.91`", ",", "8332.05`", 
     ",", "8322.08`", ",", "8330.09`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:00:00\"\>", ",", "8330.32`", ",", "8335.38`", 
     ",", "8327.14`", ",", "8329.63`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:10:00\"\>", ",", "8329.56`", ",", "8339.69`", 
     ",", "8328.88`", ",", "8333.72`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:20:00\"\>", ",", "8333.87`", ",", "8338.17`", 
     ",", "8330.47`", ",", "8333.64`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:30:00\"\>", ",", "8332.2`", ",", "8332.88`", 
     ",", "8324.27`", ",", "8329.48`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:40:00\"\>", ",", "8329.41`", ",", "8333.41`", 
     ",", "8325.78`", ",", "8330.16`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T12:50:00\"\>", ",", "8329.94`", ",", "8332.43`", 
     ",", "8327.14`", ",", "8328.58`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:00:00\"\>", ",", "8329.41`", ",", "8336.29`", 
     ",", "8329.41`", ",", "8335.15`", ",", "211", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:10:00\"\>", ",", "8335.23`", ",", "8338.7`", 
     ",", "8330.77`", ",", "8332.2`", ",", "217", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:20:00\"\>", ",", "8332.43`", ",", "8334.85`", 
     ",", "8325.63`", ",", "8328.2`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:30:00\"\>", ",", "8328.73`", ",", "8328.73`", 
     ",", "8320.34`", ",", "8326.99`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:40:00\"\>", ",", "8327.22`", ",", "8333.79`", 
     ",", "8325.33`", ",", "8328.8`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T13:50:00\"\>", ",", "8328.88`", ",", "8328.95`", 
     ",", "8316.11`", ",", "8322.`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:00:00\"\>", ",", "8321.25`", ",", "8330.24`", 
     ",", "8321.09`", ",", "8328.95`", ",", "216", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:10:00\"\>", ",", "8329.03`", ",", "8329.03`", 
     ",", "8316.41`", ",", "8318.75`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:20:00\"\>", ",", "8318.98`", ",", "8328.88`", 
     ",", "8318.83`", ",", "8322.3`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:30:00\"\>", ",", "8322.46`", ",", "8332.73`", 
     ",", "8316.03`", ",", "8329.11`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:40:00\"\>", ",", "8329.18`", ",", "8331.07`", 
     ",", "8320.64`", ",", "8321.55`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T14:50:00\"\>", ",", "8321.62`", ",", "8326.99`", 
     ",", "8320.87`", ",", "8326.91`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:00:00\"\>", ",", "8327.29`", ",", "8334.55`", 
     ",", "8327.29`", ",", "8333.41`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:10:00\"\>", ",", "8333.56`", ",", "8333.56`", 
     ",", "8325.86`", ",", "8330.54`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:20:00\"\>", ",", "8329.86`", ",", "8329.94`", 
     ",", "8300.01`", ",", "8301.75`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:30:00\"\>", ",", "8301.9`", ",", "8310.67`", 
     ",", "8294.95`", ",", "8309.53`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:40:00\"\>", ",", "8309.15`", ",", "8320.11`", 
     ",", "8306.36`", ",", "8319.81`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T15:50:00\"\>", ",", "8319.36`", ",", "8319.36`", 
     ",", "8291.55`", ",", "8291.77`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T16:00:00\"\>", ",", "8291.85`", ",", "8291.92`", 
     ",", "8289.66`", ",", "8291.92`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-02T16:10:00\"\>", ",", "8291.85`", ",", "8292.3`", 
     ",", "8287.69`", ",", "8289.81`", ",", "42", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T09:30:00\"\>", ",", "8279.3`", ",", "8279.3`", 
     ",", "8213.85`", ",", "8247.71`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T09:40:00\"\>", ",", "8247.41`", ",", "8271.97`", 
     ",", "8238.87`", ",", "8271.97`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T09:50:00\"\>", ",", "8271.59`", ",", "8271.67`", 
     ",", "8254.74`", ",", "8265.93`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:00:00\"\>", ",", "8274.09`", ",", "8280.36`", 
     ",", "8244.39`", ",", "8244.39`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:10:00\"\>", ",", "8243.48`", ",", "8243.48`", 
     ",", "8226.32`", ",", "8226.32`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:20:00\"\>", ",", "8224.59`", ",", "8225.64`", 
     ",", "8210.15`", ",", "8218.09`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:30:00\"\>", ",", "8216.73`", ",", "8219.07`", 
     ",", "8207.43`", ",", "8209.62`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:40:00\"\>", ",", "8209.4`", ",", "8224.81`", 
     ",", "8206.15`", ",", "8219.82`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T10:50:00\"\>", ",", "8219.67`", ",", "8227.38`", 
     ",", "8214.84`", ",", "8225.04`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:00:00\"\>", ",", "8225.11`", ",", "8230.41`", 
     ",", "8223.98`", ",", "8227.68`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:10:00\"\>", ",", "8227.46`", ",", "8238.49`", 
     ",", "8225.27`", ",", "8238.49`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:20:00\"\>", ",", "8238.57`", ",", "8247.71`", 
     ",", "8238.57`", ",", "8240.91`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:30:00\"\>", ",", "8240.53`", ",", "8249.53`", 
     ",", "8236.9`", ",", "8247.41`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:40:00\"\>", ",", "8247.49`", ",", "8259.95`", 
     ",", "8239.63`", ",", "8258.37`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T11:50:00\"\>", ",", "8258.07`", ",", "8260.33`", 
     ",", "8240.31`", ",", "8243.1`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:00:00\"\>", ",", "8243.33`", ",", "8251.11`", 
     ",", "8242.57`", ",", "8248.62`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:10:00\"\>", ",", "8248.69`", ",", "8258.75`", 
     ",", "8247.41`", ",", "8258.37`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:20:00\"\>", ",", "8258.52`", ",", "8262.07`", 
     ",", "8250.73`", ",", "8252.47`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:30:00\"\>", ",", "8252.55`", ",", "8254.44`", 
     ",", "8245.97`", ",", "8252.02`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:40:00\"\>", ",", "8252.1`", ",", "8252.55`", 
     ",", "8243.48`", ",", "8249.75`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T12:50:00\"\>", ",", "8249.68`", ",", "8261.24`", 
     ",", "8246.43`", ",", "8259.5`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:00:00\"\>", ",", "8259.43`", ",", "8270.01`", 
     ",", "8258.37`", ",", "8269.63`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:10:00\"\>", ",", "8269.78`", ",", "8293.74`", 
     ",", "8266.61`", ",", "8288.45`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:20:00\"\>", ",", "8288.6`", ",", "8297.74`", 
     ",", "8287.77`", ",", "8290.94`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:30:00\"\>", ",", "8291.47`", ",", "8291.62`", 
     ",", "8283.01`", ",", "8287.54`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:40:00\"\>", ",", "8287.99`", ",", "8288.37`", 
     ",", "8276.51`", ",", "8278.09`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T13:50:00\"\>", ",", "8278.17`", ",", "8287.46`", 
     ",", "8278.17`", ",", "8283.99`", ",", "218", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:00:00\"\>", ",", "8284.67`", ",", "8293.59`", 
     ",", "8283.61`", ",", "8289.81`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:10:00\"\>", ",", "8289.5`", ",", "8298.95`", 
     ",", "8287.31`", ",", "8287.62`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:20:00\"\>", ",", "8287.31`", ",", "8287.84`", 
     ",", "8278.92`", ",", "8285.05`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:30:00\"\>", ",", "8285.27`", ",", "8294.19`", 
     ",", "8280.59`", ",", "8291.92`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:40:00\"\>", ",", "8291.7`", ",", "8296.38`", 
     ",", "8281.19`", ",", "8291.62`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T14:50:00\"\>", ",", "8291.55`", ",", "8293.21`", 
     ",", "8283.08`", ",", "8290.49`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:00:00\"\>", ",", "8292.53`", ",", "8309.91`", 
     ",", "8292.53`", ",", "8300.24`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:10:00\"\>", ",", "8300.01`", ",", "8300.01`", 
     ",", "8286.03`", ",", "8290.71`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:20:00\"\>", ",", "8290.56`", ",", "8298.5`", 
     ",", "8286.86`", ",", "8292.`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:30:00\"\>", ",", "8291.24`", ",", "8291.24`", 
     ",", "8270.61`", ",", "8281.12`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:40:00\"\>", ",", "8280.59`", ",", "8326.84`", 
     ",", "8280.59`", ",", "8311.27`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T15:50:00\"\>", ",", "8311.35`", ",", "8327.82`", 
     ",", "8308.78`", ",", "8324.5`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-06T16:00:00\"\>", ",", "8324.19`", ",", "8325.18`", 
     ",", "8323.59`", ",", "8324.87`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T09:30:00\"\>", ",", "8324.95`", ",", "8324.95`", 
     ",", "8287.31`", ",", "8288.14`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T09:40:00\"\>", ",", "8288.07`", ",", "8288.82`", 
     ",", "8274.84`", ",", "8276.73`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T09:50:00\"\>", ",", "8277.34`", ",", "8292.`", 
     ",", "8276.43`", ",", "8284.97`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:00:00\"\>", ",", "8283.08`", ",", "8283.08`", 
     ",", "8261.54`", ",", "8266.68`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:10:00\"\>", ",", "8266.83`", ",", "8266.83`", 
     ",", "8249.45`", ",", "8254.82`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:20:00\"\>", ",", "8254.36`", ",", "8262.9`", 
     ",", "8252.02`", ",", "8257.84`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:30:00\"\>", ",", "8257.99`", ",", "8257.99`", 
     ",", "8239.4`", ",", "8243.71`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:40:00\"\>", ",", "8243.86`", ",", "8255.57`", 
     ",", "8243.33`", ",", "8249.37`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T10:50:00\"\>", ",", "8249.68`", ",", "8250.81`", 
     ",", "8241.36`", ",", "8244.31`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:00:00\"\>", ",", "8244.54`", ",", "8257.16`", 
     ",", "8240.46`", ",", "8253.3`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:10:00\"\>", ",", "8253.38`", ",", "8258.52`", 
     ",", "8252.55`", ",", "8254.36`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:20:00\"\>", ",", "8254.59`", ",", "8255.57`", 
     ",", "8233.65`", ",", "8233.65`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:30:00\"\>", ",", "8233.5`", ",", "8239.55`", 
     ",", "8226.4`", ",", "8229.04`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:40:00\"\>", ",", "8229.12`", ",", "8230.56`", 
     ",", "8219.98`", ",", "8224.81`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T11:50:00\"\>", ",", "8225.11`", ",", "8236.98`", 
     ",", "8224.96`", ",", "8235.92`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:00:00\"\>", ",", "8235.85`", ",", "8237.28`", 
     ",", "8230.1`", ",", "8230.78`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:10:00\"\>", ",", "8231.31`", ",", "8237.13`", 
     ",", "8230.78`", ",", "8236.68`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:20:00\"\>", ",", "8236.45`", ",", "8236.45`", 
     ",", "8221.71`", ",", "8223.6`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:30:00\"\>", ",", "8223.91`", ",", "8225.34`", 
     ",", "8215.06`", ",", "8215.06`", ",", "215", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:40:00\"\>", ",", "8214.61`", ",", "8222.7`", 
     ",", "8213.17`", ",", "8221.94`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T12:50:00\"\>", ",", "8221.71`", ",", "8225.87`", 
     ",", "8218.62`", ",", "8223.23`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:00:00\"\>", ",", "8223.38`", ",", "8234.64`", 
     ",", "8220.81`", ",", "8234.41`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:10:00\"\>", ",", "8234.49`", ",", "8247.94`", 
     ",", "8234.41`", ",", "8247.94`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:20:00\"\>", ",", "8248.17`", ",", "8253.3`", 
     ",", "8242.5`", ",", "8244.61`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:30:00\"\>", ",", "8247.56`", ",", "8257.69`", 
     ",", "8245.44`", ",", "8248.92`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:40:00\"\>", ",", "8249.07`", ",", "8251.64`", 
     ",", "8243.86`", ",", "8244.16`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T13:50:00\"\>", ",", "8244.08`", ",", "8246.35`", 
     ",", "8237.13`", ",", "8243.1`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:00:00\"\>", ",", "8243.25`", ",", "8244.69`", 
     ",", "8231.09`", ",", "8231.61`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:10:00\"\>", ",", "8231.69`", ",", "8241.06`", 
     ",", "8227.31`", ",", "8229.65`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:20:00\"\>", ",", "8230.1`", ",", "8243.18`", 
     ",", "8229.8`", ",", "8242.19`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:30:00\"\>", ",", "8242.12`", ",", "8246.5`", 
     ",", "8231.31`", ",", "8245.22`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:40:00\"\>", ",", "8245.29`", ",", "8247.86`", 
     ",", "8229.27`", ",", "8229.42`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T14:50:00\"\>", ",", "8229.35`", ",", "8230.03`", 
     ",", "8204.48`", ",", "8206.45`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:00:00\"\>", ",", "8206.67`", ",", "8206.67`", 
     ",", "8178.41`", ",", "8187.25`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:10:00\"\>", ",", "8187.03`", ",", "8193.98`", 
     ",", "8183.85`", ",", "8189.9`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:20:00\"\>", ",", "8189.82`", ",", "8197.53`", 
     ",", "8178.56`", ",", "8195.72`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:30:00\"\>", ",", "8195.49`", ",", "8197.76`", 
     ",", "8183.1`", ",", "8183.85`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:40:00\"\>", ",", "8183.55`", ",", "8186.27`", 
     ",", "8163.67`", ",", "8166.7`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T15:50:00\"\>", ",", "8167.07`", ",", "8173.5`", 
     ",", "8154.68`", ",", "8159.82`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-07T16:00:00\"\>", ",", "8159.74`", ",", "8163.6`", 
     ",", "8159.74`", ",", "8163.6`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T09:30:00\"\>", ",", "8157.02`", ",", "8193.15`", 
     ",", "8156.49`", ",", "8170.93`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T09:40:00\"\>", ",", "8170.63`", ",", "8200.48`", 
     ",", "8169.04`", ",", "8196.09`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T09:50:00\"\>", ",", "8196.55`", ",", "8201.69`", 
     ",", "8172.67`", ",", "8175.24`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:00:00\"\>", ",", "8175.54`", ",", "8210.08`", 
     ",", "8175.16`", ",", "8201.69`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:10:00\"\>", ",", "8201.38`", ",", "8213.48`", 
     ",", "8195.72`", ",", "8213.33`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:20:00\"\>", ",", "8213.63`", ",", "8219.37`", 
     ",", "8196.17`", ",", "8203.42`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:30:00\"\>", ",", "8202.29`", ",", "8203.12`", 
     ",", "8181.36`", ",", "8188.91`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:40:00\"\>", ",", "8189.07`", ",", "8202.74`", 
     ",", "8182.42`", ",", "8192.54`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T10:50:00\"\>", ",", "8187.78`", ",", "8198.29`", 
     ",", "8174.93`", ",", "8174.93`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:00:00\"\>", ",", "8173.5`", ",", "8173.5`", 
     ",", "8147.5`", ",", "8148.1`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:10:00\"\>", ",", "8147.27`", ",", "8156.64`", 
     ",", "8137.98`", ",", "8148.33`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:20:00\"\>", ",", "8147.88`", ",", "8148.48`", 
     ",", "8124.45`", ",", "8129.66`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:30:00\"\>", ",", "8129.29`", ",", "8143.42`", 
     ",", "8129.29`", ",", "8134.65`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:40:00\"\>", ",", "8134.43`", ",", "8148.1`", 
     ",", "8134.27`", ",", "8145.46`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T11:50:00\"\>", ",", "8145.69`", ",", "8145.69`", 
     ",", "8116.44`", ",", "8129.36`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:00:00\"\>", ",", "8129.29`", ",", "8134.8`", 
     ",", "8121.88`", ",", "8133.9`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:10:00\"\>", ",", "8133.67`", ",", "8135.11`", 
     ",", "8111.68`", ",", "8113.72`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:20:00\"\>", ",", "8113.49`", ",", "8115.61`", 
     ",", "8100.87`", ",", "8103.97`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:30:00\"\>", ",", "8103.82`", ",", "8113.11`", 
     ",", "8100.79`", ",", "8103.44`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:40:00\"\>", ",", "8102.83`", ",", "8119.76`", 
     ",", "8101.7`", ",", "8118.55`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T12:50:00\"\>", ",", "8118.48`", ",", "8124.3`", 
     ",", "8114.1`", ",", "8114.25`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:00:00\"\>", ",", "8114.55`", ",", "8147.73`", 
     ",", "8113.49`", ",", "8141.76`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:10:00\"\>", ",", "8141.91`", ",", "8149.77`", 
     ",", "8135.26`", ",", "8142.13`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:20:00\"\>", ",", "8141.76`", ",", "8166.09`", 
     ",", "8138.81`", ",", "8161.18`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:30:00\"\>", ",", "8160.2`", ",", "8164.88`", 
     ",", "8141.3`", ",", "8143.8`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:40:00\"\>", ",", "8143.49`", ",", "8150.82`", 
     ",", "8140.55`", ",", "8145.16`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T13:50:00\"\>", ",", "8144.85`", ",", "8145.08`", 
     ",", "8131.93`", ",", "8134.88`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:00:00\"\>", ",", "8134.8`", ",", "8140.77`", 
     ",", "8116.51`", ",", "8119.69`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:10:00\"\>", ",", "8119.76`", ",", "8120.97`", 
     ",", "8104.12`", ",", "8106.31`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:20:00\"\>", ",", "8108.73`", ",", "8120.44`", 
     ",", "8094.6`", ",", "8102.31`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:30:00\"\>", ",", "8102.08`", ",", "8112.13`", 
     ",", "8087.34`", ",", "8109.26`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:40:00\"\>", ",", "8109.33`", ",", "8116.06`", 
     ",", "8100.87`", ",", "8101.85`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T14:50:00\"\>", ",", "8102.31`", ",", "8136.16`", 
     ",", "8102.`", ",", "8132.23`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:00:00\"\>", ",", "8132.54`", ",", "8148.33`", 
     ",", "8126.72`", ",", "8138.73`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:10:00\"\>", ",", "8138.88`", ",", "8152.11`", 
     ",", "8131.55`", ",", "8141.6`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:20:00\"\>", ",", "8141.98`", ",", "8149.92`", 
     ",", "8121.8`", ",", "8129.81`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:30:00\"\>", ",", "8130.42`", ",", "8163.82`", 
     ",", "8130.34`", ",", "8158.68`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:40:00\"\>", ",", "8158.61`", ",", "8205.99`", 
     ",", "8154.98`", ",", "8185.06`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T15:50:00\"\>", ",", "8184.53`", ",", "8192.77`", 
     ",", "8167.45`", ",", "8180.07`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-08T16:00:00\"\>", ",", "8179.77`", ",", "8179.77`", 
     ",", "8178.18`", ",", "8178.41`", ",", "13", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T09:30:00\"\>", ",", "8179.01`", ",", "8228.67`", 
     ",", "8179.01`", ",", "8203.58`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T09:40:00\"\>", ",", "8202.29`", ",", "8202.29`", 
     ",", "8171.99`", ",", "8171.99`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T09:50:00\"\>", ",", "8171.23`", ",", "8195.49`", 
     ",", "8168.89`", ",", "8185.89`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:00:00\"\>", ",", "8186.12`", ",", "8207.58`", 
     ",", "8173.19`", ",", "8199.87`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:10:00\"\>", ",", "8200.18`", ",", "8203.73`", 
     ",", "8193.37`", ",", "8194.05`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:20:00\"\>", ",", "8193.9`", ",", "8194.58`", 
     ",", "8159.36`", ",", "8171.68`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:30:00\"\>", ",", "8170.7`", ",", "8180.45`", 
     ",", "8157.17`", ",", "8168.21`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:40:00\"\>", ",", "8168.36`", ",", "8192.54`", 
     ",", "8164.96`", ",", "8165.94`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T10:50:00\"\>", ",", "8166.17`", ",", "8180.68`", 
     ",", "8166.17`", ",", "8178.26`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:00:00\"\>", ",", "8177.8`", ",", "8177.8`", 
     ",", "8149.62`", ",", "8156.11`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:10:00\"\>", ",", "8155.96`", ",", "8163.07`", 
     ",", "8145.38`", ",", "8157.48`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:20:00\"\>", ",", "8157.78`", ",", "8175.08`", 
     ",", "8157.63`", ",", "8166.09`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:30:00\"\>", ",", "8165.71`", ",", "8178.71`", 
     ",", "8156.34`", ",", "8174.4`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:40:00\"\>", ",", "8174.25`", ",", "8195.41`", 
     ",", "8174.25`", ",", "8187.03`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T11:50:00\"\>", ",", "8186.95`", ",", "8188.54`", 
     ",", "8173.04`", ",", "8174.03`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:00:00\"\>", ",", "8173.8`", ",", "8183.02`", 
     ",", "8167.22`", ",", "8181.21`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:10:00\"\>", ",", "8181.13`", ",", "8187.1`", 
     ",", "8175.39`", ",", "8186.95`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:20:00\"\>", ",", "8187.03`", ",", "8195.64`", 
     ",", "8185.14`", ",", "8186.5`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:30:00\"\>", ",", "8186.72`", ",", "8189.75`", 
     ",", "8177.58`", ",", "8186.04`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:40:00\"\>", ",", "8186.34`", ",", "8187.25`", 
     ",", "8179.85`", ",", "8181.05`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T12:50:00\"\>", ",", "8180.98`", ",", "8190.58`", 
     ",", "8173.65`", ",", "8190.35`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:00:00\"\>", ",", "8190.58`", ",", "8191.18`", 
     ",", "8167.15`", ",", "8182.94`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:10:00\"\>", ",", "8183.1`", ",", "8205.24`", 
     ",", "8183.1`", ",", "8195.79`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:20:00\"\>", ",", "8195.72`", ",", "8202.74`", 
     ",", "8190.05`", ",", "8197.15`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:30:00\"\>", ",", "8198.89`", ",", "8204.63`", 
     ",", "8187.25`", ",", "8189.29`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:40:00\"\>", ",", "8189.37`", ",", "8196.62`", 
     ",", "8184.23`", ",", "8190.73`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T13:50:00\"\>", ",", "8191.11`", ",", "8194.96`", 
     ",", "8188.91`", ",", "8194.88`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:00:00\"\>", ",", "8195.11`", ",", "8197.08`", 
     ",", "8188.69`", ",", "8189.75`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:10:00\"\>", ",", "8189.97`", ",", "8214.53`", 
     ",", "8189.97`", ",", "8212.27`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:20:00\"\>", ",", "8214.08`", ",", "8221.94`", 
     ",", "8211.89`", ",", "8215.44`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:30:00\"\>", ",", "8214.76`", ",", "8221.56`", 
     ",", "8211.36`", ",", "8214.08`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:40:00\"\>", ",", "8214.23`", ",", "8215.37`", 
     ",", "8207.05`", ",", "8210.53`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T14:50:00\"\>", ",", "8210.45`", ",", "8213.55`", 
     ",", "8194.88`", ",", "8197.3`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:00:00\"\>", ",", "8196.93`", ",", "8196.93`", 
     ",", "8182.87`", ",", "8184.53`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:10:00\"\>", ",", "8184.83`", ",", "8199.95`", 
     ",", "8184.83`", ",", "8199.49`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:20:00\"\>", ",", "8199.19`", ",", "8199.34`", 
     ",", "8175.69`", ",", "8180.68`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:30:00\"\>", ",", "8181.05`", ",", "8183.78`", 
     ",", "8164.73`", ",", "8170.85`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:40:00\"\>", ",", "8171.76`", ",", "8197.`", 
     ",", "8171.76`", ",", "8191.26`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T15:50:00\"\>", ",", "8191.41`", ",", "8191.56`", 
     ",", "8164.13`", ",", "8177.65`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-09T16:00:00\"\>", ",", "8177.58`", ",", "8183.55`", 
     ",", "8177.58`", ",", "8183.17`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T09:30:00\"\>", ",", "8182.49`", ",", "8182.49`", 
     ",", "8127.7`", ",", "8154.53`", ",", "293", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T09:40:00\"\>", ",", "8154.91`", ",", "8172.89`", 
     ",", "8149.54`", ",", "8172.51`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T09:50:00\"\>", ",", "8172.74`", ",", "8183.7`", 
     ",", "8135.03`", ",", "8147.05`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:00:00\"\>", ",", "8146.74`", ",", "8154.45`", 
     ",", "8126.94`", ",", "8141.91`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:10:00\"\>", ",", "8142.06`", ",", "8177.8`", 
     ",", "8140.7`", ",", "8166.92`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:20:00\"\>", ",", "8166.39`", ",", "8169.79`", 
     ",", "8154.53`", ",", "8162.09`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:30:00\"\>", ",", "8162.54`", ",", "8163.37`", 
     ",", "8141.23`", ",", "8141.23`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:40:00\"\>", ",", "8140.55`", ",", "8141.23`", 
     ",", "8126.72`", ",", "8136.62`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T10:50:00\"\>", ",", "8136.09`", ",", "8143.12`", 
     ",", "8129.06`", ",", "8137.22`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:00:00\"\>", ",", "8136.99`", ",", "8136.99`", 
     ",", "8116.21`", ",", "8128.`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:10:00\"\>", ",", "8127.09`", ",", "8127.09`", 
     ",", "8106.69`", ",", "8110.09`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:20:00\"\>", ",", "8109.79`", ",", "8110.69`", 
     ",", "8095.28`", ",", "8103.06`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:30:00\"\>", ",", "8103.29`", ",", "8109.79`", 
     ",", "8093.84`", ",", "8099.66`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:40:00\"\>", ",", "8099.43`", ",", "8107.82`", 
     ",", "8095.2`", ",", "8106.46`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T11:50:00\"\>", ",", "8106.54`", ",", "8115.`", 
     ",", "8105.25`", ",", "8108.5`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:00:00\"\>", ",", "8108.28`", ",", "8122.33`", 
     ",", "8106.84`", ",", "8117.95`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:10:00\"\>", ",", "8117.87`", ",", "8126.11`", 
     ",", "8117.5`", ",", "8125.66`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:20:00\"\>", ",", "8125.81`", ",", "8126.49`", 
     ",", "8118.48`", ",", "8121.05`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:30:00\"\>", ",", "8120.97`", ",", "8128.91`", 
     ",", "8113.42`", ",", "8128.91`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:40:00\"\>", ",", "8129.13`", ",", "8143.57`", 
     ",", "8128.45`", ",", "8137.6`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T12:50:00\"\>", ",", "8137.3`", ",", "8137.3`", 
     ",", "8126.34`", ",", "8131.33`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:00:00\"\>", ",", "8131.18`", ",", "8137.67`", 
     ",", "8131.18`", ",", "8133.52`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:10:00\"\>", ",", "8133.59`", ",", "8142.59`", 
     ",", "8129.06`", ",", "8142.51`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:20:00\"\>", ",", "8142.59`", ",", "8142.59`", 
     ",", "8120.97`", ",", "8121.73`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:30:00\"\>", ",", "8121.88`", ",", "8131.18`", 
     ",", "8121.88`", ",", "8130.04`", ",", "212", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:40:00\"\>", ",", "8130.19`", ",", "8148.78`", 
     ",", "8130.12`", ",", "8137.07`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T13:50:00\"\>", ",", "8136.99`", ",", "8141.23`", 
     ",", "8136.09`", ",", "8138.96`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:00:00\"\>", ",", "8139.26`", ",", "8139.56`", 
     ",", "8128.08`", ",", "8128.08`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:10:00\"\>", ",", "8127.93`", ",", "8133.67`", 
     ",", "8120.97`", ",", "8133.67`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:20:00\"\>", ",", "8133.82`", ",", "8133.82`", 
     ",", "8110.47`", ",", "8120.14`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:30:00\"\>", ",", "8120.44`", ",", "8125.89`", 
     ",", "8116.97`", ",", "8123.24`", ",", "215", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:40:00\"\>", ",", "8123.32`", ",", "8128.38`", 
     ",", "8115.83`", ",", "8127.85`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T14:50:00\"\>", ",", "8127.77`", ",", "8131.02`", 
     ",", "8114.62`", ",", "8115.61`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:00:00\"\>", ",", "8115.53`", ",", "8125.51`", 
     ",", "8111.22`", ",", "8115.`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:10:00\"\>", ",", "8114.62`", ",", "8138.51`", 
     ",", "8114.55`", ",", "8136.99`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:20:00\"\>", ",", "8137.52`", ",", "8137.98`", 
     ",", "8119.84`", ",", "8124.22`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:30:00\"\>", ",", "8124.37`", ",", "8136.92`", 
     ",", "8119.54`", ",", "8131.78`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:40:00\"\>", ",", "8131.7`", ",", "8161.56`", 
     ",", "8126.41`", ",", "8156.34`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T15:50:00\"\>", ",", "8156.57`", ",", "8158.46`", 
     ",", "8140.02`", ",", "8141.15`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-10T16:00:00\"\>", ",", "8141.3`", ",", "8146.52`", 
     ",", "8141.3`", ",", "8146.52`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T09:30:00\"\>", ",", "8146.82`", ",", "8195.26`", 
     ",", "8146.82`", ",", "8178.41`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T09:40:00\"\>", ",", "8178.26`", ",", "8180.45`", 
     ",", "8164.5`", ",", "8175.16`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T09:50:00\"\>", ",", "8174.63`", ",", "8174.63`", 
     ",", "8155.21`", ",", "8157.1`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:00:00\"\>", ",", "8156.95`", ",", "8161.56`", 
     ",", "8130.8`", ",", "8135.26`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:10:00\"\>", ",", "8135.41`", ",", "8150.67`", 
     ",", "8131.25`", ",", "8148.56`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:20:00\"\>", ",", "8149.54`", ",", "8165.64`", 
     ",", "8140.55`", ",", "8165.64`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:30:00\"\>", ",", "8166.62`", ",", "8198.74`", 
     ",", "8166.62`", ",", "8190.58`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:40:00\"\>", ",", "8191.64`", ",", "8196.85`", 
     ",", "8183.32`", ",", "8191.86`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T10:50:00\"\>", ",", "8191.64`", ",", "8224.66`", 
     ",", "8188.54`", ",", "8224.21`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:00:00\"\>", ",", "8227.38`", ",", "8230.25`", 
     ",", "8215.37`", ",", "8215.52`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:10:00\"\>", ",", "8215.44`", ",", "8222.85`", 
     ",", "8208.64`", ",", "8212.34`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:20:00\"\>", ",", "8212.64`", ",", "8224.96`", 
     ",", "8212.34`", ",", "8222.7`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:30:00\"\>", ",", "8222.39`", ",", "8281.19`", 
     ",", "8222.39`", ",", "8269.86`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:40:00\"\>", ",", "8270.01`", ",", "8286.1`", 
     ",", "8267.44`", ",", "8277.11`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T11:50:00\"\>", ",", "8277.79`", ",", "8283.61`", 
     ",", "8270.23`", ",", "8270.23`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:00:00\"\>", ",", "8270.76`", ",", "8279.`", 
     ",", "8263.58`", ",", "8264.19`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:10:00\"\>", ",", "8264.41`", ",", "8266.3`", 
     ",", "8261.39`", ",", "8263.36`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:20:00\"\>", ",", "8263.58`", ",", "8267.13`", 
     ",", "8258.59`", ",", "8267.13`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:30:00\"\>", ",", "8267.06`", ",", "8270.01`", 
     ",", "8260.33`", ",", "8264.56`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:40:00\"\>", ",", "8264.49`", ",", "8275.3`", 
     ",", "8263.96`", ",", "8271.14`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T12:50:00\"\>", ",", "8271.44`", ",", "8273.1`", 
     ",", "8262.52`", ",", "8262.83`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:00:00\"\>", ",", "8262.45`", ",", "8283.16`", 
     ",", "8260.79`", ",", "8282.78`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:10:00\"\>", ",", "8280.36`", ",", "8291.55`", 
     ",", "8275.6`", ",", "8288.22`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:20:00\"\>", ",", "8288.3`", ",", "8295.63`", 
     ",", "8287.99`", ",", "8294.27`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:30:00\"\>", ",", "8295.32`", ",", "8295.7`", 
     ",", "8284.74`", ",", "8288.14`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:40:00\"\>", ",", "8287.99`", ",", "8292.3`", 
     ",", "8283.53`", ",", "8285.5`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T13:50:00\"\>", ",", "8285.2`", ",", "8287.84`", 
     ",", "8278.24`", ",", "8283.08`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:00:00\"\>", ",", "8284.14`", ",", "8287.92`", 
     ",", "8278.47`", ",", "8280.36`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:10:00\"\>", ",", "8281.34`", ",", "8292.`", 
     ",", "8281.34`", ",", "8286.56`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:20:00\"\>", ",", "8286.63`", ",", "8304.47`", 
     ",", "8286.41`", ",", "8294.72`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:30:00\"\>", ",", "8296.53`", ",", "8299.78`", 
     ",", "8292.23`", ",", "8292.53`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:40:00\"\>", ",", "8292.68`", ",", "8293.21`", 
     ",", "8287.16`", ",", "8292.`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T14:50:00\"\>", ",", "8292.07`", ",", "8301.9`", 
     ",", "8284.82`", ",", "8291.92`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:00:00\"\>", ",", "8291.77`", ",", "8302.5`", 
     ",", "8290.03`", ",", "8297.74`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:10:00\"\>", ",", "8297.89`", ",", "8306.66`", 
     ",", "8288.9`", ",", "8296.46`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:20:00\"\>", ",", "8296.38`", ",", "8296.84`", 
     ",", "8286.71`", ",", "8290.49`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:30:00\"\>", ",", "8290.34`", ",", "8313.69`", 
     ",", "8287.54`", ",", "8311.19`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:40:00\"\>", ",", "8312.48`", ",", "8326.61`", 
     ",", "8312.48`", ",", "8314.82`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T15:50:00\"\>", ",", "8314.75`", ",", "8330.09`", 
     ",", "8312.1`", ",", "8328.43`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-13T16:00:00\"\>", ",", "8330.16`", ",", "8331.68`", 
     ",", "8329.63`", ",", "8331.68`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T09:30:00\"\>", ",", "8331.37`", ",", "8360.92`", 
     ",", "8322.23`", ",", "8346.34`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T09:40:00\"\>", ",", "8346.94`", ",", "8352.99`", 
     ",", "8309.46`", ",", "8319.21`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T09:50:00\"\>", ",", "8319.28`", ",", "8329.86`", 
     ",", "8302.05`", ",", "8309.83`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:00:00\"\>", ",", "8309.53`", ",", "8346.49`", 
     ",", "8308.78`", ",", "8336.36`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:10:00\"\>", ",", "8335.61`", ",", "8340.37`", 
     ",", "8321.02`", ",", "8325.86`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:20:00\"\>", ",", "8325.78`", ",", "8326.76`", 
     ",", "8309.46`", ",", "8324.72`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:30:00\"\>", ",", "8324.65`", ",", "8325.7`", 
     ",", "8294.64`", ",", "8298.72`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:40:00\"\>", ",", "8297.89`", ",", "8303.41`", 
     ",", "8285.2`", ",", "8293.21`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T10:50:00\"\>", ",", "8294.19`", ",", "8305.83`", 
     ",", "8294.19`", ",", "8301.67`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:00:00\"\>", ",", "8301.97`", ",", "8322.83`", 
     ",", "8301.97`", ",", "8319.05`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:10:00\"\>", ",", "8319.28`", ",", "8334.17`", 
     ",", "8317.77`", ",", "8327.29`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:20:00\"\>", ",", "8327.59`", ",", "8343.01`", 
     ",", "8324.87`", ",", "8337.27`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:30:00\"\>", ",", "8337.65`", ",", "8352.31`", 
     ",", "8328.95`", ",", "8348.68`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:40:00\"\>", ",", "8348.91`", ",", "8351.02`", 
     ",", "8343.54`", ",", "8348.3`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T11:50:00\"\>", ",", "8348.6`", ",", "8359.71`", 
     ",", "8348.3`", ",", "8354.5`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:00:00\"\>", ",", "8354.65`", ",", "8359.49`", 
     ",", "8348.38`", ",", "8354.95`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:10:00\"\>", ",", "8355.03`", ",", "8355.03`", 
     ",", "8339.61`", ",", "8340.67`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:20:00\"\>", ",", "8340.74`", ",", "8341.35`", 
     ",", "8324.42`", ",", "8328.73`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:30:00\"\>", ",", "8328.88`", ",", "8335.68`", 
     ",", "8322.68`", ",", "8329.48`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:40:00\"\>", ",", "8329.56`", ",", "8337.12`", 
     ",", "8322.83`", ",", "8335.23`", ",", "217", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T12:50:00\"\>", ",", "8335.`", ",", "8340.37`", 
     ",", "8329.33`", ",", "8330.84`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:00:00\"\>", ",", "8330.92`", ",", "8342.48`", 
     ",", "8330.92`", ",", "8333.41`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:10:00\"\>", ",", "8333.26`", ",", "8337.8`", 
     ",", "8323.06`", ",", "8323.14`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:20:00\"\>", ",", "8324.04`", ",", "8333.56`", 
     ",", "8323.51`", ",", "8332.96`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:30:00\"\>", ",", "8332.81`", ",", "8338.4`", 
     ",", "8328.8`", ",", "8331.9`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:40:00\"\>", ",", "8331.83`", ",", "8340.22`", 
     ",", "8329.03`", ",", "8338.48`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T13:50:00\"\>", ",", "8338.25`", ",", "8338.55`", 
     ",", "8324.5`", ",", "8329.41`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:00:00\"\>", ",", "8329.48`", ",", "8329.48`", 
     ",", "8310.36`", ",", "8320.94`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:10:00\"\>", ",", "8321.09`", ",", "8335.3`", 
     ",", "8316.94`", ",", "8334.24`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:20:00\"\>", ",", "8334.32`", ",", "8340.37`", 
     ",", "8327.07`", ",", "8327.75`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:30:00\"\>", ",", "8327.52`", ",", "8347.92`", 
     ",", "8322.23`", ",", "8343.99`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:40:00\"\>", ",", "8343.69`", ",", "8358.58`", 
     ",", "8343.47`", ",", "8346.64`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T14:50:00\"\>", ",", "8347.02`", ",", "8349.59`", 
     ",", "8339.54`", ",", "8348.45`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:00:00\"\>", ",", "8343.69`", ",", "8356.16`", 
     ",", "8343.69`", ",", "8346.41`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:10:00\"\>", ",", "8346.49`", ",", "8357.07`", 
     ",", "8346.49`", ",", "8350.95`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:20:00\"\>", ",", "8349.89`", ",", "8356.01`", 
     ",", "8345.35`", ",", "8349.89`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:30:00\"\>", ",", "8348.98`", ",", "8354.2`", 
     ",", "8346.11`", ",", "8353.14`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:40:00\"\>", ",", "8352.84`", ",", "8356.84`", 
     ",", "8338.48`", ",", "8348.38`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T15:50:00\"\>", ",", "8348.53`", ",", "8360.02`", 
     ",", "8347.55`", ",", "8360.02`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-14T16:00:00\"\>", ",", "8360.24`", ",", "8360.24`", 
     ",", "8359.11`", ",", "8359.49`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T09:30:00\"\>", ",", "8364.02`", ",", "8465.67`", 
     ",", "8364.02`", ",", "8465.67`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T09:40:00\"\>", ",", "8467.11`", ",", "8491.29`", 
     ",", "8467.11`", ",", "8487.21`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T09:50:00\"\>", ",", "8487.28`", ",", "8499.3`", 
     ",", "8478.59`", ",", "8493.1`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:00:00\"\>", ",", "8492.88`", ",", "8503.68`", 
     ",", "8490.84`", ",", "8502.1`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:10:00\"\>", ",", "8502.32`", ",", "8509.2`", 
     ",", "8496.66`", ",", "8505.88`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:20:00\"\>", ",", "8505.95`", ",", "8519.18`", 
     ",", "8503.`", ",", "8516.53`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:30:00\"\>", ",", "8517.67`", ",", "8520.54`", 
     ",", "8512.98`", ",", "8515.4`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:40:00\"\>", ",", "8515.25`", ",", "8515.47`", 
     ",", "8506.18`", ",", "8506.18`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T10:50:00\"\>", ",", "8506.1`", ",", "8509.5`", 
     ",", "8503.83`", ",", "8506.63`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:00:00\"\>", ",", "8506.86`", ",", "8517.67`", 
     ",", "8506.86`", ",", "8515.62`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:10:00\"\>", ",", "8515.47`", ",", "8531.72`", 
     ",", "8514.94`", ",", "8530.06`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:20:00\"\>", ",", "8530.21`", ",", "8542.98`", 
     ",", "8530.14`", ",", "8539.66`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:30:00\"\>", ",", "8539.2`", ",", "8554.39`", 
     ",", "8538.3`", ",", "8550.77`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:40:00\"\>", ",", "8550.99`", ",", "8552.51`", 
     ",", "8539.43`", ",", "8540.79`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T11:50:00\"\>", ",", "8540.56`", ",", "8544.42`", 
     ",", "8540.26`", ",", "8540.26`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:00:00\"\>", ",", "8540.41`", ",", "8541.62`", 
     ",", "8531.95`", ",", "8535.05`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:10:00\"\>", ",", "8534.9`", ",", "8546.99`", 
     ",", "8532.1`", ",", "8544.8`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:20:00\"\>", ",", "8545.1`", ",", "8547.29`", 
     ",", "8542.08`", ",", "8544.65`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:30:00\"\>", ",", "8544.42`", ",", "8554.17`", 
     ",", "8542.91`", ",", "8549.18`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:40:00\"\>", ",", "8548.95`", ",", "8549.18`", 
     ",", "8546.08`", ",", "8548.42`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T12:50:00\"\>", ",", "8548.65`", ",", "8552.88`", 
     ",", "8541.85`", ",", "8544.8`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:00:00\"\>", ",", "8545.02`", ",", "8552.58`", 
     ",", "8545.02`", ",", "8549.56`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:10:00\"\>", ",", "8549.71`", ",", "8550.77`", 
     ",", "8541.32`", ",", "8542.98`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:20:00\"\>", ",", "8542.83`", ",", "8553.94`", 
     ",", "8542.83`", ",", "8553.79`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:30:00\"\>", ",", "8553.94`", ",", "8560.29`", 
     ",", "8552.05`", ",", "8559.46`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:40:00\"\>", ",", "8560.59`", ",", "8565.13`", 
     ",", "8556.59`", ",", "8564.29`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T13:50:00\"\>", ",", "8563.99`", ",", "8568.07`", 
     ",", "8561.5`", ",", "8565.5`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:00:00\"\>", ",", "8565.28`", ",", "8579.03`", 
     ",", "8555.6`", ",", "8570.04`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:10:00\"\>", ",", "8570.19`", ",", "8574.72`", 
     ",", "8562.93`", ",", "8569.96`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:20:00\"\>", ",", "8570.11`", ",", "8588.33`", 
     ",", "8567.47`", ",", "8579.33`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:30:00\"\>", ",", "8579.79`", ",", "8590.82`", 
     ",", "8579.79`", ",", "8584.47`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:40:00\"\>", ",", "8584.78`", ",", "8601.86`", 
     ",", "8582.21`", ",", "8600.72`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T14:50:00\"\>", ",", "8600.27`", ",", "8608.43`", 
     ",", "8594.45`", ",", "8607.9`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:00:00\"\>", ",", "8608.05`", ",", "8613.49`", 
     ",", "8601.78`", ",", "8612.97`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:10:00\"\>", ",", "8613.04`", ",", "8624.45`", 
     ",", "8607.6`", ",", "8613.34`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:20:00\"\>", ",", "8613.27`", ",", "8617.58`", 
     ",", "8598.68`", ",", "8598.68`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:30:00\"\>", ",", "8598.98`", ",", "8605.11`", 
     ",", "8590.75`", ",", "8604.65`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:40:00\"\>", ",", "8605.26`", ",", "8614.02`", 
     ",", "8600.04`", ",", "8610.92`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T15:50:00\"\>", ",", "8611.`", ",", "8628.23`", 
     ",", "8611.`", ",", "8618.03`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-15T16:00:00\"\>", ",", "8617.88`", ",", "8617.88`", 
     ",", "8616.21`", ",", "8616.21`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T09:30:00\"\>", ",", "8612.66`", ",", "8615.84`", 
     ",", "8585.15`", ",", "8613.8`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T09:40:00\"\>", ",", "8614.02`", ",", "8637.6`", 
     ",", "8614.02`", ",", "8635.86`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T09:50:00\"\>", ",", "8637.15`", ",", "8654.38`", 
     ",", "8637.15`", ",", "8644.03`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:00:00\"\>", ",", "8637.9`", ",", "8637.9`", 
     ",", "8596.94`", ",", "8606.31`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:10:00\"\>", ",", "8606.24`", ",", "8622.26`", 
     ",", "8603.67`", ",", "8610.55`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:20:00\"\>", ",", "8610.24`", ",", "8630.27`", 
     ",", "8606.77`", ",", "8625.13`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:30:00\"\>", ",", "8625.36`", ",", "8639.34`", 
     ",", "8625.36`", ",", "8633.67`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:40:00\"\>", ",", "8633.6`", ",", "8640.25`", 
     ",", "8629.97`", ",", "8632.09`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T10:50:00\"\>", ",", "8632.31`", ",", "8639.34`", 
     ",", "8622.56`", ",", "8624.15`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:00:00\"\>", ",", "8623.39`", ",", "8631.25`", 
     ",", "8619.99`", ",", "8621.51`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:10:00\"\>", ",", "8620.82`", ",", "8621.2`", 
     ",", "8604.73`", ",", "8607.52`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:20:00\"\>", ",", "8607.22`", ",", "8612.66`", 
     ",", "8605.48`", ",", "8607.98`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:30:00\"\>", ",", "8606.47`", ",", "8613.65`", 
     ",", "8600.8`", ",", "8613.49`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:40:00\"\>", ",", "8613.42`", ",", "8619.39`", 
     ",", "8609.26`", ",", "8612.21`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T11:50:00\"\>", ",", "8611.91`", ",", "8628.76`", 
     ",", "8610.4`", ",", "8626.95`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:00:00\"\>", ",", "8627.02`", ",", "8630.42`", 
     ",", "8624.23`", ",", "8624.53`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:10:00\"\>", ",", "8624.6`", ",", "8629.89`", 
     ",", "8619.84`", ",", "8621.05`", ",", "216", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:20:00\"\>", ",", "8621.28`", ",", "8630.5`", 
     ",", "8620.98`", ",", "8630.35`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:30:00\"\>", ",", "8629.67`", ",", "8630.27`", 
     ",", "8619.01`", ",", "8620.67`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:40:00\"\>", ",", "8620.75`", ",", "8627.17`", 
     ",", "8620.45`", ",", "8624.75`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T12:50:00\"\>", ",", "8624.68`", ",", "8629.67`", 
     ",", "8613.87`", ",", "8613.95`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:00:00\"\>", ",", "8614.48`", ",", "8620.67`", 
     ",", "8605.33`", ",", "8606.01`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:10:00\"\>", ",", "8605.94`", ",", "8613.04`", 
     ",", "8604.05`", ",", "8611.`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:20:00\"\>", ",", "8611.15`", ",", "8614.7`", 
     ",", "8607.9`", ",", "8610.32`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:30:00\"\>", ",", "8610.62`", ",", "8613.12`", 
     ",", "8606.47`", ",", "8611.`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:40:00\"\>", ",", "8611.38`", ",", "8631.1`", 
     ",", "8611.38`", ",", "8629.36`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T13:50:00\"\>", ",", "8629.97`", ",", "8644.63`", 
     ",", "8627.78`", ",", "8644.63`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:00:00\"\>", ",", "8644.93`", ",", "8649.77`", 
     ",", "8640.55`", ",", "8646.52`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:10:00\"\>", ",", "8646.67`", ",", "8658.46`", 
     ",", "8641.91`", ",", "8654.83`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:20:00\"\>", ",", "8654.76`", ",", "8673.88`", 
     ",", "8653.55`", ",", "8669.72`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:30:00\"\>", ",", "8669.57`", ",", "8681.96`", 
     ",", "8665.64`", ",", "8681.06`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:40:00\"\>", ",", "8683.33`", ",", "8693.23`", 
     ",", "8675.77`", ",", "8688.16`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T14:50:00\"\>", ",", "8688.46`", ",", "8697.91`", 
     ",", "8686.05`", ",", "8691.79`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:00:00\"\>", ",", "8691.49`", ",", "8692.55`", 
     ",", "8679.92`", ",", "8683.4`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:10:00\"\>", ",", "8683.25`", ",", "8699.88`", 
     ",", "8683.25`", ",", "8693.`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:20:00\"\>", ",", "8692.77`", ",", "8705.92`", 
     ",", "8687.56`", ",", "8703.28`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:30:00\"\>", ",", "8703.35`", ",", "8706.3`", 
     ",", "8698.36`", ",", "8702.6`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:40:00\"\>", ",", "8704.26`", ",", "8739.55`", 
     ",", "8704.26`", ",", "8727.76`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T15:50:00\"\>", ",", "8728.14`", ",", "8731.62`", 
     ",", "8698.44`", ",", "8709.25`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-16T16:00:00\"\>", ",", "8709.02`", ",", "8711.82`", 
     ",", "8709.02`", ",", "8711.82`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T09:30:00\"\>", ",", "8711.59`", ",", "8732.37`", 
     ",", "8688.46`", ",", "8688.46`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T09:40:00\"\>", ",", "8688.39`", ",", "8709.63`", 
     ",", "8688.24`", ",", "8689.98`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T09:50:00\"\>", ",", "8689.75`", ",", "8704.56`", 
     ",", "8674.63`", ",", "8701.31`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:00:00\"\>", ",", "8701.54`", ",", "8706.22`", 
     ",", "8684.91`", ",", "8698.82`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:10:00\"\>", ",", "8698.89`", ",", "8706.53`", 
     ",", "8691.71`", ",", "8705.47`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:20:00\"\>", ",", "8705.39`", ",", "8726.33`", 
     ",", "8704.64`", ",", "8724.36`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:30:00\"\>", ",", "8723.83`", ",", "8731.77`", 
     ",", "8719.68`", ",", "8724.51`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:40:00\"\>", ",", "8725.19`", ",", "8726.18`", 
     ",", "8711.21`", ",", "8711.21`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T10:50:00\"\>", ",", "8711.29`", ",", "8714.69`", 
     ",", "8698.52`", ",", "8702.67`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:00:00\"\>", ",", "8702.82`", ",", "8703.81`", 
     ",", "8695.27`", ",", "8700.63`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:10:00\"\>", ",", "8700.18`", ",", "8703.88`", 
     ",", "8695.42`", ",", "8698.74`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:20:00\"\>", ",", "8698.59`", ",", "8717.26`", 
     ",", "8697.61`", ",", "8716.12`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:30:00\"\>", ",", "8715.97`", ",", "8715.97`", 
     ",", "8702.14`", ",", "8704.64`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:40:00\"\>", ",", "8704.34`", ",", "8707.81`", 
     ",", "8698.59`", ",", "8707.81`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T11:50:00\"\>", ",", "8708.42`", ",", "8709.25`", 
     ",", "8697.61`", ",", "8699.27`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:00:00\"\>", ",", "8699.35`", ",", "8723.76`", 
     ",", "8698.82`", ",", "8720.43`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:10:00\"\>", ",", "8720.58`", ",", "8724.51`", 
     ",", "8712.57`", ",", "8722.1`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:20:00\"\>", ",", "8722.25`", ",", "8729.12`", 
     ",", "8709.85`", ",", "8713.56`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:30:00\"\>", ",", "8713.71`", ",", "8719.3`", 
     ",", "8707.51`", ",", "8719.22`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:40:00\"\>", ",", "8719.53`", ",", "8723.76`", 
     ",", "8715.37`", ",", "8719.53`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T12:50:00\"\>", ",", "8719.37`", ",", "8721.04`", 
     ",", "8709.32`", ",", "8715.97`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:00:00\"\>", ",", "8716.05`", ",", "8717.79`", 
     ",", "8713.86`", ",", "8715.75`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:10:00\"\>", ",", "8715.67`", ",", "8716.05`", 
     ",", "8710.23`", ",", "8715.75`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:20:00\"\>", ",", "8716.43`", ",", "8727.31`", 
     ",", "8716.43`", ",", "8726.4`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:30:00\"\>", ",", "8727.01`", ",", "8748.7`", 
     ",", "8726.55`", ",", "8742.88`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:40:00\"\>", ",", "8742.73`", ",", "8754.21`", 
     ",", "8740.01`", ",", "8749.08`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T13:50:00\"\>", ",", "8748.92`", ",", "8751.8`", 
     ",", "8737.74`", ",", "8737.74`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:00:00\"\>", ",", "8737.44`", ",", "8740.54`", 
     ",", "8734.64`", ",", "8734.72`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:10:00\"\>", ",", "8734.79`", ",", "8734.79`", 
     ",", "8724.21`", ",", "8726.33`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:20:00\"\>", ",", "8726.25`", ",", "8734.57`", 
     ",", "8725.65`", ",", "8731.39`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:30:00\"\>", ",", "8731.47`", ",", "8737.06`", 
     ",", "8729.73`", ",", "8732.68`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:40:00\"\>", ",", "8732.6`", ",", "8741.82`", 
     ",", "8730.41`", ",", "8733.2`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T14:50:00\"\>", ",", "8733.51`", ",", "8739.4`", 
     ",", "8728.37`", ",", "8734.94`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:00:00\"\>", ",", "8735.02`", ",", "8739.4`", 
     ",", "8729.27`", ",", "8730.03`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:10:00\"\>", ",", "8730.11`", ",", "8731.16`", 
     ",", "8724.29`", ",", "8727.23`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:20:00\"\>", ",", "8727.31`", ",", "8729.73`", 
     ",", "8724.14`", ",", "8725.27`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:30:00\"\>", ",", "8724.66`", ",", "8724.66`", 
     ",", "8708.57`", ",", "8711.67`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:40:00\"\>", ",", "8712.5`", ",", "8727.69`", 
     ",", "8712.5`", ",", "8726.93`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T15:50:00\"\>", ",", "8728.07`", ",", "8748.85`", 
     ",", "8728.07`", ",", "8746.35`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-17T16:00:00\"\>", ",", "8746.43`", ",", "8746.51`", 
     ",", "8743.56`", ",", "8743.94`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T09:30:00\"\>", ",", "8746.05`", ",", "8822.16`", 
     ",", "8746.05`", ",", "8817.7`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T09:40:00\"\>", ",", "8819.66`", ",", "8821.4`", 
     ",", "8783.99`", ",", "8797.59`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T09:50:00\"\>", ",", "8796.08`", ",", "8806.97`", 
     ",", "8794.95`", ",", "8801.68`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:00:00\"\>", ",", "8801.83`", ",", "8826.92`", 
     ",", "8794.72`", ",", "8794.72`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:10:00\"\>", ",", "8795.02`", ",", "8806.21`", 
     ",", "8784.75`", ",", "8803.64`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:20:00\"\>", ",", "8804.32`", ",", "8804.32`", 
     ",", "8786.48`", ",", "8795.1`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:30:00\"\>", ",", "8795.18`", ",", "8797.59`", 
     ",", "8767.29`", ",", "8772.13`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:40:00\"\>", ",", "8771.75`", ",", "8773.94`", 
     ",", "8754.97`", ",", "8756.1`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T10:50:00\"\>", ",", "8755.95`", ",", "8773.26`", 
     ",", "8755.95`", ",", "8768.57`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:00:00\"\>", ",", "8768.65`", ",", "8772.28`", 
     ",", "8761.77`", ",", "8768.12`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:10:00\"\>", ",", "8768.2`", ",", "8784.44`", 
     ",", "8768.04`", ",", "8779.83`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:20:00\"\>", ",", "8779.91`", ",", "8791.4`", 
     ",", "8777.79`", ",", "8790.41`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:30:00\"\>", ",", "8790.57`", ",", "8797.07`", 
     ",", "8787.54`", ",", "8787.77`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:40:00\"\>", ",", "8787.69`", ",", "8792.68`", 
     ",", "8785.05`", ",", "8785.12`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T11:50:00\"\>", ",", "8785.28`", ",", "8788.`", 
     ",", "8781.72`", ",", "8786.33`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:00:00\"\>", ",", "8786.18`", ",", "8804.62`", 
     ",", "8785.58`", ",", "8801.68`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:10:00\"\>", ",", "8801.9`", ",", "8808.25`", 
     ",", "8801.45`", ",", "8807.57`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:20:00\"\>", ",", "8807.65`", ",", "8810.82`", 
     ",", "8805.76`", ",", "8810.82`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:30:00\"\>", ",", "8811.05`", ",", "8815.81`", 
     ",", "8809.84`", ",", "8815.73`", ",", "218", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:40:00\"\>", ",", "8815.96`", ",", "8818.68`", 
     ",", "8806.36`", ",", "8806.59`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T12:50:00\"\>", ",", "8805.98`", ",", "8809.46`", 
     ",", "8802.58`", ",", "8803.04`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:00:00\"\>", ",", "8802.96`", ",", "8808.78`", 
     ",", "8802.58`", ",", "8804.02`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:10:00\"\>", ",", "8803.94`", ",", "8803.94`", 
     ",", "8790.04`", ",", "8793.97`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:20:00\"\>", ",", "8793.21`", ",", "8795.63`", 
     ",", "8788.68`", ",", "8789.21`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:30:00\"\>", ",", "8789.66`", ",", "8796.84`", 
     ",", "8781.8`", ",", "8785.12`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:40:00\"\>", ",", "8784.9`", ",", "8802.2`", 
     ",", "8784.9`", ",", "8799.94`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T13:50:00\"\>", ",", "8799.71`", ",", "8803.41`", 
     ",", "8794.34`", ",", "8800.62`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:00:00\"\>", ",", "8800.47`", ",", "8805.68`", 
     ",", "8790.87`", ",", "8801.83`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:10:00\"\>", ",", "8801.68`", ",", "8821.32`", 
     ",", "8801.22`", ",", "8818.38`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:20:00\"\>", ",", "8818.53`", ",", "8818.68`", 
     ",", "8793.51`", ",", "8800.47`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:30:00\"\>", ",", "8800.32`", ",", "8815.88`", 
     ",", "8800.32`", ",", "8806.36`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:40:00\"\>", ",", "8805.53`", ",", "8817.17`", 
     ",", "8801.68`", ",", "8815.58`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T14:50:00\"\>", ",", "8816.03`", ",", "8818.83`", 
     ",", "8809.46`", ",", "8810.06`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:00:00\"\>", ",", "8810.22`", ",", "8844.38`", 
     ",", "8809.84`", ",", "8841.88`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:10:00\"\>", ",", "8844.83`", ",", "8847.47`", 
     ",", "8837.04`", ",", "8838.4`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:20:00\"\>", ",", "8838.33`", ",", "8846.11`", 
     ",", "8835.46`", ",", "8839.84`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:30:00\"\>", ",", "8840.9`", ",", "8844.83`", 
     ",", "8829.11`", ",", "8844.83`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:40:00\"\>", ",", "8844.68`", ",", "8852.92`", 
     ",", "8834.85`", ",", "8842.03`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T15:50:00\"\>", ",", "8841.81`", ",", "8854.65`", 
     ",", "8836.97`", ",", "8843.92`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-20T16:00:00\"\>", ",", "8843.24`", ",", "8848.38`", 
     ",", "8843.24`", ",", "8848.15`", ",", "21", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T09:30:00\"\>", ",", "8848.`", ",", "8924.03`", 
     ",", "8847.7`", ",", "8924.03`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T09:40:00\"\>", ",", "8923.12`", ",", "8923.12`", 
     ",", "8889.04`", ",", "8896.45`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T09:50:00\"\>", ",", "8897.5`", ",", "8912.09`", 
     ",", "8885.64`", ",", "8907.71`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:00:00\"\>", ",", "8907.1`", ",", "8916.02`", 
     ",", "8877.1`", ",", "8881.18`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:10:00\"\>", ",", "8880.8`", ",", "8896.98`", 
     ",", "8868.11`", ",", "8896.98`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:20:00\"\>", ",", "8897.28`", ",", "8904.46`", 
     ",", "8890.17`", ",", "8901.21`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:30:00\"\>", ",", "8902.49`", ",", "8926.53`", 
     ",", "8899.47`", ",", "8919.95`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:40:00\"\>", ",", "8919.8`", ",", "8919.8`", 
     ",", "8904.84`", ",", "8906.12`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T10:50:00\"\>", ",", "8904.68`", ",", "8916.78`", 
     ",", "8898.03`", ",", "8912.09`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:00:00\"\>", ",", "8914.28`", ",", "8917.08`", 
     ",", "8901.06`", ",", "8912.62`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:10:00\"\>", ",", "8912.77`", ",", "8913.75`", 
     ",", "8889.64`", ",", "8898.64`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:20:00\"\>", ",", "8898.18`", ",", "8903.32`", 
     ",", "8888.51`", ",", "8890.55`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:30:00\"\>", ",", "8890.48`", ",", "8891.91`", 
     ",", "8874.45`", ",", "8889.64`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:40:00\"\>", ",", "8888.74`", ",", "8889.04`", 
     ",", "8856.92`", ",", "8857.3`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T11:50:00\"\>", ",", "8857.6`", ",", "8863.87`", 
     ",", "8832.59`", ",", "8837.27`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:00:00\"\>", ",", "8836.44`", ",", "8847.93`", 
     ",", "8833.72`", ",", "8846.04`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:10:00\"\>", ",", "8846.26`", ",", "8853.14`", 
     ",", "8845.06`", ",", "8850.27`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:20:00\"\>", ",", "8850.19`", ",", "8851.48`", 
     ",", "8835.99`", ",", "8850.12`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:30:00\"\>", ",", "8850.04`", ",", "8855.56`", 
     ",", "8836.89`", ",", "8854.65`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:40:00\"\>", ",", "8854.2`", ",", "8854.2`", 
     ",", "8841.05`", ",", "8844.3`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T12:50:00\"\>", ",", "8844.15`", ",", "8844.53`", 
     ",", "8826.24`", ",", "8833.72`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:00:00\"\>", ",", "8832.66`", ",", "8832.66`", 
     ",", "8811.42`", ",", "8815.05`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:10:00\"\>", ",", "8815.13`", ",", "8832.28`", 
     ",", "8815.05`", ",", "8832.13`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:20:00\"\>", ",", "8832.59`", ",", "8843.62`", 
     ",", "8828.35`", ",", "8841.43`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:30:00\"\>", ",", "8841.73`", ",", "8845.43`", 
     ",", "8833.72`", ",", "8839.24`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:40:00\"\>", ",", "8839.16`", ",", "8850.19`", 
     ",", "8837.95`", ",", "8846.34`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T13:50:00\"\>", ",", "8846.42`", ",", "8846.49`", 
     ",", "8834.1`", ",", "8840.52`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:00:00\"\>", ",", "8840.6`", ",", "8848.23`", 
     ",", "8839.69`", ",", "8844.98`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:10:00\"\>", ",", "8844.83`", ",", "8863.87`", 
     ",", "8844.83`", ",", "8852.16`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:20:00\"\>", ",", "8852.46`", ",", "8854.65`", 
     ",", "8844.75`", ",", "8852.31`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:30:00\"\>", ",", "8851.71`", ",", "8862.97`", 
     ",", "8837.72`", ",", "8858.58`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:40:00\"\>", ",", "8859.04`", ",", "8865.76`", 
     ",", "8850.95`", ",", "8865.39`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T14:50:00\"\>", ",", "8864.93`", ",", "8881.33`", 
     ",", "8864.86`", ",", "8881.26`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:00:00\"\>", ",", "8882.69`", ",", "8884.28`", 
     ",", "8873.85`", ",", "8878.91`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:10:00\"\>", ",", "8879.06`", ",", "8891.53`", 
     ",", "8878.69`", ",", "8889.95`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:20:00\"\>", ",", "8890.02`", ",", "8890.25`", 
     ",", "8881.1`", ",", "8888.28`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:30:00\"\>", ",", "8888.13`", ",", "8891.08`", 
     ",", "8879.82`", ",", "8887.38`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:40:00\"\>", ",", "8887.83`", ",", "8899.02`", 
     ",", "8885.56`", ",", "8885.94`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T15:50:00\"\>", ",", "8885.71`", ",", "8916.1`", 
     ",", "8885.71`", ",", "8916.1`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-21T16:00:00\"\>", ",", "8916.17`", ",", "8916.55`", 
     ",", "8915.11`", ",", "8915.94`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T09:30:00\"\>", ",", "8912.39`", ",", "8912.39`", 
     ",", "8860.85`", ",", "8909.6`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T09:40:00\"\>", ",", "8910.58`", ",", "8917.53`", 
     ",", "8900.45`", ",", "8914.06`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T09:50:00\"\>", ",", "8914.28`", ",", "8933.48`", 
     ",", "8911.41`", ",", "8924.94`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:00:00\"\>", ",", "8925.39`", ",", "8949.58`", 
     ",", "8925.39`", ",", "8937.86`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:10:00\"\>", ",", "8937.63`", ",", "8939.07`", 
     ",", "8912.32`", ",", "8912.32`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:20:00\"\>", ",", "8912.17`", ",", "8929.17`", 
     ",", "8907.78`", ",", "8912.54`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:30:00\"\>", ",", "8914.58`", ",", "8916.1`", 
     ",", "8901.74`", ",", "8902.49`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:40:00\"\>", ",", "8902.27`", ",", "8908.61`", 
     ",", "8894.33`", ",", "8905.89`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T10:50:00\"\>", ",", "8906.2`", ",", "8924.86`", 
     ",", "8905.21`", ",", "8923.58`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:00:00\"\>", ",", "8923.05`", ",", "8935.14`", 
     ",", "8916.7`", ",", "8932.72`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:10:00\"\>", ",", "8933.02`", ",", "8936.95`", 
     ",", "8925.77`", ",", "8928.26`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:20:00\"\>", ",", "8928.87`", ",", "8929.7`", 
     ",", "8912.85`", ",", "8916.78`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:30:00\"\>", ",", "8916.47`", ",", "8925.47`", 
     ",", "8908.84`", ",", "8910.35`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:40:00\"\>", ",", "8910.43`", ",", "8913.6`", 
     ",", "8896.67`", ",", "8896.82`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T11:50:00\"\>", ",", "8896.9`", ",", "8908.01`", 
     ",", "8896.82`", ",", "8906.27`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:00:00\"\>", ",", "8906.95`", ",", "8916.78`", 
     ",", "8906.95`", ",", "8912.09`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:10:00\"\>", ",", "8912.17`", ",", "8913.07`", 
     ",", "8900.23`", ",", "8903.4`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:20:00\"\>", ",", "8903.63`", ",", "8941.49`", 
     ",", "8903.4`", ",", "8937.03`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:30:00\"\>", ",", "8937.33`", ",", "8945.87`", 
     ",", "8932.19`", ",", "8934.31`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:40:00\"\>", ",", "8934.23`", ",", "8947.69`", 
     ",", "8927.51`", ",", "8943.76`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T12:50:00\"\>", ",", "8943.98`", ",", "8949.65`", 
     ",", "8928.41`", ",", "8930.23`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:00:00\"\>", ",", "8930.46`", ",", "8932.72`", 
     ",", "8924.56`", ",", "8931.36`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:10:00\"\>", ",", "8928.79`", ",", "8939.75`", 
     ",", "8927.13`", ",", "8932.8`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:20:00\"\>", ",", "8932.72`", ",", "8932.72`", 
     ",", "8906.8`", ",", "8913.83`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:30:00\"\>", ",", "8913.75`", ",", "8915.34`", 
     ",", "8901.74`", ",", "8912.54`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:40:00\"\>", ",", "8912.62`", ",", "8915.11`", 
     ",", "8898.64`", ",", "8908.54`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T13:50:00\"\>", ",", "8908.61`", ",", "8915.87`", 
     ",", "8904.84`", ",", "8915.34`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:00:00\"\>", ",", "8915.72`", ",", "8921.92`", 
     ",", "8892.67`", ",", "8895.92`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:10:00\"\>", ",", "8895.99`", ",", "8906.8`", 
     ",", "8895.92`", ",", "8900.53`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:20:00\"\>", ",", "8900.45`", ",", "8900.53`", 
     ",", "8886.39`", ",", "8893.2`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:30:00\"\>", ",", "8893.57`", ",", "8904.76`", 
     ",", "8891.53`", ",", "8898.49`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:40:00\"\>", ",", "8898.56`", ",", "8908.69`", 
     ",", "8893.88`", ",", "8894.63`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T14:50:00\"\>", ",", "8894.78`", ",", "8906.57`", 
     ",", "8893.05`", ",", "8904.38`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:00:00\"\>", ",", "8903.93`", ",", "8912.32`", 
     ",", "8888.59`", ",", "8889.27`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:10:00\"\>", ",", "8889.19`", ",", "8890.17`", 
     ",", "8863.57`", ",", "8871.13`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:20:00\"\>", ",", "8870.75`", ",", "8876.12`", 
     ",", "8861.`", ",", "8871.51`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:30:00\"\>", ",", "8871.66`", ",", "8881.33`", 
     ",", "8869.69`", ",", "8871.36`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:40:00\"\>", ",", "8872.94`", ",", "8890.48`", 
     ",", "8864.1`", ",", "8868.33`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T15:50:00\"\>", ",", "8868.48`", ",", "8885.03`", 
     ",", "8865.01`", ",", "8878.91`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-22T16:00:00\"\>", ",", "8879.52`", ",", "8881.41`", 
     ",", "8879.52`", ",", "8881.26`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T09:30:00\"\>", ",", "8882.31`", ",", "8901.66`", 
     ",", "8866.75`", ",", "8901.59`", ",", "296", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T09:40:00\"\>", ",", "8901.81`", ",", "8923.65`", 
     ",", "8896.22`", ",", "8918.06`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T09:50:00\"\>", ",", "8917.61`", ",", "8952.37`", 
     ",", "8906.72`", ",", "8947.08`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:00:00\"\>", ",", "8959.02`", ",", "8974.52`", 
     ",", "8940.73`", ",", "8973.31`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:10:00\"\>", ",", "8973.`", ",", "9003.31`", 
     ",", "8970.36`", ",", "8996.89`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:20:00\"\>", ",", "8998.17`", ",", "9011.47`", 
     ",", "8991.52`", ",", "8993.48`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:30:00\"\>", ",", "8995.68`", ",", "9033.01`", 
     ",", "8995.6`", ",", "9028.63`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:40:00\"\>", ",", "9028.32`", ",", "9037.17`", 
     ",", "9022.58`", ",", "9023.79`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T10:50:00\"\>", ",", "9023.87`", ",", "9027.57`", 
     ",", "9012.`", ",", "9019.86`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:00:00\"\>", ",", "9020.69`", ",", "9053.19`", 
     ",", "9019.56`", ",", "9038.45`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:10:00\"\>", ",", "9039.13`", ",", "9070.95`", 
     ",", "9039.13`", ",", "9055.61`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:20:00\"\>", ",", "9054.85`", ",", "9061.96`", 
     ",", "9054.85`", ",", "9059.91`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:30:00\"\>", ",", "9059.54`", ",", "9060.67`", 
     ",", "9045.33`", ",", "9056.21`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:40:00\"\>", ",", "9056.59`", ",", "9069.13`", 
     ",", "9053.57`", ",", "9057.57`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T11:50:00\"\>", ",", "9057.72`", ",", "9063.01`", 
     ",", "9044.27`", ",", "9044.27`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:00:00\"\>", ",", "9045.93`", ",", "9056.51`", 
     ",", "9044.57`", ",", "9055.91`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:10:00\"\>", ",", "9055.98`", ",", "9059.08`", 
     ",", "9050.85`", ",", "9058.86`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:20:00\"\>", ",", "9060.22`", ",", "9060.67`", 
     ",", "9049.33`", ",", "9059.99`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:30:00\"\>", ",", "9060.07`", ",", "9060.07`", 
     ",", "9047.67`", ",", "9050.92`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:40:00\"\>", ",", "9050.85`", ",", "9073.29`", 
     ",", "9049.26`", ",", "9070.87`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T12:50:00\"\>", ",", "9070.95`", ",", "9074.8`", 
     ",", "9056.67`", ",", "9071.4`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:00:00\"\>", ",", "9071.55`", ",", "9072.16`", 
     ",", "9055.68`", ",", "9055.76`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:10:00\"\>", ",", "9055.53`", ",", "9061.96`", 
     ",", "9053.64`", ",", "9061.58`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:20:00\"\>", ",", "9061.96`", ",", "9076.09`", 
     ",", "9061.5`", ",", "9067.09`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:30:00\"\>", ",", "9066.94`", ",", "9090.37`", 
     ",", "9064.45`", ",", "9086.37`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:40:00\"\>", ",", "9086.67`", ",", "9087.88`", 
     ",", "9073.97`", ",", "9074.12`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T13:50:00\"\>", ",", "9073.67`", ",", "9077.83`", 
     ",", "9063.54`", ",", "9063.62`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:00:00\"\>", ",", "9063.69`", ",", "9067.32`", 
     ",", "9059.69`", ",", "9059.84`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:10:00\"\>", ",", "9059.91`", ",", "9070.19`", 
     ",", "9059.46`", ",", "9070.19`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:20:00\"\>", ",", "9070.12`", ",", "9076.31`", 
     ",", "9060.67`", ",", "9067.85`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:30:00\"\>", ",", "9068.38`", ",", "9078.13`", 
     ",", "9065.51`", ",", "9068.61`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:40:00\"\>", ",", "9072.76`", ",", "9086.44`", 
     ",", "9071.93`", ",", "9084.02`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T14:50:00\"\>", ",", "9082.81`", ",", "9083.57`", 
     ",", "9073.14`", ",", "9079.41`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:00:00\"\>", ",", "9079.49`", ",", "9092.03`", 
     ",", "9079.11`", ",", "9080.7`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:10:00\"\>", ",", "9083.27`", ",", "9096.27`", 
     ",", "9077.9`", ",", "9087.65`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:20:00\"\>", ",", "9087.5`", ",", "9088.86`", 
     ",", "9080.24`", ",", "9084.4`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:30:00\"\>", ",", "9084.17`", ",", "9084.17`", 
     ",", "9071.7`", ",", "9076.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:40:00\"\>", ",", "9076.77`", ",", "9089.62`", 
     ",", "9058.71`", ",", "9060.22`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T15:50:00\"\>", ",", "9060.37`", ",", "9067.02`", 
     ",", "9048.96`", ",", "9067.02`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-23T16:00:00\"\>", ",", "9066.49`", ",", "9069.29`", 
     ",", "9066.49`", ",", "9069.29`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T09:30:00\"\>", ",", "9066.11`", ",", "9066.11`", 
     ",", "9028.78`", ",", "9035.73`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T09:40:00\"\>", ",", "9036.86`", ",", "9068.53`", 
     ",", "9036.86`", ",", "9043.82`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T09:50:00\"\>", ",", "9044.04`", ",", "9044.2`", 
     ",", "9013.97`", ",", "9017.82`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:00:00\"\>", ",", "9018.12`", ",", "9032.33`", 
     ",", "9007.54`", ",", "9017.74`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:10:00\"\>", ",", "9017.37`", ",", "9030.59`", 
     ",", "9012.38`", ",", "9023.71`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:20:00\"\>", ",", "9023.49`", ",", "9048.73`", 
     ",", "9018.42`", ",", "9048.73`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:30:00\"\>", ",", "9049.49`", ",", "9050.69`", 
     ",", "9031.73`", ",", "9033.69`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:40:00\"\>", ",", "9033.99`", ",", "9044.12`", 
     ",", "9030.67`", ",", "9040.11`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T10:50:00\"\>", ",", "9040.34`", ",", "9040.87`", 
     ",", "9010.94`", ",", "9017.37`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:00:00\"\>", ",", "9016.99`", ",", "9023.19`", 
     ",", "9008.37`", ",", "9012.98`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:10:00\"\>", ",", "9013.59`", ",", "9034.52`", 
     ",", "9012.68`", ",", "9034.52`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:20:00\"\>", ",", "9035.81`", ",", "9043.21`", 
     ",", "9034.9`", ",", "9038.98`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:30:00\"\>", ",", "9041.1`", ",", "9049.18`", 
     ",", "9040.57`", ",", "9044.72`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:40:00\"\>", ",", "9044.27`", ",", "9044.8`", 
     ",", "9034.82`", ",", "9042.68`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T11:50:00\"\>", ",", "9042.91`", ",", "9059.61`", 
     ",", "9036.34`", ",", "9055.83`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:00:00\"\>", ",", "9055.`", ",", "9059.54`", 
     ",", "9051.45`", ",", "9059.54`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:10:00\"\>", ",", "9059.76`", ",", "9061.28`", 
     ",", "9052.21`", ",", "9052.21`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:20:00\"\>", ",", "9051.6`", ",", "9052.51`", 
     ",", "9045.03`", ",", "9050.32`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:30:00\"\>", ",", "9050.39`", ",", "9051.15`", 
     ",", "9044.65`", ",", "9049.26`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:40:00\"\>", ",", "9049.18`", ",", "9059.16`", 
     ",", "9049.18`", ",", "9055.23`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T12:50:00\"\>", ",", "9055.15`", ",", "9057.72`", 
     ",", "9041.17`", ",", "9044.65`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:00:00\"\>", ",", "9044.8`", ",", "9046.46`", 
     ",", "9040.79`", ",", "9041.7`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:10:00\"\>", ",", "9041.78`", ",", "9055.91`", 
     ",", "9041.78`", ",", "9053.49`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:20:00\"\>", ",", "9053.57`", ",", "9059.39`", 
     ",", "9047.14`", ",", "9053.94`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:30:00\"\>", ",", "9054.47`", ",", "9088.86`", 
     ",", "9054.47`", ",", "9087.95`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:40:00\"\>", ",", "9087.8`", ",", "9089.16`", 
     ",", "9075.48`", ",", "9077.15`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T13:50:00\"\>", ",", "9077.3`", ",", "9082.21`", 
     ",", "9073.52`", ",", "9077.67`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:00:00\"\>", ",", "9077.52`", ",", "9090.3`", 
     ",", "9077.52`", ",", "9087.27`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:10:00\"\>", ",", "9087.05`", ",", "9089.01`", 
     ",", "9076.31`", ",", "9076.62`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:20:00\"\>", ",", "9074.43`", ",", "9074.43`", 
     ",", "9066.57`", ",", "9070.5`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:30:00\"\>", ",", "9070.42`", ",", "9076.69`", 
     ",", "9067.7`", ",", "9068.08`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:40:00\"\>", ",", "9067.7`", ",", "9083.8`", 
     ",", "9066.94`", ",", "9075.56`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T14:50:00\"\>", ",", "9074.43`", ",", "9074.43`", 
     ",", "9060.14`", ",", "9068.61`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:00:00\"\>", ",", "9068.83`", ",", "9075.71`", 
     ",", "9065.51`", ",", "9066.11`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:10:00\"\>", ",", "9066.34`", ",", "9071.18`", 
     ",", "9061.88`", ",", "9063.09`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:20:00\"\>", ",", "9063.24`", ",", "9084.48`", 
     ",", "9063.01`", ",", "9080.17`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:30:00\"\>", ",", "9079.64`", ",", "9079.94`", 
     ",", "9062.79`", ",", "9065.58`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:40:00\"\>", ",", "9065.43`", ",", "9087.88`", 
     ",", "9065.43`", ",", "9087.88`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T15:50:00\"\>", ",", "9086.14`", ",", "9100.88`", 
     ",", "9081.38`", ",", "9094.38`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-24T16:00:00\"\>", ",", "9094.3`", ",", "9094.6`", 
     ",", "9092.87`", ",", "9093.24`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T09:30:00\"\>", ",", "9093.02`", ",", "9093.02`", 
     ",", "9067.17`", ",", "9074.35`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T09:40:00\"\>", ",", "9078.2`", ",", "9099.89`", 
     ",", "9076.47`", ",", "9086.82`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T09:50:00\"\>", ",", "9086.52`", ",", "9094.`", 
     ",", "9076.92`", ",", "9081.6`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:00:00\"\>", ",", "9087.95`", ",", "9123.17`", 
     ",", "9087.95`", ",", "9098.53`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:10:00\"\>", ",", "9098.46`", ",", "9100.42`", 
     ",", "9062.71`", ",", "9064.75`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:20:00\"\>", ",", "9064.37`", ",", "9070.65`", 
     ",", "9058.33`", ",", "9059.16`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:30:00\"\>", ",", "9057.27`", ",", "9059.91`", 
     ",", "9041.25`", ",", "9046.76`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:40:00\"\>", ",", "9046.61`", ",", "9049.56`", 
     ",", "9039.51`", ",", "9049.33`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T10:50:00\"\>", ",", "9049.71`", ",", "9051.83`", 
     ",", "9034.6`", ",", "9048.35`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:00:00\"\>", ",", "9048.28`", ",", "9058.78`", 
     ",", "9048.13`", ",", "9053.42`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:10:00\"\>", ",", "9053.49`", ",", "9064.`", 
     ",", "9051.53`", ",", "9061.65`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:20:00\"\>", ",", "9061.58`", ",", "9077.6`", 
     ",", "9061.5`", ",", "9075.18`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:30:00\"\>", ",", "9075.03`", ",", "9085.76`", 
     ",", "9072.16`", ",", "9077.67`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:40:00\"\>", ",", "9077.98`", ",", "9083.27`", 
     ",", "9074.95`", ",", "9078.05`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T11:50:00\"\>", ",", "9077.98`", ",", "9080.47`", 
     ",", "9063.47`", ",", "9063.47`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:00:00\"\>", ",", "9063.69`", ",", "9064.52`", 
     ",", "9054.78`", ",", "9059.69`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:10:00\"\>", ",", "9059.46`", ",", "9066.26`", 
     ",", "9056.06`", ",", "9066.26`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:20:00\"\>", ",", "9066.34`", ",", "9069.74`", 
     ",", "9059.76`", ",", "9064.9`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:30:00\"\>", ",", "9065.05`", ",", "9066.49`", 
     ",", "9057.65`", ",", "9066.41`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:40:00\"\>", ",", "9066.57`", ",", "9071.1`", 
     ",", "9052.28`", ",", "9059.91`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T12:50:00\"\>", ",", "9059.54`", ",", "9072.84`", 
     ",", "9059.23`", ",", "9068.`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:00:00\"\>", ",", "9068.45`", ",", "9073.44`", 
     ",", "9065.05`", ",", "9069.74`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:10:00\"\>", ",", "9069.97`", ",", "9070.27`", 
     ",", "9060.22`", ",", "9064.45`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:20:00\"\>", ",", "9064.6`", ",", "9065.73`", 
     ",", "9045.1`", ",", "9046.08`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:30:00\"\>", ",", "9045.93`", ",", "9052.43`", 
     ",", "9041.1`", ",", "9051.9`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:40:00\"\>", ",", "9052.06`", ",", "9065.66`", 
     ",", "9049.56`", ",", "9064.3`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T13:50:00\"\>", ",", "9064.52`", ",", "9080.62`", 
     ",", "9064.52`", ",", "9077.67`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:00:00\"\>", ",", "9078.96`", ",", "9080.32`", 
     ",", "9068.3`", ",", "9074.95`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:10:00\"\>", ",", "9075.03`", ",", "9086.29`", 
     ",", "9074.88`", ",", "9084.63`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:20:00\"\>", ",", "9085.31`", ",", "9085.69`", 
     ",", "9074.2`", ",", "9079.64`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:30:00\"\>", ",", "9080.77`", ",", "9089.24`", 
     ",", "9075.94`", ",", "9086.37`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:40:00\"\>", ",", "9086.52`", ",", "9101.1`", 
     ",", "9085.08`", ",", "9095.28`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T14:50:00\"\>", ",", "9094.83`", ",", "9095.28`", 
     ",", "9086.29`", ",", "9091.73`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:00:00\"\>", ",", "9092.19`", ",", "9098.38`", 
     ",", "9084.1`", ",", "9088.03`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:10:00\"\>", ",", "9087.95`", ",", "9097.02`", 
     ",", "9080.4`", ",", "9080.4`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:20:00\"\>", ",", "9080.02`", ",", "9089.01`", 
     ",", "9074.8`", ",", "9083.72`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:30:00\"\>", ",", "9083.42`", ",", "9084.02`", 
     ",", "9069.29`", ",", "9079.41`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:40:00\"\>", ",", "9079.26`", ",", "9085.91`", 
     ",", "9070.57`", ",", "9073.14`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T15:50:00\"\>", ",", "9074.5`", ",", "9113.88`", 
     ",", "9074.27`", ",", "9112.06`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-27T16:00:00\"\>", ",", "9111.99`", ",", "9111.99`", 
     ",", "9108.43`", ",", "9108.51`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T09:30:00\"\>", ",", "9106.92`", ",", "9106.92`", 
     ",", "9055.38`", ",", "9090.37`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T09:40:00\"\>", ",", "9091.58`", ",", "9111.31`", 
     ",", "9084.4`", ",", "9111.31`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T09:50:00\"\>", ",", "9111.08`", ",", "9124.91`", 
     ",", "9100.42`", ",", "9122.49`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:00:00\"\>", ",", "9121.89`", ",", "9121.89`", 
     ",", "9082.51`", ",", "9088.63`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:10:00\"\>", ",", "9089.62`", ",", "9100.57`", 
     ",", "9054.1`", ",", "9064.6`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:20:00\"\>", ",", "9062.79`", ",", "9077.22`", 
     ",", "9061.2`", ",", "9075.03`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:30:00\"\>", ",", "9074.58`", ",", "9080.7`", 
     ",", "9065.96`", ",", "9070.34`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:40:00\"\>", ",", "9070.12`", ",", "9085.99`", 
     ",", "9064.`", ",", "9082.51`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T10:50:00\"\>", ",", "9082.66`", ",", "9083.19`", 
     ",", "9076.54`", ",", "9082.66`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:00:00\"\>", ",", "9081.68`", ",", "9081.68`", 
     ",", "9048.96`", ",", "9052.58`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:10:00\"\>", ",", "9052.36`", ",", "9052.36`", 
     ",", "9038.9`", ",", "9047.29`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:20:00\"\>", ",", "9046.99`", ",", "9047.07`", 
     ",", "9028.93`", ",", "9030.29`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:30:00\"\>", ",", "9030.37`", ",", "9030.74`", 
     ",", "9007.62`", ",", "9014.34`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:40:00\"\>", ",", "9015.78`", ",", "9032.56`", 
     ",", "9014.57`", ",", "9028.63`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T11:50:00\"\>", ",", "9028.25`", ",", "9037.24`", 
     ",", "9022.96`", ",", "9032.71`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:00:00\"\>", ",", "9032.18`", ",", "9034.45`", 
     ",", "9021.67`", ",", "9034.29`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:10:00\"\>", ",", "9034.22`", ",", "9045.56`", 
     ",", "9034.22`", ",", "9040.64`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:20:00\"\>", ",", "9040.79`", ",", "9044.2`", 
     ",", "9033.77`", ",", "9034.45`", ",", "211", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:30:00\"\>", ",", "9034.29`", ",", "9047.14`", 
     ",", "9033.92`", ",", "9043.67`", ",", "203", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:40:00\"\>", ",", "9043.52`", ",", "9043.52`", 
     ",", "9034.07`", ",", "9034.52`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T12:50:00\"\>", ",", "9034.67`", ",", "9038.98`", 
     ",", "9030.67`", ",", "9038.98`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:00:00\"\>", ",", "9038.83`", ",", "9040.49`", 
     ",", "9024.09`", ",", "9036.86`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:10:00\"\>", ",", "9037.02`", ",", "9054.7`", 
     ",", "9036.79`", ",", "9046.99`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:20:00\"\>", ",", "9046.92`", ",", "9061.12`", 
     ",", "9046.39`", ",", "9053.64`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:30:00\"\>", ",", "9054.62`", ",", "9059.01`", 
     ",", "9050.32`", ",", "9057.12`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:40:00\"\>", ",", "9057.04`", ",", "9058.1`", 
     ",", "9048.5`", ",", "9055.23`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T13:50:00\"\>", ",", "9055.3`", ",", "9081.45`", 
     ",", "9051.9`", ",", "9073.9`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:00:00\"\>", ",", "9074.2`", ",", "9085.61`", 
     ",", "9063.24`", ",", "9085.46`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:10:00\"\>", ",", "9085.69`", ",", "9088.18`", 
     ",", "9075.11`", ",", "9075.86`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:20:00\"\>", ",", "9075.79`", ",", "9086.59`", 
     ",", "9074.8`", ",", "9080.09`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:30:00\"\>", ",", "9079.94`", ",", "9111.31`", 
     ",", "9079.94`", ",", "9096.57`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:40:00\"\>", ",", "9096.8`", ",", "9104.13`", 
     ",", "9080.32`", ",", "9087.95`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T14:50:00\"\>", ",", "9086.67`", ",", "9086.67`", 
     ",", "9066.49`", ",", "9072.84`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:00:00\"\>", ",", "9072.91`", ",", "9075.79`", 
     ",", "9062.41`", ",", "9064.37`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:10:00\"\>", ",", "9063.92`", ",", "9079.04`", 
     ",", "9057.95`", ",", "9058.4`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:20:00\"\>", ",", "9058.33`", ",", "9067.25`", 
     ",", "9049.94`", ",", "9064.9`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:30:00\"\>", ",", "9066.04`", ",", "9078.36`", 
     ",", "9060.14`", ",", "9063.77`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:40:00\"\>", ",", "9063.24`", ",", "9101.93`", 
     ",", "9063.16`", ",", "9093.92`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T15:50:00\"\>", ",", "9095.96`", ",", "9099.67`", 
     ",", "9083.49`", ",", "9097.48`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-28T16:00:00\"\>", ",", "9096.87`", ",", "9097.02`", 
     ",", "9096.42`", ",", "9096.72`", ",", "14", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T09:30:00\"\>", ",", "9092.41`", ",", "9092.41`", 
     ",", "9028.78`", ",", "9035.81`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T09:40:00\"\>", ",", "9032.03`", ",", "9068.98`", 
     ",", "9024.7`", ",", "9060.9`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T09:50:00\"\>", ",", "9060.67`", ",", "9072.69`", 
     ",", "9039.96`", ",", "9069.82`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:00:00\"\>", ",", "9070.57`", ",", "9094.23`", 
     ",", "9068.08`", ",", "9081.6`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:10:00\"\>", ",", "9081.68`", ",", "9091.58`", 
     ",", "9069.82`", ",", "9070.87`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:20:00\"\>", ",", "9070.95`", ",", "9077.45`", 
     ",", "9063.62`", ",", "9074.27`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:30:00\"\>", ",", "9073.14`", ",", "9073.14`", 
     ",", "9047.67`", ",", "9048.28`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:40:00\"\>", ",", "9046.92`", ",", "9064.75`", 
     ",", "9044.57`", ",", "9062.18`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T10:50:00\"\>", ",", "9062.11`", ",", "9072.23`", 
     ",", "9056.44`", ",", "9061.35`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:00:00\"\>", ",", "9061.43`", ",", "9071.02`", 
     ",", "9054.4`", ",", "9069.21`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:10:00\"\>", ",", "9069.29`", ",", "9072.46`", 
     ",", "9048.05`", ",", "9048.65`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:20:00\"\>", ",", "9048.73`", ",", "9051.45`", 
     ",", "9030.74`", ",", "9037.39`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:30:00\"\>", ",", "9037.09`", ",", "9040.57`", 
     ",", "9031.88`", ",", "9032.71`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:40:00\"\>", ",", "9032.78`", ",", "9041.63`", 
     ",", "9023.19`", ",", "9041.32`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T11:50:00\"\>", ",", "9041.55`", ",", "9051.83`", 
     ",", "9040.79`", ",", "9051.`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:00:00\"\>", ",", "9051.3`", ",", "9059.23`", 
     ",", "9050.54`", ",", "9053.34`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:10:00\"\>", ",", "9053.19`", ",", "9055.98`", 
     ",", "9047.82`", ",", "9053.79`", ",", "217", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:20:00\"\>", ",", "9054.02`", ",", "9059.16`", 
     ",", "9048.35`", ",", "9057.12`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:30:00\"\>", ",", "9057.27`", ",", "9068.91`", 
     ",", "9053.26`", ",", "9067.85`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:40:00\"\>", ",", "9067.93`", ",", "9071.86`", 
     ",", "9064.52`", ",", "9068.08`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T12:50:00\"\>", ",", "9068.15`", ",", "9071.02`", 
     ",", "9061.5`", ",", "9061.58`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:00:00\"\>", ",", "9056.97`", ",", "9057.27`", 
     ",", "9032.25`", ",", "9037.54`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:10:00\"\>", ",", "9036.94`", ",", "9046.46`", 
     ",", "9029.08`", ",", "9044.12`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:20:00\"\>", ",", "9043.89`", ",", "9043.97`", 
     ",", "9016.84`", ",", "9020.99`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:30:00\"\>", ",", "9020.46`", ",", "9031.73`", 
     ",", "9014.19`", ",", "9029.91`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:40:00\"\>", ",", "9029.99`", ",", "9038.6`", 
     ",", "9029.23`", ",", "9034.98`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T13:50:00\"\>", ",", "9035.28`", ",", "9039.21`", 
     ",", "9027.42`", ",", "9037.92`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:00:00\"\>", ",", "9039.51`", ",", "9043.29`", 
     ",", "9022.66`", ",", "9025.75`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:10:00\"\>", ",", "9025.68`", ",", "9042.31`", 
     ",", "9022.05`", ",", "9037.24`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:20:00\"\>", ",", "9037.32`", ",", "9054.17`", 
     ",", "9029.16`", ",", "9052.21`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:30:00\"\>", ",", "9051.83`", ",", "9053.42`", 
     ",", "9039.74`", ",", "9044.95`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:40:00\"\>", ",", "9044.88`", ",", "9045.18`", 
     ",", "9028.32`", ",", "9029.46`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T14:50:00\"\>", ",", "9029.08`", ",", "9032.03`", 
     ",", "9018.58`", ",", "9020.09`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:00:00\"\>", ",", "9020.16`", ",", "9035.5`", 
     ",", "9019.86`", ",", "9027.19`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:10:00\"\>", ",", "9027.27`", ",", "9039.66`", 
     ",", "9027.27`", ",", "9031.95`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:20:00\"\>", ",", "9031.88`", ",", "9044.35`", 
     ",", "9031.35`", ",", "9042.15`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:30:00\"\>", ",", "9042.68`", ",", "9067.02`", 
     ",", "9039.96`", ",", "9065.81`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:40:00\"\>", ",", "9066.64`", ",", "9084.1`", 
     ",", "9058.78`", ",", "9076.09`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T15:50:00\"\>", ",", "9076.99`", ",", "9076.99`", 
     ",", "9060.97`", ",", "9073.44`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-29T16:00:00\"\>", ",", "9073.59`", ",", "9073.97`", 
     ",", "9070.72`", ",", "9070.72`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T09:30:00\"\>", ",", "9072.61`", ",", "9164.74`", 
     ",", "9072.61`", ",", "9148.41`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T09:40:00\"\>", ",", "9148.79`", ",", "9155.06`", 
     ",", "9126.5`", ",", "9134.81`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T09:50:00\"\>", ",", "9134.73`", ",", "9201.24`", 
     ",", "9134.58`", ",", "9196.71`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:00:00\"\>", ",", "9199.12`", ",", "9225.5`", 
     ",", "9196.55`", ",", "9222.85`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:10:00\"\>", ",", "9222.7`", ",", "9230.71`", 
     ",", "9217.56`", ",", "9227.16`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:20:00\"\>", ",", "9228.07`", ",", "9233.28`", 
     ",", "9214.92`", ",", "9222.25`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:30:00\"\>", ",", "9221.34`", ",", "9243.11`", 
     ",", "9217.41`", ",", "9243.11`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:40:00\"\>", ",", "9242.96`", ",", "9246.43`", 
     ",", "9237.14`", ",", "9243.94`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T10:50:00\"\>", ",", "9243.56`", ",", "9245.6`", 
     ",", "9234.27`", ",", "9240.77`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:00:00\"\>", ",", "9242.81`", ",", "9244.77`", 
     ",", "9218.92`", ",", "9218.92`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:10:00\"\>", ",", "9219.08`", ",", "9231.17`", 
     ",", "9218.09`", ",", "9226.48`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:20:00\"\>", ",", "9225.95`", ",", "9228.45`", 
     ",", "9219.08`", ",", "9221.87`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:30:00\"\>", ",", "9221.57`", ",", "9229.81`", 
     ",", "9219.68`", ",", "9219.68`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:40:00\"\>", ",", "9219.76`", ",", "9222.93`", 
     ",", "9209.25`", ",", "9220.89`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T11:50:00\"\>", ",", "9221.19`", ",", "9221.27`", 
     ",", "9201.54`", ",", "9204.26`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:00:00\"\>", ",", "9204.11`", ",", "9219.53`", 
     ",", "9200.94`", ",", "9210.99`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:10:00\"\>", ",", "9211.22`", ",", "9217.64`", 
     ",", "9204.26`", ",", "9217.03`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:20:00\"\>", ",", "9217.19`", ",", "9219.83`", 
     ",", "9206.68`", ",", "9207.06`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:30:00\"\>", ",", "9206.98`", ",", "9223.23`", 
     ",", "9206.98`", ",", "9217.49`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:40:00\"\>", ",", "9217.19`", ",", "9223.53`", 
     ",", "9215.6`", ",", "9221.12`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T12:50:00\"\>", ",", "9221.04`", ",", "9221.04`", 
     ",", "9214.31`", ",", "9214.84`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:00:00\"\>", ",", "9214.77`", ",", "9234.49`", 
     ",", "9212.58`", ",", "9230.94`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:10:00\"\>", ",", "9230.79`", ",", "9231.85`", 
     ",", "9220.44`", ",", "9225.8`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:20:00\"\>", ",", "9225.88`", ",", "9236.31`", 
     ",", "9225.35`", ",", "9226.63`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:30:00\"\>", ",", "9226.56`", ",", "9236.84`", 
     ",", "9226.56`", ",", "9233.89`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:40:00\"\>", ",", "9233.43`", ",", "9241.37`", 
     ",", "9229.28`", ",", "9229.96`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T13:50:00\"\>", ",", "9231.85`", ",", "9239.18`", 
     ",", "9218.02`", ",", "9219.08`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:00:00\"\>", ",", "9219.15`", ",", "9227.46`", 
     ",", "9213.56`", ",", "9214.09`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:10:00\"\>", ",", "9213.79`", ",", "9219.15`", 
     ",", "9207.97`", ",", "9213.56`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:20:00\"\>", ",", "9213.41`", ",", "9223.38`", 
     ",", "9207.29`", ",", "9220.13`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:30:00\"\>", ",", "9223.16`", ",", "9225.27`", 
     ",", "9214.84`", ",", "9221.95`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:40:00\"\>", ",", "9221.87`", ",", "9227.31`", 
     ",", "9216.96`", ",", "9216.96`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T14:50:00\"\>", ",", "9216.88`", ",", "9223.23`", 
     ",", "9207.89`", ",", "9213.63`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:00:00\"\>", ",", "9213.86`", ",", "9233.74`", 
     ",", "9213.86`", ",", "9228.82`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:10:00\"\>", ",", "9228.9`", ",", "9229.13`", 
     ",", "9217.56`", ",", "9223.23`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:20:00\"\>", ",", "9223.16`", ",", "9227.54`", 
     ",", "9211.67`", ",", "9219.45`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:30:00\"\>", ",", "9219.91`", ",", "9220.59`", 
     ",", "9180.15`", ",", "9187.71`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:40:00\"\>", ",", "9186.88`", ",", "9187.11`", 
     ",", "9149.85`", ",", "9168.74`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T15:50:00\"\>", ",", "9168.82`", ",", "9182.42`", 
     ",", "9146.07`", ",", "9147.88`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-30T16:00:00\"\>", ",", "9148.11`", ",", "9154.46`", 
     ",", "9148.11`", ",", "9154.46`", ",", "22", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T09:30:00\"\>", ",", "9154.61`", ",", "9187.56`", 
     ",", "9133.68`", ",", "9178.79`", ",", "294", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T09:40:00\"\>", ",", "9182.35`", ",", "9216.66`", 
     ",", "9177.58`", ",", "9204.11`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T09:50:00\"\>", ",", "9203.58`", ",", "9206.53`", 
     ",", "9173.88`", ",", "9176.98`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:00:00\"\>", ",", "9177.51`", ",", "9179.02`", 
     ",", "9135.11`", ",", "9162.02`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:10:00\"\>", ",", "9161.94`", ",", "9180.91`", 
     ",", "9161.94`", ",", "9168.74`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:20:00\"\>", ",", "9169.27`", ",", "9189.15`", 
     ",", "9168.14`", ",", "9177.43`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:30:00\"\>", ",", "9177.13`", ",", "9186.81`", 
     ",", "9175.32`", ",", "9183.25`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:40:00\"\>", ",", "9183.1`", ",", "9209.78`", 
     ",", "9180.23`", ",", "9193.91`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T10:50:00\"\>", ",", "9193.98`", ",", "9198.67`", 
     ",", "9186.96`", ",", "9189.98`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:00:00\"\>", ",", "9189.9`", ",", "9192.85`", 
     ",", "9178.19`", ",", "9179.02`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:10:00\"\>", ",", "9178.95`", ",", "9191.26`", 
     ",", "9167.99`", ",", "9168.59`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:20:00\"\>", ",", "9168.36`", ",", "9182.88`", 
     ",", "9167.16`", ",", "9175.54`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:30:00\"\>", ",", "9175.32`", ",", "9187.71`", 
     ",", "9162.47`", ",", "9184.46`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:40:00\"\>", ",", "9184.31`", ",", "9193.08`", 
     ",", "9180.53`", ",", "9181.44`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T11:50:00\"\>", ",", "9181.29`", ",", "9195.8`", 
     ",", "9180.68`", ",", "9185.44`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:00:00\"\>", ",", "9185.67`", ",", "9187.79`", 
     ",", "9171.24`", ",", "9174.94`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:10:00\"\>", ",", "9174.56`", ",", "9176.83`", 
     ",", "9163.45`", ",", "9173.13`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:20:00\"\>", ",", "9173.28`", ",", "9187.18`", 
     ",", "9172.82`", ",", "9182.27`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:30:00\"\>", ",", "9182.19`", ",", "9184.08`", 
     ",", "9167.`", ",", "9168.06`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:40:00\"\>", ",", "9167.76`", ",", "9168.06`", 
     ",", "9159.3`", ",", "9165.42`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T12:50:00\"\>", ",", "9165.49`", ",", "9171.92`", 
     ",", "9159.9`", ",", "9169.42`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:00:00\"\>", ",", "9168.14`", ",", "9170.78`", 
     ",", "9161.03`", ",", "9170.78`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:10:00\"\>", ",", "9170.63`", ",", "9175.62`", 
     ",", "9165.87`", ",", "9165.87`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:20:00\"\>", ",", "9166.02`", ",", "9177.96`", 
     ",", "9165.42`", ",", "9177.96`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:30:00\"\>", ",", "9179.1`", ",", "9181.82`", 
     ",", "9175.54`", ",", "9177.28`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:40:00\"\>", ",", "9177.51`", ",", "9191.94`", 
     ",", "9176.`", ",", "9188.39`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T13:50:00\"\>", ",", "9188.32`", ",", "9189.6`", 
     ",", "9184.08`", ",", "9184.76`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:00:00\"\>", ",", "9184.84`", ",", "9193.68`", 
     ",", "9179.78`", ",", "9188.09`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:10:00\"\>", ",", "9187.94`", ",", "9209.33`", 
     ",", "9187.18`", ",", "9201.54`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:20:00\"\>", ",", "9201.62`", ",", "9206.91`", 
     ",", "9194.66`", ",", "9199.8`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:30:00\"\>", ",", "9200.03`", ",", "9203.05`", 
     ",", "9180.15`", ",", "9183.18`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:40:00\"\>", ",", "9182.35`", ",", "9184.31`", 
     ",", "9168.82`", ",", "9176.07`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T14:50:00\"\>", ",", "9175.77`", ",", "9178.79`", 
     ",", "9166.48`", ",", "9174.49`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:00:00\"\>", ",", "9174.71`", ",", "9184.99`", 
     ",", "9170.48`", ",", "9181.36`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:10:00\"\>", ",", "9181.06`", ",", "9191.34`", 
     ",", "9176.9`", ",", "9189.98`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:20:00\"\>", ",", "9190.28`", ",", "9218.62`", 
     ",", "9188.92`", ",", "9206.08`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:30:00\"\>", ",", "9206.23`", ",", "9206.3`", 
     ",", "9182.8`", ",", "9185.14`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:40:00\"\>", ",", "9184.39`", ",", "9184.39`", 
     ",", "9160.28`", ",", "9171.16`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T15:50:00\"\>", ",", "9170.25`", ",", "9185.07`", 
     ",", "9151.66`", ",", "9163.3`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-07-31T16:00:00\"\>", ",", "9163.`", ",", "9171.61`", 
     ",", "9163.`", ",", "9171.61`", ",", "20", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T09:30:00\"\>", ",", "9173.73`", ",", "9260.57`", 
     ",", "9173.73`", ",", "9243.79`", ",", "292", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T09:40:00\"\>", ",", "9237.67`", ",", "9238.8`", 
     ",", "9204.87`", ",", "9215.3`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T09:50:00\"\>", ",", "9214.77`", ",", "9229.43`", 
     ",", "9195.95`", ",", "9227.31`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:00:00\"\>", ",", "9227.69`", ",", "9260.49`", 
     ",", "9223.76`", ",", "9223.76`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:10:00\"\>", ",", "9224.89`", ",", "9265.48`", 
     ",", "9222.85`", ",", "9251.42`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:20:00\"\>", ",", "9252.25`", ",", "9267.37`", 
     ",", "9249.76`", ",", "9263.82`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:30:00\"\>", ",", "9267.67`", ",", "9288.45`", 
     ",", "9267.22`", ",", "9279.46`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:40:00\"\>", ",", "9280.29`", ",", "9281.58`", 
     ",", "9267.82`", ",", "9275.08`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T10:50:00\"\>", ",", "9275.15`", ",", "9281.27`", 
     ",", "9250.21`", ",", "9251.12`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:00:00\"\>", ",", "9251.87`", ",", "9259.73`", 
     ",", "9249.68`", ",", "9257.54`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:10:00\"\>", ",", "9258.83`", ",", "9258.83`", 
     ",", "9236.68`", ",", "9242.88`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:20:00\"\>", ",", "9242.65`", ",", "9256.71`", 
     ",", "9237.06`", ",", "9242.58`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:30:00\"\>", ",", "9243.18`", ",", "9249.99`", 
     ",", "9241.9`", ",", "9246.58`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:40:00\"\>", ",", "9246.51`", ",", "9262.08`", 
     ",", "9244.85`", ",", "9259.13`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T11:50:00\"\>", ",", "9258.9`", ",", "9261.17`", 
     ",", "9250.82`", ",", "9261.02`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:00:00\"\>", ",", "9260.72`", ",", "9263.82`", 
     ",", "9255.28`", ",", "9261.85`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:10:00\"\>", ",", "9262.08`", ",", "9264.8`", 
     ",", "9253.69`", ",", "9256.56`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:20:00\"\>", ",", "9256.64`", ",", "9258.98`", 
     ",", "9251.65`", ",", "9255.88`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:30:00\"\>", ",", "9256.26`", ",", "9264.04`", 
     ",", "9234.11`", ",", "9243.94`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:40:00\"\>", ",", "9243.86`", ",", "9256.79`", 
     ",", "9243.86`", ",", "9255.88`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T12:50:00\"\>", ",", "9255.8`", ",", "9260.19`", 
     ",", "9251.12`", ",", "9259.66`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:00:00\"\>", ",", "9259.58`", ",", "9276.29`", 
     ",", "9259.58`", ",", "9268.8`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:10:00\"\>", ",", "9268.95`", ",", "9269.18`", 
     ",", "9259.66`", ",", "9268.88`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:20:00\"\>", ",", "9268.73`", ",", "9273.11`", 
     ",", "9266.61`", ",", "9269.86`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:30:00\"\>", ",", "9269.94`", ",", "9280.06`", 
     ",", "9268.58`", ",", "9272.43`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:40:00\"\>", ",", "9272.36`", ",", "9277.49`", 
     ",", "9271.83`", ",", "9274.02`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T13:50:00\"\>", ",", "9273.79`", ",", "9274.09`", 
     ",", "9268.12`", ",", "9272.05`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:00:00\"\>", ",", "9271.98`", ",", "9291.1`", 
     ",", "9271.9`", ",", "9284.67`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:10:00\"\>", ",", "9284.83`", ",", "9290.8`", 
     ",", "9284.52`", ",", "9287.47`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:20:00\"\>", ",", "9287.55`", ",", "9298.43`", 
     ",", "9287.17`", ",", "9287.17`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:30:00\"\>", ",", "9287.24`", ",", "9289.81`", 
     ",", "9275.98`", ",", "9280.74`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:40:00\"\>", ",", "9280.67`", ",", "9288.3`", 
     ",", "9280.67`", ",", "9281.73`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T14:50:00\"\>", ",", "9281.8`", ",", "9281.88`", 
     ",", "9272.88`", ",", "9272.88`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:00:00\"\>", ",", "9273.26`", ",", "9282.79`", 
     ",", "9272.81`", ",", "9277.12`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:10:00\"\>", ",", "9277.19`", ",", "9288.76`", 
     ",", "9277.19`", ",", "9288.76`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:20:00\"\>", ",", "9288.91`", ",", "9297.22`", 
     ",", "9285.05`", ",", "9286.79`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:30:00\"\>", ",", "9286.56`", ",", "9287.17`", 
     ",", "9274.32`", ",", "9276.74`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:40:00\"\>", ",", "9274.62`", ",", "9283.62`", 
     ",", "9258.45`", ",", "9266.16`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T15:50:00\"\>", ",", "9265.78`", ",", "9284.98`", 
     ",", "9264.5`", ",", "9284.98`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-03T16:00:00\"\>", ",", "9285.81`", ",", "9287.4`", 
     ",", "9285.73`", ",", "9286.56`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T09:30:00\"\>", ",", "9285.35`", ",", "9285.35`", 
     ",", "9251.27`", ",", "9268.65`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T09:40:00\"\>", ",", "9269.48`", ",", "9279.61`", 
     ",", "9260.57`", ",", "9268.65`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T09:50:00\"\>", ",", "9268.05`", ",", "9285.81`", 
     ",", "9261.62`", ",", "9283.69`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:00:00\"\>", ",", "9288.`", ",", "9304.1`", ",",
      "9283.01`", ",", "9283.39`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:10:00\"\>", ",", "9285.05`", ",", "9299.41`", 
     ",", "9279.38`", ",", "9281.73`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:20:00\"\>", ",", "9279.69`", ",", "9283.39`", 
     ",", "9259.96`", ",", "9270.09`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:30:00\"\>", ",", "9270.32`", ",", "9274.09`", 
     ",", "9260.94`", ",", "9267.9`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:40:00\"\>", ",", "9267.67`", ",", "9286.41`", 
     ",", "9267.52`", ",", "9282.41`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T10:50:00\"\>", ",", "9282.26`", ",", "9288.91`", 
     ",", "9277.42`", ",", "9283.84`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:00:00\"\>", ",", "9284.3`", ",", "9291.78`", 
     ",", "9283.77`", ",", "9288.6`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:10:00\"\>", ",", "9288.53`", ",", "9297.67`", 
     ",", "9287.77`", ",", "9292.61`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:20:00\"\>", ",", "9293.37`", ",", "9294.88`", 
     ",", "9277.34`", ",", "9290.8`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:30:00\"\>", ",", "9290.57`", ",", "9298.05`", 
     ",", "9284.98`", ",", "9296.16`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:40:00\"\>", ",", "9296.01`", ",", "9306.06`", 
     ",", "9292.16`", ",", "9293.21`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T11:50:00\"\>", ",", "9292.91`", ",", "9302.74`", 
     ",", "9285.58`", ",", "9301.07`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:00:00\"\>", ",", "9301.15`", ",", "9305.31`", 
     ",", "9293.14`", ",", "9305.31`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:10:00\"\>", ",", "9305.68`", ",", "9307.42`", 
     ",", "9298.73`", ",", "9306.29`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:20:00\"\>", ",", "9305.91`", ",", "9313.85`", 
     ",", "9304.1`", ",", "9309.31`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:30:00\"\>", ",", "9309.61`", ",", "9313.7`", 
     ",", "9303.34`", ",", "9304.55`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:40:00\"\>", ",", "9304.48`", ",", "9315.43`", 
     ",", "9304.4`", ",", "9315.06`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T12:50:00\"\>", ",", "9315.36`", ",", "9321.03`", 
     ",", "9309.84`", ",", "9319.97`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:00:00\"\>", ",", "9320.12`", ",", "9321.1`", 
     ",", "9306.97`", ",", "9307.2`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:10:00\"\>", ",", "9307.57`", ",", "9310.9`", 
     ",", "9302.74`", ",", "9302.81`", ",", "216", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:20:00\"\>", ",", "9302.89`", ",", "9309.24`", 
     ",", "9300.55`", ",", "9306.97`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:30:00\"\>", ",", "9307.04`", ",", "9308.4`", 
     ",", "9302.89`", ",", "9303.57`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:40:00\"\>", ",", "9303.49`", ",", "9305.84`", 
     ",", "9295.86`", ",", "9296.46`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T13:50:00\"\>", ",", "9296.16`", ",", "9296.92`", 
     ",", "9277.8`", ",", "9279.61`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:00:00\"\>", ",", "9279.69`", ",", "9287.85`", 
     ",", "9275.68`", ",", "9284.07`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:10:00\"\>", ",", "9283.99`", ",", "9292.61`", 
     ",", "9283.39`", ",", "9288.53`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:20:00\"\>", ",", "9289.06`", ",", "9295.56`", 
     ",", "9286.87`", ",", "9294.73`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:30:00\"\>", ",", "9294.42`", ",", "9304.48`", 
     ",", "9293.21`", ",", "9303.11`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:40:00\"\>", ",", "9303.57`", ",", "9307.57`", 
     ",", "9287.24`", ",", "9287.47`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T14:50:00\"\>", ",", "9286.87`", ",", "9289.21`", 
     ",", "9278.33`", ",", "9283.09`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:00:00\"\>", ",", "9282.63`", ",", "9299.94`", 
     ",", "9276.51`", ",", "9292.76`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:10:00\"\>", ",", "9292.84`", ",", "9294.5`", 
     ",", "9279.01`", ",", "9283.54`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:20:00\"\>", ",", "9283.69`", ",", "9296.39`", 
     ",", "9281.12`", ",", "9282.71`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:30:00\"\>", ",", "9282.63`", ",", "9288.3`", 
     ",", "9272.58`", ",", "9278.33`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:40:00\"\>", ",", "9279.08`", ",", "9294.57`", 
     ",", "9276.13`", ",", "9285.81`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T15:50:00\"\>", ",", "9287.92`", ",", "9319.21`", 
     ",", "9287.92`", ",", "9319.21`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-04T16:00:00\"\>", ",", "9319.67`", ",", "9320.5`", 
     ",", "9319.51`", ",", "9320.19`", ",", "17", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T09:30:00\"\>", ",", "9315.28`", ",", "9317.78`", 
     ",", "9282.18`", ",", "9309.84`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T09:40:00\"\>", ",", "9305.08`", ",", "9313.7`", 
     ",", "9273.04`", ",", "9275.3`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T09:50:00\"\>", ",", "9274.25`", ",", "9276.97`", 
     ",", "9259.21`", ",", "9269.18`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:00:00\"\>", ",", "9271.75`", ",", "9271.75`", 
     ",", "9234.87`", ",", "9242.05`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:10:00\"\>", ",", "9241.52`", ",", "9243.11`", 
     ",", "9211.74`", ",", "9215.52`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:20:00\"\>", ",", "9215.37`", ",", "9240.39`", 
     ",", "9206.45`", ",", "9239.1`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:30:00\"\>", ",", "9236.16`", ",", "9236.16`", 
     ",", "9222.02`", ",", "9224.06`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:40:00\"\>", ",", "9223.76`", ",", "9232.75`", 
     ",", "9211.29`", ",", "9218.62`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T10:50:00\"\>", ",", "9218.47`", ",", "9237.44`", 
     ",", "9212.35`", ",", "9235.4`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:00:00\"\>", ",", "9236.91`", ",", "9245.98`", 
     ",", "9227.09`", ",", "9245.98`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:10:00\"\>", ",", "9246.66`", ",", "9250.14`", 
     ",", "9242.2`", ",", "9242.81`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:20:00\"\>", ",", "9243.03`", ",", "9247.49`", 
     ",", "9236.`", ",", "9243.86`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:30:00\"\>", ",", "9243.34`", ",", "9244.09`", 
     ",", "9225.65`", ",", "9229.66`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:40:00\"\>", ",", "9229.96`", ",", "9240.61`", 
     ",", "9229.05`", ",", "9240.61`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T11:50:00\"\>", ",", "9240.69`", ",", "9245.98`", 
     ",", "9233.13`", ",", "9234.11`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:00:00\"\>", ",", "9233.96`", ",", "9237.29`", 
     ",", "9229.2`", ",", "9230.34`", ",", "218", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:10:00\"\>", ",", "9230.41`", ",", "9256.03`", 
     ",", "9229.88`", ",", "9255.73`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:20:00\"\>", ",", "9256.03`", ",", "9259.58`", 
     ",", "9246.51`", ",", "9250.21`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:30:00\"\>", ",", "9250.14`", ",", "9250.44`", 
     ",", "9238.42`", ",", "9238.72`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:40:00\"\>", ",", "9238.88`", ",", "9252.63`", 
     ",", "9238.72`", ",", "9250.29`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T12:50:00\"\>", ",", "9249.91`", ",", "9253.54`", 
     ",", "9236.23`", ",", "9239.41`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:00:00\"\>", ",", "9239.56`", ",", "9246.74`", 
     ",", "9223.08`", ",", "9229.5`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:10:00\"\>", ",", "9230.03`", ",", "9245.68`", 
     ",", "9229.66`", ",", "9242.05`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:20:00\"\>", ",", "9242.2`", ",", "9267.29`", 
     ",", "9238.2`", ",", "9264.5`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:30:00\"\>", ",", "9264.8`", ",", "9267.44`", 
     ",", "9261.62`", ",", "9265.33`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:40:00\"\>", ",", "9264.95`", ",", "9273.26`", 
     ",", "9258.68`", ",", "9260.87`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T13:50:00\"\>", ",", "9260.79`", ",", "9266.91`", 
     ",", "9254.37`", ",", "9257.01`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:00:00\"\>", ",", "9257.39`", ",", "9265.71`", 
     ",", "9252.18`", ",", "9261.93`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:10:00\"\>", ",", "9262.`", ",", "9270.92`", 
     ",", "9262.`", ",", "9266.01`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:20:00\"\>", ",", "9266.08`", ",", "9267.37`", 
     ",", "9258.68`", ",", "9263.29`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:30:00\"\>", ",", "9263.44`", ",", "9270.84`", 
     ",", "9260.41`", ",", "9270.77`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:40:00\"\>", ",", "9271.07`", ",", "9300.77`", 
     ",", "9271.07`", ",", "9300.77`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T14:50:00\"\>", ",", "9303.11`", ",", "9305.23`", 
     ",", "9290.64`", ",", "9298.2`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:00:00\"\>", ",", "9299.11`", ",", "9302.89`", 
     ",", "9289.36`", ",", "9292.69`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:10:00\"\>", ",", "9292.23`", ",", "9299.79`", 
     ",", "9291.55`", ",", "9292.46`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:20:00\"\>", ",", "9292.53`", ",", "9314.75`", 
     ",", "9292.53`", ",", "9312.18`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:30:00\"\>", ",", "9310.45`", ",", "9310.45`", 
     ",", "9275.91`", ",", "9280.37`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:40:00\"\>", ",", "9280.22`", ",", "9287.4`", 
     ",", "9267.37`", ",", "9268.73`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T15:50:00\"\>", ",", "9268.58`", ",", "9284.3`", 
     ",", "9268.35`", ",", "9277.34`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-05T16:00:00\"\>", ",", "9277.27`", ",", "9281.05`", 
     ",", "9276.74`", ",", "9280.97`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T09:30:00\"\>", ",", "9277.19`", ",", "9324.28`", 
     ",", "9275.15`", ",", "9311.28`", ",", "288", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T09:40:00\"\>", ",", "9312.79`", ",", "9325.03`", 
     ",", "9279.91`", ",", "9279.91`", ",", "285", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T09:50:00\"\>", ",", "9280.06`", ",", "9294.65`", 
     ",", "9270.01`", ",", "9275.53`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:00:00\"\>", ",", "9275.45`", ",", "9279.46`", 
     ",", "9258.`", ",", "9264.27`", ",", "278", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:10:00\"\>", ",", "9264.04`", ",", "9267.29`", 
     ",", "9231.09`", ",", "9239.18`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:20:00\"\>", ",", "9238.72`", ",", "9247.72`", 
     ",", "9230.56`", ",", "9243.86`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:30:00\"\>", ",", "9241.22`", ",", "9260.72`", 
     ",", "9239.86`", ",", "9257.54`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:40:00\"\>", ",", "9257.24`", ",", "9271.75`", 
     ",", "9252.33`", ",", "9266.39`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T10:50:00\"\>", ",", "9266.54`", ",", "9267.52`", 
     ",", "9255.65`", ",", "9262.08`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:00:00\"\>", ",", "9262.46`", ",", "9276.13`", 
     ",", "9259.89`", ",", "9264.34`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:10:00\"\>", ",", "9264.19`", ",", "9284.98`", 
     ",", "9263.59`", ",", "9281.5`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:20:00\"\>", ",", "9281.42`", ",", "9284.75`", 
     ",", "9276.06`", ",", "9277.27`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:30:00\"\>", ",", "9277.04`", ",", "9277.49`", 
     ",", "9260.34`", ",", "9270.84`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:40:00\"\>", ",", "9271.75`", ",", "9274.85`", 
     ",", "9265.1`", ",", "9272.73`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T11:50:00\"\>", ",", "9272.81`", ",", "9274.7`", 
     ",", "9259.36`", ",", "9263.14`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:00:00\"\>", ",", "9263.36`", ",", "9264.27`", 
     ",", "9248.55`", ",", "9249.08`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:10:00\"\>", ",", "9249.23`", ",", "9254.22`", 
     ",", "9238.42`", ",", "9245.45`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:20:00\"\>", ",", "9245.6`", ",", "9253.76`", 
     ",", "9242.81`", ",", "9242.81`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:30:00\"\>", ",", "9240.31`", ",", "9251.5`", 
     ",", "9240.24`", ",", "9249.53`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:40:00\"\>", ",", "9249.61`", ",", "9262.91`", 
     ",", "9249.61`", ",", "9259.73`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T12:50:00\"\>", ",", "9259.96`", ",", "9260.79`", 
     ",", "9244.62`", ",", "9246.74`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:00:00\"\>", ",", "9246.66`", ",", "9258.`", 
     ",", "9240.84`", ",", "9253.84`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:10:00\"\>", ",", "9253.54`", ",", "9256.26`", 
     ",", "9249.53`", ",", "9249.76`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:20:00\"\>", ",", "9249.83`", ",", "9262.23`", 
     ",", "9249.23`", ",", "9256.86`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:30:00\"\>", ",", "9256.71`", ",", "9270.39`", 
     ",", "9256.71`", ",", "9262.53`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:40:00\"\>", ",", "9262.61`", ",", "9262.98`", 
     ",", "9239.78`", ",", "9241.52`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T13:50:00\"\>", ",", "9240.99`", ",", "9247.34`", 
     ",", "9237.89`", ",", "9241.29`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:00:00\"\>", ",", "9241.07`", ",", "9241.07`", 
     ",", "9210.01`", ",", "9216.43`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:10:00\"\>", ",", "9216.28`", ",", "9221.04`", 
     ",", "9208.95`", ",", "9221.04`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:20:00\"\>", ",", "9221.12`", ",", "9234.11`", 
     ",", "9210.61`", ",", "9228.52`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:30:00\"\>", ",", "9228.75`", ",", "9232.45`", 
     ",", "9217.87`", ",", "9225.57`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:40:00\"\>", ",", "9225.8`", ",", "9227.77`", 
     ",", "9212.12`", ",", "9226.26`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T14:50:00\"\>", ",", "9225.5`", ",", "9238.72`", 
     ",", "9223.08`", ",", "9228.14`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:00:00\"\>", ",", "9227.24`", ",", "9244.24`", 
     ",", "9226.86`", ",", "9242.05`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:10:00\"\>", ",", "9241.97`", ",", "9252.63`", 
     ",", "9230.87`", ",", "9251.8`", ",", "263", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:20:00\"\>", ",", "9251.65`", ",", "9263.74`", 
     ",", "9247.42`", ",", "9256.86`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:30:00\"\>", ",", "9257.01`", ",", "9260.57`", 
     ",", "9244.02`", ",", "9258.3`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:40:00\"\>", ",", "9258.83`", ",", "9267.82`", 
     ",", "9250.74`", ",", "9261.62`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T15:50:00\"\>", ",", "9262.`", ",", "9262.08`", 
     ",", "9243.56`", ",", "9257.17`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-06T16:00:00\"\>", ",", "9255.96`", ",", "9256.56`", 
     ",", "9255.65`", ",", "9256.26`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T09:30:00\"\>", ",", "9258.45`", ",", "9360.85`", 
     ",", "9258.45`", ",", "9323.14`", ",", "290", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T09:40:00\"\>", ",", "9322.16`", ",", "9331.98`", 
     ",", "9313.85`", ",", "9329.79`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T09:50:00\"\>", ",", "9329.64`", ",", "9341.28`", 
     ",", "9318.53`", ",", "9318.53`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:00:00\"\>", ",", "9317.25`", ",", "9317.47`", 
     ",", "9301.3`", ",", "9309.16`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:10:00\"\>", ",", "9309.01`", ",", "9331.`", 
     ",", "9306.44`", ",", "9329.11`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:20:00\"\>", ",", "9330.55`", ",", "9346.12`", 
     ",", "9317.93`", ",", "9344.61`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:30:00\"\>", ",", "9345.13`", ",", "9387.15`", 
     ",", "9344.83`", ",", "9383.22`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:40:00\"\>", ",", "9383.`", ",", "9385.57`", 
     ",", "9368.49`", ",", "9379.67`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T10:50:00\"\>", ",", "9379.52`", ",", "9407.11`", 
     ",", "9377.63`", ",", "9398.49`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:00:00\"\>", ",", "9399.93`", ",", "9417.01`", 
     ",", "9399.93`", ",", "9402.65`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:10:00\"\>", ",", "9402.57`", ",", "9406.35`", 
     ",", "9393.88`", ",", "9395.77`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:20:00\"\>", ",", "9395.69`", ",", "9405.14`", 
     ",", "9394.71`", ",", "9397.66`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:30:00\"\>", ",", "9395.54`", ",", "9408.77`", 
     ",", "9391.69`", ",", "9405.82`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:40:00\"\>", ",", "9405.9`", ",", "9405.9`", 
     ",", "9373.7`", ",", "9381.49`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T11:50:00\"\>", ",", "9381.33`", ",", "9399.25`", 
     ",", "9380.65`", ",", "9390.1`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:00:00\"\>", ",", "9390.4`", ",", "9398.94`", 
     ",", "9389.8`", ",", "9392.82`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:10:00\"\>", ",", "9392.75`", ",", "9394.56`", 
     ",", "9381.18`", ",", "9388.36`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:20:00\"\>", ",", "9388.59`", ",", "9391.84`", 
     ",", "9382.7`", ",", "9386.78`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:30:00\"\>", ",", "9387.38`", ",", "9397.21`", 
     ",", "9386.4`", ",", "9395.85`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:40:00\"\>", ",", "9395.69`", ",", "9406.58`", 
     ",", "9394.86`", ",", "9401.14`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T12:50:00\"\>", ",", "9400.91`", ",", "9410.2`", 
     ",", "9399.4`", ",", "9401.66`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:00:00\"\>", ",", "9404.23`", ",", "9417.61`", 
     ",", "9402.8`", ",", "9407.03`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:10:00\"\>", ",", "9407.11`", ",", "9416.78`", 
     ",", "9404.01`", ",", "9412.55`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:20:00\"\>", ",", "9414.66`", ",", "9435.07`", 
     ",", "9412.62`", ",", "9433.86`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:30:00\"\>", ",", "9433.56`", ",", "9437.49`", 
     ",", "9428.12`", ",", "9429.17`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:40:00\"\>", ",", "9429.1`", ",", "9429.55`", 
     ",", "9425.47`", ",", "9427.13`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T13:50:00\"\>", ",", "9427.06`", ",", "9427.06`", 
     ",", "9420.26`", ",", "9421.99`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:00:00\"\>", ",", "9421.84`", ",", "9425.77`", 
     ",", "9420.71`", ",", "9425.77`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:10:00\"\>", ",", "9426.6`", ",", "9428.19`", 
     ",", "9420.33`", ",", "9420.33`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:20:00\"\>", ",", "9420.41`", ",", "9427.81`", 
     ",", "9411.41`", ",", "9416.78`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:30:00\"\>", ",", "9417.01`", ",", "9421.01`", 
     ",", "9407.94`", ",", "9412.47`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:40:00\"\>", ",", "9412.32`", ",", "9416.25`", 
     ",", "9406.88`", ",", "9408.92`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T14:50:00\"\>", ",", "9409.07`", ",", "9410.2`", 
     ",", "9405.37`", ",", "9406.65`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:00:00\"\>", ",", "9406.58`", ",", "9406.95`", 
     ",", "9387.91`", ",", "9397.73`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:10:00\"\>", ",", "9397.66`", ",", "9398.64`", 
     ",", "9384.96`", ",", "9386.25`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:20:00\"\>", ",", "9385.64`", ",", "9403.63`", 
     ",", "9373.17`", ",", "9401.21`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:30:00\"\>", ",", "9400.38`", ",", "9401.89`", 
     ",", "9383.6`", ",", "9386.02`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:40:00\"\>", ",", "9386.25`", ",", "9394.56`", 
     ",", "9376.35`", ",", "9382.17`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T15:50:00\"\>", ",", "9380.05`", ",", "9394.94`", 
     ",", "9368.11`", ",", "9368.11`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-07T16:00:00\"\>", ",", "9368.34`", ",", "9370.23`", 
     ",", "9368.26`", ",", "9370.07`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T09:30:00\"\>", ",", "9368.41`", ",", "9368.41`", 
     ",", "9323.52`", ",", "9340.37`", ",", "291", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T09:40:00\"\>", ",", "9343.77`", ",", "9353.22`", 
     ",", "9330.4`", ",", "9336.37`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T09:50:00\"\>", ",", "9336.82`", ",", "9355.19`", 
     ",", "9334.63`", ",", "9345.81`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:00:00\"\>", ",", "9345.74`", ",", "9345.97`", 
     ",", "9329.64`", ",", "9337.8`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:10:00\"\>", ",", "9339.54`", ",", "9356.02`", 
     ",", "9338.79`", ",", "9341.43`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:20:00\"\>", ",", "9338.86`", ",", "9360.78`", 
     ",", "9336.37`", ",", "9360.78`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:30:00\"\>", ",", "9360.63`", ",", "9366.14`", 
     ",", "9352.16`", ",", "9362.29`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:40:00\"\>", ",", "9361.76`", ",", "9364.63`", 
     ",", "9354.73`", ",", "9356.17`", ",", "255", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T10:50:00\"\>", ",", "9356.24`", ",", "9368.71`", 
     ",", "9352.92`", ",", "9361.84`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:00:00\"\>", ",", "9361.99`", ",", "9371.96`", 
     ",", "9356.85`", ",", "9357.23`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:10:00\"\>", ",", "9356.85`", ",", "9362.21`", 
     ",", "9351.63`", ",", "9355.94`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:20:00\"\>", ",", "9355.71`", ",", "9355.94`", 
     ",", "9343.77`", ",", "9348.54`", ",", "252", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:30:00\"\>", ",", "9348.76`", ",", "9359.72`", 
     ",", "9348.76`", ",", "9359.72`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:40:00\"\>", ",", "9359.8`", ",", "9360.32`", 
     ",", "9345.89`", ",", "9346.04`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T11:50:00\"\>", ",", "9346.12`", ",", "9351.33`", 
     ",", "9339.84`", ",", "9342.41`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:00:00\"\>", ",", "9342.34`", ",", "9348.46`", 
     ",", "9341.88`", ",", "9344.38`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:10:00\"\>", ",", "9344.3`", ",", "9346.12`", 
     ",", "9340.22`", ",", "9344.38`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:20:00\"\>", ",", "9344.45`", ",", "9344.45`", 
     ",", "9327.98`", ",", "9328.58`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:30:00\"\>", ",", "9328.43`", ",", "9332.82`", 
     ",", "9324.28`", ",", "9327.9`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:40:00\"\>", ",", "9328.05`", ",", "9331.76`", 
     ",", "9321.33`", ",", "9325.11`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T12:50:00\"\>", ",", "9324.96`", ",", "9331.3`", 
     ",", "9323.07`", ",", "9324.43`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:00:00\"\>", ",", "9323.82`", ",", "9324.8`", 
     ",", "9308.1`", ",", "9308.1`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:10:00\"\>", ",", "9308.4`", ",", "9314.22`", 
     ",", "9302.59`", ",", "9305.99`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:20:00\"\>", ",", "9305.84`", ",", "9315.36`", 
     ",", "9301.91`", ",", "9311.28`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:30:00\"\>", ",", "9311.5`", ",", "9323.9`", 
     ",", "9309.31`", ",", "9318.38`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:40:00\"\>", ",", "9318.68`", ",", "9328.28`", 
     ",", "9317.63`", ",", "9321.1`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T13:50:00\"\>", ",", "9321.18`", ",", "9323.67`", 
     ",", "9316.11`", ",", "9316.34`", ",", "217", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:00:00\"\>", ",", "9316.57`", ",", "9317.32`", 
     ",", "9307.95`", ",", "9315.74`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:10:00\"\>", ",", "9315.81`", ",", "9316.42`", 
     ",", "9292.23`", ",", "9293.06`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:20:00\"\>", ",", "9292.99`", ",", "9302.74`", 
     ",", "9290.95`", ",", "9301.98`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:30:00\"\>", ",", "9302.06`", ",", "9309.77`", 
     ",", "9296.92`", ",", "9305.38`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:40:00\"\>", ",", "9305.46`", ",", "9312.94`", 
     ",", "9301.91`", ",", "9303.04`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T14:50:00\"\>", ",", "9303.19`", ",", "9311.05`", 
     ",", "9295.18`", ",", "9310.37`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:00:00\"\>", ",", "9310.9`", ",", "9327.53`", 
     ",", "9310.82`", ",", "9320.65`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:10:00\"\>", ",", "9320.8`", ",", "9323.67`", 
     ",", "9314.6`", ",", "9321.93`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:20:00\"\>", ",", "9321.71`", ",", "9329.49`", 
     ",", "9316.72`", ",", "9323.82`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:30:00\"\>", ",", "9324.12`", ",", "9325.64`", 
     ",", "9314.`", ",", "9321.93`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:40:00\"\>", ",", "9321.55`", ",", "9335.61`", 
     ",", "9315.13`", ",", "9333.87`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T15:50:00\"\>", ",", "9334.1`", ",", "9339.77`", 
     ",", "9323.44`", ",", "9339.16`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-10T16:00:00\"\>", ",", "9339.32`", ",", "9339.32`", 
     ",", "9337.73`", ",", "9337.95`", ",", "15", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T09:30:00\"\>", ",", "9334.33`", ",", "9334.33`", 
     ",", "9286.56`", ",", "9302.59`", ",", "295", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T09:40:00\"\>", ",", "9296.39`", ",", "9298.43`", 
     ",", "9258.75`", ",", "9263.21`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T09:50:00\"\>", ",", "9263.51`", ",", "9282.18`", 
     ",", "9261.47`", ",", "9279.46`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:00:00\"\>", ",", "9278.1`", ",", "9278.1`", 
     ",", "9250.82`", ",", "9260.26`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:10:00\"\>", ",", "9261.55`", ",", "9268.8`", 
     ",", "9257.92`", ",", "9257.92`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:20:00\"\>", ",", "9253.31`", ",", "9258.68`", 
     ",", "9239.41`", ",", "9241.6`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:30:00\"\>", ",", "9240.84`", ",", "9248.78`", 
     ",", "9235.17`", ",", "9247.49`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:40:00\"\>", ",", "9246.21`", ",", "9253.92`", 
     ",", "9237.52`", ",", "9238.2`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T10:50:00\"\>", ",", "9238.27`", ",", "9252.63`", 
     ",", "9238.2`", ",", "9251.65`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:00:00\"\>", ",", "9251.35`", ",", "9254.07`", 
     ",", "9238.42`", ",", "9241.45`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:10:00\"\>", ",", "9240.99`", ",", "9245.07`", 
     ",", "9217.19`", ",", "9226.03`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:20:00\"\>", ",", "9226.26`", ",", "9241.14`", 
     ",", "9226.26`", ",", "9241.14`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:30:00\"\>", ",", "9239.18`", ",", "9239.78`", 
     ",", "9228.07`", ",", "9229.73`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:40:00\"\>", ",", "9229.66`", ",", "9241.14`", 
     ",", "9229.66`", ",", "9238.95`", ",", "242", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T11:50:00\"\>", ",", "9238.8`", ",", "9248.85`", 
     ",", "9236.16`", ",", "9243.11`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:00:00\"\>", ",", "9242.73`", ",", "9246.58`", 
     ",", "9239.48`", ",", "9244.62`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:10:00\"\>", ",", "9245.`", ",", "9257.09`", 
     ",", "9243.03`", ",", "9257.01`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:20:00\"\>", ",", "9257.17`", ",", "9257.17`", 
     ",", "9241.45`", ",", "9245.15`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:30:00\"\>", ",", "9245.38`", ",", "9250.21`", 
     ",", "9244.92`", ",", "9250.21`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:40:00\"\>", ",", "9250.44`", ",", "9250.44`", 
     ",", "9234.11`", ",", "9234.11`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T12:50:00\"\>", ",", "9234.34`", ",", "9234.49`", 
     ",", "9229.13`", ",", "9229.81`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:00:00\"\>", ",", "9229.73`", ",", "9240.77`", 
     ",", "9229.73`", ",", "9238.04`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:10:00\"\>", ",", "9236.99`", ",", "9241.14`", 
     ",", "9230.49`", ",", "9240.46`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:20:00\"\>", ",", "9240.61`", ",", "9246.13`", 
     ",", "9233.66`", ",", "9233.81`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:30:00\"\>", ",", "9233.59`", ",", "9249.`", 
     ",", "9233.43`", ",", "9242.13`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:40:00\"\>", ",", "9242.43`", ",", "9250.59`", 
     ",", "9240.92`", ",", "9250.21`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T13:50:00\"\>", ",", "9250.06`", ",", "9257.24`", 
     ",", "9249.91`", ",", "9250.82`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:00:00\"\>", ",", "9252.63`", ",", "9271.75`", 
     ",", "9252.63`", ",", "9267.59`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:10:00\"\>", ",", "9268.5`", ",", "9274.77`", 
     ",", "9265.18`", ",", "9270.09`", ",", "247", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:20:00\"\>", ",", "9270.39`", ",", "9277.19`", 
     ",", "9260.34`", ",", "9266.16`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:30:00\"\>", ",", "9266.01`", ",", "9268.58`", 
     ",", "9260.26`", ",", "9261.93`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:40:00\"\>", ",", "9262.08`", ",", "9266.69`", 
     ",", "9252.33`", ",", "9254.9`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T14:50:00\"\>", ",", "9255.65`", ",", "9272.58`", 
     ",", "9255.12`", ",", "9271.52`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:00:00\"\>", ",", "9271.45`", ",", "9281.73`", 
     ",", "9265.25`", ",", "9266.76`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:10:00\"\>", ",", "9267.14`", ",", "9269.64`", 
     ",", "9259.43`", ",", "9264.12`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:20:00\"\>", ",", "9264.65`", ",", "9273.72`", 
     ",", "9259.89`", ",", "9269.33`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:30:00\"\>", ",", "9269.11`", ",", "9269.56`", 
     ",", "9259.36`", ",", "9261.47`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:40:00\"\>", ",", "9261.1`", ",", "9276.29`", 
     ",", "9260.34`", ",", "9266.08`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T15:50:00\"\>", ",", "9266.46`", ",", "9267.37`", 
     ",", "9240.24`", ",", "9240.77`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-11T16:00:00\"\>", ",", "9240.92`", ",", "9241.67`", 
     ",", "9240.69`", ",", "9241.45`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T09:30:00\"\>", ",", "9236.06`", ",", "9276.59`", 
     ",", "9221.1`", ",", "9271.6`", ",", "287", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T09:40:00\"\>", ",", "9270.92`", ",", "9299.87`", 
     ",", "9270.92`", ",", "9296.62`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T09:50:00\"\>", ",", "9297.75`", ",", "9327.`", 
     ",", "9297.75`", ",", "9327.`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:00:00\"\>", ",", "9328.21`", ",", "9349.74`", 
     ",", "9328.21`", ",", "9341.96`", ",", "280", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:10:00\"\>", ",", "9341.51`", ",", "9358.06`", 
     ",", "9337.43`", ",", "9352.92`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:20:00\"\>", ",", "9356.4`", ",", "9366.9`", 
     ",", "9351.56`", ",", "9363.2`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:30:00\"\>", ",", "9358.96`", ",", "9359.8`", 
     ",", "9350.58`", ",", "9353.15`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:40:00\"\>", ",", "9352.99`", ",", "9369.85`", 
     ",", "9349.97`", ",", "9367.88`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T10:50:00\"\>", ",", "9368.26`", ",", "9376.5`", 
     ",", "9364.1`", ",", "9366.3`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:00:00\"\>", ",", "9366.14`", ",", "9370.91`", 
     ",", "9358.81`", ",", "9360.78`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:10:00\"\>", ",", "9361.01`", ",", "9369.62`", 
     ",", "9360.1`", ",", "9367.13`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:20:00\"\>", ",", "9367.2`", ",", "9368.34`", 
     ",", "9361.23`", ",", "9363.95`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:30:00\"\>", ",", "9364.33`", ",", "9373.02`", 
     ",", "9362.14`", ",", "9373.02`", ",", "249", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:40:00\"\>", ",", "9373.1`", ",", "9374.38`", 
     ",", "9362.52`", ",", "9368.79`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T11:50:00\"\>", ",", "9369.55`", ",", "9370.68`", 
     ",", "9361.46`", ",", "9366.67`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:00:00\"\>", ",", "9366.6`", ",", "9370.98`", 
     ",", "9364.86`", ",", "9365.77`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:10:00\"\>", ",", "9365.84`", ",", "9367.5`", 
     ",", "9361.53`", ",", "9363.2`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:20:00\"\>", ",", "9363.05`", ",", "9364.41`", 
     ",", "9356.17`", ",", "9362.14`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:30:00\"\>", ",", "9362.52`", ",", "9363.8`", 
     ",", "9353.98`", ",", "9363.57`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:40:00\"\>", ",", "9363.65`", ",", "9363.8`", 
     ",", "9350.73`", ",", "9356.17`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T12:50:00\"\>", ",", "9355.87`", ",", "9369.09`", 
     ",", "9355.11`", ",", "9368.26`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:00:00\"\>", ",", "9368.11`", ",", "9373.32`", 
     ",", "9360.32`", ",", "9372.79`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:10:00\"\>", ",", "9372.87`", ",", "9386.1`", 
     ",", "9372.79`", ",", "9382.09`", ",", "254", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:20:00\"\>", ",", "9382.01`", ",", "9387.99`", 
     ",", "9381.79`", ",", "9382.7`", ",", "228", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:30:00\"\>", ",", "9382.54`", ",", "9382.7`", 
     ",", "9371.43`", ",", "9372.95`", ",", "214", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:40:00\"\>", ",", "9372.64`", ",", "9375.59`", 
     ",", "9364.78`", ",", "9368.86`", ",", "217", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T13:50:00\"\>", ",", "9368.79`", ",", "9370.38`", 
     ",", "9361.53`", ",", "9369.47`", ",", "216", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:00:00\"\>", ",", "9369.62`", ",", "9371.89`", 
     ",", "9364.48`", ",", "9368.26`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:10:00\"\>", ",", "9368.03`", ",", "9382.85`", 
     ",", "9346.87`", ",", "9349.44`", ",", "266", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:20:00\"\>", ",", "9355.26`", ",", "9376.8`", 
     ",", "9346.19`", ",", "9346.19`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:30:00\"\>", ",", "9345.06`", ",", "9354.28`", 
     ",", "9334.63`", ",", "9337.95`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:40:00\"\>", ",", "9337.73`", ",", "9352.99`", 
     ",", "9328.28`", ",", "9350.58`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T14:50:00\"\>", ",", "9350.88`", ",", "9410.2`", 
     ",", "9344.68`", ",", "9410.2`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:00:00\"\>", ",", "9409.98`", ",", "9420.48`", 
     ",", "9403.55`", ",", "9414.06`", ",", "261", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:10:00\"\>", ",", "9413.53`", ",", "9424.11`", 
     ",", "9407.18`", ",", "9419.2`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:20:00\"\>", ",", "9419.12`", ",", "9421.77`", 
     ",", "9399.47`", ",", "9410.05`", ",", "268", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:30:00\"\>", ",", "9412.85`", ",", "9422.15`", 
     ",", "9403.4`", ",", "9413.08`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:40:00\"\>", ",", "9413.`", ",", "9413.15`", 
     ",", "9374.16`", ",", "9386.02`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T15:50:00\"\>", ",", "9385.49`", ",", "9388.14`", 
     ",", "9348.69`", ",", "9359.72`", ",", "276", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-12T16:00:00\"\>", ",", "9358.89`", ",", "9361.61`", 
     ",", "9358.59`", ",", "9361.61`", ",", "19", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T09:30:00\"\>", ",", "9362.29`", ",", "9382.17`", 
     ",", "9359.57`", ",", "9378.24`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T09:40:00\"\>", ",", "9377.4`", ",", "9377.56`", 
     ",", "9363.42`", ",", "9364.1`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T09:50:00\"\>", ",", "9363.88`", ",", "9363.88`", 
     ",", "9305.84`", ",", "9315.74`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:00:00\"\>", ",", "9315.58`", ",", "9338.11`", 
     ",", "9305.38`", ",", "9331.68`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:10:00\"\>", ",", "9331.53`", ",", "9344.38`", 
     ",", "9326.85`", ",", "9333.12`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:20:00\"\>", ",", "9333.27`", ",", "9350.88`", 
     ",", "9333.27`", ",", "9347.63`", ",", "218", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:30:00\"\>", ",", "9347.4`", ",", "9366.6`", 
     ",", "9344.53`", ",", "9365.24`", ",", "190", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:40:00\"\>", ",", "9365.16`", ",", "9377.18`", 
     ",", "9365.16`", ",", "9373.47`", ",", "206", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T10:50:00\"\>", ",", "9373.78`", ",", "9395.69`", 
     ",", "9372.19`", ",", "9389.8`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:00:00\"\>", ",", "9390.03`", ",", "9393.12`", 
     ",", "9383.45`", ",", "9385.94`", ",", "191", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:10:00\"\>", ",", "9385.79`", ",", "9386.17`", 
     ",", "9368.49`", ",", "9370.6`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:20:00\"\>", ",", "9370.45`", ",", "9374.`", 
     ",", "9364.93`", ",", "9372.34`", ",", "245", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:30:00\"\>", ",", "9372.42`", ",", "9376.8`", 
     ",", "9361.46`", ",", "9376.57`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:40:00\"\>", ",", "9376.72`", ",", "9377.4`", 
     ",", "9373.17`", ",", "9373.63`", ",", "214", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T11:50:00\"\>", ",", "9373.32`", ",", "9382.24`", 
     ",", "9370.38`", ",", "9382.24`", ",", "227", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:00:00\"\>", ",", "9382.09`", ",", "9386.32`", 
     ",", "9377.78`", ",", "9382.7`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:10:00\"\>", ",", "9382.17`", ",", "9383.6`", 
     ",", "9376.88`", ",", "9382.47`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:20:00\"\>", ",", "9382.17`", ",", "9382.7`", 
     ",", "9374.38`", ",", "9376.65`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:30:00\"\>", ",", "9376.88`", ",", "9385.57`", 
     ",", "9375.82`", ",", "9385.11`", ",", "213", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:40:00\"\>", ",", "9385.04`", ",", "9386.93`", 
     ",", "9374.53`", ",", "9378.92`", ",", "214", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T12:50:00\"\>", ",", "9378.84`", ",", "9379.29`", 
     ",", "9374.91`", ",", "9377.4`", ",", "215", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:00:00\"\>", ",", "9377.48`", ",", "9405.97`", 
     ",", "9377.4`", ",", "9399.17`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:10:00\"\>", ",", "9399.02`", ",", "9402.34`", 
     ",", "9373.1`", ",", "9376.65`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:20:00\"\>", ",", "9376.57`", ",", "9381.86`", 
     ",", "9369.32`", ",", "9369.32`", ",", "234", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:30:00\"\>", ",", "9369.39`", ",", "9379.37`", 
     ",", "9369.39`", ",", "9377.56`", ",", "211", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:40:00\"\>", ",", "9377.48`", ",", "9378.08`", 
     ",", "9371.21`", ",", "9374.84`", ",", "197", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T13:50:00\"\>", ",", "9374.91`", ",", "9374.91`", 
     ",", "9354.43`", ",", "9361.91`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:00:00\"\>", ",", "9361.99`", ",", "9362.67`", 
     ",", "9346.57`", ",", "9349.06`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:10:00\"\>", ",", "9348.99`", ",", "9349.14`", 
     ",", "9331.61`", ",", "9347.55`", ",", "257", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:20:00\"\>", ",", "9347.48`", ",", "9354.73`", 
     ",", "9342.94`", ",", "9352.99`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:30:00\"\>", ",", "9353.07`", ",", "9366.6`", 
     ",", "9352.16`", ",", "9362.67`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:40:00\"\>", ",", "9362.52`", ",", "9372.79`", 
     ",", "9362.52`", ",", "9364.03`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T14:50:00\"\>", ",", "9365.62`", ",", "9366.75`", 
     ",", "9349.9`", ",", "9350.73`", ",", "239", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:00:00\"\>", ",", "9351.1`", ",", "9364.86`", 
     ",", "9351.03`", ",", "9361.01`", ",", "225", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:10:00\"\>", ",", "9361.08`", ",", "9383.83`", 
     ",", "9360.32`", ",", "9380.35`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:20:00\"\>", ",", "9380.43`", ",", "9384.66`", 
     ",", "9375.59`", ",", "9377.56`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:30:00\"\>", ",", "9377.4`", ",", "9385.11`", 
     ",", "9373.17`", ",", "9375.97`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:40:00\"\>", ",", "9375.52`", ",", "9396.9`", 
     ",", "9375.52`", ",", "9383.75`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T15:50:00\"\>", ",", "9383.45`", ",", "9399.32`", 
     ",", "9377.4`", ",", "9399.32`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-13T16:00:00\"\>", ",", "9399.17`", ",", "9399.55`", 
     ",", "9397.51`", ",", "9398.19`", ",", "18", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T09:30:00\"\>", ",", "9398.04`", ",", "9401.66`", 
     ",", "9360.63`", ",", "9369.55`", ",", "289", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T09:40:00\"\>", ",", "9368.94`", ",", "9369.92`", 
     ",", "9341.51`", ",", "9349.67`", ",", "284", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T09:50:00\"\>", ",", "9350.42`", ",", "9361.61`", 
     ",", "9309.54`", ",", "9320.42`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:00:00\"\>", ",", "9318.23`", ",", "9318.23`", 
     ",", "9281.35`", ",", "9293.74`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:10:00\"\>", ",", "9294.35`", ",", "9295.18`", 
     ",", "9251.42`", ",", "9264.34`", ",", "279", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:20:00\"\>", ",", "9263.82`", ",", "9283.84`", 
     ",", "9257.17`", ",", "9276.74`", ",", "275", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:30:00\"\>", ",", "9274.77`", ",", "9280.22`", 
     ",", "9262.`", ",", "9277.34`", ",", "271", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:40:00\"\>", ",", "9277.12`", ",", "9282.63`", 
     ",", "9265.33`", ",", "9271.`", ",", "272", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T10:50:00\"\>", ",", "9270.77`", ",", "9270.77`", 
     ",", "9253.24`", ",", "9269.56`", ",", "269", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:00:00\"\>", ",", "9270.01`", ",", "9278.93`", 
     ",", "9264.65`", ",", "9268.12`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:10:00\"\>", ",", "9268.05`", ",", "9268.58`", 
     ",", "9251.57`", ",", "9257.24`", ",", "251", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:20:00\"\>", ",", "9257.09`", ",", "9261.02`", 
     ",", "9246.66`", ",", "9246.66`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:30:00\"\>", ",", "9245.68`", ",", "9257.24`", 
     ",", "9233.06`", ",", "9252.93`", ",", "256", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:40:00\"\>", ",", "9252.78`", ",", "9261.55`", 
     ",", "9251.12`", ",", "9254.67`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T11:50:00\"\>", ",", "9254.37`", ",", "9257.39`", 
     ",", "9247.72`", ",", "9251.65`", ",", "243", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:00:00\"\>", ",", "9251.04`", ",", "9251.27`", 
     ",", "9245.53`", ",", "9248.55`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:10:00\"\>", ",", "9248.63`", ",", "9258.37`", 
     ",", "9247.11`", ",", "9258.3`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:20:00\"\>", ",", "9258.15`", ",", "9258.3`", 
     ",", "9244.7`", ",", "9248.1`", ",", "222", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:30:00\"\>", ",", "9246.06`", ",", "9255.65`", 
     ",", "9243.41`", ",", "9255.5`", ",", "208", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:40:00\"\>", ",", "9255.58`", ",", "9261.78`", 
     ",", "9247.79`", ",", "9260.04`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T12:50:00\"\>", ",", "9260.11`", ",", "9269.64`", 
     ",", "9258.15`", ",", "9268.27`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:00:00\"\>", ",", "9268.43`", ",", "9268.8`", 
     ",", "9259.05`", ",", "9260.26`", ",", "203", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:10:00\"\>", ",", "9260.34`", ",", "9269.79`", 
     ",", "9257.69`", ",", "9268.2`", ",", "215", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:20:00\"\>", ",", "9268.27`", ",", "9269.56`", 
     ",", "9260.57`", ",", "9263.14`", ",", "214", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:30:00\"\>", ",", "9263.06`", ",", "9263.21`", 
     ",", "9250.97`", ",", "9253.39`", ",", "223", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:40:00\"\>", ",", "9253.46`", ",", "9259.58`", 
     ",", "9253.46`", ",", "9258.9`", ",", "209", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T13:50:00\"\>", ",", "9258.68`", ",", "9259.96`", 
     ",", "9252.86`", ",", "9259.96`", ",", "188", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:00:00\"\>", ",", "9261.78`", ",", "9264.19`", 
     ",", "9255.96`", ",", "9255.96`", ",", "219", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:10:00\"\>", ",", "9255.8`", ",", "9267.97`", 
     ",", "9249.83`", ",", "9261.93`", ",", "248", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:20:00\"\>", ",", "9261.78`", ",", "9261.78`", 
     ",", "9254.37`", ",", "9258.53`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:30:00\"\>", ",", "9256.71`", ",", "9264.5`", 
     ",", "9253.61`", ",", "9264.27`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:40:00\"\>", ",", "9264.19`", ",", "9275.38`", 
     ",", "9263.51`", ",", "9274.25`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T14:50:00\"\>", ",", "9273.94`", ",", "9276.59`", 
     ",", "9264.04`", ",", "9269.33`", ",", "240", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:00:00\"\>", ",", "9269.79`", ",", "9280.97`", 
     ",", "9269.48`", ",", "9278.33`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:10:00\"\>", ",", "9278.4`", ",", "9279.23`", 
     ",", "9265.1`", ",", "9271.6`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:20:00\"\>", ",", "9272.36`", ",", "9273.79`", 
     ",", "9256.64`", ",", "9257.54`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:30:00\"\>", ",", "9257.62`", ",", "9283.92`", 
     ",", "9257.32`", ",", "9281.42`", ",", "253", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:40:00\"\>", ",", "9281.35`", ",", "9298.73`", 
     ",", "9274.85`", ",", "9283.01`", ",", "265", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T15:50:00\"\>", ",", "9284.07`", ",", "9321.4`", 
     ",", "9281.88`", ",", "9321.1`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-14T16:00:00\"\>", ",", "9321.86`", ",", "9322.46`", 
     ",", "9321.33`", ",", "9321.4`", ",", "14", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T09:30:00\"\>", ",", "9313.85`", ",", "9313.85`", 
     ",", "9129.75`", ",", "9131.64`", ",", "297", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T09:40:00\"\>", ",", "9129.9`", ",", "9151.51`", 
     ",", "9123.78`", ",", "9130.35`", ",", "281", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T09:50:00\"\>", ",", "9130.65`", ",", "9155.14`", 
     ",", "9129.07`", ",", "9148.19`", ",", "286", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:00:00\"\>", ",", "9145.54`", ",", "9152.42`", 
     ",", "9131.94`", ",", "9140.7`", ",", "283", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:10:00\"\>", ",", "9140.78`", ",", "9142.44`", 
     ",", "9125.97`", ",", "9138.66`", ",", "274", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:20:00\"\>", ",", "9136.77`", ",", "9147.66`", 
     ",", "9133.98`", ",", "9146.9`", ",", "270", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:30:00\"\>", ",", "9146.15`", ",", "9147.28`", 
     ",", "9132.16`", ",", "9139.19`", ",", "258", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:40:00\"\>", ",", "9138.66`", ",", "9145.77`", 
     ",", "9132.16`", ",", "9137.45`", ",", "264", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T10:50:00\"\>", ",", "9137.53`", ",", "9143.73`", 
     ",", "9127.55`", ",", "9130.12`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:00:00\"\>", ",", "9130.5`", ",", "9132.47`", 
     ",", "9123.93`", ",", "9127.71`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:10:00\"\>", ",", "9127.18`", ",", "9133.22`", 
     ",", "9116.52`", ",", "9118.86`", ",", "273", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:20:00\"\>", ",", "9119.24`", ",", "9145.92`", 
     ",", "9119.24`", ",", "9145.84`", ",", "267", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:30:00\"\>", ",", "9145.99`", ",", "9151.59`", 
     ",", "9144.11`", ",", "9151.59`", ",", "260", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:40:00\"\>", ",", "9151.81`", ",", "9157.26`", 
     ",", "9147.13`", ",", "9151.44`", ",", "250", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T11:50:00\"\>", ",", "9151.36`", ",", "9170.1`", 
     ",", "9149.77`", ",", "9164.43`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:00:00\"\>", ",", "9165.57`", ",", "9168.36`", 
     ",", "9153.33`", ",", "9156.58`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:10:00\"\>", ",", "9155.89`", ",", "9160.13`", 
     ",", "9152.95`", ",", "9159.37`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:20:00\"\>", ",", "9159.45`", ",", "9165.27`", 
     ",", "9158.39`", ",", "9163.07`", ",", "232", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:30:00\"\>", ",", "9163.23`", ",", "9166.63`", 
     ",", "9156.27`", ",", "9157.26`", ",", "233", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:40:00\"\>", ",", "9157.33`", ",", "9159.6`", 
     ",", "9153.93`", ",", "9155.52`", ",", "236", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T12:50:00\"\>", ",", "9155.67`", ",", "9159.67`", 
     ",", "9154.76`", ",", "9157.1`", ",", "221", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:00:00\"\>", ",", "9157.18`", ",", "9157.18`", 
     ",", "9140.63`", ",", "9150.`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:10:00\"\>", ",", "9150.15`", ",", "9161.71`", 
     ",", "9148.79`", ",", "9161.41`", ",", "224", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:20:00\"\>", ",", "9161.49`", ",", "9161.64`", 
     ",", "9153.85`", ",", "9159.9`", ",", "238", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:30:00\"\>", ",", "9159.14`", ",", "9159.3`", 
     ",", "9147.96`", ",", "9150.68`", ",", "220", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:40:00\"\>", ",", "9150.53`", ",", "9160.73`", 
     ",", "9142.74`", ",", "9156.05`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T13:50:00\"\>", ",", "9156.12`", ",", "9167.`", 
     ",", "9152.34`", ",", "9165.8`", ",", "235", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:00:00\"\>", ",", "9165.87`", ",", "9165.87`", 
     ",", "9152.87`", ",", "9155.82`", ",", "229", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:10:00\"\>", ",", "9154.84`", ",", "9161.49`", 
     ",", "9154.23`", ",", "9158.69`", ",", "226", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:20:00\"\>", ",", "9158.62`", ",", "9167.38`", 
     ",", "9149.02`", ",", "9167.38`", ",", "241", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:30:00\"\>", ",", "9167.08`", ",", "9168.82`", 
     ",", "9159.9`", ",", "9160.13`", ",", "246", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:40:00\"\>", ",", "9159.45`", ",", "9159.9`", 
     ",", "9145.16`", ",", "9146.15`", ",", "244", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T14:50:00\"\>", ",", "9146.07`", ",", "9157.18`", 
     ",", "9144.79`", ",", "9152.19`", ",", "230", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:00:00\"\>", ",", "9152.12`", ",", "9163.98`", 
     ",", "9151.51`", ",", "9156.73`", ",", "231", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:10:00\"\>", ",", "9156.65`", ",", "9157.56`", 
     ",", "9145.54`", ",", "9150.`", ",", "237", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:20:00\"\>", ",", "9150.23`", ",", "9154.46`", 
     ",", "9134.43`", ",", "9145.16`", ",", "259", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:30:00\"\>", ",", "9143.88`", ",", "9151.66`", 
     ",", "9140.78`", ",", "9147.43`", ",", "262", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:40:00\"\>", ",", "9146.22`", ",", "9148.56`", 
     ",", "9126.42`", ",", "9136.17`", ",", "277", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T15:50:00\"\>", ",", "9136.09`", ",", "9152.19`", 
     ",", "9134.51`", ",", "9136.47`", ",", "282", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"2009-08-17T16:00:00\"\>", ",", "9136.62`", ",", "9136.62`", 
     ",", "9135.19`", ",", "9135.34`", ",", "19", ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.454587860835125*^9, {3.45458791016325*^9, 3.454587929257*^9}, {
   3.4545880750695*^9, 3.4545880968195*^9}, 3.454670632374875*^9, {
   3.45927583465625*^9, 3.45927584103125*^9}, 3.45927587253125*^9, 
   3.459275908296875*^9, 3.459275940140625*^9, 3.4592761865625*^9, {
   3.4592762426875*^9, 3.459276365453125*^9}, {3.459276398*^9, 
   3.459276447859375*^9}, 3.459276890546875*^9, 3.459276925328125*^9, {
   3.459276997140625*^9, 3.45927701709375*^9}, {3.459529895065959*^9, 
   3.459529945065959*^9}, {3.459529975722209*^9, 3.459530002956584*^9}, 
   3.459530197800334*^9, {3.459530270456584*^9, 3.459530286253459*^9}, 
   3.459531244940959*^9, 3.459531288925334*^9, 3.459531395284709*^9, 
   3.459531581878459*^9, 3.459531981284709*^9, 3.459532017128459*^9, 
   3.459532173347209*^9, 3.459532232519084*^9, 3.459532338097209*^9, 
   3.459532711753459*^9, 3.459532813394084*^9, 3.459533406128459*^9, 
   3.459533587128459*^9, 3.459534563237834*^9, 3.459535730628459*^9, 
   3.459535815581584*^9, 3.459535947440959*^9, 3.459536129909709*^9, 
   3.459536580862834*^9, 3.459536665534709*^9, 3.459539127315959*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can parse the date/time format used by BBGetIntraday."
}], "Text",
 CellChangeTimes->{{3.4542528605916348`*^9, 3.454252904368682*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DateList", "[", "\"\<2009-06-10T16:15:03\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.454252892416704*^9, 3.4542528942134066`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2009", ",", "6", ",", "10", ",", "16", ",", "15", ",", "3.`"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.454252895025829*^9, 3.454670635093625*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"daylightsavingbeg", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"2009", ",", "3", ",", "8"}], "}"}]}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{
   "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
    "year"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"daylightsavingend", " ", "=", " ", 
    RowBox[{"{", 
     RowBox[{"2009", ",", "11", ",", "1"}], "}"}]}], ";"}], 
  RowBox[{"(*", 
   RowBox[{
   "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
    "year"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mytime", " ", "=", "\"\<2009-08-17T13:30:00.000+00:00\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"AbsoluteTime", "[", "daylightsavingbeg", "]"}], "<=", 
    RowBox[{"AbsoluteTime", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"StringDrop", "[", 
        RowBox[{"mytime", ",", 
         RowBox[{"-", "10"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
         "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
         "\"\<Hour\>\"", ",", "\"\<:\>\"", ",", "\"\<Minute\>\"", ",", 
         "\"\<:\>\"", ",", "\"\<Second\>\""}], "}"}]}], "}"}], "]"}]}], ",", 
   "True", ",", "False"}], "]"}]}], "Input",
 CellChangeTimes->{{3.459534812409709*^9, 3.459534822425334*^9}, {
  3.459534880331584*^9, 3.459534902284709*^9}, {3.459535040253459*^9, 
  3.459535060347209*^9}, {3.459535101269084*^9, 3.459535106628459*^9}, {
  3.459535148706584*^9, 3.459535261925334*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{
  3.459535060644084*^9, 3.459535107269084*^9, {3.459535149894084*^9, 
   3.459535170722209*^9}, 3.459535262769084*^9}]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1082, 1021},
WindowMargins->{{13, Automatic}, {Automatic, 193}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
ShowSelection->True,
Magnification->0.8999999761581421,
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (December 4, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 421, 12, 133, "Title"],
Cell[991, 36, 379, 5, 26, "Subsubtitle"],
Cell[1373, 43, 3448, 63, 311, "Text"],
Cell[4824, 108, 1023, 19, 134, "Text"],
Cell[5850, 129, 48729, 1085, 2772, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[54604, 1218, 616, 13, 24, "Input"],
Cell[55223, 1233, 233, 3, 23, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55493, 1241, 367, 6, 24, "Input"],
Cell[55863, 1249, 327, 5, 23, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56227, 1259, 426, 11, 24, "Input"],
Cell[56656, 1272, 395, 8, 23, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57088, 1285, 940, 17, 64, "Input"],
Cell[58031, 1304, 513, 9, 37, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58581, 1318, 839, 15, 37, "Input"],
Cell[59423, 1335, 2277, 74, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61737, 1414, 1690, 27, 24, "Input"],
Cell[63430, 1443, 175899, 5031, 5667, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[239366, 6479, 280, 7, 24, "Input"],
Cell[239649, 6488, 169, 4, 23, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[239855, 6497, 1208, 20, 24, "Input"],
Cell[241066, 6519, 929406, 16349, 36757, "Output"]
}, Open  ]],
Cell[1170487, 22871, 202, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[1170714, 22880, 155, 2, 20, "Input"],
Cell[1170872, 22884, 188, 4, 20, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171097, 22893, 1651, 42, 81, "Input"],
Cell[1172751, 22937, 170, 3, 20, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
