(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     92379,       2098]
NotebookOptionsPosition[     90792,       2039]
NotebookOutlinePosition[     91238,       2058]
CellTagsIndexPosition[     91195,       2055]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Bloomberg Data to Mathematica", "Title",
 CellChangeTimes->{{3.4348218733245*^9, 3.434821880183875*^9}, {
  3.44951690785*^9, 3.449516919975*^9}, {3.4542526066628895`*^9, 
  3.4542526102875414`*^9}, {3.454671279093625*^9, 3.4546712805155*^9}, {
  3.6328723944795456`*^9, 3.632872394854727*^9}}],

Cell["February 12, 2015", "Subsubtitle",
 CellChangeTimes->{{3.4348218841995*^9, 3.434821885262*^9}, {
  3.45466980885925*^9, 3.454669810093625*^9}, {3.45960533434375*^9, 
  3.459605336546875*^9}, {3.46385140536852*^9, 3.46385140624352*^9}, {
  3.465744082552105*^9, 3.465744082989582*^9}, {3.50306886475525*^9, 
  3.50306886694275*^9}, {3.548606507408375*^9, 3.548606510252125*^9}, {
  3.6029615119274063`*^9, 3.602961513863517*^9}, {3.632843524427566*^9, 
  3.632843528133463*^9}}],

Cell[TextData[{
 "This code will not work unless you have a Bloomberg Professional license, \
have installed the Bloomberg API, and are logged in to Bloomberg Professional \
when you run the functions below.\[LineSeparator]\[LineSeparator]Because this \
updated interface relies on DateObjects, TimeSeries and EventSeries, it will \
not work with versions of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " earlier than 10.0, though the underlying binaries will do so. If you are \
using an old version of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", use the version of this interface titled \
\[OpenCurlyDoubleQuote]BBGetNF1-bloomberg into mathematica 201206\
\[CloseCurlyDoubleQuote].\n\nTo install, extract the attached zip file into a \
folder (typically under \[OpenCurlyDoubleQuote]C:\\Program Files\\Wolfram \
Research\\\[CloseCurlyDoubleQuote], and by default \
\[OpenCurlyDoubleQuote]C:\\Program Files\\Wolfram Research\\msFunctions\\\
\[CloseCurlyDoubleQuote] though this can be changed below. Note that this is \
a different default folder name than in previous versions of this interface. \
If an earlier version of the executables exist at that location, replace them \
with the new (October 2009) version. This interface consists of four \
commands:\n\n",
 StyleBox["msBBGetNF2",
  FontWeight->"Bold"],
 ": loads historical data from Bloomberg and returns it in NF2 format, about \
which more below. Used for, for example, U.S. CPI monthly back to 1969, or \
the closing price of Apple Inc. daily since 1978.\n",
 StyleBox["msBBGetCurrent",
  FontWeight->"Bold"],
 ": loads a current descriptive field (such as Walmart's state of \
incorporation or shares outstanding). The FLDS command in Bloomberg can be \
used to identify what data is available for each security. Will operate on \
either a single security or a list of them.\n",
 StyleBox["msBBGetIntraday",
  FontWeight->"Bold"],
 ": loads intraday trade data (or ask data, or bid data)\n",
 StyleBox["msBBGetIntradayBars",
  FontWeight->"Bold"],
 ":  loads periodic intraday data (price snapshots every minute, for example)."
}], "Text",
 CellChangeTimes->{{3.449516680834375*^9, 3.449516759615625*^9}, {
   3.449517050928125*^9, 3.4495171807875*^9}, {3.45121680611825*^9, 
   3.451216810758875*^9}, {3.451226165587*^9, 3.451226216337*^9}, {
   3.4542526240830917`*^9, 3.45425270795004*^9}, 3.45466982904675*^9, {
   3.454669864468625*^9, 3.454670137843625*^9}, {3.454670220812375*^9, 
   3.45467024460925*^9}, {3.45960536375*^9, 3.459605372546875*^9}, {
   3.459605466140625*^9, 3.45960547009375*^9}, {3.463339646437875*^9, 
   3.463339662781625*^9}, 3.463851368790395*^9, {3.465744092957822*^9, 
   3.465744093426548*^9}, {3.4657446581476326`*^9, 3.4657446588819704`*^9}, 
   3.503068715239625*^9, {3.548673433158375*^9, 3.548673448814625*^9}, 
   3.54867356558025*^9, {3.60296120918609*^9, 3.602961250406448*^9}, {
   3.630504117512353*^9, 3.630504117580346*^9}, {3.632843484272464*^9, 
   3.632843512567583*^9}, 3.632872432481097*^9, {3.632872499389725*^9, 
   3.6328725519546843`*^9}, {3.633088347874323*^9, 3.6330884437531657`*^9}, {
   3.633088900363977*^9, 3.6330889395208364`*^9}, {3.6330901457699895`*^9, 
   3.6330901459263444`*^9}, {3.63309058595151*^9, 3.6330906245035267`*^9}, {
   3.633136777425167*^9, 3.6331367906007757`*^9}, {3.6331956806793118`*^9, 
   3.63319568097927*^9}}],

Cell["\<\
BBGetNF2 produces output in \"nf2 format,\" a data type that I find useful \
for financial data. It includes a header and a time series, in this format:

\tAssocation[name\[Rule]\[CloseCurlyDoubleQuote]whatever\
\[CloseCurlyDoubleQuote], type\[Rule]\[CloseCurlyDoubleQuote]pctchanges\
\[CloseCurlyDoubleQuote], data\[Rule]TemporalData[]]

where type \[Rule] \"pctchanges\" or \"delta,\" or \"value\".

By default, pctchanges are stored as EventSeries[] and values are stored as \
TimeSeries[]. I have created hundreds of functions that work with NF2 \
objects, providing different sorts of analysis, graphing, and so forth.\
\>", "Text",
 CellChangeTimes->{{3.4456145947571564`*^9, 3.4456146143821564`*^9}, {
  3.4456146618040314`*^9, 3.4456146816165314`*^9}, {3.4456148011009064`*^9, 
  3.4456148375852814`*^9}, {3.4456148694134064`*^9, 3.4456149245852814`*^9}, {
  3.44951718619375*^9, 3.44951720569375*^9}, {3.4542527350880594`*^9, 
  3.454252740415673*^9}, {3.454669949437375*^9, 3.454669949593625*^9}, {
  3.454670107843625*^9, 3.454670107999875*^9}, {3.632843504944244*^9, 
  3.632843597324831*^9}, {3.633088486629698*^9, 3.633088563054042*^9}, {
  3.6330900916640396`*^9, 3.633090092383074*^9}, {3.633090197975569*^9, 
  3.633090211600718*^9}, {3.633195717802006*^9, 3.633195718756811*^9}}],

Cell[CellGroupData[{

Cell["Code", "Section",
 CellChangeTimes->{{3.632872467451565*^9, 3.6328724678414745`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
   "baseBBGpath", " ", "=", " ", 
    "\"\<C:/Program Files/Wolfram Research/msFunctions/BBRetrieve/\>\""}], 
   ";"}], " ", "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "msBBGetNF2", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"adjust", "\[Rule]", "\"\<CALENDAR\>\""}], ",", 
     RowBox[{"type", "\[Rule]", "\"\<value\>\""}], ",", 
     RowBox[{"name", "\[Rule]", "\"\<df\>\""}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"msBBGetNF2", "[", 
     RowBox[{
     "ticker_String", ",", "field_String", ",", "start_DateObject", ",", 
      "end_", ",", "freq_String", ",", 
      RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", " ", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "name1", ",", "output", ",", "link", ",", " ", "date", ",", "endS"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"name1", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"OptionValue", "[", "name", "]"}], "\[Equal]", 
           "\"\<df\>\""}], ",", 
          RowBox[{"ticker", "<>", "\"\<: \>\"", "<>", "field"}], ",", 
          RowBox[{"OptionValue", "[", "name", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"link", " ", "=", " ", 
        RowBox[{"Install", "[", 
         RowBox[{"baseBBGpath", "<>", "\"\<BBRetrieve\>\""}], "]"}]}], ";", 
       " ", 
       RowBox[{"(*", " ", 
        RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
         RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"date", "=", 
        RowBox[{"Switch", "[", 
         RowBox[{
          RowBox[{"ToUpperCase", "[", "freq", "]"}], " ", ",", "\"\<DAY\>\"", 
          ",", "\"\<DAILY\>\"", ",", "\"\<WEEK\>\"", ",", "\"\<WEEKLY\>\"", 
          ",", "\"\<MONTH\>\"", ",", "\"\<MONTHLY\>\"", ",", 
          "\"\<QUARTER\>\"", ",", "\"\<QUARTERLY\>\"", ",", "\"\<YEAR\>\"", 
          ",", "\"\<YEARLY\>\"", ",", "\"\<ANNUAL\>\"", ",", "\"\<YEARLY\>\"",
           ",", "_", ",", 
          RowBox[{"ToUpperCase", "[", "freq", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", " ", 
       RowBox[{"endS", "=", 
        RowBox[{"Switch", "[", 
         RowBox[{
          RowBox[{"Head", "[", "end", "]"}], ",", "List", ",", 
          RowBox[{"DateString", "[", 
           RowBox[{"end", ",", 
            RowBox[{"{", 
             RowBox[{
             "\"\<Year\>\"", ",", "\"\<Month\>\"", ",", "\"\<Day\>\""}], 
             "}"}]}], "]"}], ",", "DateObject", ",", 
          RowBox[{"DateString", "[", 
           RowBox[{"end", ",", 
            RowBox[{"{", 
             RowBox[{
             "\"\<Year\>\"", ",", "\"\<Month\>\"", ",", "\"\<Day\>\""}], 
             "}"}]}], "]"}], ",", "String", ",", "end"}], "]"}]}], ";", 
       RowBox[{"output", " ", "=", 
        RowBox[{"ToExpression", "[", 
         RowBox[{"BBRetrieve", "[", 
          RowBox[{"ticker", ",", " ", "field", ",", 
           RowBox[{"DateString", "[", 
            RowBox[{"start", ",", 
             RowBox[{"{", 
              RowBox[{
              "\"\<Year\>\"", ",", "\"\<Month\>\"", ",", "\"\<Day\>\""}], 
              "}"}]}], "]"}], ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"endS", "==", "\"\<\>\""}], ",", "endS", ",", 
             RowBox[{"DateString", "[", 
              RowBox[{"end", ",", 
               RowBox[{"{", 
                RowBox[{
                "\"\<Year\>\"", ",", "\"\<Month\>\"", ",", "\"\<Day\>\""}], 
                "}"}]}], "]"}]}], "]"}], ",", "date", ",", 
           RowBox[{"ToUpperCase", "[", 
            RowBox[{"OptionValue", "[", "adjust", "]"}], "]"}]}], "]"}], 
         "]"}]}], ";", " ", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"build", " ", "nf1", " ", "format"}], ";", " ", 
         RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
          RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ",
           "list"}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Uninstall", "[", "link", "]"}], ";", " ", 
       RowBox[{"(*", " ", 
        RowBox[{"close", " ", "link"}], " ", "*)"}], " ", 
       "\[IndentingNewLine]", 
       RowBox[{"output", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Take", "[", 
              RowBox[{
               RowBox[{"DateList", "[", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "]"}], ",", "3"}], "]"}], 
             ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}],
          "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"output", "=", 
        RowBox[{"Switch", "[", 
         RowBox[{
          RowBox[{"OptionValue", "[", "type", "]"}], ",", "\"\<value\>\"", 
          ",", 
          RowBox[{"TimeSeries", "[", "output", "]"}], ",", 
          "\"\<pctchanges\>\"", ",", 
          RowBox[{"EventSeries", "[", "output", "]"}], ",", "\"\<delta\>\"", 
          ",", 
          RowBox[{"EventSeries", "[", "output", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Association", "[", 
        RowBox[{
         RowBox[{"\"\<name\>\"", "\[Rule]", "name1"}], ",", 
         RowBox[{"\"\<type\>\"", "\[Rule]", 
          RowBox[{"OptionValue", "[", "type", "]"}]}], ",", 
         RowBox[{"\"\<data\>\"", "\[Rule]", "output"}]}], "]"}]}]}], 
     "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"msBBGetNF2", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<msBBGetNF2[ticker_,field_,start_,end_,freq_,OptionsPattern[]] \
retrieves data from Bloomberg for a given ticker and field within the \
specified period, returning an NF2 association.\nPARAMETERS:\nticker: e.g., \
\\\"AAPL Equity\\\"\nfield: e.g., \\\"Px_Last\\\"\nstart: DateObject that \
indicates start date\nend: DateObject that indicates end date, or \\\"\\\" \
for current day\nfreq: is the data frequency. Choices: \\\"day\\\", \
\\\"week\\\", \\\"month\\\", \\\"quarter\\\", or \\\"year\\\"\\n\n\n\
Optionally, you can pass  adjust\[Rule] for the period adjustment (default is \
\\\"CALENDAR\\\"). Choices, \\\"ACTUAL\\\", \\\"CALENDAR\\\"\ntype\[Rule] is \
NF2 datatype (default is \\\"value\\\"). Choices, \\\"delta\\\", \
\\\"pctchanges\\\", \\\"value\\\"\nname\[Rule]\\\"whatever\\\" to set the \
name in the NF2.\n\nBy default, pctchanges and delta-type data are stored as \
EventSeries[], while value-type data is stored as TimesSeries[].\>\""}], " ", 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"msBBGetCurrent", "[", 
    RowBox[{"ticker_", ",", "field_String"}], "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "output", ",", "link", ",", " ", "date", ",", "temp", ",", " ", "x", 
       ",", "mytickers"}], "}"}], ",", 
     RowBox[{
      RowBox[{"link", " ", "=", " ", 
       RowBox[{"Install", "[", 
        RowBox[{"baseBBGpath", "<>", "\"\<BBRetrieveCurrent\>\""}], "]"}]}], 
      ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
        RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], " ", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"StringQ", "[", "ticker", "]"}], ",", 
        RowBox[{"x", "=", "  ", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "ticker"}], "]"}]}], ",", 
        RowBox[{"x", " ", "=", " ", "ticker"}]}], "]"}], ";", 
      "\[IndentingNewLine]", " ", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"BBRetrieveCurrent", "[", 
        RowBox[{
         RowBox[{"StringJoin", "[", 
          RowBox[{"Riffle", "[", 
           RowBox[{"x", ",", "\"\< & \>\""}], "]"}], "]"}], ",", "field"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Partition", "[", 
         RowBox[{
          RowBox[{"StringSplit", "[", 
           RowBox[{"output", ",", 
            RowBox[{"{", 
             RowBox[{"\"\<{\>\"", ",", "\"\<}\>\"", ",", "\"\<,\>\""}], 
             "}"}]}], "]"}], ",", "2"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"mytickers", "=", 
       RowBox[{"output", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"output", " ", "=", " ", 
       RowBox[{"Map", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
         RowBox[{"output", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "output", "]"}], "==", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"NumberQ", "[", 
            RowBox[{
             RowBox[{"ToExpression", "[", "output", "]"}], "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}], "&&", 
           RowBox[{"Not", "[", 
            RowBox[{"StringMatchQ", "[", 
             RowBox[{
              RowBox[{"output", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", 
              RowBox[{
              "__", "~~", "\"\<-\>\"", "~~", "__", "~~", "\"\<-\>\"", "~~", 
               "__"}]}], "]"}], "]"}]}], ",", 
          RowBox[{"output", "=", 
           RowBox[{"ToExpression", "[", 
            RowBox[{"output", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
          RowBox[{"output", "=", 
           RowBox[{"output", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}]}], "]"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"output", " ", "=", " ", 
          RowBox[{"Map", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
            RowBox[{"output", "[", 
             RowBox[{"[", 
              RowBox[{"Flatten", "[", 
               RowBox[{"Map", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Position", "[", 
                   RowBox[{
                    RowBox[{"Map", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
                    "mytickers"}], "]"}], ",", "#"}], "]"}], "&"}], ",", 
                 RowBox[{"Map", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"StringTrim", "[", "#", "]"}], "&"}], ",", 
                   "ticker"}], "]"}]}], "]"}], "]"}], "]"}], "]"}]}], "]"}]}],
          ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"NumberQ", "[", 
             RowBox[{
              RowBox[{"ToExpression", "[", "output", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "]"}], "&&", 
            RowBox[{"Not", "[", 
             RowBox[{"StringMatchQ", "[", 
              RowBox[{
               RowBox[{"output", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{
               "__", "~~", "\"\<-\>\"", "~~", "__", "~~", "\"\<-\>\"", "~~", 
                "__"}]}], "]"}], "]"}]}], ",", 
           RowBox[{"output", " ", "=", " ", 
            RowBox[{"Flatten", "[", 
             RowBox[{"ToExpression", "[", "output", "]"}], "]"}]}], ",", 
           RowBox[{"output", "=", 
            RowBox[{"Flatten", "[", "output", "]"}]}]}], "]"}]}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Uninstall", "[", "link", "]"}], ";", 
      RowBox[{"(*", " ", 
       RowBox[{"close", " ", "link"}], " ", "*)"}], " ", "output"}]}], 
    "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"msBBGetCurrent", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<msBBGetCurrent[ticker_,field_] retrieves specified data from \
Bloomberg for a given ticker, or for a {list of tickers}. \nPARAMETERS:\n\
ticker: e.g., \\\"AAPL US Equity\\\" or {\\\"AAPL US Equity\\\", \\\"IBM US \
Equity\\\"}\nfield:  Anything from the \\\"FLDS\\\" database on \
Bloomberg,\\\"General_Counsel\\\" for example\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Options", "[", "msBBGetIntraday", "]"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"daylightsavingbeg", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2015", ",", "3", ",", "8"}], "}"}]}], ",", 
      RowBox[{"daylightsavingend", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2015", ",", "11", ",", "1"}], "}"}]}], ",", 
      RowBox[{"name", "\[Rule]", "\"\<df\>\""}], ",", 
      RowBox[{"dateformat", "\[Rule]", "\"\<bbg\>\""}], ",", 
      RowBox[{"ticktype", "\[Rule]", "\"\<TRADE\>\""}]}], "}"}]}], ";", 
   RowBox[{
    RowBox[{"msBBGetIntraday", "[", 
     RowBox[{
     "ticker_String", ",", "starttime_DateObject", ",", "endtime_DateObject", 
      ",", 
      RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "output", ",", " ", "link", ",", "st", ",", "et", ",", 
        "daylighttimezone", ",", "dateformat", ",", "x", ",", "fld", ",", 
        "ok"}], "}"}], ",", " ", 
      RowBox[{
       RowBox[{"ok", "=", "True"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"\"\<trade\>\"", ",", "\"\<bid\>\"", ",", "\"\<ask\>\""}],
             "}"}], ",", 
           RowBox[{"ToLowerCase", "[", 
            RowBox[{"OptionValue", "[", "ticktype", "]"}], "]"}]}], "]"}], 
         ",", 
         RowBox[{"fld", "=", 
          RowBox[{"ToUpperCase", "[", 
           RowBox[{"OptionValue", "[", "ticktype", "]"}], "]"}]}], ",", 
         RowBox[{"ok", "=", "False"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"link", " ", "=", " ", 
        RowBox[{"Install", "[", 
         RowBox[{"baseBBGpath", "<>", "\"\<BBRetrieveIntraday\>\""}], "]"}]}],
        ";", " ", 
       RowBox[{"(*", " ", 
        RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
         RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"daylighttimezone", " ", "=", " ", 
        RowBox[{"-", "4"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"dateformat", " ", "=", " ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
          "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
          "\"\<Time\>\""}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"st", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "starttime", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"st", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"AbsoluteTime", "[", 
             RowBox[{"OptionValue", "[", "daylightsavingbeg", "]"}], "]"}], 
            "\[LessEqual]", "st"}], "&&", 
           RowBox[{
            RowBox[{"AbsoluteTime", "[", 
             RowBox[{"OptionValue", "[", "daylightsavingend", "]"}], "]"}], 
            "\[GreaterEqual]", " ", "st"}]}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"st", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
                "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"st", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                 "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}],
            "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"et", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "endtime", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"et", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"AbsoluteTime", "[", 
             RowBox[{"OptionValue", "[", "daylightsavingbeg", "]"}], "]"}], 
            "\[LessEqual]", "et"}], "&&", 
           RowBox[{
            RowBox[{"AbsoluteTime", "[", 
             RowBox[{"OptionValue", "[", "daylightsavingend", "]"}], "]"}], 
            "\[GreaterEqual]", " ", "et"}]}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"et", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
                "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"et", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                 "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}],
            "]"}]}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
       RowBox[{"x", "=", 
        RowBox[{"BBRetrieveIntraday", "[", 
         RowBox[{"ticker", ",", "fld", ",", "st", ",", "et"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"output", " ", "=", " ", 
        RowBox[{"ToExpression", "[", "x", "]"}]}], ";", 
       RowBox[{"(*", 
        RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
         RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
         "list"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"output", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"AbsoluteTime", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"StringDrop", "[", 
                 RowBox[{
                  RowBox[{"#", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], ",", 
                  RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
              "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}],
          "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"output", " ", "=", " ", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"AbsoluteTime", "[", 
                  RowBox[{"OptionValue", "[", "daylightsavingbeg", "]"}], 
                  "]"}], "\[LessEqual]", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "&&", " ", 
                RowBox[{
                 RowBox[{"AbsoluteTime", "[", 
                  RowBox[{"OptionValue", "[", "daylightsavingend", "]"}], 
                  "]"}], "\[GreaterEqual]", " ", 
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}]}], ",", 
               RowBox[{"DateObject", "[", 
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"daylighttimezone", ",", "\"\<Hour\>\""}], 
                    "}"}]}], "]"}], "]"}], "]"}], ",", 
               RowBox[{"DateObject", "[", 
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"daylighttimezone", "-", "1"}], ",", 
                    "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], "]"}]}], "]"}], 
             ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}],
          "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Uninstall", "[", "link", "]"}], ";", 
       RowBox[{"(*", 
        RowBox[{"close", " ", "link"}], "*)"}], " ", "output"}]}], "]"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"msBBGetIntraday", "::", "usage"}], " ", "=", " ", 
    "\[IndentingNewLine]", 
    "\"\<msBBGetIntraday[ticker_,starttime_,endtime_] retrieves intraday \
ticks from Bloomberg for a given ticker within the specified period. \n\
PARAMETERS:\nticker: e.g., \\\"AAPL Equity\\\"\nfield:  \\\"TRADE\\\" or \
\\\"BID\\\" or \\\"ASK\\\"\nstarttime: DateObject indicating start date and \
time\nendtime: DateObject indicating end date and time\n\nReturns a \
three-member list for every datapoint available: {time, price, size}. \
Optional parameters include \!\(\*
StyleBox[\"daylightsavingbeg\",
FontVariations->{\"Underline\"->True}]\) and \!\(\*
StyleBox[\"daylightsavingend\",
FontVariations->{\"Underline\"->True}]\), which change every year. For a \
liquid security, the number of datapoints here is likely to be enormous, and \
you may want to consider using  msBBGetIntradayBars[] instead.\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Options", "[", "msBBGetIntradayBars", "]"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"daylightsavingbeg", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2015", ",", "3", ",", "8"}], "}"}]}], ",", 
      RowBox[{"daylightsavingend", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2015", ",", "11", ",", "1"}], "}"}]}], ",", 
      RowBox[{"fill", "\[Rule]", "1"}], ",", 
      RowBox[{"ticktype", "\[Rule]", "\"\<TRADE\>\""}]}], "}"}]}], ";", 
   RowBox[{
    RowBox[{"msBBGetIntradayBars", "[", 
     RowBox[{
     "ticker_String", ",", "starttime_DateObject", ",", "endtime_DateObject", 
      ",", 
      RowBox[{"barsize_", "/;", 
       RowBox[{"NumberQ", "[", "barsize", "]"}]}], ",", 
      RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "output", ",", " ", "link", ",", "st", ",", "et", ",", "x", ",", 
        "daylighttimezone", ",", "dateformat", ",", "fld", ",", "ok", ",", 
        "dsb", ",", "dse"}], "}"}], ",", "\[IndentingNewLine]", " ", 
      RowBox[{
       RowBox[{"ok", "=", "True"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"\"\<trade\>\"", ",", "\"\<bid\>\"", ",", "\"\<ask\>\""}],
             "}"}], ",", 
           RowBox[{"ToLowerCase", "[", 
            RowBox[{"OptionValue", "[", "ticktype", "]"}], "]"}]}], "]"}], 
         ",", 
         RowBox[{"fld", "=", 
          RowBox[{"ToUpperCase", "[", 
           RowBox[{"OptionValue", "[", "ticktype", "]"}], "]"}]}], ",", 
         RowBox[{"ok", "=", "False"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"link", " ", "=", " ", 
        RowBox[{"Install", "[", 
         RowBox[{"baseBBGpath", "<>", "\"\<BBRetrieveIntradayBars\>\""}], 
         "]"}]}], ";", " ", 
       RowBox[{"(*", " ", 
        RowBox[{"set", " ", "up", " ", "link", " ", "to", " ", 
         RowBox[{"C", "++"}], " ", "Code"}], " ", "*)"}], 
       "\[IndentingNewLine]", " ", 
       RowBox[{"dsb", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", 
         RowBox[{"OptionValue", "[", "daylightsavingbeg", "]"}], "]"}]}], ";",
        " ", 
       RowBox[{"(*", 
        RowBox[{
        "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
         "year"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"dse", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", 
         RowBox[{"OptionValue", "[", "daylightsavingend", "]"}], "]"}]}], ";", 
       RowBox[{"(*", 
        RowBox[{
        "Remember", " ", "to", " ", "reset", " ", "this", " ", "every", " ", 
         "year"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"daylighttimezone", " ", "=", " ", 
        RowBox[{"-", "4"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"dateformat", " ", "=", " ", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Year\>\"", ",", "\"\<-\>\"", ",", "\"\<Month\>\"", ",", 
          "\"\<-\>\"", ",", "\"\<Day\>\"", ",", "\"\<T\>\"", ",", 
          "\"\<Time\>\""}], "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"st", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "starttime", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"st", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"dsb", "\[LessEqual]", "st"}], "&&", 
           RowBox[{"dse", "\[GreaterEqual]", " ", "st"}]}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"st", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
                "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"st", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                 "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}],
            "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"et", " ", "=", " ", 
        RowBox[{"AbsoluteTime", "[", "endtime", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"et", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"dsb", "\[LessEqual]", "et"}], "&&", 
           RowBox[{"dse", "\[GreaterEqual]", " ", "et"}]}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"et", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "daylighttimezone"}], ",", "\"\<Hour\>\""}], 
                "}"}]}], "]"}], "]"}], ",", "dateformat"}], "]"}], ",", 
          RowBox[{"DateString", "[", 
           RowBox[{
            RowBox[{"DateList", "[", 
             RowBox[{"DatePlus", "[", 
              RowBox[{"et", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"(", 
                   RowBox[{"daylighttimezone", "-", "1"}], ")"}]}], ",", 
                 "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], ",", "dateformat"}],
            "]"}]}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
       RowBox[{"x", "=", 
        RowBox[{"BBRetrieveIntradayBars", "[", 
         RowBox[{
         "ticker", ",", "fld", ",", "st", ",", "et", ",", "barsize", ",", 
          RowBox[{"OptionValue", "[", "fill", "]"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"output", " ", "=", " ", 
        RowBox[{"ToExpression", "[", "x", "]"}]}], ";", 
       RowBox[{"(*", 
        RowBox[{"using", " ", "ToExpression", " ", "to", " ", "convert", " ", 
         RowBox[{"C", "++"}], " ", "string", " ", "output", " ", "into", " ", 
         "list"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"output", " ", "=", " ", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"dsb", "\[LessEqual]", 
                 RowBox[{"AbsoluteTime", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
                  "]"}]}], "&&", " ", 
                RowBox[{"dse", "\[GreaterEqual]", " ", 
                 RowBox[{"AbsoluteTime", "[", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], ",", "dateformat"}], "}"}], 
                  "]"}]}]}], ",", 
               RowBox[{"DateObject", "[", 
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", 
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"daylighttimezone", ",", "\"\<Hour\>\""}], 
                    "}"}]}], "]"}], "]"}], "]"}], ",", 
               RowBox[{"DateObject", "[", 
                RowBox[{"DateList", "[", 
                 RowBox[{"DatePlus", "[", 
                  RowBox[{
                   RowBox[{"AbsoluteTime", "[", 
                    RowBox[{"StringDrop", "[", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                    RowBox[{"-", "10"}]}], "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"daylighttimezone", "-", "1"}], ",", 
                    "\"\<Hour\>\""}], "}"}]}], "]"}], "]"}], "]"}]}], "]"}], 
             ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "2", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "4", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "5", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "6", "]"}], "]"}], ",", 
             RowBox[{"#", "[", 
              RowBox[{"[", "7", "]"}], "]"}]}], "}"}], "&"}], ",", "output"}],
          "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Uninstall", "[", "link", "]"}], ";", 
       RowBox[{"(*", 
        RowBox[{"close", " ", "link"}], "*)"}], " ", "output"}]}], "]"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"msBBGetIntradayBars", "::", "usage"}], " ", "=", " ", 
   "\[IndentingNewLine]", 
   "\"\<msBBGetIntradayBars[ticker_,starttime_,endtime_,barsize_,fill_] \
retrieves intraday samples from Bloomberg for a given ticker within the \
specified period. \nPARAMETERS:\nticker: e.g., \\\"AAPL Equity\\\"\n\
starttime: DateObject indicating start date and time\nendtime: DateObject \
indicating end date and time\nbarsize: is the sample frequency, in minutes\n\
Optionally, can pass\n\tfield \[Rule]  \\\"TRADE\\\" or \\\"BID\\\" or \
\\\"ASK\\\", and/or\n\tfill\[Rule] If this is set to 1, the API will append \
previous day close information as first datapoint as the start of trading for \
the specified period. Otherwise, the first data point will be the actual \
first trading on that day.\n\n\nReturns a list of seven items for each bar \
\[LongDash]\n1: Time\n2: Open\n3: High\n4: Low\n5: Close\n6: Tick Counts per \
bar\n7: Shares per bar\n\nIf you want every trade, rather than periodic \
samples, use msBBGetIntraday[].\>\""}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.43377820609375*^9, 3.433778217125*^9}, {
   3.433778369359375*^9, 3.4337783988125*^9}, {3.433778468640625*^9, 
   3.433778489890625*^9}, {3.433778538796875*^9, 3.4337785634375*^9}, {
   3.433778790203125*^9, 3.433778845015625*^9}, {3.433778883484375*^9, 
   3.43377897465625*^9}, {3.43377905840625*^9, 3.4337794041875*^9}, {
   3.43377961803125*^9, 3.433779682890625*^9}, {3.433779729625*^9, 
   3.433779897828125*^9}, {3.4337799566875*^9, 3.4337799731875*^9}, {
   3.433780884796875*^9, 3.433780946109375*^9}, {3.433781254125*^9, 
   3.433781322390625*^9}, {3.433781357421875*^9, 3.43378158246875*^9}, 
   3.433781778125*^9, {3.433782220828125*^9, 3.43378224171875*^9}, {
   3.43378227840625*^9, 3.433782299484375*^9}, {3.433782354078125*^9, 
   3.43378242434375*^9}, {3.4337824698125*^9, 3.4337824986875*^9}, {
   3.433782655109375*^9, 3.433782658078125*^9}, {3.43378277053125*^9, 
   3.433782774078125*^9}, {3.433783084765625*^9, 3.433783159296875*^9}, {
   3.4337836699375*^9, 3.433783686203125*^9}, {3.43378376453125*^9, 
   3.43378379465625*^9}, {3.4337839035625*^9, 3.43378393*^9}, {
   3.4337840006875*^9, 3.433784010015625*^9}, {3.43378433059375*^9, 
   3.43378439140625*^9}, {3.43378443309375*^9, 3.433784477609375*^9}, {
   3.433784914015625*^9, 3.433784925765625*^9}, {3.449484662119875*^9, 
   3.4494846632605*^9}, {3.451214767875*^9, 3.451214772609375*^9}, {
   3.451214839390625*^9, 3.45121488534375*^9}, {3.451215319578125*^9, 
   3.451215326109375*^9}, {3.45121551015625*^9, 3.451215610640625*^9}, {
   3.45121574925*^9, 3.4512157503125*^9}, {3.451215789109375*^9, 
   3.4512158235625*^9}, {3.45121654265625*^9, 3.451216554578125*^9}, {
   3.45122493940625*^9, 3.451224945359375*^9}, {3.453720309296875*^9, 
   3.453720373859375*^9}, {3.453722020296875*^9, 3.4537221075*^9}, {
   3.453722138390625*^9, 3.4537222125*^9}, {3.453722867875*^9, 
   3.45372286840625*^9}, {3.45372289959375*^9, 3.4537229055625*^9}, {
   3.4537229406875*^9, 3.453722942203125*^9}, {3.453723266109375*^9, 
   3.4537233575*^9}, {3.453723671984375*^9, 3.453723679453125*^9}, {
   3.45372474390625*^9, 3.453724763796875*^9}, {3.45372495090625*^9, 
   3.4537249681875*^9}, {3.4537250716875*^9, 3.453725085953125*^9}, {
   3.453725140265625*^9, 3.45372516584375*^9}, {3.453725229140625*^9, 
   3.453725280875*^9}, {3.453726303921875*^9, 3.45372640371875*^9}, {
   3.45372648965625*^9, 3.453726493890625*^9}, {3.45372677446875*^9, 
   3.453726790484375*^9}, {3.454587560147625*^9, 3.4545875931945*^9}, {
   3.454587712491375*^9, 3.45458773335075*^9}, {3.45458779828825*^9, 
   3.454587813741375*^9}, {3.45468062529675*^9, 3.4546806765155*^9}, {
   3.45468071492175*^9, 3.45468099385925*^9}, {3.454681101093625*^9, 
   3.454681168578*^9}, {3.454681264281125*^9, 3.45468127704675*^9}, {
   3.4552881827285*^9, 3.455288186462875*^9}, {3.455288241087875*^9, 
   3.455288255712875*^9}, {3.455288573806625*^9, 3.455288596291*^9}, {
   3.45528868075975*^9, 3.45528868719725*^9}, {3.455289061306625*^9, 
   3.455289062962875*^9}, {3.45528922063475*^9, 3.455289289150375*^9}, {
   3.455289391400375*^9, 3.455289402494125*^9}, {3.45528944500975*^9, 
   3.455289495681625*^9}, {3.455289528025375*^9, 3.455289531916*^9}, {
   3.455289602056625*^9, 3.455289645166*^9}, {3.45528996400975*^9, 
   3.455290069931625*^9}, {3.455290252291*^9, 3.455290341025375*^9}, {
   3.455290674541*^9, 3.45529072182225*^9}, {3.45529085113475*^9, 
   3.455290897025375*^9}, {3.455291050244125*^9, 3.455291062416*^9}, {
   3.455291173275375*^9, 3.45529123819725*^9}, {3.455291320712875*^9, 
   3.455291354494125*^9}, {3.455291486525375*^9, 3.45529149082225*^9}, {
   3.4552915826035*^9, 3.455291590650375*^9}, {3.455291871525375*^9, 
   3.455291880041*^9}, {3.45529204319725*^9, 3.45529207863475*^9}, {
   3.4554578317285*^9, 3.45545787838475*^9}, {3.455458073619125*^9, 
   3.45545808157225*^9}, {3.4554581147285*^9, 3.455458120025375*^9}, {
   3.45545910250975*^9, 3.45545916763475*^9}, {3.455459276869125*^9, 
   3.45545931825975*^9}, {3.455459634994125*^9, 3.455459642056625*^9}, {
   3.455459690650375*^9, 3.45545973613475*^9}, {3.455459797306625*^9, 
   3.455459798462875*^9}, {3.455459860681625*^9, 3.45545986132225*^9}, {
   3.4554599044785*^9, 3.45545993744725*^9}, {3.455460045837875*^9, 
   3.45546004688475*^9}, {3.45546026813475*^9, 3.455460272806625*^9}, {
   3.45546059400975*^9, 3.45546066425975*^9}, {3.455460694400375*^9, 
   3.455460717244125*^9}, {3.455460798541*^9, 3.455460887900375*^9}, {
   3.455461010400375*^9, 3.455461048166*^9}, {3.455461084041*^9, 
   3.45546111444725*^9}, {3.45546122513475*^9, 3.455461234931625*^9}, {
   3.45546146963475*^9, 3.4554614739785*^9}, {3.455461523775375*^9, 
   3.45546153900975*^9}, {3.4554616149785*^9, 3.45546163294725*^9}, {
   3.45546179269725*^9, 3.455461796275375*^9}, {3.455464796712875*^9, 
   3.455464836041*^9}, {3.455464866494125*^9, 3.455464874494125*^9}, {
   3.455464932962875*^9, 3.4554649664785*^9}, {3.455465118931625*^9, 
   3.45546520775975*^9}, {3.455465384369125*^9, 3.45546538863475*^9}, {
   3.455465549837875*^9, 3.455465572119125*^9}, {3.45546563457225*^9, 
   3.45546563869725*^9}, {3.459185233875*^9, 3.45918524546875*^9}, {
   3.459185435109375*^9, 3.459185441234375*^9}, {3.45918557815625*^9, 
   3.459185583578125*^9}, {3.459187231875*^9, 3.459187239453125*^9}, {
   3.459187282234375*^9, 3.459187326484375*^9}, {3.459529027456584*^9, 
   3.459529046534709*^9}, {3.459532667394084*^9, 3.459532678050334*^9}, {
   3.459532741128459*^9, 3.459532768284709*^9}, {3.459533542565959*^9, 
   3.459533546144084*^9}, {3.459534496519084*^9, 3.459534536675334*^9}, {
   3.459534620300334*^9, 3.459534728894084*^9}, {3.459534915940959*^9, 
   3.459534982831584*^9}, 3.459535023722209*^9, {3.459535362222209*^9, 
   3.459535465019084*^9}, {3.459535502128459*^9, 3.459535548972209*^9}, {
   3.459535585190959*^9, 3.459535707987834*^9}, {3.459535856534709*^9, 
   3.459535864237834*^9}, {3.459536151175334*^9, 3.459536222284709*^9}, 
   3.459536318300334*^9, {3.459537205144084*^9, 3.459537233909709*^9}, {
   3.459537568847209*^9, 3.459537569409709*^9}, {3.459537695534709*^9, 
   3.459537839206584*^9}, {3.459537888769084*^9, 3.459537973987834*^9}, {
   3.459538071300334*^9, 3.459538072112834*^9}, {3.459538155347209*^9, 
   3.459538159519084*^9}, {3.459538236456584*^9, 3.459538237159709*^9}, {
   3.459538576409709*^9, 3.459538629019084*^9}, {3.459538678378459*^9, 
   3.459538679487834*^9}, {3.459538711581584*^9, 3.459538919940959*^9}, 
   3.459539029862834*^9, {3.462038129234375*^9, 3.46203813575*^9}, {
   3.46385132571227*^9, 3.46385134505602*^9}, 3.4657441558764753`*^9, {
   3.4657442125610733`*^9, 3.4657442129673023`*^9}, {3.5030681517865*^9, 
   3.503068190802125*^9}, {3.503068220864625*^9, 3.503068228599*^9}, {
   3.503068278927125*^9, 3.503068315270875*^9}, {3.503068670677125*^9, 
   3.5030687025365*^9}, {3.50306873606775*^9, 3.50306875813025*^9}, {
   3.548603887564625*^9, 3.548603940939625*^9}, {3.54860416989275*^9, 
   3.5486041943615*^9}, {3.548604243127125*^9, 3.54860425789275*^9}, {
   3.548604323049*^9, 3.548604446595875*^9}, {3.54860625558025*^9, 
   3.548606495939625*^9}, {3.5486067067365*^9, 3.548606706877125*^9}, {
   3.548606782002125*^9, 3.54860681533025*^9}, {3.548673457924*^9, 
   3.548673467877125*^9}, {3.54867358195525*^9, 3.548673739314625*^9}, {
   3.590445344182831*^9, 3.5904453470088997`*^9}, {3.6029612011376295`*^9, 
   3.6029613030754604`*^9}, {3.6158181720349517`*^9, 
   3.6158182280225496`*^9}, {3.6158183792316694`*^9, 3.615818521505895*^9}, {
   3.63050412370323*^9, 3.630504136087242*^9}, {3.6328433638612823`*^9, 
   3.632843465320928*^9}, {3.632862246113303*^9, 3.632862326916292*^9}, {
   3.63286238795494*^9, 3.632862472215893*^9}, {3.632862524863677*^9, 
   3.6328626737292547`*^9}, {3.6328627114479303`*^9, 3.6328627202744102`*^9}, 
   3.632862754667706*^9, {3.632862803575556*^9, 3.6328630005677967`*^9}, {
   3.632863069857132*^9, 3.632863295744474*^9}, {3.6328645987348833`*^9, 
   3.6328646189543047`*^9}, {3.632864651080536*^9, 3.632864788288644*^9}, {
   3.6328648485876303`*^9, 3.6328649295593452`*^9}, {3.632868125612153*^9, 
   3.632868135799319*^9}, {3.6328699786141195`*^9, 3.6328699966146407`*^9}, {
   3.6328700595077205`*^9, 3.632870082211614*^9}, {3.6328701166510706`*^9, 
   3.632870182516685*^9}, {3.632870247425474*^9, 3.6328704258534017`*^9}, {
   3.632870462526541*^9, 3.632870489731013*^9}, {3.632870539983821*^9, 
   3.632870549609159*^9}, {3.632870721365389*^9, 3.6328707648981543`*^9}, {
   3.6328708381977234`*^9, 3.6328708382606897`*^9}, {3.6328708936840315`*^9, 
   3.6328708953089223`*^9}, {3.6328709846248083`*^9, 3.632871002000211*^9}, {
   3.632871032391883*^9, 3.6328710920355396`*^9}, 3.6328711340205946`*^9, {
   3.63287118002214*^9, 3.6328712716977944`*^9}, {3.632871801802657*^9, 
   3.632871840132388*^9}, {3.6328718796026993`*^9, 3.6328719371833634`*^9}, {
   3.632872019076294*^9, 3.632872049155695*^9}, {3.633088572773142*^9, 
   3.6330886933246317`*^9}, {3.63308873526328*^9, 3.633088784686661*^9}, {
   3.6330891415677347`*^9, 3.6330891909759674`*^9}, {3.633089237836506*^9, 
   3.6330893099336863`*^9}, {3.6330894055627723`*^9, 
   3.6330894276558294`*^9}, {3.6330894861089554`*^9, 3.633089543050373*^9}, {
   3.6330898285609303`*^9, 3.633089838436306*^9}, {3.6330899412366858`*^9, 
   3.633089989285503*^9}, {3.6330900565378237`*^9, 3.633090062303399*^9}, {
   3.6330901037371984`*^9, 3.6330901043934517`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Section",
 CellChangeTimes->{{3.6328723196334095`*^9, 3.6328723203053007`*^9}, 
   3.6328724760292673`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"msBBGetCurrent", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<IBM Equity\>\"", ",", "\"\<AAPL Equity\>\""}], "}"}], ",", 
   "\"\<PX_LAST\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.455290478791*^9, 3.455290479541*^9}, {
  3.45529072813475*^9, 3.45529074138475*^9}, {3.455290797431625*^9, 
  3.455290806087875*^9}, {3.455290906791*^9, 3.455290964650375*^9}, {
  3.455291070244125*^9, 3.45529108969725*^9}, {3.455292108587875*^9, 
  3.455292116525375*^9}, {3.455292361244125*^9, 3.455292364556625*^9}, {
  3.632871411615878*^9, 3.6328714350398493`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"160.4`", ",", "127.08`"}], "}"}]], "Output",
 CellChangeTimes->{3.632871435695438*^9, 3.633088838782712*^9, 
  3.6330889604282074`*^9, 3.633089348794422*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"msBBGetCurrent", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<AAPL US Equity\>\""}], "}"}],
      ",", "\"\<Name\>\""}], "]"}], ",", 
   RowBox[{"msBBGetCurrent", "[", 
    RowBox[{"\"\<IBM US Equity\>\"", ",", "\"\<General_Counsel\>\""}], 
    "]"}]}], "}"}]], "Input",
 CellChangeTimes->{{3.455291454837875*^9, 3.455291464025375*^9}, {
  3.6328714470398912`*^9, 3.6328714490087557`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"INTL BUSINESS MACHINES CORP\"\>", ",", "\<\"APPLE INC\"\>"}],
     "}"}], ",", "\<\"MICHELLE H BROWDY\"\>"}], "}"}]], "Output",
 CellChangeTimes->{{3.45529144569725*^9, 3.455291465025375*^9}, 
   3.455291502275375*^9, 3.45529159969725*^9, 3.455291718041*^9, 
   3.455292200181625*^9, 3.4552923547285*^9, 3.455465655931625*^9, 
   3.602961309094805*^9, 3.6328714505863905`*^9, 3.6330889626777744`*^9, 
   3.633089350107164*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"msBBGetCurrent", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<UYG US Equity\>\"", ",", "\"\<SKF US Equity\>\"", ",", 
     "\"\<UWM US Equity\>\"", ",", "\"\<RWM US Equity\>\"", ",", 
     "\"\<SDD US Equity\>\"", ",", "\"\<SEF US Equity\>\"", ",", 
     "\"\<DAG US Equity\>\"", ",", "\"\<SAA US Equity\>\"", ",", 
     "\"\<SFK US Equity\>\"", ",", "\"\<SJH US Equity\>\"", ",", 
     "\"\<UVT US Equity\>\"", ",", "\"\<UKK US Equity\>\"", ",", 
     "\"\<IBM US Equity\>\"", ",", "\"\<AAPL US Equity\>\"", ",", 
     "\"\<POT US Equity\>\"", ",", " ", "\"\<YHOO US Equity\>\"", ",", " ", 
     "\"\<SPY US Equity\>\""}], "}"}], ",", "\"\<Last_Price\>\""}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.45546567700975*^9, 3.455465825712875*^9}, {
  3.632871460415027*^9, 3.6328715078704433`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "148.01`", ",", "49.82`", ",", "92.76`", ",", "15.11`", ",", "37.52`", ",", 
   "17.5199`", ",", "4.6901`", ",", "117.1521`", ",", "20.514`", ",", 
   "49.87`", ",", "58.9013`", ",", "104.31`", ",", "160.4`", ",", "127.08`", 
   ",", "37.04`", ",", "44.42`", ",", "209.78`"}], "}"}]], "Output",
 CellChangeTimes->{{3.455465729837875*^9, 3.455465751291*^9}, {
   3.455465795556625*^9, 3.455465826900375*^9}, 3.602961311233927*^9, {
   3.6328714631965966`*^9, 3.6328715088079195`*^9}, 3.6330889642402678`*^9, 
   3.633089351123212*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sampleNF2", "=", 
  RowBox[{"msBBGetNF2", "[", 
   RowBox[{"\"\<AAPL Equity\>\"", ",", "\"\<Px_Last\>\"", ",", 
    RowBox[{"DateObject", "[", "\"\<20070901\>\"", "]"}], ",", 
    RowBox[{"DateObject", "[", "\"\<20081020\>\"", "]"}], ",", 
    "\"\<month\>\""}], "]"}]}]], "Input",
 CellChangeTimes->{{3.60296134205569*^9, 3.6029613822159867`*^9}, {
  3.6328715136830525`*^9, 3.6328715414186525`*^9}, {3.6328721005789337`*^9, 
  3.632872104704257*^9}, {3.6330896053657017`*^9, 3.6330896055062437`*^9}, {
  3.6330898839379168`*^9, 3.6330898853911257`*^9}}],

Cell[BoxData[
 RowBox[{"\[LeftAssociation]", 
  RowBox[{
   RowBox[{"\<\"name\"\>", "\[Rule]", "\<\"AAPL Equity: Px_Last\"\>"}], ",", 
   RowBox[{"\<\"type\"\>", "\[Rule]", "\<\"value\"\>"}], ",", 
   RowBox[{"\<\"data\"\>", "\[Rule]", 
    InterpretationBox[
     RowBox[{
      TagBox["TimeSeries",
       "SummaryHead"], "[", 
      DynamicModuleBox[{Typeset`open$$ = False}, 
       PanelBox[
        PaneSelectorBox[{False->GridBox[{
           {
            PaneBox[
             ButtonBox[
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "SquarePlusIconMedium"],
               ImageSizeCache->{11., {0., 11.}}],
              Appearance->None,
              ButtonFunction:>(Typeset`open$$ = True),
              Evaluator->Automatic,
              Method->"Preemptive"],
             Alignment->{Center, Center},
             
             ImageSize->
              Dynamic[{
               Automatic, 3.5 CurrentValue["FontCapHeight"]/
                AbsoluteCurrentValue[Magnification]}]], 
            GraphicsBox[{{}, {}, 
              {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6],
                Opacity[1.], LineBox[CompressedData["
1:eJwVkn001Qccxm/uqHM2E7YckZcy5yCVe7k/93cdfb/mLTl5GSImlNeikchd
KldeSq7XOg2ljiYzhlaIkbzVcM8Jeb1bnVHs20GTS8Vsu/vjOc/fn8/zGIed
8A5X4XA4dsr833X2mp7jPoTf2zmv35izgZe6HeEjvoTbx7hg+kIA1XN7emb9
CJslUoX2DANt1WvLswcJ465pmF2R28ILlS+Npv0J8z1T0p0GhFDo7V68FEDY
r1vvETTIQmFu3cTiIUKu7GkY/c3Cjo5nUapBhKcPhzgGPWSh63LSovbXhJYH
n6QvSFiod0zYvzmY8MfJ10n5bizsUux+Y3yYcCDJWD9enYUPUpejxiGErg++
MCqNY2G4Sz6kF0pYG5oWWvM7C/Z/WeVYhBHGLvDKd/mJYGXj1GdmRwgj3zVL
bvaKYK+Oz5+io4RtL4vD0sEOrnY43N8XTjgaF/dQcpMFkW/OV04RhI3TIyb8
WVuI/+OKyC+SsC9w3iF3EwMK/1WFT5SSJ/q6Yf8+G9j0xOh9VDRhfMr6vbvh
fDAf7FIkxxD6JK+FoyofSno6OhOPEXZKd541aOJBacO5yczjhOcCFnZHhvGA
sq91ZcYSmtYsZR3Q5oHTqmVlWhyhbr6VWekGHvCOZAwVnCD8bTDhFUp5ENsk
eJr3DaF4dZt9uiYfhNxqnYp4wgfOnz/vLueDNn3n35BAODP1qCrWwhrYlumY
2pNKPzvNVAdk1vBW8qa3PZGQz68z9263gfcCX0XbKULNS2a94zcE8EkGo9ec
RFhSuRKxJGagoU+e8ziZcGvSsMWv0bZQsTYh6z5NWGn+eiZPQwimJiUBYymE
5cudlR8uCOEdc69xTEzYotX/uOytEEZfuakPf6vk4Vs26Qaz4JxTUTt9hvBA
i71HwxALXH0/w6lUwn9WvDtuFbDgdbtvcvksYY3VitxrXgg/9bbmfXSecN6k
6DrHXQj+z9S2bEgjrCqqM864bwtFE9ozWhLCQEGL1iEtW9DvE2dppROe2qJ7
1b2dgZ99grerXyD8tNRaPiBhQD7K3bgtgzDB9F+7vQ4MeDpy27ZmErqMZbSq
qDHg90P3L5ZZhLPSVr/mbgEcq1JpFGUTuh23jj2zLABZYcl55qJy/5MaOwyd
GOhIVBS7XlL+6ePqlNRSBjgOVWKXHMIRwXiV3hwDg2oG1YGXCWU9ZTqLHFto
CN88HJNLaGzReDE7QgDP64JkkVJCJnJo/vasNZQtxmiK8whD7ngqsl35cMdg
2FScT7i/fXBusdsKvB4teKQWEK7f4tbftdsD/wFpV9FW
                "]]}},
             AspectRatio->1,
             Axes->{False, False},
             AxesLabel->{None, None},
             AxesOrigin->{3.4*^9, 15.},
             DisplayFunction->Identity,
             Frame->{{True, True}, {True, True}},
             FrameLabel->{{None, None}, {None, None}},
             FrameStyle->Directive[
               Opacity[0.5], 
               Thickness[Tiny], 
               RGBColor[0.368417, 0.506779, 0.709798]],
             FrameTicks->{{None, None}, {None, None}},
             GridLines->{None, None},
             GridLinesStyle->Directive[
               GrayLevel[0.5, 0.4]],
             
             ImageSize->
              Dynamic[{
               Automatic, 3.5 CurrentValue["FontCapHeight"]/
                AbsoluteCurrentValue[Magnification]}],
             
             Method->{
              "DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
             
             PlotRange->
              NCache[{{3399926400, 3431721600}, {16.237101555716425`, 
                28.252004777289347`}}, {{3.3999264*^9, 3.4317216*^9}, {
               16.237101555716425`, 28.252004777289347`}}],
             PlotRangeClipping->True,
             PlotRangePadding->{{
                Scaled[0.1], 
                Scaled[0.1]}, {
                Scaled[0.1], 
                Scaled[0.1]}},
             Ticks->{Automatic, Automatic}], GridBox[{
              {
               TagBox[
                TemplateBox[{RowBox[{
                    TagBox["\"Time: \"", "SummaryItemAnnotation"], 
                    "\[InvisibleSpace]", 
                    TagBox["\"28 Sep 2007\"", "SummaryItem"]}],RowBox[{
                    TagBox["\" to \"", "SummaryItemAnnotation"], 
                    "\[InvisibleSpace]", 
                    TagBox["\"30 Sep 2008\"", "SummaryItem"]}]},
                 "RowDefault"],
                "SummaryItem"], "\[SpanFromLeft]"},
              {
               RowBox[{
                TagBox["\<\"Data points: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["13",
                 "SummaryItem"]}], "\[SpanFromLeft]"}
             },
             AutoDelete->False,
             
             BaseStyle->{
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False},
             
             GridBoxAlignment->{
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {{2}}, "Rows" -> {{Automatic}}}]}
          },
          AutoDelete->False,
          BaselinePosition->{1, 1},
          GridBoxAlignment->{"Rows" -> {{Top}}},
          
          GridBoxItemSize->{
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], True->
         GridBox[{
           {
            PaneBox[
             ButtonBox[
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "SquareMinusIconMedium"]],
              Appearance->None,
              ButtonFunction:>(Typeset`open$$ = False),
              Evaluator->Automatic,
              Method->"Preemptive"],
             Alignment->{Center, Center},
             
             ImageSize->
              Dynamic[{
               Automatic, 3.5 CurrentValue["FontCapHeight"]/
                AbsoluteCurrentValue[Magnification]}]], 
            GraphicsBox[{{}, {}, 
              {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6],
                Opacity[1.], LineBox[CompressedData["
1:eJwVkn001Qccxm/uqHM2E7YckZcy5yCVe7k/93cdfb/mLTl5GSImlNeikchd
KldeSq7XOg2ljiYzhlaIkbzVcM8Jeb1bnVHs20GTS8Vsu/vjOc/fn8/zGIed
8A5X4XA4dsr833X2mp7jPoTf2zmv35izgZe6HeEjvoTbx7hg+kIA1XN7emb9
CJslUoX2DANt1WvLswcJ465pmF2R28ILlS+Npv0J8z1T0p0GhFDo7V68FEDY
r1vvETTIQmFu3cTiIUKu7GkY/c3Cjo5nUapBhKcPhzgGPWSh63LSovbXhJYH
n6QvSFiod0zYvzmY8MfJ10n5bizsUux+Y3yYcCDJWD9enYUPUpejxiGErg++
MCqNY2G4Sz6kF0pYG5oWWvM7C/Z/WeVYhBHGLvDKd/mJYGXj1GdmRwgj3zVL
bvaKYK+Oz5+io4RtL4vD0sEOrnY43N8XTjgaF/dQcpMFkW/OV04RhI3TIyb8
WVuI/+OKyC+SsC9w3iF3EwMK/1WFT5SSJ/q6Yf8+G9j0xOh9VDRhfMr6vbvh
fDAf7FIkxxD6JK+FoyofSno6OhOPEXZKd541aOJBacO5yczjhOcCFnZHhvGA
sq91ZcYSmtYsZR3Q5oHTqmVlWhyhbr6VWekGHvCOZAwVnCD8bTDhFUp5ENsk
eJr3DaF4dZt9uiYfhNxqnYp4wgfOnz/vLueDNn3n35BAODP1qCrWwhrYlumY
2pNKPzvNVAdk1vBW8qa3PZGQz68z9263gfcCX0XbKULNS2a94zcE8EkGo9ec
RFhSuRKxJGagoU+e8ziZcGvSsMWv0bZQsTYh6z5NWGn+eiZPQwimJiUBYymE
5cudlR8uCOEdc69xTEzYotX/uOytEEZfuakPf6vk4Vs26Qaz4JxTUTt9hvBA
i71HwxALXH0/w6lUwn9WvDtuFbDgdbtvcvksYY3VitxrXgg/9bbmfXSecN6k
6DrHXQj+z9S2bEgjrCqqM864bwtFE9ozWhLCQEGL1iEtW9DvE2dppROe2qJ7
1b2dgZ99grerXyD8tNRaPiBhQD7K3bgtgzDB9F+7vQ4MeDpy27ZmErqMZbSq
qDHg90P3L5ZZhLPSVr/mbgEcq1JpFGUTuh23jj2zLABZYcl55qJy/5MaOwyd
GOhIVBS7XlL+6ePqlNRSBjgOVWKXHMIRwXiV3hwDg2oG1YGXCWU9ZTqLHFto
CN88HJNLaGzReDE7QgDP64JkkVJCJnJo/vasNZQtxmiK8whD7ngqsl35cMdg
2FScT7i/fXBusdsKvB4teKQWEK7f4tbftdsD/wFpV9FW
                "]]}},
             AspectRatio->1,
             Axes->{False, False},
             AxesLabel->{None, None},
             AxesOrigin->{3.4*^9, 15.},
             DisplayFunction->Identity,
             Frame->{{True, True}, {True, True}},
             FrameLabel->{{None, None}, {None, None}},
             FrameStyle->Directive[
               Opacity[0.5], 
               Thickness[Tiny], 
               RGBColor[0.368417, 0.506779, 0.709798]],
             FrameTicks->{{None, None}, {None, None}},
             GridLines->{None, None},
             GridLinesStyle->Directive[
               GrayLevel[0.5, 0.4]],
             
             ImageSize->
              Dynamic[{
               Automatic, 3.5 CurrentValue["FontCapHeight"]/
                AbsoluteCurrentValue[Magnification]}],
             
             Method->{
              "DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
             
             PlotRange->
              NCache[{{3399926400, 3431721600}, {16.237101555716425`, 
                28.252004777289347`}}, {{3.3999264*^9, 3.4317216*^9}, {
               16.237101555716425`, 28.252004777289347`}}],
             PlotRangeClipping->True,
             PlotRangePadding->{{
                Scaled[0.1], 
                Scaled[0.1]}, {
                Scaled[0.1], 
                Scaled[0.1]}},
             Ticks->{Automatic, Automatic}], GridBox[{
              {
               TagBox[
                TemplateBox[{RowBox[{
                    TagBox["\"Time: \"", "SummaryItemAnnotation"], 
                    "\[InvisibleSpace]", 
                    TagBox["\"28 Sep 2007\"", "SummaryItem"]}],RowBox[{
                    TagBox["\" to \"", "SummaryItemAnnotation"], 
                    "\[InvisibleSpace]", 
                    TagBox["\"30 Sep 2008\"", "SummaryItem"]}]},
                 "RowDefault"],
                "SummaryItem"], "\[SpanFromLeft]"},
              {
               RowBox[{
                TagBox["\<\"Data points: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["13",
                 "SummaryItem"]}], "\[SpanFromLeft]"},
              {
               RowBox[{
                TagBox["\<\"Regular: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["False",
                 "SummaryItem"]}], 
               RowBox[{
                TagBox["\<\"Output dimension: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["1",
                 "SummaryItem"]}]},
              {
               RowBox[{
                TagBox["\<\"Metadata: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["None",
                 "SummaryItem"]}], 
               RowBox[{
                TagBox["\<\"Minimum Increment: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["2505600",
                 "SummaryItem"]}]},
              {
               TagBox["\<\"\"\>",
                "SummaryItem"], 
               RowBox[{
                TagBox["\<\"Resampling: \"\>",
                 "SummaryItemAnnotation"], "\[InvisibleSpace]", 
                TagBox["\<\"LinearInterpolation\"\>",
                 "SummaryItem"]}]}
             },
             AutoDelete->False,
             
             BaseStyle->{
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False},
             
             GridBoxAlignment->{
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {{2}}, "Rows" -> {{Automatic}}}]}
          },
          AutoDelete->False,
          BaselinePosition->{1, 1},
          GridBoxAlignment->{"Rows" -> {{Top}}},
          
          GridBoxItemSize->{
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]}, Dynamic[
         Typeset`open$$],
         ImageSize->Automatic],
        BaselinePosition->Baseline],
       DynamicModuleValues:>{}], "]"}],
     TemporalData[
     TimeSeries, {{{21.9243, 27.1357, 26.0314, 28.2971, 19.3371, 17.86, 20.5, 
       24.85, 26.9643, 23.92, 22.7071, 24.2186, 16.2371}}, {{{3399926400, 
       3402777600, 3405369600, 3408048000, 3410726400, 3413232000, 3415910400,
        3418502400, 3421094400, 3423772800, 3426451200, 3428956800, 
       3431721600}}}, 1, {"Continuous", 1}, {"Discrete", 1}, 1, {
       ValueDimensions -> 1, 
        ResamplingMethod -> {"Interpolation", InterpolationOrder -> 1}}}, 
      True, 10.],
     Editable->False,
     SelectWithContents->True,
     Selectable->False]}]}], "\[RightAssociation]"}]], "Output",
 CellChangeTimes->{{3.6029613383204765`*^9, 3.60296138262201*^9}, {
   3.6328715192930365`*^9, 3.6328715417154875`*^9}, 3.6328721054387245`*^9, 
   3.633088966943593*^9, 3.6330893526382413`*^9, 3.633089606131159*^9, 
   3.633089910564449*^9}]
}, Open  ]],

Cell["You can tap the Bloomberg output with", "Text",
 CellChangeTimes->{{3.6330898587026567`*^9, 3.633089901908188*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sampleNF2", "[", "\"\<data\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.633089899470603*^9, 3.6330899084856496`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   TagBox["TimeSeries",
    "SummaryHead"], "[", 
   DynamicModuleBox[{Typeset`open$$ = False}, 
    PanelBox[
     PaneSelectorBox[{False->GridBox[{
        {
         PaneBox[
          ButtonBox[
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "SquarePlusIconMedium"],
            ImageSizeCache->{11., {0., 11.}}],
           Appearance->None,
           ButtonFunction:>(Typeset`open$$ = True),
           Evaluator->Automatic,
           Method->"Preemptive"],
          Alignment->{Center, Center},
          
          ImageSize->
           Dynamic[{
            Automatic, 3.5 CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
             Magnification]}]], 
         GraphicsBox[{{}, {}, 
           {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
            Opacity[1.], LineBox[CompressedData["
1:eJwVkn001Qccxm/uqHM2E7YckZcy5yCVe7k/93cdfb/mLTl5GSImlNeikchd
KldeSq7XOg2ljiYzhlaIkbzVcM8Jeb1bnVHs20GTS8Vsu/vjOc/fn8/zGIed
8A5X4XA4dsr833X2mp7jPoTf2zmv35izgZe6HeEjvoTbx7hg+kIA1XN7emb9
CJslUoX2DANt1WvLswcJ465pmF2R28ILlS+Npv0J8z1T0p0GhFDo7V68FEDY
r1vvETTIQmFu3cTiIUKu7GkY/c3Cjo5nUapBhKcPhzgGPWSh63LSovbXhJYH
n6QvSFiod0zYvzmY8MfJ10n5bizsUux+Y3yYcCDJWD9enYUPUpejxiGErg++
MCqNY2G4Sz6kF0pYG5oWWvM7C/Z/WeVYhBHGLvDKd/mJYGXj1GdmRwgj3zVL
bvaKYK+Oz5+io4RtL4vD0sEOrnY43N8XTjgaF/dQcpMFkW/OV04RhI3TIyb8
WVuI/+OKyC+SsC9w3iF3EwMK/1WFT5SSJ/q6Yf8+G9j0xOh9VDRhfMr6vbvh
fDAf7FIkxxD6JK+FoyofSno6OhOPEXZKd541aOJBacO5yczjhOcCFnZHhvGA
sq91ZcYSmtYsZR3Q5oHTqmVlWhyhbr6VWekGHvCOZAwVnCD8bTDhFUp5ENsk
eJr3DaF4dZt9uiYfhNxqnYp4wgfOnz/vLueDNn3n35BAODP1qCrWwhrYlumY
2pNKPzvNVAdk1vBW8qa3PZGQz68z9263gfcCX0XbKULNS2a94zcE8EkGo9ec
RFhSuRKxJGagoU+e8ziZcGvSsMWv0bZQsTYh6z5NWGn+eiZPQwimJiUBYymE
5cudlR8uCOEdc69xTEzYotX/uOytEEZfuakPf6vk4Vs26Qaz4JxTUTt9hvBA
i71HwxALXH0/w6lUwn9WvDtuFbDgdbtvcvksYY3VitxrXgg/9bbmfXSecN6k
6DrHXQj+z9S2bEgjrCqqM864bwtFE9ozWhLCQEGL1iEtW9DvE2dppROe2qJ7
1b2dgZ99grerXyD8tNRaPiBhQD7K3bgtgzDB9F+7vQ4MeDpy27ZmErqMZbSq
qDHg90P3L5ZZhLPSVr/mbgEcq1JpFGUTuh23jj2zLABZYcl55qJy/5MaOwyd
GOhIVBS7XlL+6ePqlNRSBjgOVWKXHMIRwXiV3hwDg2oG1YGXCWU9ZTqLHFto
CN88HJNLaGzReDE7QgDP64JkkVJCJnJo/vasNZQtxmiK8whD7ngqsl35cMdg
2FScT7i/fXBusdsKvB4teKQWEK7f4tbftdsD/wFpV9FW
             "]]}},
          AspectRatio->1,
          Axes->{False, False},
          AxesLabel->{None, None},
          AxesOrigin->{3.4*^9, 15.},
          DisplayFunction->Identity,
          Frame->{{True, True}, {True, True}},
          FrameLabel->{{None, None}, {None, None}},
          FrameStyle->Directive[
            Opacity[0.5], 
            Thickness[Tiny], 
            RGBColor[0.368417, 0.506779, 0.709798]],
          FrameTicks->{{None, None}, {None, None}},
          GridLines->{None, None},
          GridLinesStyle->Directive[
            GrayLevel[0.5, 0.4]],
          
          ImageSize->
           Dynamic[{
            Automatic, 3.5 CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
             Magnification]}],
          
          Method->{
           "DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
          
          PlotRange->
           NCache[{{3399926400, 3431721600}, {16.237101555716425`, 
             28.252004777289347`}}, {{3.3999264*^9, 3.4317216*^9}, {
            16.237101555716425`, 28.252004777289347`}}],
          PlotRangeClipping->True,
          PlotRangePadding->{{
             Scaled[0.1], 
             Scaled[0.1]}, {
             Scaled[0.1], 
             Scaled[0.1]}},
          Ticks->{Automatic, Automatic}], GridBox[{
           {
            TagBox[
             TemplateBox[{RowBox[{
                 TagBox["\"Time: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"28 Sep 2007\"", "SummaryItem"]}],RowBox[{
                 TagBox["\" to \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"30 Sep 2008\"", "SummaryItem"]}]},
              "RowDefault"],
             "SummaryItem"], "\[SpanFromLeft]"},
           {
            RowBox[{
             TagBox["\<\"Data points: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["13",
              "SummaryItem"]}], "\[SpanFromLeft]"}
          },
          AutoDelete->False,
          
          BaseStyle->{
           ShowStringCharacters -> False, NumberMarks -> False, 
            PrintPrecision -> 3, ShowSyntaxStyles -> False},
          GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
          
          GridBoxItemSize->{
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
          GridBoxSpacings->{"Columns" -> {{2}}, "Rows" -> {{Automatic}}}]}
       },
       AutoDelete->False,
       BaselinePosition->{1, 1},
       GridBoxAlignment->{"Rows" -> {{Top}}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], True->GridBox[{
        {
         PaneBox[
          ButtonBox[
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "SquareMinusIconMedium"]],
           Appearance->None,
           ButtonFunction:>(Typeset`open$$ = False),
           Evaluator->Automatic,
           Method->"Preemptive"],
          Alignment->{Center, Center},
          
          ImageSize->
           Dynamic[{
            Automatic, 3.5 CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
             Magnification]}]], 
         GraphicsBox[{{}, {}, 
           {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
            Opacity[1.], LineBox[CompressedData["
1:eJwVkn001Qccxm/uqHM2E7YckZcy5yCVe7k/93cdfb/mLTl5GSImlNeikchd
KldeSq7XOg2ljiYzhlaIkbzVcM8Jeb1bnVHs20GTS8Vsu/vjOc/fn8/zGIed
8A5X4XA4dsr833X2mp7jPoTf2zmv35izgZe6HeEjvoTbx7hg+kIA1XN7emb9
CJslUoX2DANt1WvLswcJ465pmF2R28ILlS+Npv0J8z1T0p0GhFDo7V68FEDY
r1vvETTIQmFu3cTiIUKu7GkY/c3Cjo5nUapBhKcPhzgGPWSh63LSovbXhJYH
n6QvSFiod0zYvzmY8MfJ10n5bizsUux+Y3yYcCDJWD9enYUPUpejxiGErg++
MCqNY2G4Sz6kF0pYG5oWWvM7C/Z/WeVYhBHGLvDKd/mJYGXj1GdmRwgj3zVL
bvaKYK+Oz5+io4RtL4vD0sEOrnY43N8XTjgaF/dQcpMFkW/OV04RhI3TIyb8
WVuI/+OKyC+SsC9w3iF3EwMK/1WFT5SSJ/q6Yf8+G9j0xOh9VDRhfMr6vbvh
fDAf7FIkxxD6JK+FoyofSno6OhOPEXZKd541aOJBacO5yczjhOcCFnZHhvGA
sq91ZcYSmtYsZR3Q5oHTqmVlWhyhbr6VWekGHvCOZAwVnCD8bTDhFUp5ENsk
eJr3DaF4dZt9uiYfhNxqnYp4wgfOnz/vLueDNn3n35BAODP1qCrWwhrYlumY
2pNKPzvNVAdk1vBW8qa3PZGQz68z9263gfcCX0XbKULNS2a94zcE8EkGo9ec
RFhSuRKxJGagoU+e8ziZcGvSsMWv0bZQsTYh6z5NWGn+eiZPQwimJiUBYymE
5cudlR8uCOEdc69xTEzYotX/uOytEEZfuakPf6vk4Vs26Qaz4JxTUTt9hvBA
i71HwxALXH0/w6lUwn9WvDtuFbDgdbtvcvksYY3VitxrXgg/9bbmfXSecN6k
6DrHXQj+z9S2bEgjrCqqM864bwtFE9ozWhLCQEGL1iEtW9DvE2dppROe2qJ7
1b2dgZ99grerXyD8tNRaPiBhQD7K3bgtgzDB9F+7vQ4MeDpy27ZmErqMZbSq
qDHg90P3L5ZZhLPSVr/mbgEcq1JpFGUTuh23jj2zLABZYcl55qJy/5MaOwyd
GOhIVBS7XlL+6ePqlNRSBjgOVWKXHMIRwXiV3hwDg2oG1YGXCWU9ZTqLHFto
CN88HJNLaGzReDE7QgDP64JkkVJCJnJo/vasNZQtxmiK8whD7ngqsl35cMdg
2FScT7i/fXBusdsKvB4teKQWEK7f4tbftdsD/wFpV9FW
             "]]}},
          AspectRatio->1,
          Axes->{False, False},
          AxesLabel->{None, None},
          AxesOrigin->{3.4*^9, 15.},
          DisplayFunction->Identity,
          Frame->{{True, True}, {True, True}},
          FrameLabel->{{None, None}, {None, None}},
          FrameStyle->Directive[
            Opacity[0.5], 
            Thickness[Tiny], 
            RGBColor[0.368417, 0.506779, 0.709798]],
          FrameTicks->{{None, None}, {None, None}},
          GridLines->{None, None},
          GridLinesStyle->Directive[
            GrayLevel[0.5, 0.4]],
          
          ImageSize->
           Dynamic[{
            Automatic, 3.5 CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
             Magnification]}],
          
          Method->{
           "DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
          
          PlotRange->
           NCache[{{3399926400, 3431721600}, {16.237101555716425`, 
             28.252004777289347`}}, {{3.3999264*^9, 3.4317216*^9}, {
            16.237101555716425`, 28.252004777289347`}}],
          PlotRangeClipping->True,
          PlotRangePadding->{{
             Scaled[0.1], 
             Scaled[0.1]}, {
             Scaled[0.1], 
             Scaled[0.1]}},
          Ticks->{Automatic, Automatic}], GridBox[{
           {
            TagBox[
             TemplateBox[{RowBox[{
                 TagBox["\"Time: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"28 Sep 2007\"", "SummaryItem"]}],RowBox[{
                 TagBox["\" to \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"30 Sep 2008\"", "SummaryItem"]}]},
              "RowDefault"],
             "SummaryItem"], "\[SpanFromLeft]"},
           {
            RowBox[{
             TagBox["\<\"Data points: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["13",
              "SummaryItem"]}], "\[SpanFromLeft]"},
           {
            RowBox[{
             TagBox["\<\"Regular: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["False",
              "SummaryItem"]}], 
            RowBox[{
             TagBox["\<\"Output dimension: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["1",
              "SummaryItem"]}]},
           {
            RowBox[{
             TagBox["\<\"Metadata: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["None",
              "SummaryItem"]}], 
            RowBox[{
             TagBox["\<\"Minimum Increment: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["2505600",
              "SummaryItem"]}]},
           {
            TagBox["\<\"\"\>",
             "SummaryItem"], 
            RowBox[{
             TagBox["\<\"Resampling: \"\>",
              "SummaryItemAnnotation"], "\[InvisibleSpace]", 
             TagBox["\<\"LinearInterpolation\"\>",
              "SummaryItem"]}]}
          },
          AutoDelete->False,
          
          BaseStyle->{
           ShowStringCharacters -> False, NumberMarks -> False, 
            PrintPrecision -> 3, ShowSyntaxStyles -> False},
          GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
          
          GridBoxItemSize->{
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
          GridBoxSpacings->{"Columns" -> {{2}}, "Rows" -> {{Automatic}}}]}
       },
       AutoDelete->False,
       BaselinePosition->{1, 1},
       GridBoxAlignment->{"Rows" -> {{Top}}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]}, Dynamic[
      Typeset`open$$],
      ImageSize->Automatic],
     BaselinePosition->Baseline],
    DynamicModuleValues:>{}], "]"}],
  TemporalData[
  TimeSeries, {{{21.9243, 27.1357, 26.0314, 28.2971, 19.3371, 17.86, 20.5, 
    24.85, 26.9643, 23.92, 22.7071, 24.2186, 16.2371}}, {{{3399926400, 
    3402777600, 3405369600, 3408048000, 3410726400, 3413232000, 3415910400, 
    3418502400, 3421094400, 3423772800, 3426451200, 3428956800, 3431721600}}},
     1, {"Continuous", 1}, {"Discrete", 1}, 1, {
    ValueDimensions -> 1, 
     ResamplingMethod -> {"Interpolation", InterpolationOrder -> 1}}}, True, 
   10.],
  Editable->False,
  SelectWithContents->True,
  Selectable->False]], "Output",
 CellChangeTimes->{3.633089912438972*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"msBBGetIntraday", "[", 
   RowBox[{"\"\<SPX Index\>\"", ",", 
    RowBox[{"DateObject", "[", "\"\<2014-12-03T09:30:00\>\"", "]"}], ",", 
    RowBox[{"DateObject", "[", "\"\<2014-12-03T09:31:00\>\"", "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.453722229078125*^9, 3.45372224440625*^9}, {
   3.45372252646875*^9, 3.453722563296875*^9}, {3.45372283796875*^9, 
   3.453722852375*^9}, {3.453723053*^9, 3.453723074375*^9}, {
   3.453723404671875*^9, 3.453723405109375*^9}, {3.453723643*^9, 
   3.453723643421875*^9}, {3.45372459578125*^9, 3.45372459971875*^9}, {
   3.453724728734375*^9, 3.45372473225*^9}, {3.45372511259375*^9, 
   3.453725113015625*^9}, {3.4592743951875*^9, 3.459274444046875*^9}, {
   3.459274482484375*^9, 3.45927448271875*^9}, {3.459274547890625*^9, 
   3.45927459928125*^9}, {3.459274696375*^9, 3.45927470203125*^9}, {
   3.4592748145625*^9, 3.459274836265625*^9}, {3.459526650144084*^9, 
   3.459526656644084*^9}, {3.459526911097209*^9, 3.459526921456584*^9}, {
   3.459528595940959*^9, 3.459528596394084*^9}, {3.459528738690959*^9, 
   3.459528742753459*^9}, 3.459528828269084*^9, {3.459528866784709*^9, 
   3.459528869409709*^9}, {3.459528908925334*^9, 3.459528909565959*^9}, {
   3.459528946659709*^9, 3.459528946925334*^9}, {3.459530212628459*^9, 
   3.459530217050334*^9}, {3.459536738925334*^9, 3.459536748456584*^9}, {
   3.459537472362834*^9, 3.459537509644084*^9}, {3.459537555878459*^9, 
   3.459537559894084*^9}, {3.459538035800334*^9, 3.459538035987834*^9}, {
   3.459538288581584*^9, 3.459538288737834*^9}, {3.459538354831584*^9, 
   3.459538363269084*^9}, {3.459538432909709*^9, 3.459538433222209*^9}, {
   3.459538939315959*^9, 3.459538939847209*^9}, {3.602961402848167*^9, 
   3.602961440288308*^9}, {3.632872113189006*^9, 3.6328721856448746`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:01\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "1.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.45`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:06\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "6.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.62`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:11\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "11.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.59`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:16\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "16.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.45`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:21\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "21.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.74`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:26\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "26.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.82`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:31\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "31.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.79`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:36\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "36.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2067.93`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:41\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "41.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2068.15`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:46\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "46.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2068.38`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:51\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "51.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2068.39`", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Wed 3 Dec 2014 09:30:56\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "3"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "56.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "2068.37`", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.459538949628459*^9, {3.602961403760219*^9, 3.602961441387371*^9}, {
   3.6328721298300576`*^9, 3.632872186644769*^9}, 3.633089005179681*^9, 
   3.63308935682585*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", "=", 
  RowBox[{"msBBGetIntradayBars", "[", 
   RowBox[{"\"\<INDU Index\>\"", ",", 
    RowBox[{"DateObject", "[", "\"\<2014-12-02T09:30:00\>\"", "]"}], ",", 
    RowBox[{"DateObject", "[", "\"\<2014-12-02T10:30:00\>\"", "]"}], ",", 
    "10"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.454587848335125*^9, 3.454587859507*^9}, {
   3.45458790747575*^9, 3.454587927960125*^9}, {3.45458807360075*^9, 
   3.45458809510075*^9}, {3.459275881515625*^9, 3.45927592615625*^9}, {
   3.459276168515625*^9, 3.4592761741875*^9}, {3.459276238203125*^9, 
   3.459276300421875*^9}, {3.459276330921875*^9, 3.459276410234375*^9}, {
   3.45927644059375*^9, 3.459276440921875*^9}, {3.4592768755625*^9, 
   3.459276880734375*^9}, {3.459276914078125*^9, 3.45927691478125*^9}, {
   3.459276982796875*^9, 3.459277007109375*^9}, {3.459529880003459*^9, 
   3.459530002284709*^9}, {3.459530269034709*^9, 3.459530284925334*^9}, {
   3.459531241440959*^9, 3.459531277034709*^9}, {3.459531384425334*^9, 
   3.459531387597209*^9}, {3.459532305581584*^9, 3.459532307831584*^9}, {
   3.459534557331584*^9, 3.459534561800334*^9}, {3.459535735472209*^9, 
   3.459535738894084*^9}, {3.459536102940959*^9, 3.459536103206584*^9}, 
   3.459536596815959*^9, {3.602961457161274*^9, 3.6029614889120893`*^9}, {
   3.6328722194274797`*^9, 3.632872285601246*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 09:30:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "30", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17778.85`", ",", "17826.82`", ",", "17778.85`", 
     ",", "17819.98`", ",", "300", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 09:40:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "40", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17819.44`", ",", "17824.45`", ",", "17801.87`", 
     ",", "17805.54`", ",", "300", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 09:50:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"9", ",", "50", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17806.41`", ",", "17817.77`", ",", "17797.6`", 
     ",", "17811.79`", ",", "300", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 10:00:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"10", ",", "0", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17813.01`", ",", "17844.29`", ",", "17813.01`", 
     ",", "17836.52`", ",", "300", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 10:10:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"10", ",", "10", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17836.23`", ",", "17838.06`", ",", "17829.68`", 
     ",", "17837.64`", ",", "300", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     TemplateBox[{
      RowBox[{"\"Tue 2 Dec 2014 10:20:00\""}],RowBox[{"DateObject", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{"2014", ",", "12", ",", "2"}], "}"}], ",", 
           RowBox[{"TimeObject", "[", 
             RowBox[{"{", 
               RowBox[{"10", ",", "20", ",", "0.`"}], "}"}], "]"}]}], "]"}]},
      "DateObject",
      Editable->False], ",", "17837.39`", ",", "17840.91`", ",", "17832.51`", 
     ",", "17833.23`", ",", "300", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.454587860835125*^9, {3.45458791016325*^9, 3.454587929257*^9}, {
   3.4545880750695*^9, 3.4545880968195*^9}, 3.454670632374875*^9, {
   3.45927583465625*^9, 3.45927584103125*^9}, 3.45927587253125*^9, 
   3.459275908296875*^9, 3.459275940140625*^9, 3.4592761865625*^9, {
   3.4592762426875*^9, 3.459276365453125*^9}, {3.459276398*^9, 
   3.459276447859375*^9}, 3.459276890546875*^9, 3.459276925328125*^9, {
   3.459276997140625*^9, 3.45927701709375*^9}, {3.459529895065959*^9, 
   3.459529945065959*^9}, {3.459529975722209*^9, 3.459530002956584*^9}, 
   3.459530197800334*^9, {3.459530270456584*^9, 3.459530286253459*^9}, 
   3.459531244940959*^9, 3.459531288925334*^9, 3.459531395284709*^9, 
   3.459531581878459*^9, 3.459531981284709*^9, 3.459532017128459*^9, 
   3.459532173347209*^9, 3.459532232519084*^9, 3.459532338097209*^9, 
   3.459532711753459*^9, 3.459532813394084*^9, 3.459533406128459*^9, 
   3.459533587128459*^9, 3.459534563237834*^9, 3.459535730628459*^9, 
   3.459535815581584*^9, 3.459535947440959*^9, 3.459536129909709*^9, 
   3.459536580862834*^9, 3.459536665534709*^9, 3.459539127315959*^9, {
   3.6029614704700346`*^9, 3.602961490077156*^9}, {3.632872243631094*^9, 
   3.632872286163984*^9}, 3.6330890070385294`*^9, 3.6330893604665155`*^9}]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{783, 691},
WindowMargins->{{25, Automatic}, {Automatic, 31}},
ShowSelection->True,
Magnification->0.8999999761581421,
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (December 4, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 300, 4, 82, "Title"],
Cell[870, 28, 483, 7, 26, "Subsubtitle"],
Cell[1356, 37, 3383, 58, 346, "Text"],
Cell[4742, 97, 1307, 22, 169, "Text"],
Cell[CellGroupData[{
Cell[6074, 123, 91, 1, 57, "Section"],
Cell[6168, 126, 42547, 908, 2630, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[48752, 1039, 125, 2, 45, "Section"],
Cell[CellGroupData[{
Cell[48902, 1045, 585, 11, 27, "Input"],
Cell[49490, 1058, 197, 4, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49724, 1067, 483, 12, 42, "Input"],
Cell[50210, 1081, 500, 10, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50747, 1096, 830, 16, 72, "Input"],
Cell[51580, 1114, 575, 10, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52192, 1129, 579, 10, 42, "Input"],
Cell[52774, 1141, 12669, 292, 58, "Output"]
}, Open  ]],
Cell[65458, 1436, 121, 1, 27, "Text"],
Cell[CellGroupData[{
Cell[65604, 1441, 141, 2, 27, "Input"],
Cell[65748, 1445, 11430, 277, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77215, 1727, 1837, 28, 42, "Input"],
Cell[79055, 1757, 5773, 150, 178, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84865, 1912, 1347, 21, 35, "Input"],
Cell[86215, 1935, 4537, 99, 136, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
